@@ -20,9 +20,7 @@ package com.aelitis.azureus.activities;
 
 import java.util.*;
 
-import org.gudy.azureus2.core3.disk.DiskManagerFileInfo;
 import org.gudy.azureus2.core3.download.DownloadManager;
-import org.gudy.azureus2.core3.download.DownloadManagerListener;
 import org.gudy.azureus2.core3.download.DownloadManagerState;
 import org.gudy.azureus2.core3.global.GlobalManager;
 import org.gudy.azureus2.core3.global.GlobalManagerListener;
@@ -30,11 +28,14 @@ import org.gudy.azureus2.core3.torrent.TOTorrent;
 import org.gudy.azureus2.core3.util.*;
 
 import com.aelitis.azureus.core.AzureusCore;
+import com.aelitis.azureus.core.AzureusCoreLifecycleAdapter;
+import com.aelitis.azureus.core.cnetwork.*;
 import com.aelitis.azureus.core.messenger.config.PlatformRatingMessenger;
 import com.aelitis.azureus.core.messenger.config.PlatformVuzeActivitiesMessenger;
 import com.aelitis.azureus.core.messenger.config.RatingUpdateListener2;
 import com.aelitis.azureus.core.torrent.*;
-import com.aelitis.azureus.util.Constants;
+import com.aelitis.azureus.util.ConstantsVuze;
+import com.aelitis.azureus.util.DataSourceUtils;
 import com.aelitis.azureus.util.MapUtils;
 
 /**
@@ -56,8 +57,6 @@ public class VuzeActivitiesManager
 
 	private static final String SAVE_FILENAME = "VuzeActivities.config";
 
-	protected static final boolean SHOW_DM_REMOVED_ACTIVITY = false;
-
 	private static ArrayList listeners = new ArrayList();
 
 	private static ArrayList allEntries = new ArrayList();
@@ -70,14 +69,15 @@ public class VuzeActivitiesManager
 
 	private static AEDiagnosticsLogger diag_logger;
 
-	private static long lastVuzeNewsAt;
+	/** Key: NetworkID, Value: last time we pulled news **/ 
+	private static Map<String, Long> lastNewsAt = new HashMap();
 
 	private static boolean skipAutoSave = true;
 
-	private static DownloadManagerListener dmListener;
-
 	private static AEMonitor config_mon = new AEMonitor("ConfigMon");
 
+	private static boolean saveEventsOnClose = false;
+
 	static {
 		if (System.getProperty("debug.vuzenews", "0").equals("1")) {
 			diag_logger = AEDiagnostics.getLogger("v3.vuzenews");
@@ -99,9 +99,41 @@ public class VuzeActivitiesManager
 		if (diag_logger != null) {
 			diag_logger.log("Initialize Called");
 		}
+		
+		core.addLifecycleListener(new AzureusCoreLifecycleAdapter() {
+			public void stopping(AzureusCore core) {
+				if (saveEventsOnClose) {
+					saveEventsNow();
+				}
+			}
+		});
 
 		loadEvents();
 
+		ContentNetworkManager cnm = ContentNetworkManagerFactory.getSingleton();
+		if (cnm != null) {
+			ContentNetwork[] contentNetworks = cnm.getContentNetworks();
+			cnm.addListener(new ContentNetworkListener() {
+
+				public void networkRemoved(ContentNetwork network) {
+				}
+
+				public void networkChanged(ContentNetwork network) {
+				}
+
+				public void networkAdded(ContentNetwork cn) {
+					setupContentNetwork(cn);
+				}
+
+				public void networkAddFailed(long network_id, Throwable error) {
+				}
+			});
+			
+			for (ContentNetwork cn : contentNetworks) {
+				setupContentNetwork(cn);
+			}
+		}
+		
 		replyListener = new PlatformVuzeActivitiesMessenger.GetEntriesReplyListener() {
 			public void gotVuzeNewsEntries(VuzeActivitiesEntry[] entries,
 					long refreshInMS) {
@@ -119,10 +151,7 @@ public class VuzeActivitiesManager
 				SimpleTimer.addEvent("GetVuzeNews",
 						SystemTime.getOffsetTime(refreshInMS), new TimerEventPerformer() {
 							public void perform(TimerEvent event) {
-								PlatformVuzeActivitiesMessenger.getEntries(Math.min(
-										SystemTime.getCurrentTime() - lastVuzeNewsAt, MAX_LIFE_MS),
-										5000, replyListener);
-								lastVuzeNewsAt = SystemTime.getCurrentTime();
+								pullActivitiesNow(5000);
 							}
 						});
 			}
@@ -162,90 +191,12 @@ public class VuzeActivitiesManager
 			}
 		});
 
-		dmListener = new DownloadManagerListener() {
-
-			public void stateChanged(DownloadManager manager, int state) {
-				// TODO Auto-generated method stub
-
-			}
-
-			public void positionChanged(DownloadManager download, int oldPosition,
-					int newPosition) {
-				// TODO Auto-generated method stub
-
-			}
-
-			public void filePriorityChanged(DownloadManager download,
-					DiskManagerFileInfo file) {
-				// TODO Auto-generated method stub
-
-			}
-
-			public void downloadComplete(DownloadManager dm) {
-				VuzeActivitiesEntry entry = createDMCompleteEntry(dm);
-				if (entry != null) {
-					addEntries(new VuzeActivitiesEntry[] {
-						entry
-					});
-				}
-				dm.removeListener(this);
-			}
-
-			public void completionChanged(DownloadManager manager, boolean completed) {
-				// TODO Auto-generated method stub
-
-			}
-		};
-
 		GlobalManagerListener gmListener = new GlobalManagerListener() {
 
 			public void seedingStatusChanged(boolean seeding_only_mode, boolean b) {
 			}
 
 			public void downloadManagerRemoved(DownloadManager dm) {
-				try {
-					if (PlatformTorrentUtils.getAdId(dm.getTorrent()) != null) {
-						return;
-					}
-
-					VuzeActivitiesEntry[] entries = getAllEntries();
-					for (int i = 0; i < entries.length; i++) {
-						VuzeActivitiesEntry oldEntry = entries[i];
-						DownloadManager oldDM = oldEntry.getDownloadManger();
-						if (oldDM != null && oldDM.equals(dm)) {
-							removeEntries(new VuzeActivitiesEntry[] {
-								oldEntry
-							});
-						}
-					}
-					if (SHOW_DM_REMOVED_ACTIVITY) {
-						VuzeActivitiesEntry entry = new VuzeActivitiesEntry();
-
-						String hash = dm.getTorrent().getHashWrapper().toBase32String();
-						String title;
-						if (PlatformTorrentUtils.isContent(dm.getTorrent(), true)) {
-							String url = Constants.URL_PREFIX + Constants.URL_DETAILS + hash
-									+ ".html?" + Constants.URL_SUFFIX + "&client_ref=activity-"
-									+ VuzeActivitiesConstants.TYPEID_DL_REMOVE;
-							title = "<A HREF=\"" + url + "\">"
-									+ PlatformTorrentUtils.getContentTitle2(dm) + "</A>";
-							entry.setAssetHash(hash);
-						} else {
-							title = PlatformTorrentUtils.getContentTitle2(dm);
-						}
-
-						entry.setID(hash + ";r" + entry.getTimestamp());
-						entry.setText(title + " has been removed from your library");
-						entry.setTypeID(VuzeActivitiesConstants.TYPEID_DL_REMOVE, true);
-						addEntries(new VuzeActivitiesEntry[] {
-							entry
-						});
-					}
-				} catch (Throwable t) {
-					// ignore
-				}
-
-				dm.removeListener(dmListener);
 			}
 
 			public void downloadManagerAdded(DownloadManager dm) {
@@ -262,6 +213,9 @@ public class VuzeActivitiesManager
 			}
 		};
 
+		
+		
+		
 		List newEntries = new ArrayList();
 		GlobalManager gm = core.getGlobalManager();
 		gm.addListener(gmListener, false);
@@ -289,6 +243,28 @@ public class VuzeActivitiesManager
 		}
 	}
 
+	/**
+	 * @param cn
+	 *
+	 * @since 4.0.0.5
+	 */
+	private static void setupContentNetwork(final ContentNetwork cn) {
+		cn.addPersistentPropertyChangeListener(new ContentNetworkPropertyChangeListener() {
+			// @see com.aelitis.azureus.core.cnetwork.ContentNetworkPropertyChangeListener#propertyChanged(java.lang.String)
+			public void propertyChanged(String name) {
+				if (!ContentNetwork.PP_ACTIVE.equals(name)) {
+					return;
+				}
+				Object oIsActive = cn.getPersistentProperty(ContentNetwork.PP_ACTIVE);
+				boolean isActive = (oIsActive instanceof Boolean)
+						? ((Boolean) oIsActive).booleanValue() : false;
+				if (isActive) {
+					pullActivitiesNow(2000);
+				}
+			}
+		});
+	}
+
 	/**
 	 * @param allEntries2
 	 *
@@ -330,59 +306,6 @@ public class VuzeActivitiesManager
 		}
 	}
 
-	/**
-	 * @param dm
-	 * @return
-	 *
-	 * @since 3.0.4.3
-	 */
-	protected static VuzeActivitiesEntry createDMCompleteEntry(DownloadManager dm) {
-		try {
-			String hash = dm.getTorrent().getHashWrapper().toBase32String();
-
-			long completedTime = dm.getDownloadState().getLongParameter(
-					DownloadManagerState.PARAM_DOWNLOAD_COMPLETED_TIME);
-
-			String id = hash + ";c" + completedTime;
-
-			//System.out.println("DC " + dm.getDisplayName());
-			VuzeActivitiesEntry[] entries = getAllEntries();
-			for (int i = 0; i < entries.length; i++) {
-				VuzeActivitiesEntry oldEntry = entries[i];
-				DownloadManager oldDM = oldEntry.getDownloadManger();
-				if (oldDM != null && oldDM.equals(dm)
-						&& VuzeActivitiesConstants.TYPEID_DL_ADDED.equals(oldEntry.getTypeID())) {
-					//System.out.println("remove added entry " + oldEntry.id);
-					removeEntries(new VuzeActivitiesEntry[] {
-						oldEntry
-					});
-				}
-			}
-
-			String title;
-			if (PlatformTorrentUtils.isContent(dm.getTorrent(), true)) {
-				String url = Constants.URL_PREFIX + Constants.URL_DETAILS + hash
-						+ ".html?" + Constants.URL_SUFFIX + "&client_ref=activity-"
-						+ VuzeActivitiesConstants.TYPEID_DL_COMPLETE;
-				title = "<A HREF=\"" + url + "\">"
-						+ PlatformTorrentUtils.getContentTitle2(dm) + "</A>";
-			} else {
-				title = PlatformTorrentUtils.getContentTitle2(dm);
-			}
-
-			VuzeActivitiesEntry entry = new VuzeActivitiesEntry();
-			entry.setTimestamp(completedTime);
-			entry.setID(id);
-			entry.setText(title + " has completed downloading");
-			entry.setTypeID(VuzeActivitiesConstants.TYPEID_DL_COMPLETE, true);
-			entry.setDownloadManager(dm);
-
-			return entry;
-		} catch (Throwable t) {
-			Debug.out(t);
-		}
-		return null;
-	}
 
 	private static List registerDM(DownloadManager dm) {
 		TOTorrent torrent = dm.getTorrent();
@@ -394,53 +317,13 @@ public class VuzeActivitiesManager
 
 		List entries = new ArrayList();
 
-		if (dm.getAssumedComplete()) {
-			VuzeActivitiesEntry entry = createDMCompleteEntry(dm);
-			if (entry != null) {
-				entries.add(entry);
-			}
-		} else {
-			try {
-				long addedOn = (dm == null) ? 0
-						: dm.getDownloadState().getLongParameter(
-								DownloadManagerState.PARAM_DOWNLOAD_ADDED_TIME);
-				if (addedOn < getCutoffTime()) {
-					return null;
-				}
-
-				VuzeActivitiesEntry entry = new VuzeActivitiesEntry();
-				entries.add(entry);
-				String hash = torrent.getHashWrapper().toBase32String();
-
-				String title;
-				if (isContent) {
-					String url = Constants.URL_PREFIX + Constants.URL_DETAILS + hash
-							+ ".html?" + Constants.URL_SUFFIX + "&client_ref=activity-"
-							+ VuzeActivitiesConstants.TYPEID_DL_ADDED;
-					title = "<A HREF=\"" + url + "\">"
-							+ PlatformTorrentUtils.getContentTitle2(dm) + "</A>";
-					entry.setAssetHash(hash);
-				} else {
-					title = PlatformTorrentUtils.getContentTitle2(dm);
-				}
-
-				entry.setID(hash + ";a" + addedOn);
-				entry.setText(title + " has been added to your download list");
-				entry.setTimestamp(addedOn);
-				entry.setTypeID(VuzeActivitiesConstants.TYPEID_DL_ADDED, true);
-				entry.setDownloadManager(dm);
-			} catch (Throwable t) {
-				// ignore
-			}
-
-			dm.addListener(dmListener);
-		}
-
 		try {
 			if (isContent) {
+				long networkID = PlatformTorrentUtils.getContentNetworkID(torrent);
 				long completedOn = dm.getDownloadState().getLongParameter(
 						DownloadManagerState.PARAM_DOWNLOAD_COMPLETED_TIME);
 				if (completedOn > 0
+						&& networkID == ConstantsVuze.getDefaultContentNetwork().getID()
 						&& SystemTime.getCurrentTime() - completedOn > RATING_REMINDER_DELAY) {
 					int userRating = PlatformTorrentUtils.getUserRating(torrent);
 					if (userRating < 0) {
@@ -449,11 +332,16 @@ public class VuzeActivitiesManager
 
 						String hash = torrent.getHashWrapper().toBase32String();
 						String title;
-						String url = Constants.URL_PREFIX + Constants.URL_DETAILS + hash
-								+ ".html?" + Constants.URL_SUFFIX + "&client_ref=activity-"
-								+ VuzeActivitiesConstants.TYPEID_RATING_REMINDER;
-						title = "<A HREF=\"" + url + "\">"
-								+ PlatformTorrentUtils.getContentTitle2(dm) + "</A>";
+						ContentNetwork cn = DataSourceUtils.getContentNetwork(dm);
+						if (cn == null) {
+							title = PlatformTorrentUtils.getContentTitle2(dm);
+						} else {
+							String url = cn.getContentDetailsService(hash, "activity-"
+									+ VuzeActivitiesConstants.TYPEID_RATING_REMINDER);
+
+							title = "<A HREF=\"" + url + "\">"
+									+ PlatformTorrentUtils.getContentTitle2(dm) + "</A>";
+						}
 						entry.setAssetHash(hash);
 
 						entry.setDownloadManager(dm);
@@ -473,20 +361,6 @@ public class VuzeActivitiesManager
 		return entries;
 	}
 
-	/**
-	 * Pull entries from webapp
-	 * 
-	 * @param delay max time to wait before running request
-	 *
-	 * @since 3.0.4.3
-	 */
-	public static void pullActivitiesNow(long delay) {
-		PlatformVuzeActivitiesMessenger.getEntries(Math.min(
-				SystemTime.getCurrentTime() - lastVuzeNewsAt, MAX_LIFE_MS), delay,
-				replyListener);
-		lastVuzeNewsAt = SystemTime.getCurrentTime();
-	}
-
 	/**
 	 * Pull entries from webapp
 	 * 
@@ -495,9 +369,41 @@ public class VuzeActivitiesManager
 	 *
 	 * @since 3.0.4.3
 	 */
-	public static void pullActivitiesNow(long agoMS, long delay) {
-		PlatformVuzeActivitiesMessenger.getEntries(agoMS, delay, replyListener);
-		lastVuzeNewsAt = SystemTime.getCurrentTime();
+	public static void pullActivitiesNow(long delay) {
+		ContentNetworkManager cnm = ContentNetworkManagerFactory.getSingleton();
+		if (cnm == null) {
+			return;
+		}
+		
+		ContentNetwork[] contentNetworks = cnm.getContentNetworks();
+		for (ContentNetwork cn : contentNetworks) {
+			if (cn == null) {
+				continue;
+			}
+			
+			Object oIsActive = cn.getPersistentProperty(ContentNetwork.PP_ACTIVE);
+			boolean isActive = (oIsActive instanceof Boolean)
+					? ((Boolean) oIsActive).booleanValue() : false;
+			if (!isActive) {
+				continue;
+			}
+			
+			String id = "" + cn.getID();
+			Long oLastPullTime = lastNewsAt.get(id);
+			long lastPullTime = oLastPullTime != null ? oLastPullTime.longValue() : 0;
+			long now = SystemTime.getCurrentTime();
+			long diff = now - lastPullTime;
+			if (diff > MAX_LIFE_MS) {
+				diff = MAX_LIFE_MS;
+			}
+			PlatformVuzeActivitiesMessenger.getEntries(cn.getID(), diff, delay,
+					replyListener);
+			lastNewsAt.put(id, new Long(now));
+		}
+	}
+	
+	public static void clearLastPullTimes() {
+		lastNewsAt = new HashMap();
 	}
 
 	/**
@@ -512,6 +418,15 @@ public class VuzeActivitiesManager
 		saveEvents();
 	}
 
+	/**
+	 * 
+	 *
+	 * @since 3.1.1.1
+	 */
+	private static void saveEvents() {
+		saveEventsOnClose  = true;
+	}
+
 	/**
 	 * 
 	 *
@@ -523,10 +438,24 @@ public class VuzeActivitiesManager
 		try {
 			Map map = FileUtil.readResilientConfigFile(SAVE_FILENAME);
 
-			lastVuzeNewsAt = MapUtils.getMapLong(map, "LastCheck", 0);
 			long cutoffTime = getCutoffTime();
-			if (lastVuzeNewsAt < cutoffTime) {
-				lastVuzeNewsAt = cutoffTime;
+
+			try {
+				lastNewsAt = MapUtils.getMapMap(map, "LastChecks", new HashMap());
+			} catch (Exception e) {
+				Debug.out(e);
+			}
+
+			// "LastCheck" backward compat
+			if (lastNewsAt.size() == 0) {
+  			long lastVuzeNewsAt = MapUtils.getMapLong(map, "LastCheck", 0);
+  			if (lastVuzeNewsAt > 0) {
+    			if (lastVuzeNewsAt < cutoffTime) {
+    				lastVuzeNewsAt = cutoffTime;
+    			}
+  				lastNewsAt.put("" + ContentNetwork.CONTENT_NETWORK_VUZE, new Long(
+  						lastVuzeNewsAt));
+  			}
 			}
 
 			Object value;
@@ -553,6 +482,7 @@ public class VuzeActivitiesManager
 			}
 
 			List entries = (List) value;
+			List entriesToAdd = new ArrayList(entries.size());
 			for (Iterator iter = entries.iterator(); iter.hasNext();) {
 				value = iter.next();
 				if (!(value instanceof Map)) {
@@ -567,18 +497,21 @@ public class VuzeActivitiesManager
 					}
 
 					if (entry.getTimestamp() > cutoffTime) {
-						addEntries(new VuzeActivitiesEntry[] {
-							entry
-						});
+						entriesToAdd.add(entry);
 					}
 				}
 			}
+
+			int num = entriesToAdd.size();
+			if (num > 0) {
+				addEntries((VuzeActivitiesEntry[]) entriesToAdd.toArray(new VuzeActivitiesEntry[num]));
+			}
 		} finally {
 			skipAutoSave = false;
 		}
 	}
 
-	private static void saveEvents() {
+	private static void saveEventsNow() {
 		if (skipAutoSave) {
 			return;
 		}
@@ -587,7 +520,7 @@ public class VuzeActivitiesManager
 			config_mon.enter();
 
 			Map mapSave = new HashMap();
-			mapSave.put("LastCheck", new Long(lastVuzeNewsAt));
+			mapSave.put("LastChecks", lastNewsAt);
 
 			List entriesList = new ArrayList();
 
@@ -666,12 +599,11 @@ public class VuzeActivitiesManager
 			allEntries_mon.exit();
 		}
 
-		saveEvents();
-		//Collections.sort(allEntries);
-
 		VuzeActivitiesEntry[] newEntriesArray = (VuzeActivitiesEntry[]) newEntries.toArray(new VuzeActivitiesEntry[newEntries.size()]);
 
 		if (newEntriesArray.length > 0) {
+			saveEventsNow();
+
 			Object[] listenersArray = listeners.toArray();
 			for (int i = 0; i < listenersArray.length; i++) {
 				VuzeActivitiesListener l = (VuzeActivitiesListener) listenersArray[i];
@@ -680,6 +612,10 @@ public class VuzeActivitiesManager
 		}
 
 		if (existingEntries.size() > 0) {
+			if (newEntriesArray.length == 0) {
+				saveEvents();
+			}
+
   		for (Iterator iter = existingEntries.iterator(); iter.hasNext();) {
   			VuzeActivitiesEntry entry = (VuzeActivitiesEntry) iter.next();
   			triggerEntryChanged(entry);
@@ -719,7 +655,7 @@ public class VuzeActivitiesManager
 			VuzeActivitiesListener l = (VuzeActivitiesListener) listenersArray[i];
 			l.vuzeNewsEntriesRemoved(entries);
 		}
-		saveEvents();
+		saveEventsNow();
 	}
 
 	public static VuzeActivitiesEntry getEntryByID(String id) {
@@ -746,6 +682,10 @@ public class VuzeActivitiesManager
 	public static VuzeActivitiesEntry[] getAllEntries() {
 		return (VuzeActivitiesEntry[]) allEntries.toArray(new VuzeActivitiesEntry[allEntries.size()]);
 	}
+	
+	public static int getNumEntries() {
+		return allEntries.size();
+	}
 
 	public static void log(String s) {
 		if (diag_logger != null) {
@@ -767,14 +707,20 @@ public class VuzeActivitiesManager
 		saveEvents();
 	}
 
+	public static VuzeActivitiesEntry createEntryFromMap(Map map,
+			boolean internalMap) {
+		return createEntryFromMap(ContentNetwork.CONTENT_NETWORK_VUZE, map,
+				internalMap);
+	}
+
 	/**
 	 * @param map
 	 * @return
 	 *
 	 * @since 3.0.5.3
 	 */
-	public static VuzeActivitiesEntry createEntryFromMap(Map map,
-			boolean internalMap) {
+	public static VuzeActivitiesEntry createEntryFromMap(
+			long defaultContentNetworkID, Map map, boolean internalMap) {
 		VuzeActivitiesEntry entry;
 		String typeID = MapUtils.getMapString(map, "typeID", MapUtils.getMapString(
 				map, "type-id", null));
@@ -782,9 +728,12 @@ public class VuzeActivitiesManager
 			entry = new VuzeActivitiesEntryBuddyRequest();
 		} else if (VuzeActivitiesConstants.TYPEID_BUDDYSHARE.equals(typeID)) {
 			entry = new VuzeActivitiesEntryContentShare();
+		} else if (VuzeActivitiesConstants.TYPEID_BUDDYLINKUP.equals(typeID)) {
+			entry = new VuzeActivitiesEntryBuddyLinkup();
 		} else {
 			entry = new VuzeActivitiesEntry();
 		}
+		entry.setContentNetworkID(defaultContentNetworkID);
 		if (internalMap) {
 			entry.loadFromInternalMap(map);
 		} else {
