@@ -19,18 +19,23 @@
 package com.aelitis.azureus.buddy.impl;
 
 import java.io.File;
+import java.net.MalformedURLException;
+import java.net.URL;
 import java.util.*;
 
 import org.gudy.azureus2.core3.config.COConfigurationManager;
+import org.gudy.azureus2.core3.download.DownloadManager;
 import org.gudy.azureus2.core3.internat.MessageText;
 import org.gudy.azureus2.core3.util.*;
 
 import com.aelitis.azureus.activities.*;
 import com.aelitis.azureus.buddy.*;
 import com.aelitis.azureus.core.AzureusCoreFactory;
+import com.aelitis.azureus.core.cnetwork.ContentNetwork;
 import com.aelitis.azureus.core.crypto.VuzeCryptoException;
 import com.aelitis.azureus.core.crypto.VuzeCryptoListener;
 import com.aelitis.azureus.core.crypto.VuzeCryptoManager;
+import com.aelitis.azureus.core.messenger.PlatformAuthorizedSender;
 import com.aelitis.azureus.core.messenger.PlatformMessenger;
 import com.aelitis.azureus.core.messenger.config.*;
 import com.aelitis.azureus.login.NotLoggedInException;
@@ -39,7 +44,6 @@ import com.aelitis.azureus.ui.UIFunctions;
 import com.aelitis.azureus.ui.UIFunctionsManager;
 import com.aelitis.azureus.ui.selectedcontent.SelectedContentV3;
 import com.aelitis.azureus.util.*;
-import com.aelitis.azureus.util.Constants;
 import com.aelitis.azureus.util.LoginInfoManager.LoginInfo;
 
 import org.gudy.azureus2.plugins.Plugin;
@@ -73,7 +77,9 @@ public class VuzeBuddyManager
 
 	public static final String VMT_BUDDY_MESSAGE = "BuddyMessage";
 
-	public static final String VMT_BUDDYSYNC = "BuddySync"; 
+	public static final String VMT_BUDDYSYNC = "BuddySync";
+
+	private static final String UNKNOWN_MSG_TYPE = "Unknown Message Type"; 
 
 	private static BuddyPlugin buddyPlugin = null;
 
@@ -129,11 +135,24 @@ public class VuzeBuddyManager
 				
 				String response = MapUtils.getMapString(reply, "response", "");
 				
+				if (response.equals(UNKNOWN_MSG_TYPE)) {
+					try {
+						Map map = message.getRequest();
+						VuzeBuddyManager.log("  " + response + "; "
+								+ MapUtils.getMapString(map, "VuzeMessageType", "null"));
+					} catch (Throwable t) {
+					}
+					// fake that we handled it..
+					return true;
+				}
+				
 				if ( !response.toLowerCase().equals("ok")){
 					
 					sendViaRelayServer( message );
 					
 						// false here will re-attempt this call later (and keep message around)
+					// when the sendViaRelayServer succeeds, the message will be removed.
+					// If it fails, the message will be retried later
 					
 					return( false );
 				}
@@ -348,6 +367,9 @@ public class VuzeBuddyManager
 		public void loginUpdate(final LoginInfo info, boolean isNewLoginID) {
 			loginUpdateTriggered(info, isNewLoginID);
 		}
+
+		public void avatarURLUpdated(String newAvatarURL) {
+		};
 	};
 
 	private static VuzeRelayListener vuzeRelayListener = new VuzeRelayListener() {
@@ -418,8 +440,9 @@ public class VuzeBuddyManager
 
 				VuzeBuddy vuzeBuddy = (VuzeBuddy) mapPKtoVuzeBuddy.get(pk);
 				if (vuzeBuddy != null) {
-					//vuzeBuddy.setDisplayName(buddy.getName());
-					triggerChangeListener(vuzeBuddy);
+					// no need to save.. these are things like YGM marker, buddyactive,
+					// fragment rec'd, closerequest, removeconnection, etc
+					triggerChangeListener(vuzeBuddy, false);
 				} else {
 					buddyAdded(buddy);
 				}
@@ -513,7 +536,6 @@ public class VuzeBuddyManager
 		setSaveDelayed(true);
 
 		try {
-			LoginInfoManager.getInstance().addListener(loginInfoListener);
 			log("setupBuddyPlugin");
 
 			boolean newPluginEnabled = false;
@@ -527,7 +549,7 @@ public class VuzeBuddyManager
 					Plugin plugin = pi.getPlugin();
 					if (plugin instanceof BuddyPlugin) {
 						((BuddyPlugin) plugin).addListener(buddyPluginListener);
-						if (!pi.isDisabled()) {
+						if (!pi.getPluginState().isDisabled()) {
 							buddyPlugin = (BuddyPlugin) plugin;
 							newPluginEnabled = buddyPlugin.isEnabled();
 						}
@@ -598,6 +620,8 @@ public class VuzeBuddyManager
 			} catch (Throwable t) {
 				Debug.out(t);
 			}
+
+			LoginInfoManager.getInstance().addListener(loginInfoListener);
 		} finally {
 			setSaveDelayed(false);
 		}
@@ -693,6 +717,27 @@ public class VuzeBuddyManager
   						}
 						}
 					}
+					
+					// @see com.aelitis.azureus.core.messenger.config.PlatformKeyExchangeMessenger.platformPasswordListener#passwordRetrievalFailed()
+					public void passwordRetrievalFailed() {
+						try {
+							final String url = ContentNetworkUtils.getUrl(
+									ConstantsVuze.getDefaultContentNetwork(),
+									ContentNetwork.SERVICE_LOGOUT);
+							// Authorized Sender has the session cookies the webapp needs
+							// in order to logout.  Use that and ignore the results
+							// We can't just load the logout page into the sidebar's browser,
+							// because that will activate it and confuse the user
+							PlatformAuthorizedSender sender = PlatformMessenger.getAuthorizedTransferListener();
+							AESemaphore sem = new AESemaphore("logoutWait");
+							sender.startDownload(new URL(url), null, sem, false);
+							sem.reserve();
+							// since Authorized Sender doesn't have a BrowserContext,
+							// tell the UI directly that there's no login
+							LoginInfoManager.getInstance().logout();
+						} catch (MalformedURLException e) {
+						}
+					}
 				});
 			} catch (NotLoggedInException e) {
 				// ignore.. we should be logged in!
@@ -793,6 +838,7 @@ public class VuzeBuddyManager
 					log(e);
 				}
 			}
+			return "Ok";
 		} else if (mt.equals(VMT_BUDDY_MESSAGE)) {
 			VuzeBuddy buddyByPK = getBuddyByPK(pkSender);
 			String namespace = MapUtils.getMapString(mapPayload, "namespace", null);
@@ -811,7 +857,7 @@ public class VuzeBuddyManager
 		}
 
 		log("processPayLoadMap from " + pkSender + ": Unknown Message Type " + mt);
-		return "Unknown Message Type";
+		return UNKNOWN_MSG_TYPE;
 	}
 
 	/**
@@ -845,7 +891,19 @@ public class VuzeBuddyManager
 	public static BuddyPlugin getBuddyPlugin() {
 		return buddyPlugin;
 	}
-
+	
+	public static boolean
+	canSetPublishedCategory(
+		String		category )
+	{
+		if ( buddyPlugin == null ){
+			
+			return( false );
+		}
+		
+		return( !buddyPlugin.isPublicCategory( category ));
+	}
+	
 	/**
 	 * Retrieve a list of all buddies
 	 * 
@@ -918,7 +976,7 @@ public class VuzeBuddyManager
 	public static void log(String s) {
 		AEDiagnosticsLogger diag_logger = AEDiagnostics.getLogger("v3.Friends");
 		diag_logger.log(s);
-		if (Constants.DIAG_TO_STDOUT) {
+		if (ConstantsVuze.DIAG_TO_STDOUT) {
 			System.out.println(Thread.currentThread().getName() + "|"
 					+ System.currentTimeMillis() + "] " + s);
 		}
@@ -933,7 +991,7 @@ public class VuzeBuddyManager
 	public static void log(Exception e) {
 		AEDiagnosticsLogger diag_logger = AEDiagnostics.getLogger("v3.Friends");
 		diag_logger.log(e);
-		if (Constants.DIAG_TO_STDOUT) {
+		if (ConstantsVuze.DIAG_TO_STDOUT) {
 			System.err.println(Thread.currentThread().getName() + "|"
 					+ System.currentTimeMillis() + "] ");
 			e.printStackTrace();
@@ -1298,7 +1356,7 @@ public class VuzeBuddyManager
 	 * @since 3.0.5.3
 	 */
 	public static void inviteWithShare(final Map invites,
-			final SelectedContentV3 contentToShare, final String shareMessage,
+			final VuzeShareable contentToShare, final String shareMessage,
 			final VuzeBuddy[] buddies)
 			throws NotLoggedInException {
 		if (!LoginInfoManager.getInstance().isLoggedIn()) {
@@ -1316,7 +1374,7 @@ public class VuzeBuddyManager
 	}
 
 	private static void _inviteWithShare(Map invites,
-			SelectedContentV3 contentToShare, String shareMessage,
+			VuzeShareable contentToShare, String shareMessage,
 			VuzeBuddy[] buddies)
 			throws NotLoggedInException {
 
@@ -1326,8 +1384,11 @@ public class VuzeBuddyManager
 
 		String name = "na";
 		if (contentToShare != null) {
-			name = contentToShare.getDM() == null ? contentToShare.getDisplayName()
-					: contentToShare.getDM().toString();
+			
+			DownloadManager dm = contentToShare.getDownloadManager();
+			
+			name = dm == null ? contentToShare.getDisplayName()
+					: dm.toString();
 		}
 
 		if (buddies != null && contentToShare != null) {
@@ -1381,7 +1442,7 @@ public class VuzeBuddyManager
 		}
 	}
 
-	private static void queueShare(SelectedContentV3 content, String message,
+	private static void queueShare(VuzeShareable content, String message,
 			String code) {
 		if (content == null) {
 			return;
@@ -1559,17 +1620,23 @@ public class VuzeBuddyManager
 		}
 	}
 
+	protected static void triggerChangeListener(VuzeBuddy buddy) {
+		triggerChangeListener(buddy, false);
+	}
+
 	/**
 	 * @param buddy
 	 *
 	 * @since 3.0.5.3
 	 */
-	protected static void triggerChangeListener(VuzeBuddy buddy) {
+	protected static void triggerChangeListener(VuzeBuddy buddy, boolean save) {
 		if (!buddyList.contains(buddy)) {
 			return;
 		}
 
-		saveVuzeBuddies();
+		if (save) {
+			saveVuzeBuddies();
+		}
 		Object[] listenersArray = listeners.toArray();
 		for (int i = 0; i < listenersArray.length; i++) {
 			VuzeBuddyListener l = (VuzeBuddyListener) listenersArray[i];
@@ -1600,7 +1667,7 @@ public class VuzeBuddyManager
 			return;
 		}
 
-		log("save");
+		log("save " + Debug.getCompressedStackTrace());
 		Map mapSave = new HashMap();
 		List storedBuddyList = new ArrayList();
 		mapSave.put("buddies", storedBuddyList);
@@ -1720,13 +1787,7 @@ public class VuzeBuddyManager
 		StringBuffer buf = new StringBuffer();
 
 		buf.append("<A HREF=\"");
-		buf.append(Constants.URL_PREFIX);
-		buf.append(Constants.URL_PROFILE);
-		buf.append(UrlUtils.encode(loginID));
-		buf.append("?");
-		buf.append(Constants.URL_SUFFIX);
-		buf.append("&client_ref=");
-		buf.append(UrlUtils.encode(referer));
+		buf.append(ConstantsVuze.getDefaultContentNetwork().getProfileService(loginID,referer));
 		buf.append("\" TITLE=\"");
 		buf.append(displayName);
 		if (!loginID.equals(displayName)) {
