@@ -30,7 +30,8 @@ import org.gudy.azureus2.core3.ipfilter.IpFilter;
 import org.gudy.azureus2.core3.ipfilter.IpFilterManagerFactory;
 import org.gudy.azureus2.core3.util.AEMonitor;
 import org.gudy.azureus2.core3.util.AESemaphore;
-import org.gudy.azureus2.core3.util.AEThread;
+import org.gudy.azureus2.core3.util.AEThread2;
+import org.gudy.azureus2.core3.util.Debug;
 import org.gudy.azureus2.core3.util.HashWrapper;
 import org.gudy.azureus2.core3.util.SimpleTimer;
 import org.gudy.azureus2.core3.util.SystemTime;
@@ -64,16 +65,21 @@ public class
 DHTDBImpl
 	implements DHTDB, DHTDBStats
 {	
+	private static final long MAX_VALUE_LIFETIME	= 3*24*60*60*1000L;
+	
+	
 	private int			original_republish_interval;
 	
 		// the grace period gives the originator time to republish their data as this could involve
 		// some work on their behalf to find closest nodes etc. There's no real urgency here anyway
 	
-	private int			ORIGINAL_REPUBLISH_INTERVAL_GRACE	= 60*60*1000;
+	public static int			ORIGINAL_REPUBLISH_INTERVAL_GRACE	= 60*60*1000;
+	
+	private static final int	PRECIOUS_CHECK_INTERVAL				= 2*60*60*1000;
 	
 	private int			cache_republish_interval;
 	
-	private long		MIN_CACHE_EXPIRY_CHECK_INTERVAL		= 60000;
+	private long		MIN_CACHE_EXPIRY_CHECK_INTERVAL		= 60*1000;
 	private long		last_cache_expiry_check;
 	
 	private static final long	IP_BLOOM_FILTER_REBUILD_PERIOD		= 15*60*1000;
@@ -96,9 +102,11 @@ DHTDBImpl
 	
 	private static final long	MAX_TOTAL_SIZE	= 4*1024*1024;
 	
-	private long		total_size;
-	private long		total_values;
-	private long		total_keys;
+	private int		total_size;
+	private int		total_values;
+	private int		total_keys;
+	private int		total_local_keys;
+	
 	
 	private boolean force_original_republish;
 	
@@ -118,7 +126,19 @@ DHTDBImpl
 		cache_republish_interval		= _cache_republish_interval;
 		logger							= _logger;
 				
-		
+		SimpleTimer.addPeriodicEvent(
+			"DHTDB:precious",
+			PRECIOUS_CHECK_INTERVAL/4,
+			true, // absolute, we don't want effective time changes (computer suspend/resume) to shift these
+			new TimerEventPerformer()
+			{
+				public void
+				perform(
+					TimerEvent	event )
+				{
+					checkPreciousStuff();
+				}
+			});
 		
 		SimpleTimer.addPeriodicEvent(
 			"DHTDB:op",
@@ -248,25 +268,61 @@ DHTDBImpl
 	store(
 		HashWrapper		key,
 		byte[]			value,
-		byte			flags )
+		byte			flags,
+		byte			life_hours )
 	{
 			// local store
 		
-		try{
-			this_mon.enter();
-				
-				// don't police max check for locally stored data
-				// only that received
+		if ( (flags & DHT.FLAG_PUT_AND_FORGET ) == 0 ){
 			
-			DHTDBMapping	mapping = (DHTDBMapping)stored_values.get( key );
-			
-			if ( mapping == null ){
+			if ( life_hours > 0 ){
 				
-				mapping = new DHTDBMapping( this, key, true );
-				
-				stored_values.put( key, mapping );
+				if ( life_hours*60*60*1000 < original_republish_interval ){
+					
+					Debug.out( "Don't put persistent values with a lifetime less than republish period - lifetime over-ridden" );
+					
+					life_hours = 0;
+				}
 			}
 			
+			try{
+				this_mon.enter();
+					
+				total_local_keys++;
+				
+					// don't police max check for locally stored data
+					// only that received
+				
+				DHTDBMapping	mapping = (DHTDBMapping)stored_values.get( key );
+				
+				if ( mapping == null ){
+					
+					mapping = new DHTDBMapping( this, key, true );
+					
+					stored_values.put( key, mapping );
+				}
+				
+				DHTDBValueImpl res =	
+					new DHTDBValueImpl( 
+							SystemTime.getCurrentTime(), 
+							value, 
+							getNextValueVersion(),
+							local_contact, 
+							local_contact,
+							true,
+							flags,
+							life_hours );
+		
+				mapping.add( res );
+				
+				return( res );
+				
+			}finally{
+				
+				this_mon.exit();
+			}
+		}else{
+			
 			DHTDBValueImpl res =	
 				new DHTDBValueImpl( 
 						SystemTime.getCurrentTime(), 
@@ -275,18 +331,25 @@ DHTDBImpl
 						local_contact, 
 						local_contact,
 						true,
-						flags );
-	
-			mapping.add( res );
+						flags,
+						life_hours );
 			
 			return( res );
-			
-		}finally{
-			
-			this_mon.exit();
 		}
 	}
 	
+	/*
+	private long store_ops;
+	private long store_ops_bad1;
+	private long store_ops_bad2;
+	
+	private void
+	logStoreOps()
+	{
+		System.out.println( "sops (" + control.getTransport().getNetwork() + ")=" + store_ops + ",bad1=" + store_ops_bad1 + ",bad2=" + store_ops_bad2 );
+	}
+	*/
+	
 	public byte
 	store(
 		DHTTransportContact 	sender, 
@@ -310,11 +373,25 @@ DHTDBImpl
 			// the correct place to store a value. Part of this will in general have 
 			// needed them to query us for example. Therefore, limit values to those
 			// that are at least as close to us
+			// used to just use K here but this is a little too strict as we end up rejecting
+			// a fair few valid stores - widened to 2*K
 		
-		List closest_contacts = control.getClosestKContactsList( key.getHash(), true );
+			// dropped a bit, especially on CVS DHT due to smallness
+		
+		int	c_factor = router.getK();
+		
+		if ( adapter.getNetwork() != DHT.NW_CVS ){
+			
+			c_factor += ( c_factor/2 );
+		}
+		
+		List closest_contacts = control.getClosestContactsList( key.getHash(), c_factor, true );
 		
 		boolean	store_it	= false;
 		
+		
+		// store_ops++;
+		
 		for (int i=0;i<closest_contacts.size();i++){
 			
 			if ( router.isID(((DHTTransportContact)closest_contacts.get(i)).getID())){
@@ -329,6 +406,9 @@ DHTDBImpl
 			
 			DHTLog.log( "Not storing " + DHTLog.getString2(key.getHash()) + " as key too far away" );
 
+			// store_ops_bad1++;
+			// logStoreOps();
+			
 			return( DHT.DT_NONE );
 		}
 		
@@ -356,7 +436,7 @@ DHTDBImpl
 				
 			byte[]	my_id	= local_contact.getID();
 			
-			closest_contacts = control.getClosestKContactsList( my_id, true );
+			closest_contacts = control.getClosestContactsList( my_id, c_factor, true );
 			
 			DHTTransportContact	furthest = (DHTTransportContact)closest_contacts.get( closest_contacts.size()-1);
 						
@@ -370,9 +450,14 @@ DHTDBImpl
 			
 			DHTLog.log( "Not storing " + DHTLog.getString2(key.getHash()) + " as cache forward and sender too far away" );
 			
+			// store_ops_bad2++;			
+			// logStoreOps();
+			
 			return( DHT.DT_NONE );
 		}
 		
+		// logStoreOps();
+		
 		try{
 			this_mon.enter();
 						
@@ -527,6 +612,8 @@ DHTDBImpl
 				
 				if ( res != null ){
 					
+					total_local_keys--;
+					
 					return( res.getValueForDeletion( getNextValueVersion()));
 				}
 				
@@ -633,6 +720,24 @@ DHTDBImpl
 		return( (int)total_keys );
 	}
 	
+	public int
+	getLocalKeyCount()
+	{
+		return( total_local_keys );
+	}
+	
+	public int
+	getValueCount()
+	{
+		return( (int)total_values );
+	}
+	
+	public int
+	getSize()
+	{
+		return( (int)total_size );
+	}
+	
 	public int[]
 	getValueDetails()
 	{
@@ -1086,6 +1191,11 @@ DHTDBImpl
 						continue;
 					}
 					
+					if ( router.isID( contact.getID())){
+						
+						continue;	// ignore ourselves
+					}
+
 					if ( contact.getProtocolVersion() >= DHTTransportUDP.PROTOCOL_VERSION_BLOCK_KEYS ){
 						
 						final Runnable task = 
@@ -1185,11 +1295,11 @@ DHTDBImpl
 				DHTDBMapping	mapping = (DHTDBMapping)it.next();
 	
 				if ( mapping.getValueCount() == 0 ){
+										
+					it.remove();
 					
 					mapping.destroy();
-					
-					it.remove();
-										
+
 				}else{
 					
 					Iterator	it2 = mapping.getValues();
@@ -1204,7 +1314,25 @@ DHTDBImpl
 								// when deciding whether or not to remove this, plus a bit, as the 
 								// original publisher is supposed to republish these
 							
-							if ( now - value.getCreationTime() > original_republish_interval + ORIGINAL_REPUBLISH_INTERVAL_GRACE ){
+							int life_hours = value.getLifeTimeHours();
+							
+							long	max_age;
+							
+							if ( life_hours < 1 ){
+								
+								max_age = original_republish_interval;
+								
+							}else{
+								
+								max_age = life_hours * 60*60*1000;
+								
+								if ( max_age > MAX_VALUE_LIFETIME ){
+									
+									max_age = MAX_VALUE_LIFETIME;
+								}
+							}
+							
+							if ( now > value.getCreationTime() + max_age + ORIGINAL_REPUBLISH_INTERVAL_GRACE ){
 								
 								DHTLog.log( "removing cache entry (" + value.getString() + ")" );
 								
@@ -1220,6 +1348,81 @@ DHTDBImpl
 		}
 	}
 	
+	protected void
+	checkPreciousStuff()
+	{
+		long	 now = SystemTime.getCurrentTime();
+		
+		Map	republish = new HashMap();
+
+		try{
+
+			this_mon.enter();
+						
+			Iterator	it = stored_values.entrySet().iterator();
+			
+			while( it.hasNext()){
+				
+				Map.Entry	entry = (Map.Entry)it.next();
+				
+				HashWrapper		key		= (HashWrapper)entry.getKey();
+				
+				DHTDBMapping	mapping	= (DHTDBMapping)entry.getValue();
+
+				Iterator	it2 = mapping.getValues();
+				
+				List	values = new ArrayList();
+
+				while( it2.hasNext()){
+					
+					DHTDBValueImpl	value = (DHTDBValueImpl)it2.next();				
+
+					if ( value.isLocal()){
+						
+						if (( value.getFlags() | DHT.FLAG_PRECIOUS ) != 0 ){
+							
+							if ( now - value.getCreationTime() > PRECIOUS_CHECK_INTERVAL ){
+								
+								value.setCreationTime();
+
+								values.add( value );
+							}
+						}
+					}
+				}
+				
+				if ( values.size() > 0 ){
+					
+					republish.put( key, values );
+					
+				}
+			}
+		}finally{
+			
+			this_mon.exit();
+		}
+		
+		
+		Iterator	it = republish.entrySet().iterator();
+		
+		while( it.hasNext()){
+			
+			Map.Entry	entry = (Map.Entry)it.next();
+			
+			HashWrapper			key		= (HashWrapper)entry.getKey();
+			
+			List		values	= (List)entry.getValue();
+			
+				// no point in worry about multi-value puts here as it is extremely unlikely that
+				// > 1 value will locally stored, or > 1 value will go to the same contact
+			
+			for (int i=0;i<values.size();i++){
+								
+				control.putEncodedKey( key.getHash(), "Precious republish", (DHTDBValueImpl)values.get(i), 0, true );
+			}
+		}
+	}
+	
 	protected DHTTransportContact
 	getLocalContact()
 	{
@@ -1246,7 +1449,8 @@ DHTDBImpl
 	}
 	
 	public void
-	print()
+	print(
+		boolean	full )
 	{
 		Map	count = new TreeMap();
 		
@@ -1255,6 +1459,11 @@ DHTDBImpl
 			
 			logger.log( "Stored keys = " + stored_values.size() + ", values = " + getValueDetails()[DHTDBStats.VD_VALUE_COUNT]); 
 
+			if ( !full ){
+				
+				return;
+			}
+			
 			Iterator	it = stored_values.entrySet().iterator();
 			
 			while( it.hasNext()){
@@ -1265,6 +1474,8 @@ DHTDBImpl
 				
 				DHTDBMapping	mapping = (DHTDBMapping)entry.getValue();
 				
+				// mapping.print();
+				
 				DHTDBValue[]	values = mapping.get(null,0,(byte)0);
 					
 				for (int i=0;i<values.length;i++){
@@ -1350,11 +1561,17 @@ DHTDBImpl
 	banContact(
 		final DHTTransportContact	contact,
 		final String				reason )
-	{
-		new AEThread( "DHTDBImpl:delayed flood delete", true )
+	{		
+		// CVS DHT can be significantly smaller than mainline (e.g. 1000) so will trigger
+		// un-necessary banning which then obviously affects the main DHTs. So we disable
+		// banning for CVS
+
+		final boolean ban_ip = control.getTransport().getNetwork() != DHT.NW_CVS;
+		
+		new AEThread2( "DHTDBImpl:delayed flood delete", true )
 		{
 			public void
-			runSupport()
+			run()
 			{
 					// delete their data on a separate thread so as not to 
 					// interfere with the current action
@@ -1363,27 +1580,43 @@ DHTDBImpl
 					this_mon.enter();
 					
 					Iterator	it = stored_values.values().iterator();
-												
+						
+					boolean	overall_deleted = false;
+					
+					HashWrapper value_id = new HashWrapper( contact.getID());
+					
 					while( it.hasNext()){
 						
 						DHTDBMapping	mapping = (DHTDBMapping)it.next();
-
-						Iterator	it2 = mapping.getDirectValues();
 						
-						while( it2.hasNext()){
+						boolean	deleted = false;
+						
+						if ( mapping.removeDirectValue( value_id ) != null ){
 							
-							DHTDBValueImpl	val = (DHTDBValueImpl)it2.next();
+							deleted = true;
+						}
+	
+						if ( mapping.removeIndirectValue( value_id ) != null ){
 							
-							if ( !val.isLocal()){
-								
-								if ( Arrays.equals( val.getOriginator().getID(), contact.getID())){
-									
-									it.remove();
-								}
-							}
+							deleted = true;
+						}
+
+						
+						if ( deleted && !ban_ip ){
+						
+								// if we're not banning then rebuild bloom to avoid us continually
+								// going through this ban code
+							
+							mapping.rebuildIPBloomFilter( false );
+							
+							overall_deleted = true;
 						}
 					}
 
+					if ( overall_deleted && !ban_ip ){
+						
+						rebuildIPBloomFilter( false );
+					}
 				}finally{
 					
 					this_mon.exit();
@@ -1392,11 +1625,14 @@ DHTDBImpl
 			}
 		}.start();
 	
-		logger.log( "Banning " + contact.getString() + " due to store flooding (" + reason + ")" );
-		
-		ip_filter.ban( 
-				contact.getAddress().getAddress().getHostAddress(),
-				"DHT: Sender stored excessive entries at this node (" + reason + ")", false );		
+		if ( ban_ip ){	
+
+			logger.log( "Banning " + contact.getString() + " due to store flooding (" + reason + ")" );
+			
+			ip_filter.ban( 
+					contact.getAddress().getAddress().getHostAddress(),
+					"DHT: Sender stored excessive entries at this node (" + reason + ")", false );
+		}
 	}
 	
 	protected void
@@ -1410,6 +1646,8 @@ DHTDBImpl
 			// this gives 6 values per address
 			// with a factor of 10 error this is still only 60 per address
 		
+			// However, for CVS DHTs we can have sizes of 1000 or less. 
+		
 		int	hit_count = ip_count_bloom_filter.add( contact.getAddress().getAddress().getAddress());
 		
 		if ( DHTLog.GLOBAL_BLOOM_TRACE ){
@@ -1426,8 +1664,8 @@ DHTDBImpl
 		
 		if ( hit_count > 64 ){
 			
-			// obviously being spammed, drop all data originated by this IP and ban it
-			
+				// obviously being spammed, drop all data originated by this IP and ban it
+						
 			banContact( contact, "global flood" );
 		}
 	}
@@ -1611,6 +1849,11 @@ DHTDBImpl
 			Debug.out( "Actual keys != total: " + actual_keys + "/" + total_keys );
 		}
 		
+		if ( adapter.getKeyCount() != actual_keys ){
+			
+			Debug.out( "SM keys != total: " + actual_keys + "/" + adapter.getKeyCount());
+		}
+		
 		if ( actual_values != total_values ){
 			
 			Debug.out( "Actual values != total: " + actual_values + "/" + total_values );
@@ -1621,6 +1864,11 @@ DHTDBImpl
 			Debug.out( "Actual size != total: " + actual_size + "/" + total_size );
 		}
 		
+		if ( actual_values < actual_keys ){
+			
+			Debug.out( "Actual values < actual keys: " + actual_values + "/" + actual_keys );
+		}
+		
 		System.out.println( "DHTDB: " + op + " - keys=" + total_keys + ", values=" + total_values + ", size=" + total_size );
 		*/
 	}
@@ -1674,6 +1922,12 @@ DHTDBImpl
 			delegate = _delegate;
 		}
 		
+		public int 
+		getNetwork() 
+		{
+			return( delegate.getNetwork());
+		}
+		
 		public DHTStorageKey
 		keyCreated(
 			HashWrapper		key,
@@ -1693,10 +1947,16 @@ DHTDBImpl
 			DHTStorageKey	adapter_key )
 		{
 			total_keys--;
-			
-			reportSizes( "keyDeleted" );
-			
+						
 			delegate.keyDeleted( adapter_key );
+
+			reportSizes( "keyDeleted" );
+		}
+		
+		public int 
+		getKeyCount() 
+		{
+			return( delegate.getKeyCount());
 		}
 		
 		public void
@@ -1794,9 +2054,10 @@ DHTDBImpl
 		getExistingDiversification(
 			byte[]			key,
 			boolean			put_operation,
-			boolean			exhaustive_get )
+			boolean			exhaustive_get,
+			int				max_depth )
 		{
-			return( delegate.getExistingDiversification( key, put_operation, exhaustive_get ));
+			return( delegate.getExistingDiversification( key, put_operation, exhaustive_get, max_depth ));
 		}
 		
 		public byte[][]
@@ -1805,9 +2066,10 @@ DHTDBImpl
 			byte[]				key,
 			boolean				put_operation,
 			byte				diversification_type,
-			boolean				exhaustive_get )
+			boolean				exhaustive_get,
+			int					max_depth )
 		{
-			return( delegate.createNewDiversification( cause, key, put_operation, diversification_type, exhaustive_get ));
+			return( delegate.createNewDiversification( cause, key, put_operation, diversification_type, exhaustive_get, max_depth ));
 		}
 		
 		public int
@@ -1860,5 +2122,17 @@ DHTDBImpl
 		{
 			return( delegate.getStorageForKey(key));
 		}
+		
+		public int
+		getRemoteFreqDivCount()
+		{
+			return( delegate.getRemoteFreqDivCount());
+		}
+		
+		public int
+		getRemoteSizeDivCount()
+		{
+			return( delegate.getRemoteSizeDivCount());
+		}
 	}
 }
