@@ -113,7 +113,8 @@ DHTImpl
 						boolean				existing,
 						byte[]				key,
 						byte				type,
-						boolean				exhaustive )
+						boolean				exhaustive,
+						int					max_depth )
 					{
 						boolean	valid;
 						
@@ -132,11 +133,11 @@ DHTImpl
 							
 							if ( existing ){
 								
-								return( storage_adapter.getExistingDiversification( key, put_operation, exhaustive ));
+								return( storage_adapter.getExistingDiversification( key, put_operation, exhaustive, max_depth ));
 								
 							}else{
 								
-								return( storage_adapter.createNewDiversification( cause, key, put_operation, type, exhaustive ));
+								return( storage_adapter.createNewDiversification( cause, key, put_operation, type, exhaustive, max_depth ));
 							}
 						}else{
 							
@@ -209,7 +210,32 @@ DHTImpl
 		byte					flags,
 		DHTOperationListener	listener )
 	{
-		control.put( key, description, value, flags, listener );
+		control.put( key, description, value, flags, (byte)0, true, listener );
+	}
+	
+	public void
+	put(
+		byte[]					key,
+		String					description,
+		byte[]					value,
+		byte					flags,
+		boolean					high_priority,
+		DHTOperationListener	listener )
+	{
+		control.put( key, description, value, flags, (byte)0, high_priority, listener );
+	}
+	
+	public void
+	put(
+		byte[]					key,
+		String					description,
+		byte[]					value,
+		byte					flags,
+		byte					life_hours,
+		boolean					high_priority,
+		DHTOperationListener	listener )
+	{
+		control.put( key, description, value, flags, life_hours, high_priority, listener );
 	}
 	
 	public DHTTransportValue
@@ -350,8 +376,9 @@ DHTImpl
 	}
 	
 	public void
-	print()
+	print(
+		boolean	full )
 	{
-		control.print();
+		control.print( full );
 	}
 }
