@@ -25,6 +25,7 @@ package com.aelitis.azureus.core.dht.impl;
 import com.aelitis.azureus.core.AzureusCoreFactory;
 import com.aelitis.azureus.core.dht.*;
 import com.aelitis.azureus.core.dht.control.DHTControlContact;
+import com.aelitis.azureus.core.dht.db.impl.DHTDBImpl;
 import com.aelitis.azureus.core.dht.nat.DHTNATPuncherAdapter;
 import com.aelitis.azureus.core.dht.nat.impl.DHTNATPuncherImpl;
 import com.aelitis.azureus.core.dht.transport.*;
@@ -43,6 +44,7 @@ import java.security.spec.RSAPrivateKeySpec;
 import java.util.*;
 
 import org.gudy.azureus2.core3.util.AEThread;
+import org.gudy.azureus2.core3.util.ByteFormatter;
 import org.gudy.azureus2.core3.util.SHA1Simple;
 import org.gudy.azureus2.core3.util.Timer;
 import org.gudy.azureus2.core3.util.TimerEvent;
@@ -85,8 +87,10 @@ Test
 	int		fail_percentage	= 00;
 	
 	static Properties	dht_props = new Properties();
+	
+	static{		
+		DHTDBImpl.ORIGINAL_REPUBLISH_INTERVAL_GRACE = 0;
 
-	static{
 		dht_props.put( DHT.PR_CONTACTS_PER_NODE, new Integer(K));
 		dht_props.put( DHT.PR_NODE_SPLIT_FACTOR, new Integer(B));
 		dht_props.put( DHT.PR_CACHE_REPUBLISH_INTERVAL, new Integer(30000));
@@ -325,8 +329,8 @@ Test
 					
 					DHT	dht = dhts[dht_index];
 					
-					String	lhs = str.substring(0,pos);
-					String	rhs = str.substring(pos+1);
+					String	lhs = str.substring(0,pos).trim();
+					String	rhs = str.substring(pos+1).trim();
 					
 					DHTTransportStats	stats_before 	= null;
 					
@@ -349,7 +353,37 @@ Test
 							String	key = rhs.substring(0,pos);
 							String	val = rhs.substring(pos+1);
 							
-							dht.put( key.getBytes(), "", val.getBytes(), (byte)(Math.random()*255), new DHTOperationAdapter() );
+							pos = val.indexOf( ' ' );
+							
+							byte flags 	= 0;
+							byte life 	= 0;
+							
+							if ( pos != -1 ){
+								
+								String	opts = val.substring( pos+1 );
+								
+								String[] x = opts.split( "," );
+								
+								for ( String s: x ){
+									
+									String[] y = s.split("=");
+									
+									String	opt = y[0];
+									
+									if ( opt.equals( "f" )){
+										
+										flags = (byte)Integer.parseInt(y[1]);
+										
+									}else if ( opt.equals( "l" )){
+										
+										life = (byte)Integer.parseInt(y[1]);
+									}
+								}
+								
+								val = val.substring(0,pos);
+							}
+							
+							dht.put( key.getBytes(), "", val.getBytes(), flags, life, false, new DHTOperationAdapter() );
 						}
 					}else if ( command == 'x' ){
 						
@@ -403,7 +437,7 @@ Test
 										DHTTransportContact	contact,
 										DHTTransportValue	value )
 									{
-										System.out.println( "-> " + new String(value.getValue()));
+										System.out.println( "-> " + getString( value ));
 									}
 																	
 									public void
@@ -431,7 +465,7 @@ Test
 										DHTTransportContact	contact,
 										DHTTransportValue	value )
 									{
-										System.out.println( "-> " + new String(value.getValue()) + ", flags=" + value.getFlags());
+										System.out.println( "-> " + getString( value ));
 										
 										try{
 											DHTStorageKeyStats	stats = f_dht.getStorageAdapter().deserialiseStats( new DataInputStream( new ByteArrayInputStream( value.getValue())));
@@ -482,7 +516,8 @@ Test
 									}
 									
 									public void
-									diversified()
+									diversified(
+										String		desc )
 									{
 									}
 									
@@ -536,7 +571,7 @@ Test
 	
 							stats_before = dht.getTransport().getStats().snapshot();
 							
-							dht.print();
+							dht.print( true );
 	
 							List	l = dht.getControl().getContacts();
 							
@@ -645,7 +680,7 @@ Test
 						
 						dht.integrate( true );
 	
-						dht.print();
+						dht.print( true );
 						
 					}else if ( command == 'r' ){
 						
@@ -870,6 +905,16 @@ Test
 		}
 	}
 	
+	protected String
+	getString(
+		DHTTransportValue		value )
+	{
+		return( new String( value.getValue()) + 
+				"; flags=" + value.getFlags() +
+				"; life=" + value.getLifeTimeHours() +
+				", orig=" + value.getOriginator().getAddress());
+	}
+	
 	protected void
 	createDHT(
 		DHT[]			dhts,
