@@ -652,7 +652,8 @@ DHTNATPuncherImpl
       										{}
       										
       										public void
-      										diversified()
+      										diversified(
+      											String		desc )
       										{
       										}
       										
@@ -701,7 +702,8 @@ DHTNATPuncherImpl
       										{}
       										
       										public void
-      										diversified()
+      										diversified(
+      											String		desc )
       										{
       										}
       										
@@ -753,7 +755,8 @@ DHTNATPuncherImpl
       									{}
       									
       									public void
-      									diversified()
+      									diversified(
+      										String		desc )
       									{
       									}
       									
@@ -1263,8 +1266,11 @@ DHTNATPuncherImpl
 	}
 	
 	
-	
-	protected int
+
+    /**
+     * XXX: unused
+     */
+    private int
 	sendQuery(
 		DHTTransportContact	target )
 	{
