@@ -23,21 +23,28 @@
 package com.aelitis.azureus.core.dht.transport.udp.impl;
 
 import java.io.*;
+import java.net.Inet4Address;
+import java.net.Inet6Address;
 import java.net.InetAddress;
 import java.net.InetSocketAddress;
 import java.security.SecureRandom;
 import java.util.*;
 
+import org.gudy.azureus2.core3.config.COConfigurationManager;
 import org.gudy.azureus2.core3.internat.MessageText;
 import org.gudy.azureus2.core3.ipfilter.IpFilter;
 import org.gudy.azureus2.core3.ipfilter.IpFilterManagerFactory;
 import org.gudy.azureus2.core3.util.AEMonitor;
+import org.gudy.azureus2.core3.util.AERunnable;
 import org.gudy.azureus2.core3.util.AESemaphore;
-import org.gudy.azureus2.core3.util.AEThread;
+import org.gudy.azureus2.core3.util.AEThread2;
 import org.gudy.azureus2.core3.util.Average;
 import org.gudy.azureus2.core3.util.ByteFormatter;
+import org.gudy.azureus2.core3.util.Constants;
 import org.gudy.azureus2.core3.util.Debug;
+import org.gudy.azureus2.core3.util.DelayedEvent;
 import org.gudy.azureus2.core3.util.HashWrapper;
+import org.gudy.azureus2.core3.util.RandomUtils;
 import org.gudy.azureus2.core3.util.SimpleTimer;
 import org.gudy.azureus2.core3.util.SystemTime;
 import org.gudy.azureus2.core3.util.TimerEvent;
@@ -57,6 +64,8 @@ import com.aelitis.azureus.core.dht.transport.udp.impl.packethandler.DHTUDPPacke
 import com.aelitis.azureus.core.dht.transport.udp.impl.packethandler.DHTUDPPacketReceiver;
 import com.aelitis.azureus.core.dht.transport.udp.impl.packethandler.DHTUDPRequestHandler;
 import com.aelitis.azureus.core.dht.transport.util.DHTTransportRequestCounter;
+import com.aelitis.azureus.core.util.average.AverageFactory;
+import com.aelitis.azureus.core.util.average.MovingImmediateAverage;
 import com.aelitis.azureus.core.util.bloom.BloomFilter;
 import com.aelitis.azureus.core.util.bloom.BloomFilterFactory;
 import com.aelitis.net.udp.uc.PRUDPPacketHandler;
@@ -72,12 +81,18 @@ DHTTransportUDPImpl
 {
 	public static boolean TEST_EXTERNAL_IP	= false;
 	
-	public static final int	TRANSFER_QUEUE_MAX	= 64;
-	
+	public static final int	TRANSFER_QUEUE_MAX			= 64;
+	public static final long MAX_TRANSFER_QUEUE_BYTES	= 8*1024*1024;	 
+		
 	public static final long	WRITE_XFER_RESEND_DELAY		= 12500;
 	public static final long	READ_XFER_REREQUEST_DELAY	= 5000;
 	public static final long	WRITE_REPLY_TIMEOUT			= 60000;		
 	
+	public static final int		MIN_ADDRESS_CHANGE_PERIOD_INIT_DEFAULT	= 5*60*1000;
+	public static final int		MIN_ADDRESS_CHANGE_PERIOD_NEXT_DEFAULT	= 10*60*1000;
+	
+	public static final int		STORE_TIMEOUT_MULTIPLIER = 2;
+	
 	private static boolean	XFER_TRACE	= false;
 	
 	static{
@@ -88,9 +103,11 @@ DHTTransportUDPImpl
 	
 	
 	private String				external_address;
+	private int					min_address_change_period = MIN_ADDRESS_CHANGE_PERIOD_INIT_DEFAULT;
 	
 	private byte				protocol_version;
 	private int					network;
+	private boolean				v6;
 	private String				ip_override;
 	private int					port;
 	private int					max_fails_for_live;
@@ -114,6 +131,9 @@ DHTTransportUDPImpl
 	private Map	read_transfers		= new HashMap();
 	private Map write_transfers		= new HashMap();
 	
+	private int 	active_write_queue_processor_count;
+	private long	total_bytes_on_transfer_queues;
+	
 	private Map	call_transfers		= new HashMap();
 	
 	private long last_address_change;
@@ -129,7 +149,7 @@ DHTTransportUDPImpl
 	
 	
 	private static final int CONTACT_HISTORY_MAX 		= 32;
-	private static final int CONTACT_HISTORY_PING_SIZE	= 16;
+	private static final int CONTACT_HISTORY_PING_SIZE	= 24;
 	
 	private Map	contact_history = 
 		new LinkedHashMap(CONTACT_HISTORY_MAX,0.75f,true)
@@ -156,8 +176,9 @@ DHTTransportUDPImpl
 		};
 			
 		
-	private long	other_routable_total;
-	private long	other_non_routable_total;
+	private long					other_routable_total;
+	private long					other_non_routable_total;
+	private MovingImmediateAverage	routeable_percentage_average = AverageFactory.MovingImmediateAverage(8);
 	
 	private static final int RECENT_REPORTS_HISTORY_MAX = 32;
 
@@ -193,10 +214,14 @@ DHTTransportUDPImpl
 	
 	private AEMonitor	this_mon	= new AEMonitor( "DHTTransportUDP" );
 
+	private boolean		initial_address_change_deferred;
+	private boolean		address_changing;
+	
 	public
 	DHTTransportUDPImpl(
 		byte			_protocol_version,
 		int				_network,
+		boolean			_v6,
 		String			_ip,
 		String			_default_ip,
 		int				_port,
@@ -213,6 +238,7 @@ DHTTransportUDPImpl
 	{
 		protocol_version		= _protocol_version;
 		network					= _network;
+		v6						= _v6;
 		ip_override				= _ip;
 		port					= _port;
 		max_fails_for_live		= _max_fails_for_live;
@@ -224,10 +250,10 @@ DHTTransportUDPImpl
 		reachable				= _initial_reachability;
 		logger					= _logger;
 				
-		store_timeout			= request_timeout * 2;
+		store_timeout			= request_timeout * STORE_TIMEOUT_MULTIPLIER;
 		
 		try{
-			random = new SecureRandom();
+			random = RandomUtils.SECURE_RANDOM;
 			
 		}catch( Throwable e ){
 			
@@ -236,6 +262,13 @@ DHTTransportUDPImpl
 			logger.log( e );
 		}
 		
+		int last_pct = COConfigurationManager.getIntParameter( "dht.udp.net" + network + ".routeable_pct", -1 );
+		
+		if ( last_pct > 0 ){
+			
+			routeable_percentage_average.update( last_pct );
+		}
+		
 		createPacketHandler();
 		
 		SimpleTimer.addPeriodicEvent(
@@ -243,15 +276,17 @@ DHTTransportUDPImpl
 			STATS_PERIOD,
 			new TimerEventPerformer()
 			{
+				private int tick_count;
+				
 				public void
 				perform(
 					TimerEvent	event )
 				{
-					updateStats();
+					updateStats( tick_count++);
 				}
 			});
 		
-		String	default_ip = _default_ip==null?"127.0.0.1":_default_ip;
+		String	default_ip = _default_ip==null?(v6?"::1":"127.0.0.1"):_default_ip;
 				
 		getExternalAddress( default_ip, logger );
 		
@@ -296,7 +331,7 @@ DHTTransportUDPImpl
 		
 		if ( stats == null ){
 			
-			stats =  new DHTTransportUDPStatsImpl( protocol_version, packet_handler.getStats());
+			stats =  new DHTTransportUDPStatsImpl( this, protocol_version, packet_handler.getStats());
 			
 		}else{
 			
@@ -305,7 +340,8 @@ DHTTransportUDPImpl
 	}
 	
 	protected void
-	updateStats()
+	updateStats(
+		int	tick_count )
 	{
 		long	alien_count	= 0;
 		
@@ -335,13 +371,21 @@ DHTTransportUDPImpl
 				// only fiddle with the initial view of reachability when things have had
 				// time to stabilise
 			
+			if ( Constants.isCVSVersion()){
+							
+				long fv_average 		= alien_fv_average.getAverage();
+				long all_average 		= alien_average.getAverage();
+			
+				logger.log( "Aliens for net " + network + ": " + fv_average + "/" + all_average );
+			}
+			
 			if ( now - stats_start_time > STATS_INIT_PERIOD ){
 				
 				reachable_accurate	= true;
 				
 				boolean	old_reachable	= reachable;
 				
-				if ( alien_fv_average.getAverage() > 0 ){
+				if ( alien_fv_average.getAverage() > 1 ){
 					
 					reachable	= true;
 					
@@ -370,11 +414,29 @@ DHTTransportUDPImpl
 			}
 		}
 		
+		int	pct = getRouteablePercentage();
+			
+		if ( pct > 0 ){
+			
+			COConfigurationManager.setParameter("dht.udp.net" + network + ".routeable_pct", pct );
+		}
+		
 		// System.out.println( "routables=" + other_routable_total + ", non=" + other_non_routable_total );
 		
 		// System.out.println( "net " + network + ": aliens = " + alien_average.getAverage() + ", alien fv = " + alien_fv_average.getAverage());
 	}
 	
+	protected void
+	recordSkew(
+		InetSocketAddress	originator_address,
+		long				skew )
+	{
+		if ( stats != null ){
+		
+			stats.recordSkew( originator_address, skew );
+		}
+	}
+	
 	protected int
 	getNodeStatus()
 	{
@@ -433,6 +495,27 @@ DHTTransportUDPImpl
 		setLocalContact();
 	}
 	
+	public long
+	getTimeout()
+	{
+		return( request_timeout );
+	}
+	
+	public void
+	setTimeout(
+		long		timeout )
+	{
+		if ( request_timeout == timeout ){
+			
+			return;
+		}
+		
+		request_timeout = timeout;
+		store_timeout   = request_timeout * STORE_TIMEOUT_MULTIPLIER;
+		
+		packet_handler.setDelays( dht_send_delay, dht_receive_delay, (int)request_timeout );
+	}
+	
 	public int
 	getNetwork()
 	{
@@ -485,7 +568,7 @@ DHTTransportUDPImpl
 			DHTTransportUDPContactImpl c1 = (DHTTransportUDPContactImpl)it.next();
 			DHTTransportUDPContactImpl c2 = (DHTTransportUDPContactImpl)it.next();
 			
-			externalAddressChange( c1, c2.getExternalAddress());
+			externalAddressChange( c1, c2.getExternalAddress(), true );
 			//externalAddressChange( c, new InetSocketAddress( "192.168.0.7", 6881 ));
 			
 		}catch( Throwable e ){
@@ -519,7 +602,7 @@ DHTTransportUDPImpl
 				
 				if ( TEST_EXTERNAL_IP ){
 					
-					new_external_address	= "127.0.0.1";
+					new_external_address	= v6?"::1":"127.0.0.1";
 					
 					log.log( "    External IP address obtained from test data: " + new_external_address );
 				}
@@ -607,7 +690,7 @@ DHTTransportUDPImpl
 				
 				if ( new_external_address == null ){
 			
-					InetAddress public_address = logger.getPluginInterface().getUtilities().getPublicAddress();
+					InetAddress public_address = logger.getPluginInterface().getUtilities().getPublicAddress( v6 );
 											
 					if ( public_address != null ){
 							
@@ -660,8 +743,9 @@ DHTTransportUDPImpl
 		
 	protected void
 	externalAddressChange(
-		DHTTransportUDPContactImpl	reporter,
-		InetSocketAddress			new_address )
+		final DHTTransportUDPContactImpl	reporter,
+		final InetSocketAddress				new_address,
+		boolean								force )
 	
 		throws DHTTransportException
 	{
@@ -690,6 +774,18 @@ DHTTransportUDPImpl
 			throw( new DHTTransportException( "Address '" + new_address + "' is unresolved" ));
 		}
 		
+			// dump addresses incompatible with our protocol
+		
+		if ( 	( ia instanceof Inet4Address && v6  ) ||
+				( ia instanceof Inet6Address && !v6 )){
+			
+				// reduce debug spam, just return
+			
+			// throw( new DHTTransportException( "Address " + new_address + " is incompatible with protocol family for " + external_address ));
+			
+			return;
+		}
+		
 		final String	new_ip = ia.getHostAddress();
 		
 		if ( new_ip.equals( external_address )){
@@ -705,7 +801,36 @@ DHTTransportUDPImpl
 	
 			long	now = SystemTime.getCurrentTime();
 	
-			if ( now - last_address_change < 5*60*1000 ){
+			if ( now - last_address_change < min_address_change_period ){
+				
+				return;
+			}
+						
+			if ( contact_history.size() < CONTACT_HISTORY_MAX && !force ){
+				
+				if ( !initial_address_change_deferred ){
+					
+					initial_address_change_deferred = true;
+					
+					logger.log( "Node " + reporter.getString() + " has reported that the external IP address is '" + new_address + "': deferring new checks" );
+
+					new DelayedEvent(
+						"DHTTransportUDP:delayAC",
+						30*1000,
+						new AERunnable()
+						{
+							public void
+							runSupport()
+							{
+								try{
+									externalAddressChange( reporter, new_address, true );
+									
+								}catch( Throwable e ){
+									
+								}
+							}
+						});
+				}
 				
 				return;
 			}
@@ -734,6 +859,12 @@ DHTTransportUDPImpl
 			
 			last_address_change	= now;
 			
+				// bump up min period for subsequent changes 
+			
+			if ( min_address_change_period == MIN_ADDRESS_CHANGE_PERIOD_INIT_DEFAULT ){
+				
+				min_address_change_period = MIN_ADDRESS_CHANGE_PERIOD_NEXT_DEFAULT;
+			}
 		}finally{
 			
 			this_mon.exit();
@@ -746,22 +877,51 @@ DHTTransportUDPImpl
 			// (the test attempts to ping contacts)
 			
 		
-		new AEThread( "DHTTransportUDP:getAddress", true )
+		new AEThread2( "DHTTransportUDP:getAddress", true )
 		{
 			public void
-			runSupport()
+			run()
 			{
-				getExternalAddress( new_ip, logger );
-				
-				if ( old_external_address.equals( external_address )){
+				try{
+					this_mon.enter();
+
+					if ( address_changing ){
+						
+						return;
+					}
 					
-						// address hasn't changed, notifier must be perceiving different address
-						// due to proxy or something
-									
-					return;
+					address_changing	= true;
+					
+				}finally{
+					
+					this_mon.exit();
+				}
+					
+				try{
+					getExternalAddress( new_ip, logger );
+					
+					if ( old_external_address.equals( external_address )){
+						
+							// address hasn't changed, notifier must be perceiving different address
+							// due to proxy or something
+										
+						return;
+					}
+					
+					setLocalContact();
+					
+				}finally{
+					
+					try{
+						this_mon.enter();
+
+						address_changing	= false;
+						
+					}finally{
+						
+						this_mon.exit();
+					}
 				}
-				
-				setLocalContact();
 			}
 		}.start();
 	}
@@ -804,7 +964,8 @@ DHTTransportUDPImpl
 	protected void
 	updateContactStatus(
 		DHTTransportUDPContactImpl	contact,
-		int							status )
+		int							status,
+		boolean						incoming )
 	{
 		try{
 			this_mon.enter();
@@ -817,15 +978,25 @@ DHTTransportUDPImpl
 					
 					boolean	other_routable = (status & DHTTransportUDPContactImpl.NODE_STATUS_ROUTABLE) != 0;
 					
+						// only maintain stats on incoming requests so we get a fair sample. 
+						// in general we'll only get replies from routable contacts so if we
+						// take this into account then everything gets skewed
+					
 					if ( other_routable ){
 						
-						other_routable_total++;
+						if ( incoming ){
+							
+							other_routable_total++;
+						}
 						
 						routable_contact_history.put( contact.getTransportAddress(), contact );
 						
 					}else{
 						
-						other_non_routable_total++;
+						if ( incoming ){
+							
+							other_non_routable_total++;
+						}
 					}
 				}
 			}
@@ -836,6 +1007,70 @@ DHTTransportUDPImpl
 		}
 	}
 	
+	public int 
+	getRouteablePercentage() 
+	{
+		synchronized( routeable_percentage_average ){
+		
+			double	average = routeable_percentage_average.getAverage();
+			
+			long	both_total = other_routable_total + other_non_routable_total;
+			
+			int	current_percent;
+			
+			if ( both_total == 0 ){
+				
+				current_percent = 0;
+				
+			}else{
+				
+				current_percent = (int)(( other_routable_total * 100 )/ both_total );
+			}
+			 
+			if ( both_total >= 300 ){
+				
+					// add current to average and reset
+				
+				if ( current_percent > 0 ){
+				
+					average = routeable_percentage_average.update( current_percent );
+					
+					other_routable_total = other_non_routable_total = 0;
+				}
+			}else if ( both_total >= 100 ){
+				
+					// if we have enough samples and no existing average then use current
+				
+				if ( average == 0 ){
+					
+					average = current_percent;
+					
+				}else{
+					
+						// factor in current percantage
+					
+					int samples = routeable_percentage_average.getSampleCount();
+					
+					if ( samples > 0 ){
+						
+						average = (( samples * average ) + current_percent ) / ( samples + 1 );
+					}
+				}
+			}
+			
+			int result = (int)average;
+			
+			if ( result == 0 ){
+				
+					// -1 indicates we have no usable value
+				
+				result = -1;
+			}
+			
+			return( result );
+		}
+	}
+	
 	protected boolean
 	invalidExternalAddress(
 		InetAddress	ia )
@@ -873,6 +1108,8 @@ DHTTransportUDPImpl
 	
 			logger.log( "External address changed: " + s_address );
 			
+			Debug.out( "DHTTransport: address changed to " + s_address );
+			
 			for (int i=0;i<listeners.size();i++){
 				
 				try{
@@ -1061,7 +1298,7 @@ DHTTransportUDPImpl
 			}
 			
 			if ( ip_filter.isInRange( 
-					contact.getTransportAddress().getAddress(), "DHT",
+					contact.getTransportAddress().getAddress(), "DHT", null,
 					logger.isEnabled( DHTLogger.LT_IP_FILTER ))){
 				
 					// don't let an attacker deliberately fill up our filter so we start
@@ -1425,7 +1662,8 @@ DHTTransportUDPImpl
 		final DHTTransportUDPContactImpl	contact,
 		final DHTTransportReplyHandler		handler,
 		byte[][]							keys,
-		DHTTransportValue[][]				value_sets )
+		DHTTransportValue[][]				value_sets,
+		int									priority )
 	{
 		final long	connection_id = getConnectionID();
 		
@@ -1627,7 +1865,7 @@ DHTTransportUDPImpl
 						}
 					},
 					store_timeout,
-					PRUDPPacketHandler.PRIORITY_LOW );
+					priority );
 
 			}
 		}catch( Throwable e ){
@@ -1661,6 +1899,10 @@ DHTTransportUDPImpl
 			
 			request.setID( nid );
 			
+			request.setNodeStatus( getNodeStatus());
+			
+			request.setEstimatedDHTSize( request_handler.getTransportEstimatedDHTSize());
+			
 			requestSendRequestProcessor( contact, request );
 
 			packet_handler.sendAndReceive(
@@ -1691,13 +1933,33 @@ DHTTransportUDPImpl
 							
 							contact.setRandomID( reply.getRandomID());
 														
-							updateContactStatus( contact, reply.getNodeStatus());
+							updateContactStatus( contact, reply.getNodeStatus(), false );
 							
 							request_handler.setTransportEstimatedDHTSize( reply.getEstimatedDHTSize());
 							
 							stats.findNodeOK();
 								
-							handler.findNodeReply( contact, reply.getContacts());
+							DHTTransportContact[] contacts = reply.getContacts();
+							
+								// scavenge any contacts here to help bootstrap process
+								// when ip wrong and no import history
+							
+							for (int i=0; contact_history.size() < CONTACT_HISTORY_MAX && i<contacts.length;i++){
+								
+								DHTTransportUDPContact c = (DHTTransportUDPContact)contacts[i];
+								
+								try{
+									this_mon.enter();
+									
+									contact_history.put( c.getTransportAddress(), c );
+
+								}finally{
+									
+									this_mon.exit();
+								}
+							}
+							
+							handler.findNodeReply( contact, contacts );
 							
 						}catch( DHTUDPPacketHandlerException e ){
 							
@@ -2182,6 +2444,10 @@ DHTTransportUDPImpl
 		
 		byte	packet_type = req.getPacketType();
 		
+		if ( XFER_TRACE ){
+			System.out.println( "dataRequest: originator=" + originator.getAddress() + ",packet=" + req.getString());
+		}
+		
 		if ( packet_type == DHTUDPPacketData.PT_READ_REPLY ){
 			
 			transferQueue	queue = lookupTransferQueue( read_transfers, req.getConnectionId());
@@ -2244,7 +2510,7 @@ DHTTransportUDPImpl
 						logger.log( "No transfer handler registered for key '" + ByteFormatter.encodeString(transfer_key) + "'" );
 						
 					}else{
-					
+											
 						try{
 							final transferQueue new_queue = new transferQueue( read_transfers, req.getConnectionId());
 						
@@ -2253,11 +2519,31 @@ DHTTransportUDPImpl
 							new_queue.add( req );
 							
 								// set up the queue processor
-												
-							new AEThread( "DHTTransportUDP:writeQueueProcessor", true )
+								
+							try{
+								this_mon.enter();
+								
+								if ( active_write_queue_processor_count >= TRANSFER_QUEUE_MAX ){
+									
+									new_queue.destroy();
+									
+									throw( new DHTTransportException( "Active write queue process thread limit exceeded" ));
+								}
+								
+								active_write_queue_processor_count++;
+								
+								if ( XFER_TRACE ){
+									System.out.println( "active_write_queue_processor_count=" + active_write_queue_processor_count );
+								}
+							}finally{
+								
+								this_mon.exit();
+							}
+
+							new AEThread2( "DHTTransportUDP:writeQueueProcessor", true )
 								{
 									public void
-									runSupport()
+									run()
 									{
 										try{
 											byte[] write_data = 
@@ -2361,6 +2647,21 @@ DHTTransportUDPImpl
 										}catch( DHTTransportException e ){
 											
 											logger.log( "Failed to process transfer queue: " + Debug.getNestedExceptionMessage(e));
+											
+										}finally{
+											
+											try{
+												this_mon.enter();
+																								
+												active_write_queue_processor_count--;
+												
+												if ( XFER_TRACE ){
+													System.out.println( "active_write_queue_processor_count=" + active_write_queue_processor_count );
+												}
+											}finally{
+												
+												this_mon.exit();
+											}
 										}
 									}
 								}.start();
@@ -2847,7 +3148,12 @@ DHTTransportUDPImpl
 
 			requestReceiveRequestProcessor( originating_contact, request );
 			
-			if ( !originating_contact.addressMatchesID()){
+			boolean	bad_originator = !originating_contact.addressMatchesID();
+			
+				// bootstrap node returns details regardless of whether the originator ID matches
+				// as the details will help the sender discover their correct ID (hopefully)
+			
+			if ( bad_originator && !bootstrap_node ){
 				
 				String	contact_string = originating_contact.getString();
 
@@ -2876,7 +3182,25 @@ DHTTransportUDPImpl
 
 			}else{
 				
-				contactAlive( originating_contact );
+				if ( bad_originator ){
+				
+						// we need to patch the originator up otherwise we'll be populating our
+						// routing table with crap
+					
+					originating_contact = 
+						new DHTTransportUDPContactImpl( 
+								false,
+								this, 
+								transport_address, 
+								transport_address, 		// set originator address to transport
+								request.getOriginatorVersion(),
+								request.getOriginatorInstanceID(),
+								request.getClockSkew());
+
+				}else{
+					
+					contactAlive( originating_contact );
+				}
 				
 				if ( request instanceof DHTUDPPacketRequestPing ){
 					
@@ -3049,7 +3373,9 @@ DHTTransportUDPImpl
 					
 					if ( bootstrap_node ){
 						
-						acceptable = Arrays.equals( find_request.getID(), originating_contact.getID());
+							// let bad originators through to aid bootstrapping with bad IP
+						
+						acceptable = bad_originator || Arrays.equals( find_request.getID(), originating_contact.getID());
 												
 					}else{
 						
@@ -3058,6 +3384,13 @@ DHTTransportUDPImpl
 					
 					if ( acceptable ){
 						
+						if ( find_request.getProtocolVersion() >= DHTTransportUDP.PROTOCOL_VERSION_MORE_NODE_STATUS ){
+						
+							updateContactStatus( originating_contact, find_request.getNodeStatus(), true );
+											
+							request_handler.setTransportEstimatedDHTSize( find_request.getEstimatedDHTSize());
+						}
+						
 						DHTTransportContact[]	res = 
 							request_handler.findNodeRequest(
 										originating_contact,
@@ -3306,7 +3639,7 @@ DHTTransportUDPImpl
 				case DHTUDPPacketReplyError.ET_ORIGINATOR_ADDRESS_WRONG:
 				{
 					try{
-						externalAddressChange( remote_contact, error.getOriginatingAddress());
+						externalAddressChange( remote_contact, error.getOriginatingAddress(), false );
 						
 					}catch( DHTTransportException e ){
 						
@@ -3415,12 +3748,13 @@ DHTTransportUDPImpl
 	protected class
 	transferQueue
 	{
-		Map			transfers;
-		long		id;
+		private Map			transfers;
+		private long		id;
+		private boolean		destroyed;
 		
-		List		packets	= new ArrayList();
+		private List		packets	= new ArrayList();
 		
-		AESemaphore	packets_sem	= new AESemaphore("DHTUDPTransport:transferQueue");
+		private AESemaphore	packets_sem	= new AESemaphore("DHTUDPTransport:transferQueue");
 		
 		protected
 		transferQueue(
@@ -3437,10 +3771,21 @@ DHTTransportUDPImpl
 
 				if ( transfers.size() > TRANSFER_QUEUE_MAX ){
 					
+					Debug.out( "Transfer queue count limit exceeded" );
+					
 					throw( new DHTTransportException( "Transfer queue limit exceeded" ));
 				}
 				
-				transfers.put( new Long( id ), this );
+				Long l_id = new Long( id );
+				
+				transferQueue	existing = (transferQueue)transfers.get( l_id );
+				
+				if ( existing != null ){
+					
+					existing.destroy();
+				}
+				
+				transfers.put( l_id, this );
 				
 			}finally{
 				
@@ -3461,6 +3806,28 @@ DHTTransportUDPImpl
 			try{
 				this_mon.enter();
 	
+				if ( destroyed ){
+					
+					return;
+				}
+				
+				if ( total_bytes_on_transfer_queues > MAX_TRANSFER_QUEUE_BYTES ){
+					
+					Debug.out( "Transfer queue byte limit exceeded" );
+				
+						// just drop the packet
+					
+					return;
+				}
+				
+				int	length = packet.getLength();
+				
+				total_bytes_on_transfer_queues += length;
+				
+				if ( XFER_TRACE ){
+					System.out.println( "total_bytes_on_transfer_queues=" + total_bytes_on_transfer_queues );
+				}
+				
 				packets.add( packet );
 				
 			}finally{
@@ -3479,8 +3846,23 @@ DHTTransportUDPImpl
 				
 				try{
 					this_mon.enter();
-								
-					return((DHTUDPPacketData)packets.remove(0));
+						
+					if ( destroyed ){
+						
+						return( null );
+					}
+					
+					DHTUDPPacketData packet = (DHTUDPPacketData)packets.remove(0);
+					
+					int	length = packet.getLength();
+					
+					total_bytes_on_transfer_queues -= length;
+
+					if ( XFER_TRACE ){
+						System.out.println( "total_bytes_on_transfer_queues=" + total_bytes_on_transfer_queues );
+					}
+					
+					return( packet );
 					
 				}finally{
 					
@@ -3497,9 +3879,28 @@ DHTTransportUDPImpl
 		{
 			try{
 				this_mon.enter();
-							
+					
+				destroyed = true;
+				
 				transfers.remove( new Long( id ));
 				
+				for (int i=0;i<packets.size();i++){
+					
+					DHTUDPPacketData	packet = (DHTUDPPacketData)packets.get(i);
+					
+					int	length = packet.getLength();
+					
+					total_bytes_on_transfer_queues -= length;
+					
+					if ( XFER_TRACE ){
+						System.out.println( "total_bytes_on_transfer_queues=" + total_bytes_on_transfer_queues );
+					}
+				}
+				
+				packets.clear();
+				
+				packets_sem.releaseForever();
+				
 			}finally{
 				
 				this_mon.exit();
