@@ -31,9 +31,11 @@ import java.util.Random;
 
 import org.gudy.azureus2.core3.util.Debug;
 import org.gudy.azureus2.core3.util.SHA1Simple;
+import org.gudy.azureus2.core3.util.SystemTime;
 
 
 import com.aelitis.azureus.core.dht.DHT;
+import com.aelitis.azureus.core.dht.impl.DHTLog;
 import com.aelitis.azureus.core.dht.netcoords.DHTNetworkPosition;
 import com.aelitis.azureus.core.dht.netcoords.DHTNetworkPositionManager;
 import com.aelitis.azureus.core.dht.transport.DHTTransportContact;
@@ -122,7 +124,7 @@ DHTUDPUtils
 	{
 		if ( len > max_length ){
 			
-			throw( new IOException( "Invalid data length" ));
+			throw( new IOException( "Invalid DHT data length: max=" + max_length + ",actual=" + len ));
 		}
 		
 		if ( max_length < 256 ){
@@ -163,7 +165,7 @@ DHTUDPUtils
 		
 		if ( len > max_length ){
 			
-			throw( new IOException( "Invalid data length" ));
+			throw( new IOException( "Invalid DHT data length: max=" + max_length + ",actual=" + len ));
 		}
 		
 		return( len );
@@ -331,15 +333,13 @@ DHTUDPUtils
 	
 		throws IOException, DHTTransportException
 	{
-		if ( contact instanceof DHTTransportUDPContactImpl ){
+		if ( contact.getTransport() instanceof DHTTransportUDP ){
 			
 			os.writeByte( CT_UDP );		// 1
 			
 			os.writeByte( contact.getProtocolVersion());	// 2
-			
-			DHTTransportUDPContactImpl c = (DHTTransportUDPContactImpl)contact;
-								
-			serialiseAddress( os, c.getExternalAddress() );	// 2 + address
+											
+			serialiseAddress( os, contact.getExternalAddress() );	// 2 + address
 			
 		}else{
 			
@@ -455,6 +455,17 @@ DHTUDPUtils
 		
 		final int flags	= is.readByte()&0xff;
 		
+		final int life_hours;
+		
+		if ( packet.getProtocolVersion() >= DHTTransportUDP.PROTOCOL_VERSION_LONGER_LIFE ){
+
+			life_hours = is.readByte()&0xff;
+			
+		}else{
+			
+			life_hours = 0;
+		}
+		
 		DHTTransportValue value = 
 			new DHTTransportValue()
 			{
@@ -494,17 +505,26 @@ DHTUDPUtils
 					return( flags );
 				}
 				
+				public int
+				getLifeTimeHours()
+				{
+					return( life_hours );
+				}
+				
 				public String
 				getString()
 				{
-					return( new String(getValue()));
+					long	now = SystemTime.getCurrentTime();
+					
+					return( DHTLog.getString( value_bytes ) + " - " + new String(value_bytes) + "{v=" + version + ",f=" + 
+							Integer.toHexString(flags) + ",l=" + life_hours + ",ca=" + (now - created ) + ",or=" + originator.getString() +"}" );
 				}
 			};
 			
 		return( value );
 	}
 	
-	public static final int DHTTRANSPORTVALUE_SIZE_WITHOUT_VALUE	= 15 + DHTTRANSPORTCONTACT_SIZE;
+	public static final int DHTTRANSPORTVALUE_SIZE_WITHOUT_VALUE	= 16 + DHTTRANSPORTCONTACT_SIZE;
 		
 	protected static void
 	serialiseTransportValue(
@@ -538,6 +558,11 @@ DHTUDPUtils
 		serialiseContact( os, value.getOriginator());	// 12 + 2+X + contact
 		
 		os.writeByte( value.getFlags());	// 13 + 2+ X + contact
+		
+		if ( packet.getProtocolVersion() >= DHTTransportUDP.PROTOCOL_VERSION_LONGER_LIFE ){
+
+			os.writeByte( value.getLifeTimeHours()); // 14 + 2+ X + contact
+		}
 	}
 	
 	protected static void
@@ -671,7 +696,7 @@ DHTUDPUtils
 				byte	type = is.readByte();
 				byte	size = is.readByte();
 				
-				DHTNetworkPosition	np = DHTNetworkPositionManager.deserialise( type, is );
+				DHTNetworkPosition	np = DHTNetworkPositionManager.deserialise( reply.getAddress().getAddress(), type, is );
 				
 				if ( np == null ){
 					
@@ -705,7 +730,7 @@ DHTUDPUtils
 			}
 		}else{
 			
-			nps = new DHTNetworkPosition[]{ DHTNetworkPositionManager.deserialise( DHTNetworkPosition.POSITION_TYPE_VIVALDI_V1, is )};	
+			nps = new DHTNetworkPosition[]{ DHTNetworkPositionManager.deserialise( reply.getAddress().getAddress(), DHTNetworkPosition.POSITION_TYPE_VIVALDI_V1, is )};	
 		}
 		
 		boolean	v1_found = false;
@@ -769,6 +794,15 @@ DHTUDPUtils
 			os.writeLong( stats.getDBKeysBlocked());
 			os.writeLong( stats.getTotalKeyBlocksReceived());
 		}
+		
+		if ( version >= DHTTransportUDP.PROTOCOL_VERSION_MORE_STATS ){
+			
+			os.writeLong( stats.getDBKeyCount());
+			os.writeLong( stats.getDBValueCount());
+			os.writeLong( stats.getDBStoreSize());
+			os.writeLong( stats.getDBKeyDivFreqCount());
+			os.writeLong( stats.getDBKeyDivSizeCount());
+		}
 	}
 	
 	protected static DHTTransportFullStats
@@ -812,8 +846,31 @@ DHTUDPUtils
 			db_keys_blocked				= is.readLong();
 			total_key_blocks_received	= is.readLong();
 		}else{
-			db_keys_blocked				= 0;
-			total_key_blocks_received	= 0;
+			db_keys_blocked				= -1;
+			total_key_blocks_received	= -1;
+		}
+		
+		final long db_key_count;
+		final long db_value_count;
+		final long db_store_size;
+		final long db_freq_divs;
+		final long db_size_divs;
+		
+		if ( version >= DHTTransportUDP.PROTOCOL_VERSION_MORE_STATS ){
+			
+			db_key_count 	= is.readLong();
+			db_value_count	= is.readLong();
+			db_store_size	= is.readLong();
+			db_freq_divs	= is.readLong();
+			db_size_divs	= is.readLong();
+			
+		}else{
+			
+			db_key_count 	= -1;
+			db_value_count	= -1;
+			db_store_size	= -1;
+			db_freq_divs	= -1;
+			db_size_divs	= -1;
 		}
 		
 		DHTTransportFullStats	res = 
@@ -831,6 +888,36 @@ DHTUDPUtils
 					return( db_keys_blocked );
 				}
 				
+				public long
+				getDBValueCount()
+				{
+					return( db_value_count );
+				}
+				
+				public long
+				getDBKeyCount()
+				{
+					return( db_key_count );
+				}
+								
+				public long
+				getDBKeyDivSizeCount()
+				{
+					return( db_size_divs );
+				}
+				
+				public long
+				getDBKeyDivFreqCount()
+				{
+					return( db_freq_divs );
+				}
+
+				public long
+				getDBStoreSize()
+				{
+					return( db_store_size );
+				}
+				
 					// Router
 				
 				public long
@@ -977,8 +1064,13 @@ DHTUDPUtils
 							getRouterLeaves() + "," +
 							getRouterContacts() + 
 							",database:" +
-							getDBValuesStored()+ "," +
-							getDBKeysBlocked()+
+							getDBKeyCount() + ","+
+							getDBValueCount() + ","+
+							getDBValuesStored() + ","+
+							getDBStoreSize() + ","+
+							getDBKeyDivFreqCount() + ","+
+							getDBKeyDivSizeCount() + ","+
+							getDBKeysBlocked()+ 
 							",version:" + getVersion()+","+
 							getRouterUptime() + ","+
 							getRouterCount());
