@@ -1204,9 +1204,8 @@ CacheFileWithCache
 		}
 
 		try{
-			Iterator it = cache.subSet(new CacheEntry(first-1), new CacheEntry(last)).iterator();
-			
-			//cache.subSet(new CacheEntry())
+			Iterator it = cache.subSet(new CacheEntry(first-1-baseOffset), new CacheEntry(last-baseOffset)).iterator();
+			//Iterator it = cache.iterator();
 
 			while(it.hasNext())
 			{
@@ -1324,6 +1323,23 @@ CacheFileWithCache
 		}	
 	}
 	
+	public void
+	renameFile(
+		String		new_name )
+	
+		throws CacheFileManagerException
+	{
+		try{
+			flushCachePublic( true, -1 );
+			
+			file.renameFile( new_name );
+			
+		}catch( FMFileManagerException e ){
+			
+			manager.rethrow(this,e);
+		}	
+	}
+	
 	public void
 	setAccessMode(
 		int		mode )
