@@ -27,9 +27,12 @@ import java.io.RandomAccessFile;
 import java.nio.ByteBuffer;
 import java.nio.channels.FileChannel;
 
+import org.gudy.azureus2.core3.util.AEThread2;
 import org.gudy.azureus2.core3.util.Debug;
 import org.gudy.azureus2.core3.util.DirectByteBuffer;
+import org.gudy.azureus2.core3.util.SystemTime;
 
+import com.aelitis.azureus.core.diskmanager.file.FMFile;
 import com.aelitis.azureus.core.diskmanager.file.FMFileManagerException;
 
 public class 
@@ -61,6 +64,8 @@ FMFileAccessLinear
 		throws FMFileManagerException
 	{
 		try{
+			AEThread2.setDebug( owner );
+			
 			return( raf.length());
 			
 		}catch( Throwable e ){
@@ -76,7 +81,9 @@ FMFileAccessLinear
 	
 		throws FMFileManagerException
 	{
-		try{			
+		try{
+			AEThread2.setDebug( owner );
+			
 			raf.setLength( length );
 			
 		}catch( Throwable e ){
@@ -107,6 +114,8 @@ FMFileAccessLinear
 			throw( new FMFileManagerException( "read - file is closed"));
 		}
 
+		AEThread2.setDebug( owner );
+		
 		try{
 			fc.position(offset);
 			
@@ -125,6 +134,7 @@ FMFileAccessLinear
 	
 	public void
 	read(
+		FMFile				file,
 		RandomAccessFile	raf,
 		DirectByteBuffer[]	buffers,
 		long				offset )
@@ -145,8 +155,12 @@ FMFileAccessLinear
 			throw( new FMFileManagerException( "read - file is closed"));
 		}
 		
+		AEThread2.setDebug( owner );
+		
 		int[]	original_positions = null;
 		
+		long read_start = SystemTime.getHighPrecisionCounter();
+		
 		try{			
 			fc.position(offset);
 			
@@ -239,6 +253,14 @@ FMFileAccessLinear
 			}
 			
 			throw( new FMFileManagerException( "read fails", e ));
+		}finally{
+			
+			long elapsed_millis = ( SystemTime.getHighPrecisionCounter() - read_start )/1000000;
+
+			if ( elapsed_millis > 10*1000 ){
+				
+				System.out.println( "read took " + elapsed_millis + " for " + file.getName());
+			}
 		}
 	}
 	
@@ -264,6 +286,8 @@ FMFileAccessLinear
 			throw( new FMFileManagerException( "read - file is closed"));
 		}
 
+		AEThread2.setDebug( owner );
+		
 		int[]	original_positions = null;
 
 		try{
