@@ -48,6 +48,10 @@ FMFileImpl
 	private static Map			file_map = new HashMap();
 	private static AEMonitor	file_map_mon	= new AEMonitor( "FMFile:map");
 	
+	// If there is an exception that occurs, which causes us to try and perform
+	// a reopen, setting this flag to true will print it to debug.
+	private static boolean OUTPUT_REOPEN_RELATED_ERRORS = false; 
+	
 	static{
 		AEDiagnostics.addEvidenceGenerator(
 			new AEDiagnosticsEvidenceGenerator()
@@ -361,6 +365,95 @@ FMFileImpl
 		}
 	}
 	
+	public void
+	renameFile(
+		String		new_name )
+	
+		throws FMFileManagerException
+	{
+		try{
+			this_mon.enter();
+		
+			String	new_canonical_path;
+			
+			File 	new_linked_file = new File( linked_file.getParentFile(), new_name );
+						
+			try{
+        
+		        try {
+					
+		          new_canonical_path = new_linked_file.getCanonicalPath();
+				  
+	
+		        }catch( IOException ioe ) {
+					
+		          String msg = ioe.getMessage();
+				  
+		          if( msg != null && msg.indexOf( "There are no more files" ) != -1 ) {
+		            String abs_path = new_linked_file.getAbsolutePath();
+		            String error = "Caught 'There are no more files' exception during new_file.getCanonicalPath(). " +
+		                           "os=[" +Constants.OSName+ "], new_file.getPath()=[" +new_linked_file.getPath()+ "], new_file.getAbsolutePath()=[" +abs_path+ "]. ";
+		                           //"new_canonical_path temporarily set to [" +abs_path+ "]";
+		            Debug.out( error, ioe );
+		          }
+		          throw ioe;
+		        }
+						
+			}catch( Throwable e ){
+				
+				throw( new FMFileManagerException( "getCanonicalPath fails", e ));
+			}	
+			
+			if ( new_linked_file.exists()){
+				
+				throw( new FMFileManagerException( "renameFile fails - file '" + new_canonical_path + "' already exists"));	
+			}
+			
+			boolean	was_open	= isOpen();
+			
+			close();	// full close, this will release any slots in the limited file case
+				        
+			if ( !linked_file.exists() || linked_file.renameTo( new_linked_file )){
+				
+				linked_file		= new_linked_file;
+				canonical_path	= new_canonical_path;
+				
+				reserveFile();
+				
+				if ( was_open ){
+					
+					ensureOpen( "renameFile target" );	// ensure open will regain slots in limited file case
+				}
+				
+			}else{
+			
+				try{
+					reserveFile();
+					
+				}catch( FMFileManagerException e ){
+					
+					Debug.printStackTrace( e );
+				}
+				
+				if ( was_open ){
+					
+					try{
+						ensureOpen( "renameFile recovery" );
+						
+					}catch( FMFileManagerException e){
+						
+						Debug.printStackTrace( e );
+					}
+				}
+				
+				throw( new FMFileManagerException( "renameFile fails"));
+			}	
+		}finally{
+			
+			this_mon.exit();
+		}
+	}
+	
 	public void
 	ensureOpen(
 		String	reason )
@@ -393,6 +486,8 @@ FMFileImpl
 			
 		}catch( FMFileManagerException e ){
 			
+			if (OUTPUT_REOPEN_RELATED_ERRORS) {Debug.printStackTrace(e);}
+			
 			try{
 				reopen();
 				
@@ -415,6 +510,8 @@ FMFileImpl
 			file_access.setLength( raf, length );
 			
 		}catch( FMFileManagerException e ){
+			
+			if (OUTPUT_REOPEN_RELATED_ERRORS) {Debug.printStackTrace(e);}
 						
 			try{
 				reopen();
@@ -572,14 +669,16 @@ FMFileImpl
 	{
 		try{
 		
-			file_access.read( raf, buffers, position );
+			file_access.read( this, raf, buffers, position );
 			
 		}catch( FMFileManagerException e ){
 			
+			if (OUTPUT_REOPEN_RELATED_ERRORS) {Debug.printStackTrace(e);}
+			
 			try{
 				reopen();
 
-				file_access.read( raf, buffers, position );
+				file_access.read( this, raf, buffers, position );
 				
 			}catch( Throwable e2 ){
 				
@@ -611,6 +710,8 @@ FMFileImpl
 	
 		}catch( FMFileManagerException e ){
 			
+			if (OUTPUT_REOPEN_RELATED_ERRORS) {Debug.printStackTrace(e);}
+			
 			try{
 				reopen();
 
@@ -651,7 +752,7 @@ FMFileImpl
 		try{
 			file_map_mon.enter();
 			
-			// System.out.println( "FMFile::reserveFile:" + canonical_path + "("+ owner.getName() + ")" );
+			// System.out.println( "FMFile::reserveFile:" + canonical_path + "("+ owner.getName() + ")" + " - " + Debug.getCompressedStackTrace() );
 			
 			List	owners = (List)file_map.get(canonical_path);
 			
@@ -659,6 +760,8 @@ FMFileImpl
 				
 				owners = new ArrayList();
 				
+				//System.out.println( "    creating new owners entr" );
+				
 				file_map.put( canonical_path, owners );				
 			}
 			
@@ -666,7 +769,11 @@ FMFileImpl
 				
 				Object[]	entry = (Object[])it.next();
 			
-				if ( owner.getName().equals(((FMFileOwner)entry[0]).getName())){
+				String	entry_name = ((FMFileOwner)entry[0]).getName();
+				
+				//System.out.println( "    existing entry: " + entry_name );
+				
+				if ( owner.getName().equals( entry_name )){
 				
 						// already present, start off read-access
 					
@@ -700,7 +807,7 @@ FMFileImpl
 		try{
 			file_map_mon.enter();
 			
-			// System.out.println( "FMFile::reserveAccess:" + canonical_path + "("+ owner.getName() + ")" + " [" + (access_mode==FM_WRITE?"write":"read") + "]" );
+			//System.out.println( "FMFile::reserveAccess:" + canonical_path + "("+ owner.getName() + ")" + " [" + (access_mode==FM_WRITE?"write":"read") + "]" + " - " + Debug.getCompressedStackTrace());
 			
 			List	owners = (List)file_map.get( canonical_path );
 			
@@ -717,7 +824,11 @@ FMFileImpl
 					
 				Object[]	entry = (Object[])it.next();
 				
-				if ( owner.getName().equals(((FMFileOwner)entry[0]).getName())){
+				String	entry_name = ((FMFileOwner)entry[0]).getName();
+				
+				//System.out.println( "    existing entry: " + entry_name );
+				
+				if ( owner.getName().equals( entry_name )){
 					
 					my_entry	= entry;
 				}
@@ -804,7 +915,7 @@ FMFileImpl
 		try{
 			file_map_mon.enter();
 		
-			// System.out.println( "FMFile::releaseFile:" + canonical_path + "("+ owner.getName() + ")" );
+			// System.out.println( "FMFile::releaseFile:" + canonical_path + "("+ owner.getName() + ")" + " - " + Debug.getCompressedStackTrace());
 					
 			List	owners = (List)file_map.get( canonical_path );
 			
