@@ -2,7 +2,6 @@
  * Created on 13-Jul-2004
  * Created by Paul Gardner
  * Copyright (C) 2004, 2005, 2006 Aelitis, All Rights Reserved.
- *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
@@ -22,6 +21,7 @@
 
 package com.aelitis.azureus.core.impl;
 
+import java.net.InetAddress;
 import java.util.*;
 
 import org.gudy.azureus2.core3.config.COConfigurationManager;
@@ -37,7 +37,9 @@ import org.gudy.azureus2.core3.internat.*;
 import org.gudy.azureus2.core3.ipfilter.IpFilterManager;
 import org.gudy.azureus2.core3.logging.*;
 import org.gudy.azureus2.core3.ipfilter.*;
+import org.gudy.azureus2.core3.peer.PEPeerSource;
 import org.gudy.azureus2.core3.security.SESecurityManager;
+import org.gudy.azureus2.core3.torrent.TOTorrent;
 import org.gudy.azureus2.core3.tracker.client.TRTrackerAnnouncer;
 import org.gudy.azureus2.core3.tracker.client.TRTrackerAnnouncerResponse;
 import org.gudy.azureus2.core3.tracker.host.*;
@@ -45,30 +47,42 @@ import org.gudy.azureus2.core3.util.*;
 import org.gudy.azureus2.platform.PlatformManagerFactory;
 import org.gudy.azureus2.platform.PlatformManagerListener;
 import org.gudy.azureus2.plugins.*;
+import org.gudy.azureus2.plugins.utils.DelayedTask;
 import org.gudy.azureus2.pluginsimpl.local.PluginInitializer;
+import org.gudy.azureus2.pluginsimpl.local.download.DownloadManagerImpl;
+import org.gudy.azureus2.pluginsimpl.local.utils.UtilitiesImpl;
 
 import com.aelitis.azureus.core.*;
+import com.aelitis.azureus.core.custom.CustomizationManagerFactory;
 import com.aelitis.azureus.core.dht.DHT;
 import com.aelitis.azureus.core.instancemanager.AZInstanceManager;
+import com.aelitis.azureus.core.instancemanager.AZInstanceManagerAdapter;
 import com.aelitis.azureus.core.instancemanager.AZInstanceManagerFactory;
+import com.aelitis.azureus.core.instancemanager.AZInstanceTracked;
+import com.aelitis.azureus.core.instancemanager.AZInstanceManagerAdapter.VCPublicAddress;
 import com.aelitis.azureus.core.nat.NATTraverser;
 import com.aelitis.azureus.core.networkmanager.NetworkManager;
 import com.aelitis.azureus.core.networkmanager.admin.NetworkAdmin;
 import com.aelitis.azureus.core.networkmanager.admin.NetworkAdminNetworkInterface;
 import com.aelitis.azureus.core.networkmanager.admin.NetworkAdminNetworkInterfaceAddress;
 import com.aelitis.azureus.core.networkmanager.admin.NetworkAdminPropertyChangeListener;
+import com.aelitis.azureus.core.networkmanager.impl.tcp.TCPNetworkManager;
+import com.aelitis.azureus.core.networkmanager.impl.udp.UDPNetworkManager;
 import com.aelitis.azureus.core.peermanager.PeerManager;
-import com.aelitis.azureus.core.peermanager.download.session.TorrentSessionManager;
 import com.aelitis.azureus.core.peermanager.nat.PeerNATTraverser;
+import com.aelitis.azureus.plugins.clientid.ClientIDPlugin;
 import com.aelitis.azureus.core.security.CryptoManager;
 import com.aelitis.azureus.core.security.CryptoManagerFactory;
 import com.aelitis.azureus.core.speedmanager.SpeedManager;
 import com.aelitis.azureus.core.speedmanager.SpeedManagerAdapter;
 import com.aelitis.azureus.core.speedmanager.SpeedManagerFactory;
 import com.aelitis.azureus.core.update.AzureusRestarterFactory;
+import com.aelitis.azureus.core.util.CopyOnWriteList;
+import com.aelitis.azureus.core.versioncheck.VersionCheckClient;
 import com.aelitis.azureus.launcher.classloading.PrimaryClassloader;
 import com.aelitis.azureus.plugins.dht.DHTPlugin;
 import com.aelitis.azureus.plugins.tracker.dht.DHTTrackerPlugin;
+import com.aelitis.azureus.plugins.upnp.UPnPPlugin;
 
 /**
  * @author parg
@@ -86,6 +100,13 @@ AzureusCoreImpl
 	private static final String DM_ANNOUNCE_KEY	= "AzureusCore:announce_key";
 	private static final boolean LOAD_PLUGINS_IN_OTHER_THREAD = true;
 	
+	/** 
+	 * Listeners that will be fired after core has completed initialization
+	 */
+	static List<AzureusCoreRunningListener> coreRunningListeners = new ArrayList<AzureusCoreRunningListener>(1);
+	
+	static AEMonitor mon_coreRunningListeners = new AEMonitor("CoreCreationListeners");
+	
 	public static AzureusCore
 	create()
 	
@@ -114,7 +135,13 @@ AzureusCoreImpl
 	{
 		return( singleton != null );
 	}
-	
+
+	public static boolean
+	isCoreRunning()
+	{
+		return( singleton != null && singleton.isStarted() );
+	}
+
 	public static AzureusCore
 	getSingleton()
 	
@@ -138,12 +165,12 @@ AzureusCoreImpl
 	private long create_time = SystemTime.getCurrentTime();
 
 
-	private boolean				started;
-	private boolean				stopped;
-	private boolean				restarting;
+	private volatile boolean				started;
+	private volatile boolean				stopped;
+	private volatile boolean				restarting;
 	
-	private List				listeners				= new ArrayList();
-	private List				lifecycle_listeners		= new ArrayList();
+	private CopyOnWriteList		listeners				= new CopyOnWriteList();
+	private CopyOnWriteList		lifecycle_listeners		= new CopyOnWriteList();
 	private List				operation_listeners		= new ArrayList();
 	
 	private AESemaphore			stopping_sem	= new AESemaphore( "AzureusCore::stopping" );
@@ -164,8 +191,10 @@ AzureusCoreImpl
 		
 		MessageText.loadBundle();
 		
-		AEDiagnostics.startup();
+		AEDiagnostics.startup( COConfigurationManager.getBooleanParameter( "diags.enable.pending.writes", false ));
 		
+		COConfigurationManager.setParameter( "diags.enable.pending.writes", false );
+				
 		AEDiagnostics.markDirty();
 		
 		AETemporaryFileHandler.startup();
@@ -214,21 +243,212 @@ AzureusCoreImpl
 		
 			//ensure early initialization
 		
+		CustomizationManagerFactory.getSingleton();
+		
 		NetworkManager.getSingleton();
 		
 		PeerManager.getSingleton();
-        
-	    TorrentSessionManager.getSingleton().init();
-    
+		
+			// Used to be a plugin, but not any more...
+		
+		ClientIDPlugin.initialize();
+		
 		pi = PluginInitializer.getSingleton( this, initialisation_op );
 		
-		instance_manager = AZInstanceManagerFactory.getSingleton( this );
+		
+		instance_manager = 
+			AZInstanceManagerFactory.getSingleton( 
+				new AZInstanceManagerAdapter()
+				{
+					public String
+					getID()
+					{
+						return( COConfigurationManager.getStringParameter( "ID", "" ));
+					}
+					
+					public InetAddress 
+					getPublicAddress() 
+					{	
+						return( PluginInitializer.getDefaultInterface().getUtilities().getPublicAddress());
+					}
+					
+					public int[]
+					getPorts()
+					{
+						return( new int[]{
+							TCPNetworkManager.getSingleton().getTCPListeningPortNumber(),
+							UDPNetworkManager.getSingleton().getUDPListeningPortNumber(),
+							UDPNetworkManager.getSingleton().getUDPNonDataListeningPortNumber()});
+
+					}
+					public VCPublicAddress
+					getVCPublicAddress()
+					{
+						return(
+							new VCPublicAddress()
+							{
+								private VersionCheckClient vcc = VersionCheckClient.getSingleton();
+								
+								public String
+								getAddress()
+								{
+									return( vcc.getExternalIpAddress( true, false ));
+								}
+								
+								public long
+								getCacheTime()
+								{
+									return( vcc.getSingleton().getCacheTime( false ));
+								}
+							});
+					}
+
+					public AZInstanceTracked.TrackTarget
+					track(
+						byte[]		hash )
+					{
+						List	dms = getGlobalManager().getDownloadManagers();
+						
+						Iterator	it = dms.iterator();
+						
+						DownloadManager	matching_dm = null;
+						
+						try{
+							while( it.hasNext()){
+								
+								DownloadManager	dm = (DownloadManager)it.next();
+								
+								TOTorrent	torrent = dm.getTorrent();
+								
+								if ( torrent == null ){
+									
+									continue;
+								}
+								
+								byte[]	sha1_hash = (byte[])dm.getData( "AZInstanceManager::sha1_hash" );
+								
+								if ( sha1_hash == null ){			
+
+									sha1_hash	= new SHA1Simple().calculateHash( torrent.getHash());
+									
+									dm.setData( "AZInstanceManager::sha1_hash", sha1_hash );
+								}
+								
+								if ( Arrays.equals( hash, sha1_hash )){
+									
+									matching_dm	= dm;
+									
+									break;
+								}
+							}
+						}catch( Throwable e ){
+							
+							Debug.printStackTrace(e);
+						}
+						
+						if ( matching_dm == null ){
+							
+							return( null );
+						}
+						
+						if ( !matching_dm.getDownloadState().isPeerSourceEnabled( PEPeerSource.PS_PLUGIN )){
+							
+							return( null );
+						}
+						
+						int	dm_state = matching_dm.getState();
+						
+						if ( dm_state == DownloadManager.STATE_ERROR || dm_state == DownloadManager.STATE_STOPPED ){
+							
+							return( null );
+						}
+						
+						try{
+						
+							final Object target = DownloadManagerImpl.getDownloadStatic( matching_dm );
+						
+							final boolean	is_seed = matching_dm.isDownloadComplete(true);
+							
+							return(
+								new AZInstanceTracked.TrackTarget()
+								{
+									public Object
+									getTarget()
+									{
+										return( target );
+									}
+									
+									public boolean
+									isSeed()
+									{
+										return( is_seed );
+									}
+								});
+							
+						}catch( Throwable e ){
+							
+							return( null );
+						}
+					}
+					
+					public DHTPlugin 
+					getDHTPlugin()
+					{
+						PluginInterface pi = getPluginManager().getPluginInterfaceByClass( DHTPlugin.class );
+						
+						if ( pi != null ){
+							
+							return( (DHTPlugin)pi.getPlugin());
+						}
+						
+						return( null );
+					}
+					
+					public UPnPPlugin 
+					getUPnPPlugin() 
+					{
+						PluginInterface pi = getPluginManager().getPluginInterfaceByClass( UPnPPlugin.class );
+						
+						if ( pi != null ){
+							
+							return((UPnPPlugin)pi.getPlugin());
+						}
+						
+						return( null );
+					}
+					
+					public void 
+					addListener(
+						final StateListener listener) 
+					{
+						AzureusCoreImpl.this.addLifecycleListener(
+							new AzureusCoreLifecycleAdapter()
+							{
+								public void 
+								started(
+									AzureusCore core) 
+								{
+									listener.started();
+								}
+								
+								public void
+								stopping(
+									AzureusCore		core )
+								{
+									listener.stopped();
+								}
+							});
+					}
+				});
 		
 		speed_manager	= 
 			SpeedManagerFactory.createSpeedManager( 
 					this,
 					new SpeedManagerAdapter()
 					{
+						private final int UPLOAD_SPEED_ADJUST_MIN_KB_SEC		= 10;
+						private final int DOWNLOAD_SPEED_ADJUST_MIN_KB_SEC		= 300;
+						
 						private boolean setting_limits;
 						
 						public int
@@ -324,7 +544,12 @@ AzureusCoreImpl
 									
 								}else{
 								
-									k_per_second = bytes_per_second/1024;
+									k_per_second = (bytes_per_second+1023)/1024;
+								}
+								
+								if ( k_per_second > 0 ){
+								
+									k_per_second = Math.max( k_per_second, UPLOAD_SPEED_ADJUST_MIN_KB_SEC );
 								}
 								
 								COConfigurationManager.setParameter( key, k_per_second );
@@ -341,6 +566,16 @@ AzureusCoreImpl
 						setCurrentDownloadLimit(
 							int		bytes_per_second )
 						{
+							if ( bytes_per_second == Integer.MAX_VALUE ){
+								
+								bytes_per_second = 0;
+							}
+							
+							if ( bytes_per_second > 0 ){
+								
+								bytes_per_second = Math.max( bytes_per_second, DOWNLOAD_SPEED_ADJUST_MIN_KB_SEC*1024 );
+							}
+							
 							TransferSpeedValidator.setGlobalDownloadRateLimitBytesPerSecond( bytes_per_second );
 						}
 						
@@ -435,7 +670,7 @@ AzureusCoreImpl
 				
 				DownloadManager	dm = (DownloadManager)downloads.get(i);
 				
-				Long	last_announce_l = (Long)dm.getData( DM_ANNOUNCE_KEY );
+				Long	last_announce_l = (Long)dm.getUserData( DM_ANNOUNCE_KEY );
 				
 				long	last_announce	= last_announce_l==null?create_time:last_announce_l.longValue();
 				
@@ -450,7 +685,7 @@ AzureusCoreImpl
 							last_announce_response.getStatus() == TRTrackerAnnouncerResponse.ST_OFFLINE ||
 							force ){
 	
-						dm.setData( DM_ANNOUNCE_KEY, new Long( now ));
+						dm.setUserData( DM_ANNOUNCE_KEY, new Long( now ));
 						
 						Logger.log(	new LogEvent(LOGID, "    updating tracker for " + dm.getDisplayName()));
 	
@@ -501,6 +736,37 @@ AzureusCoreImpl
 			this_mon.exit();
 		}
 		
+		// If a user sets this property, it is an alias for the following settings.
+		if ("1".equals(System.getProperty("azureus.safemode"))) {
+			if (Logger.isEnabled())
+				Logger.log(new LogEvent(LOGID, "Safe mode enabled"));
+			
+			Constants.isSafeMode = true;
+			System.setProperty("azureus.loadplugins", "0");
+			System.setProperty("azureus.disabledownloads", "1");
+			System.setProperty("azureus.skipSWTcheck", "1");
+			
+			// Not using localised text - not sure it's safe to this early.
+			Logger.log(new LogAlert(LogAlert.UNREPEATABLE, LogEvent.LT_WARNING,
+				"You are running " + Constants.APP_NAME + " in safe mode - you " +
+					"can change your configuration, but any downloads added will " +
+					"not be remembered when you close " + Constants.APP_NAME + "."
+			));
+		}
+		
+	   /**
+	    * test to see if UI plays nicely with a really slow initialization
+	    */
+	   String sDelayCore = System.getProperty("delay.core", null);
+	   if (sDelayCore != null) {
+	  	 try {
+	  		 long delayCore = Long.parseLong(sDelayCore);
+	  		 Thread.sleep(delayCore);
+	  	 } catch (Exception e) {
+	  		 e.printStackTrace();
+	  	 }
+	   }
+
 
 		// run plugin loading in parallel to the global manager loading
 		AEThread2 pluginload = new AEThread2("PluginLoader",true)
@@ -508,7 +774,7 @@ AzureusCoreImpl
 			public void run() {
 				if (Logger.isEnabled())
 					Logger.log(new LogEvent(LOGID, "Loading of Plugins starts"));
-				pi.loadPlugins(AzureusCoreImpl.this, false);
+				pi.loadPlugins(AzureusCoreImpl.this, false, !"0".equals(System.getProperty("azureus.loadplugins")), true, true);
 				if (Logger.isEnabled())
 					Logger.log(new LogEvent(LOGID, "Loading of Plugins complete"));
 			}
@@ -551,16 +817,31 @@ AzureusCoreImpl
 						initialisation_op.reportPercent( percent );
 					}
 				}, 0);
+		
+		if (stopped) {
+			System.err.println("Core stopped while starting");
+			return;
+		}
 
 		// wait until plugin loading is done
 		if (LOAD_PLUGINS_IN_OTHER_THREAD) {
 			pluginload.join();
 		}
+
+		if (stopped) {
+			System.err.println("Core stopped while starting");
+			return;
+		}
 		
 		triggerLifeCycleComponentCreated(global_manager);
 
 		pi.initialisePlugins();
 
+		if (stopped) {
+			System.err.println("Core stopped while starting");
+			return;
+		}
+		
 		if (Logger.isEnabled())
 			Logger.log(new LogEvent(LOGID, "Initializing Plugins complete"));
 
@@ -630,55 +911,14 @@ AzureusCoreImpl
 		}catch( Throwable e ){
 		}
 		
-	    new AEThread2("Plugin Init Complete", false )
-	       {
-	        	public void
-	        	run()
-	        	{
-	        		for (int i=0;i<lifecycle_listeners.size();i++){
-	        			
-	        			try{
-	        				AzureusCoreLifecycleListener listener = (AzureusCoreLifecycleListener)lifecycle_listeners.get(i);
-	        				
-	        				if ( !listener.requiresPluginInitCompleteBeforeStartedEvent()){
-	        				
-	        					listener.started( AzureusCoreImpl.this );
-	        				}
-	        			}catch( Throwable e ){
-	        				
-	        				Debug.printStackTrace(e);
-	        			}
-	        		}
-	        		
-	        		pi.initialisationComplete();
-	        		
-	        		for (int i=0;i<lifecycle_listeners.size();i++){
-	        			
-	        			try{
-	        				AzureusCoreLifecycleListener listener = (AzureusCoreLifecycleListener)lifecycle_listeners.get(i);
-	        				
-	        				if ( listener.requiresPluginInitCompleteBeforeStartedEvent()){
-	        				
-	        					listener.started( AzureusCoreImpl.this );
-	        				}				
-	        			}catch( Throwable e ){
-	        				
-	        				Debug.printStackTrace(e);
-	        			}
-	        		}
-	        	}
-	       }.start();
-       
 	   if ( COConfigurationManager.getBooleanParameter( "Resume Downloads On Start" )){
 	   
 		   global_manager.resumeDownloads();
 	   }
-	   
-       //late inits
-	   NetworkManager.getSingleton().initialize(); 
-          
+	    
 	   instance_manager.initialize();
-	   
+
+	   NetworkManager.getSingleton().initialize(this); 
          
 	   Runtime.getRuntime().addShutdownHook( new AEThread("Shutdown Hook") {
 	     public void runSupport() {
@@ -686,90 +926,193 @@ AzureusCoreImpl
 			AzureusCoreImpl.this.stop();
 	     }
 	   });	
-	   	   
-	   AEDiagnostics.checkDumpsAndNatives();
-	   
-	   NetworkAdmin na = NetworkAdmin.getSingleton();
-	   
-	   na.runInitialChecks();
-	   
-	   na.addPropertyChangeListener(
-			   new NetworkAdminPropertyChangeListener()
-			   {
-				   private String	last_as;
-				   
-				   public void
-				   propertyChanged(
-						   String		property )
-				   {
-					   NetworkAdmin na = NetworkAdmin.getSingleton();
+	   	  
 
-					   if ( property.equals( NetworkAdmin.PR_NETWORK_INTERFACES )){
+	   DelayedTask delayed_task = 
+	   		UtilitiesImpl.addDelayedTask(
+	   			"Core", 
+	   			new Runnable()
+	   			{
+	   				public void
+	   				run()
+	   				{
+	   					new AEThread2( "core:delayTask", true )
+	   					{
+	   						public void
+	   						run()
+	   						{				
+			   					AEDiagnostics.checkDumpsAndNatives();
+		
+			   					COConfigurationManager.setParameter( "diags.enable.pending.writes", true );
+			   					
+			   					AEDiagnostics.flushPendingLogs();
+			   					
+			   					NetworkAdmin na = NetworkAdmin.getSingleton();
+		
+			   					na.runInitialChecks(AzureusCoreImpl.this);
+		
+			   					na.addPropertyChangeListener(
+			   							new NetworkAdminPropertyChangeListener()
+			   							{
+			   								private String	last_as;
+		
+			   								public void
+			   								propertyChanged(
+			   										String		property )
+			   								{
+			   									NetworkAdmin na = NetworkAdmin.getSingleton();
+		
+			   									if ( property.equals( NetworkAdmin.PR_NETWORK_INTERFACES )){
+		
+			   										boolean	found_usable = false;
+		
+			   										NetworkAdminNetworkInterface[] intf = na.getInterfaces();
+		
+			   										for (int i=0;i<intf.length;i++){
+		
+			   											NetworkAdminNetworkInterfaceAddress[] addresses = intf[i].getAddresses();
+		
+			   											for (int j=0;j<addresses.length;j++){
+		
+			   												if ( !addresses[j].isLoopback()){
+		
+			   													found_usable = true;
+			   												}
+			   											}
+			   										}
+		
+			   										// ignore event if nothing usable
+		
+			   										if ( !found_usable ){
+		
+			   											return;
+			   										}
+		
+			   										Logger.log(	new LogEvent(LOGID, "Network interfaces have changed (new=" + na.getNetworkInterfacesAsString() + ")"));
+		
+			   										announceAll( false );
+		
+			   									}else if ( property.equals( NetworkAdmin.PR_AS )){
+		
+			   										String	as = na.getCurrentASN().getAS();
+		
+			   										if ( last_as == null ){
+		
+			   											last_as = as;
+		
+			   										}else if ( !as.equals( last_as )){
+		
+			   											Logger.log(	new LogEvent(LOGID, "AS has changed (new=" + as + ")" ));
+		
+			   											last_as = as;
+		
+			   											announceAll( false );
+			   										}
+			   									}
+			   								}
+			   							});
+	   						}
+	   					}.start();
+	   				}
+	   			});
 
-						   boolean	found_usable = false;
-						   
-						   NetworkAdminNetworkInterface[] intf = na.getInterfaces();
-						   
-						   for (int i=0;i<intf.length;i++){
-							   
-							   NetworkAdminNetworkInterfaceAddress[] addresses = intf[i].getAddresses();
-							   
-							   for (int j=0;j<addresses.length;j++){
-								   
-								   if ( !addresses[j].isLoopback()){
-									   
-									   found_usable = true;
-								   }
-							   }
-						   }
-						   
-						   		// ignore event if nothing usable
-						   
-						   if ( !found_usable ){
-							   
-							   return;
-						   }
-						   
-						   Logger.log(	new LogEvent(LOGID, "Network interfaces have changed (new=" + na.getNetworkInterfacesAsString() + ")"));
+	   delayed_task.queue();
 
-						   announceAll( false );
-						   
-					   }else if ( property.equals( NetworkAdmin.PR_AS )){
-						   
-						   String	as = na.getCurrentASN().getAS();
-						   
-						   if ( last_as == null ){
-							   
-							   last_as = as;
-							   
-						   }else if ( !as.equals( last_as )){
-							   
-							   Logger.log(	new LogEvent(LOGID, "AS has changed (new=" + as + ")" ));
+			if (stopped) {
+				System.err.println("Core stopped while starting");
+				return;
+			}
 
-							   last_as = as;
-							   
-							   announceAll( false );
-						   }
-					   }
-				   }
-			   });
+	   Object[] runningListeners;
+	   mon_coreRunningListeners.enter();
+	   try {
+	  	 if (coreRunningListeners == null) {
+	  		 runningListeners = new Object[0];
+	  	 } else {
+	  		 runningListeners = coreRunningListeners.toArray();
+	  		 coreRunningListeners = null;
+	  	 }
+	  	 
+	   } finally {
+	  	 mon_coreRunningListeners.exit();
+	   }
+
+		// Trigger Listeners now that core is started
+		new AEThread2("Plugin Init Complete", false )
+		{
+			public void
+			run()
+			{
+				Iterator	it = lifecycle_listeners.iterator();
+				
+				while( it.hasNext()){
+					
+					try{
+						AzureusCoreLifecycleListener listener = (AzureusCoreLifecycleListener)it.next();
+						
+						if ( !listener.requiresPluginInitCompleteBeforeStartedEvent()){
+						
+							listener.started( AzureusCoreImpl.this );
+						}
+					}catch( Throwable e ){
+						
+						Debug.printStackTrace(e);
+					}
+				}
+				
+				pi.initialisationComplete();
+				
+				it = lifecycle_listeners.iterator();
+				
+				while( it.hasNext()){
+					
+					try{
+						AzureusCoreLifecycleListener listener = (AzureusCoreLifecycleListener)it.next();
+						
+						if ( listener.requiresPluginInitCompleteBeforeStartedEvent()){
+						
+							listener.started( AzureusCoreImpl.this );
+						}				
+					}catch( Throwable e ){
+						
+						Debug.printStackTrace(e);
+					}
+				}
+			}
+		}.start();
+		
+		for (Object l : runningListeners) {
+			try {
+				((AzureusCoreRunningListener) l).azureusCoreRunning(this);
+			} catch (Throwable t) {
+				Debug.out(t);
+			}
+		}
+		
+		Debug.out("Core Start Complete");
 	}
-	
+
 	public boolean
 	isStarted()
 	{
-		return( started );
+	   mon_coreRunningListeners.enter();
+	   try {
+	  	 return( started && coreRunningListeners == null );
+	   } finally {
+	  	 mon_coreRunningListeners.exit();
+	   }
 	}
 	
 	public void 
 	triggerLifeCycleComponentCreated(
 		AzureusCoreComponent component )
 	{
-		for (int i = 0; i < lifecycle_listeners.size(); i++) {
+		Iterator it = lifecycle_listeners.iterator();
+		
+		while( it.hasNext()){
 
 			try{
-				((AzureusCoreLifecycleListener) lifecycle_listeners.get(i))
-					.componentCreated(this, component);
+				((AzureusCoreLifecycleListener)it.next()).componentCreated(this, component);
 				
 			}catch( Throwable e ){
 				
@@ -851,6 +1194,8 @@ AzureusCoreImpl
 	
 		throws AzureusCoreException
 	{
+		AEDiagnostics.flushPendingLogs();
+		
 		try{
 			this_mon.enter();
 		
@@ -894,9 +1239,10 @@ AzureusCoreImpl
 		List	sync_listeners 	= new ArrayList();
 		List	async_listeners	= new ArrayList();
 		
-		for (int i=0;i<lifecycle_listeners.size();i++){
+		Iterator it = lifecycle_listeners.iterator();
 			
-			AzureusCoreLifecycleListener	l = (AzureusCoreLifecycleListener)lifecycle_listeners.get(i);
+		while( it.hasNext()){
+			AzureusCoreLifecycleListener	l = (AzureusCoreLifecycleListener)it.next();
 			
 			if ( l.syncInvokeRequired()){
 				sync_listeners.add( l );
@@ -945,7 +1291,9 @@ AzureusCoreImpl
 			if (Logger.isEnabled())
 				Logger.log(new LogEvent(LOGID, "Stopping global manager"));
 
-			global_manager.stopGlobalManager();
+			if (global_manager != null) {
+				global_manager.stopGlobalManager();
+			}
 			
 			if (Logger.isEnabled())
 				Logger.log(new LogEvent(LOGID, "Invoking synchronous 'stopped' listeners"));
@@ -1052,9 +1400,11 @@ AzureusCoreImpl
 		runNonDaemon(new AERunnable() {
 			public void runSupport() {
 
-				for (int i = 0; i < lifecycle_listeners.size(); i++) {
+				Iterator it = lifecycle_listeners.iterator();
+				
+				while( it.hasNext()){
 
-					if (!((AzureusCoreLifecycleListener) lifecycle_listeners.get(i))
+					if (!((AzureusCoreLifecycleListener)it.next())
 							.stopRequested(AzureusCoreImpl.this)) {
 						if (Logger.isEnabled())
 							Logger.log(new LogEvent(LOGID, LogEvent.LT_WARNING,
@@ -1103,9 +1453,10 @@ AzureusCoreImpl
             public void runSupport() {
                 checkRestartSupported();
 
-                for (int i = 0; i < lifecycle_listeners.size(); i++) {
-                    AzureusCoreLifecycleListener l = (AzureusCoreLifecycleListener) lifecycle_listeners
-                            .get(i);
+                Iterator it = lifecycle_listeners.iterator();
+                
+                while( it.hasNext()){
+                    AzureusCoreLifecycleListener l = (AzureusCoreLifecycleListener)it.next();
 
                     if (!l.restartRequested(AzureusCoreImpl.this)) {
 
@@ -1176,7 +1527,7 @@ AzureusCoreImpl
 	
 		throws AzureusCoreException
 	{
-			// don't test for runnign here, the restart process calls this after terminating the core...
+			// don't test for running here, the restart process calls this after terminating the core...
 		
 		return( PluginInitializer.getDefaultInterface().getPluginManager());
 	}
@@ -1328,10 +1679,12 @@ AzureusCoreImpl
 			PluginInitializer.fireEvent( PluginEvent.PEV_INITIALISATION_PROGRESS_TASK, currentTask );
 		}
 		
-		for (int i=0;i<listeners.size();i++){
+		Iterator it = listeners.iterator();
+		
+		while( it.hasNext()){
 			
 			try{
-				((AzureusCoreListener)listeners.get(i)).reportCurrentTask( op, currentTask );
+				((AzureusCoreListener)it.next()).reportCurrentTask( op, currentTask );
 				
 			}catch( Throwable e ){
 				
@@ -1350,10 +1703,12 @@ AzureusCoreImpl
 			PluginInitializer.fireEvent( PluginEvent.PEV_INITIALISATION_PROGRESS_PERCENT, new Integer( percent ));
 		}
 
-		for (int i=0;i<listeners.size();i++){
+		Iterator it = listeners.iterator();
+		
+		while( it.hasNext()){
 			
 			try{
-				((AzureusCoreListener)listeners.get(i)).reportPercent( op, percent );
+				((AzureusCoreListener)it.next()).reportPercent( op, percent );
 				
 			}catch( Throwable e ){
 				
@@ -1403,4 +1758,19 @@ AzureusCoreImpl
 	{
 		operation_listeners.remove(l);
 	}
+
+	public static void addCoreRunningListener(AzureusCoreRunningListener l) {
+	   mon_coreRunningListeners.enter();
+	   try {
+    		if (AzureusCoreImpl.coreRunningListeners != null) {
+    			coreRunningListeners.add(l);
+    			
+    			return;
+    		}
+	   } finally {
+	  	 mon_coreRunningListeners.exit();
+	   }
+	   
+	   l.azureusCoreRunning(AzureusCoreImpl.getSingleton());
+	}
 }
