@@ -25,6 +25,7 @@ import java.util.*;
 import org.gudy.azureus2.core3.util.Debug;
 import org.gudy.azureus2.core3.util.SystemTime;
 
+import com.aelitis.azureus.core.cnetwork.ContentNetwork;
 import com.aelitis.azureus.login.NotLoggedInException;
 import com.aelitis.azureus.util.JSONUtils;
 import com.aelitis.azureus.util.LoginInfoManager;
@@ -49,11 +50,15 @@ public class PlatformMessage
 	private final long messageCreatedOn;
 
 	private long lSequenceNo = -1;
-	
+
 	private boolean requiresAuthorization = false;
-	
+
 	private boolean loginAndRetry = false;
 
+	private boolean sendAZID = true;
+
+	private long contentNetworkID;
+
 	/**
 	 * @param messageID
 	 * @param listenerID
@@ -68,6 +73,7 @@ public class PlatformMessage
 		this.listenerID = listenerID;
 		this.operationID = operationID;
 		this.parameters = JSONUtils.encodeToJSONObject(parameters);
+		this.contentNetworkID = ContentNetwork.CONTENT_NETWORK_VUZE;
 
 		messageCreatedOn = SystemTime.getCurrentTime();
 		fireBeforeDate = messageCreatedOn + maxDelayMS;
@@ -81,6 +87,7 @@ public class PlatformMessage
 		this.operationID = operationID;
 
 		this.parameters = JSONUtils.encodeToJSONObject(parseParams(parameters));
+		this.contentNetworkID = ContentNetwork.CONTENT_NETWORK_VUZE;
 
 		messageCreatedOn = SystemTime.getCurrentTime();
 		fireBeforeDate = messageCreatedOn + maxDelayMS;
@@ -146,6 +153,9 @@ public class PlatformMessage
 	public String toString() {
 		String paramString = parameters.toString();
 		return "PlaformMessage {"
+				+ "cn"
+				+ contentNetworkID
+				+ ", "
 				+ lSequenceNo
 				+ ", "
 				+ messageID
@@ -162,10 +172,9 @@ public class PlatformMessage
 	 * @param requiresAuthorization the requiresAuthorization to set
 	 * @throws NotLoggedInException 
 	 */
-	public void setRequiresAuthorization(
-		boolean requiresAuthorization,
-		boolean promptUser)
-	throws NotLoggedInException {
+	public void setRequiresAuthorization(boolean requiresAuthorization,
+			boolean promptUser)
+			throws NotLoggedInException {
 		this.requiresAuthorization = requiresAuthorization;
 		this.loginAndRetry = promptUser;
 
@@ -174,6 +183,10 @@ public class PlatformMessage
 		}
 	}
 
+	public void setRequiresAuthorizationNoCheck() {
+		this.requiresAuthorization = true;
+	}
+
 	/**
 	 * @return the requiresAuthorization
 	 */
@@ -194,8 +207,33 @@ public class PlatformMessage
 	public boolean getLoginAndRetry() {
 		return loginAndRetry;
 	}
-	
+
 	public String toShortString() {
-		return getMessageID() + "." + getListenerID() + "."+ getOperationID();
+		return (requiresAuthorization ? "AUTH: " : "") + getMessageID() + "."
+				+ getListenerID() + "." + getOperationID();
+	}
+
+	/**
+	 * @return
+	 *
+	 * @since 3.1.1.1
+	 */
+	public boolean sendAZID() {
+		return sendAZID;
+	}
+
+	public void setSendAZID(boolean send) {
+		sendAZID = send;
+		if (send && requiresAuthorization) {
+			System.err.println("requiresAuthorization overrides sendAZID disabling");
+		}
+	}
+
+	public long getContentNetworkID() {
+		return contentNetworkID;
+	}
+
+	public void setContentNetworkID(long contentNetworkID) {
+		this.contentNetworkID = contentNetworkID;
 	}
 }
