@@ -29,9 +29,13 @@ import java.net.URLDecoder;
 import java.util.*;
 
 
+import org.gudy.azureus2.core3.config.COConfigurationManager;
+import org.gudy.azureus2.core3.internat.MessageText;
 import org.gudy.azureus2.core3.util.AEDiagnostics;
 import org.gudy.azureus2.core3.util.BEncoder;
+import org.gudy.azureus2.core3.util.Base32;
 import org.gudy.azureus2.core3.util.Debug;
+import org.gudy.azureus2.core3.util.RandomUtils;
 import org.gudy.azureus2.core3.util.UrlUtils;
 import org.json.simple.JSONArray;
 import org.json.simple.JSONObject;
@@ -41,20 +45,26 @@ import com.aelitis.azureus.core.metasearch.Engine;
 import com.aelitis.azureus.core.metasearch.Result;
 import com.aelitis.azureus.core.metasearch.ResultListener;
 import com.aelitis.azureus.core.metasearch.SearchException;
+import com.aelitis.azureus.core.metasearch.SearchLoginException;
 import com.aelitis.azureus.core.metasearch.SearchParameter;
 import com.aelitis.azureus.core.metasearch.impl.plugin.PluginEngine;
 import com.aelitis.azureus.core.metasearch.impl.web.json.JSONEngine;
 import com.aelitis.azureus.core.metasearch.impl.web.regex.RegexEngine;
+import com.aelitis.azureus.core.metasearch.impl.web.rss.RSSEngine;
 import com.aelitis.azureus.core.vuzefile.VuzeFile;
 import com.aelitis.azureus.core.vuzefile.VuzeFileComponent;
 import com.aelitis.azureus.core.vuzefile.VuzeFileHandler;
-import com.aelitis.azureus.util.Constants;
+import com.aelitis.azureus.util.ConstantsVuze;
+import com.aelitis.azureus.util.ImportExportUtils;
 import com.aelitis.azureus.util.JSONUtils;
 
+
 public abstract class 
 EngineImpl
 	implements Engine
 {
+	private static final int DEFAULT_UPDATE_CHECK_SECS = 24*60*60;
+	
 	protected static EngineImpl
 	importFromBEncodedMap(
 		MetaSearchImpl		meta_search,
@@ -76,6 +86,10 @@ EngineImpl
 			
 			return( PluginEngine.importFromBEncodedMap( meta_search, map ));
 			
+		}else if ( type == Engine.ENGINE_TYPE_RSS ){
+			
+			return( RSSEngine.importFromBEncodedMap( meta_search, map ));
+			
 		}else{
 			
 			throw( new IOException( "Unknown engine type " + type ));
@@ -103,12 +117,31 @@ EngineImpl
 			
 			return( RegexEngine.importFromJSONString( meta_search, id, last_updated, name, map ));
 			
+		}else if ( type == Engine.ENGINE_TYPE_RSS ){
+			
+			return( RSSEngine.importFromJSONString( meta_search, id, last_updated, name, map ));
+			
 		}else{
 			
 			throw( new IOException( "Unknown engine type " + type ));
 		}	
 	}
 	
+		// if you extend this with state pertaining to what has been downloaded then
+		// make sure you extend the code in reset();
+	
+	protected static final String		LD_COOKIES				= "cookies";
+	protected static final String		LD_ETAG					= "etag";
+	protected static final String		LD_LAST_MODIFIED		= "last_mod";
+	
+	protected static final String		LD_LAST_UPDATE_CHECK	= "last_update_check";
+	protected static final String		LD_UPDATE_CHECK_SECS	= "update_check_secs";
+	protected static final String		LD_CREATED_BY_ME		= "mine";
+	
+	protected static final String		LD_AUTO_DL_SUPPORTED	= "auto_dl_supported";
+	protected static final String		LD_LINK_IS_TORRENT		= "link_is_torrent";
+
+	
 	private MetaSearchImpl	meta_search;
 	
 	private int			type;
@@ -116,6 +149,12 @@ EngineImpl
 	private long		last_updated;
 	private String		name;
 	
+	private byte[]		uid;
+	private int			version;
+	
+	private boolean		is_public					= true;
+	
+	private int			az_version;
 	
 	private int			selection_state				= SEL_STATE_DESELECTED;
 	private boolean		selection_state_recorded	= true;
@@ -125,13 +164,20 @@ EngineImpl
 		// first mappings used to canonicalise names and map field to same field
 		// typically used for categories (musak->music)
 	
-	List		first_level_mapping	= new ArrayList();
+	private List		first_level_mapping	= new ArrayList();
 	
 		// second mappings used to generate derived field values
 		// typically used to derive content_type from category (music->AUDIO)
 	
-	List		second_level_mapping	= new ArrayList();
+	private List		second_level_mapping	= new ArrayList();
 
+		// applicable to non-vuze hosted templates
+	
+	private String 		update_url;
+	private int			update_check_default_secs;
+	
+	private Map			user_data;
+	
 		// manual constructor
 	
 	protected
@@ -147,6 +193,11 @@ EngineImpl
 		id				= _id;
 		last_updated	= _last_updated;
 		name			= _name;
+		
+		version			= 1;
+		az_version		= AZ_VERSION;
+			
+		allocateUID( id );
 	}
 	
 		// bencoded constructor
@@ -162,17 +213,36 @@ EngineImpl
 		
 		type			= ((Long)map.get( "type" )).intValue();
 		id				= ((Long)map.get( "id")).longValue();
-		last_updated	= importLong( map, "last_updated" );
-		name			= importString( map, "name" );
+		last_updated	= ImportExportUtils.importLong( map, "last_updated" );
+		name			= ImportExportUtils.importString( map, "name" );
 		
-		selection_state	= (int)importLong( map, "selected", SEL_STATE_DESELECTED );
+		selection_state	= (int)ImportExportUtils.importLong( map, "selected", SEL_STATE_DESELECTED );
 		
-		selection_state_recorded = importBoolean(map,"select_rec", true );
+		selection_state_recorded = ImportExportUtils.importBoolean(map,"select_rec", true );
 		
-		source			= (int)importLong( map, "source", ENGINE_SOURCE_UNKNOWN );
+		source			= (int)ImportExportUtils.importLong( map, "source", ENGINE_SOURCE_UNKNOWN );
 		
 		first_level_mapping 	= importBEncodedMappings( map, "l1_map" );
 		second_level_mapping 	= importBEncodedMappings( map, "l2_map" );
+		
+		version = (int)ImportExportUtils.importLong( map, "version", 1 );
+		
+		az_version = (int)ImportExportUtils.importLong( map, "az_version", AZ_VERSION );
+
+		if ( az_version > AZ_VERSION ){
+			
+			throw( new IOException( MessageText.getString("metasearch.template.version.bad", new String[]{ name })));
+		}
+		
+		uid		= (byte[])map.get( "uid" );
+		
+		if ( uid == null ){
+			
+			allocateUID( id );
+		}
+		
+		update_url 					= ImportExportUtils.importString( map, "update_url" );
+		update_check_default_secs	= (int)ImportExportUtils.importLong( map, "update_url_check_secs", DEFAULT_UPDATE_CHECK_SECS );
 	}
 	
 		// bencoded export
@@ -187,16 +257,27 @@ EngineImpl
 		map.put( "id", new Long( id ));
 		map.put( "last_updated", new Long( last_updated ));
 		
-		exportString( map, "name", name );
+		ImportExportUtils.exportString( map, "name", name );
 		
 		map.put( "selected", new Long( selection_state ));
 		
-		exportBoolean( map, "select_rec", selection_state_recorded );
+		ImportExportUtils.exportBoolean( map, "select_rec", selection_state_recorded );
 		
 		map.put( "source", new Long( source ));
 		
 		exportBEncodedMappings( map, "l1_map", first_level_mapping );
 		exportBEncodedMappings( map, "l2_map", second_level_mapping );
+		
+		map.put( "version", new Long( version ));
+		map.put( "az_version", new Long( az_version ));
+		map.put( "uid", uid );
+		
+		if ( update_url != null ){
+		
+			ImportExportUtils.exportString( map, "update_url", update_url );
+		}
+		
+		map.put( "update_url_check_secs", new Long( update_check_default_secs ));
 	}
 	
 		// json constructor
@@ -216,6 +297,28 @@ EngineImpl
 		
 		first_level_mapping 	= importJSONMappings( map, "value_map", true );
 		second_level_mapping 	= importJSONMappings( map, "ctype_map", false );
+		
+		version 	= (int)ImportExportUtils.importLong( map, "version", 1 );
+		az_version 	= (int)ImportExportUtils.importLong( map, "az_version", AZ_VERSION );
+		
+		if ( az_version > AZ_VERSION ){
+			
+			throw( new IOException( MessageText.getString( "metasearch.template.version.bad", new String[]{ name })));
+		}
+		
+		String uid_str		= (String)map.get( "uid" );
+		
+		if ( uid_str == null ){
+			
+			allocateUID( id );
+			
+		}else{
+			
+			uid = Base32.decode( uid_str );
+		}
+		
+		update_url 					= ImportExportUtils.importString( map, "update_url" );
+		update_check_default_secs	= (int)ImportExportUtils.importLong( map, "update_url_check_secs", DEFAULT_UPDATE_CHECK_SECS );
 	}
 	
 		// json export
@@ -228,6 +331,18 @@ EngineImpl
 	{
 		exportJSONMappings( res, "value_map", first_level_mapping, true );
 		exportJSONMappings( res, "ctype_map", second_level_mapping, false );
+		
+		res.put( "version", new Long( version ));
+		res.put( "az_version", new Long( az_version ));
+		
+		res.put( "uid", Base32.encode( uid ));
+		
+		if ( update_url != null ){
+			
+			ImportExportUtils.exportJSONString( res, "update_url", update_url );
+		}
+		
+		res.put( "update_url_check_secs", new Long( update_check_default_secs ));
 	}
 	
 	protected List
@@ -379,8 +494,8 @@ EngineImpl
 					
 					Map	entry2 = (Map)l2.get(j);
 					
-					String	from_str 	= importString( entry2, "from" );
-					String	to_str 		= importString( entry2, "to" );
+					String	from_str 	= ImportExportUtils.importString( entry2, "from" );
+					String	to_str 		= ImportExportUtils.importString( entry2, "to" );
 					
 					mappings[j] = new FieldRemapping( from_str, to_str );
 				}
@@ -429,8 +544,8 @@ EngineImpl
 				
 				l2.add( m2 );
 				
-				exportString( m2, "from", fr.getMatchString());
-				exportString( m2, "to", fr.getReplacement());
+				ImportExportUtils.exportString( m2, "from", fr.getMatchString());
+				ImportExportUtils.exportString( m2, "to", fr.getReplacement());
 			}
 		}
 	}
@@ -447,16 +562,70 @@ EngineImpl
 		return( obj.toString());
 	}
 	
+	public int
+	getAZVersion()
+	{
+		return( az_version );
+	}
+	
+	public int
+	getVersion()
+	{
+		return( version );
+	}
+	
+	protected void
+	setVersion(
+		int		_v )
+	{
+		version	= _v;
+	}
+	
+	public String
+	getUID()
+	{
+		return( Base32.encode( uid ));
+	}
+	
+	protected void
+	setUID(
+		String	str )
+	{
+		uid	= Base32.decode( str );
+	}
+	
+	protected void
+	allocateUID(
+		long		id )
+	{
+		uid = new byte[10];
+
+		if ( id >= 0 && id < Integer.MAX_VALUE ){
+		
+				// id is a vuze one, derive uid from it as already unique
+			
+			uid[0] 	= (byte)(id>>24);
+			uid[1]	= (byte)(id>>16);
+			uid[2]	= (byte)(id>>8);
+			uid[3]	= (byte)id;
+
+		}else{
+	
+			RandomUtils.nextSecureBytes( uid );
+			
+			configDirty();
+		}
+	}
 
 	public boolean
-	sameAs(
+	sameLogicAs(
 		Engine	other )
 	{
 		try{
 			Map	m1 = exportToBencodedMap();
 			Map	m2 = other.exportToBencodedMap();
 		
-			String[]	to_remove = {"type","id","last_updated","selected","select_rec","source"};
+			String[]	to_remove = {"type","id","last_updated","selected","select_rec","source", "version", "az_version", "uid" };
 			
 			for (int i=0;i<to_remove.length;i++){
 				
@@ -473,59 +642,93 @@ EngineImpl
 			return( false );
 		}
 	}
-	
+		
 	public Result[]
 	search(
-		SearchParameter[] 	params )
+		SearchParameter[] 	params,
+		Map					context,
+		int					desired_max_matches,
+		int					absolute_max_matches,
+		String				headers,
+		ResultListener		listener )
 	
 		throws SearchException
 	{
-		return( searchAndMap( params, -1, null, null ));
-	}
-	
-	public Result[]
-  	search(
-  		SearchParameter[] 	params,
-  		String				headers )
-  	
-  		throws SearchException
-  	{
-		return( searchAndMap( params, -1, headers, null ));
-  	}
-	
-	public void
-	search(
-		SearchParameter[] 	params,
-		int					max_matches,
-		String				headers,
-		ResultListener		listener )
-	{
+		if ( context == null ){
+			
+			context = new HashMap();
+		}
+		
 		try{
-			Result[] results = searchAndMap( params, max_matches, headers, listener) ;
+			Result[] results = searchAndMap( params, context, desired_max_matches, absolute_max_matches, headers, listener) ;
 			
-			listener.resultsReceived( this, results );
+			if ( listener != null ){
+				
+				listener.resultsReceived( this, results );
+				
+				listener.resultsComplete( this );
+			}
 			
-			listener.resultsComplete( this );
+			return( results );
 			
 		}catch( Throwable e ){
 			
-			listener.engineFailed( this, e);
+			if ( e instanceof SearchLoginException ){
+				
+				if ( listener != null ){
+				
+					listener.engineRequiresLogin(this, e);
+				}
+				
+				throw((SearchLoginException)e);
+				
+			}else if ( e instanceof SearchException ){
+				
+				if ( listener != null ){
+				
+					listener.engineFailed( this, e);
+				}
+				
+				throw((SearchException)e);
+				
+			}else{
+				
+				if ( listener != null ){
+				
+					listener.engineFailed( this, e);
+				}
+				
+				throw( new SearchException( "Search failed", e ));
+			}
 		}
 	}
 	
 	protected Result[]
 	searchAndMap(
 		SearchParameter[] 			params,
-		int							max_matches,
+		Map							context,
+		int							desired_max_matches,
+		int							absolute_max_matches,
 		String						headers,
 		final ResultListener		listener )
 	
 		throws SearchException
 	{
-		 Result[] results = 
+			// default values
+		
+		context.put( Engine.SC_AZID, 	ConstantsVuze.AZID );
+
+		if ( context.get( Engine.SC_SOURCE ) == null ){
+			
+			context.put( Engine.SC_SOURCE, "search" );
+		}
+		
+		Result[] results = 
 			 searchSupport( 
 					params, 
-					max_matches, 
+					context,
+					desired_max_matches,
+					absolute_max_matches,
 					headers, 
 					new ResultListener()
 					{
@@ -579,6 +782,18 @@ EngineImpl
 								listener.engineFailed(engine, cause);
 							}
 						}
+						
+						public void 
+						engineRequiresLogin(
+							Engine engine, 
+							Throwable cause )
+						{
+							log( "Search requires login", cause );
+							
+							if ( listener != null ){
+								listener.engineRequiresLogin(engine, cause);
+							}
+						}
 					});
 		 
 		 return( mapResults( results ));
@@ -613,7 +828,9 @@ EngineImpl
 	protected abstract Result[]
 	searchSupport(
 		SearchParameter[] 	params,
-		int					max_matches,
+		Map					searchContext,
+		int					desired_max_matches,
+		int					absolute_max_matches,
 		String				headers,
 		ResultListener		listener )
 	
@@ -654,116 +871,7 @@ EngineImpl
 		
 		return( null );
 	}
-	
-	protected void
-	exportString(
-		Map		map,
-		String	key,
-		String	value )
-	
-		throws IOException
-	{
-		if ( value != null ){
-	
-			map.put( key, value.getBytes( "UTF-8" ));
-		}
-	}
-	
-	protected String
-	importString(
-		Map		map,
-		String	key )
-	
-		throws IOException
-	{
-		Object	obj = map.get( key );
-		
-		if ( obj instanceof String ){
-			
-			return((String)obj);
-			
-		}else if ( obj instanceof byte[]){
-			
-			return( new String((byte[])obj, "UTF-8" ));
-		}
-		
-		return( null );
-	}
-	
-	protected long
-	importLong(
-		Map		map,
-		String	key )
-	
-		throws IOException
-	{
-		return( importLong( map, key, 0 ));
-	}
-	
-	protected long
-	importLong(
-		Map		map,
-		String	key,
-		long	def )
-	
-		throws IOException
-	{
-		Object	obj = map.get( key );
-		
-		if ( obj instanceof Long){
-			
-			return(((Long)obj).longValue());
-			
-		}else if ( obj instanceof String ){
-			
-			return( Long.parseLong((String)obj));
-		}
-		
-		return( def );
-	}
 
-	protected void
-	exportBoolean(
-		Map		map,
-		String	key,
-		boolean	value )
-	
-		throws IOException
-	{
-		map.put( key, new Long( value?1:0 ));
-	}
-	
-	protected boolean
-	importBoolean(
-		Map		map,
-		String	key )
-	
-		throws IOException
-	{
-		return( importBoolean( map, key, false ));
-	}
-	
-	protected boolean
-	importBoolean(
-		Map		map,
-		String	key,
-		boolean	def )
-	
-		throws IOException
-	{
-		Object	obj = map.get( key );
-		
-		if ( obj instanceof Long){
-			
-			return(((Long)obj).longValue() == 1 );
-		}else if ( obj instanceof Boolean ){
-			
-			return(((Boolean)obj).booleanValue());
-		}
-		
-		return( def );
-	}
-	
 	public int
 	getType()
 	{
@@ -792,7 +900,14 @@ EngineImpl
 	public String 
 	getName() 
 	{
-		return name;
+		return( name );
+	}
+	
+	protected void
+	setName(
+		String		n )
+	{
+		name		= n;
 	}
 	
 	public boolean
@@ -801,6 +916,19 @@ EngineImpl
 		return(	getSelectionState() != SEL_STATE_DESELECTED );
 	}
 	
+	public boolean
+	isPublic()
+	{
+		return( is_public );
+	}
+	
+	protected void
+	setPublic(
+		boolean		p )
+	{
+		is_public = p;
+	}
+	
 	public int
 	getSelectionState()
 	{
@@ -850,7 +978,7 @@ EngineImpl
 				
 				log( "Marking template id " + getId() + " as selected=" + selected );
 				
-				PlatformMetaSearchMessenger.setTemplatetSelected( getId(), Constants.AZID, selected);
+				PlatformMetaSearchMessenger.setTemplatetSelected( getId(), ConstantsVuze.AZID, selected);
 				
 				selection_state_recorded = true;
 				
@@ -871,11 +999,93 @@ EngineImpl
 	setSource(
 		int		_source )
 	{
-		source	= _source;
+		if ( source != _source ){
 		
-		configDirty();
+			source	= _source;
+		
+			configDirty();
+		}
 	}
 	
+	public boolean
+	isMine()
+	{
+		return( getLocalBoolean( LD_CREATED_BY_ME, false ));
+	}
+	
+	public void
+	setMine(
+		boolean		mine )
+	{
+		setLocalBoolean( LD_CREATED_BY_ME, mine );
+	}
+	
+	protected String
+	getUpdateURL()
+	{
+		return( update_url );
+	}
+	
+	protected void
+	setUpdateURL(
+		String	url )
+	{
+		update_url	= url;
+	}
+	
+	protected int
+	getUpdateCheckSecs()
+	{
+		long	l = getLocalLong( LD_UPDATE_CHECK_SECS, 0 );
+		
+		if ( l != 0 ){
+			
+			return((int)l);
+		}
+		
+		return( update_check_default_secs );
+	}
+	
+	protected void
+	setDefaultUpdateCheckSecs(
+		int		secs )
+	{
+		update_check_default_secs = secs;
+	}
+	
+	protected void
+	setLocalUpdateCheckSecs(
+		int		secs )
+	{
+		setLocalLong( LD_UPDATE_CHECK_SECS, secs );
+	}
+			
+	protected long
+	getLastUpdateCheck()
+	{
+		return( getLocalLong( LD_LAST_UPDATE_CHECK, 0 ));
+	}
+	
+	protected void
+	setLastUpdateCheck(
+		long		when )
+	{
+		setLocalLong( LD_LAST_UPDATE_CHECK, when );
+	}
+	
+	public int 
+	getAutoDownloadSupported() 
+	{
+			// default impl based on availability of fields that permit auto-dl
+			// overridden in RSS engine ( for example ) to be based on explicit
+			// channel tag
+		
+		boolean ok =
+			supportsField( Engine.FIELD_TORRENTLINK ) || 
+			supportsField( Engine.FIELD_DOWNLOADBTNLINK );
+
+		return( ok?AUTO_DL_SUPPORTED_YES:AUTO_DL_SUPPORTED_NO );
+	}
 	protected void
 	configDirty()
 	{
@@ -900,6 +1110,187 @@ EngineImpl
 		vf.write( target );
 	}
 	
+	private String
+	getLocalKey()
+	{
+		return( "metasearch.engine." + id + ".local" );
+	}
+	
+	public void
+	reset()
+	{
+		synchronized( this ){
+			
+			Map map = COConfigurationManager.getMapParameter( getLocalKey(), new HashMap());
+
+			map.remove(	LD_COOKIES );
+			map.remove(	LD_ETAG );
+			map.remove(	LD_LAST_MODIFIED );
+			
+			COConfigurationManager.setParameter( getLocalKey(), map );
+		}
+	}
+	
+	protected void
+	setLocalString(
+		String		key,
+		String		value )
+	{
+		synchronized( this ){
+	
+			String	existing = getLocalString( key );
+			
+			if ( existing != null && value != null && existing.equals( value )){
+				
+				return;
+			}
+			
+			Map map = COConfigurationManager.getMapParameter( getLocalKey(), new HashMap());
+			
+			try{
+				ImportExportUtils.exportString(map, key, value);
+				
+				COConfigurationManager.setParameter( getLocalKey(), map );
+				
+			}catch( Throwable e ){
+				
+				Debug.printStackTrace( e );
+			}
+		}
+	}
+	
+	protected String
+	getLocalString(
+		String		key )
+	{
+		synchronized( this ){
+		
+			Map map = COConfigurationManager.getMapParameter( getLocalKey(), new HashMap());
+			
+			try{
+				return( ImportExportUtils.importString( map, key ));
+				
+			}catch( Throwable e ){
+				
+				Debug.printStackTrace( e );
+				
+				return( null );
+			}
+		}
+	}
+	
+	protected void
+	setLocalBoolean(
+		String		key,
+		boolean		value )
+	{
+		setLocalLong( key, value?1:0 );
+	}
+	
+	protected boolean
+	getLocalBoolean(
+		String		key,
+		boolean		def )
+	{
+		return( getLocalLong( key, def?1:0 ) == 1 );
+	}
+	
+	protected void
+	setLocalLong(
+		String		key,
+		long		value )
+	{
+		synchronized( this ){
+	
+			long	existing = getLocalLong( key, 0 );
+			
+			if ( existing == value ){
+				
+				return;
+			}
+			
+			Map map = COConfigurationManager.getMapParameter( getLocalKey(), new HashMap());
+			
+			try{
+				map.put( key, new Long( value));
+				
+				COConfigurationManager.setParameter( getLocalKey(), map );
+				
+			}catch( Throwable e ){
+				
+				Debug.printStackTrace( e );
+			}
+		}
+	}
+	
+	protected long
+	getLocalLong(
+		String		key,
+		long		def )
+	{
+		synchronized( this ){
+		
+			Map map = COConfigurationManager.getMapParameter( getLocalKey(), new HashMap());
+			
+			try{
+				return( ImportExportUtils.importLong( map, key, def ));
+				
+			}catch( Throwable e ){
+				
+				Debug.printStackTrace( e );
+				
+				return( def );
+			}
+		}
+	}
+	
+	protected void
+	setUserData(
+		Object	key,
+		Object	value )
+	{
+		synchronized( this ){
+			
+			if ( user_data == null ){
+				
+				if ( key == null ){
+					
+					return;
+				}
+				
+				user_data = new HashMap(4);
+			}
+			
+			if ( key == null ){
+				
+				user_data.remove( key );
+				
+				if ( user_data.size() == 0 ){
+					
+					user_data = null;
+				}
+			}else{
+				
+				user_data.put( key, value );
+			}
+		}
+	}
+	
+	protected Object
+	getUserData(
+		Object		key )
+	{
+		synchronized( this ){
+			
+			if ( user_data == null ){
+
+				return( null );
+			}
+			
+			return( user_data.get( key ));
+		}
+	}
+	
 	protected File
 	getDebugFile()
 	{
@@ -962,4 +1353,11 @@ EngineImpl
 	{
 		return( "id=" + getId() + ", name=" + getName() + ", source=" + ENGINE_SOURCE_STRS[getSource()] + ", selected=" + SEL_STATE_STRINGS[getSelectionState()]);
 	}
+	
+	public String
+	getString(
+		boolean		full )
+	{
+		return( getString());
+	}
 }
