@@ -24,9 +24,11 @@
 package com.aelitis.azureus.core.networkmanager.admin;
 
 import java.net.InetAddress;
+import java.nio.channels.UnsupportedAddressTypeException;
 
 import org.gudy.azureus2.core3.util.IndentWriter;
 
+import com.aelitis.azureus.core.AzureusCore;
 import com.aelitis.azureus.core.networkmanager.admin.impl.NetworkAdminImpl;
 
 public abstract class 
@@ -38,6 +40,10 @@ NetworkAdmin
 	public static final String PR_DEFAULT_BIND_ADDRESS	= "Default Bind IP";
 	public static final String PR_AS					= "AS";
 	
+	public static final int			IP_PROTOCOL_VERSION_AUTO		= 0;
+	public static final int			IP_PROTOCOL_VERSION_REQUIRE_V4	= 1;
+	public static final int			IP_PROTOCOL_VERSION_REQUIRE_V6	= 2;
+	
 	public static final String[]	PR_NAMES = {
 		PR_NETWORK_INTERFACES,
 		PR_DEFAULT_BIND_ADDRESS,
@@ -54,36 +60,58 @@ NetworkAdmin
 		
 		return( singleton );
 	}
+
+	public InetAddress getSingleHomedServiceBindAddress() {return getSingleHomedServiceBindAddress(IP_PROTOCOL_VERSION_AUTO);}
 	
-	public abstract InetAddress
-	getSingleHomedServiceBindAddress();
+	/**
+	 * @throws UnsupportedAddressTypeException when no address matching the v4/v6 requirements is found, always returns an address when auto is selected
+	 */
+	public abstract InetAddress 
+	getSingleHomedServiceBindAddress(int protocolVersion) throws UnsupportedAddressTypeException;
 	
 	public abstract InetAddress[]
-	getMultiHomedServiceBindAddresses();
+	getMultiHomedServiceBindAddresses(boolean forNIO);
 	
 	public abstract InetAddress
-	getMultiHomedOutgoingRoundRobinBindAddress();
+	getMultiHomedOutgoingRoundRobinBindAddress(InetAddress target);
 	
 	public abstract String
 	getNetworkInterfacesAsString();
 	
 	public abstract InetAddress[]
-	getAllBindAddresses();
+	getAllBindAddresses(
+		boolean	include_wildcard );
 
+	public abstract InetAddress
+	guessRoutableBindAddress();
+	
+		/**
+		 * Returns the list of current addresses that can successfully be bound
+		 * to with an ephemeral port
+		 * @return
+		 */
+	
+	public abstract InetAddress[]
+	getBindableAddresses();
+	
 	public abstract NetworkAdminNetworkInterface[]
 	getInterfaces();
 	
 	public abstract boolean
 	hasIPV4Potential();
+
+	public boolean hasIPV6Potential() {return hasIPV6Potential(false);}
 	
 	public abstract boolean
-	hasIPV6Potential();
-
-	public abstract NetworkAdminProtocol[]
-	getOutboundProtocols();
+	hasIPV6Potential(boolean forNIO);
 	
 	public abstract NetworkAdminProtocol[]
-	getInboundProtocols();
+	getOutboundProtocols(
+			AzureusCore azureus_core);
+	
+	public abstract NetworkAdminProtocol[]
+	getInboundProtocols(
+			AzureusCore azureus_core );
 	
 	public abstract InetAddress
 	testProtocol(
@@ -98,7 +126,8 @@ NetworkAdmin
 	getHTTPProxy();
 	
 	public abstract NetworkAdminNATDevice[]
-	getNATDevices();
+	getNATDevices(
+			AzureusCore azureus_core);
 	
 		/**
 		 * Only call if the supplied address is believed to be the current public address
@@ -128,6 +157,13 @@ NetworkAdmin
 		InetAddress		address )
 	
 		throws NetworkAdminException;
+	
+	public abstract void
+	lookupASN(
+		InetAddress					address,
+		NetworkAdminASNListener		listener );
+	
+	
 	public abstract boolean
 	canTraceRoute();
 	
@@ -150,6 +186,9 @@ NetworkAdmin
 	
 		throws NetworkAdminException;
 	
+	public abstract InetAddress
+	getDefaultPublicAddress();
+	
 	public abstract void
 	addPropertyChangeListener(
 		NetworkAdminPropertyChangeListener	listener );
@@ -163,7 +202,8 @@ NetworkAdmin
 		NetworkAdminPropertyChangeListener	listener );
 	
 	public abstract void
-	runInitialChecks();
+	runInitialChecks(
+			AzureusCore azureus_core);
 	
 	public abstract void
 	logNATStatus(
