@@ -24,54 +24,25 @@
 package com.aelitis.azureus.core.networkmanager.admin.impl;
 
 import java.io.PrintWriter;
-import java.net.Authenticator;
-import java.net.Inet4Address;
-import java.net.Inet6Address;
-import java.net.InetAddress;
-import java.net.NetworkInterface;
-import java.net.PasswordAuthentication;
-import java.net.SocketException;
-import java.net.UnknownHostException;
-import java.util.ArrayList;
-import java.util.Arrays;
-import java.util.Enumeration;
-import java.util.HashMap;
-import java.util.HashSet;
-import java.util.Iterator;
-import java.util.List;
-import java.util.Map;
-import java.util.Set;
+import java.net.*;
+import java.nio.channels.ServerSocketChannel;
+import java.nio.channels.UnsupportedAddressTypeException;
+import java.util.*;
 import java.util.regex.Pattern;
 
 import org.gudy.azureus2.core3.config.COConfigurationManager;
 import org.gudy.azureus2.core3.config.ParameterListener;
-import org.gudy.azureus2.core3.logging.LogAlert;
-import org.gudy.azureus2.core3.logging.LogEvent;
-import org.gudy.azureus2.core3.logging.LogIDs;
-import org.gudy.azureus2.core3.logging.Logger;
-import org.gudy.azureus2.core3.util.AEDiagnostics;
-import org.gudy.azureus2.core3.util.AEDiagnosticsEvidenceGenerator;
-import org.gudy.azureus2.core3.util.AESemaphore;
-import org.gudy.azureus2.core3.util.AEThread2;
-import org.gudy.azureus2.core3.util.Debug;
-import org.gudy.azureus2.core3.util.IndentWriter;
-import org.gudy.azureus2.core3.util.SimpleTimer;
-import org.gudy.azureus2.core3.util.SystemTime;
-import org.gudy.azureus2.core3.util.TimerEvent;
-import org.gudy.azureus2.core3.util.TimerEventPerformer;
-import org.gudy.azureus2.platform.PlatformManager;
-import org.gudy.azureus2.platform.PlatformManagerCapabilities;
-import org.gudy.azureus2.platform.PlatformManagerFactory;
-import org.gudy.azureus2.platform.PlatformManagerPingCallback;
+import org.gudy.azureus2.core3.logging.*;
+import org.gudy.azureus2.core3.util.*;
+import org.gudy.azureus2.platform.*;
 import org.gudy.azureus2.plugins.PluginInterface;
 import org.gudy.azureus2.plugins.platform.PlatformManagerException;
+import org.gudy.azureus2.plugins.utils.Utilities;
+import org.gudy.azureus2.pluginsimpl.local.PluginInitializer;
 
 import com.aelitis.azureus.core.AzureusCore;
 import com.aelitis.azureus.core.AzureusCoreFactory;
-import com.aelitis.azureus.core.instancemanager.AZInstance;
-import com.aelitis.azureus.core.instancemanager.AZInstanceManager;
-import com.aelitis.azureus.core.instancemanager.AZInstanceManagerListener;
-import com.aelitis.azureus.core.instancemanager.AZInstanceTracked;
+import com.aelitis.azureus.core.instancemanager.*;
 import com.aelitis.azureus.core.networkmanager.admin.*;
 import com.aelitis.azureus.core.networkmanager.impl.http.HTTPNetworkManager;
 import com.aelitis.azureus.core.networkmanager.impl.tcp.TCPNetworkManager;
@@ -91,8 +62,11 @@ NetworkAdminImpl
 	
 	private static final boolean	FULL_INTF_PROBE	= false;
 	
-	private Set				old_network_interfaces;
-	private InetAddress[]	currentBindIPs = new InetAddress[] {null};
+	private Set							old_network_interfaces;
+	private InetAddress[]				currentBindIPs			= new InetAddress[] { null };
+	private boolean						supportsIPv6withNIO		= true;
+	private boolean						supportsIPv6 = true;
+	private boolean						supportsIPv4 = true;
 	
 	private CopyOnWriteList	listeners = new CopyOnWriteList();
 		
@@ -133,6 +107,19 @@ NetworkAdminImpl
 	
 	private List as_history = new ArrayList();
 	
+	private AsyncDispatcher		async_asn_dispacher 	= new AsyncDispatcher();
+	private static final int	MAX_ASYNC_ASN_LOOKUPS	= 1024;
+	
+	private Map<InetAddress, NetworkAdminASN>	async_asn_history = 
+		new LinkedHashMap<InetAddress, NetworkAdminASN>(256,0.75f,true)
+		{
+			protected boolean 
+			removeEldestEntry(
+		   		Map.Entry<InetAddress, NetworkAdminASN> eldest) 
+			{
+				return size() > 256;
+			}
+		};
 		
 	public
 	NetworkAdminImpl()
@@ -228,6 +215,58 @@ NetworkAdminImpl
 			}
 			
 			if ( changed ){
+				
+				
+				boolean newV6 = false;
+				boolean newV4 = false;
+				
+				Set interfaces = old_network_interfaces;
+				if (interfaces != null)
+				{
+					Iterator it = interfaces.iterator();
+					while (it.hasNext())
+					{
+						NetworkInterface ni = (NetworkInterface) it.next();
+						Enumeration addresses = ni.getInetAddresses();
+						while (addresses.hasMoreElements())
+						{
+							InetAddress ia = (InetAddress) addresses.nextElement();
+
+							if (ia.isLoopbackAddress())
+								continue;
+							if (ia instanceof Inet6Address && !ia.isLinkLocalAddress())
+								newV6 = true;
+							else if (ia instanceof Inet4Address)
+								newV4 = true;
+						}
+					}
+				}
+				
+				supportsIPv4 = newV4;
+				supportsIPv6 = newV6;
+				
+				Logger.log(new LogEvent(LOGID, "NetworkAdmin: ipv4 supported: "+supportsIPv4+"; ipv6: "+supportsIPv6+"; probing v6+nio functionality"));
+				
+				if(newV6)
+				{
+					ServerSocketChannel channel = ServerSocketChannel.open();
+					
+					try
+					{
+						channel.configureBlocking(false);
+						channel.socket().bind(new InetSocketAddress(anyLocalAddressIPv6, 0));
+						Logger.log(new LogEvent(LOGID, "NetworkAdmin: testing nio + ipv6 bind successful"));
+
+						supportsIPv6withNIO = true;
+					} catch (Exception e)
+					{
+						Logger.log(new LogEvent(LOGID,LogEvent.LT_WARNING, "nio + ipv6 test failed",e));
+						supportsIPv6withNIO = false;
+					}
+					
+					channel.close();
+				} else
+					supportsIPv6withNIO = false;
 					
 				if ( !first_time ){
 					
@@ -244,39 +283,125 @@ NetworkAdminImpl
 		}
 	}
 	
-	private int roundRobinCounter = 0; 
+	private int roundRobinCounterV4 = 0;
+	private int roundRobinCounterV6 = 0;
 	
-	public InetAddress getMultiHomedOutgoingRoundRobinBindAddress()
+	public InetAddress getMultiHomedOutgoingRoundRobinBindAddress(InetAddress target)
 	{
 		InetAddress[]	addresses = currentBindIPs;
-		roundRobinCounter++;
-		int next = ( roundRobinCounter %= addresses.length );
-		return addresses[next];
+		boolean v6 = target instanceof Inet6Address; 
+		int previous = (v6 ? roundRobinCounterV6 : roundRobinCounterV4) % addresses.length;
+		InetAddress toReturn = null;
+		
+		int i = previous;
+
+		do
+		{
+			i++;i%= addresses.length;
+			if (target == null || (v6 && addresses[i] instanceof Inet6Address) || (!v6 && addresses[i] instanceof Inet4Address))
+			{
+				toReturn = addresses[i];
+				break;
+			} else if(!v6 && addresses[i].isAnyLocalAddress())
+			{
+				toReturn = anyLocalAddressIPv4;
+				break;				
+			}
+		} while(i!=previous);
+			
+		if(v6)
+			roundRobinCounterV6 = i;
+		else
+			roundRobinCounterV4 = i;
+		return toReturn != null ? toReturn : (v6 ? localhostV6 : localhostV4);
 	}
 	
-	public InetAddress[] getMultiHomedServiceBindAddresses()
+	private static InetAddress anyLocalAddressIPv4;
+	private static InetAddress anyLocalAddressIPv6;
+	private static InetAddress localhostV4;
+	private static InetAddress localhostV6;
+	
+	static
+	{
+		try
+		{
+			anyLocalAddressIPv4 = InetAddress.getByAddress(new byte[] { 0,0,0,0 });
+			anyLocalAddressIPv6  = InetAddress.getByAddress(new byte[] {0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0});
+			localhostV4 = InetAddress.getByAddress(new byte[] {127,0,0,1});
+			localhostV6 = InetAddress.getByAddress(new byte[] {0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,1});
+		} catch (UnknownHostException e)
+		{
+			e.printStackTrace();
+		}
+	}
+	
+	public InetAddress[] getMultiHomedServiceBindAddresses(boolean nio)
 	{
 		InetAddress[] bindIPs = currentBindIPs;
 		for(int i=0;i<bindIPs.length;i++)
 		{
-			if(bindIPs[i] == null)
-				return new InetAddress[] {null};
 			if(bindIPs[i].isAnyLocalAddress())
-				return new InetAddress[] {bindIPs[i]};
-			
+				return new InetAddress[] {nio && !supportsIPv6withNIO && bindIPs[i] instanceof Inet6Address ? anyLocalAddressIPv4 : bindIPs[i]};
 		}
 		return bindIPs;
 	}
 	
-	public InetAddress getSingleHomedServiceBindAddress()
+	public InetAddress getSingleHomedServiceBindAddress(int proto)
 	{
-		return currentBindIPs[0]; 
+		InetAddress[] addrs = currentBindIPs;
+		if(proto == IP_PROTOCOL_VERSION_AUTO){
+			return addrs[0];
+		}else{
+			for( InetAddress addr: addrs ){
+		
+				if( (proto == IP_PROTOCOL_VERSION_REQUIRE_V4 && addr instanceof Inet4Address || addr.isAnyLocalAddress()) ||
+					(proto == IP_PROTOCOL_VERSION_REQUIRE_V6 && addr instanceof Inet6Address) ){
+				
+					if ( addr.isAnyLocalAddress()){
+						
+						if ( proto == IP_PROTOCOL_VERSION_REQUIRE_V4 ){
+							
+							return( anyLocalAddressIPv4 );
+							
+						}else{
+							
+							return( anyLocalAddressIPv6 );
+						}
+					}else{
+						
+						return( addr );
+					}
+				}
+			}
+		}
+		
+		throw new UnsupportedAddressTypeException();
 	}
 	
 	public InetAddress[]
-	getAllBindAddresses()
+	getAllBindAddresses(
+		boolean	include_wildcard )
 	{
-		return( currentBindIPs );
+		if ( include_wildcard ){
+			
+			return( currentBindIPs );
+			
+		}else{
+			
+			List<InetAddress> res = new ArrayList<InetAddress>();
+			
+			InetAddress[] bind_ips = currentBindIPs;
+			
+			for ( InetAddress ip: bind_ips ){
+			
+				if( !ip.isAnyLocalAddress()){
+					
+					res.add( ip );
+				}
+			}
+			
+			return( res.toArray( new InetAddress[ res.size()]));
+		}
 	}
 	
 	private InetAddress[] calcBindAddresses(final String addressString, boolean enforceBind)
@@ -297,7 +422,7 @@ NetworkAdminImpl
 				{ // literal ipv4 or ipv6 address
 					if(currentAddress.indexOf('.') != -1 || currentAddress.indexOf(':') != -1)
 						parsedAddress = InetAddress.getByName(currentAddress);
-				} catch (UnknownHostException e)
+				} catch (Exception e)
 				{ // ignore, could be an interface name containing a ':'
 				}
 				
@@ -349,18 +474,8 @@ NetworkAdminImpl
 				}
 			}
 		
-		InetAddress localhost = null;
-		try
-		{
-			localhost = InetAddress.getByName("127.0.0.1");
-		} catch (UnknownHostException e)
-		{
-			e.printStackTrace();
-		}
-
-		
 		if(addrs.size() < 1)
-			return new InetAddress[] {enforceBind ? localhost : null};
+			return new InetAddress[] {enforceBind ? localhostV4 : (hasIPV6Potential() ? anyLocalAddressIPv6 : anyLocalAddressIPv4)};
 		return (InetAddress[])addrs.toArray(new InetAddress[0]);
 	}
 	
@@ -373,7 +488,7 @@ NetworkAdminImpl
 		InetAddress[] addrs = calcBindAddresses(bind_ip, enforceBind);
 		changed = !Arrays.equals(currentBindIPs, addrs);
 		if(changed){
-			currentBindIPs = addrs;					
+			currentBindIPs = addrs;
 			if (!first_time)
 			{
 				String logmsg = "NetworkAdmin: default bind ip has changed to '";
@@ -399,7 +514,7 @@ NetworkAdminImpl
 		{
 			NetworkInterface ni = (NetworkInterface) it.next();
 			Enumeration addresses = ni.getInetAddresses();
-			str+=ni.getName()+"\n";
+			str+=ni.getName()+"\t("+ni.getDisplayName()+")\n";
 			int i = 0;
 			while(addresses.hasMoreElements())
 				str+="\t"+ni.getName()+"["+(i++)+"]\t"+((InetAddress)addresses.nextElement()).getHostAddress()+"\n";
@@ -410,80 +525,374 @@ NetworkAdminImpl
 	public boolean
 	hasIPV4Potential()
 	{
-		Set	interfaces = old_network_interfaces;
-		
-		if ( interfaces == null ){
-			
-			return( false );
-		}
-		
-		Iterator	it = interfaces.iterator();
-				
-		while( it.hasNext()){
-			
-			NetworkInterface ni = (NetworkInterface)it.next();
-			
-			Enumeration addresses = ni.getInetAddresses();
-
-			while( addresses.hasMoreElements()){
-								
-				InetAddress	ia = (InetAddress)addresses.nextElement();
-				
-				if ( ia.isLoopbackAddress()){
-					
-					continue;
-				}
-				
-				if ( ia instanceof Inet4Address ){
-					
-					return( true );
-				}
-			}
-		}
-		
-		return( false );	
+		return supportsIPv4;
 	}
 	
 	public boolean
-	hasIPV6Potential()
+	hasIPV6Potential(boolean nio)
 	{
-		Set	interfaces = old_network_interfaces;
-		
-		if ( interfaces == null ){
-			
-			return( false );
-		}
-		
-		Iterator	it = interfaces.iterator();
-				
-		while( it.hasNext()){
-			
-			NetworkInterface ni = (NetworkInterface)it.next();
-			
-			Enumeration addresses = ni.getInetAddresses();
+		return nio ? supportsIPv6withNIO : supportsIPv6;
+	}
+	
+	public InetAddress[]
+  	getBindableAddresses()
+  	{
+  		List<InetAddress>	bindable = new ArrayList<InetAddress>();
+  		
+  		NetworkAdminNetworkInterface[] interfaces = NetworkAdmin.getSingleton().getInterfaces();
+  		
+  		for ( NetworkAdminNetworkInterface intf: interfaces ){
+  			
+  			NetworkAdminNetworkInterfaceAddress[] addresses = intf.getAddresses();
+  			
+  			for ( NetworkAdminNetworkInterfaceAddress address: addresses ){
 
-			while( addresses.hasMoreElements()){
-								
-				InetAddress	ia = (InetAddress)addresses.nextElement();
+  				InetAddress a = address.getAddress();
+  				
+  				if ( canBind( a )){
+  					
+  					bindable.add( a );
+  				}
+  			}
+  		}
+  		
+  		return( bindable.toArray( new InetAddress[ bindable.size()]));
+  	}
+  	
+  	protected boolean
+  	canBind(
+  		InetAddress	bind_ip )
+  	{
+  		ServerSocketChannel ssc = null;
+  		
+  		try{
+  			ssc = ServerSocketChannel.open();
+  		
+  			ssc.socket().bind( new InetSocketAddress(bind_ip,0), 16 );
+  			
+  			return( true );
+  			
+  		}catch( Throwable e ){
+  			
+  			return( false );
+  			
+  		}finally{
+  			
+  			if ( ssc != null ){
+  	
+  				try{
+  					ssc.close();
+  					
+  				}catch( Throwable e ){
+  					
+  					Debug.out( e );
+  				}
+  			}
+  		}
+  	}
+	
+	public InetAddress 
+	guessRoutableBindAddress() 
+	{
+		try{
+				// see if we have a choice
+			
+			List	local_addresses 	= new ArrayList();
+			List	non_local_addresses = new ArrayList();
+			
+			try{
+				NetworkAdminNetworkInterface[] interfaces = getInterfaces();
 				
-				if ( ia.isLoopbackAddress()){
+				List possible = new ArrayList();
+				
+				for (int i=0;i<interfaces.length;i++){
 					
-					continue;
+					NetworkAdminNetworkInterface intf = interfaces[i];
+					
+					NetworkAdminNetworkInterfaceAddress[] addresses = intf.getAddresses();
+					
+					for (int j=0;j<addresses.length;j++){
+						
+						NetworkAdminNetworkInterfaceAddress address = addresses[j];
+						
+						InetAddress ia = address.getAddress();
+						
+						if ( ia.isLoopbackAddress()){
+							
+							continue;
+						}
+						
+						if ( ia.isLinkLocalAddress() ||	ia.isSiteLocalAddress()){
+							
+							local_addresses.add( ia );
+							
+						}else{
+							
+							non_local_addresses.add( ia );
+						}
+						
+						if ( 	( hasIPV4Potential() && ia instanceof Inet4Address ) ||
+								( hasIPV6Potential() && ia instanceof Inet6Address )){
+							
+							possible.add( ia );
+						}
+					}
 				}
 				
-				if ( ia instanceof Inet6Address ){
+				if ( possible.size() == 1 ){
 					
-					Inet6Address v6 = (Inet6Address)ia;
+					return((InetAddress)possible.get(0));
+				}
+			}catch( Throwable e ){				
+			}
+			
+				// if we have a socks server then let's use a compatible address for it
+			
+			try{
+				NetworkAdminSocksProxy[] socks = getSocksProxies();
+				
+				if ( socks.length > 0 ){
 					
-					if ( !v6.isLinkLocalAddress()){
+					return( mapAddressToBindIP( InetAddress.getByName( socks[0].getHost())));
+				}
+			}catch( Throwable e ){				
+			}
+			
+				// next, same for nat devices
+			
+			try{
+				NetworkAdminNATDevice[] nat = getNATDevices(AzureusCoreFactory.getSingleton());
+				
+				if ( nat.length > 0 ){
+					
+					return( mapAddressToBindIP( nat[0].getAddress()));
+				}
+			}catch( Throwable e ){				
+			}
+			
+			try{
+				final AESemaphore 	sem 		= new AESemaphore( "NA:conTest" );
+				final InetAddress[]	can_connect = { null };
+				
+				final int	timeout = 10*1000;
+				
+				for (int i=0;i<local_addresses.size();i++){
+					
+					final InetAddress address = (InetAddress)local_addresses.get(i);
+					
+					new AEThread2( "NA:conTest", true )
+					{
+						public void
+						run()
+						{
+							if ( canConnectWithBind( address, timeout )){
+								
+								can_connect[0] = address;
+								
+								sem.release();
+							}
+						}
+					}.start();
+				}
+				
+				if ( sem.reserve( timeout )){
+					
+					return( can_connect[0] );
+				}
+			}catch( Throwable e ){
+				
+			}
+			
+				// take a chance on any non local addresses we have
+			
+			if ( non_local_addresses.size() > 0 ){
+				
+				return( guessAddress( non_local_addresses ));
+			}
+			
+				// lastly, select local one at random
+			
+			if ( local_addresses.size() > 0 ){
+				
+				return( guessAddress( local_addresses ));
+			}
+			
+				// ho hum
+			
+			return( null );
+			
+		}catch( Throwable e ){
+			
+			Debug.printStackTrace(e);
+			
+			return( null );
+		}
+	}
 	
-						return( true );
+	protected boolean
+	canConnectWithBind(
+		InetAddress	bind_address,
+		int			timeout )
+	{
+		Socket	socket = null;
+	
+		try{
+			socket = new Socket();
+
+			socket.bind( new InetSocketAddress( bind_address, 0 ));
+
+			socket.setSoTimeout( timeout );
+
+			socket.connect( new InetSocketAddress( "www.google.com", 80 ), timeout );
+			
+			return( true );
+			
+		}catch( Throwable e ){
+			
+			return( false );
+			
+		}finally{
+			
+			if ( socket != null ){
+				
+				try{
+					socket.close();
+					
+				}catch( Throwable f ){
+				}
+			}
+		}
+	}
+	
+	protected InetAddress
+	mapAddressToBindIP(
+		InetAddress	address )
+	{
+		boolean[]	address_bits = bytesToBits( address.getAddress());
+
+		NetworkAdminNetworkInterface[] interfaces = getInterfaces();
+		
+		InetAddress	best_bind_address	= null;
+		int			best_prefix			= 0;
+		
+		for (int i=0;i<interfaces.length;i++){
+			
+			NetworkAdminNetworkInterface intf = interfaces[i];
+			
+			NetworkAdminNetworkInterfaceAddress[] addresses = intf.getAddresses();
+			
+			for (int j=0;j<addresses.length;j++){
+				
+				NetworkAdminNetworkInterfaceAddress bind_address = addresses[j];
+				
+				InetAddress ba = bind_address.getAddress();
+
+				byte[]	bind_bytes = ba.getAddress();
+		
+				if ( address_bits.length == bind_bytes.length ){
+			
+					boolean[]	bind_bits = bytesToBits( bind_bytes );
+
+					for (int k=0;k<bind_bits.length;k++){
+				
+						if ( address_bits[k] != bind_bits[k] ){
+					
+							break;
+						}
+				
+						if ( k > best_prefix ){
+					
+							best_prefix	= k;
+					
+							best_bind_address	= ba;
+						}
 					}
 				}
 			}
 		}
 		
-		return( false );			
+		return( best_bind_address );
+	}
+		
+	protected boolean[]
+  	bytesToBits(
+  		byte[]	bytes )
+  	{
+  		boolean[]	res = new boolean[bytes.length*8];
+  		
+  		for (int i=0;i<bytes.length;i++){
+  			
+  			byte	b = bytes[i];
+  			
+  			for (int j=0;j<8;j++){
+  				
+  				res[i*8+j] = (b&(byte)(0x01<<(7-j))) != 0;
+  			}
+  		}
+  				
+  		return( res );
+  	}
+	
+	protected InetAddress
+	guessAddress(
+		List	addresses )
+	{
+			// prioritise 192.168.0.* and 192.168.1.* as common
+			// then ipv4 over ipv6
+		
+		for (int i=0;i<addresses.size();i++){
+			
+			InetAddress address = (InetAddress)addresses.get(i);
+			
+			String str = address.getHostAddress();
+			
+			if ( str.startsWith( "192.168.0." ) || str.startsWith( "192.168.1." )){
+				
+				return( address );
+			}
+		}
+		
+		for (int i=0;i<addresses.size();i++){
+			
+			InetAddress address = (InetAddress)addresses.get(i);
+			
+			if ( address instanceof Inet4Address ){
+				
+				return( address );
+			}
+		}
+		
+		for (int i=0;i<addresses.size();i++){
+			
+			InetAddress address = (InetAddress)addresses.get(i);
+			
+			if ( address instanceof Inet6Address ){
+				
+				return( address );
+			}
+		}
+		
+		if ( addresses.size() > 0 ){
+			
+			return((InetAddress)addresses.get(0));
+		}
+		
+		return( null );
+	}
+	
+	public InetAddress
+	getDefaultPublicAddress()
+	{
+		Utilities utils = PluginInitializer.getDefaultInterface().getUtilities();
+		
+		InetAddress address = utils.getPublicAddress();
+		
+		if ( address != null ){
+			
+			return( address );
+		}
+		
+		return( utils.getPublicAddress( true ));
 	}
 	
 	protected void
@@ -531,10 +940,9 @@ NetworkAdminImpl
 	}
 
 	public NetworkAdminProtocol[]
- 	getOutboundProtocols()
+ 	getOutboundProtocols(
+ 			AzureusCore azureus_core)
 	{
-		AzureusCore azureus_core = AzureusCoreFactory.getSingleton();
-				
 		NetworkAdminProtocol[]	res = 
 			{
 				new NetworkAdminProtocolImpl( azureus_core, NetworkAdminProtocol.PT_HTTP ),
@@ -546,10 +954,9 @@ NetworkAdminImpl
 	}
  	
  	public NetworkAdminProtocol[]
- 	getInboundProtocols()
+ 	getInboundProtocols(
+ 			AzureusCore azureus_core)
  	{
-		AzureusCore azureus_core = AzureusCoreFactory.getSingleton();
-
 		List	protocols = new ArrayList();
 		
 		TCPNetworkManager	tcp_manager = TCPNetworkManager.getSingleton();
@@ -669,13 +1076,14 @@ NetworkAdminImpl
 	}
 	
 	public NetworkAdminNATDevice[]
-	getNATDevices()
+	getNATDevices(
+			AzureusCore azureus_core )
 	{
-		List	devices = new ArrayList();
+		List<NetworkAdminNATDeviceImpl>	devices = new ArrayList<NetworkAdminNATDeviceImpl>();
 		
 		try{
 	
-		    PluginInterface upnp_pi = AzureusCoreFactory.getSingleton().getPluginManager().getPluginInterfaceByClass( UPnPPlugin.class );
+		    PluginInterface upnp_pi = azureus_core.getPluginManager().getPluginInterfaceByClass( UPnPPlugin.class );
 		    
 		    if ( upnp_pi != null ){
 	    	
@@ -683,9 +1091,26 @@ NetworkAdminImpl
 		    	
 		    	UPnPPluginService[]	services = upnp.getServices();
 		    	
-		    	for (int i=0;i<services.length;i++){
+		    	for ( UPnPPluginService service: services ){
 		    		
-		    		devices.add( new NetworkAdminNATDeviceImpl( services[i] ));
+		    		NetworkAdminNATDeviceImpl dev = new NetworkAdminNATDeviceImpl( service );
+		    		
+		    		boolean same = false;
+		    		
+		    		for ( NetworkAdminNATDeviceImpl d: devices ){
+		    			
+		    			if ( d.sameAs( dev )){
+		    				
+		    				same = true;
+		    				
+		    				break;
+		    			}
+		    		}
+		    		
+		    		if ( !same ){
+		    		
+		    			devices.add( dev );
+		    		}
 		    	}
 		    }
 		}catch( Throwable e ){
@@ -869,6 +1294,72 @@ NetworkAdminImpl
 		return( result );
 	}
 		
+	public void
+	lookupASN(
+		final InetAddress					address,
+		final NetworkAdminASNListener		listener )
+	{
+		synchronized( async_asn_history ){
+			
+			NetworkAdminASN existing = async_asn_history.get( address );
+			
+			if ( existing != null ){
+				
+				listener.success( existing );
+			}
+		}
+		
+		int	queue_size = async_asn_dispacher.getQueueSize();
+				
+		if ( queue_size >= MAX_ASYNC_ASN_LOOKUPS ){
+			
+			listener.failed( new NetworkAdminException( "Too many outstanding lookups" ));
+			
+		}else{
+			
+			async_asn_dispacher.dispatch(
+				new AERunnable()
+				{
+					public void
+					runSupport()
+					{
+						synchronized( async_asn_history ){
+							
+							NetworkAdminASN existing = async_asn_history.get( address );
+							
+							if ( existing != null ){
+								
+								listener.success( existing );
+								
+								return;
+							}
+						}
+						
+						try{				
+							NetworkAdminASNLookupImpl lookup = new NetworkAdminASNLookupImpl( address );
+					
+							NetworkAdminASNImpl result = lookup.lookup();
+
+							synchronized( async_asn_history ){
+
+								async_asn_history.put( address, result );
+							}
+							
+							listener.success( result );
+							
+						}catch( NetworkAdminException e ){
+							
+							listener.failed( e );
+							
+						}catch( Throwable e ){
+							
+							listener.failed( new NetworkAdminException( "lookup failed", e ));
+						}
+					}
+				});
+		}
+	}
+	
 	protected void
 	addToASHistory(
 		NetworkAdminASN	asn )
@@ -922,9 +1413,10 @@ NetworkAdminImpl
 	}
 	
 	public void
-	runInitialChecks()
+	runInitialChecks(
+			AzureusCore azureus_core)
 	{
-		AZInstanceManager i_man = AzureusCoreFactory.getSingleton().getInstanceManager();
+		AZInstanceManager i_man = azureus_core.getInstanceManager();
 		
 		final AZInstance	my_instance = i_man.getMyInstance();
 		
@@ -1449,7 +1941,45 @@ NetworkAdminImpl
 		
 		try{
 			writer.indent();
-			
+		
+			try{
+				writer.println( "Binding Details" );
+				
+				writer.indent();
+				
+				writer.println( "bind to: " + getString( getAllBindAddresses( false )));
+				
+				writer.println( "bindable: " + getString( getBindableAddresses()));
+				
+				writer.println( "ipv4_potential=" + hasIPV4Potential());
+				writer.println( "ipv6_potential=" + hasIPV6Potential(false) + "/" + hasIPV6Potential(true));
+	
+				try{
+					writer.println( "single homed: " + getSingleHomedServiceBindAddress());
+				}catch( Throwable e ){
+					writer.println( "single homed: none" );
+				}
+				
+				try{
+					writer.println( "single homed (4): " + getSingleHomedServiceBindAddress( IP_PROTOCOL_VERSION_REQUIRE_V4 ));
+				}catch( Throwable e ){
+					writer.println( "single homed (4): none" );
+				}
+				
+				try{
+					writer.println( "single homed (6): " + getSingleHomedServiceBindAddress( IP_PROTOCOL_VERSION_REQUIRE_V6 ));
+				}catch( Throwable e ){
+					writer.println( "single homed (6): none" );
+				}
+				
+				writer.println( "multi homed, nio=false: " + getString( getMultiHomedServiceBindAddresses( false )));
+				writer.println( "multi homed, nio=true:  " + getString( getMultiHomedServiceBindAddresses( true )));
+					
+			}finally{
+				
+				writer.exdent();
+			}		
+				
 			NetworkAdminHTTPProxy	proxy = getHTTPProxy();
 			
 			if ( proxy == null ){
@@ -1507,17 +2037,22 @@ NetworkAdminImpl
 				}
 			}
 			
-			NetworkAdminNATDevice[]	nat_devices = getNATDevices();
-			
-			writer.println( "NAT Devices: " + nat_devices.length );
-			
-			for (int i=0;i<nat_devices.length;i++){
+			try {
+				NetworkAdminNATDevice[]	nat_devices = getNATDevices(AzureusCoreFactory.getSingleton());
+
+				writer.println( "NAT Devices: " + nat_devices.length );
+
+				for (int i=0;i<nat_devices.length;i++){
+
+					NetworkAdminNATDevice	device = nat_devices[i];
+
+					writer.println( "    " + device.getName() + ",address=" + device.getAddress().getHostAddress() + ":" + device.getPort() + ",ext=" + device.getExternalAddress());
+				}
+			}catch (Exception e){
 				
-				NetworkAdminNATDevice	device = nat_devices[i];
-				
-				writer.println( "    " + device.getName() + ",address=" + device.getAddress().getHostAddress() + ":" + device.getPort() + ",ext=" + device.getExternalAddress());
+				writer.println( "Nat Devices: Can't get -> " + e.toString());
 			}
-			
+  			
 			writer.println( "Interfaces" );
 			
 			writer.println( "   " + getNetworkInterfacesAsString());
@@ -1528,6 +2063,20 @@ NetworkAdminImpl
 		}
 	}
 	
+	private String
+	getString(
+		InetAddress[]	addresses )
+	{
+		String	str = "";
+		
+		for ( InetAddress address: addresses ){
+			
+			str += (str.length()==0?"":", ") + address.getHostAddress();
+		}
+		
+		return( str );
+	}
+	
 	public void 
 	generateDiagnostics(
 		final IndentWriter iw )
@@ -1591,19 +2140,23 @@ NetworkAdminImpl
 			}
 		}
 		
-		NetworkAdminNATDevice[]	nat_devices = getNATDevices();
-		
-		iw.println( "NAT Devices: " + nat_devices.length );
-		
-		for (int i=0;i<nat_devices.length;i++){
-			
-			NetworkAdminNATDevice	device = nat_devices[i];
-			
-			iw.println( "    " + device.getName() + ",address=" + device.getAddress().getHostAddress() + ":" + device.getPort() + ",ext=" + device.getExternalAddress());
-			
-			public_addresses.add( device.getExternalAddress());
+		try {
+			NetworkAdminNATDevice[]	nat_devices = getNATDevices(AzureusCoreFactory.getSingleton());
+
+			iw.println( "NAT Devices: " + nat_devices.length );
+
+			for (int i=0;i<nat_devices.length;i++){
+
+				NetworkAdminNATDevice	device = nat_devices[i];
+
+				iw.println( "    " + device.getName() + ",address=" + device.getAddress().getHostAddress() + ":" + device.getPort() + ",ext=" + device.getExternalAddress());
+
+				public_addresses.add( device.getExternalAddress());
+			}
+		} catch (Exception e) {
+			iw.println( "Nat Devices: Can't get -> " + e.toString());
 		}
-		
+ 		
 		iw.println( "Interfaces" );
 		
 		NetworkAdminNetworkInterface[] interfaces = getInterfaces();
@@ -1695,51 +2248,56 @@ NetworkAdminImpl
 		
 		iw.println( "Inbound protocols: default routing" );
 		
-		NetworkAdminProtocol[]	protocols = getInboundProtocols();
-		
-		for (int i=0;i<protocols.length;i++){
-			
-			NetworkAdminProtocol	protocol = protocols[i];
-			
-			try{
-				InetAddress	ext_addr = testProtocol( protocol );
-	
-				if ( ext_addr != null ){
-					
-					public_addresses.add( ext_addr );
-				}
-	
-				iw.println( "    " + protocol.getName() + " - " + ext_addr );
-				
-			}catch( NetworkAdminException e ){
-				
-				iw.println( "    " + protocol.getName() + " - " + Debug.getNestedExceptionMessage(e));
-			}
-		}
-		
-		iw.println( "Outbound protocols: default routing" );
-		
-		protocols = getOutboundProtocols();
-		
-		for (int i=0;i<protocols.length;i++){
-			
-			NetworkAdminProtocol	protocol = protocols[i];
-			
-			try{
 
-				InetAddress	ext_addr = testProtocol( protocol );
-				
-				if ( ext_addr != null ){
-				
-					public_addresses.add( ext_addr );
-				}
-				
-				iw.println( "    " + protocol.getName() + " - " + ext_addr );
-				
-			}catch( NetworkAdminException e ){
-				
-				iw.println( "    " + protocol.getName() + " - " + Debug.getNestedExceptionMessage(e));
-			}
+		if (AzureusCoreFactory.isCoreRunning()) {
+			AzureusCore azureus_core = AzureusCoreFactory.getSingleton();
+
+			NetworkAdminProtocol[]	protocols = getInboundProtocols(azureus_core);
+		
+  		for (int i=0;i<protocols.length;i++){
+  			
+  			NetworkAdminProtocol	protocol = protocols[i];
+  			
+  			try{
+  				InetAddress	ext_addr = testProtocol( protocol );
+  	
+  				if ( ext_addr != null ){
+  					
+  					public_addresses.add( ext_addr );
+  				}
+  	
+  				iw.println( "    " + protocol.getName() + " - " + ext_addr );
+  				
+  			}catch( NetworkAdminException e ){
+  				
+  				iw.println( "    " + protocol.getName() + " - " + Debug.getNestedExceptionMessage(e));
+  			}
+  		}
+  		
+  		iw.println( "Outbound protocols: default routing" );
+  		
+  		protocols = getOutboundProtocols(azureus_core);
+  		
+  		for (int i=0;i<protocols.length;i++){
+  			
+  			NetworkAdminProtocol	protocol = protocols[i];
+  			
+  			try{
+  
+  				InetAddress	ext_addr = testProtocol( protocol );
+  				
+  				if ( ext_addr != null ){
+  				
+  					public_addresses.add( ext_addr );
+  				}
+  				
+  				iw.println( "    " + protocol.getName() + " - " + ext_addr );
+  				
+  			}catch( NetworkAdminException e ){
+  				
+  				iw.println( "    " + protocol.getName() + " - " + Debug.getNestedExceptionMessage(e));
+  			}
+  		}
 		}
 		
 		Iterator	it = public_addresses.iterator();
@@ -1945,7 +2503,9 @@ NetworkAdminImpl
 												
 						iw.println( "Outbound protocols: bound" );
 						
-						NetworkAdminProtocol[]	protocols = getOutboundProtocols();
+						AzureusCore azureus_core = AzureusCoreFactory.getSingleton();
+						
+						NetworkAdminProtocol[]	protocols = getOutboundProtocols(azureus_core);
 						
 						for (int i=0;i<protocols.length;i++){
 							
@@ -1969,7 +2529,7 @@ NetworkAdminImpl
 						
 						iw.println( "Inbound protocols: bound" );
 						
-						protocols = getInboundProtocols();
+						protocols = getInboundProtocols(azureus_core);
 						
 						for (int i=0;i<protocols.length;i++){
 							
@@ -2083,7 +2643,9 @@ NetworkAdminImpl
 	logNATStatus(
 		IndentWriter		iw )
 	{
-		generateDiagnostics( iw, getInboundProtocols());
+		if (AzureusCoreFactory.isCoreRunning()) {
+			generateDiagnostics( iw, getInboundProtocols(AzureusCoreFactory.getSingleton()));
+		}
 	}
 	
 	public static void
