@@ -36,6 +36,7 @@ import org.gudy.azureus2.core3.util.*;
 import com.aelitis.azureus.core.peermanager.control.PeerControlScheduler;
 import com.aelitis.azureus.core.peermanager.control.PeerControlSchedulerFactory;
 import com.aelitis.azureus.core.peermanager.control.SpeedTokenDispenser;
+import com.aelitis.azureus.core.peermanager.peerdb.PeerItem;
 import com.aelitis.azureus.core.peermanager.piecepicker.*;
 import com.aelitis.azureus.core.peermanager.piecepicker.util.BitFlags;
 import com.aelitis.azureus.core.util.CopyOnWriteList;
@@ -1162,7 +1163,7 @@ implements PiecePicker
 		return rarestOverride;
 	}
 	
-	private final SpeedTokenDispenser dispenser = PeerControlSchedulerFactory.getSingleton().getSpeedTokenDispenser();
+	private final SpeedTokenDispenser dispenser = PeerControlSchedulerFactory.getSingleton(0).getSpeedTokenDispenser();
 
 	/**
 	 * @param pt the PEPeerTransport we're working on
@@ -2528,6 +2529,17 @@ implements PiecePicker
 			peer.removeListener(peerListener);
 		}
 
+		public void peerDiscovered(PEPeerManager manager, PeerItem peer, PEPeer finder) {
+		}
+		
+		public void 
+		peerSentBadData(
+			PEPeerManager 	manager,
+			PEPeer 			peer, 
+			int 			pieceNumber) 
+		{							
+		}
+		
 		public void
 		destroyed()
 		{		
