@@ -127,6 +127,8 @@ public class UnchokerUtilTest {
         public void setSnubbed(boolean b){}
         public boolean isChokingMe(){ return true;  }
         public boolean isChokedByMe() {  return true;  }
+        public void sendChoke(){}
+        public void sendUnChoke(){}
         public boolean isInteresting(){  return true;  }
         public boolean isInterested(){  return true;  }
         public boolean isRelativeSeed() { return false; }
@@ -222,6 +224,14 @@ public class UnchokerUtilTest {
 			// TODO Auto-generated method stub
 			
 		}
+		public boolean isPriorityConnection() {
+			// TODO Auto-generated method stub
+			return false;
+		}
+		public void setPriorityConnection(boolean is_priority) {
+			// TODO Auto-generated method stub
+			
+		}
       };
       
      f_stats[0] = new PEPeerStats() {
@@ -264,6 +274,15 @@ public class UnchokerUtilTest {
     return peers;
   }
   
-  
-  
+  public static class
+  UF
+  	extends UnchokerFactory
+  {
+	public Unchoker 
+	getUnchoker(
+		boolean seeding) 
+	{
+		return super.getUnchoker(seeding);
+	}  
+  } 
 }
