@@ -238,12 +238,15 @@ public class BTPeerIDByteDecoderDefinitions {
 		addAzStyle("AT", "Artemis");
 		addAzStyle("BB", "BitBuddy", "1.234");
 		addAzStyle("BC", "BitComet", VER_AZ_SKIP_FIRST_ONE_MAJ_TWO_MIN);
+		addAzStyle("BE", "BitTorrent SDK");
 		addAzStyle("BF", "BitFlu", NO_VERSION);
 		addAzStyle("BG", "BTG", VER_AZ_FOUR_DIGITS);
 		addAzStyle("bk", "BitKitten (libtorrent)");
 		addAzStyle("BR", "BitRocket", "1.2(34)");
 		addAzStyle("BS", "BTSlave");
+		addAzStyle("BW", "BitWombat");
 		addAzStyle("BX", "BittorrentX");
+		addAzStyle("CB", "Shareaza Plus");
 		addAzStyle("CD", "Enhanced CTorrent", VER_AZ_TWO_MAJ_TWO_MIN);      
 		addAzStyle("CT", "CTorrent", "1.2.34");
 		addAzStyle("DP", "Propogate Data Client");
@@ -258,6 +261,7 @@ public class BTPeerIDByteDecoderDefinitions {
 		addAzStyle("HL", "Halite", VER_AZ_THREE_DIGITS);
 		addAzStyle("HN", "Hydranode");
 		addAzStyle("KT", "KTorrent", VER_AZ_KTORRENT_STYLE);
+		addAzStyle("LC", "LeechCraft");
 		addAzStyle("LH", "LH-ABC");
 		addAzStyle("LK", "linkage", VER_AZ_THREE_DIGITS);
 		addAzStyle("LP", "Lphant", VER_AZ_TWO_MAJ_TWO_MIN);
@@ -271,8 +275,10 @@ public class BTPeerIDByteDecoderDefinitions {
 		addAzStyle("NE", "BT Next Evolution", VER_AZ_THREE_DIGITS);
 		addAzStyle("NX", "Net Transport");
 		addAzStyle("OT", "OmegaTorrent");
-		addAzStyle("PC", PeerClassifier.CACHE_LOGIC);
+		addAzStyle("PC", PeerClassifier.CACHE_LOGIC, "12.3-4" );
 		addAzStyle("PD", "Pando");
+		addAzStyle("PE", "PeerProject");
+		addAzStyle("pX", "pHoeniX");
 		addAzStyle("qB", "qBittorrent", VER_AZ_THREE_DIGITS);
 		addAzStyle("QD", "qqdownload");
 		addAzStyle("RT", "Retriever");
@@ -289,10 +295,11 @@ public class BTPeerIDByteDecoderDefinitions {
 		addAzStyle("TT", "TuoTu", VER_AZ_THREE_DIGITS);
 		addAzStyle("UL", "uLeecher!");
 		addAzStyle("UT", "\u00B5Torrent", VER_AZ_THREE_DIGITS_PLUS_MNEMONIC);
+		addAzStyle("UM", "\u00B5Torrent Mac", VER_AZ_THREE_DIGITS_PLUS_MNEMONIC);
 		addAzStyle("WT", "Bitlet");
 		addAzStyle("WY", "Wyzo");
 		addAzStyle("VG", "\u54c7\u560E (Vagaa)", VER_AZ_FOUR_DIGITS);
-		addAzStyle("XL", "\u8FC5\u96F7\u5728\u7EBF (Xunlei)");
+		addAzStyle("XL", "\u8FC5\u96F7\u5728\u7EBF (Xunlei)"); // Apparently, the English name of the client is "Thunderbolt".
 		addAzStyle("XT", "XanTorrent");
 		addAzStyle("XX", "XTorrent", "v1234");
 		addAzStyle("ZT", "ZipTorrent"); 
@@ -358,10 +365,15 @@ public class BTPeerIDByteDecoderDefinitions {
 		// DNA01000 becomes version 1.0 - we'll ignore the other digits for now.
 		client = addSimpleClient("BitTorrent DNA", "DNA");
 		addVersionedClient(client, VER_BYTE_BLOCK_DOTTED_CHAR, 2, 4);
-		
+
+		// Pre build 10000 versions.
 		client = addSimpleClient("Opera", "OP");
 		addVersionedClient(client, VER_BLOCK, 4, " (Build %s)");
-		
+
+		// Post build 10000 versions.
+		client = addSimpleClient("Opera", "O");
+		addVersionedClient(client, VER_BLOCK, 5, " (Build %s)");
+
 		client = addSimpleClient("Burst!", "Mbrst");
 		addVersionedClient(client, VER_DOTTED_BLOCK, 5); // 3 version components, 5 chars which describe it
 		
@@ -402,6 +414,15 @@ public class BTPeerIDByteDecoderDefinitions {
 		client = addSimpleClient("BitMagnet", "BM", 2);
 		addVersionedClient(client, VER_TWO_BYTE_THREE_PART, 2, 0);
 
+		client = addSimpleClient("QVOD", "QVOD");
+		addVersionedClient(client, VER_BLOCK, 4, " (Build %s)");
+		
+		// Top-BT - based on BitTornado, but doesn't quite stick
+		// to Shadow's naming conventions - so we'll use substring
+		// matching instead.
+		client = addSimpleClient("Top-BT", "TB");
+		addVersionedClient(client, VER_BYTE_BLOCK_DOTTED_CHAR, 3);
+		
 	}
 	
 	static class ClientData {
