@@ -36,6 +36,7 @@ import org.gudy.azureus2.pluginsimpl.local.torrent.TorrentImpl;
 
 import com.aelitis.azureus.core.AzureusCore;
 import com.aelitis.azureus.core.AzureusCoreFactory;
+import com.aelitis.azureus.core.cnetwork.ContentNetwork;
 import com.aelitis.azureus.core.messenger.PlatformMessenger;
 import com.aelitis.azureus.core.messenger.config.PlatformRatingMessenger;
 import com.aelitis.azureus.core.messenger.config.PlatformTorrentMessenger;
@@ -58,6 +59,8 @@ public class PlatformTorrentUtils
 
 	private static final long MAX_MD_REFRESH_MS = 1000L * 60 * 60 * 24 * 30;
 
+	public static final String AELITIS_HOST_CORE	= "aelitis.com";			// needs to be lowercase
+
 	public static final boolean DEBUG_CACHING = System.getProperty(
 			"az3.debug.caching", "0").equals("1");
 
@@ -69,6 +72,8 @@ public class PlatformTorrentUtils
 
 	private static final String TOR_AZ_PROP_DESCRIPTION = "Description";
 
+	private static final String TOR_AZ_PROP_CONTENT_TYPE = "Content Type";
+
 	private static final String TOR_AZ_PROP_AUTHOR = "Author";
 
 	private static final String TOR_AZ_PROP_PUBLISHER = "Publisher";
@@ -77,6 +82,8 @@ public class PlatformTorrentUtils
 
 	private static final String TOR_AZ_PROP_THUMBNAIL = "Thumbnail";
 
+	private static final String TOR_AZ_PROP_THUMBNAIL_URL = "Thumbnail.url";
+
 	private static final String TOR_AZ_PROP_QUALITY = "Quality";
 
 	private static final String TOR_AZ_PROP_USER_RATING = "UserRating";
@@ -99,6 +106,8 @@ public class PlatformTorrentUtils
 
 	private static final String TOR_AZ_PROP_QOS_CLASS = "QOS Class";
 
+	private static final String TOR_AZ_PROP_CONTENT_NETWORK = "Content Network";
+		
 	private static final String TOR_AZ_PROP_AD_ID = "Ad ID";
 
 	private static final String TOR_AZ_PROP_AD_ENABLED = "Ad Enabled";
@@ -107,7 +116,9 @@ public class PlatformTorrentUtils
 	
 	private static final String TOR_AZ_PROP_PRIMARY_FILE = "Primary File Index";
 
-	private static final ArrayList metaDataListeners = new ArrayList();
+	private static final ArrayList metaDataListeners = new ArrayList(1);
+
+	private static final ArrayList hasBeenOpenedListeners = new ArrayList(1);
 
 	private static final String TOR_AZ_PROP_USE_EMP = "useEMP";
 	
@@ -119,6 +130,10 @@ public class PlatformTorrentUtils
 
 	private static final String TOR_AZ_PROP_VIDEO_HEIGHT = "Video Height";
 
+	private static final String TOR_AZ_PROP_VIDEO_RUNNINGTIME = "Running Time";
+
+	private static final String TOR_AZ_PROP_OPENED = "Opened";
+
 	private static ArrayList listPlatformHosts = null;
 
 	private static final Map mapPlatformTrackerTorrents = new WeakHashMap();
@@ -256,7 +271,7 @@ public class PlatformTorrentUtils
 		mapContent.put(key, new Long(value));
 	}
 
-	private static void setContentMapMap(TOTorrent torrent, String key,
+	public static void setContentMapMap(TOTorrent torrent, String key,
 			Map value) {
 		if (torrent == null) {
 			return;
@@ -282,6 +297,10 @@ public class PlatformTorrentUtils
 		return getContentMapString(torrent, TOR_AZ_PROP_DESCRIPTION);
 	}
 
+	public static String getContentType(TOTorrent torrent) {
+		return getContentMapString(torrent, TOR_AZ_PROP_CONTENT_TYPE);
+	}
+
 	public static String getContentAuthor(TOTorrent torrent) {
 		return getContentMapString(torrent, TOR_AZ_PROP_AUTHOR);
 	}
@@ -315,6 +334,24 @@ public class PlatformTorrentUtils
 		setContentMapLong(torrent, TOR_AZ_PROP_QOS_CLASS, cla);
 	}
 
+	public static long getContentNetworkID(TOTorrent torrent){
+		return( getContentNetworkID( torrent, ContentNetwork.CONTENT_NETWORK_UNKNOWN ));
+	}
+	
+	public static long getContentNetworkID(TOTorrent torrent, long def) {
+		long id = getContentMapLong(torrent, TOR_AZ_PROP_CONTENT_NETWORK,
+				ContentNetwork.CONTENT_NETWORK_UNKNOWN);
+		if (id == ContentNetwork.CONTENT_NETWORK_UNKNOWN) {
+			return isContent(torrent, false) ? ContentNetwork.CONTENT_NETWORK_VUZE
+					: def;
+		}
+		return id;
+	}
+
+	public static void setContentNetworkID(TOTorrent torrent, long cnet) {
+		setContentMapLong(torrent, TOR_AZ_PROP_CONTENT_NETWORK, cnet);
+	}
+	
 	private static void putOrRemove(Map map, String key, Object obj) {
 		if (obj == null || obj.equals(null)) {
 			map.remove(key);
@@ -331,6 +368,9 @@ public class PlatformTorrentUtils
 	}
 
 	private static void writeTorrentIfExists(TOTorrent torrent) {
+		if (!AzureusCoreFactory.isCoreRunning()) {
+			return;
+		}
 		AzureusCore core = AzureusCoreFactory.getSingleton();
 		if (core == null || !core.isStarted()) {
 			return;
@@ -359,6 +399,10 @@ public class PlatformTorrentUtils
 		return null;
 	}
 
+	public static String getContentThumbnailUrl(TOTorrent torrent) {
+		return getContentMapString(torrent, TOR_AZ_PROP_THUMBNAIL_URL);
+	}
+
 	public static void setContentThumbnail(TOTorrent torrent, byte[] thumbnail) {
 		Map mapContent = getContentMap(torrent);
 		putOrRemove(mapContent, TOR_AZ_PROP_THUMBNAIL, thumbnail);
@@ -554,6 +598,47 @@ public class PlatformTorrentUtils
 		}
 		return false;
 	}
+	
+	public static boolean isUpdateDM(DownloadManager dm) {
+		Boolean oisUpdate = (Boolean) dm.getUserData("isUpdate");
+		if (oisUpdate != null) {
+			return oisUpdate.booleanValue();
+		}
+
+		boolean isUpdate = true;
+		TOTorrent torrent = dm.getTorrent();
+		if (torrent == null) {
+			isUpdate = false;
+		} else {
+			URL announceURL = torrent.getAnnounceURL();
+
+			if (announceURL != null) {
+				if (announceURL.getHost().indexOf(AELITIS_HOST_CORE) == -1) {
+					isUpdate = false;
+				}
+			}
+			
+			if (isUpdate) {
+				TOTorrentAnnounceURLSet[] sets = torrent.getAnnounceURLGroup().getAnnounceURLSets();
+
+				for (int i = 0; i < sets.length; i++) {
+
+					URL[] urls = sets[i].getAnnounceURLs();
+
+					for (int j = 0; j < urls.length; j++) {
+
+						if (urls[j].getHost().indexOf(AELITIS_HOST_CORE) == -1) {
+							isUpdate = false;
+							break;
+						}
+					}
+				}
+			}
+		}
+
+		dm.setUserData("isUpdate", new Boolean(isUpdate));
+		return isUpdate;
+	}
 
 	public static String getAdId(TOTorrent torrent) {
 		return getContentMapString(torrent, TOR_AZ_PROP_AD_ID);
@@ -570,7 +655,7 @@ public class PlatformTorrentUtils
 	/**
 	 * @param torrent
 	 * @param maxDelayMS TODO
-	 */
+	 *
 	public static void updateMetaData(final TOTorrent[] torrents, long maxDelayMS) {
 		if (torrents == null) {
 			log("no torrents to update MD on");
@@ -593,7 +678,7 @@ public class PlatformTorrentUtils
 			}
 		});
 	}
-
+*/
 	/**
 	 * @param torrent
 	 * @param maxDelayMS TODO
@@ -607,17 +692,18 @@ public class PlatformTorrentUtils
 
 		log(torrent, "updateMD");
 
-		PlatformTorrentMessenger.getMetaData(new TOTorrent[] {
-			torrent
-		}, maxDelayMS, new PlatformTorrentMessenger.GetMetaDataReplyListener() {
+		PlatformTorrentMessenger.getMetaData(getContentNetworkID(torrent),
+				new TOTorrent[] {
+					torrent
+				}, maxDelayMS, new PlatformTorrentMessenger.GetMetaDataReplyListener() {
 
-			public void messageSent() {
-			}
+					public void messageSent() {
+					}
 
-			public void replyReceived(String replyType, Map mapHashes) {
-				updateMetaData_handleReply(torrent, null, replyType, mapHashes);
-			}
-		});
+					public void replyReceived(String replyType, Map mapHashes) {
+						updateMetaData_handleReply(torrent, null, replyType, mapHashes);
+					}
+				});
 	}
 
 	private static void updateMetaData_handleReply(TOTorrent torrent,
@@ -642,7 +728,7 @@ public class PlatformTorrentUtils
 		if (replyType.equals(PlatformMessenger.REPLY_EXCEPTION)) {
 			if (torrent != null) {
   			// try again in a bit
-  			log(torrent, "Exception, retrying later");
+  			log(torrent, "Exception, retrying later (~" + RETRY_METADATA + "ms)");
   			SimpleTimer.addEvent("Update MD Retry", SystemTime.getCurrentTime()
   					+ RETRY_METADATA, new TimerEventPerformer() {
   				public void perform(TimerEvent event) {
@@ -653,7 +739,7 @@ public class PlatformTorrentUtils
 			}
 		} else {
 			Map jsonMapMetaData = hash == null ? null : (Map) mapHashes.get(hash);
-			if (jsonMapMetaData != null) {
+			if (jsonMapMetaData != null && !jsonMapMetaData.isEmpty()) {
 				long oldLastUpdated = getContentLastUpdated(torrent);
 				long expireyMins = 0;
 
@@ -834,13 +920,17 @@ public class PlatformTorrentUtils
 	private static long getContentVideoHeight(TOTorrent torrent) {
 		return getContentMapLong(torrent, TOR_AZ_PROP_VIDEO_HEIGHT, -1);
 	}
+
+	public static long getContentVideoRunningTime(TOTorrent torrent) {
+		return getContentMapLong(torrent, TOR_AZ_PROP_VIDEO_RUNNINGTIME, -1);
+	}
 	
 	public static int[] getContentVideoResolution(TOTorrent torrent) {
 		long width = getContentVideoWidth(torrent);
 		if (width <= 0) {
 			return null;
 		}
-		long height = getContentVideoWidth(torrent);
+		long height = getContentVideoHeight(torrent);
 		if (height <= 0) {
 			return null;
 		}
@@ -880,8 +970,8 @@ public class PlatformTorrentUtils
 
 		try {
 			PluginInterface pi = AzureusCoreFactory.getSingleton().getPluginManager().getPluginInterfaceByID(
-					"azemp");
-			if (pi != null && pi.isOperational() && !pi.isDisabled()) {
+					"azemp", true);
+			if (pi != null) {
 				embeddedPlayerAvail = true;
 			}
 		} catch (Throwable e1) {
@@ -909,4 +999,42 @@ public class PlatformTorrentUtils
 		}
 		return name;
 	}
+	
+	public static void setHasBeenOpened(DownloadManager dm, boolean opened) {
+		TOTorrent torrent = dm.getTorrent();
+		if (torrent == null) {
+			return;
+		}
+		if (opened == getHasBeenOpened(dm)) {
+			return;
+		}
+		setContentMapLong(torrent, TOR_AZ_PROP_OPENED, opened ? 1 : 0);
+		writeTorrentIfExists(torrent);
+		Object[] array = hasBeenOpenedListeners.toArray();
+		for (int i = 0; i < array.length; i++) {
+			try {
+				HasBeenOpenedListener l = (HasBeenOpenedListener) array[i];
+				l.hasBeenOpenedChanged(dm, opened);
+			} catch (Exception e) {
+				Debug.out(e);
+			}
+		}
+	}
+
+	public static boolean getHasBeenOpened(DownloadManager dm) {
+		TOTorrent torrent = dm.getTorrent();
+		if (torrent == null) {
+			return true;
+		}
+		boolean opened = getContentMapLong(torrent, TOR_AZ_PROP_OPENED, -1) > 0;
+		if (opened || getAdId(torrent) != null || isUpdateDM(dm)) {
+			return true;
+		}
+		
+		return false;
+	}
+
+	public static void addHasBeenOpenedListener(HasBeenOpenedListener l) {
+		hasBeenOpenedListeners.add(l);
+	}
 }
