@@ -132,7 +132,7 @@ public class VersionCheckClient {
   
   private long last_feature_flag_cache;
   private long last_feature_flag_cache_time;
-  
+    
   
   private 
   VersionCheckClient()
@@ -225,11 +225,11 @@ public class VersionCheckClient {
 	        }
 	        catch(SocketException t) {
 	        	// internet is broken
-	        	Debug.out(t.getClass().getName() + ": " + t.getMessage());
+	        	// Debug.out(t.getClass().getName() + ": " + t.getMessage());
 	        }
 	        catch(UnknownHostException t) {
 	        	// dns is broken
-	        	Debug.out(t.getClass().getName() + ": " + t.getMessage());
+	        	// Debug.out(t.getClass().getName() + ": " + t.getMessage());
 	        }
 	        catch( Throwable t ) {
 	        	Debug.out(t);
@@ -342,7 +342,7 @@ public class VersionCheckClient {
 	  }
   }
   
-  protected Map
+  public Map
   getMostRecentVersionCheckData()
   {
 	  	// currently we maintain v4 much more accurately than v6
@@ -733,50 +733,87 @@ public class VersionCheckClient {
 		  os.flush();
 		  
 		  InputStream	is = socket.getInputStream();
-		  
-		  ByteArrayOutputStream	baos = new ByteArrayOutputStream();
-		  		
-		  byte[]	buffer = new byte[1024];
+		  		  		
+		  byte[]	buffer = new byte[1];
 
-		  int	total_len = 0;
+		  String	header = "";
+		  
+		  int content_length = -1;
 		  
 		  while( true ){
 			  
 			  int	len = is.read( buffer );
-			  
+			  			  
 			  if ( len <= 0 ){
 				  
 				  break;
 			  }
 			  
-			  total_len += len;
-			  
-			  if ( total_len > 16000 ){
+			  header += (char)buffer[0];
+			  			  
+			  if ( header.endsWith( "\r\n\r\n" )){
 				  
-				  throw( new IOException( "reply too large" ));
+				  header = header.toLowerCase( MessageText.LOCALE_ENGLISH );
+				  
+				  int	pos = header.indexOf( "content-length:" );
+				  
+				  if ( pos == -1 ){
+					  
+					  throw( new IOException( "content length missing" ));
+				  }
+				  
+				  header = header.substring( pos+15 );
+				  
+				  pos = header.indexOf( '\r' );
+				  
+				  header = header.substring(0,pos).trim();
+				  
+				  content_length = Integer.parseInt( header );
+				  
+				  if ( content_length > 10000 ){
+					  
+					  throw( new IOException( "content length too large" ));
+				  }
+				  
+				  break;
 			  }
 			  
+			  if ( header.length() > 2048 ){
+				  
+				  throw( new IOException( "header too large" ));
+			  }
+		  }
+		  
+		  ByteArrayOutputStream	baos = new ByteArrayOutputStream( content_length );
+
+		  buffer = new byte[content_length];
+		  
+		  while( content_length > 0 ){
+			  
+			  int	len = is.read( buffer );
+  			  
+			  if ( len <= 0 ){
+				  
+				  break;
+			  }
+
 			  baos.write( buffer, 0, len );
+			  
+			  content_length -= len;
+		  }
+		  
+		  if ( content_length != 0 ){
+			  
+			  throw( new IOException( "error reading reply" ));
 		  }
 		  
 		  byte[]	reply_bytes = baos.toByteArray();
+		    		  
+		  Map reply = BDecoder.decode( new BufferedInputStream( new ByteArrayInputStream( reply_bytes )));
 		  
-		  for (int i=3;i<reply_bytes.length;i++){
-			  
-			  if ( 		reply_bytes[i-3]== (byte)'\015' &&
-					  	reply_bytes[i-2]== (byte)'\012' &&
-					  	reply_bytes[i-1]== (byte)'\015' &&
-					  	reply_bytes[i-0]== (byte)'\012' ){
-			  		  
-				  Map reply = BDecoder.decode( new BufferedInputStream( new ByteArrayInputStream( reply_bytes, i+1, reply_bytes.length - (i+1 ))));
-				  
-				  preProcessReply( reply, v6 );
+		  preProcessReply( reply, v6 );
 
-				  return( reply );
-			  }
-		  }
-		  
-		  throw( new Exception( "Invalid reply: " + new String( reply_bytes )));
+		  return( reply );
 		  
 	  }finally{
 		  
@@ -1005,32 +1042,60 @@ public class VersionCheckClient {
    * @return message to send
    */
   private static Map constructVersionCheckMessage( String reason ) {
+	  
+	  //only send if anonymous-check flag is not set
+	  
+	boolean send_info = COConfigurationManager.getBooleanParameter( "Send Version Info" );
+
     Map message = new HashMap();
-    
-    message.put( "appid", SystemProperties.getApplicationIdentifier());
+
+    //always send
+    message.put( "appid",   SystemProperties.getApplicationIdentifier());
     message.put( "version", Constants.AZUREUS_VERSION );
+    message.put( "ui",      COConfigurationManager.getStringParameter( "ui", "unknown" ) );
+    message.put( "os",      Constants.OSName );
+    message.put( "os_version", System.getProperty( "os.version" ) );
+    message.put( "os_arch", System.getProperty( "os.arch" ) );   //see http://lopica.sourceforge.net/os.html
+
+    boolean using_phe = COConfigurationManager.getBooleanParameter( "network.transport.encrypted.require" );
+    message.put( "using_phe", using_phe ? new Long(1) : new Long(0) );
+
+    //swt stuff
+    try {
+      Class c = Class.forName( "org.eclipse.swt.SWT" );
+      
+      String swt_platform = (String)c.getMethod( "getPlatform", new Class[]{} ).invoke( null, new Object[]{} );
+      message.put( "swt_platform", swt_platform );
+      
+      Integer swt_version = (Integer)c.getMethod( "getVersion", new Class[]{} ).invoke( null, new Object[]{} );
+      message.put( "swt_version", new Long( swt_version.longValue() ) );
+
+      if ( send_info ){
+	      c = Class.forName("org.gudy.azureus2.ui.swt.mainwindow.MainWindow");
+	      if (c != null) {
+	    	  c.getMethod("addToVersionCheckMessage", new Class[] { Map.class }).invoke(
+	    			  null, new Object[] { message });
+	      }   
+      }
+    }
+    catch( ClassNotFoundException e ) {  /* ignore */ }
+    catch( NoClassDefFoundError er ) {  /* ignore */ }
+    catch( InvocationTargetException err ) {  /* ignore */ }
+    catch( Throwable t ) {  t.printStackTrace();  }
+    
+  
+    int last_send_time = COConfigurationManager.getIntParameter( "Send Version Info Last Time", -1 );
+    int current_send_time = (int)(SystemTime.getCurrentTime()/1000);    
+    COConfigurationManager.setParameter( "Send Version Info Last Time", current_send_time );
+
     
     String id = COConfigurationManager.getStringParameter( "ID", null );
-    boolean send_info = COConfigurationManager.getBooleanParameter( "Send Version Info" );
-    
-    int last_send_time = COConfigurationManager.getIntParameter( "Send Version Info Last Time", -1 );
 
-    int current_send_time = (int)(SystemTime.getCurrentTime()/1000);
+    if( id != null && send_info ) {    	
+      message.put( "id", id );      
     
-    COConfigurationManager.setParameter( "Send Version Info Last Time", current_send_time );
-    
-    if( id != null && send_info ) {
-    	
-      message.put( "id", id );
-      message.put( "os", Constants.OSName );
-      
-      message.put( "os_version", System.getProperty( "os.version" ) );
-      message.put( "os_arch", System.getProperty( "os.arch" ) );   //see http://lopica.sourceforge.net/os.html
-    
-      if ( last_send_time != -1 && last_send_time < current_send_time ){
-    	  
-    	  	// tims since last
-    	  
+      if ( last_send_time != -1 && last_send_time < current_send_time ){    	  
+    	  // time since last    	  
     	  message.put( "tsl", new Long(current_send_time-last_send_time));
       }
       
@@ -1094,11 +1159,6 @@ public class VersionCheckClient {
     	  
     	  Debug.out( e );
       }
-      
-      String ui = COConfigurationManager.getStringParameter("ui");
-      if (ui.length() > 0) {
-      	message.put("ui", ui);
-      }
 
       // send locale, so we can determine which languages need attention
       message.put("locale", Locale.getDefault().toString());
@@ -1188,34 +1248,7 @@ public class VersionCheckClient {
     	  Debug.out( e );
       }
     }
-    
-    
-    //swt stuff
-    try {
-      Class c = Class.forName( "org.eclipse.swt.SWT" );
-      
-      String swt_platform = (String)c.getMethod( "getPlatform", new Class[]{} ).invoke( null, new Object[]{} );
-      message.put( "swt_platform", swt_platform );
-      
-      if( send_info ) {
-        Integer swt_version = (Integer)c.getMethod( "getVersion", new Class[]{} ).invoke( null, new Object[]{} );
-        message.put( "swt_version", new Long( swt_version.longValue() ) );
-
-        c = Class.forName("org.gudy.azureus2.ui.swt.mainwindow.MainWindow");
-				if (c != null) {
-					c.getMethod("addToVersionCheckMessage", new Class[] { Map.class }).invoke(
-							null, new Object[] { message });
-				}
-      }
-    }
-    catch( ClassNotFoundException e ) {  /* ignore */ }
-    catch( NoClassDefFoundError er ) {  /* ignore */ }
-    catch( InvocationTargetException err ) {  /* ignore */ }
-    catch( Throwable t ) {  t.printStackTrace();  }
-    
-    
-    boolean using_phe = COConfigurationManager.getBooleanParameter( "network.transport.encrypted.require" );
-    message.put( "using_phe", using_phe ? new Long(1) : new Long(0) );
+        
     
     return message;
   }
@@ -1230,10 +1263,10 @@ public class VersionCheckClient {
 		  boolean v6= false;
 		  
 		  // Test connectivity.
-		  if (false) {
-			  System.out.println( "UDP:  " + getSingleton().getExternalIpAddressUDP(null,0,v6));
+		  if (true) {
+			//  System.out.println( "UDP:  " + getSingleton().getExternalIpAddressUDP(null,0,v6));
 			  System.out.println( "TCP:  " + getSingleton().getExternalIpAddressTCP(null,0,v6));
-			  System.out.println( "HTTP: " + getSingleton().getExternalIpAddressHTTP(v6));
+			//  System.out.println( "HTTP: " + getSingleton().getExternalIpAddressHTTP(v6));
 		  }
 		  
 		  Map data = constructVersionCheckMessage(VersionCheckClient.REASON_UPDATE_CHECK_START);
