@@ -25,47 +25,39 @@ package com.aelitis.azureus.plugins.dht;
 
 
 import java.net.InetSocketAddress;
-import java.util.ArrayList;
-import java.util.HashMap;
-import java.util.List;
-import java.util.Map;
-
-
-import org.gudy.azureus2.core3.util.AEMonitor;
-import org.gudy.azureus2.core3.util.AESemaphore;
-import org.gudy.azureus2.core3.util.AEThread;
-import org.gudy.azureus2.core3.util.Constants;
-
-import org.gudy.azureus2.core3.util.Debug;
+import java.util.*;
 
+import org.gudy.azureus2.core3.config.COConfigurationManager;
+import org.gudy.azureus2.core3.util.*;
 import org.gudy.azureus2.plugins.*;
-
 import org.gudy.azureus2.plugins.logging.LoggerChannel;
 import org.gudy.azureus2.plugins.logging.LoggerChannelListener;
-
 import org.gudy.azureus2.plugins.ui.UIManager;
 import org.gudy.azureus2.plugins.ui.components.UITextField;
 import org.gudy.azureus2.plugins.ui.config.*;
-import org.gudy.azureus2.plugins.ui.model.*;
+import org.gudy.azureus2.plugins.ui.model.BasicPluginConfigModel;
+import org.gudy.azureus2.plugins.ui.model.BasicPluginViewModel;
+import org.gudy.azureus2.plugins.utils.DelayedTask;
 import org.gudy.azureus2.plugins.utils.UTTimerEvent;
 import org.gudy.azureus2.plugins.utils.UTTimerEventPerformer;
 
-
 import com.aelitis.azureus.core.AzureusCoreFactory;
 import com.aelitis.azureus.core.dht.DHT;
 import com.aelitis.azureus.core.dht.DHTLogger;
-
 import com.aelitis.azureus.core.dht.control.DHTControlActivity;
+import com.aelitis.azureus.core.dht.control.DHTControlContact;
+import com.aelitis.azureus.core.dht.nat.DHTNATPuncher;
+import com.aelitis.azureus.core.dht.router.DHTRouterContact;
 import com.aelitis.azureus.core.dht.transport.DHTTransportContact;
 import com.aelitis.azureus.core.dht.transport.DHTTransportFullStats;
-import com.aelitis.azureus.core.dht.transport.DHTTransportListener;
 import com.aelitis.azureus.core.dht.transport.udp.DHTTransportUDP;
 import com.aelitis.azureus.core.dht.transport.udp.impl.DHTTransportUDPImpl;
-
+import com.aelitis.azureus.core.networkmanager.admin.NetworkAdmin;
 import com.aelitis.azureus.core.networkmanager.impl.udp.UDPNetworkManager;
 import com.aelitis.azureus.core.versioncheck.VersionCheckClient;
+import com.aelitis.azureus.plugins.dht.impl.DHTPluginContactImpl;
 import com.aelitis.azureus.plugins.dht.impl.DHTPluginImpl;
-
+import com.aelitis.azureus.plugins.dht.impl.DHTPluginImplAdapter;
 import com.aelitis.azureus.plugins.upnp.UPnPMapping;
 import com.aelitis.azureus.plugins.upnp.UPnPPlugin;
 
@@ -92,6 +84,9 @@ DHTPlugin
 	public static final byte		FLAG_SEEDING		= DHT.FLAG_SEEDING;
 	public static final byte		FLAG_MULTI_VALUE	= DHT.FLAG_MULTI_VALUE;
 	public static final byte		FLAG_STATS			= DHT.FLAG_STATS;
+	public static final byte		FLAG_ANON			= DHT.FLAG_ANON;
+	public static final byte		FLAG_PRECIOUS		= DHT.FLAG_PRECIOUS;
+	
 	
 	public static final byte		DT_NONE				= DHT.DT_NONE;
 	public static final byte		DT_FREQUENCY		= DHT.DT_FREQUENCY;
@@ -99,27 +94,23 @@ DHTPlugin
 	
 	public static final int			MAX_VALUE_SIZE		= DHT.MAX_VALUE_SIZE;
 
-	private static final String	PLUGIN_VERSION	= "1.0";
-	private static final String	PLUGIN_NAME		= "Distributed DB";
+	private static final String	PLUGIN_VERSION			= "1.0";
+	private static final String	PLUGIN_NAME				= "Distributed DB";
 	private static final String	PLUGIN_CONFIGSECTION_ID	= "plugins.dht";
+	private static final String PLUGIN_RESOURCE_ID		= "ConfigView.section.plugins.dht";
 	
-	private static final boolean	TRACE_NON_MAIN = false;
-	private static final boolean	MAIN_DHT_ENABLE	= true;
-	private static final boolean	CVS_DHT_ENABLE	= true;
+	private static final boolean	MAIN_DHT_ENABLE		= COConfigurationManager.getBooleanParameter( "dht.net.main_v4.enable", true );
+	private static final boolean	CVS_DHT_ENABLE		= COConfigurationManager.getBooleanParameter( "dht.net.cvs_v4.enable", true );
+	private static final boolean	MAIN_DHT_V6_ENABLE	= COConfigurationManager.getBooleanParameter( "dht.net.main_v6.enable", true );
 	
-	
-	static{
-		
-		if ( TRACE_NON_MAIN ){
-			
-			System.out.println( "**** DHTPlugin - tracing non-main network actions ****" );
-		}
-	}
 		
 	private PluginInterface		plugin_interface;
 	
-	private int					status		= STATUS_INITALISING;
+	private int					status		= STATUS_DISABLED;
 	private DHTPluginImpl[]		dhts;
+	private DHTPluginImpl		main_dht;
+	private DHTPluginImpl		cvs_dht;
+	private DHTPluginImpl		main_v6_dht;
 	
 	private ActionParameter		reseed;
 		
@@ -135,7 +126,8 @@ DHTPlugin
 	private boolean				port_changing;
 	private int					port_change_outstanding;
 	
-	private BooleanParameter	ipfilter_logging;
+	private boolean[]           ipfilter_logging = new boolean[1];
+	private BooleanParameter	warn_user;
 	
 	private UPnPMapping			upnp_mapping;
 	
@@ -144,17 +136,24 @@ DHTPlugin
 	
 	private List				listeners	= new ArrayList();
 	
-
+	private long				start_mono_time = SystemTime.getMonotonousTime();
+	
+	public static void
+	load(
+		PluginInterface		plugin_interface )
+	{
+		plugin_interface.getPluginProperties().setProperty( "plugin.version", 	PLUGIN_VERSION );
+		plugin_interface.getPluginProperties().setProperty( "plugin.name", 		PLUGIN_NAME );
+	}
 		
 	public void
 	initialize(
 		PluginInterface 	_plugin_interface )
 	{
+		status = STATUS_INITALISING;
+		
 		plugin_interface	= _plugin_interface;
 				
-		plugin_interface.getPluginProperties().setProperty( "plugin.version", 	PLUGIN_VERSION );
-		plugin_interface.getPluginProperties().setProperty( "plugin.name", 		PLUGIN_NAME );
-
 		dht_data_port = UDPNetworkManager.getSingleton().getUDPNonDataListeningPortNumber();
 
 		log = plugin_interface.getLogger().getTimeStampedChannel(PLUGIN_NAME);
@@ -162,7 +161,7 @@ DHTPlugin
 		UIManager	ui_manager = plugin_interface.getUIManager();
 
 		final BasicPluginViewModel model = 
-			ui_manager.createBasicPluginViewModel( PLUGIN_NAME);
+			ui_manager.createBasicPluginViewModel( PLUGIN_RESOURCE_ID );
 
 		model.setConfigSectionID(PLUGIN_CONFIGSECTION_ID);
 		
@@ -199,7 +198,15 @@ DHTPlugin
 		config.createGroup( "dht.reseed.group",
 				new Parameter[]{ reseed_label, reseed_ip, reseed_port, reseed });
 		
-		ipfilter_logging = config.addBooleanParameter2( "dht.ipfilter.log", "dht.ipfilter.log", true );
+		final BooleanParameter ipfilter_logging_param = config.addBooleanParameter2( "dht.ipfilter.log", "dht.ipfilter.log", true );
+		ipfilter_logging[0] = ipfilter_logging_param.getValue();
+		ipfilter_logging_param.addListener(new ParameterListener() {
+			public void parameterChanged(Parameter p) {
+				ipfilter_logging[0] = ipfilter_logging_param.getValue();
+			}
+		});
+
+		warn_user = config.addBooleanParameter2( "dht.warn.user", "dht.warn.user", true );
 
 		final BooleanParameter	advanced = config.addBooleanParameter2( "dht.advanced", "dht.advanced", false );
 
@@ -218,7 +225,7 @@ DHTPlugin
 		ActionParameter	execute = config.addActionParameter2( "dht.execute.info", "dht.execute");
 		
 		final BooleanParameter	logging = config.addBooleanParameter2( "dht.logging", "dht.logging", false );
-
+		
 		config.createGroup( "dht.diagnostics.group",
 				new Parameter[]{ command, execute, logging });
 
@@ -247,6 +254,12 @@ DHTPlugin
 				{
 				}
 				
+				public void 
+				starts(
+					byte[] 				key) 
+				{
+				}
+				
 				public void
 				valueRead(
 					DHTPluginContact	originator,
@@ -272,6 +285,7 @@ DHTPlugin
 				
 				public void
 				complete(
+					byte[]	key,
 					boolean	timeout_occurred )
 				{
 					log.log( "complete: timeout = " + timeout_occurred );
@@ -285,11 +299,11 @@ DHTPlugin
 				parameterChanged(
 					Parameter	param )
 				{
-					Thread t = 
-						new AEThread( "DHT:commandrunner" )
+					AEThread2 t = 
+						new AEThread2( "DHT:commandrunner", true )
 						{
 							public void
-							runSupport()
+							run()
 							{
 								if ( dhts == null ){
 									
@@ -308,10 +322,60 @@ DHTPlugin
 									
 									if ( lc.equals("print")){
 										
-										dht.print();
+										dht.print( true );
 										
 										dhts[i].logStats();
 										
+									}else if ( lc.equals( "pingall" )){
+										
+										if ( i == 1 ){
+											
+											dht.getControl().pingAll();
+										}
+										
+									}else if ( lc.equals( "versions" )){
+										
+										List<DHTRouterContact> contacts = dht.getRouter().getAllContacts();
+										
+										Map<Byte,Integer>	counts = new TreeMap<Byte, Integer>();
+										
+										for ( DHTRouterContact r: contacts ){
+											
+											DHTControlContact contact = (DHTControlContact)r.getAttachment();
+											
+											byte v = contact.getTransportContact().getProtocolVersion();
+											
+											Integer count = counts.get( v );
+											
+											if ( count == null ){
+												
+												counts.put( v, 1 );
+												
+											}else{
+												
+												counts.put( v, count+1 );
+											}
+										}
+										
+										log.log( "Net " + dht.getTransport().getNetwork());
+										
+										int	total = contacts.size();
+										
+										if ( total == 0 ){
+											
+											log.log( "   no contacts" );
+											
+										}else{
+											
+											String ver = "";
+											
+											for ( Map.Entry<Byte, Integer> entry: counts.entrySet()){
+											
+												ver += (ver.length()==0?"":", " ) + entry.getKey() + "=" + 100*entry.getValue()/total;
+											}
+											
+											log.log( "    " + ver );
+										}
 									}else if ( lc.equals( "testca" )){
 																
 										((DHTTransportUDPImpl)transport).testExternalAddressChange();
@@ -362,12 +426,16 @@ DHTPlugin
 												
 												originator_data.put( "hello", "mum" );
 
-												dht.getNATPuncher().punch( "Test", transport.getLocalContact(), null, originator_data );
+												DHTNATPuncher puncher = dht.getNATPuncher();
 												
+												if ( puncher != null ){
+												
+													puncher.punch( "Test", transport.getLocalContact(), null, originator_data );
+												}
 											}else if ( lhs.equals( "stats" )){
 												
 												try{
-													pos = rhs.indexOf( ":" );
+													pos = rhs.lastIndexOf( ":" );
 													
 													DHTTransportContact	contact;
 													
@@ -386,6 +454,8 @@ DHTPlugin
 																		transport.getProtocolVersion());
 													}
 													
+													log.log( "Stats request to " + contact.getName());
+													
 													DHTTransportFullStats stats = contact.getStats();
 														
 													log.log( "Stats:" + (stats==null?"<null>":stats.getString()));
@@ -407,9 +477,7 @@ DHTPlugin
 								}
 							}
 						};
-						
-					t.setDaemon(true);
-					
+											
 					t.start();
 				}
 			});
@@ -423,11 +491,11 @@ DHTPlugin
 					{
 						reseed.setEnabled( false );						
 
-						Thread t = 
-							new AEThread( "DHT:reseeder" )
+						AEThread2 t = 
+							new AEThread2( "DHT:reseeder", true )
 							{
 								public void
-								runSupport()
+								run()
 								{
 									try{
 										String	ip 	= reseed_ip.getValue().trim();
@@ -462,9 +530,7 @@ DHTPlugin
 									}
 								}
 							};
-							
-						t.setDaemon( true );
-						
+													
 						t.start();
 					}
 				});
@@ -526,7 +592,7 @@ DHTPlugin
 				{
 					if ( log_type == DHTLogger.LT_IP_FILTER ){
 						
-						return( ipfilter_logging.getValue());
+						return ipfilter_logging[0];
 					}
 					
 					return( true );
@@ -551,21 +617,6 @@ DHTPlugin
 			return;
 		}
 		
-		PluginInterface pi_upnp = plugin_interface.getPluginManager().getPluginInterfaceByClass( UPnPPlugin.class );
-		
-		if ( pi_upnp == null ){
-
-			log.log( "UPnP plugin not found, can't map port" );
-			
-		}else{
-			
-			upnp_mapping = ((UPnPPlugin)pi_upnp.getPlugin()).addMapping( 
-							plugin_interface.getPluginName(), 
-							false, 
-							dht_data_port, 
-							true );
-		}
-
 		setPluginInfo();
 		
 		plugin_interface.addListener(
@@ -574,6 +625,21 @@ DHTPlugin
 				public void
 				initializationComplete()
 				{
+					PluginInterface pi_upnp = plugin_interface.getPluginManager().getPluginInterfaceByClass( UPnPPlugin.class );
+					
+					if ( pi_upnp == null ){
+
+						log.log( "UPnP plugin not found, can't map port" );
+						
+					}else{
+						
+						upnp_mapping = ((UPnPPlugin)pi_upnp.getPlugin()).addMapping( 
+										plugin_interface.getPluginName(), 
+										false, 
+										dht_data_port, 
+										true );
+					}
+
 					String	ip = null;
 					
 					if ( advanced.getValue()){
@@ -599,6 +665,8 @@ DHTPlugin
 							dhts[i].closedownInitiated();
 						}
 					}
+					
+					saveClockSkew();
 				}
 				
 				public void
@@ -627,6 +695,8 @@ DHTPlugin
 						}
 						
 						setPluginInfo();
+						
+						saveClockSkew();
 					}
 				});
 
@@ -656,10 +726,10 @@ DHTPlugin
 			port_change_mon.exit();
 		}
 		
-		new AEThread("DHTPlugin:portChanger", true )
+		new AEThread2("DHTPlugin:portChanger", true )
 		{
 			public void
-			runSupport()
+			run()
 			{
 				while( true ){
 					
@@ -727,12 +797,14 @@ DHTPlugin
 		final boolean			logging,
 		final String			override_ip )
 	{
-		Thread t = 
-			new AEThread( "DHTPlugin.init" )
+		AEThread2 t = 
+			new AEThread2( "DHTPlugin.init", true )
 			{
 				public void
-				runSupport()
+				run()
 				{
+					boolean	went_async = false;
+					
 					try{							
 						
 						enabled = VersionCheckClient.getSingleton().DHTEnableAllowed();
@@ -741,73 +813,147 @@ DHTPlugin
 							
 							status_area.setText( "Initialising" );
 							
-							List	plugins = new ArrayList();
-							
-							if ( MAIN_DHT_ENABLE ){
-								
-								DHTPluginImpl plug = new DHTPluginImpl(
-												plugin_interface,
-												AzureusCoreFactory.getSingleton().getNATTraverser(),
-												DHTTransportUDP.PROTOCOL_VERSION_MAIN,
-												DHT.NW_MAIN,
-												override_ip,
-												dht_data_port,
-												reseed,
-												logging,
-												log, dht_log );
-								
-								plugins.add( plug );
-							}
-							
-							if ( Constants.isCVSVersion() && CVS_DHT_ENABLE ){
-								
-								plugins.add( new DHTPluginImpl(
-										plugin_interface,
-										AzureusCoreFactory.getSingleton().getNATTraverser(),
-										DHTTransportUDP.PROTOCOL_VERSION_CVS,
-										DHT.NW_CVS,
-										override_ip,
-										dht_data_port,
-										reseed,
-										logging,
-										log, dht_log ));
-							}
-							
-							DHTPluginImpl[]	_dhts = new DHTPluginImpl[plugins.size()];
-							
-							plugins.toArray( _dhts );
+							final DelayedTask dt = plugin_interface.getUtilities().createDelayedTask(new Runnable()
+							{
+								public void 
+								run() 
+								{
+										// go async again as don't want to block other tasks
+									
+									new AEThread2(  "DHTPlugin.init2", true )
+									{
+										public void
+										run()
+										{
+											try{
+												List	plugins = new ArrayList();
 												
-							dhts = _dhts;
+													// adapter only added to first DHTPluginImpl we create
+												
+												DHTPluginImplAdapter adapter = 
+										        		new DHTPluginImplAdapter()
+										        		{
+										        			public void
+										        			localContactChanged(
+										        				DHTPluginContact	local_contact )
+										        			{
+										        				for (int i=0;i<listeners.size();i++){
+										        					
+										        					((DHTPluginListener)listeners.get(i)).localAddressChanged( local_contact );
+										        				}
+										        			}
+										        		};
+										        		
+												if ( MAIN_DHT_ENABLE ){
+													
+													main_dht = 
+														new DHTPluginImpl(
+																	plugin_interface,
+																	AzureusCoreFactory.getSingleton().getNATTraverser(),
+																	adapter,
+																	DHTTransportUDP.PROTOCOL_VERSION_MAIN,
+																	DHT.NW_MAIN,
+																	false,
+																	override_ip,
+																	dht_data_port,
+																	reseed,
+																	warn_user,
+																	logging,
+																	log, dht_log );
+																					
+													plugins.add( main_dht );
+													
+													adapter = null;
+												}
+												
+												if ( MAIN_DHT_V6_ENABLE ){
+													
+													if ( NetworkAdmin.getSingleton().hasIPV6Potential(false)){
+														
+														main_v6_dht = 
+															new DHTPluginImpl(
+																plugin_interface,
+																AzureusCoreFactory.getSingleton().getNATTraverser(),
+																adapter,
+																DHTTransportUDP.PROTOCOL_VERSION_MAIN,
+																DHT.NW_MAIN_V6,
+																true,
+																null,
+																dht_data_port,
+																reseed,
+																warn_user,
+																logging,
+																log, dht_log );
+																					
+														plugins.add( main_v6_dht );
+														
+														adapter = null;
+													}
+												}
+												
+												if ( Constants.isCVSVersion() && CVS_DHT_ENABLE ){
+													
+													cvs_dht = 
+														new DHTPluginImpl(
+															plugin_interface,
+															AzureusCoreFactory.getSingleton().getNATTraverser(),
+															adapter,
+															DHTTransportUDP.PROTOCOL_VERSION_CVS,
+															DHT.NW_CVS,
+															false,
+															override_ip,
+															dht_data_port,
+															reseed,
+															warn_user,
+															logging,
+															log, dht_log );
+												
+													plugins.add( cvs_dht );
+													
+													adapter = null;
+												}
+												
+												DHTPluginImpl[]	_dhts = new DHTPluginImpl[plugins.size()];
+												
+												plugins.toArray( _dhts );
+																	
+												dhts = _dhts;
+												
+												status = dhts[0].getStatus();
+												
+												status_area.setText( dhts[0].getStatusText());
+												
+											}catch( Throwable e ){
+												
+												enabled	= false;
+												
+												status	= STATUS_DISABLED;
+		
+												status_area.setText( "Disabled due to error during initialisation" );
+		
+												log.log( e );
+												
+												Debug.printStackTrace(e);
+												
+											}finally{
+												
+												init_sem.releaseForever();
+											}
+											
+												// pick up any port changes that occurred during init
+											
+											if ( status == STATUS_RUNNING ){
+											
+												changePort( dht_data_port );
+											}
+										}
+									}.start();
+								}
+							});
 							
-							status = dhts[0].getStatus();
-							
-							status_area.setText( dhts[0].getStatusText());
-						
-							dhts[0].getDHT().getTransport().addListener(
-					        		new DHTTransportListener()
-					        		{
-					        			public void
-					        			localContactChanged(
-					        				DHTTransportContact	local_contact )
-					        			{
-					        				for (int i=0;i<listeners.size();i++){
-					        					
-					        					((DHTPluginListener)listeners.get(i)).localAddressChanged(dhts[0].getLocalAddress());
-					        				}
-					        			}
-					        			
-					        			public void
-					        			currentAddress(
-					        				String		address )
-					        			{
-					        			}
-					        			
-					        			public void
-					        			reachabilityChanged(
-					        				boolean	reacheable )
-					        			{
-					        			}
-					        		});							
+							dt.queue();
+								
+							went_async = true;
 							
 						}else{
 							
@@ -829,20 +975,14 @@ DHTPlugin
 						
 					}finally{
 						
-						init_sem.releaseForever();
-					}
-					
-						// pick up any port changes that occurred during init
-					
-					if ( status == STATUS_RUNNING ){
-					
-						changePort( dht_data_port );
+						if ( !went_async ){
+						
+							init_sem.releaseForever();
+						}
 					}
 				}
 			};
-			
-		t.setDaemon(true);
-		
+					
 		t.start();
 	}
 	
@@ -855,10 +995,27 @@ DHTPlugin
 				"plugin.info", 
 				reachable?"1":"0" );
 	}
-
+	
 	public boolean
 	isEnabled()
 	{
+		if ( plugin_interface == null ){
+		
+			Debug.out( "Called too early!" );
+			
+			return false;
+		}
+		
+		if ( plugin_interface.isInitialisationThread()){
+			
+			if ( !init_sem.isReleasedForever()){
+
+				Debug.out( "Initialisation deadlock detected" );
+				
+				return( true );
+			}
+		}
+		
 		init_sem.reserve();
 		
 		return( enabled );
@@ -875,6 +1032,12 @@ DHTPlugin
 		return( true );	// don't know yet
 	}
 	
+	public boolean
+	isInitialising()
+	{
+		return( !init_sem.isReleasedForever());
+	}
+	
 	public boolean
 	isExtendedUseAllowed()
 	{
@@ -916,140 +1079,439 @@ DHTPlugin
 		return( dhts[0].isDiversified( key ));
 	}
 	
+	public void
+	put(
+		byte[]						key,
+		String						description,
+		byte[]						value,
+		byte						flags,
+		DHTPluginOperationListener	listener)
+	{
+		put( key, description, value, flags, true, listener );
+	}
+	
 	public void
 	put(
 		final byte[]						key,
 		final String						description,
 		final byte[]						value,
 		final byte							flags,
+		final boolean						high_priority,
 		final DHTPluginOperationListener	listener)
 	{
 		if ( !isEnabled()){
 			
 			throw( new RuntimeException( "DHT isn't enabled" ));
 		}
-				
-		dhts[0].put( key, description, value, flags, listener );
-		
-		for (int i=1;i<dhts.length;i++){
-
-			final int f_i	= i;
 			
-			new AEThread( "mutli-dht: put", true )
-			{
-				public void
-				runSupport()
+		if( dhts.length == 1 ){
+			
+			dhts[0].put( key, description, value, flags, high_priority, listener );
+			
+		}else{
+			
+			final int[]	completes_to_go = { dhts.length };
+			
+			DHTPluginOperationListener main_listener = 
+				new DHTPluginOperationListener()
 				{
-					dhts[f_i].put( key, description, value, flags, 
-							new DHTPluginOperationListener()
+					public void
+					diversified()
+					{
+						listener.diversified();
+					}
+					
+					public void 
+					starts(
+						byte[] 				key ) 
+					{
+						listener.starts(key);
+					}
+					
+					public void
+					valueRead(
+						DHTPluginContact	originator,
+						DHTPluginValue		value )
+					{
+						listener.valueRead(originator, value);
+					}
+					
+					public void
+					valueWritten(
+						DHTPluginContact	target,
+						DHTPluginValue		value )
+					{
+						listener.valueWritten(target, value);
+					}
+					
+					public void
+					complete(
+						byte[]	key,
+						boolean	timeout_occurred )
+					{
+						synchronized( completes_to_go ){
+						
+							completes_to_go[0]--;
+							
+							if ( completes_to_go[0] == 0 ){
+								
+								listener.complete(key, timeout_occurred);
+							}
+						}
+					}
+				};
+				
+			dhts[0].put( key, description, value, flags, high_priority, main_listener );
+
+			for (int i=1;i<dhts.length;i++){
+
+				dhts[i].put( 
+						key, description, value, flags, high_priority,
+						new DHTPluginOperationListener()
+						{
+							public void
+							diversified()
 							{
-								public void
-								diversified()
-								{
-								}
-								
-								public void
-								valueRead(
-									DHTPluginContact	originator,
-									DHTPluginValue		value )
-								{
-									if ( TRACE_NON_MAIN ){
-										System.out.println( "DHT_" + f_i + ":put valueRead" );
+							}
+							
+							public void 
+							starts(
+								byte[] 				key ) 
+							{
+							}
+							
+							public void
+							valueRead(
+								DHTPluginContact	originator,
+								DHTPluginValue		value )
+							{
+							}
+							
+							public void
+							valueWritten(
+								DHTPluginContact	target,
+								DHTPluginValue		value )
+							{
+							}
+							
+							public void
+							complete(
+								byte[]	key,
+								boolean	timeout_occurred )
+							{
+								synchronized( completes_to_go ){
+									
+									completes_to_go[0]--;
+									
+									if ( completes_to_go[0] == 0 ){
+										
+										listener.complete(key, timeout_occurred);
 									}
 								}
-								
-								public void
-								valueWritten(
-									DHTPluginContact	target,
-									DHTPluginValue		value )
-								{
-									if ( TRACE_NON_MAIN ){
-										System.out.println( "DHT_" + f_i + ":put valueWritten" );
-									}
-								}
-								
-								public void
-								complete(
-									boolean	timeout_occurred )
-								{
-									if ( TRACE_NON_MAIN ){
-										System.out.println( "DHT_" + f_i + ":put complete, timeout=" + timeout_occurred );
-									}
-								}
-							});
-				}
-			}.start();
+							}
+						});
+			}
 		}
 	}
 	
+	public DHTPluginValue
+	getLocalValue(
+		byte[]		key )
+	{
+		return( main_dht.getLocalValue( key ));
+	}
+	
 	public void
 	get(
-		final byte[]								key,
+		final byte[]								original_key,
 		final String								description,
 		final byte									flags,
 		final int									max_values,
 		final long									timeout,
 		final boolean								exhaustive,
 		final boolean								high_priority,
-		final DHTPluginOperationListener			listener )
+		final DHTPluginOperationListener			original_listener )
 	{
 		if ( !isEnabled()){
 			
 			throw( new RuntimeException( "DHT isn't enabled" ));
 		}
-				
-		dhts[0].get( key, description, flags, max_values, timeout, exhaustive, high_priority, listener );
+					
+		final DHTPluginOperationListener main_listener;
 		
-		for (int i=1;i<dhts.length;i++){
-
-			final int f_i	= i;
+		if ( cvs_dht == null ){
 			
-			new AEThread( "mutli-dht: get", true )
-			{
-				public void
-				runSupport()
+			main_listener = original_listener;
+			
+		}else{
+			
+				// hook into CVS completion to prevent runaway CVS dht operations
+			
+			final int[]		completes_to_go = { 2 };
+			final boolean[]	main_timeout 	= { false };
+			
+			main_listener = 
+				new DHTPluginOperationListener()
 				{
-					dhts[f_i].get( 
-							key, description, flags, max_values, timeout, exhaustive, high_priority,
-							new DHTPluginOperationListener()
-							{
-								public void
-								diversified()
-								{
-								}
+					public void
+					diversified()
+					{
+						original_listener.diversified();
+					}
+					
+					public void 
+					starts(
+						byte[] 				key ) 
+					{
+						original_listener.starts( original_key );
+					}
+					
+					public void
+					valueRead(
+						DHTPluginContact	originator,
+						DHTPluginValue		value )
+					{
+						original_listener.valueRead(originator, value);
+					}
+					
+					public void
+					valueWritten(
+						DHTPluginContact	target,
+						DHTPluginValue		value )
+					{
+						original_listener.valueWritten(target, value);
+					}
+					
+					public void
+					complete(
+						byte[]	key,
+						boolean	timeout_occurred )
+					{
+						synchronized( completes_to_go ){
+						
+							completes_to_go[0]--;
+							
+							main_timeout[0] = timeout_occurred;
+							
+							if ( completes_to_go[0] == 0 ){
 								
-								public void
-								valueRead(
-									DHTPluginContact	originator,
-									DHTPluginValue		value )
-								{
-									if ( TRACE_NON_MAIN ){
-										System.out.println( "DHT_" + f_i + ":get valueRead" );
-									}
-								}
+								original_listener.complete( original_key, timeout_occurred );
+							}
+						}
+					}
+				};
+			
+			cvs_dht.get( 
+					original_key, description, flags, max_values, timeout, exhaustive, high_priority,
+					new DHTPluginOperationListener()
+					{
+						public void
+						diversified()
+						{
+						}
+						
+						public void 
+						starts(
+							byte[] 				key ) 
+						{
+						}
+						
+						public void
+						valueRead(
+							DHTPluginContact	originator,
+							DHTPluginValue		value )
+						{
+						}
+						
+						public void
+						valueWritten(
+							DHTPluginContact	target,
+							DHTPluginValue		value )
+						{
+						}
+						
+						public void
+						complete(
+							byte[]	key,
+							boolean	timeout_occurred )
+						{
+							synchronized( completes_to_go ){
 								
-								public void
-								valueWritten(
-									DHTPluginContact	target,
-									DHTPluginValue		value )
-								{
-									if ( TRACE_NON_MAIN ){
-										System.out.println( "DHT_" + f_i + ":get valueWritten" );
-									}
-								}
+								completes_to_go[0]--;
 								
-								public void
-								complete(
-									boolean	timeout_occurred )
-								{
-									if ( TRACE_NON_MAIN ){
-										System.out.println( "DHT_" + f_i + ":get complete, timeout=" + timeout_occurred );
-									}
+								if ( completes_to_go[0] == 0 ){
+									
+									original_listener.complete( original_key, main_timeout[0] );
 								}
-							});
-				}
-			}.start();
+							}
+						}
+					});
 		}
+		
+		if ( main_dht != null && main_v6_dht == null ){
+			
+			main_dht.get( original_key, description, flags, max_values, timeout, exhaustive, high_priority, main_listener );
+			
+		}else if ( main_dht == null && main_v6_dht != null ){
+			
+			main_v6_dht.get( original_key, description, flags, max_values, timeout, exhaustive, high_priority, main_listener );
+
+		}else{
+			
+				// both DHTs active. Initially (at least :) V6 is going to be very sparse. We therefore
+				// don't want to be blocking the "get" operation waiting for V6 to timeout when V4 is
+				// returning hits 
+			
+			final	byte[]	v4_key	= original_key;
+			final	byte[]	v6_key	= (byte[])original_key.clone();
+			
+			DHTPluginOperationListener	dual_listener =
+				new DHTPluginOperationListener()
+				{
+					private long start_time = SystemTime.getCurrentTime();
+
+					private boolean	started;
+					
+					private int	complete_count 	= 0;
+					private int	result_count	= 0;
+					
+					public void
+					diversified()
+					{
+					}
+					
+					public void 
+					starts(
+						byte[] 				key ) 
+					{
+						synchronized( this ){
+
+							if ( started ){
+								
+								return;
+							}
+							
+							started = true;
+						}
+						
+						main_listener.starts( original_key );
+					}
+					
+					public void
+					valueRead(
+						DHTPluginContact	originator,
+						DHTPluginValue		value )
+					{												
+						synchronized( this ){
+
+							result_count++;
+	
+								// only report if not yet complete
+							
+							if ( complete_count < 2 ){
+						
+								main_listener.valueRead( originator, value );
+							}
+						}
+					}
+					
+					public void
+					valueWritten(
+						DHTPluginContact	target,
+						DHTPluginValue		value )
+					{
+						Debug.out( "eh?" );
+					}
+					
+					public void
+					complete(
+						final byte[]		timeout_key,
+						final boolean		timeout_occurred )
+					{
+							// we are guaranteed to come through here at least twice
+						
+						synchronized( this ){
+							
+							complete_count++;
+							
+							if ( complete_count == 2 ){
+								
+									// if we have reported any results then we can't report 
+									// timeout!
+								
+								main_listener.complete( original_key, result_count>0?false:timeout_occurred );
+								
+								return;
+								
+							}else if ( complete_count > 2 ){
+								
+								return;
+							}
+						
+								// One of the two gets, see how much longer we're happy to hang around for
+								// Only of interest if timeout then uninterested as the other will be 
+								// about to timeout
+								
+							if ( timeout_occurred ){
+								
+								return;
+							}
+							
+								// ignore a v6 completion ahead of a v4
+							
+							if ( timeout_key == v6_key ){
+								
+								return;
+							}
+
+							long	now = SystemTime.getCurrentTime();
+							
+							long	elapsed = now - start_time;
+							
+							long	rem = timeout - elapsed;
+							
+							if ( rem <= 0 ){
+								
+								complete( timeout_key, true );
+								
+							}else{
+								
+								SimpleTimer.addEvent(
+									"DHTPlugin:dual_dht_early_timeout",
+									now + rem,
+									new TimerEventPerformer()
+									{
+										public void 
+										perform(
+											TimerEvent event) 
+										{
+											complete( timeout_key, true );
+										}
+									});
+							}
+						}
+					}
+				};
+			
+				// hack - use different keys so we can distinguish which completion event we
+				// have received above
+				
+			main_dht.get( v4_key, description, flags, max_values, timeout, exhaustive, high_priority, dual_listener );
+			
+			main_v6_dht.get( v6_key, description, flags, max_values, timeout, exhaustive, high_priority, dual_listener );
+		}
+	}
+	
+	public boolean
+	hasLocalKey(
+		byte[]		hash )
+	{
+		if ( !isEnabled()){
+			
+			throw( new RuntimeException( "DHT isn't enabled" ));
+		}
+		
+		return( dhts[0].getLocalValue( hash ) != null );
 	}
 	
 	public void
@@ -1069,10 +1531,10 @@ DHTPlugin
 
 			final int f_i	= i;
 			
-			new AEThread( "mutli-dht: remove", true )
+			new AEThread2( "multi-dht: remove", true )
 			{
 				public void
-				runSupport()
+				run()
 				{
 					dhts[f_i].remove( 
 							key, description, 
@@ -1083,14 +1545,17 @@ DHTPlugin
 								{
 								}
 								
+								public void 
+								starts(
+									byte[] 				key ) 
+								{
+								}
+								
 								public void
 								valueRead(
 									DHTPluginContact	originator,
 									DHTPluginValue		value )
 								{
-									if ( TRACE_NON_MAIN ){
-										System.out.println( "DHT_" + f_i + ":remove valueRead" );
-									}
 								}
 								
 								public void
@@ -1098,18 +1563,13 @@ DHTPlugin
 									DHTPluginContact	target,
 									DHTPluginValue		value )
 								{
-									if ( TRACE_NON_MAIN ){
-										System.out.println( "DHT_" + f_i + ":remove valueWritten" );
-									}
 								}
 								
 								public void
 								complete(
+									byte[]	key,
 									boolean	timeout_occurred )
 								{
-									if ( TRACE_NON_MAIN ){
-										System.out.println( "DHT_" + f_i + ":remove complete, timeout=" + timeout_occurred );
-									}
 								}
 							});
 				}
@@ -1117,6 +1577,103 @@ DHTPlugin
 		}
 	}
 	
+	public void
+	remove(
+		DHTPluginContact[]			targets,
+		byte[]						key,
+		String						description,
+		DHTPluginOperationListener	listener )
+	{
+		if ( !isEnabled()){
+			
+			throw( new RuntimeException( "DHT isn't enabled" ));
+		}
+			
+		Map	dht_map = new HashMap();
+		
+		for (int i=0;i<targets.length;i++){
+			
+			DHTPluginContactImpl target = (DHTPluginContactImpl)targets[i];
+			
+			DHTPluginImpl dht = target.getDHT();
+			
+			List	c = (List)dht_map.get(dht);
+			
+			if ( c == null ){
+				
+				c = new ArrayList();
+				
+				dht_map.put( dht, c );
+			}
+			
+			c.add( target );
+		}
+		
+		Iterator	it = dht_map.entrySet().iterator();
+		
+		boolean 	primary = true;
+		
+		while( it.hasNext()){
+			
+			Map.Entry entry = (Map.Entry)it.next();
+			
+			DHTPluginImpl 	dht 		= (DHTPluginImpl)entry.getKey();
+			List			contacts 	= (List)entry.getValue();
+			
+			DHTPluginContact[]	dht_targets = new DHTPluginContact[contacts.size()];
+			
+			contacts.toArray( dht_targets );
+			
+			if ( primary ){
+				
+				primary = false;
+				
+				dht.remove( dht_targets, key, description, listener );
+				
+			}else{
+				
+					// lazy - just report ops on one dht
+				
+				dht.remove(
+						dht_targets, key, description,
+						new DHTPluginOperationListener()
+						{
+							public void
+							diversified()
+							{
+							}
+							
+							public void 
+							starts(
+								byte[] 				key ) 
+							{
+							}
+							
+							public void
+							valueRead(
+								DHTPluginContact	originator,
+								DHTPluginValue		value )
+							{
+							}
+							
+							public void
+							valueWritten(
+								DHTPluginContact	target,
+								DHTPluginValue		value )
+							{
+							}
+							
+							public void
+							complete(
+								byte[]	key,
+								boolean	timeout_occurred )
+							{
+							}
+						});
+			}
+		}
+	}
+	
 	public DHTPluginContact
 	importContact(
 		InetSocketAddress				address )
@@ -1161,66 +1718,6 @@ DHTPlugin
 			dhts[i].registerHandler( handler_key, handler );
 		}
 	}
-	
-	public byte[]
-	read(
-		final DHTPluginProgressListener	listener,
-		final DHTPluginContact			target,
-		final byte[]					handler_key,
-		final byte[]					key,
-		final long						timeout )
-	{
-		if ( !isEnabled()){
-			
-			throw( new RuntimeException( "DHT isn't enabled" ));
-		}
-		
-		for (int i=1;i<dhts.length;i++){
-
-			final int f_i	= i;
-			
-			new AEThread( "mutli-dht: readXfer", true )
-			{
-				public void
-				runSupport()
-				{
-					dhts[f_i].read( 
-							new DHTPluginProgressListener()
-							{
-								public void
-								reportSize(
-									long	size )
-								{
-									if ( TRACE_NON_MAIN ){
-										System.out.println( "DHT_" + f_i + ":readXfer: size = " + size );
-									}
-								}
-								
-								public void
-								reportActivity(
-									String	str )
-								{
-									if ( TRACE_NON_MAIN ){
-										System.out.println( "DHT_" + f_i + ":readXfer: act = " + str );
-									}
-								}
-								
-								public void
-								reportCompleteness(
-									int		percent )
-								{
-									if ( TRACE_NON_MAIN ){
-										System.out.println( "DHT_" + f_i + ":readXfer: % = " + percent );
-									}
-								}
-							},
-							target, handler_key, key, timeout );
-				}
-			}.start();
-		}
-		
-		return( dhts[0].read( listener, target, handler_key, key, timeout ));
-	}
 
 	public int
 	getStatus()
@@ -1246,6 +1743,71 @@ DHTPlugin
 		return( res );
 	}
 	
+	public DHT
+	getDHT(
+		int		network )
+	{
+		if ( dhts == null ){
+			
+			return( null );
+		}
+		
+		for (int i=0;i<dhts.length;i++){
+			
+			if ( dhts[i].getDHT().getTransport().getNetwork() == network ){
+				
+				return( dhts[i].getDHT());
+			}
+		}
+		
+		return( null );
+	}
+	
+	protected long
+	loadClockSkew()
+	{
+		return( plugin_interface.getPluginconfig().getPluginLongParameter( "dht.skew", 0 ));
+	}
+	
+	protected void
+	saveClockSkew()
+	{
+		long	existing 	= loadClockSkew();
+		long	current		= getClockSkew();
+		
+		if ( Math.abs( existing - current ) > 5000 ){
+		
+			plugin_interface.getPluginconfig().setPluginParameter( "dht.skew", getClockSkew());
+		}
+	}
+	
+	public long 
+	getClockSkew() 
+	{
+		if ( dhts == null || dhts.length == 0 ){
+			
+			return( 0 );
+		}
+	
+		long uptime = SystemTime.getMonotonousTime() - start_mono_time;
+		
+		if ( uptime < 5*60*1000 ){
+			
+			return( loadClockSkew());
+		}
+		
+		long skew = dhts[0].getClockSkew();
+		
+		if ( skew > 24*60*60*1000 ){
+			
+			skew = 0;
+		}
+		
+		skew = ( skew/500 )*500;
+		
+		return( skew );
+	}
+	
 	public DHTPluginKeyStats
 	decodeStats(
 		DHTPluginValue		value )
