@@ -33,6 +33,8 @@ import org.gudy.azureus2.plugins.clientid.ClientIDGenerator;
 import org.gudy.azureus2.plugins.download.Download;
 import org.gudy.azureus2.plugins.peers.Peer;
 import org.gudy.azureus2.plugins.peers.PeerManager;
+import org.gudy.azureus2.plugins.peers.PeerManagerEvent;
+import org.gudy.azureus2.plugins.peers.PeerManagerListener2;
 import org.gudy.azureus2.plugins.peers.PeerReadRequest;
 import org.gudy.azureus2.plugins.peers.PeerStats;
 import org.gudy.azureus2.plugins.peers.Piece;
@@ -41,6 +43,7 @@ import org.gudy.azureus2.plugins.utils.Monitor;
 import org.gudy.azureus2.plugins.utils.PooledByteBuffer;
 import org.gudy.azureus2.plugins.utils.Semaphore;
 
+import com.aelitis.azureus.core.util.CopyOnWriteSet;
 import com.aelitis.azureus.plugins.extseed.ExternalSeedException;
 import com.aelitis.azureus.plugins.extseed.ExternalSeedPeer;
 import com.aelitis.azureus.plugins.extseed.ExternalSeedPlugin;
@@ -50,7 +53,7 @@ import com.aelitis.azureus.plugins.extseed.util.ExternalSeedHTTPDownloaderListen
 
 public abstract class 
 ExternalSeedReaderImpl 
-	implements ExternalSeedReader
+	implements ExternalSeedReader, PeerManagerListener2
 {
 	public static final int	RECONNECT_DEFAULT 			= 30*1000;
 	public static final int INITIAL_DELAY				= 30*1000;
@@ -99,6 +102,8 @@ ExternalSeedReaderImpl
 	private int					rate_bytes_read;
 	private int					rate_bytes_permitted;
 
+	private volatile CopyOnWriteSet<MutableInteger>		bad_pieces = new CopyOnWriteSet<MutableInteger>();
+	
 	protected
 	ExternalSeedReaderImpl(
 		ExternalSeedPlugin 		_plugin,
@@ -210,6 +215,24 @@ ExternalSeedReaderImpl
 		}
 	}
 	
+	public void
+	eventOccurred(
+		PeerManagerEvent	event )
+	{
+		if ( event.getType() == PeerManagerEvent.ET_PEER_SENT_BAD_DATA ){
+			
+			if ( event.getPeer().getIp().equals( getIP())){
+									
+				if ( bad_pieces.size() > 128 ){
+						
+					return;
+				}
+						
+				bad_pieces.add(new MutableInteger((Integer)event.getData()));
+			}
+		}
+	}
+	
 	protected boolean
 	readyToActivate(
 		PeerManager	peer_manager,
@@ -255,11 +278,18 @@ ExternalSeedReaderImpl
 				return( false );
 			}
 			
-			if ( peer_manager.getDownload().getState() == Download.ST_SEEDING ){
+			if ( peer_manager.getDownload().getState() != Download.ST_DOWNLOADING ){
 				
 				return( false );
 			}
-					
+				
+				// check dnd completeness too
+			
+			if ( peer_manager.getDownload().isComplete()){
+				
+				return( false );
+			}
+			
 				// now the more interesting stuff
 			
 			if ( transient_seed ){
@@ -470,8 +500,18 @@ ExternalSeedReaderImpl
 			
 			peer_manager_change_time	= now;
 			
+			if ( current_manager != null ){
+				
+				current_manager.removeListener( this );
+			}
+			
 			current_manager	= peer_manager;
 			
+			if ( current_manager != null ){
+
+				current_manager.addListener( this );
+			}
+			
 			setActive( false );
 		}
 		
@@ -751,8 +791,17 @@ ExternalSeedReaderImpl
 			int	max_free_reqs		= 0;
 			int max_free_reqs_piece	= -1;
 			
+			MutableInteger	mi = new MutableInteger(0);
+			
 			for (int i=0;i<pieces.length;i++){
 				
+				mi.setValue( i );
+				
+				if ( bad_pieces.contains(mi)){
+					
+					continue;
+				}
+				
 				Piece	piece = pieces[i];
 				
 				if ( piece.isFullyAllocatable()){
@@ -1269,4 +1318,46 @@ ExternalSeedReaderImpl
 	{
 		return( getIntParam( map, name, def?1:0) != 0 );
 	}
+	
+	protected static class
+	MutableInteger
+	{
+		private int	value;
+		
+		protected
+		MutableInteger(
+			int		v )
+		{
+			value = v;
+		}
+
+		protected void
+		setValue(
+			int	v )
+		{
+			value = v;
+		}
+		
+		protected int
+		getValue()
+		{
+			return( value );
+		}
+		
+		public int 
+		hashCode() 
+		{
+			return value;
+		}
+
+		public boolean 
+		equals(
+			Object obj )
+		{
+			if (obj instanceof MutableInteger) {
+				return value == ((MutableInteger)obj).value;
+			}
+			return false;
+		}
+	}
 }
