@@ -21,38 +21,23 @@
 
 package com.aelitis.azureus.plugins.net.buddy;
 
-import java.io.File;
-import java.net.InetAddress;
-import java.security.SecureRandom;
+import java.io.*;
+import java.net.*;
 import java.util.*;
 
 import org.gudy.azureus2.core3.config.COConfigurationManager;
-import org.gudy.azureus2.core3.util.AERunnable;
-import org.gudy.azureus2.core3.util.AESemaphore;
-import org.gudy.azureus2.core3.util.AEThread2;
-import org.gudy.azureus2.core3.util.AsyncDispatcher;
-import org.gudy.azureus2.core3.util.BDecoder;
-import org.gudy.azureus2.core3.util.BEncoder;
-import org.gudy.azureus2.core3.util.Base32;
-import org.gudy.azureus2.core3.util.Debug;
-import org.gudy.azureus2.core3.util.DisplayFormatters;
-import org.gudy.azureus2.core3.util.SHA1Simple;
-import org.gudy.azureus2.core3.util.SimpleTimer;
-import org.gudy.azureus2.core3.util.SystemTime;
-import org.gudy.azureus2.core3.util.TimerEvent;
-import org.gudy.azureus2.core3.util.TimerEventPerformer;
-import org.gudy.azureus2.plugins.Plugin;
-import org.gudy.azureus2.plugins.PluginInterface;
-import org.gudy.azureus2.plugins.PluginListener;
-import org.gudy.azureus2.plugins.ddb.DistributedDatabase;
-import org.gudy.azureus2.plugins.ddb.DistributedDatabaseContact;
-import org.gudy.azureus2.plugins.ddb.DistributedDatabaseEvent;
-import org.gudy.azureus2.plugins.ddb.DistributedDatabaseKey;
-import org.gudy.azureus2.plugins.ddb.DistributedDatabaseListener;
-import org.gudy.azureus2.plugins.ddb.DistributedDatabaseValue;
+import org.gudy.azureus2.core3.download.DownloadManager;
+import org.gudy.azureus2.core3.download.DownloadManagerState;
+import org.gudy.azureus2.core3.util.*;
+import org.gudy.azureus2.core3.util.protocol.azplug.AZPluginConnection;
+import org.gudy.azureus2.core3.xml.util.XUXmlWriter;
+import org.gudy.azureus2.plugins.*;
+import org.gudy.azureus2.plugins.ddb.*;
 import org.gudy.azureus2.plugins.disk.DiskManagerFileInfo;
 import org.gudy.azureus2.plugins.download.Download;
 import org.gudy.azureus2.plugins.download.DownloadException;
+import org.gudy.azureus2.plugins.download.DownloadScrapeResult;
+import org.gudy.azureus2.plugins.ipc.IPCException;
 import org.gudy.azureus2.plugins.logging.LoggerChannel;
 import org.gudy.azureus2.plugins.messaging.MessageException;
 import org.gudy.azureus2.plugins.messaging.MessageManager;
@@ -61,14 +46,10 @@ import org.gudy.azureus2.plugins.messaging.generic.GenericMessageHandler;
 import org.gudy.azureus2.plugins.messaging.generic.GenericMessageRegistration;
 import org.gudy.azureus2.plugins.network.RateLimiter;
 import org.gudy.azureus2.plugins.torrent.Torrent;
+import org.gudy.azureus2.plugins.torrent.TorrentAttribute;
 import org.gudy.azureus2.plugins.ui.UIInstance;
 import org.gudy.azureus2.plugins.ui.UIManagerListener;
-import org.gudy.azureus2.plugins.ui.config.BooleanParameter;
-import org.gudy.azureus2.plugins.ui.config.IntParameter;
-import org.gudy.azureus2.plugins.ui.config.Parameter;
-import org.gudy.azureus2.plugins.ui.config.ParameterListener;
-import org.gudy.azureus2.plugins.ui.config.StringListParameter;
-import org.gudy.azureus2.plugins.ui.config.StringParameter;
+import org.gudy.azureus2.plugins.ui.config.*;
 import org.gudy.azureus2.plugins.ui.menus.MenuItem;
 import org.gudy.azureus2.plugins.ui.menus.MenuItemFillListener;
 import org.gudy.azureus2.plugins.ui.menus.MenuItemListener;
@@ -80,16 +61,17 @@ import org.gudy.azureus2.plugins.utils.*;
 import org.gudy.azureus2.plugins.utils.security.SEPublicKey;
 import org.gudy.azureus2.plugins.utils.security.SEPublicKeyLocator;
 import org.gudy.azureus2.plugins.utils.security.SESecurityManager;
+import org.gudy.azureus2.pluginsimpl.local.PluginCoreUtils;
 import org.gudy.azureus2.ui.swt.plugins.UISWTInstance;
 
 import com.aelitis.azureus.core.AzureusCoreFactory;
-import com.aelitis.azureus.core.security.CryptoHandler;
-import com.aelitis.azureus.core.security.CryptoManagerFactory;
-import com.aelitis.azureus.core.security.CryptoManagerKeyListener;
-import com.aelitis.azureus.core.security.CryptoManagerPasswordException;
+import com.aelitis.azureus.core.security.*;
+import com.aelitis.azureus.core.util.AZ3Functions;
 import com.aelitis.azureus.core.util.CopyOnWriteList;
 import com.aelitis.azureus.core.util.bloom.BloomFilter;
 import com.aelitis.azureus.core.util.bloom.BloomFilterFactory;
+import com.aelitis.azureus.plugins.magnet.MagnetPlugin;
+import com.aelitis.azureus.plugins.magnet.MagnetPluginProgressListener;
 import com.aelitis.azureus.plugins.net.buddy.swt.BuddyPluginView;
 import com.aelitis.azureus.plugins.net.buddy.tracker.BuddyPluginTracker;
 
@@ -106,6 +88,7 @@ BuddyPlugin
 	
 	public static final int MT_V3_CHAT		= 1;
 	
+	private static final int FEED_UPDATE_MIN_MILLIS	= 6*60*60*1000;
 	
 	public static final int MAX_MESSAGE_SIZE	= 4*1024*1024;
 	
@@ -191,6 +174,7 @@ BuddyPlugin
 	private StringParameter 		nick_name_param;
 	private StringListParameter 	online_status_param;
 	private BooleanParameter 		enable_chat_notifications; 
+	private StringParameter 		cat_pub;
 
 	private boolean			ready_to_publish;
 	private publishDetails	current_publish		= new publishDetails();
@@ -204,11 +188,14 @@ BuddyPlugin
 	
 	private CryptoHandler ecc_handler = CryptoManagerFactory.getSingleton().getECCHandler();
 
-	private List	buddies 	= new ArrayList();
-	private Map		buddies_map	= new HashMap();
+	private List<BuddyPluginBuddy>	buddies 	= new ArrayList<BuddyPluginBuddy>();
 	
-	private CopyOnWriteList		listeners 			= new CopyOnWriteList();
-	private CopyOnWriteList		request_listeners	= new CopyOnWriteList(); 
+	private List<BuddyPluginBuddy>	connected_at_close;
+	
+	private Map<String,BuddyPluginBuddy>		buddies_map	= new HashMap<String,BuddyPluginBuddy>();
+	
+	private CopyOnWriteList<BuddyPluginListener>				listeners 			= new CopyOnWriteList<BuddyPluginListener>();
+	private CopyOnWriteList<BuddyPluginBuddyRequestListener>	request_listeners	= new CopyOnWriteList<BuddyPluginBuddyRequestListener>(); 
 		
 	private SESecurityManager	sec_man;
 
@@ -219,31 +206,43 @@ BuddyPlugin
 	
 	private RateLimiter	inbound_limiter = 
 		new RateLimiter()
+	{
+		public String 
+		getName() 
 		{
-			public int 
-			getRateLimitBytesPerSecond() 
-			{
-				return( inbound_limit );
-			}
-		};
+			return( "buddy_up" );
+		}
+
+		public int 
+		getRateLimitBytesPerSecond() 
+		{
+			return( inbound_limit );
+		}
+	};
 		
 	private RateLimiter	outbound_limiter = 
 		new RateLimiter()
+	{
+		public String 
+		getName() 
 		{
-			public int 
-			getRateLimitBytesPerSecond() 
-			{
-				return( outbound_limit );
-			}
-		};
+			return( "buddy_down" );
+		}
+
+		public int 
+		getRateLimitBytesPerSecond() 
+		{
+			return( outbound_limit );
+		}
+	};
 			
 	private boolean		config_dirty;
 	
-	private Random	random = new SecureRandom();
+	private Random	random = RandomUtils.SECURE_RANDOM;
 	
 	private BuddyPluginAZ2		az2_handler;
 	
-	private List	publish_write_contacts = new ArrayList();
+	private List<DistributedDatabaseContact>	publish_write_contacts = new ArrayList<DistributedDatabaseContact>();
 	
 	private int		status_seq;
 	
@@ -254,15 +253,20 @@ BuddyPlugin
 		}
 	}
 		
-	private Set			pd_preinit		= new HashSet();
-	private List		pd_queue 		= new ArrayList();
-	private AESemaphore	pd_queue_sem	= new AESemaphore( "BuddyPlugin:persistDispatch");
-	private AEThread2	pd_thread;
+	private Set<BuddyPluginBuddy>			pd_preinit		= new HashSet<BuddyPluginBuddy>();
+	
+	private List<BuddyPluginBuddy>			pd_queue 		= new ArrayList<BuddyPluginBuddy>();
+	private AESemaphore						pd_queue_sem	= new AESemaphore( "BuddyPlugin:persistDispatch");
+	private AEThread2						pd_thread;
 	
 	private boolean		bogus_ygm_written;
 	
 	private BuddyPluginTracker	buddy_tracker;
 	
+	private TorrentAttribute	ta_category;
+
+	private Set<String>	public_categories = new HashSet<String>();
+	
 	public static void
 	load(
 		PluginInterface		plugin_interface )
@@ -277,9 +281,11 @@ BuddyPlugin
 	public void
 	initialize(
 		final PluginInterface		_plugin_interface )
-	{
+	{		
 		plugin_interface	= _plugin_interface;
 		
+		ta_category		= plugin_interface.getTorrentManager().getAttribute( TorrentAttribute.TA_CATEGORY );
+
 		az2_handler = new BuddyPluginAZ2( this );
 				
 		sec_man = plugin_interface.getUtilities().getSecurityManager();
@@ -375,6 +381,25 @@ BuddyPlugin
 		
 		enable_chat_notifications = config.addBooleanParameter2( "azbuddy.enable_chat_notif", "azbuddy.enable_chat_notif", true );
 		
+			// default published cats
+		
+		cat_pub = config.addStringParameter2( "azbuddy.enable_cat_pub", "azbuddy.enable_cat_pub", "" );
+		
+		cat_pub.setGenerateIntermediateEvents( false );
+		
+		setPublicCats( cat_pub.getValue(), false );
+		
+		cat_pub.addListener(
+			new ParameterListener()
+			{
+				public void 
+				parameterChanged(
+					Parameter 	param ) 
+				{
+					setPublicCats( cat_pub.getValue(), false);
+				}
+			});
+		
 			// config end
 		
 		
@@ -394,6 +419,8 @@ BuddyPlugin
 					MenuItem	menu,
 					Object		_target )
 				{
+					menu.removeAllChildItems();
+
 					if ( !( isEnabled() && isAvailable())){
 						
 						menu.setEnabled( false );
@@ -401,33 +428,77 @@ BuddyPlugin
 						return;
 					}
 					
-					Object	obj = null;
-					
+					final List<Torrent>	torrents = new ArrayList<Torrent>();
+								
 					if ( _target instanceof TableRow ){
 						
-						obj = ((TableRow)_target).getDataSource();
+						addDownload( torrents, (TableRow)_target );
 	
 					}else{
 						
 						TableRow[] rows = (TableRow[])_target;
 					     
-						if ( rows.length > 0 ){
-						
-							obj = rows[0].getDataSource();
+						for ( TableRow row: rows ){
+							
+							addDownload( torrents, row );
 						}
 					}
 					
-					if ( obj == null ){
+					if ( torrents.size() == 0 ){
 						
 						menu.setEnabled( false );
 
-						return;
-					}
+					}else{
 					
-					Download				download;
+						List<BuddyPluginBuddy> buddies = getBuddies();
+						
+						boolean	incomplete = ((TableContextMenuItem)menu).getTableID() == TableManager.TABLE_MYTORRENTS_INCOMPLETE;
+						
+						TableContextMenuItem parent = incomplete?menu_item_itorrents:menu_item_ctorrents;
+												
+						for (int i=0;i<buddies.size();i++){
+							
+							final BuddyPluginBuddy	buddy = (BuddyPluginBuddy)buddies.get(i);
+							
+							if ( buddy.isOnline( true )){
+								
+								TableContextMenuItem item =
+									plugin_interface.getUIManager().getTableManager().addContextMenuItem(
+										parent,
+										"!" + buddy.getName() + "!");
+								
+								item.addMultiListener(
+									new MenuItemListener()
+									{
+										public void 
+										selected(
+											MenuItem 	menu,
+											Object 		target ) 
+										{
+											for ( Torrent torrent: torrents ){
+											
+												az2_handler.sendAZ2Torrent( torrent, buddy );
+											}
+										}
+									});
+							}
+						}
+					
+						menu.setEnabled( true );
+					}
+				}
+				
+				protected void
+				addDownload(
+					List<Torrent>		torrents,
+					TableRow			row )
+				{
+					Object obj = row.getDataSource();
+					
+					Download	download;
 					
 					if ( obj instanceof Download ){
-					
+						
 						download = (Download)obj;
 						
 					}else{
@@ -447,44 +518,10 @@ BuddyPlugin
 					
 					Torrent torrent = download.getTorrent();
 					
-					boolean enabled = torrent != null && !torrent.isPrivate();
-					
-					menu.removeAllChildItems();
-
-					if ( enabled ){
-					
-						List buddies = getBuddies();
+					if ( torrent != null && !TorrentUtils.isReallyPrivate( PluginCoreUtils.unwrap( torrent ))){
 						
-						boolean	incomplete = ((TableContextMenuItem)menu).getTableID() == TableManager.TABLE_MYTORRENTS_INCOMPLETE;
-						
-						TableContextMenuItem parent = incomplete?menu_item_itorrents:menu_item_ctorrents;
-						
-						final Download f_download = download;
-						
-						for (int i=0;i<buddies.size();i++){
-							
-							final BuddyPluginBuddy	buddy = (BuddyPluginBuddy)buddies.get(i);
-							
-							TableContextMenuItem item =
-								plugin_interface.getUIManager().getTableManager().addContextMenuItem(
-									parent,
-									"!" + buddy.getName() + "!");
-							
-							item.addListener(
-								new MenuItemListener()
-								{
-									public void 
-									selected(
-										MenuItem 	menu,
-										Object 		target ) 
-									{
-										az2_handler.sendAZ2Torrent( f_download.getTorrent(), buddy );
-									}
-								});
-						}
+						torrents.add( torrent );
 					}
-					
-					menu.setEnabled( enabled );
 				}
 			};
 			
@@ -601,7 +638,7 @@ BuddyPlugin
 				}
 			});
 	}
-	
+		
 	protected void
 	updateLocale(
 		LocaleUtilities	lu )
@@ -644,8 +681,8 @@ BuddyPlugin
 						}
 
 						if (promptUserOnDisable(ui) != 0) {
-							plugin_interface.setDisabled(false);
-							COConfigurationManager.setParameter(parameterName, true);
+							plugin_interface.getPluginState().setDisabled(false);
+							plugin_interface.getPluginState().setLoadedAtStartup(true);
 						} else {
 							fireEnabledStateChanged();
 						}
@@ -772,6 +809,20 @@ BuddyPlugin
 			
 			fireInitialised( true );
 			
+				// try to re-establish connection to previously connectd buddies
+			
+			List<BuddyPluginBuddy> buddies = getBuddies();
+			
+			for ( BuddyPluginBuddy buddy: buddies ){
+				
+				if ( buddy.getIP() != null && !buddy.isConnected()){
+				
+					log( "Attempting reconnect to " + buddy.getString());
+					
+					buddy.sendKeepAlive();
+				}
+			}
+			
 		}catch( Throwable e ){
 		
 			log( "Initialisation failed", e );
@@ -783,6 +834,7 @@ BuddyPlugin
 	public boolean
 	isEnabled()
 	{
+		if (enabled_param == null) {return false;}
 		return( enabled_param.getValue());
 	}
 	
@@ -790,6 +842,7 @@ BuddyPlugin
 	setEnabled(
 		boolean		enabled )
 	{
+		if (enabled_param == null) {return;}
 		enabled_param.setValue( enabled );
 	}
 	
@@ -848,6 +901,164 @@ BuddyPlugin
 		return( enable_chat_notifications );
 	}
 	
+	protected String
+	normaliseCat(
+		String		str )
+	{
+		if ( str == null ){
+			
+			return( null );
+			
+		}else if ( str.toLowerCase().equals( "all" )){
+			
+			return( "All" );
+			
+		}else{
+			
+			return( str );
+		}
+	}
+	
+	protected void
+	normaliseCats(
+		Set<String>	cats )
+	{
+		if ( cats != null ){
+			
+			boolean	all_found = false;
+			
+			Iterator<String> it = cats.iterator();
+			
+			while( it.hasNext()){
+				
+				if ( it.next().toLowerCase().equals( "all" )){
+					
+					it.remove();
+					
+					all_found = true;
+				}
+			}
+			
+			if ( all_found ){
+				
+				cats.add( "All" );
+			}
+		}
+	}
+	
+	public boolean
+	isPublicCategory(
+		String	cat )
+	{
+		cat = normaliseCat( cat );
+		
+		return( public_categories.contains( cat ));
+	}
+	
+	public void
+	addPublicCategory(
+		String	cat )
+	{
+		cat = normaliseCat( cat );
+		
+		Set<String> new_cats = new HashSet( public_categories );
+		
+		if ( new_cats.add( cat )){
+		
+			setPublicCats( new_cats, true );
+		}
+	}
+	
+	public void
+	removePublicCategory(
+		String	cat )
+	{
+		cat = normaliseCat( cat );
+		
+		Set<String> new_cats = new HashSet( public_categories );
+		
+		if ( new_cats.remove( cat )){
+		
+			setPublicCats( new_cats, true );
+		}
+	}
+	
+	protected void
+	setPublicCats(
+		String	str,
+		boolean	persist )
+	{
+		Set<String>	new_pub_cats = new HashSet<String>();
+		
+		String[]	bits = str.split(",");
+		
+		for (String s: bits ){
+			
+			s = s.trim();
+			
+			if ( bits.length > 0 ){
+				
+				new_pub_cats.add( normaliseCat( s ));
+			}
+		}
+		
+		setPublicCats( new_pub_cats, persist );
+	}
+	
+	protected void
+	setPublicCats(
+		Set<String>	new_pub_cats,
+		boolean		persist )
+	{
+		if ( !public_categories.equals( new_pub_cats )){
+			
+			Set<String> removed = new HashSet<String>( public_categories );
+			
+			removed.removeAll( new_pub_cats );
+			
+			public_categories = new_pub_cats;
+			
+			if ( persist ){
+				
+				String cat_str = "";
+				
+				for ( String s: public_categories ){
+					
+					cat_str += (cat_str.length()==0?"":",") + s;
+				}
+				
+				cat_pub.setValue( cat_str );
+			}
+			
+			List<BuddyPluginBuddy> buds = getBuddies();
+			
+			for ( BuddyPluginBuddy b: buds ){
+				
+				Set<String> local = b.getLocalAuthorisedRSSCategories();
+				
+				if ( local != null || new_pub_cats.size() > 0 ){
+					
+					if ( local == null ){
+						
+						local = new HashSet<String>();
+						
+					}else{
+						
+							// gotta clone else we're messing with stuff that ain't ours
+						
+						local = new HashSet<String>( local );
+					}
+					
+					local.addAll( new_pub_cats );
+				
+					local.removeAll( removed );
+				
+					b.setLocalAuthorisedRSSCategories( local );
+				}
+			}
+		}
+	}
+	
 	protected void
 	registerMessageHandler()
 	{
@@ -1558,7 +1769,7 @@ BuddyPlugin
 				ddb.write(
 					new DistributedDatabaseListener()
 					{
-						private List	write_contacts = new ArrayList();
+						private List<DistributedDatabaseContact>	write_contacts = new ArrayList<DistributedDatabaseContact>();
 						
 						public void
 						event(
@@ -1654,11 +1865,24 @@ BuddyPlugin
 	{
 		logMessage( "Closing down" );
 
+		List<BuddyPluginBuddy>	buddies = getAllBuddies();
+		
+		synchronized( this ){
+			
+			connected_at_close = new ArrayList<BuddyPluginBuddy>();
+			
+			for ( BuddyPluginBuddy buddy: buddies ){
+				
+				if ( buddy.isConnected()){
+					
+					connected_at_close.add( buddy );
+				}
+			}
+		}
+		
 		if ( ddb != null ){
 			
-			boolean	restarting = AzureusCoreFactory.getSingleton().isRestarting();
-		
-			List	buddies = getAllBuddies();
+			boolean	restarting = AzureusCoreFactory.isCoreAvailable() ? AzureusCoreFactory.getSingleton().isRestarting() : false;
 			
 			logMessage( "   closing buddy connections" );
 			
@@ -1813,8 +2037,27 @@ BuddyPlugin
 						Long l_ver = (Long)details.get("v");
 						
 						int	ver = l_ver==null?VERSION_INITIAL:l_ver.intValue();
+													
+						String	loc_cat = decodeString((byte[])details.get( "lc" ));
+						String	rem_cat = decodeString((byte[])details.get( "rc" ));
+						
+						BuddyPluginBuddy buddy = new BuddyPluginBuddy( this, subsystem, true, key, nick, ver, loc_cat, rem_cat, last_seq, last_time_online, recent_ygm );
+						
+						byte[]	ip_bytes = (byte[])details.get( "ip" );
+						
+						if ( ip_bytes != null ){
+							
+							try{
+								InetAddress ip = InetAddress.getByAddress( ip_bytes );
 								
-						BuddyPluginBuddy buddy = new BuddyPluginBuddy( this, subsystem, true, key, nick, ver, last_seq, last_time_online, recent_ygm );
+								int	tcp_port = ((Long)details.get( "tcp" )).intValue();
+								int	udp_port = ((Long)details.get( "udp" )).intValue();
+								
+								buddy.setCachedStatus( ip, tcp_port, udp_port );
+								
+							}catch( Throwable e ){
+							}
+						}
 						
 						logMessage( "Loaded buddy " + buddy.getString());
 						
@@ -1896,6 +2139,32 @@ BuddyPlugin
 					
 					map.put( "v", new Long( buddy.getVersion()));
 					
+					if ( buddy.getLocalAuthorisedRSSCategoriesAsString() != null ){
+						map.put( "lc", buddy.getLocalAuthorisedRSSCategoriesAsString());
+					}
+					
+					if ( buddy.getRemoteAuthorisedRSSCategoriesAsString() != null ){
+						map.put( "rc", buddy.getRemoteAuthorisedRSSCategoriesAsString());
+					}
+
+					boolean connected = 
+						buddy.isConnected() ||
+						( connected_at_close != null && connected_at_close.contains( buddy ));
+					
+					if ( connected ){
+						
+						InetAddress	ip 			= buddy.getIP();
+						int			tcp_port	= buddy.getTCPPort();
+						int			udp_port	= buddy.getUDPPort();
+						
+						if ( ip != null ){
+							
+							map.put( "ip", ip.getAddress());
+							map.put( "tcp", new Long( tcp_port ));
+							map.put( "udp", new Long( udp_port ));
+						}
+					}
+					
 					buddies_config.add( map );
 				}
 				
@@ -1971,7 +2240,7 @@ BuddyPlugin
 			if ( buddy_to_return == null ){
 				
 				buddy_to_return = 
-					new BuddyPluginBuddy( this, subsystem, authorised, key, null, VERSION_INITIAL, 0, 0, null );
+					new BuddyPluginBuddy( this, subsystem, authorised, key, null, VERSION_CURRENT, null, null, 0, 0, null );
 				
 				buddies.add( buddy_to_return );
 				
@@ -2828,12 +3097,12 @@ BuddyPlugin
 		 * Returns authorised buddies only
 		 */
 	
-	public List
+	public List<BuddyPluginBuddy>
 	getBuddies()
 	{
 		synchronized( this ){
 			
-			List	result = new ArrayList();
+			List<BuddyPluginBuddy>	result = new ArrayList<BuddyPluginBuddy>();
 			
 			for (int i=0;i<buddies.size();i++){
 				
@@ -2849,12 +3118,12 @@ BuddyPlugin
 		}
 	}
 	
-	protected List
+	protected List<BuddyPluginBuddy>
 	getAllBuddies()
 	{
 		synchronized( this ){
 			
-			return( new ArrayList( buddies ));
+			return( new ArrayList<BuddyPluginBuddy>( buddies ));
 		}
 	}
 	
@@ -2986,6 +3255,8 @@ BuddyPlugin
    	{
 		if ( buddy.isAuthorised()){
 
+			buddy.setLocalAuthorisedRSSCategories( public_categories );
+			
 	   		List	 listeners_ref = listeners.getList();
 	   		
 	   		for (int i=0;i<listeners_ref.size();i++){
@@ -3064,7 +3335,7 @@ BuddyPlugin
 	protected void
  	fireEnabledStateChanged()
  	{
-		final boolean enabled = !plugin_interface.isDisabled() && isEnabled();
+		final boolean enabled = !plugin_interface.getPluginState().isDisabled() && isEnabled();
 
  		List	 listeners_ref = listeners.getList();
  		
@@ -3100,6 +3371,704 @@ BuddyPlugin
 		}
 	}
 	
+
+	public InputStream
+	handleURLProtocol(
+		 AZPluginConnection			connection,
+		String						arg_str )
+	
+		throws IPCException
+	{
+		String[]	args = arg_str.split( "&" );
+		
+		String		pk 			= null;
+		String		category	= "All";
+		byte[]		hash		= null;
+		
+		for (String arg: args ){
+			
+			String[]	bits = arg.split( "=" );
+			
+			String	lhs = bits[0];
+			String	rhs	= UrlUtils.decode( bits[1] );
+			
+			if ( lhs.equals( "pk" )){
+				
+				pk		= rhs;
+				
+			}else if ( lhs.equals( "cat" )){
+				
+				category = rhs;
+				
+			}else if ( lhs.equals( "hash" )){
+				
+				hash	= Base32.decode(rhs);
+			}
+		}
+		
+		if ( pk == null ){
+			
+			throw( new IPCException( "Public key missing from '" + arg_str + "'" ));
+		}
+		
+		BuddyPluginBuddy	buddy	= getBuddyFromPublicKey( pk );
+
+		if ( buddy == null ){
+			
+			throw( new IPCException( "Buddy with public key '" + pk + "' not found" ));
+		}
+		
+		if ( hash == null ){
+			
+			return( handleUPRSS( connection, buddy, category ));
+			
+		}else{
+			
+			return( handleUPTorrent( connection, buddy, category, hash ));
+		}
+	}
+	
+	public InputStream
+	handleUPRSS(
+		final AZPluginConnection	connection,
+		BuddyPluginBuddy			buddy,
+		String						category )
+	
+		throws IPCException
+	{
+		if ( !buddy.isOnline( true )){
+			
+			throw( new IPCException( "Buddy isn't online" ));
+		}
+		
+		Map<String,Object>	msg = new HashMap<String, Object>();
+
+		final String if_mod 	= connection.getRequestProperty( "If-Modified-Since" );
+
+		try{
+			msg.put( "cat", category.getBytes( "UTF-8" ));
+									
+			if ( if_mod != null ){
+				
+				msg.put( "if_mod", if_mod );
+			}
+			
+			// String etag		= connection.getRequestProperty( "If-None-Match" );
+					
+		}catch( Throwable e ){
+			
+			Debug.out( e );
+		}
+		
+		final Object[] 		result 		= { null };
+		final AESemaphore	result_sem 	= new AESemaphore( "BuddyPlugin:rss" );
+				
+		final String	etag = buddy.getPublicKey() + "-" + category;
+		
+		az2_handler.sendAZ2RSSMessage( 
+			buddy,
+			msg,
+			new BuddyPluginAZ2TrackerListener()
+			{
+				public Map
+				messageReceived(
+					BuddyPluginBuddy	buddy,
+					Map					message )
+				{
+					try{						
+						byte[] bytes = (byte[])message.get( "rss" );
+					
+						ByteArrayInputStream bais = new ByteArrayInputStream(bytes);
+						
+						result[0] = bais;
+						
+						connection.setHeaderField( "ETag", etag );
+						
+						byte[] b_last_mod = (byte[])message.get( "last_mod" );
+						
+						if ( b_last_mod != null ){
+						
+							String	last_mod = new String( b_last_mod, "UTF-8" );
+							
+							connection.setHeaderField( "Last-Modified", last_mod );
+							
+							if ( if_mod != null && if_mod.equals( last_mod ) && bytes.length == 0 ){
+								
+								connection.setResponse( HttpURLConnection.HTTP_NOT_MODIFIED, "Not Modified" );
+							}
+						}
+						
+						result_sem.release();
+						
+					}catch( Throwable e ){
+					
+						messageFailed( buddy, e );
+					}
+					
+					return( null );
+				}
+				
+				public void
+				messageFailed(
+					BuddyPluginBuddy	buddy,
+					Throwable			cause )
+				{					
+					result[0] = new IPCException( "Read failed", cause );
+					
+					result_sem.release();
+				}
+			});
+		
+		result_sem.reserve( 60*1000 );
+		
+		if ( result[0] == null ){
+			
+			throw( new IPCException( "Timeout" ));
+			
+		}else if ( result[0] instanceof InputStream ){
+			
+			return((InputStream)result[0]);
+			
+		}else{
+			
+			throw((IPCException)result[0]);
+		}
+	}
+
+	public InputStream
+	handleUPTorrent(
+		final AZPluginConnection	connection,
+		final BuddyPluginBuddy		buddy,
+		String						category,
+		final byte[]				hash )
+	
+		throws IPCException
+	{
+		final long timeout = 120*1000;
+
+		final Object[] 		result 		= { null };
+		final AESemaphore	result_sem 	= new AESemaphore( "BuddyPlugin:upt" );
+			
+		log( "Attempting to download torrent for " + Base32.encode( hash ));
+		
+			// first try and get torrent direct from the buddy
+		
+		if ( buddy.isOnline( true )){
+
+			try{
+				
+				Map<String,Object>	msg = new HashMap<String, Object>();
+			
+				try{
+					msg.put( "cat", category.getBytes( "UTF-8" ));
+									
+					msg.put( "hash", hash );
+							
+				}catch( Throwable e ){
+					
+					Debug.out( e );
+				}
+							
+				az2_handler.sendAZ2RSSMessage( 
+					buddy,
+					msg,
+					new BuddyPluginAZ2TrackerListener()
+					{
+						private boolean	result_set;
+					
+						public Map
+						messageReceived(
+							BuddyPluginBuddy	buddy,
+							Map					message )
+						{
+							try{						
+								byte[] bytes = (byte[])message.get( "torrent" );
+									
+								log( "    torrent downloaded from buddy" );
+								
+								setResult( bytes );
+																
+							}catch( Throwable e ){
+							
+								messageFailed( buddy, e );
+							}
+							
+							return( null );
+						}
+						
+						public void
+						messageFailed(
+							BuddyPluginBuddy	buddy,
+							Throwable			cause )
+						{					
+							setResult( new IPCException( "Read failed", cause ));
+						}
+						
+						protected void
+						setResult(
+							Object		obj )
+						{
+							synchronized( result ){
+								
+								if ( result_set ){
+									
+									return;
+								}
+								
+								result_set = true;
+								
+								if ( !( result[0] instanceof byte[] )){
+								
+									result[0] = obj;
+								}
+								
+								result_sem.release();
+							}
+						}
+					});
+			}catch( Throwable e ){
+				
+				result[0] = new IPCException( "Buddy torrent get failed", e );
+				
+				result_sem.release();
+			}
+		}else{
+			
+			result[0] = new IPCException( "Buddy is offline" );
+
+			result_sem.release();
+		}
+		
+			// second try and get via magnet
+		
+		final MagnetPlugin	magnet_plugin = getMagnetPlugin();
+		
+		if ( magnet_plugin == null ){
+		
+			synchronized( result ){
+				
+				if ( result[0] == null ){
+					
+					result[0] = new IPCException( "Magnet plugin unavailable" );
+				}
+			}
+				
+			result_sem.release();
+			
+		}else{
+
+			new AEThread2( "BuddyPlugin:mag", true )
+			{
+				private boolean result_set;
+				
+				public void
+				run()
+				{
+					try{
+						
+						if ( buddy.isOnline( true )){
+							
+							Thread.sleep(10*1000);
+						}
+						
+						synchronized( result ){
+
+							if ( result[0] instanceof byte[] ){
+								
+								setResult( null );
+								
+								return;
+							}
+						}
+						
+						byte[] torrent_data = magnet_plugin.download(
+							new MagnetPluginProgressListener()
+							{
+								public void
+								reportSize(
+									long	size )
+								{
+								}
+								
+								public void
+								reportActivity(
+									String	str )
+								{
+									if ( 	str.indexOf( " found " ) == -1 &&
+											str.indexOf( " is dead " ) == -1 ){
+						
+										log( "    MagnetDownload: " + str );
+									}
+								}
+								
+								public void
+								reportCompleteness(
+									int		percent )
+								{
+								}
+								
+								public void
+								reportContributor(
+									InetSocketAddress	address )
+								{
+								}
+							},
+							hash,
+							new InetSocketAddress[0],
+							timeout );
+						
+						if ( torrent_data == null ){
+							
+							setResult( new IPCException( "Magnet timeout" ));
+													
+						}else{					
+							
+							log( "    torrent downloaded from magnet" );
+
+							setResult( torrent_data );
+						}
+					}catch( Throwable e ){
+						
+						setResult( new IPCException( "Magnet get failed", e ));
+					}
+				}
+				
+				protected void
+				setResult(
+					Object		obj )
+				{
+					synchronized( result ){
+						
+						if ( result_set ){
+							
+							return;
+						}
+						
+						result_set = true;
+						
+						if ( obj != null ){
+							
+							if ( 	result[0] == null ||
+									( obj instanceof byte[] && !( result[0] instanceof byte[] ))){
+								
+								result[0] = obj;
+							}
+						}
+							
+						result_sem.release();
+					}
+				}
+			}.start();
+		}
+		
+		long	start = SystemTime.getMonotonousTime();
+				
+		if ( result_sem.reserve(timeout )){
+			
+			if ( !( result[0] instanceof byte[] )){
+		
+				long	rem = timeout - ( SystemTime.getMonotonousTime() - start );
+				
+				if ( rem > 0 ){
+				
+					result_sem.reserve(rem );
+				}
+			}
+		}
+		
+		if ( result[0] == null ){
+			
+			log( "    torrent download timeout" );
+
+			throw( new IPCException( "Timeout" ));
+			
+		}else if ( result[0] instanceof byte[] ){
+						
+			return( new ByteArrayInputStream((byte[])result[0]));
+			
+		}else{
+			
+			IPCException error = (IPCException)result[0];
+			
+			log( "    torrent downloaded failed: " + Debug.getNestedExceptionMessage( error ));
+
+			throw( error );
+		}
+	}
+
+	protected MagnetPlugin
+	getMagnetPlugin()
+	{
+		PluginInterface  pi  = AzureusCoreFactory.getSingleton().getPluginManager().getPluginInterfaceByClass( MagnetPlugin.class );
+	
+		if ( pi == null ){
+			
+			return( null );
+		}
+		
+		return((MagnetPlugin)pi.getPlugin());
+	}
+	
+	public feedDetails
+	getRSS(
+		BuddyPluginBuddy		buddy,
+		String					category,
+		String					if_mod )
+	
+		throws BuddyPluginException
+	{
+		if ( !buddy.isLocalRSSCategoryAuthorised( category )){
+			
+			throw( new BuddyPluginException( "Unauthorised category '" + category + "'" ));
+		}
+		
+		buddy.localRSSCategoryRead( category );
+		
+		Download[] downloads = plugin_interface.getDownloadManager().getDownloads();
+		
+		List<Download>	selected_dls = new ArrayList<Download>();
+		
+		long	fingerprint	= 0;
+		
+		for (int i=0;i<downloads.length;i++){
+			
+			Download download = downloads[i];
+			
+			Torrent torrent = download.getTorrent();
+			
+			if ( torrent == null ){
+				
+				continue;
+			}
+			
+			String dl_cat = download.getAttribute( ta_category );
+		
+			if ( 	category.equalsIgnoreCase( "all" ) ||
+					( dl_cat != null && dl_cat.equals( category ))){
+				
+				if ( !TorrentUtils.isReallyPrivate( PluginCoreUtils.unwrap( torrent ))){
+					
+					selected_dls.add( download );
+					
+					byte[] hash = torrent.getHash();
+					
+					int	num = (hash[0]<<24)&0xff000000 | (hash[1] << 16)&0x00ff0000 | (hash[2] << 8)&0x0000ff00 | hash[3]&0x000000ff;
+
+					fingerprint += num;
+				}
+			}
+		}
+		
+		PluginConfig pc = plugin_interface.getPluginconfig();
+
+		String	feed_finger_key = "feed_finger.category." + category;
+		String	feed_date_key 	= "feed_date.category." + category;
+
+		long	existing_fingerprint 	= pc.getPluginLongParameter( feed_finger_key, 0 );
+		long	feed_date 				= pc.getPluginLongParameter( feed_date_key, 0 );
+
+		long	now = SystemTime.getCurrentTime();
+
+		if ( existing_fingerprint == fingerprint ){
+			
+				// update every now and then to pick up new peer/seed values
+			
+			if ( selected_dls.size() > 0 ){
+
+				if ( 	now < feed_date ||
+						now - feed_date > FEED_UPDATE_MIN_MILLIS ){
+					
+					feed_date = now;
+				
+					pc.setPluginParameter( feed_date_key, feed_date );
+				}
+			}
+		}else{
+			
+			pc.setPluginParameter( feed_finger_key, fingerprint );
+			
+			
+				// ensure feed date goes up
+			
+			if ( now <= feed_date ){
+				
+				feed_date++;
+				
+			}else{
+				
+				feed_date = now;
+			}
+			
+			pc.setPluginParameter( feed_date_key, feed_date );
+		}
+		
+		String last_modified = TimeFormatter.getHTTPDate( feed_date );
+
+		if ( if_mod != null && if_mod.equals( last_modified )){
+			
+			return( new feedDetails( new byte[0], last_modified ));
+		}
+				
+		ByteArrayOutputStream	os = new ByteArrayOutputStream();
+			
+		AZ3Functions.provider az3 = AZ3Functions.getProvider();
+		
+		try{
+			PrintWriter pw = new PrintWriter(new OutputStreamWriter( os, "UTF-8" ));
+			
+			pw.println( "<?xml version=\"1.0\" encoding=\"utf-8\"?>" );
+			
+			pw.println( "<rss version=\"2.0\" xmlns:vuze=\"http://www.vuze.com\">" );
+			
+			pw.println( "<channel>" );
+			
+			pw.println( "<title>" + escape( category ) + "</title>" );
+			
+			Collections.sort(
+				selected_dls,
+				new Comparator<Download>()
+				{
+					public int 
+					compare(
+						Download d1, 
+						Download d2) 
+					{
+						long	added1 = getAddedTime( d1 )/1000;
+						long	added2 = getAddedTime( d2 )/1000;
+		
+						return((int)(added2 - added1 ));
+					}
+				});
+								
+							
+			pw.println(	"<pubDate>" + last_modified + "</pubDate>" );
+		
+			for (int i=0;i<selected_dls.size();i++){
+				
+				Download download = (Download)selected_dls.get( i );
+				
+				DownloadManager	core_download = PluginCoreUtils.unwrap( download );
+				
+				Torrent torrent = download.getTorrent();
+				
+				String	hash_str = Base32.encode( torrent.getHash());
+				
+				pw.println( "<item>" );
+				
+				pw.println( "<title>" + escape( download.getName()) + "</title>" );
+				
+				pw.println( "<guid>" + hash_str + "</guid>" );
+				
+				if ( az3 != null ){
+					
+					String cdp = az3.getCDPURL( core_download );
+					
+					if ( cdp != null ){
+						
+						pw.println( "<link>" + escape(cdp) + "</link>" );
+					}
+				}
+				
+				long added = core_download.getDownloadState().getLongParameter(DownloadManagerState.PARAM_DOWNLOAD_ADDED_TIME);
+				
+				pw.println(	"<pubDate>" + TimeFormatter.getHTTPDate( added ) + "</pubDate>" );
+				
+				pw.println(	"<vuze:size>" + torrent.getSize()+ "</vuze:size>" );
+				pw.println(	"<vuze:assethash>" + hash_str + "</vuze:assethash>" );
+				
+				String url = "azplug:?id=azbuddy&name=Friends&arg=";
+				
+				String arg = "pk=" + getPublicKey() + "&cat=" + category + "&hash=" + Base32.encode(torrent.getHash());
+
+				url += URLEncoder.encode( arg, "UTF-8" );
+			
+				pw.println( "<vuze:downloadurl>" + escape( url ) + "</vuze:downloadurl>" );
+		
+				DownloadScrapeResult scrape = download.getLastScrapeResult();
+				
+				if ( scrape != null && scrape.getResponseType() == DownloadScrapeResult.RT_SUCCESS ){
+					
+					pw.println(	"<vuze:seeds>" + scrape.getSeedCount() + "</vuze:seeds>" );
+					pw.println(	"<vuze:peers>" + scrape.getNonSeedCount() + "</vuze:peers>" );
+				}
+				
+				pw.println( "</item>" );
+			}
+			
+			pw.println( "</channel>" );
+			
+			pw.println( "</rss>" );
+		
+			pw.flush();
+			
+			return( new feedDetails( os.toByteArray(), last_modified ));
+			
+		}catch( IOException e ){
+			
+			throw( new BuddyPluginException( "", e ));
+		}
+	}
+	
+	public byte[]
+	getRSSTorrent(
+		BuddyPluginBuddy		buddy,
+		String					category,
+		byte[]					hash )
+	
+		throws BuddyPluginException
+	{
+		if ( !buddy.isLocalRSSCategoryAuthorised( category )){
+			
+			throw( new BuddyPluginException( "Unauthorised category '" + category + "'" ));
+		}
+		
+		try{
+			Download download = plugin_interface.getDownloadManager().getDownload( hash );
+			
+			if ( download != null ){
+				
+				Torrent	torrent = download.getTorrent();
+			
+				if ( torrent != null ){
+								
+					String dl_cat = download.getAttribute( ta_category );
+					
+					if ( 	category.equalsIgnoreCase( "all" ) ||
+							( dl_cat != null && dl_cat.equals( category ))){
+						
+						if ( !TorrentUtils.isReallyPrivate( PluginCoreUtils.unwrap( torrent ))){
+							
+							torrent = torrent.removeAdditionalProperties();
+							
+							return( torrent.writeToBEncodedData());
+						}
+					}
+				}
+			}
+		}catch( Throwable e ){
+			
+			throw( new BuddyPluginException( "getTorrent failed", e ));
+		}
+		
+		throw( new BuddyPluginException( "Not found" ));
+	}		
+
+	protected long
+	getAddedTime(
+		Download	download )
+	{
+		DownloadManager	core_download = PluginCoreUtils.unwrap( download );
+		
+		return( core_download.getDownloadState().getLongParameter(DownloadManagerState.PARAM_DOWNLOAD_ADDED_TIME));
+	}
+	
+	protected String
+	escape(
+		String	str )
+	{
+		return( XUXmlWriter.escapeXML(str));
+	}
+	
 	public void
 	addRequestListener(
 		BuddyPluginBuddyRequestListener	listener )
@@ -3337,4 +4306,32 @@ BuddyPlugin
 		public byte[]
 		getPayload();
 	}
+	
+	protected class
+	feedDetails
+	{
+		private byte[]		contents;
+		private String		last_modified;
+		
+		protected
+		feedDetails(
+			byte[]		_contents,
+			String		_last_modified )
+		{
+			contents		= _contents;
+			last_modified	= _last_modified;
+		}
+		
+		protected byte[]
+		getContent()
+		{
+			return( contents );
+		}
+		
+		protected String
+		getLastModified()
+		{
+			return( last_modified );
+		}
+	}
 }
