@@ -24,9 +24,12 @@ package com.aelitis.azureus.plugins.net.buddy;
 import java.io.File;
 import java.net.InetAddress;
 import java.net.InetSocketAddress;
+import java.net.URL;
+import java.net.URLEncoder;
 import java.util.*;
 
 import org.gudy.azureus2.core3.util.AERunnable;
+import org.gudy.azureus2.core3.util.AESemaphore;
 import org.gudy.azureus2.core3.util.AEThread2;
 import org.gudy.azureus2.core3.util.AddressUtils;
 import org.gudy.azureus2.core3.util.BDecoder;
@@ -46,6 +49,8 @@ import org.gudy.azureus2.plugins.utils.security.SEPublicKey;
 import org.gudy.azureus2.plugins.utils.security.SEPublicKeyLocator;
 import org.gudy.azureus2.plugins.utils.security.SESecurityManager;
 
+import com.aelitis.azureus.core.util.AZ3Functions;
+
 
 public class 
 BuddyPluginBuddy 
@@ -69,7 +74,7 @@ BuddyPluginBuddy
 	private boolean			authorised;
 	private String			public_key;
 	private String			nick_name;
-	private List			recent_ygm;
+	private List<Long>		recent_ygm;
 	
 	private int				last_status_seq;
 	
@@ -79,7 +84,7 @@ BuddyPluginBuddy
 	private int				udp_port;
 	private int				online_status	= BuddyPlugin.STATUS_ONLINE;	// default
 		
-	private int				version		= BuddyPlugin.VERSION_INITIAL;
+	private int				version		= BuddyPlugin.VERSION_CHAT;	// assume everyone now supports chat
 	
 	private boolean			online;
 	private long			last_time_online;
@@ -89,8 +94,8 @@ BuddyPluginBuddy
 	
 	private boolean			check_active;
 		
-	private List						connections	= new ArrayList();
-	private List						messages	= new ArrayList();
+	private List<buddyConnection>		connections	= new ArrayList<buddyConnection>();
+	private List<buddyMessage>			messages	= new ArrayList<buddyMessage>();
 	private buddyMessage				current_message;
 	
 	private int	next_connection_id;
@@ -102,7 +107,7 @@ BuddyPluginBuddy
 	private long 	latest_ygm_time;
 	private String	last_message_received;
 	
-	private Set		offline_seq_set;
+	private Set<Long>		offline_seq_set;
 	
 	private int		message_out_count;
 	private int		message_in_count;
@@ -113,12 +118,22 @@ BuddyPluginBuddy
 	
 	private BuddyPluginBuddyMessageHandler		persistent_msg_handler;
 
-	private Map			user_data = new LightHashMap();
+	private Map<Object,Object>		user_data = new LightHashMap<Object,Object>();
 	
 	private boolean	keep_alive_outstanding;
 	private volatile long	last_connect_attempt	= SystemTime.getCurrentTime();
 	private volatile int	consec_connect_fails;
 	
+	private long last_auto_reconnect	= -1;
+	
+	private Object				rss_lock = new Object();
+	
+	private Set<String>			rss_local_cats;
+	private Set<String>			rss_remote_cats;
+	private Set<String>			rss_cats_read;
+	
+	private AESemaphore			outgoing_connect_sem = new AESemaphore( "BPB:outcon", 1 );
+	
 	private volatile boolean	closing;
 	private volatile boolean	destroyed;
 	
@@ -130,16 +145,20 @@ BuddyPluginBuddy
 		String		_pk,
 		String		_nick_name,
 		int			_version,
+		String		_rss_local_cats,
+		String		_rss_remote_cats,
 		int			_last_status_seq,
 		long		_last_time_online,
-		List		_recent_ygm )
+		List<Long>	_recent_ygm )
 	{
 		plugin				= _plugin;
 		subsystem			= _subsystem;
 		authorised			= _authorised;
 		public_key 			= _pk;
 		nick_name			= _nick_name;
-		version				= _version;
+		version				= Math.max( version, _version );
+		rss_local_cats		= stringToCats( _rss_local_cats );
+		rss_remote_cats		= stringToCats( _rss_remote_cats );
 		last_status_seq		= _last_status_seq;
 		last_time_online	= _last_time_online;
 		recent_ygm			= _recent_ygm;
@@ -260,6 +279,366 @@ BuddyPluginBuddy
 		}
 	}
 	
+	public String
+	getLocalAuthorisedRSSCategoriesAsString()
+	{
+		synchronized( rss_lock ){
+		
+			return( catsToString( rss_local_cats ));
+		}
+	}
+	
+	public Set<String>
+  	getLocalAuthorisedRSSCategories()
+  	{
+		synchronized( rss_lock ){
+  		
+			return( rss_local_cats );
+		}
+  	}
+	
+	public void
+	addLocalAuthorisedRSSCategory(
+		String	category )
+	{
+		category = plugin.normaliseCat( category );
+	
+		boolean dirty;
+		
+		synchronized( rss_lock ){
+			
+			if ( rss_local_cats == null ){
+				
+				rss_local_cats = new HashSet<String>();
+			}
+			
+			if ( dirty = !rss_local_cats.contains( category )){
+							
+				rss_local_cats.add( category );
+			}
+		}
+		
+		if ( dirty ){
+			
+			plugin.setConfigDirty();
+			
+			plugin.fireDetailsChanged( this );
+			
+				// tell buddy of change
+			
+			if ( isConnected()){
+				
+				sendKeepAlive();
+			}
+		}
+	}
+	
+	public void
+	removeLocalAuthorisedRSSCategory(
+		String	category )
+	{
+		category = plugin.normaliseCat( category );
+		
+		boolean	dirty;
+		
+		synchronized( rss_lock ){
+			
+			if ( rss_local_cats == null ){
+			
+				return;
+			
+			}else{
+		
+				dirty = rss_local_cats.remove( category );
+			}
+		}
+		
+		if ( dirty ){
+			
+			plugin.setConfigDirty();
+		
+			plugin.fireDetailsChanged( this );
+			
+				// tell buddy of change
+			
+			if ( isConnected()){
+				
+				sendKeepAlive();
+			}
+		}
+	}
+	
+	public void
+	setLocalAuthorisedRSSCategories(
+		String			new_cats )
+	{
+		setLocalAuthorisedRSSCategories( stringToCats( new_cats ));
+	}
+	
+	public void
+	setLocalAuthorisedRSSCategories(
+		Set<String>		new_cats )
+	{	
+		plugin.normaliseCats( new_cats );
+		
+		boolean dirty;
+		
+		synchronized( rss_lock ){
+			
+			if ( dirty = !catsIdentical( new_cats, rss_local_cats) ){
+			
+				rss_local_cats = new_cats;
+			}
+		}
+		
+		if ( dirty ){
+			
+			plugin.setConfigDirty();
+			
+			plugin.fireDetailsChanged( this );
+			
+				// tell buddy of change
+			
+			if ( isConnected()){
+				
+				sendKeepAlive();
+			}
+		}	
+	}
+	
+	public Set<String>
+  	getRemoteAuthorisedRSSCategories()
+  	{
+  		return( rss_remote_cats );
+  	}
+	
+	public String
+	getRemoteAuthorisedRSSCategoriesAsString()
+	{
+		return( catsToString( rss_remote_cats ));
+	}
+	
+	protected void
+	setRemoteAuthorisedRSSCategories(
+		Set<String>		new_cats )
+	{
+		plugin.normaliseCats( new_cats );
+		
+		boolean	dirty;
+		
+		synchronized( rss_lock ){
+			
+			if ( dirty = !catsIdentical( new_cats, rss_remote_cats) ){
+			
+				rss_remote_cats = new_cats;
+			}
+		}
+		
+		if ( dirty ){
+			
+			plugin.setConfigDirty();
+			 
+			plugin.fireDetailsChanged( this );
+		}
+	}
+	
+	public boolean
+	isLocalRSSCategoryAuthorised(
+		String	category )
+	{
+		category = plugin.normaliseCat( category );
+	
+		synchronized( rss_lock ){
+			
+			if ( rss_local_cats != null ){
+			
+				return( rss_local_cats.contains( category ));
+			}
+		
+			return( false );
+		}
+	}
+	
+	public boolean
+	isRemoteRSSCategoryAuthorised(
+		String	category )
+	{
+		category = plugin.normaliseCat( category );
+		
+		synchronized( rss_lock ){
+			
+			if ( rss_remote_cats != null ){
+			
+				return( rss_remote_cats.contains( category ));
+			}
+			
+			return( false );
+		}
+	}
+	
+	protected void
+	localRSSCategoryRead(
+		String		str )
+	{
+		boolean dirty;
+		
+		synchronized( rss_lock ){
+			
+			if ( rss_cats_read == null ){
+				
+				rss_cats_read = new HashSet<String>();
+			}
+			
+			dirty = rss_cats_read.add( str );
+		}
+		
+		if ( dirty ){
+			
+			// not persisted currently - plugin.setConfigDirty();
+			 
+			plugin.fireDetailsChanged( this );
+		}
+	}
+	
+	public String
+	getLocalReadCategoriesAsString()
+	{
+		synchronized( rss_lock ){
+
+			return( catsToString( rss_cats_read ));
+		}
+	}
+	
+	public URL
+	getSubscriptionURL(
+		String		cat )
+	{
+		String url = "azplug:?id=azbuddy&name=Friends&arg=";
+		
+		String arg = "pk=" + getPublicKey() + "&cat=" + cat;
+
+		try{
+			url += URLEncoder.encode( arg, "UTF-8" );
+			
+			return( new URL( url ));
+			
+		}catch( Throwable e ){
+	
+			Debug.out( e );
+			
+			return( null );
+		}
+	}
+	
+	public void
+	subscribeToCategory(
+		String	cat )
+	
+		throws BuddyPluginException
+	{
+		AZ3Functions.provider az3 = AZ3Functions.getProvider();
+
+		if ( az3 == null ){
+			
+			throw( new BuddyPluginException( "AZ3 subsystem not available" ));
+		}
+		
+		try{
+			az3.subscribeToRSS( 
+				getName() + ": " + cat, 
+				getSubscriptionURL(cat),
+				15,
+				false,
+				getPublicKey() + ":" + cat );
+				
+		}catch( Throwable e ){
+				
+			throw( new BuddyPluginException( "Failed to add subscription", e ));
+		}
+	}
+	
+	public boolean
+	isSubscribedToCategory(
+		String	cat,
+		String	creator_ref )
+	{
+		if ( creator_ref == null ){
+			
+			return( false );
+		}
+		
+		return( creator_ref.equals( getPublicKey() + ":" + cat ));
+	}
+	
+	protected String
+	catsToString(
+		Set<String>	cats )
+	{
+		if ( cats == null || cats.size() == 0 ){
+			
+			return( null );
+		}
+		
+		String	str = "";
+		
+		for (String s:cats ){
+			
+			str += (str.length()==0?"":",") + s;
+		}
+		
+		return( str );
+	}
+	
+	protected boolean
+	catsIdentical(
+		Set<String>	c1,
+		Set<String>	c2 )
+	{
+		if ( c1 == null && c2 == null ){
+			
+			return( true );
+			
+		}else if ( c1 == null || c2 == null ){
+			
+			return( false );
+			
+		}else{
+			
+			return( c1.equals( c2 ));
+		}
+	}
+	
+	protected Set<String>
+	stringToCats(
+		String	str )
+	{
+		if ( str == null ){
+			
+			return( null );
+		}
+		
+		String[] bits = str.split( "," );
+		
+		Set<String> res = new HashSet<String>( bits.length );
+		
+		for ( String b: bits ){
+			
+			b = b.trim();
+			
+			if ( b.length() > 0 ){
+				
+				res.add( b );
+			}
+		}
+		
+		if ( res.size() == 0 ){
+			
+			return( null );
+		}
+		
+		return( res );
+	}
+	
 	public int
 	getOnlineStatus()
 	{
@@ -459,7 +838,7 @@ BuddyPluginBuddy
 		return( persistent_msg_handler.storeExplicitMessage( type, msg ));
 	}
 	
-	public List
+	public List<BuddyPluginBuddyMessage>
 	retrieveMessages(
 		int		type )
 	{
@@ -492,7 +871,7 @@ BuddyPluginBuddy
 				{
 					boolean	retry;
 					
-					synchronized( this ){
+					synchronized( BuddyPluginBuddy.this ){
 						
 						ygm_active = false;
 						
@@ -531,7 +910,7 @@ BuddyPluginBuddy
 		
 			if ( recent_ygm == null ){
 				
-				recent_ygm = new ArrayList();
+				recent_ygm = new ArrayList<Long>();
 			}
 			
 			if ( recent_ygm.contains( l )){
@@ -803,7 +1182,7 @@ BuddyPluginBuddy
 	receivedCloseRequest(
 		Map		request )
 	{
-		List	closing = new ArrayList();
+		List<buddyConnection>	closing = new ArrayList<buddyConnection>();
 		
 		synchronized( this ){
 			
@@ -832,7 +1211,7 @@ BuddyPluginBuddy
 					
 					if ( offline_seq_set == null ){
 						
-						offline_seq_set = new HashSet();
+						offline_seq_set = new HashSet<Long>();
 					}
 					
 					offline_seq_set.add( new Long( last_status_seq ));
@@ -1040,7 +1419,7 @@ BuddyPluginBuddy
 							
 							// logMessage( "Msg " + message.getString() + " retrying" );
 
-							synchronized( this ){
+							synchronized( BuddyPluginBuddy.this ){
 								
 								messages.add( 0, message );
 							}
@@ -1079,6 +1458,8 @@ BuddyPluginBuddy
 		buddyMessage 	allocated_message 	= null;
 		Throwable		failed_msg_error 	= null;
 		
+		boolean	inform_dirty	= false;
+		
 		synchronized( this ){
 	
 			if ( current_message != null || messages.size() == 0 || closing ){
@@ -1100,34 +1481,17 @@ BuddyPluginBuddy
 			
 			if ( bc == null ){
 				
-				try{
-					if ( destroyed ){
-						
-						throw( new BuddyPluginException( "Friend destroyed" ));
-					}
+				if ( destroyed ){
 					
-					if ( connections.size() >= MAX_ACTIVE_CONNECTIONS ){
-						
-						throw( new BuddyPluginException( "Too many active connections" ));
-					}
+					failed_msg_error = new BuddyPluginException( "Friend destroyed" );
 					
-					GenericMessageConnection generic_connection = outgoingConnection();
+				}else if ( connections.size() >= MAX_ACTIVE_CONNECTIONS ){
 					
-					bc = new buddyConnection( generic_connection , true );
-					
-					connections.add( bc );
-					
-					// logMessage( "Con " + bc.getString() + " added: num=" + connections.size() );
-
-				}catch( Throwable e ){
-			
-					failed_msg_error	= e;
+					failed_msg_error = new BuddyPluginException( "Too many active connections" );
 				}
 			}
 		}
 		
-			// take outside sync block
-		
 		if ( failed_msg_error != null ){
 			
 			allocated_message.reportFailed( failed_msg_error );
@@ -1135,6 +1499,95 @@ BuddyPluginBuddy
 			return;
 		}
 		
+		if ( bc == null ){
+			
+				// single-thread outgoing connect attempts
+			
+			try{
+				outgoing_connect_sem.reserve();
+			
+				synchronized( this ){
+
+					if ( current_message != allocated_message ){
+						
+						failed_msg_error = new BuddyPluginException( "current message no longer active" );
+						
+					}else if ( closing ){
+						
+						return;
+					}
+						
+					if ( failed_msg_error == null ){
+						
+						for (int i=0;i<connections.size();i++){
+							
+							buddyConnection c = (buddyConnection)connections.get(i);
+							
+							if ( !c.hasFailed()){
+								
+								bc	= c;
+							}
+						}
+						
+						if ( bc == null ){
+							
+							if ( destroyed ){
+								
+								failed_msg_error = new BuddyPluginException( "Friend destroyed" );
+								
+							}else if ( connections.size() >= MAX_ACTIVE_CONNECTIONS ){
+								
+								failed_msg_error = new BuddyPluginException( "Too many active connections" );
+							}
+						}
+					}
+				}
+				
+				if ( bc == null && failed_msg_error == null ){
+					
+					try{
+							// can't perform connect op while synchronized as may deadlock on password
+							// aquisition
+						
+						GenericMessageConnection generic_connection = outgoingConnection();
+
+						synchronized( this ){
+
+							if ( current_message != allocated_message ){
+								
+								failed_msg_error = new BuddyPluginException( "current message no longer active" );
+								
+								generic_connection.close();
+								
+							}else{
+								
+								bc = new buddyConnection( generic_connection, true );
+								
+								inform_dirty = connections.size() == 0;
+								
+								connections.add( bc );
+							
+								// logMessage( "Con " + bc.getString() + " added: num=" + connections.size() );
+							}
+						}
+					}catch( Throwable e ){
+						
+						failed_msg_error = e;
+					}
+				}
+			}finally{
+				
+				outgoing_connect_sem.release();
+			}
+		}
+			
+		if ( failed_msg_error != null ){
+			
+			allocated_message.reportFailed( failed_msg_error );
+			
+			return;
+		}
+			
 		try{
 			// logMessage( "Allocating msg " + allocated_message.getString() + " to con " + bc.getString());
 
@@ -1144,6 +1597,11 @@ BuddyPluginBuddy
 			
 			allocated_message.reportFailed( e );
 		}
+		
+		if ( inform_dirty ){
+			
+			plugin.setConfigDirty();
+		}
 	}
 	
 	protected void
@@ -1159,6 +1617,11 @@ BuddyPluginBuddy
 			size = connections.size();
 		}
 		
+		if ( size == 0 ){
+							
+			plugin.setConfigDirty();
+		}
+		
 		if ( size == 0 && bc.isConnected() && !bc.isClosing() && !bc.isRemoteClosing()){
 			
 				// dropped connection, kick in a keep alive 
@@ -1167,32 +1630,50 @@ BuddyPluginBuddy
 				
 				if ( consec_connect_fails == 0 ){
 					
-						// delay a bit
+					long	now = SystemTime.getMonotonousTime();
 					
-					new DelayedEvent(
-							"BuddyPluginBuddy:recon",
-							new Random().nextInt( 3000 ),
-							new AERunnable()
-							{
-								public void
-								runSupport()
-								{
-									int	size;
-									
-									synchronized( this ){
-																				
-										size = connections.size();
-									}
-									
-									if ( consec_connect_fails == 0 && size == 0 ){
-			
-										log( "Attempting reconnect after dropped connection" );
-										
-										sendKeepAlive();
-									}
-								}
-							});
+					boolean do_it = false;
+					
+					synchronized( this ){
+					
+						if ( 	last_auto_reconnect == -1 || 
+								now - last_auto_reconnect > 30*1000 ){
+							
+							last_auto_reconnect = now;
+							
+							do_it = true;
+						}
+					}
+					
+					if ( do_it ){
 						
+							// delay a bit
+						
+						new DelayedEvent(
+								"BuddyPluginBuddy:recon",
+								new Random().nextInt( 3000 ),
+								new AERunnable()
+								{
+									public void
+									runSupport()
+									{
+										int	size;
+										
+										synchronized( BuddyPluginBuddy.this ){
+																					
+											size = connections.size();
+										}
+										
+										if ( consec_connect_fails == 0 && size == 0 ){
+				
+											log( "Attempting reconnect after dropped connection" );
+											
+											sendKeepAlive();
+										}
+									}
+								});
+					}
+							
 				}else{
 					
 					long	delay = 60*1000;
@@ -1278,6 +1759,23 @@ BuddyPluginBuddy
 		}
 	}
 	
+	protected void
+	setCachedStatus(
+		InetAddress		_ip,
+		int				_tcp_port,
+		int				_udp_port )
+	{
+		synchronized( this ){
+
+			if ( ip == null ){
+			
+				ip			= _ip;
+				tcp_port	= _tcp_port;
+				udp_port	= _udp_port;
+			}
+		}
+	}
+	
 	protected void
 	statusCheckComplete(
 		long			_post_time,
@@ -1570,7 +2068,7 @@ BuddyPluginBuddy
 							BuddyPluginBuddy	from_buddy,
 							Map					reply )
 						{
-							synchronized( this ){
+							synchronized( BuddyPluginBuddy.this ){
 								
 								keep_alive_outstanding = false;
 							}
@@ -1581,7 +2079,7 @@ BuddyPluginBuddy
 							BuddyPluginBuddy		to_buddy,
 							BuddyPluginException	cause )
 						{
-							synchronized( this ){
+							synchronized( BuddyPluginBuddy.this ){
 								
 								keep_alive_outstanding = false;
 							}
@@ -1639,7 +2137,7 @@ BuddyPluginBuddy
 	protected void
 	destroy()
 	{
-		List	to_close = new ArrayList();
+		List<buddyConnection>	to_close = new ArrayList<buddyConnection>();
 		
 		synchronized( this ){
 			
@@ -1802,10 +2300,12 @@ BuddyPluginBuddy
 	
 		throws BuddyPluginException
 	{
-		int	size;
+		//int	size;
 		
 		buddyConnection bc = new buddyConnection( _connection, false );
 		
+		boolean inform_dirty = false;
+		
 		synchronized( this ){
 			
 			if ( destroyed ){
@@ -1813,12 +2313,19 @@ BuddyPluginBuddy
 				throw( new BuddyPluginException( "Friend has been destroyed" ));
 			}
 			
+			inform_dirty = connections.size() == 0;
+			
 			connections.add( bc );
 			
-			size = connections.size();
+			//size = connections.size();
 		}
 		
 		// logMessage( "Con " + bc.getString() + " added: num=" + size );
+		
+		if ( inform_dirty ){
+			
+			plugin.setConfigDirty();
+		}
 	}
 	
 	public void
@@ -2265,6 +2772,12 @@ BuddyPluginBuddy
 			send_map.put( "oz", new Long( plugin.getOnlineStatus()));
 			send_map.put( "v", new Long( BuddyPlugin.VERSION_CURRENT ));
 			
+			String	loc_cat = getLocalAuthorisedRSSCategoriesAsString();
+			
+			if ( loc_cat != null ){
+				send_map.put( "cat", loc_cat );
+			}
+			
 			try{
 				// logMessage( "Sending " + msg.getString() + " to " + getString());
 
@@ -2316,6 +2829,17 @@ BuddyPluginBuddy
 					setVersion( l_ver.intValue());
 				}
 				
+				byte[]	b_rem_cat = (byte[])data_map.get( "cat" );
+				
+				if ( b_rem_cat == null ){
+					
+					setRemoteAuthorisedRSSCategories( null );
+					
+				}else{
+					
+					setRemoteAuthorisedRSSCategories( stringToCats( new String( b_rem_cat, "UTF-8" )));
+				}
+				
 				if ( type == RT_REQUEST_DATA ){
 					
 					// logMessage( "Received type=" + type + " from " + getString());
@@ -2368,6 +2892,12 @@ BuddyPluginBuddy
 					reply_map.put( "id", data_map.get( "id" ) );
 					reply_map.put( "oz", new Long( plugin.getOnlineStatus()));
 
+					String	loc_cat = getLocalAuthorisedRSSCategoriesAsString();
+					
+					if ( loc_cat != null ){
+						reply_map.put( "cat", loc_cat );
+					}
+					
 					reply_map.put( "rep", reply );
 					
 						// don't record as active here as (1) we recorded as active above when 
