@@ -25,11 +25,9 @@ import java.util.*;
 
 import org.gudy.azureus2.core3.config.COConfigurationListener;
 import org.gudy.azureus2.core3.config.COConfigurationManager;
+import org.gudy.azureus2.core3.config.impl.ConfigurationDefaults;
 import org.gudy.azureus2.core3.util.*;
-import org.gudy.azureus2.plugins.Plugin;
-import org.gudy.azureus2.plugins.PluginConfig;
-import org.gudy.azureus2.plugins.PluginInterface;
-import org.gudy.azureus2.plugins.PluginListener;
+import org.gudy.azureus2.plugins.*;
 import org.gudy.azureus2.plugins.disk.DiskManagerFileInfo;
 import org.gudy.azureus2.plugins.download.*;
 import org.gudy.azureus2.plugins.logging.LoggerChannel;
@@ -101,7 +99,7 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 	 * Interval in ms between checks to see if the {@link #somethingChanged} 
 	 * flag changed 
 	 */
-	private static final int PROCESS_CHECK_PERIOD = 120;
+	private static final int PROCESS_CHECK_PERIOD = 1500;
 
 	/** Wait xx ms before starting completed torrents (so scrapes can come in) */
 	private static final int MIN_SEEDING_STARTUP_WAIT = 20000;
@@ -126,7 +124,7 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 	private RecalcSeedingRanksTask recalcSeedingRanksTask;
 
 	/** Map to relate downloadData to a Download */
-	private static Map downloadDataMap = AEMonitor.getSynchronisedMap(new HashMap());
+	private static Map downloadDataMap = Collections.synchronizedMap(new HashMap());
 	/**
 	 * this is used to reduce the number of comperator invocations
 	 * by keeping a mostly sorted copy around, must be nulled whenever the map is changed
@@ -153,6 +151,9 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 
 	private int minSpeedForActiveSeeding;
 
+	/** Maximimum # of stalled torrents that are in seeding mode */
+	private int maxStalledSeeding;
+
 	// count x peers as a full copy, but..
 	private int numPeersAsFullCopy;
 
@@ -192,6 +193,8 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 	private boolean bSWTUI = false;
 	
 	private CopyOnWriteList listenersFP = new CopyOnWriteList();
+
+	public static boolean pauseChangeFlagChecker = false;
 	
 	public static void
 	load(
@@ -213,33 +216,24 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 		startedOn = SystemTime.getCurrentTime();
 
 		pi = _plugin_interface;
-		download_manager = pi.getDownloadManager();
 
-		pi.getPluginconfig().setPluginConfigKeyPrefix("");
+		plugin_config = pi.getPluginconfig();
+
+		plugin_config.setPluginConfigKeyPrefix("");
+
+		download_manager = pi.getDownloadManager();
 
 		// Create a configModel for StartStopRules
 		// We always need to do this in order to set up configuration defaults
 		UIManager manager = pi.getUIManager();
+		
 		// TODO: don't name it Q
 		final BasicPluginConfigModel configModel = manager.createBasicPluginConfigModel(
 				ConfigSection.SECTION_ROOT, "Q");
 		setupConfigModel(configModel);
 
 		pi.addListener(new PluginListener() {
-			public void initializationComplete() {
-				// CPU Intensive, delay until a little after all plugin initializations
-				// XXX Would be better if we could delay it until UI is done,
-				//     but there may be no UI..
-				new DelayedEvent("StartStop:initComp", 12000, new AERunnable() {
-					public void runSupport() {
-						download_manager.addListener(new StartStopDMListener());
-						SimpleTimer.addPeriodicEvent("StartStop:gross",
-								CHECK_FOR_GROSS_CHANGE_PERIOD, new ChangeCheckerTimerTask());
-						SimpleTimer.addPeriodicEvent("StartStop:check",
-								PROCESS_CHECK_PERIOD, new ChangeFlagCheckerTask());
-					}
-				});
-			}
+			public void initializationComplete() {}
 
 			public void closedownInitiated() {
 				closingDown = true;
@@ -253,15 +247,27 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 			}
 		});
 
-		log = pi.getLogger().getChannel("StartStopRules");
+		Runnable r = new Runnable() {
+			public void run() {
+				download_manager.addListener(new StartStopDMListener());
+				SimpleTimer.addPeriodicEvent("StartStop:gross",
+						CHECK_FOR_GROSS_CHANGE_PERIOD, new ChangeCheckerTimerTask());
+				SimpleTimer.addPeriodicEvent("StartStop:check",
+						PROCESS_CHECK_PERIOD, new ChangeFlagCheckerTask());
+			}
+		};
+		
+		pi.getUtilities().createDelayedTask(r).queue();
+
+		log = pi.getLogger().getTimeStampedChannel("StartStopRules");
+		
 		log.log(LoggerChannel.LT_INFORMATION,
 				"Default StartStopRules Plugin Initialisation");
 
 		COConfigurationManager.addListener(this);
 
-		plugin_config = pi.getPluginconfig();
-
 		try {
+			pi.getUIManager().createLoggingViewModel(log, true);
 			pi.getUIManager().addUIListener(new UIManagerListener() {
 				public void UIAttached(UIInstance instance) {
 					TableManager tm = pi.getUIManager().getTableManager();
@@ -342,7 +348,10 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 				"ConfigView.label.minSpeedForActiveDL", 512);
 		configModel.addIntParameter2("StartStopManager_iMinSpeedForActiveSeeding",
 				"ConfigView.label.minSpeedForActiveSeeding", 512);
+		configModel.addIntParameter2("StartStopManager_iMaxStalledSeeding",
+				"ConfigView.label.maxStalledSeeding", 5);
 
+		
 		configModel.addBooleanParameter2("StartStopManager_bDebugLog",
 				"ConfigView.label.queue.debuglog", false);
 		configModel.addBooleanParameter2("StartStopManager_bNewSeedsMoveTop",
@@ -382,8 +391,13 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 
 		configModel.addBooleanParameter2(
 				"StartStopManager_bFirstPriority_ignore0Peer",
-				"ConfigView.label.seeding.firstPriority.ignore0Peer", false);
+				"ConfigView.label.seeding.firstPriority.ignore0Peer",
+				!COConfigurationManager.getStringParameter("ui", "").equals("az2"));
 
+		configModel.addIntParameter2(
+				"StartStopManager_iFirstPriority_ignoreIdleHours",
+				"ConfigView.label.seeding.firstPriority.ignoreIdleHours", 24);
+		
 		// seeding subsection
 		configModel.addIntParameter2("StartStopManager_iAddForSeedingDLCopyCount",
 				"ConfigView.label.seeding.addForSeedingDLCopyCount", 1);
@@ -456,12 +470,15 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 		long cycleNo = 0;
 
 		public void perform(TimerEvent event) {
-			if (closingDown) {
+			if (closingDown || pauseChangeFlagChecker ) {
 				return;
 			}
 
 			cycleNo++;
 			if (cycleNo > FORCE_CHECK_CYCLES) {
+				if (bDebugLog) {
+					log.log(LoggerChannel.LT_INFORMATION, ">>force process");
+				}
 				somethingChanged = true;
 			}
 
@@ -486,6 +503,14 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 			if (dlData != null) {
 				// force a SR recalc, so that it gets position properly next process()
 				requestProcessCycle(dlData);
+				if (new_state == Download.ST_READY || new_state == Download.ST_WAITING) {
+					new AEThread2("processReady", true) {
+						public void run() {
+							process();
+						}
+					}.start();
+				}
+				
 				if (bDebugLog)
 					log.log(dlData.dl.getTorrent(), LoggerChannel.LT_INFORMATION,
 							"somethingChanged: stateChange from " + sStates.charAt(old_state)
@@ -552,6 +577,11 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 			//		+ event.getActivationCount());
 			Download download = event.getDownload();
 			DefaultRankCalculator dlData = (DefaultRankCalculator) downloadDataMap.get(download);
+
+			if (bDebugLog) {
+				log.log(download, LoggerChannel.LT_INFORMATION, ">> somethingChanged: ActivationRequest");
+			}
+
 			// ok to be null
 			requestProcessCycle(dlData);
 
@@ -709,6 +739,11 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 
 			int iNewRankType = plugin_config.getIntParameter("StartStopManager_iRankType");
 			minSpeedForActiveSeeding = plugin_config.getIntParameter("StartStopManager_iMinSpeedForActiveSeeding");
+			maxStalledSeeding = plugin_config.getIntParameter("StartStopManager_iMaxStalledSeeding");
+			if (maxStalledSeeding <= 0) {
+				// insanity :)
+				maxStalledSeeding = 999;
+			}
 			_maxActive = plugin_config.getIntParameter("max active torrents");
 			_maxActiveWhenSeedingEnabled = plugin_config.getBooleanParameter("StartStopManager_bMaxActiveTorrentsWhenSeedingEnabled");
 			_maxActiveWhenSeeding = plugin_config.getIntParameter("StartStopManager_iMaxActiveTorrentsWhenSeeding");
@@ -1110,10 +1145,11 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 	 */
 	public class ProcessVars
 	{
+		/** Running count of torrents waiting or seeding, not including stalled */
 		int numWaitingOrSeeding; // Running Count
 
 		int numWaitingOrDLing; // Running Count
-		
+
 		long accumulatedDownloadSpeed;
 		
 		long accumulatedUploadSpeed;
@@ -1133,6 +1169,8 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 		int posComplete;
 
 		boolean bStopAndQueued;
+		
+		int stalledSeeders; // Running Count
 	}
 
 	private long processCount = 0;
@@ -1213,7 +1251,7 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 				};
 			}
 
-			// Sort
+			// Sort: SeedingRank Desc, Position Desc
 			Arrays.sort(dlDataArray);
 
 			ProcessVars vars = new ProcessVars();
@@ -1226,6 +1264,7 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 			vars.higherCDtoStart = false;
 			vars.higherDLtoStart = false;
 			vars.posComplete = 0;
+			vars.stalledSeeders = 0; // Running Count;
 
 			// Loop 2 of 2:
 			// - Start/Stop torrents based on criteria
@@ -1320,6 +1359,9 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 			if (now > 0) {
 				processCount++;
 				long timeTaken = (SystemTime.getCurrentTime() - now);
+				if (bDebugLog) {
+					log.log(LoggerChannel.LT_INFORMATION, "process() took " + timeTaken);
+				}
 				processTotalMS += timeTaken;
 				if (timeTaken > processMaxMS) {
 					processMaxMS = timeTaken;
@@ -1539,6 +1581,9 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 		state = download.getState();
 
 		if (oldState != state) {
+			if (bDebugLog) {
+				log.log(LoggerChannel.LT_INFORMATION, ">> somethingChanged: state");
+			}
 			somethingChanged = true;
 		}
 
@@ -1581,6 +1626,7 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 
 		Download download = dlData.dl;
 		int state = download.getState();
+		boolean stateReadyOrSeeding = state == Download.ST_READY || state == Download.ST_SEEDING;
 
 		String[] debugEntries = null;
 		String sDebugLine = "";
@@ -1611,7 +1657,9 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 				"maxCDrs=" + totals.maxSeeders,
 				"FP=" + boolDebug(isFP),
 				"nActCDing=" + totals.activelyCDing,
-				"ActCDing=" + boolDebug(dlData.getActivelySeeding())
+				"ActCDing=" + boolDebug(dlData.getActivelySeeding()),
+				"nSeeds=" + dlData.lastModifiedScrapeResultSeeds,
+				"nPeers=" + dlData.lastModifiedScrapeResultPeers
 			};
 		}
 
@@ -1653,6 +1701,36 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 				return;
 			}
 
+			int rank = download.getSeedingRank();
+
+			// Short Circuit: if rank is set to IGNORED, we can skip everything
+			// except when:
+			// (1) torrent is force started
+			// (2) the torrent is in ready or seeding state (we have to stop the torrent)
+			// (3) we auto start 0 peers
+			if (rank < DefaultRankCalculator.SR_IGNORED_LESS_THAN
+					&& !download.isForceStart() && !stateReadyOrSeeding
+					&& !bAutoStart0Peers) {
+				if (bDebugLog) {
+					sDebugLine += "\n  Skip !forceStart";
+					int idx = rank * -1;
+					if (idx < DefaultRankCalculator.SR_NEGATIVE_DEBUG.length) {
+						sDebugLine += " && " + DefaultRankCalculator.SR_NEGATIVE_DEBUG[idx];
+					}
+				}
+				return;
+			}
+
+			// Short Circuit: if seed higher in the queue is marked to start,
+			// we can skip everything, except when:
+			// (1) torrent is force started
+			// (2) we auto start 0 peers
+			// (3) the torrent is in ready or seeding state (we have to stop the torrent)
+			if (vars.higherCDtoStart && !download.isForceStart() && !bAutoStart0Peers
+					&& !stateReadyOrSeeding) {
+				sDebugLine += " a torrent with a higher rank is queued or starting";
+			}
+
 			if (bDebugLog && bAutoStart0Peers && numPeers == 0 && !bScrapeOk
 					&& (state == Download.ST_QUEUED || state == Download.ST_READY)) {
 				sDebugLine += "\n  NOT starting 0 Peer torrent because scrape isn't ok";
@@ -1682,8 +1760,11 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 			globalDownLimitReached = false;
 			fakedActively = false;
 		}
-				
-			
+		
+  		if (state == Download.ST_SEEDING && !bActivelySeeding) {
+  			vars.stalledSeeders++;
+  		}
+
 			// Is it OK to set this download to a queued state?
 			// It is if:
 			//   1) It is either READY or SEEDING; and
@@ -1702,13 +1783,12 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 			//           active torrents count.
 			//
 			//   3) It hasn't been force started.
-			boolean okToQueue = (state == Download.ST_READY || state == Download.ST_SEEDING)
+			boolean okToQueue = stateReadyOrSeeding
 					&& (!isFP || (isFP && ((totals.maxActive != 0 && vars.numWaitingOrSeeding >= totals.maxActive
 							- minDownloads))))
 					//&& (!isFP || (isFP && ((vars.numWaitingOrSeeding >= totals.maxSeeders) || (!bActivelySeeding && (vars.numWaitingOrSeeding + totals.totalStalledSeeders) >= totals.maxSeeders))) )
 					&& (!download.isForceStart());
-			int rank = download.getSeedingRank();
-
+			
 			// in RANK_TIMED mode, we use minTimeAlive for rotation time, so
 			// skip check
 			// XXX do we want changes to take effect immediately  ?
@@ -1738,9 +1818,10 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 			// XXX Change to waiting if queued and we have an open slot
 			if (!okToQueue
 					&& (state == Download.ST_QUEUED)
-					&& (totals.maxActive == 0 || vars.numWaitingOrSeeding < totals.maxSeeders)
+					&& (totals.maxActive == 0 || vars.numWaitingOrSeeding < totals.maxSeeders) 
 					//&& (totals.maxActive == 0 || (activeSeedingCount + activeDLCount) < totals.maxActive) &&
 					&& (rank >= DefaultRankCalculator.SR_IGNORED_LESS_THAN)
+					&& (vars.stalledSeeders < maxStalledSeeding)
 					&& !vars.higherCDtoStart) {
 				try {
 					if (bDebugLog)
@@ -1760,18 +1841,38 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 				state = download.getState();
 			} else if (bDebugLog && state == Download.ST_QUEUED) {
 				sDebugLine += "\n  NOT restarting:";
-				if (rank < DefaultRankCalculator.SR_IGNORED_LESS_THAN)
+				if (rank < DefaultRankCalculator.SR_IGNORED_LESS_THAN) {
 					sDebugLine += " torrent is being ignored";
-				else if (vars.higherCDtoStart)
+					int idx = rank * -1;
+					if (idx < DefaultRankCalculator.SR_NEGATIVE_DEBUG.length) {
+						sDebugLine += ": " + DefaultRankCalculator.SR_NEGATIVE_DEBUG[idx];
+					}
+				} else if (vars.higherCDtoStart) {
 					sDebugLine += " a torrent with a higher rank is queued or starting";
-				else {
+				} else {
 					if (okToQueue)
 						sDebugLine += " no starting of okToQueue'd;";
 
-					if (vars.numWaitingOrSeeding >= totals.maxSeeders)
+					if (vars.numWaitingOrSeeding >= totals.maxSeeders) {
 						sDebugLine += " at limit, numWaitingOrSeeding("
 								+ vars.numWaitingOrSeeding + ") >= maxSeeders("
 								+ totals.maxSeeders + ")";
+					} else if (vars.stalledSeeders >= maxStalledSeeding) {
+						sDebugLine += " at limit, stalledSeeders(" + vars.stalledSeeders
+								+ ") >= maxStalledSeeding("
+								+ maxStalledSeeding + ") ";
+					} else {
+						sDebugLine += "huh? qd="
+								+ (state == Download.ST_QUEUED)
+								+ "; "
+								+ totals.maxActive
+								+ ";"
+								+ (vars.numWaitingOrSeeding < totals.maxSeeders)
+								+ ";"
+								+ (vars.stalledSeeders <= maxStalledSeeding)
+								+ ";ignore?"
+								+ (rank >= DefaultRankCalculator.SR_IGNORED_LESS_THAN);
+					}
 				}
 			}
 
@@ -1812,6 +1913,7 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 				if (!okToStop) {
 					// break up the logic into variables to make more readable
 					boolean bOverLimit = vars.numWaitingOrSeeding > totals.maxSeeders
+				      || (!bActivelySeeding && vars.stalledSeeders > maxStalledSeeding)
 							|| (vars.numWaitingOrSeeding >= totals.maxSeeders && vars.higherCDtoStart);
 					boolean bSeeding = state == Download.ST_SEEDING;
 
@@ -1825,10 +1927,13 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 						if (okToStop) {
 							sDebugLine += "\n  stopAndQueue: ";
 							if (bOverLimit) {
-								if (vars.higherCDtoStart)
+								if (vars.higherCDtoStart) {
 									sDebugLine += "higherQueued (it should be seeding instead of this one)";
-								else
+								} else if (!bActivelySeeding && vars.stalledSeeders > totals.maxSeeders) {
+									sDebugLine += "over stale seeds limit";
+								} else {
 									sDebugLine += "over limit";
+								}
 							} else if (rank < DefaultRankCalculator.SR_IGNORED_LESS_THAN)
 								sDebugLine += "ignoreRule met";
 
@@ -1926,7 +2031,9 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 					"maxCDrs=" + totals.maxSeeders,
 					"FP=" + boolDebug(isFP),
 					"nActCDing=" + totals.activelyCDing,
-					"ActCDing=" + boolDebug(dlData.getActivelySeeding())
+					"ActCDing=" + boolDebug(dlData.getActivelySeeding()),
+					"nSeeds=" + dlData.lastModifiedScrapeResultSeeds,
+					"nPeers=" + dlData.lastModifiedScrapeResultPeers
 				};
 				printDebugChanges("", debugEntries, debugEntries2, sDebugLine, "  ",
 						true, dlData);
@@ -2085,8 +2192,9 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 
 		try {
 			writer.indent();
-			writer.println("Started " + (SystemTime.getCurrentTime() - startedOn)
-					+ "ms ago");
+			writer.println("Started " + TimeFormatter.format100ths(SystemTime.getCurrentTime() - startedOn)
+					+ " ago");
+			writer.println("debugging = " + bDebugLog);
 			writer.println("downloadDataMap size = " + downloadDataMap.size());
 			if (changeCheckCount > 0) {
 				writer.println("changeCheck CPU ms: avg="
