@@ -20,19 +20,21 @@
 
 package com.aelitis.azureus.plugins.startstoprules.defaultplugin.ui.swt;
 
+import java.util.ArrayList;
+
 import org.eclipse.swt.SWT;
-import org.eclipse.swt.graphics.Image;
 import org.eclipse.swt.layout.GridData;
 import org.eclipse.swt.layout.GridLayout;
 import org.eclipse.swt.widgets.Composite;
 import org.eclipse.swt.widgets.Label;
 
 import org.gudy.azureus2.core3.util.DisplayFormatters;
-import org.gudy.azureus2.ui.swt.ImageRepository;
 import org.gudy.azureus2.ui.swt.Messages;
 import org.gudy.azureus2.ui.swt.config.*;
 import org.gudy.azureus2.ui.swt.plugins.UISWTConfigSection;
 
+import com.aelitis.azureus.ui.swt.imageloader.ImageLoader;
+
 import org.gudy.azureus2.plugins.ui.config.ConfigSection;
 
 /** General Queueing options
@@ -44,6 +46,10 @@ public class ConfigSectionQueue implements UISWTConfigSection
 	public String configSectionGetParentSection() {
 		return ConfigSection.SECTION_ROOT;
 	}
+	
+	public int maxUserMode() {
+		return 0;
+	}
 
 	/**
 	 * Create the "Queue" Tab in the Configuration view
@@ -95,12 +101,11 @@ public class ConfigSectionQueue implements UISWTConfigSection
 		gridData.horizontalSpan = 2;
 		cMaxActiveOptionsArea.setLayoutData(gridData);
 
+		ImageLoader imageLoader = ImageLoader.getInstance();
 		label = new Label(cMaxActiveOptionsArea, SWT.NULL);
-		Image img = ImageRepository.getImage("subitem");
-		img.setBackground(label.getBackground());
+		imageLoader.setLabelImage(label, "subitem");
 		gridData = new GridData(GridData.VERTICAL_ALIGN_BEGINNING);
 		label.setLayoutData(gridData);
-		label.setImage(img);
 
 		gridData = new GridData();
 		BooleanParameter maxActiveWhenSeedingEnabled = new BooleanParameter(
@@ -158,29 +163,33 @@ public class ConfigSectionQueue implements UISWTConfigSection
 
 		
 		// row
+		
+		final ArrayList values = new ArrayList();
+		int exp = 29;
+		for(int val = 0; val <= 8*1024*1024;)
+		{
+			values.add(new Integer(val));
+			if(val < 256)
+				val+=64;
+			else if(val < 1024)
+				val+=256;
+			else if(val < 16*1024)
+				val+=1024;
+			else
+				val = (int)(Math.pow(2, exp++/2) + (exp % 2 == 0 ? Math.pow(2,  (exp-3)/2) : 0));
+		}
+		String[] activeDLLabels = new String[values.size()];
+		int[] activeDLValues = new int[activeDLLabels.length];
+		
 
 		label = new Label(cSection, SWT.NULL);
 		Messages.setLanguageText(label, "ConfigView.label.minSpeedForActiveDL");
-		final String activeDLLabels[] = new String[57];
-		final int activeDLValues[] = new int[57];
-		int pos = 0;
-		for (int i = 0; i < 256; i += 64) {
-			activeDLValues[pos] = i;
-			activeDLLabels[pos] = DisplayFormatters.formatByteCountToKiBEtcPerSec(
-					activeDLValues[pos], true);
-			pos++;
-		}
-		for (int i = 256; i < 1024; i += 256) {
-			activeDLValues[pos] = i;
-			activeDLLabels[pos] = DisplayFormatters.formatByteCountToKiBEtcPerSec(
-					activeDLValues[pos], true);
-			pos++;
-		}
-		for (int i = 1; pos < activeDLLabels.length; i++) {
-			activeDLValues[pos] = i * 1024;
-			activeDLLabels[pos] = DisplayFormatters.formatByteCountToKiBEtcPerSec(
-					activeDLValues[pos], true);
-			pos++;
+		for(int i=0;i<activeDLLabels.length;i++)
+		{
+			activeDLValues[i] = ((Integer)values.get(i)).intValue();
+			activeDLLabels[i] = DisplayFormatters.formatByteCountToKiBEtcPerSec(
+				activeDLValues[i], true);
+			
 		}
 		new IntListParameter(cSection, "StartStopManager_iMinSpeedForActiveDL",
 				activeDLLabels, activeDLValues);
@@ -189,32 +198,50 @@ public class ConfigSectionQueue implements UISWTConfigSection
 
 		label = new Label(cSection, SWT.NULL);
 		Messages.setLanguageText(label, "ConfigView.label.minSpeedForActiveSeeding");
-		final String activeSeedingLabels[] = new String[27];
-		final int activeSeedingValues[] = new int[27];
-		pos = 0;
+		String[] activeSeedingLabels = new String[values.size()-4];
+		int[] activeSeedingValues = new int[activeSeedingLabels.length];
+		System.arraycopy(activeDLLabels, 0, activeSeedingLabels, 0, activeSeedingLabels.length);
+		System.arraycopy(activeDLValues, 0, activeSeedingValues, 0, activeSeedingValues.length);
 
-		for (int i = 0; i < 256; i += 64) {
-			activeSeedingValues[pos] = i;
-			activeSeedingLabels[pos] = DisplayFormatters.formatByteCountToKiBEtcPerSec(
-					activeSeedingValues[pos], true);
-			pos++;
-		}
-		for (int i = 256; i < 1024; i += 256) {
-			activeSeedingValues[pos] = i;
-			activeSeedingLabels[pos] = DisplayFormatters.formatByteCountToKiBEtcPerSec(
-					activeSeedingValues[pos], true);
-			pos++;
-		}
-		for (int i = 1; pos < activeSeedingLabels.length; i++) {
-			activeSeedingValues[pos] = i * 1024;
-			activeSeedingLabels[pos] = DisplayFormatters.formatByteCountToKiBEtcPerSec(
-					activeSeedingValues[pos], true);
-			pos++;
-		}
 		new IntListParameter(cSection,
 				"StartStopManager_iMinSpeedForActiveSeeding", activeSeedingLabels,
 				activeSeedingValues);
 
+		// subrow
+
+		final Composite cMinSpeedActiveCDing = new Composite(cSection, SWT.NULL);
+		layout = new GridLayout();
+		layout.numColumns = 3;
+		layout.marginWidth = 0;
+		layout.marginHeight = 0;
+		cMinSpeedActiveCDing.setLayout(layout);
+		gridData = new GridData();
+		gridData.horizontalIndent = 15;
+		gridData.horizontalSpan = 2;
+		cMinSpeedActiveCDing.setLayoutData(gridData);
+
+		label = new Label(cMinSpeedActiveCDing, SWT.NULL);
+		imageLoader.setLabelImage(label, "subitem");
+		gridData = new GridData(GridData.VERTICAL_ALIGN_BEGINNING);
+		label.setLayoutData(gridData);
+
+		label = new Label(cMinSpeedActiveCDing, SWT.NULL);
+		Messages.setLanguageText(label, "ConfigView.label.maxStalledSeeding");
+
+		gridData = new GridData();
+		final IntParameter maxStalledSeeding = new IntParameter(
+				cMinSpeedActiveCDing, "StartStopManager_iMaxStalledSeeding");
+		maxStalledSeeding.setMinimumValue(0);
+		maxStalledSeeding.setLayoutData(gridData);
+
+		
+		// row
+
+		gridData = new GridData();
+		gridData.horizontalSpan = 2;
+		new BooleanParameter(cSection, "StartStopManager_bStopOnceBandwidthMet",
+				"ConfigView.label.queue.stoponcebandwidthmet").setLayoutData(gridData);
+
 		// row
 
 		gridData = new GridData();
