@@ -22,6 +22,7 @@ package com.aelitis.azureus.ui;
 import org.gudy.azureus2.core3.download.DownloadManager;
 
 import com.aelitis.azureus.core.AzureusCoreComponent;
+import com.aelitis.azureus.ui.common.updater.UIUpdater;
 
 /**
  * @author TuxPaper
@@ -39,28 +40,20 @@ public interface UIFunctions
 
 	public static int STATUSICON_ERROR = 2;
 
-	/**
-	 * Display the stats view
-	 */
-	void showStats();
-
-	/**
-	 * Display the Stats View -> Transfer subview
-	 */
-	void showStatsTransfers();
-
-	/**
-	 * Display the Stats View -> HHT subview
-	 */
-	void showStatsDHT();
-
-	/**
-	 * Display the config window
-	 * 
-	 * @param string Section to show.  null for root 
-	 * @return true: section was found
-	 */
-	boolean showConfig(String string);
+	
+	public static final int VIEW_CONSOLE = 0;
+	public static final int VIEW_STATS = 1;
+	public static final int VIEW_CONFIG = 4;
+	public static final int VIEW_DM_DETAILS = 5;
+	public static final int VIEW_DM_MULTI_OPTIONS = 6;
+	public static final int VIEW_MYSHARES = 7;
+	public static final int VIEW_MYTORRENTS = 8;
+	public static final int VIEW_MYTRACKER = 9;
+	public static final int VIEW_ALLPEERS = 10;
+	public static final int VIEW_DETAILED_LISTVIEW = 11;
+	
+	public static final int ACTION_FULL_UPDATE	= 1;	// arg: String - url; response Boolean - ok
+	
 
 	/**
 	 * Bring main window to the front
@@ -76,47 +69,16 @@ public interface UIFunctions
 	 */
 	void bringToFront(boolean noTricks);
 	
-	/**
-	 * Request the UI be shut down.
-	 * 
-	 * @return true - request granted, UI is being shut down
-	 *         false - request denied (example: password entry failed)
-	 */
-	boolean requestShutdown();
-
 	/**
 	 * Change/Refresh the language of the UI
 	 */
 	void refreshLanguage();
 
-	/**
-	 * @param dm
-	 */
-	void openManagerView(DownloadManager dm);
-
 	/**
 	 * 
 	 */
 	void refreshIconBar();
 
-	/**
-	 * 
-	 */
-	void showMyTracker();
-
-	/**
-	 * 
-	 */
-	void showMyShares();
-
-	/**
-	 * 
-	 */
-	void showMyTorrents();
-	
-	void showAllPeersView();
-
-	void showMultiOptionsView( DownloadManager[] dms );
 	
 	/**
 	 * @param manager
@@ -130,27 +92,22 @@ public interface UIFunctions
 
 	void setStatusText(int statustype, String string, UIStatusTextClickListener l);
 
+	/**
+	 * Request the UI be shut down.
+	 * 
+	 * @return true - request granted, UI is being shut down
+	 *         false - request denied (example: password entry failed)
+	 */
 	boolean dispose(boolean for_restart, boolean close_already_in_progress);
 
-	/**
-	 * 
-	 */
-	void showConsole();
-
-	/**
-	 * @param url
-	 * @param target TODO
-	 * @param w
-	 * @param h
-	 * @param allowResize 
-	 * @return success level
-	 */
 	boolean viewURL(String url, String target, int w, int h, boolean allowResize,
 			boolean isModal);
 
 	boolean viewURL(String url, String target, double wPct, double hPct,
 			boolean allowResize, boolean isModal);
 
+	void viewURL(String url, String target, String sourceRef);
+
 
 	public UIFunctionsUserPrompter getUserPrompter(String title, String text,
 			String[] buttons, int defaultOption);
@@ -158,4 +115,44 @@ public interface UIFunctions
 	public int promptUser(String title, String text, String[] buttons,
 			int defaultOption, String rememberID, String rememberText,
 			boolean bRememberByDefault, int autoCloseInMS);
+	
+	/**
+	 * Retrieves the class that handles periodically updating the UI
+	 * 
+	 * @return
+	 *
+	 * @since 3.1.1.1
+	 */
+	public UIUpdater getUIUpdater();
+
+	/**
+	 * @param viewID
+	 * @param data
+	 *
+	 * @since 3.1.1.1
+	 */
+	void openView(int viewID, Object datasource);
+	
+	void doSearch(String searchText);
+	
+	
+	/**
+	 * 
+	 * @param action_id
+	 * @param args
+	 * @param listener
+	 */
+	public void
+	performAction(
+		int				action_id,
+		Object			args,
+		actionListener	listener );
+	
+	interface 
+	actionListener
+	{
+		public void
+		actionComplete(
+			Object		result );
+	}		
 }
