@@ -28,9 +28,9 @@ package com.aelitis.azureus.ui.common.table;
  * @author Olivier
  *
  */
-public interface TableStructureModificationListener
+public interface TableStructureModificationListener<T>
 {
-	void tableStructureChanged();
+	void tableStructureChanged(boolean columnAddedOrRemoved, Class forPluginDataSourceType );
 
 	void columnOrderChanged(int[] iPositions);
 
@@ -38,5 +38,5 @@ public interface TableStructureModificationListener
 
 	void columnInvalidate(TableColumnCore tableColumn);
 	
-	void cellInvalidate(TableColumnCore tableColumn, Object data_source);
+	void cellInvalidate(TableColumnCore tableColumn, T data_source);
 }
