@@ -31,6 +31,13 @@ import org.gudy.azureus2.ui.swt.views.table.utils.TableColumnManager;
 
 import com.aelitis.azureus.ui.common.table.*;
 
+import org.gudy.azureus2.plugins.disk.DiskManagerFileInfo;
+import org.gudy.azureus2.plugins.download.Download;
+import org.gudy.azureus2.plugins.download.DownloadTypeComplete;
+import org.gudy.azureus2.plugins.download.DownloadTypeIncomplete;
+import org.gudy.azureus2.plugins.peers.Peer;
+import org.gudy.azureus2.plugins.sharing.ShareResource;
+import org.gudy.azureus2.plugins.tracker.TrackerTorrent;
 import org.gudy.azureus2.plugins.ui.UIRuntimeException;
 import org.gudy.azureus2.plugins.ui.tables.*;
 
@@ -91,11 +98,17 @@ public class TableColumnImpl
 	private ArrayList cellMouseMoveListeners;
 
 	private ArrayList cellVisibilityListeners;
+	
+	private ArrayList<TableColumnExtraInfoListener> columnExtraInfoListeners;
 
+	private Map mapOtherCellListeners;
+	
 	private int iConsecutiveErrCount;
 
-	private ArrayList menuItems;
+	private ArrayList<TableContextMenuItem> menuItemsHeader;
 
+	private ArrayList<TableContextMenuItem> menuItemsColumn;
+	
 	private boolean bObfusticateData;
 
 	protected AEMonitor this_mon = new AEMonitor("TableColumn");
@@ -132,12 +145,51 @@ public class TableColumnImpl
 	
 	private Map userData;
 
+	private boolean removed;
+	
+	private Class forPluginDataSourceType;
+
+	/**
+	 * @deprecated
+	 */
+	public TableColumnImpl(String tableID, String columnID) {
+		// Guess forPluginDataSourceType based on tableID
+		Class forPluginDataSourceType = null;
+		if (TableManager.TABLE_MYTORRENTS_ALL_BIG.equals(tableID)
+				|| TableManager.TABLE_MYTORRENTS_UNOPENED.equals(tableID)
+				|| TableManager.TABLE_MYTORRENTS_UNOPENED_BIG.equals(tableID)) {
+			forPluginDataSourceType = Download.class;
+		} else if (TableManager.TABLE_MYTORRENTS_INCOMPLETE_BIG.equals(tableID)
+				|| TableManager.TABLE_MYTORRENTS_INCOMPLETE.equals(tableID)) {
+			forPluginDataSourceType = DownloadTypeIncomplete.class;
+		} else if (TableManager.TABLE_MYTORRENTS_COMPLETE.equals(tableID)
+				|| TableManager.TABLE_MYTORRENTS_COMPLETE_BIG.equals(tableID)) {
+			forPluginDataSourceType = DownloadTypeComplete.class;
+		} else if (TableManager.TABLE_TORRENT_PEERS.equals(tableID)) {
+			forPluginDataSourceType = Peer.class;
+		} else if (TableManager.TABLE_TORRENT_FILES.equals(tableID)) {
+			forPluginDataSourceType = DiskManagerFileInfo.class;
+		} else if (TableManager.TABLE_MYTRACKER.equals(tableID)) {
+			forPluginDataSourceType = TrackerTorrent.class;
+		} else if (TableManager.TABLE_MYSHARES.equals(tableID)) {
+			forPluginDataSourceType = ShareResource.class;
+		}
+		init(forPluginDataSourceType, tableID, columnID);
+	}
+
 	/** Create a column object for the specified table.
 	 *
 	 * @param tableID table in which the column belongs to
 	 * @param columnID name/id of the column
 	 */
-	public TableColumnImpl(String tableID, String columnID) {
+	public TableColumnImpl(Class forDataSourceType, String tableID,
+			String columnID) {
+		init(forDataSourceType, tableID, columnID);
+	}
+	
+	private void init(Class forDataSourceType, String tableID,
+			String columnID) {
+		this.forPluginDataSourceType = forDataSourceType;
 		sTableID = tableID;
 		sName = columnID;
 		iType = TYPE_TEXT_ONLY;
@@ -154,7 +206,7 @@ public class TableColumnImpl
 		int iSortDirection = COConfigurationManager.getIntParameter(CFG_SORTDIRECTION);
 		bSortAscending = iSortDirection == 1 ? false : true;
 	}
-
+	
 	public void initialize(int iAlignment, int iPosition, int iWidth,
 			int iInterval) {
 		if (bColumnAdded) {
@@ -352,6 +404,46 @@ public class TableColumnImpl
 		}
 	}
 
+	public void addCellOtherListener(String listenerID, Object listener) {
+		try {
+			this_mon.enter();
+			
+			if (mapOtherCellListeners == null) {
+				mapOtherCellListeners = new HashMap(1);
+			}
+
+			List list = (List) mapOtherCellListeners.get(listenerID);
+			if (list == null) {
+				list = new ArrayList(1);
+				mapOtherCellListeners.put(listenerID, list);
+			}
+
+			list.add(listener);
+
+		} finally {
+
+			this_mon.exit();
+		}
+	}
+	
+	public Object[] getCellOtherListeners(String listenerID) {
+		if (mapOtherCellListeners == null) {
+			return null;
+		}
+
+		List list = (List) mapOtherCellListeners.get(listenerID);
+		if (list == null) {
+			return null;
+		}
+		return list.toArray();
+	}
+	
+	// @see com.aelitis.azureus.ui.common.table.TableColumnCore#hasCellOtherListeners(java.lang.String)
+	public boolean hasCellOtherListeners(String listenerID) {
+		return mapOtherCellListeners != null
+				&& mapOtherCellListeners.get(listenerID) != null;
+	}
+
 	public List getCellAddedListeners() {
 		try {
 			this_mon.enter();
@@ -537,6 +629,52 @@ public class TableColumnImpl
 		}
 	}
 
+	public List<TableColumnExtraInfoListener> getColumnExtraInfoListeners() {
+		try {
+			this_mon.enter();
+
+			if (columnExtraInfoListeners == null) {
+				return (new ArrayList<TableColumnExtraInfoListener>(0));
+			}
+
+			return (new ArrayList<TableColumnExtraInfoListener>(columnExtraInfoListeners));
+
+		} finally {
+
+			this_mon.exit();
+		}
+	}
+
+	public void addColumnExtraInfoListener(TableColumnExtraInfoListener listener) {
+		try {
+			this_mon.enter();
+
+			if (columnExtraInfoListeners == null) {
+				columnExtraInfoListeners = new ArrayList(1);
+			}
+
+			columnExtraInfoListeners.add(listener);
+
+		} finally {
+			this_mon.exit();
+		}
+	}
+
+	public void removeColumnExtraInfoListener(TableColumnExtraInfoListener listener) {
+		try {
+			this_mon.enter();
+
+			if (columnExtraInfoListeners == null) {
+				return;
+			}
+
+			columnExtraInfoListeners.remove(listener);
+
+		} finally {
+			this_mon.exit();
+		}
+	}
+
 	public void invalidateCells() {
 		TableStructureEventDispatcher tsed = TableStructureEventDispatcher.getInstance(sTableID);
 		tsed.columnInvalidate(this);
@@ -575,6 +713,9 @@ public class TableColumnImpl
 		if (listenerObject instanceof TableCellVisibilityListener) {
 			addCellVisibilityListener((TableCellVisibilityListener) listenerObject);
 		}
+		if (listenerObject instanceof TableColumnExtraInfoListener) {
+			addColumnExtraInfoListener((TableColumnExtraInfoListener) listenerObject);
+		}
 	}
 
 	/* Start of not plugin public API functions */
@@ -749,7 +890,25 @@ public class TableColumnImpl
 		if(userData.size() < 1)
 			userData = null;
 	}
+	
+	public void
+	remove()
+	{
+		removed = true;
+		
+		TableColumnManager.getInstance().removeColumns( new TableColumnCore[]{ this });
+			
+		TableStructureEventDispatcher tsed = TableStructureEventDispatcher.getInstance(sTableID);
+					
+		tsed.tableStructureChanged(true, forPluginDataSourceType);
+	}
 
+	public boolean 
+	isRemoved() 
+	{
+		return( removed );
+	}
+	
 	public final void loadSettings(Map mapSettings) {
 		// Format: Key = [TableID].column.[columnname]
 		// Value[] = { visible, width, position, autotooltip, sortorder }
@@ -868,13 +1027,6 @@ public class TableColumnImpl
 				}
 
 				String sKeyPrefix;
-				// Try a generic one of "TableColumn." + columnid
-				sKeyPrefix = "TableColumn.header.";
-				if (MessageText.keyExists(sKeyPrefix + sName)) {
-					sTitleLanguageKey = sKeyPrefix + sName;
-					return sTitleLanguageKey;
-				}
-
 				// Support "Old Style" language keys, which have a prefix of TableID + "View."
 				// Also, "MySeeders" is actually stored in "MyTorrents"..
 				sKeyPrefix = (sTableID.equals(TableManager.TABLE_MYTORRENTS_COMPLETE)
@@ -901,7 +1053,15 @@ public class TableColumnImpl
 					}
 				}
 				
+				// Try a generic one of "TableColumn." + columnid
+				sKeyPrefix = "TableColumn.header.";
+				if (MessageText.keyExists(sKeyPrefix + sName)) {
+					sTitleLanguageKey = sKeyPrefix + sName;
+					return sTitleLanguageKey;
+				}
+
 				// another "Old Style"
+				// 99% sure this can be removed now, but why risk it..
 				sKeyPrefix = "MyTorrentsView." + sName;
 				//System.out.println(sKeyPrefix + ";" + MessageText.getString(sKeyPrefix));
 				if (MessageText.keyExists(sKeyPrefix)) {
@@ -925,30 +1085,52 @@ public class TableColumnImpl
 	}
 
 	public void removeContextMenuItem(TableContextMenuItem menuItem) {
-		if (menuItems == null) {
-			return;
+		if (menuItemsColumn != null) {
+			menuItemsColumn.remove(menuItem);
+		}
+		if (menuItemsHeader != null) {
+			menuItemsColumn.remove(menuItem);
 		}
-
-		menuItems.remove(menuItem);
 	}
 
+	// @see org.gudy.azureus2.plugins.ui.tables.TableColumn#addContextMenuItem(java.lang.String, int)
 	public TableContextMenuItem addContextMenuItem(String key) {
-		if (menuItems == null) {
-			menuItems = new ArrayList();
-		}
+		return addContextMenuItem(key, MENU_STYLE_COLUMN_DATA);
+	}
 
+	public TableContextMenuItem addContextMenuItem(String key, int menuStyle) {
+		ArrayList<TableContextMenuItem> menuItems;		
+		if (menuStyle == MENU_STYLE_COLUMN_DATA) {
+			if (menuItemsColumn == null) {
+				menuItemsColumn = new ArrayList<TableContextMenuItem>();
+			}
+			menuItems = menuItemsColumn;
+		} else {
+			if (menuItemsHeader == null) {
+				menuItemsHeader = new ArrayList<TableContextMenuItem>();
+			}
+			menuItems = menuItemsHeader;
+		}
+		
 		// Hack.. should be using our own implementation..
-		TableContextMenuItemImpl item = new TableContextMenuItemImpl("", key);
+		TableContextMenuItemImpl item = new TableContextMenuItemImpl(null,"", key);
 		menuItems.add(item);
 		return item;
 	}
 
-	public TableContextMenuItem[] getContextMenuItems() {
+	public TableContextMenuItem[] getContextMenuItems(int menuStyle) {
+		ArrayList<TableContextMenuItem> menuItems;		
+		if (menuStyle == MENU_STYLE_COLUMN_DATA) {
+			menuItems = menuItemsColumn;
+		} else {
+			menuItems = menuItemsHeader;
+		}
+
 		if (menuItems == null) {
 			return new TableContextMenuItem[0];
 		}
 
-		return (TableContextMenuItem[]) menuItems.toArray(new TableContextMenuItem[0]);
+		return menuItems.toArray(new TableContextMenuItem[0]);
 	}
 
 	public boolean isObfusticated() {
@@ -1162,6 +1344,9 @@ public class TableColumnImpl
 		//		+ " via " + Debug.getCompressedStackTrace());
 
 		bVisible = visible;
+		if (bVisible && iPosition == POSITION_INVISIBLE) {
+			iPosition = POSITION_LAST;
+		}
 		invalidateCells();
 	}
 
@@ -1252,4 +1437,12 @@ public class TableColumnImpl
 		return false;
 	}
 
+	public Class getForDataSourceType() {
+		return forPluginDataSourceType;
+	}
+
+	public void setForDataSourceType(Class forDataSourceType) {
+		this.forPluginDataSourceType = forDataSourceType;
+	}
+
 }
