@@ -44,6 +44,7 @@ import org.gudy.azureus2.ui.swt.updater2.PreUpdateChecker;
 import org.gudy.azureus2.ui.swt.updater2.SWTUpdateChecker;
 
 import com.aelitis.azureus.core.*;
+import com.aelitis.azureus.core.cnetwork.ContentNetwork;
 import com.aelitis.azureus.core.messenger.ClientMessageContext;
 import com.aelitis.azureus.core.messenger.PlatformMessenger;
 import com.aelitis.azureus.core.messenger.config.PlatformConfigMessenger;
@@ -51,12 +52,15 @@ import com.aelitis.azureus.core.util.CopyOnWriteList;
 import com.aelitis.azureus.launcher.Launcher;
 import com.aelitis.azureus.ui.IUIIntializer;
 import com.aelitis.azureus.ui.InitializerListener;
+import com.aelitis.azureus.ui.UIFunctionsManager;
+import com.aelitis.azureus.ui.swt.browser.PlatformAuthorizedSenderImpl;
 import com.aelitis.azureus.ui.swt.browser.listener.*;
 import com.aelitis.azureus.ui.swt.browser.msg.MessageDispatcherSWT;
+import com.aelitis.azureus.ui.swt.content.RelatedContentUI;
+import com.aelitis.azureus.ui.swt.devices.DeviceManagerUI;
 import com.aelitis.azureus.ui.swt.shells.main.MainWindow;
+import com.aelitis.azureus.ui.swt.subscriptions.SubscriptionManagerUI;
 import com.aelitis.azureus.ui.swt.utils.UIMagnetHandler;
-import com.aelitis.azureus.ui.swt.utils.UIUpdaterFactory;
-import com.aelitis.azureus.util.Constants;
 import com.aelitis.azureus.util.InitialisationFunctions;
 
 /**
@@ -67,6 +71,12 @@ import com.aelitis.azureus.util.InitialisationFunctions;
 public class Initializer
 	implements IUIIntializer
 {
+	// Whether to initialize the UI before the core has been started
+	private static boolean STARTUP_UIFIRST = System.getProperty("ui.startfirst", "1").equals("1");
+
+	// Used in debug to find out how long initialization took
+	public static final long startTime = System.currentTimeMillis();
+
 	private static StartServer startServer;
 
 	private final AzureusCore core;
@@ -80,7 +90,11 @@ public class Initializer
 	private int curPercent = 0;
 
 	private AESemaphore init_task = new AESemaphore("delayed init");
-	  
+
+	private MainWindow mainWindow;
+	
+	private static Initializer lastInitializer;
+
 	public static void main(final String args[]) {
 		if (Launcher.checkAndLaunch(Initializer.class, args))
 			return;
@@ -101,6 +115,7 @@ public class Initializer
 			String[] args) {
 		this.core = core;
 		this.args = args;
+		lastInitializer = this;
 
 		if (createSWTThreadAndRun) {
 			try {
@@ -109,14 +124,34 @@ public class Initializer
 				Debug.printStackTrace(e);
 			}
 		} else {
-			Constants.initialize(core);
 
 			initializePlatformClientMessageContext();
 
-			PlatformConfigMessenger.login(0);
+			PlatformMessenger.setAuthorizedTransferListener(new PlatformAuthorizedSenderImpl());
+
+			PlatformConfigMessenger.login(ContentNetwork.CONTENT_NETWORK_VUZE, 0);
 			// typically the caller will call run() now 
 		}
 	}
+	
+	public void runInSWTThread() {
+		COConfigurationManager.setBooleanDefault("ui.startfirst", true);
+		STARTUP_UIFIRST = STARTUP_UIFIRST
+				&& COConfigurationManager.getBooleanParameter("ui.startfirst", true);
+		
+		if (!STARTUP_UIFIRST) {
+			return;
+		}
+
+		// Ensure colors initialized
+		Colors.getInstance();
+
+		UIConfigDefaultsSWT.initialize();
+
+		UIConfigDefaultsSWTv3.initialize(core);
+
+		mainWindow = new MainWindow(Display.getDefault(), this);
+	}
 
 	public void run() {
 		
@@ -129,7 +164,7 @@ public class Initializer
 					}
 				});
 
-		delayed_task.queue();
+		delayed_task.queueFirst();
 		
 		// initialise the SWT locale util
 		long startTime = SystemTime.getCurrentTime();
@@ -139,15 +174,16 @@ public class Initializer
 		final Display display = SWTThread.getInstance().getDisplay();
 
 		new UIMagnetHandler(core);
-
-		UIConfigDefaultsSWT.initialize();
-
-		UIConfigDefaultsSWTv3.initialize(core);
-
-		ImageRepository.loadImagesForSplashWindow(display);
+		
+		if (!STARTUP_UIFIRST) {
+			UIConfigDefaultsSWT.initialize();
+			UIConfigDefaultsSWTv3.initialize(core);
+		} else {
+			COConfigurationManager.setBooleanDefault("Show Splash", false);
+		}
 
 		if (COConfigurationManager.getBooleanParameter("Show Splash")) {
-			display.syncExec(new AERunnable() {
+			display.asyncExec(new AERunnable() {
 				public void runSupport() {
 					new SplashWindow(display, Initializer.this);
 				}
@@ -181,9 +217,10 @@ public class Initializer
 				}
 
 				if (sLastTask != null && !sLastTask.startsWith("Loading Torrent")) {
+						
 					long diff = now - startTime;
 					if (diff > 10 && diff < 1000 * 60 * 5) {
-						System.out.println("   Core: " + diff + "ms for " + sLastTask);
+						System.out.println(TimeFormatter.milliStamp() + "   Core: " + diff + "ms for activity between '" + sLastTask + "' and '" + currentTask + "'");
 					}
 					startTime = SystemTime.getCurrentTime();
 				}
@@ -192,6 +229,7 @@ public class Initializer
 			}
 
 			public void reportPercent(AzureusCoreOperation op, int percent) {
+				/*
 				if (op.getOperationType() != AzureusCoreOperation.OP_INITIALISATION) {
 					return;
 				}
@@ -202,6 +240,7 @@ public class Initializer
 						System.out.println("   Core: " + diff + "ms for " + sLastTask);
 					}
 				}
+				*/
 				// TODO Auto-generated method stub
 			}
 
@@ -248,7 +287,11 @@ public class Initializer
 					
 					main_window_will_report_complete = true;
 					
-					new MainWindow(core, Display.getDefault(), Initializer.this);
+					if (STARTUP_UIFIRST) {
+						mainWindow.init(core);
+					} else {
+						new MainWindow(core, Display.getDefault(), Initializer.this);
+					}
 					
 					reportCurrentTaskByKey("splash.openViews");
 	
@@ -313,6 +356,30 @@ public class Initializer
 
 		reportCurrentTaskByKey("splash.initializeCore");
 
+		try{
+			new SubscriptionManagerUI();
+			
+		}catch( Throwable e ){
+			
+			Debug.printStackTrace(e);
+		}
+		
+		try{
+			new RelatedContentUI();
+			
+		}catch( Throwable e ){
+			
+			Debug.printStackTrace(e);
+		}
+		
+		try{
+			new DeviceManagerUI( core );
+				
+		}catch( Throwable e ){
+				
+			Debug.printStackTrace(e);
+		}
+		
 		core.start();
 
 		reportPercent(50);
@@ -359,7 +426,11 @@ public class Initializer
 
 			//			Cursors.dispose();
 
-			UIUpdaterFactory.getInstance().stopIt();
+			try {
+				UIFunctionsManager.getUIFunctions().getUIUpdater().stopIt();
+			} catch (Exception e) {
+				Debug.out(e);
+			}
 
 			Utils.execSWTThread(new AERunnable() {
 				public void runSupport() {
@@ -498,12 +569,13 @@ public class Initializer
 
 		  new DelayedEvent( 
 				  "SWTInitComplete:delay",
-				  8500,
+				  2500,
 				  new AERunnable()
 				  {
 					  public void
 					  runSupport()
 					  {
+					  	//System.out.println("Release Init. Task");
 						  init_task.release();
 					  }
 				  });
@@ -519,12 +591,16 @@ public class Initializer
 		if (clientMsgContext != null) {
 			clientMsgContext.setMessageDispatcher(new MessageDispatcherSWT(clientMsgContext));
 			clientMsgContext.addMessageListener(new TorrentListener());
+			clientMsgContext.addMessageListener(new VuzeListener());
 			clientMsgContext.addMessageListener(new DisplayListener(null));
 			clientMsgContext.addMessageListener(new ConfigListener(null));
 			clientMsgContext.addMessageListener(new LightBoxBrowserRequestListener());
 			clientMsgContext.addMessageListener(new StatusListener());
 			clientMsgContext.addMessageListener(new BrowserRpcBuddyListener());
-			clientMsgContext.addMessageListener(new MetaSearchListener());
 		}
 	}
+
+	public static Initializer getLastInitializer() {
+		return lastInitializer;
+	}
 }
