@@ -81,7 +81,10 @@ public class UIConfigDefaultsSWTv3
 		}
 
 		boolean virginSwitch = config.getBooleanParameter("az3.virgin.switch", false);
-		if (Constants.compareVersions(sFirstVersion, "3.0.0.0") >= 0 || virginSwitch) {
+		boolean immediateSwitch = config.getBooleanParameter(
+				"az3.switch.immediate", false);
+		if (Constants.compareVersions(sFirstVersion, "3.0.0.0") >= 0
+				|| immediateSwitch) {
 
 			if (!config.isNewInstall()
 					&& Constants.compareVersions(sFirstVersion, "3.0.0.4") < 0) {
@@ -120,26 +123,21 @@ public class UIConfigDefaultsSWTv3
 			defaults.addParameter("Use default data dir", true);
 			defaults.addParameter("Add URL Silently", true);
 			defaults.addParameter("add_torrents_silently", true);
-			defaults.addParameter("Popup Download Finished", true);
-			defaults.addParameter("Popup Download Added", true);
+			defaults.addParameter("Popup Download Finished", false);
+			defaults.addParameter("Popup Download Added", false);
 
 			defaults.addParameter("Status Area Show SR", false);
 			defaults.addParameter("Status Area Show NAT", false);
 			defaults.addParameter("Status Area Show IPF", false);
 			
-			defaults.addParameter("Message Popup Autoclose in Seconds", 5);
+			defaults.addParameter("Message Popup Autoclose in Seconds", 10 );
 
 			defaults.addParameter("window.maximized", true);
 
 			defaults.addParameter("update.autodownload", true);
 			
-			defaults.addParameter("bFirstPriority_ignore0Peer", true);
-
 			defaults.addParameter("suppress_file_download_dialog", true);
 			
-			// New style messagebox is not ready yet for full usage
-			defaults.addParameter("useNewStyleMessageBox", false);
-			
 			defaults.addParameter("auto_remove_inactive_items", false);
 			
 			defaults.addParameter("show_torrents_menu", false);
@@ -147,19 +145,31 @@ public class UIConfigDefaultsSWTv3
 			config.save();
 		}
 
+
 		defaults.addParameter("v3.topbar.show.frog", false);
 		defaults.addParameter("v3.topbar.show.plugin", false);
 		defaults.addParameter("ui.toolbar.uiswitcher", false);
 		defaults.addParameter(SkinConstants.VIEWID_PLUGINBAR + ".visible", false);
-		defaults.addParameter("v3.home-tab.starttab", SkinConstants.VIEWID_ACTIVITY_TAB);
+		config.removeParameter("v3.home-tab.starttab");
 		defaults.addParameter("v3.topbar.height", 60);
+		defaults.addParameter("v3.Start Advanced", false);
+		defaults.addParameter("MyTorrentsView.table.style", 0);
+		defaults.addParameter("v3.Show Welcome", true);
 		
+		boolean startAdvanced = config.getBooleanParameter("v3.Start Advanced");
+		defaults.addParameter("Library.viewmode", startAdvanced ? 1 : 0);
+		defaults.addParameter("LibraryDL.viewmode", startAdvanced ? 1 : 0);
+		defaults.addParameter("LibraryUnopened.viewmode", startAdvanced ? 1 : 0);
+		defaults.addParameter("LibraryCD.viewmode", startAdvanced ? 1 : 0);
 		
+		defaults.addParameter("list.dm.dblclick", "0");
+
 		//=== defaults used by MainWindow
 		defaults.addParameter("vista.adminquit", false);
 		defaults.addParameter("Password enabled", false);
 		defaults.addParameter("Start Minimized", false);
 		defaults.addParameter("Password enabled", false);
+		defaults.addParameter("ToolBar.showText", true);
 		
 
 		// by default, turn off some slidey warning
