@@ -22,27 +22,34 @@ package com.aelitis.azureus.ui.swt.browser;
 import java.net.HttpURLConnection;
 import java.net.URL;
 import java.net.URLConnection;
-import java.util.Collection;
-import java.util.Map;
+import java.util.*;
+import java.util.List;
 
 import org.eclipse.swt.SWT;
 import org.eclipse.swt.browser.*;
 import org.eclipse.swt.events.DisposeEvent;
 import org.eclipse.swt.events.DisposeListener;
+import org.eclipse.swt.graphics.Color;
 import org.eclipse.swt.graphics.Point;
+import org.eclipse.swt.layout.FillLayout;
 import org.eclipse.swt.program.Program;
 import org.eclipse.swt.widgets.*;
 
 import org.gudy.azureus2.core3.util.*;
+import org.gudy.azureus2.plugins.utils.StaticUtilities;
+import org.gudy.azureus2.plugins.utils.resourcedownloader.ResourceDownloader;
+import org.gudy.azureus2.pluginsimpl.local.PluginInitializer;
 import org.gudy.azureus2.ui.swt.Utils;
 
-import com.aelitis.azureus.core.impl.AzureusCoreImpl;
 import com.aelitis.azureus.core.messenger.ClientMessageContextImpl;
 import com.aelitis.azureus.core.messenger.browser.listeners.BrowserMessageListener;
-import com.aelitis.azureus.core.messenger.config.PlatformConfigMessenger;
+import com.aelitis.azureus.core.vuzefile.VuzeFile;
+import com.aelitis.azureus.core.vuzefile.VuzeFileHandler;
+import com.aelitis.azureus.plugins.net.netstatus.swt.NetStatusPluginTester.loggerProvider;
 import com.aelitis.azureus.ui.swt.browser.msg.MessageDispatcherSWT;
-import com.aelitis.azureus.util.Constants;
+import com.aelitis.azureus.util.ConstantsVuze;
 import com.aelitis.azureus.util.JSONUtils;
+import com.aelitis.azureus.util.UrlFilter;
 
 /**
  * Manages the context for a single SWT {@link Browser} component,
@@ -64,6 +71,8 @@ public class BrowserContext
 	private Display display;
 
 	private boolean pageLoading = false;
+	
+	private long pageLoadingStart = 0;
 
 	private String lastValidURL = null;
 
@@ -72,54 +81,48 @@ public class BrowserContext
 	private TimerEventPeriodic checkURLEvent;
 
 	private Control widgetWaitIndicator;
-
+	
 	private MessageDispatcherSWT messageDispatcherSWT;
 
 	protected boolean wiggleBrowser = org.gudy.azureus2.core3.util.Constants.isOSX;
 
+	private torrentURLHandler		torrentURLHandler;
+
+	private List loadingListeners = Collections.EMPTY_LIST;
+
+	private long pageLoadTime;
+	
+	private long contentNetworkID = ConstantsVuze.DEFAULT_CONTENT_NETWORK_ID;
+	
+	private AEMonitor mon_listJS = new AEMonitor("listJS");
+	
+	private List<String> listJS = new ArrayList<String>(1);
+	
 	/**
 	 * Creates a context and registers the given browser.
 	 * 
 	 * @param id unique identifier of this context
 	 * @param browser the browser to be registered
 	 */
-	public BrowserContext(String id, Browser browser,
-			Control widgetWaitingIndicator, boolean forceVisibleAfterLoad) {
-		this(id, forceVisibleAfterLoad);
-		registerBrowser(browser, widgetWaitingIndicator);
-	}
-
-	/**
-	 * Creates a context without a registered browser.
-	 * This method should rarely be used.
-	 * 
-	 * @param id unique identifier of this context
-	 */
-	public BrowserContext(String id, boolean forceVisibleAfterLoad) {
-		super(id, null);
-		messageDispatcherSWT = new MessageDispatcherSWT(this);
-		setMessageDispatcher(messageDispatcherSWT);
-		this.forceVisibleAfterLoad = forceVisibleAfterLoad;
-	}
-
-	public void registerBrowser(Object oBrowser,
-			Object oWidgetWaitIndicator) {
-		if (this.browser != null) {
-			throw new IllegalStateException("Context " + getID()
-					+ " already has a registered browser");
-		}
-
-		if (oBrowser instanceof Browser) {
-			this.browser = (Browser) oBrowser;
-		} else {
-			throw new IllegalStateException("Context " + getID()
-					+ ": brwoser isn't a Browser");
-		}
-
-		if (oWidgetWaitIndicator instanceof Control) {
-			this.widgetWaitIndicator = (Control) oWidgetWaitIndicator;
-		}
+	public 
+	BrowserContext(
+		String 		_id, 
+		Browser 	_browser,
+		Control 	_widgetWaitingIndicator, 
+		boolean 	_forceVisibleAfterLoad ) 
+	{
+		super( _id, null );
 		
+		browser 				= _browser;
+		forceVisibleAfterLoad 	= _forceVisibleAfterLoad;
+		widgetWaitIndicator 	= _widgetWaitingIndicator;
+
+		// System.out.println( "Registered browser context: id=" + getID());
+		
+		messageDispatcherSWT = new MessageDispatcherSWT(this);
+		
+		setMessageDispatcher( messageDispatcherSWT );
+				
 		final TimerEventPerformer showBrowersPerformer = new TimerEventPerformer() {
 			public void perform(TimerEvent event) {
 				if (browser != null && !browser.isDisposed()) {
@@ -137,6 +140,7 @@ public class BrowserContext
 
 		final TimerEventPerformer hideIndicatorPerformer = new TimerEventPerformer() {
 			public void perform(TimerEvent event) {
+				setPageLoading(false, browser.getUrl());
 				if (widgetWaitIndicator != null && !widgetWaitIndicator.isDisposed()) {
 					Utils.execSWTThread(new AERunnable() {
 						public void runSupport() {
@@ -173,13 +177,19 @@ public class BrowserContext
 		};
 
 		if (forceVisibleAfterLoad) {
+			
 			browser.setVisible(false);
 		}
+		
+		setPageLoading(false, browser.getUrl());
+		
 		if (widgetWaitIndicator != null && !widgetWaitIndicator.isDisposed()) {
 			widgetWaitIndicator.setVisible(false);
 		}
+		
 		browser.addTitleListener(new TitleListener() {
 			public void changed(TitleEvent event) {
+				
 				/*
 				 * The browser might have been disposed already by the time this method is called 
 				 */
@@ -192,7 +202,7 @@ public class BrowserContext
 							System.currentTimeMillis() + 700, showBrowersPerformer);
 				}
 				if (event.title.startsWith("err: ")) {
-					fillWithRetry(event.title);
+					fillWithRetry(event.title, "err in title");
 				}
 			}
 		});
@@ -217,7 +227,7 @@ public class BrowserContext
 				}
 
 				browser.execute("try { if (azureusClientWelcome) { azureusClientWelcome('"
-						+ Constants.AZID
+						+ ConstantsVuze.AZID
 						+ "',"
 						+ "{ 'azv':'"
 						+ org.gudy.azureus2.core3.util.Constants.AZUREUS_VERSION
@@ -250,27 +260,41 @@ public class BrowserContext
 		browser.addOpenWindowListener(new OpenWindowListener() {
 			public void open(WindowEvent event) {
 				event.required = true;
-
-				final Browser subBrowser = new Browser(browser,SWT.NONE);
-				subBrowser.addLocationListener(new LocationListener() {
-					public void changed(LocationEvent arg0) {
-						// TODO Auto-generated method stub
-						
-					}
-					public void changing(LocationEvent event) {
-						event.doit = false;
-						System.out.println("SubBrowser URL : " + event.location);
-						if(event.location.startsWith("http://") || event.location.startsWith("https://")) {
-							Program.launch(event.location);
-						}
-						Utils.execSWTThreadLater(0, new AERunnable() {
-							public void runSupport() {
-								subBrowser.dispose();
-							}
-						});
-					}
-				});
-				event.browser = subBrowser;
+				
+				if (browser.getUrl().contains("js.debug=1")) {
+  				Shell shell = new Shell(Utils.findAnyShell(), SWT.SHELL_TRIM);
+  				shell.setLayout(new FillLayout());
+					Browser subBrowser = new Browser(shell,
+							Utils.getInitialBrowserStyle(SWT.NONE));
+					shell.open();
+					event.browser = subBrowser;
+				} else {
+				
+  				final Browser subBrowser = new Browser(browser,
+  						Utils.getInitialBrowserStyle(SWT.NONE));
+  				subBrowser.addLocationListener(new LocationListener() {
+  					public void changed(LocationEvent arg0) {
+  						// TODO Auto-generated method stub
+  						
+  					}
+  					public void changing(LocationEvent event) {
+  						event.doit = false;
+  						if (!UrlFilter.getInstance().urlIsBlocked(event.location)
+  								&& (event.location.startsWith("http://") || event.location.startsWith("https://"))) {
+  							debug("open sub browser: " + event.location);
+  							Program.launch(event.location);
+  						} else {
+  							debug("blocked open sub browser: " + event.location);
+  						}
+  						Utils.execSWTThreadLater(0, new AERunnable() {
+  							public void runSupport() {
+  								subBrowser.dispose();
+  							}
+  						});
+  					}
+  				});
+					event.browser = subBrowser;
+				}
 			}
 		});
 		
@@ -278,55 +302,81 @@ public class BrowserContext
 			private TimerEvent timerevent;
 
 			public void changed(LocationEvent event) {
-				//System.out.println("cd" + event.location);
+				debug("browser.changed " + event.location);
 				if (timerevent != null) {
 					timerevent.cancel();
 				}
 				checkURLEventPerformer.perform(null);
+				setPageLoading(false, event.top ? event.location : null);
 				if (widgetWaitIndicator != null && !widgetWaitIndicator.isDisposed()) {
 					widgetWaitIndicator.setVisible(false);
 				}
-			}
 
-			public void changing(LocationEvent event) {
-				//System.out.println("cing " + event.location); 
-				
-				/*
-				 * The browser might have been disposed already by the time this method is called 
-				 */
-				if(true == browser.isDisposed()){
-					return;
-				}
-				
-				if (event.location.startsWith("javascript")
-						&& event.location.indexOf("back()") > 0) {
-					if (browser.isBackEnabled()) {
-						browser.back();
-					} else if (lastValidURL != null) {
-						fillWithRetry(event.location);
-					}
+				// event.top is only filled on changed event (not changing!)
+				if (!event.top) {
 					return;
 				}
 				boolean isWebURL = event.location.startsWith("http://")
 						|| event.location.startsWith("https://");
 				if (!isWebURL) {
-					if (event.location.startsWith("res://") && lastValidURL != null) {
-						fillWithRetry(event.location);
+					if (event.location.startsWith("res://")) {
+						fillWithRetry(event.location, "top changed");
+						return;
 					}
 					// we don't get a changed state on non URLs (mailto, javascript, etc)
+				}
+
+				if (UrlFilter.getInstance().isWhitelisted(event.location)) {
+					lastValidURL = event.location;
+				}
+
+				//System.out.println("cd" + event.location);
+			}
+
+			public void changing(LocationEvent event) {
+				debug("browser.changing " + event.location);
+				/*
+				 * The browser might have been disposed already by the time this method is called 
+				 */
+				
+				if ( browser.isDisposed()){
+					return;
+				}
+				
+				String event_location = event.location;
+				
+				//Utils.openMessageBox(Utils.findAnyShell(), SWT.OK, "Location Changing", "Navigating to " + event_location );
+
+				if (event_location.startsWith("javascript")
+						&& event_location.indexOf("back()") > 0) {
+					if (browser.isBackEnabled()) {
+						browser.back();
+					} else if (lastValidURL != null) {
+						fillWithRetry(event_location, "back");
+					}
+					return;
+				}
+
+				boolean isWebURL = event_location.startsWith("http://")
+						|| event_location.startsWith("https://");
+				if (!isWebURL) {
+					// we don't get a changed state on non URLs (mailto, javascript, etc)
 					return;
 				}
 
-				boolean blocked = PlatformConfigMessenger.isURLBlocked(event.location);
+				boolean blocked = UrlFilter.getInstance().urlIsBlocked(event_location);
 
 				if (blocked) {
 					event.doit = false;
 					Utils.openMessageBox(Utils.findAnyShell(), SWT.OK, "URL blocked",
-							"Tried to open " + event.location + " but it's blocked");
+							"Tried to open " + event_location + " but it's blocked");
 					browser.back();
 				} else {
+					if (UrlFilter.getInstance().isWhitelisted(event_location)) {
+						lastValidURL = event_location;
+					}
+					setPageLoading(true, event.location);
 					if(event.top) {
-						lastValidURL = event.location;
 						if (widgetWaitIndicator != null && !widgetWaitIndicator.isDisposed()) {
 							widgetWaitIndicator.setVisible(true);
 						}
@@ -335,17 +385,25 @@ public class BrowserContext
 						timerevent = SimpleTimer.addEvent("Hide Indicator",
 								System.currentTimeMillis() + 20000, hideIndicatorPerformer);
 					} else {
-						boolean isTorrent = false;
+						boolean isTorrent 	= false;
+						boolean isVuzeFile	= false;
+						
 						//Try to catch .torrent files
-						if(event.location.endsWith(".torrent")) {
+						if(event_location.endsWith(".torrent")) {
 							isTorrent = true;
 						} else {
 							//If it's not obviously a web page
-							if (!PlatformConfigMessenger.urlCanRPC(event.location)
-									&& event.location.indexOf(".htm") == -1) {
+							
+							boolean	can_rpc = UrlFilter.getInstance().urlCanRPC(event_location);
+							
+							boolean	test_for_torrent 	= !can_rpc && event_location.indexOf(".htm") == -1;
+							boolean	test_for_vuze		= can_rpc &&  ( event_location.endsWith( ".xml" ) || event_location.endsWith( ".vuze" ));
+							
+							if ( test_for_torrent || test_for_vuze ){
+								
 								try {
 									//See what the content type is
-									URL url = new URL(event.location);
+									URL url = new URL(event_location);
 									URLConnection conn = url.openConnection();
 									
 										// we're only trying to get the content type so just use head
@@ -371,22 +429,46 @@ public class BrowserContext
 									
 									String contentType = conn.getContentType();
 									
-									if(contentType != null && contentType.indexOf("torrent") != -1) {
-										isTorrent = true;
-									}
-									String contentDisposition = conn.getHeaderField("Content-Disposition");
-									if(contentDisposition != null && contentDisposition.indexOf(".torrent") != -1) {
-										isTorrent = true;
+									if ( contentType != null ){
+										
+										if ( test_for_torrent && contentType.indexOf("torrent") != -1 ) {
+									
+											isTorrent = true;
+										}
+										
+										if ( test_for_vuze && contentType.indexOf("vuze") != -1 ) {
+											
+											isVuzeFile = true;
+										}
 									}
 									
-								} catch(Exception e) {
-									e.printStackTrace();
+									String contentDisposition = conn.getHeaderField("Content-Disposition");
+									
+									if (contentDisposition != null ){
+										
+										if ( test_for_torrent && contentDisposition.indexOf(".torrent") != -1) {
+									
+											isTorrent = true;
+										}
+										
+										if ( test_for_vuze && contentDisposition.indexOf(".vuze") != -1) {
+											
+											isVuzeFile = true;
+										}
+			
+									}
+									
+								}catch( Throwable e){
 								}
+								
+								//System.out.println( "Test for t/v: " + event_location + " -> " + isTorrent + "/" + isVuzeFile );
 							}
 						}
 						
-						if(isTorrent) {
+						if ( isTorrent ){
+							
 							event.doit = false;
+							
 							try {
 								String referer_str = null;
 
@@ -395,13 +477,75 @@ public class BrowserContext
 
 								}catch( Throwable e ){
 								}
-								
+																
 								Map headers = UrlUtils.getBrowserHeaders( referer_str );
+											
 								
-								AzureusCoreImpl.getSingleton().getPluginManager().getDefaultPluginInterface().getDownloadManager().addDownload(
-										new URL(event.location), headers );
+								String cookies = (String) ((Browser)event.widget).getData("current-cookies");
+								
+								if (cookies != null ){
+									
+									headers.put("Cookie", cookies);
+								}
+								
+								String	url = event_location;
+								
+								if ( torrentURLHandler != null ){
+									
+									try{
+										torrentURLHandler.handleTorrentURL(url);
+										
+									}catch( Throwable e ){
+										
+										Debug.printStackTrace(e);
+									}
+								}
+								
+								PluginInitializer.getDefaultInterface().getDownloadManager().addDownload(
+										new URL(url), headers );
+								
+							}catch( Throwable e ){
+								
+								e.printStackTrace();
+							}
+						}else if ( isVuzeFile ){
+							
+							event.doit = false;
+							
+							try {
+								String referer_str = null;
+
+								try{
+									referer_str = new URL(((Browser)event.widget).getUrl()).toExternalForm();
+
+								}catch( Throwable e ){
+								}
+																
+								Map headers = UrlUtils.getBrowserHeaders( referer_str );
+																		
+								String cookies = (String) ((Browser)event.widget).getData("current-cookies");
+								
+								if ( cookies != null ){
+									
+									headers.put("Cookie", cookies);
+								}
+								
+								ResourceDownloader rd = StaticUtilities.getResourceDownloaderFactory().create( new URL( event_location ));
+								
+								VuzeFileHandler vfh = VuzeFileHandler.getSingleton();
+								
+								VuzeFile vf = vfh.loadVuzeFile( rd.download());
+								
+								if ( vf == null ){
+									
+									event.doit = true;
+									
+								}else{
+									
+									vfh.handleFiles( new VuzeFile[]{ vf }, 0 );
+								}
+							}catch( Throwable e ){
 								
-							} catch(Exception e) {
 								e.printStackTrace();
 							}
 						}
@@ -427,23 +571,83 @@ public class BrowserContext
 		this.display = browser.getDisplay();
 	}
 
-	public void fillWithRetry(String s) {
-		browser.setText("<html><body style='overflow:auto; font-family: verdana; font-size: 10pt' bgcolor=#1b1b1b text=#a0a0a0>"
+	/**
+	 * @param b
+	 * @param url 
+	 *
+	 * @since 3.1.1.1
+	 */
+	protected void setPageLoading(boolean b, String url) {
+		// we may get multiple "load done"s (from each frame) which we don't
+		// want to skip
+		if (b && pageLoading) {
+			return;
+		}
+		mon_listJS.enter();
+		try {
+  		pageLoading = b;
+  		if (pageLoading) {
+  			pageLoadingStart = SystemTime.getCurrentTime();
+  			pageLoadTime = -1;
+  		} else if (pageLoadingStart > 0 && url != null) {
+  			pageLoadTime = SystemTime.getCurrentTime() - pageLoadingStart;
+  			executeInBrowser("clientSetLoadTime(" + pageLoadTime + ");");
+  			
+  			pageLoadingStart = 0;
+  		}
+  		if (!pageLoading && listJS.size() > 0) {
+  			debug(listJS.size() + " javascripts queued.  Executing now..");
+  			for (String js : listJS) {
+					executeInBrowser(js);
+				}
+  			listJS.clear();
+  		}
+		} finally {
+			mon_listJS.exit();
+		}
+		
+		Object[] listeners = loadingListeners.toArray();
+		for (int i = 0; i < listeners.length; i++) {
+			loadingListener l = (loadingListener) listeners[i];
+			l.browserLoadingChanged(b, url);
+		}
+	}
+
+	public void 
+	setTorrentURLHandler(
+		torrentURLHandler handler) 
+	{
+		torrentURLHandler = handler;
+	}
+	
+	public void fillWithRetry(String s, String s2) {
+		Color bg = browser.getDisplay().getSystemColor(SWT.COLOR_WIDGET_BACKGROUND);
+		Color fg = browser.getDisplay().getSystemColor(SWT.COLOR_WIDGET_FOREGROUND);
+		
+		browser.setText("<html><body style='overflow:auto; font-family: verdana; font-size: 10pt' bgcolor=#"
+				+ Utils.toColorHexString(bg)
+				+ " text=#" + Utils.toColorHexString(fg) + ">"
 				+ "<br>Sorry, there was a problem loading this page.<br> "
 				+ "Please check if your internet connection is working and click <a href='"
 				+ lastValidURL
 				+ "' style=\"color: rgb(100, 155, 255); \">retry</a> to continue."
-				+ "<div style='word-wrap: break-word'><font size=1 color=#2e2e2e>"
-				+ s
+				+ "<div style='word-wrap: break-word'><font size=1 color=#"
+				+ Utils.toColorHexString(bg)
+				+ ">"
+				+ s + "<br><br>" + s2
 				+ "</font></div>" + "</body></html>");
 	}
-
-	public void deregisterBrowser() {
+	
+	private void 
+	deregisterBrowser() 
+	{
 		if (browser == null) {
 			throw new IllegalStateException("Context " + getID()
 					+ " doesn't have a registered browser");
 		}
 
+		// System.out.println( "Unregistered browser context: id=" + getID());
+	
 		browser.setData(CONTEXT_KEY, null);
 		browser.removeDisposeListener(this);
 		messageDispatcherSWT.deregisterBrowser(browser);
@@ -517,11 +721,16 @@ public class BrowserContext
 	}
 
 	public boolean executeInBrowser(final String javascript) {
-		if (!mayExecute(javascript)) {
-			debug("BLOCKED: browser.execute( " + getShortJavascript(javascript)
-					+ " )");
-			return false;
+		mon_listJS.enter();
+		try {
+			if (!mayExecute(javascript)) {
+				listJS.add(javascript);
+				return false;
+			}
+		} finally {
+			mon_listJS.exit();
 		}
+
 		if (display == null || display.isDisposed()) {
 			debug("CANNOT: browser.execute( " + getShortJavascript(javascript) + " )");
 			return false;
@@ -571,4 +780,28 @@ public class BrowserContext
 	public void setWiggleBrowser(boolean wiggleBrowser) {
 		this.wiggleBrowser = wiggleBrowser;
 	}
+
+	public boolean isPageLoading() {
+		return pageLoading;
+	}
+
+	
+	public void addListener(loadingListener l) {
+		if (loadingListeners == Collections.EMPTY_LIST) {
+			loadingListeners = new ArrayList(1);
+		}
+		loadingListeners.add(l);
+	}
+	
+	public static interface loadingListener {
+		public void browserLoadingChanged(boolean loading, String url);
+	}
+
+	public long getContentNetworkID() {
+		return contentNetworkID;
+	}
+
+	public void setContentNetworkID(long contentNetworkID) {
+		this.contentNetworkID = contentNetworkID;
+	}
 }
\ No newline at end of file
