@@ -25,8 +25,9 @@ import org.gudy.azureus2.core3.download.DownloadManager;
 import org.gudy.azureus2.ui.swt.pluginsimpl.UISWTGraphicImpl;
 import org.gudy.azureus2.ui.swt.views.table.utils.CoreTableColumn;
 
-import com.aelitis.azureus.ui.swt.utils.ImageLoaderFactory;
+import com.aelitis.azureus.ui.swt.imageloader.ImageLoader;
 
+import org.gudy.azureus2.plugins.download.Download;
 import org.gudy.azureus2.plugins.ui.tables.TableCell;
 import org.gudy.azureus2.plugins.ui.tables.TableCellAddedListener;
 import org.gudy.azureus2.plugins.ui.tables.TableCellRefreshListener;
@@ -40,6 +41,8 @@ public class ColumnIsPrivate
 	extends CoreTableColumn
 	implements TableCellAddedListener, TableCellRefreshListener
 {
+	public static final Class DATASOURCE_TYPE = Download.class;
+
 	public static String COLUMN_ID = "IsPrivate";
 
 	private static UISWTGraphicImpl graphicCheck;
@@ -47,7 +50,7 @@ public class ColumnIsPrivate
 	private static int width;
 
 	static {
-		Image img = ImageLoaderFactory.getInstance().getImage("image.check");
+		Image img = ImageLoader.getInstance().getImage("image.check");
 		width = img.getBounds().width;
 		graphicCheck = new UISWTGraphicImpl(img);
 	}
@@ -56,9 +59,8 @@ public class ColumnIsPrivate
 	 * 
 	 */
 	public ColumnIsPrivate(String sTableID) {
-		super(COLUMN_ID, sTableID);
-		initializeAsGraphic(POSITION_LAST, width);
-		setAlignment(ALIGN_CENTER);
+		super(DATASOURCE_TYPE, COLUMN_ID, ALIGN_CENTER, width, sTableID);
+		initializeAsGraphic(width);
 	}
 
 	// @see org.gudy.azureus2.plugins.ui.tables.TableCellAddedListener#cellAdded(org.gudy.azureus2.plugins.ui.tables.TableCell)
