@@ -47,14 +47,15 @@ import com.aelitis.azureus.ui.common.table.TableRowCore;
 import com.aelitis.azureus.ui.selectedcontent.ISelectedContent;
 import com.aelitis.azureus.ui.selectedcontent.SelectedContentManager;
 import com.aelitis.azureus.ui.swt.columns.utils.ColumnImageClickArea;
+import com.aelitis.azureus.ui.swt.imageloader.ImageLoader;
 import com.aelitis.azureus.ui.swt.utils.ColorCache;
-import com.aelitis.azureus.ui.swt.utils.ImageLoader;
-import com.aelitis.azureus.ui.swt.utils.ImageLoaderFactory;
 import com.aelitis.azureus.ui.swt.views.skin.TorrentListViewsUtils;
 import com.aelitis.azureus.ui.swt.views.skin.VuzeShareUtils;
+import com.aelitis.azureus.util.DLReferals;
 import com.aelitis.azureus.util.DataSourceUtils;
 import com.aelitis.azureus.util.PlayUtils;
 
+import org.gudy.azureus2.plugins.download.Download;
 import org.gudy.azureus2.plugins.ui.Graphic;
 import org.gudy.azureus2.plugins.ui.tables.*;
 
@@ -69,12 +70,14 @@ public class ColumnMediaThumb
 	TableCellDisposeListener, TableCellVisibilityListener,
 	TableCellMouseMoveListener, TableRowMouseListener, TableCellToolTipListener
 {
+	public static final Class DATASOURCE_TYPE = Download.class;
+
 	public static String COLUMN_ID = "MediaThumb";
 
 	public static final boolean ROW_HOVER = System.getProperty("rowhover", "0").equals(
 			"1");
 
-	private static final boolean SET_ALPHA = true;
+	private static final boolean SET_ALPHA = false;
 
 	private static final int WIDTH = 60;
 
@@ -94,6 +97,8 @@ public class ColumnMediaThumb
 
 	private static final int IMG_SIZE_HALF = IMG_SIZE / 2;
 
+	private static final boolean ICONS_BELOW = true;
+
 	private Map mapCellTorrent = new HashMap();
 
 	private final int maxThumbHeight;
@@ -118,11 +123,18 @@ public class ColumnMediaThumb
 	 * 
 	 */
 	public ColumnMediaThumb(String sTableID, int maxThumbHeight) {
-		super(COLUMN_ID, sTableID);
+		this(sTableID, maxThumbHeight, WIDTH);
+	}
+
+	public ColumnMediaThumb(String sTableID) {
+		this(sTableID, 40, WIDTH);
+	}
+
+	public ColumnMediaThumb(String sTableID, int maxThumbHeight, int WIDTH) {
+		super(DATASOURCE_TYPE, COLUMN_ID, ALIGN_CENTER, WIDTH, sTableID);
 		this.maxThumbHeight = maxThumbHeight;
-		initializeAsGraphic(POSITION_LAST, WIDTH);
+		initializeAsGraphic(WIDTH);
 		setWidthLimits(WIDTH, WIDTH);
-		setAlignment(ALIGN_CENTER);
 
 		if (imgPlay == null) {
 			loadImages();
@@ -157,7 +169,7 @@ public class ColumnMediaThumb
 	}
 
 	private void loadImages() {
-		ImageLoader imageLoader = ImageLoaderFactory.getInstance();
+		ImageLoader imageLoader = ImageLoader.getInstance();
 		imgPlay = imageLoader.getImage("image.thumb.play");
 		if (imgPlay != null) {
 			imgPlayBounds = imgPlay.getBounds();
@@ -224,6 +236,7 @@ public class ColumnMediaThumb
 
 		boolean cellHasMouse = (cell instanceof TableCellCore)
 				? ((TableCellCore) cell).isMouseOver() : false;
+				cellHasMouse = true;
 
 		torrent = newTorrent;
 		mapCellTorrent.put(cell, torrent);
@@ -266,7 +279,13 @@ public class ColumnMediaThumb
 			}
 		}
 
+		int cellWidth = cell.getWidth();
+		int cellHeight = cell.getHeight();
+
 		int MAXH = maxThumbHeight < 0 ? cell.getHeight() : maxThumbHeight;
+		if (ICONS_BELOW) {
+			MAXH -= (IMG_SIZE + 5);
+		}
 
 		TableRow row = cell.getTableRow();
 		if (ROW_HOVER) {
@@ -289,29 +308,33 @@ public class ColumnMediaThumb
 		}
 		Image newImg = null;
 		try {
-			int cellWidth = cell.getWidth();
-			int cellHeight = cell.getHeight();
-
 			int w = firstImage.getBounds().width;
 			int h = firstImage.getBounds().height;
 
 			int h2;
 			int w2;
+			int hImg;
+			int wImg;
+			boolean smaller = false;
 
 			if (h > MAXH) {
-				h2 = MAXH;
-				w2 = h2 * w / h;
+				hImg = h2 = MAXH;
+				wImg = w2 = h2 * w / h;
 			} else {
 				h2 = MAXH;
+				hImg = h;
 				w2 = h2 * w / h;
+				wImg = hImg * w / h;
+				dy = (h2 - hImg) / 2;
+				smaller = true;
 			}
 
-			if (cellWidth > w2) {
-				dx = (cellWidth - w2) / 2;
+			if (cellWidth > wImg) {
+				dx = (cellWidth - wImg) / 2;
 			}
 			//dx += 18;
 
-			newImg = new Image(firstImage.getDevice(), cellWidth, h2);
+			newImg = new Image(firstImage.getDevice(), cellWidth, cellHeight);
 
 			GC gc = new GC(newImg);
 			gc.setAdvanced(true);
@@ -321,16 +344,29 @@ public class ColumnMediaThumb
 			} catch (Exception e) {
 				// may not be avail
 			}
-
 			//gc.setBackground(ColorCache.getColor(firstImage.getDevice(), 40, 40, 40));
-			//gc.fillRoundRectangle(0, 0, cellWidth, h2, 7, 7);
+			int[] bg = cell.getBackground();
+			if (bg != null) {
+				gc.setBackground(ColorCache.getColor(firstImage.getDevice(), bg));
+				gc.fillRectangle(newImg.getBounds());
+			}
 
-			gc.setBackground(ColorCache.getColor(firstImage.getDevice(), 30, 30, 30));
-			//gc.fillRectangle(2, 2, (int) (h2 * 1.77f) - 4, h2 - 4);
-			gc.fillRectangle(newImg.getBounds());
+			gc.setBackground(ColorCache.getColor(firstImage.getDevice(),
+	  			"COLOR_LIST_BACKGROUND"));
+			gc.setForeground(ColorCache.getColor(firstImage.getDevice(),
+			"COLOR_WIDGET_LIGHT_SHADOW"));
+			if (smaller) {
+  			gc.fillRoundRectangle(0, 0, cellWidth - 1, h2, 11, 8);
+  			gc.drawRoundRectangle(0, 0, cellWidth - 1, h2, 11, 8);
+  			//gc.drawRectangle(0, 0, cellWidth - 1, h2);
+			}
+			gc.fillRoundRectangle(0, h2 + 2, cellWidth - 1, cellHeight - h2 - 3, 11, 8);
+			//gc.drawRoundRectangle(0, h2 + 2, cellWidth - 1, cellHeight - h2 - 3, 11, 8);
+  			//gc.fillRectangle(2, 2, (int) (h2 * 1.77f) - 4, h2 - 4);
+  			//gc.fillRectangle(newImg.getBounds());
 
 			if (cellHasMouse && SET_ALPHA) {
-				int[] bg = cell.getBackground();
+				bg = cell.getBackground();
 				if (bg != null) {
 					gc.setBackground(ColorCache.getColor(firstImage.getDevice(), bg));
 					gc.fillRectangle(newImg.getBounds());
@@ -342,7 +378,7 @@ public class ColumnMediaThumb
 				}
 			}
 			gc.drawImage(firstImage, 0, 0, w, h, dx + BORDER_SIZE, dy + BORDER_SIZE,
-					w2 - (BORDER_SIZE * 2), h2 - (BORDER_SIZE * 2));
+					wImg - (BORDER_SIZE * 2), hImg - (BORDER_SIZE * 2));
 
 			if (cell instanceof TableCellSWT) {
 				TableCellSWT cellSWT = (TableCellSWT) cell;
@@ -363,42 +399,32 @@ public class ColumnMediaThumb
 			int imgSizeHalf = (int) (IMG_SIZE_HALF * scale);
 			int border = (int) (4 * scale);
 			
-			boolean canShare = dm != null;
-			if (!canShare && (ds instanceof VuzeActivitiesEntry)) {
-				try {
-					ISelectedContent sc = ((VuzeActivitiesEntry)ds).createSelectedContentObject();
-					canShare = sc != null;
-				} catch (Exception e) {
-					canShare = false;
-				}
-			}
+			boolean canShare = VuzeShareUtils.getInstance().canShare(ds);
+			
+			int yPos = ICONS_BELOW ? cellHeight - imgSize - 2 :  cellHeight / 2 - imgSizeHalf;
 
 			if (clickAreaDL != null) {
-				clickAreaDL.setPosition(cellWidth - imgSize - border, cellHeight / 2
-						- imgSizeHalf);
+				clickAreaDL.setPosition(cellWidth - imgSize - border, yPos);
 				clickAreaDL.setVisible(dm == null);
 				if (dm == null) {
 					canShare = false;
 				}
 			}
 			if (clickAreaDetails != null) {
-				clickAreaDetails.setPosition(border, cellHeight / 2 - imgSizeHalf);
+				clickAreaDetails.setPosition(border, yPos);
 				clickAreaDetails.setVisible(getHash(ds, true) != null);
 			}
 			if (clickAreaRun != null) {
-				clickAreaRun.setPosition(border, cellHeight / 2
-						- imgSizeHalf);
+				clickAreaRun.setPosition(border, yPos);
 				clickAreaRun.setVisible(canRun);
 			}
 			if (clickAreaPlay != null) {
-				clickAreaPlay.setPosition(cellWidth / 2 - imgSizeHalf, cellHeight / 2
-						- imgSizeHalf);
+				clickAreaPlay.setPosition(cellWidth / 2 - imgSizeHalf, yPos);
 				clickAreaPlay.setVisible(canPlay);
 			}
 			
 			if (clickAreaShare != null) {
-				clickAreaShare.setPosition(cellWidth - imgSize - border, cellHeight / 2
-						- imgSizeHalf);
+				clickAreaShare.setPosition(cellWidth - imgSize - border, yPos);
 				clickAreaShare.setVisible(canShare);
 			}
 
@@ -420,7 +446,7 @@ public class ColumnMediaThumb
 
 				if (cellHasMouse) {
 					clickArea.setScale(scale);
-					clickArea.drawImage(gc);
+					clickArea.drawImage(cell, gc);
 				}
 			}
 
@@ -500,13 +526,12 @@ public class ColumnMediaThumb
 				if (ds instanceof VuzeActivitiesEntry) {
 					if (((VuzeActivitiesEntry) ds).isDRM()
 							&& ((VuzeActivitiesEntry) ds).getDownloadManger() == null) {
-						String hash = getHash(event.cell.getDataSource(), true);
-						if (hash != null) {
-							TorrentListViewsUtils.viewDetails(hash, "thumb");
+						if (DataSourceUtils.isPlatformContent(ds)) {
+							TorrentListViewsUtils.viewDetailsFromDS(ds, "thumb");
 						}
 						return;
 					}
-					referal = "playdashboardactivity-"
+					referal = DLReferals.DL_REFERAL_PLAYDASHACTIVITY + "-"
 							+ ((VuzeActivitiesEntry) ds).getTypeID();
 				}
 				TorrentListViewsUtils.playOrStreamDataSource(ds, null, referal);
@@ -515,31 +540,31 @@ public class ColumnMediaThumb
 				Object ds = event.cell.getDataSource();
 				if (ds instanceof VuzeActivitiesEntry) {
 					if (((VuzeActivitiesEntry) ds).isDRM()) {
-						String hash = getHash(event.cell.getDataSource(), true);
-						if (hash != null) {
-							TorrentListViewsUtils.viewDetails(hash, "thumb");
+						if (DataSourceUtils.isPlatformContent(ds)) {
+							TorrentListViewsUtils.viewDetailsFromDS(ds, "thumb");
 						}
 						return;
 					}
 
-					referal = "dashboardactivity-"
+					referal = DLReferals.DL_REFERAL_DASHACTIVITY + "-"
 							+ ((VuzeActivitiesEntry) ds).getTypeID();
 				}
 				TorrentListViewsUtils.downloadDataSource(ds, false, referal);
 			} else if (id.equals(BTN_DETAILS)) {
-				String hash = getHash(event.cell.getDataSource(), true);
-				if (hash != null) {
-					TorrentListViewsUtils.viewDetails(hash, "thumb");
+				Object ds = event.cell.getDataSource();
+				if (DataSourceUtils.isPlatformContent(ds)) {
+					TorrentListViewsUtils.viewDetailsFromDS(ds, "thumb");
 				}
 			} else if (id.equals(BTN_RUN)) {
 				// run via play or stream so we get the security warning
 				Object ds = event.cell.getDataSource();
-				TorrentListViewsUtils.playOrStreamDataSource(ds, null, "unknown");
+				TorrentListViewsUtils.playOrStreamDataSource(ds, null,
+						DLReferals.DL_REFERAL_UNKNOWN);
 			} else if (id.equals(BTN_SHARE)) {
 				ISelectedContent[] contents = SelectedContentManager.getCurrentlySelectedContent();
 				if (contents.length > 0) {
 					String referer = event.cell.getTableID() + "-media-thumb-btn";
-					VuzeShareUtils.getInstance().shareTorrent(contents[0], referer);
+					VuzeShareUtils.getInstance().shareContent(contents[0], null, referer);
 				}
 			}
 
