@@ -21,24 +21,26 @@ package com.aelitis.azureus.ui.swt.columns.torrent;
 
 import java.util.Map;
 
+import org.eclipse.swt.SWT;
 import org.eclipse.swt.graphics.Image;
 import org.eclipse.swt.graphics.Rectangle;
 
 import org.gudy.azureus2.core3.download.DownloadManager;
 import org.gudy.azureus2.core3.torrent.TOTorrent;
-import org.gudy.azureus2.core3.torrent.TOTorrentException;
 import org.gudy.azureus2.core3.util.Debug;
-import org.gudy.azureus2.ui.swt.Utils;
 import org.gudy.azureus2.ui.swt.plugins.UISWTGraphic;
 import org.gudy.azureus2.ui.swt.pluginsimpl.UISWTGraphicImpl;
+import org.gudy.azureus2.ui.swt.views.table.TableCellSWT;
 import org.gudy.azureus2.ui.swt.views.table.utils.CoreTableColumn;
 
 import com.aelitis.azureus.core.messenger.PlatformMessage;
 import com.aelitis.azureus.core.messenger.PlatformMessengerListener;
 import com.aelitis.azureus.core.messenger.config.PlatformRatingMessenger;
 import com.aelitis.azureus.core.torrent.PlatformTorrentUtils;
-import com.aelitis.azureus.ui.swt.utils.ImageLoaderFactory;
+import com.aelitis.azureus.ui.swt.imageloader.ImageLoader;
 
+import org.gudy.azureus2.plugins.download.Download;
+import org.gudy.azureus2.plugins.ui.Graphic;
 import org.gudy.azureus2.plugins.ui.tables.*;
 
 /**
@@ -52,69 +54,80 @@ public class ColumnRateUpDown
 	implements TableCellAddedListener, TableCellRefreshListener,
 	TableCellMouseListener
 {
-	public static String COLUMN_ID = "RateIt";
+	public static final Class DATASOURCE_TYPE = Download.class;
 
-	private static UISWTGraphicImpl graphicRateMe;
+	public static final String COLUMN_ID = "RateIt";
 
-	private static UISWTGraphicImpl graphicUp;
+	private static final String ID_ICON_RATE = "icon.rate.library";
+	
+	private static final String ID_ICON_RATE_UP = "icon.rate.library.up";
+	
+	private static final String ID_ICON_RATE_DOWN = "icon.rate.library.down";
+	
+	private UISWTGraphicImpl graphicRate;
+	
+	private UISWTGraphicImpl graphicRateDown;
+	
+	private UISWTGraphicImpl graphicRateUp;
+	
+	private UISWTGraphicImpl graphicsWait[];
+	
+	private Rectangle boundsRate;
 
-	private static UISWTGraphicImpl graphicDown;
-
-	private static UISWTGraphicImpl graphicWait;
-
-	private static UISWTGraphicImpl graphicRateMeButton;
-
-	private static UISWTGraphicImpl graphicRateMeButtonEnabled;
-
-	private static UISWTGraphicImpl graphicRateMeButtonDisabled;
-
-	private static Rectangle boundsRateMe;
-
-	private static int width;
+	private static int width = 50;
 
 	private boolean useButton = false;
 
-	private boolean mouseIn = false;
-
 	private boolean disabled = false;
-
-	static {
-		Image img = ImageLoaderFactory.getInstance().getImage("icon.rateme");
-		graphicRateMe = new UISWTGraphicImpl(img);
-		boundsRateMe = img.getBounds();
-		width = boundsRateMe.width;
-
-		img = ImageLoaderFactory.getInstance().getImage("icon.rateme-button");
-		graphicRateMeButtonEnabled = new UISWTGraphicImpl(img);
-		graphicRateMeButton = graphicRateMeButtonEnabled;
-		width = Math.max(width, img.getBounds().width);
-
-		img = ImageLoaderFactory.getInstance().getImage(
-				"icon.rateme-button-disabled");
-		graphicRateMeButtonDisabled = new UISWTGraphicImpl(img);
-		width = Math.max(width, img.getBounds().width);
-
-		img = ImageLoaderFactory.getInstance().getImage("icon.rate.up");
-		graphicUp = new UISWTGraphicImpl(img);
-		width = Math.max(width, img.getBounds().width);
-
-		img = ImageLoaderFactory.getInstance().getImage("icon.rate.down");
-		graphicDown = new UISWTGraphicImpl(img);
-		width = Math.max(width, img.getBounds().width);
-
-		img = ImageLoaderFactory.getInstance().getImage("icon.rate.wait");
-		graphicWait = new UISWTGraphicImpl(img);
-		width = Math.max(width, img.getBounds().width);
-	}
+	
 
 	/**
 	 * 
 	 */
 	public ColumnRateUpDown(String sTableID) {
-		super(COLUMN_ID, sTableID);
-		initializeAsGraphic(POSITION_LAST, width);
-		setAlignment(ALIGN_CENTER);
+		super(DATASOURCE_TYPE, COLUMN_ID, ALIGN_CENTER, width, sTableID);
+		initializeAsGraphic(width);
 		setWidthLimits(width, width);
+
+		Image img;
+		
+		img = ImageLoader.getInstance().getImage(ID_ICON_RATE);
+		graphicRate = new UISWTGraphicImpl(img);
+		
+		img = ImageLoader.getInstance().getImage(ID_ICON_RATE_DOWN);
+		graphicRateDown = new UISWTGraphicImpl(img);
+		
+		img = ImageLoader.getInstance().getImage(ID_ICON_RATE_UP);
+		graphicRateUp = new UISWTGraphicImpl(img);
+		
+		boundsRate = img.getBounds();
+		
+		Image[] imgs = ImageLoader.getInstance().getImages("image.sidebar.vitality.dots");
+		graphicsWait = new UISWTGraphicImpl[imgs.length];
+		for(int i = 0 ; i < imgs.length  ;i++) {
+			graphicsWait[i] =  new UISWTGraphicImpl(imgs[i]);
+		}
+	}
+	
+	public void remove() {
+		super.remove();
+
+		ImageLoader imageLoader = ImageLoader.getInstance();
+		if (graphicRate != null) {
+			imageLoader.releaseImage(ID_ICON_RATE);
+		}
+		if (graphicRateDown != null) {
+			imageLoader.releaseImage(ID_ICON_RATE_DOWN); 
+		}
+		if (graphicRateUp != null) {
+			imageLoader.releaseImage(ID_ICON_RATE_UP); 
+		}
+	}
+
+	public void fillTableColumnInfo(TableColumnInfo info) {
+		info.addCategories(new String[] {
+			CAT_CONTENT,
+		});
 	}
 
 	public void cellAdded(TableCell cell) {
@@ -123,6 +136,7 @@ public class ColumnRateUpDown
 	}
 
 	public void refresh(TableCell cell) {
+		
 		Object ds = cell.getDataSource();
 		TOTorrent torrent = null;
 		if (ds instanceof TOTorrent) {
@@ -145,29 +159,35 @@ public class ColumnRateUpDown
 		int rating = PlatformTorrentUtils.getUserRating(torrent);
 
 		if (!cell.setSortValue(rating) && cell.isValid()) {
-			return;
+			if(rating != -2) {
+				return;
+			}
 		}
+		
+		
+		
 		if (!cell.isShown()) {
 			return;
 		}
-
+		
 		UISWTGraphic graphic;
 		switch (rating) {
 			case -2: // waiting
-				graphic = graphicWait;
+				int i = TableCellRefresher.getRefreshIndex(1, graphicsWait.length);
+				graphic = graphicsWait[i];
+				TableCellRefresher.addCell(this,cell);
 				break;
 
 			case -1: // unrated
-				graphic = (useButton && !mouseIn) || disabled ? graphicRateMeButton
-						: graphicRateMe;
+				graphic = graphicRate;
 				break;
 
 			case 0:
-				graphic = graphicDown;
+				graphic = graphicRateDown;
 				break;
 
 			case 1:
-				graphic = graphicUp;
+				graphic = graphicRateUp;
 				break;
 
 			default:
@@ -175,14 +195,20 @@ public class ColumnRateUpDown
 		}
 
 		cell.setGraphic(graphic);
+		
+		if ( !disabled && graphic != null && cell instanceof TableCellSWT ){
+			
+			((TableCellSWT)cell).setCursorID(SWT.CURSOR_HAND);
+		}
 	}
 
-	boolean bMouseDowned = false;
+	TableRow previousSelection = null;
 
 	public void cellMouseTrigger(final TableCellMouseEvent event) {
 		if (disabled) {
 			return;
 		}
+		
 		Object ds = event.cell.getDataSource();
 		TOTorrent torrent0 = null;
 		if (ds instanceof TOTorrent) {
@@ -199,91 +225,69 @@ public class ColumnRateUpDown
 			return;
 		}
 
-		if (useButton) {
-			if (event.eventType == TableCellMouseEvent.EVENT_MOUSEENTER) {
-				mouseIn = true;
-				refresh(event.cell);
-			} else if (event.eventType == TableCellMouseEvent.EVENT_MOUSEEXIT) {
-				mouseIn = false;
-				refresh(event.cell);
-			}
-		}
-
 		final TOTorrent torrent = torrent0;
 
-		// middle button == refresh rate from platform
-		if (event.eventType == TableCellMouseEvent.EVENT_MOUSEUP
-				&& event.button == 2) {
-
-			try {
-				final String fHash = torrent.getHashWrapper().toBase32String();
-				PlatformRatingMessenger.getUserRating(new String[] {
-					PlatformRatingMessenger.RATE_TYPE_CONTENT
-				}, new String[] {
-					fHash
-				}, 5000);
-			} catch (TOTorrentException e) {
-				Debug.out(e);
-			}
-			Utils.beep();
-		}
-
 		// only first button
 		if (event.button != 1) {
 			return;
 		}
 
-		// no rating if row isn't selected yet
-		TableRow row = event.cell.getTableRow();
-		if (row != null && !row.isSelected()) {
-			return;
-		}
 
 		if (!PlatformTorrentUtils.isContent(torrent, true)) {
 			return;
 		}
 
-		if (event.eventType == TableCellMouseEvent.EVENT_MOUSEDOWN) {
-			bMouseDowned = true;
-			return;
-		}
 
-		if (event.eventType == TableCellMouseEvent.EVENT_MOUSEUP && bMouseDowned) {
-			Comparable sortValue = event.cell.getSortValue();
-			if (sortValue == null || sortValue.equals(new Long(-1))) {
-				// not set
-				int cellWidth = event.cell.getWidth();
-				int cellHeight = event.cell.getHeight();
-				int x = event.x - ((cellWidth - boundsRateMe.width) / 2);
-				int y = event.y - ((cellHeight - boundsRateMe.height) / 2);
+		if (event.eventType == TableCellMouseEvent.EVENT_MOUSEUP ) {
 
-				if (x >= 0 && y >= 0 && x < boundsRateMe.width
-						&& y < boundsRateMe.height) {
+			//By default, let's cancel the setting
+			boolean cancel = true;
+
+			// Are we in the graphics area? (and not canceling)
+			int cellWidth = event.cell.getWidth();
+			int cellHeight = event.cell.getHeight();
+			int x = event.x - ((cellWidth - boundsRate.width) / 2);
+			int y = event.y - ((cellHeight - boundsRate.height) / 2);
+
+			Graphic currentGraphic = event.cell.getGraphic();
+			
+			if (x >= 0 && y >= 0 && x < boundsRate.width
+					&& y < boundsRate.height &&  (graphicRate.equals(currentGraphic) || graphicRateUp.equals(currentGraphic) || graphicRateDown.equals(currentGraphic)) ) {
+				//The event is within the graphic, are we on a non-transparent pixel ?
+				int alpha = graphicRate.getImage().getImageData().getAlpha(x,y);
+				if(alpha > 0) {
 					try {
-						final int value = (x < (boundsRateMe.height - y + 1)) ? 1 : 0;
-						refresh(event.cell);
-						PlatformRatingMessenger.setUserRating(torrent, value, true, 0,
-								new PlatformMessengerListener() {
-									public void replyReceived(PlatformMessage message,
-											String replyType, Map reply) {
-										refresh(event.cell);
-									}
-
-									public void messageSent(PlatformMessage message) {
-									}
-								});
+						cancel = false;
+						final int value = (x < (boundsRate.width / 2)) ? 0 : 1;
+						int previousValue = PlatformTorrentUtils.getUserRating(torrent);
+						//Changing the value
+						if(value != previousValue) {
+							
+							PlatformRatingMessenger.setUserRating(torrent, value, true, 0,
+									new PlatformMessengerListener() {
+										public void replyReceived(PlatformMessage message,
+												String replyType, Map reply) {
+											refresh(event.cell);
+										}
+		
+										public void messageSent(PlatformMessage message) {
+										}
+									});
+							refresh(event.cell);
+						}
 					} catch (Exception e) {
 						Debug.out(e);
 					}
 				}
-			} else {
+			}
+			
+			 if(cancel) {
 				// remove setting
 				try {
 					final int oldValue = PlatformTorrentUtils.getUserRating(torrent);
-					if (oldValue == -2) {
+					if (oldValue == -2 || oldValue == -1) {
 						return;
 					}
-					refresh(event.cell);
 					PlatformRatingMessenger.setUserRating(torrent, -1, true, 0,
 							new PlatformMessengerListener() {
 								public void replyReceived(PlatformMessage message,
@@ -294,12 +298,12 @@ public class ColumnRateUpDown
 								public void messageSent(PlatformMessage message) {
 								}
 							});
+					refresh(event.cell);
 				} catch (Exception e) {
 					Debug.out(e);
 				}
 			}
 		}
-		bMouseDowned = false;
 	}
 
 	public boolean useButton() {
@@ -312,8 +316,5 @@ public class ColumnRateUpDown
 
 	public void setDisabled(boolean disabled) {
 		this.disabled = disabled;
-		graphicRateMeButton = disabled ? graphicRateMeButtonDisabled
-				: graphicRateMeButtonEnabled;
-		this.invalidateCells();
 	}
 }
