@@ -28,9 +28,7 @@ import org.eclipse.swt.program.Program;
 
 import org.gudy.azureus2.core3.download.DownloadManager;
 import org.gudy.azureus2.core3.internat.MessageText;
-import org.gudy.azureus2.core3.torrent.TOTorrentException;
 import org.gudy.azureus2.core3.util.Debug;
-import org.gudy.azureus2.core3.util.DisplayFormatters;
 import org.gudy.azureus2.ui.swt.Utils;
 import org.gudy.azureus2.ui.swt.debug.ObfusticateCellText;
 import org.gudy.azureus2.ui.swt.plugins.UISWTGraphic;
@@ -40,7 +38,6 @@ import org.gudy.azureus2.ui.swt.shells.GCStringPrinter.URLInfo;
 import org.gudy.azureus2.ui.swt.views.table.TableCellSWT;
 import org.gudy.azureus2.ui.swt.views.table.utils.CoreTableColumn;
 
-import com.aelitis.azureus.core.messenger.config.PlatformConfigMessenger;
 import com.aelitis.azureus.core.torrent.PlatformTorrentUtils;
 import com.aelitis.azureus.ui.common.table.TableRowCore;
 import com.aelitis.azureus.ui.skin.SkinConstants;
@@ -49,9 +46,10 @@ import com.aelitis.azureus.ui.swt.UIFunctionsSWT;
 import com.aelitis.azureus.ui.swt.skin.SWTSkinFactory;
 import com.aelitis.azureus.ui.swt.skin.SWTSkinProperties;
 import com.aelitis.azureus.ui.swt.utils.ColorCache;
-import com.aelitis.azureus.util.Constants;
 import com.aelitis.azureus.util.DataSourceUtils;
+import com.aelitis.azureus.util.UrlFilter;
 
+import org.gudy.azureus2.plugins.download.Download;
 import org.gudy.azureus2.plugins.ui.Graphic;
 import org.gudy.azureus2.plugins.ui.tables.*;
 
@@ -65,8 +63,14 @@ public class ColumnTitle
 	implements TableCellRefreshListener, ObfusticateCellText,
 	TableCellMouseMoveListener, TableCellDisposeListener, TableCellVisibilityListener
 {
+	public static final Class DATASOURCE_TYPE = Download.class;
+
 	public static String COLUMN_ID = "name";
 
+	public void fillTableColumnInfo(TableColumnInfo info) {
+		info.addCategories(new String[] { CAT_CONTENT, CAT_ESSENTIAL });
+	}
+
 	public static boolean SHOW_EXT_INFO = false;
 	
 	public static boolean LINKIFY = false;
@@ -79,7 +83,7 @@ public class ColumnTitle
 
 	/** Default Constructor */
 	public ColumnTitle(String sTableID) {
-		super(COLUMN_ID, POSITION_LAST, 250, sTableID);
+		super(COLUMN_ID, POSITION_INVISIBLE, 250, sTableID);
 		setMinWidth(70);
 		setObfustication(true);
 		if (LINKIFY) {
@@ -145,14 +149,6 @@ public class ColumnTitle
 			}
 		}
 
-		if (ColumnProgressETA.TRY_NAME_COLUMN_EXPANDER) {
-			if (dm.getAssumedComplete()) {
-				long size = dm.getSize() - dm.getStats().getRemaining();
-				name += "\nCompleted. "
-						+ DisplayFormatters.formatByteCountToKiBEtc(size);
-			}
-		}
-
 		Graphic graphic = cell.getBackgroundGraphic();
 		if (!(graphic instanceof UISWTGraphic)) {
 			cell.setText(name);
@@ -166,10 +162,11 @@ public class ColumnTitle
 		String sText = name;
 		if (DataSourceUtils.isPlatformContent(dm)) {
 			try {
-				sText = "<A HREF=\"" + Constants.URL_PREFIX + Constants.URL_DETAILS
-						+ dm.getTorrent().getHashWrapper().toBase32String() + ".html?"
-						+ Constants.URL_SUFFIX + "\">" + name + "</A>";
-			} catch (TOTorrentException e) {
+				sText = "<A HREF=\""
+						+ DataSourceUtils.getContentNetwork(dm).getContentDetailsService(
+								dm.getTorrent().getHashWrapper().toBase32String(), null)
+						+ "\">" + name + "</A>";
+			} catch (Exception e) {
 				Debug.out(e);
 			}
 		}
@@ -250,13 +247,13 @@ public class ColumnTitle
 				if (hitUrl != null) {
 					if (event.eventType == TableCellMouseEvent.EVENT_MOUSEUP
 							&& event.button == 1) {
-						if (PlatformConfigMessenger.isURLBlocked(hitUrl.url)) {
+						if (UrlFilter.getInstance().urlIsBlocked(hitUrl.url)) {
 							Utils.launch(hitUrl.url);
 						} else {
 							UIFunctionsSWT uif = UIFunctionsManagerSWT.getUIFunctionsSWT();
 							if (uif != null) {
-								uif.viewURL(hitUrl.url, SkinConstants.VIEWID_BROWSER_BROWSE, 0,
-										0, false, false);
+								uif.viewURL(hitUrl.url, SkinConstants.VIEWID_BROWSER_BROWSE,
+										"column.title");
 								return;
 							}
 						}
