@@ -22,19 +22,23 @@ package com.aelitis.azureus.ui.swt.shells;
 
 import org.eclipse.swt.SWT;
 import org.eclipse.swt.browser.*;
+import org.eclipse.swt.events.TraverseEvent;
+import org.eclipse.swt.events.TraverseListener;
 import org.eclipse.swt.graphics.Rectangle;
 import org.eclipse.swt.layout.FillLayout;
-import org.eclipse.swt.widgets.Display;
-import org.eclipse.swt.widgets.Shell;
+import org.eclipse.swt.widgets.*;
 
+import org.gudy.azureus2.core3.util.*;
 import org.gudy.azureus2.ui.swt.Utils;
 import org.gudy.azureus2.ui.swt.components.shell.ShellFactory;
+import org.gudy.azureus2.ui.swt.shells.MessageBoxShell;
 
 import com.aelitis.azureus.core.messenger.ClientMessageContext;
 import com.aelitis.azureus.ui.swt.browser.BrowserContext;
 import com.aelitis.azureus.ui.swt.browser.listener.ConfigListener;
 import com.aelitis.azureus.ui.swt.browser.listener.DisplayListener;
 import com.aelitis.azureus.ui.swt.browser.listener.TorrentListener;
+import com.aelitis.azureus.ui.swt.browser.listener.VuzeListener;
 
 /**
  * @author TuxPaper
@@ -46,6 +50,10 @@ public class BrowserWindow
 
 	private Shell shell;
 
+	private ClientMessageContext context;
+
+	private Browser browser;
+	
 	public BrowserWindow(Shell parent, String url, double wPct, double hPct,
 			boolean allowResize, boolean isModal) {
 		if (parent == null) {
@@ -82,8 +90,34 @@ public class BrowserWindow
 		shell.setLayout(new FillLayout());
 
 		Utils.setShellIcon(shell);
+		
+		shell.setText(url);
+		
+		shell.addTraverseListener(new TraverseListener() {
+			public void keyTraversed(TraverseEvent e) {
+    		if (e.detail == SWT.TRAVERSE_ESCAPE) {
+    			shell.dispose();
+    			e.doit = false;
+    		}
+			}
+		});
+		
+		final Listener escListener = new Listener() {
+			public void handleEvent(Event event) {
+				if (event.keyCode == 27) {
+					shell.dispose();
+				}
+			}
+		};
+		shell.getDisplay().addFilter(SWT.KeyDown, escListener);
+		shell.addListener(SWT.Dispose, new Listener() {
+			public void handleEvent(Event event) {
+				event.display.removeFilter(SWT.KeyDown, escListener);
+			}
+		});
 
-		Browser browser = null;
+
+		browser = null;
 		
 		try {
 			browser = new Browser(shell, Utils.getInitialBrowserStyle(SWT.NONE));
@@ -97,9 +131,10 @@ public class BrowserWindow
 			return;
 		}
 
-		final ClientMessageContext context = new BrowserContext("browser-window"
+		context = new BrowserContext("browser-window"
 				+ Math.random(), browser, null, true);
 		context.addMessageListener(new TorrentListener());
+		context.addMessageListener(new VuzeListener());
 		context.addMessageListener(new DisplayListener(browser));
 		context.addMessageListener(new ConfigListener(browser));
 
@@ -114,6 +149,7 @@ public class BrowserWindow
 
 		browser.addCloseWindowListener(new CloseWindowListener() {
 			public void close(WindowEvent event) {
+				context.debug("window.close called");
 				shell.dispose();
 			}
 		});
@@ -125,9 +161,42 @@ public class BrowserWindow
 			}
 
 		});
+		
+		browser.addStatusTextListener(new StatusTextListener() {
+			public void changed(StatusTextEvent event) {
+				if(MessageBoxShell.STATUS_TEXT_CLOSE.equals(event.text)) {
+					//For some reason disposing the shell / browser in the same Thread makes
+					//ieframe.dll crash on windows.
+					Utils.execSWTThreadLater(0, new Runnable() {
+						public void run() {
+							if(!browser.isDisposed() && ! shell.isDisposed()) {
+								shell.close();
+							}
+						}
+					});
+				}
+			}
+			
+		});
+		
+		SimpleTimer.addEvent("showWin", SystemTime.getOffsetTime(3000), new TimerEventPerformer() {
+		
+			public void perform(TimerEvent event) {
+				Utils.execSWTThread(new AERunnable() {
+				
+					public void runSupport() {
+						if (shell != null && !shell.isDisposed()) {
+							shell.open();
+						}
+					}
+				});
+			}
+		});
 
 		if (w > 0 && h > 0) {
-			shell.setSize(w, h);
+			Rectangle computeTrim = shell.computeTrim(0, 0, w, h);
+			shell.setSize(computeTrim.width, computeTrim.height);
+			//shell.setSize(w, h);
 		}
 
 		Utils.centerWindowRelativeTo(shell, parent);
@@ -144,6 +213,12 @@ public class BrowserWindow
 		}
 	}
 
+	public ClientMessageContext
+	getContext()
+	{
+		return( context );
+	}
+	
 	public static void main(String[] args) {
 		Display display = new Display();
 		Shell shell = new Shell(display, SWT.DIALOG_TRIM);
