@@ -23,31 +23,59 @@ import java.util.HashMap;
 import java.util.Iterator;
 import java.util.Map;
 
-import org.eclipse.swt.widgets.Menu;
-import org.eclipse.swt.widgets.Shell;
+import org.eclipse.swt.SWT;
+import org.eclipse.swt.events.PaintEvent;
+import org.eclipse.swt.events.PaintListener;
+import org.eclipse.swt.graphics.Point;
+import org.eclipse.swt.widgets.*;
+
 import org.gudy.azureus2.core3.download.DownloadManager;
+import org.gudy.azureus2.core3.internat.MessageText;
 import org.gudy.azureus2.core3.logging.LogEvent;
 import org.gudy.azureus2.core3.logging.LogIDs;
 import org.gudy.azureus2.core3.logging.Logger;
+import org.gudy.azureus2.core3.torrent.TOTorrent;
+import org.gudy.azureus2.core3.torrent.TOTorrentException;
 import org.gudy.azureus2.core3.util.AEMonitor;
 import org.gudy.azureus2.core3.util.AERunnable;
+import org.gudy.azureus2.core3.util.Debug;
 import org.gudy.azureus2.plugins.PluginView;
+import org.gudy.azureus2.plugins.ui.sidebar.SideBarEntry;
 import org.gudy.azureus2.ui.swt.Utils;
 import org.gudy.azureus2.ui.swt.mainwindow.*;
 import org.gudy.azureus2.ui.swt.mainwindow.MainWindow;
 import org.gudy.azureus2.ui.swt.minibar.AllTransfersBar;
 import org.gudy.azureus2.ui.swt.plugins.*;
 import org.gudy.azureus2.ui.swt.pluginsimpl.UISWTInstanceImpl;
+import org.gudy.azureus2.ui.swt.pluginsimpl.UISWTViewImpl;
+import org.gudy.azureus2.ui.swt.shells.CoreWaiterSWT;
 import org.gudy.azureus2.ui.swt.shells.MessageBoxShell;
-import org.gudy.azureus2.ui.swt.views.AbstractIView;
-import org.gudy.azureus2.ui.swt.views.IView;
+import org.gudy.azureus2.ui.swt.update.FullUpdateWindow;
+import org.gudy.azureus2.ui.swt.views.*;
+import org.gudy.azureus2.ui.swt.views.stats.StatsView;
+import org.gudy.azureus2.ui.swt.views.table.TableViewSWT;
 
+import com.aelitis.azureus.core.AzureusCore;
 import com.aelitis.azureus.core.AzureusCoreFactory;
+import com.aelitis.azureus.core.AzureusCoreRunningListener;
+import com.aelitis.azureus.core.cnetwork.ContentNetwork;
+import com.aelitis.azureus.ui.InitializerListener;
 import com.aelitis.azureus.ui.UIFunctionsUserPrompter;
 import com.aelitis.azureus.ui.UIStatusTextClickListener;
+import com.aelitis.azureus.ui.common.updater.UIUpdater;
+import com.aelitis.azureus.ui.selectedcontent.SelectedContentManager;
+import com.aelitis.azureus.ui.swt.Initializer;
 import com.aelitis.azureus.ui.swt.UIFunctionsSWT;
 import com.aelitis.azureus.ui.swt.shells.BrowserWindow;
-import com.aelitis.azureus.ui.swt.skin.SWTSkin;
+import com.aelitis.azureus.ui.swt.skin.*;
+import com.aelitis.azureus.ui.swt.skin.SWTSkinButtonUtility.ButtonListenerAdapter;
+import com.aelitis.azureus.ui.swt.uiupdater.UIUpdaterSWT;
+import com.aelitis.azureus.ui.swt.utils.ColorCache;
+import com.aelitis.azureus.ui.swt.views.skin.*;
+import com.aelitis.azureus.ui.swt.views.skin.SkinnedDialog.SkinnedDialogClosedListener;
+import com.aelitis.azureus.ui.swt.views.skin.sidebar.SideBar;
+import com.aelitis.azureus.util.ContentNetworkUtils;
+import com.aelitis.azureus.util.UrlFilter;
 
 /**
  * @author TuxPaper
@@ -186,11 +214,25 @@ public class UIFunctionsImpl
 	public void closePluginView(IView view) {
 		try {
 			UIFunctionsSWT uiFunctions = mainWindow.getOldUIFunctions(false);
-			if (uiFunctions == null) {
-				return;
+			if (uiFunctions != null) {
+				uiFunctions.closePluginView(view);
 			}
 
-			uiFunctions.closePluginView(view);
+			SkinView sideBarView = SkinViewManager.getByClass(SideBar.class);
+			if (sideBarView instanceof SideBar) {
+				SideBar sideBar = (SideBar) sideBarView;
+				String id;
+				if (view instanceof UISWTViewImpl) {
+					id = ((UISWTViewImpl)view).getViewID();
+				} else {
+  				id = view.getClass().getName();
+  				int i = id.lastIndexOf('.');
+  				if (i > 0) {
+  					id = id.substring(i + 1);
+  				}
+				}
+				sideBar.closeEntry(id);
+			}
 
 		} catch (Exception e) {
 			Logger.log(new LogEvent(LOGID, "closePluginView", e));
@@ -202,12 +244,16 @@ public class UIFunctionsImpl
 	public void closePluginViews(String sViewID) {
 		try {
 			UIFunctionsSWT uiFunctions = mainWindow.getOldUIFunctions(false);
-			if (uiFunctions == null) {
-				return;
+			if (uiFunctions != null) {
+				uiFunctions.closePluginViews(sViewID);
 			}
 
-			uiFunctions.closePluginViews(sViewID);
-
+			SkinView sideBarView = SkinViewManager.getByClass(SideBar.class);
+			if (sideBarView instanceof SideBar) {
+				SideBar sideBar = (SideBar) sideBarView;
+				sideBar.closeEntry(sViewID);
+			}
+			
 		} catch (Exception e) {
 			Logger.log(new LogEvent(LOGID, "closePluginViews", e));
 		}
@@ -265,34 +311,18 @@ public class UIFunctionsImpl
 		return null;
 	}
 
-	// @see com.aelitis.azureus.ui.UIFunctions#openManagerView(org.gudy.azureus2.core3.download.DownloadManager)
-	public void openManagerView(DownloadManager dm) {
-		try {
-			UIFunctionsSWT uiFunctions = mainWindow.getOldUIFunctions(true);
-			if (uiFunctions == null) {
-				return;
-			}
-
-			mainWindow.switchToAdvancedTab();
-			uiFunctions.openManagerView(dm);
-
-		} catch (Exception e) {
-			Logger.log(new LogEvent(LOGID, "openManagerView", e));
-		}
-
-	}
-
 	// @see com.aelitis.azureus.ui.swt.UIFunctionsSWT#openPluginView(org.gudy.azureus2.ui.swt.views.AbstractIView, java.lang.String)
 	public void openPluginView(AbstractIView view, String name) {
 		try {
-			UIFunctionsSWT uiFunctions = mainWindow.getOldUIFunctions(true);
-			if (uiFunctions == null) {
-				return;
+			SkinView sideBarView = SkinViewManager.getByClass(SideBar.class);
+			if (sideBarView instanceof SideBar) {
+				SideBar sideBar = (SideBar) sideBarView;
+
+				if (sideBar.createTreeItemFromIView(null, view, name, null, true, true,
+						true) != null) {
+					return;
+				}
 			}
-
-			mainWindow.switchToAdvancedTab();
-			uiFunctions.openPluginView(view, name);
-
 		} catch (Exception e) {
 			Logger.log(new LogEvent(LOGID, "openPluginView", e));
 		}
@@ -301,33 +331,31 @@ public class UIFunctionsImpl
 
 	// @see com.aelitis.azureus.ui.swt.UIFunctionsSWT#openPluginView(org.gudy.azureus2.plugins.PluginView)
 	public void openPluginView(PluginView view) {
-		try {
-			UIFunctionsSWT uiFunctions = mainWindow.getOldUIFunctions(true);
-			if (uiFunctions == null) {
-				return;
-			}
-
-			mainWindow.switchToAdvancedTab();
-			uiFunctions.openPluginView(view);
-
-		} catch (Exception e) {
-			Logger.log(new LogEvent(LOGID, "openPluginView", e));
-		}
-
+		openPluginView(view, view.getPluginViewName());
 	}
 
 	// @see com.aelitis.azureus.ui.swt.UIFunctionsSWT#openPluginView(java.lang.String, java.lang.String, org.gudy.azureus2.ui.swt.plugins.UISWTViewEventListener, java.lang.Object, boolean)
 	public void openPluginView(String sParentID, String sViewID,
 			UISWTViewEventListener l, Object dataSource, boolean bSetFocus) {
 		try {
-			UIFunctionsSWT uiFunctions = mainWindow.getOldUIFunctions(true);
-			if (uiFunctions == null) {
-				return;
+			SideBar sideBar = (SideBar) SkinViewManager.getByClass(SideBar.class);
+
+			if (sideBar != null) {
+				
+				String sidebarParentID = null;
+				
+				if (UISWTInstance.VIEW_MYTORRENTS.equals(sParentID)) {
+					sidebarParentID = SideBar.SIDEBAR_SECTION_LIBRARY;
+				} else if (!UISWTInstance.VIEW_MAIN.equals(sParentID)) {
+					System.err.println("Can't find parent " + sParentID + " for " + sViewID);
+				}
+				
+				sideBar.createTreeItemFromEventListener(sidebarParentID, null, l, sViewID,
+						true, dataSource);
+				if (bSetFocus) {
+					sideBar.showEntryByID(sViewID);
+				}
 			}
-
-			mainWindow.switchToAdvancedTab();
-			uiFunctions.openPluginView(sParentID, sViewID, l, dataSource, bSetFocus);
-
 		} catch (Exception e) {
 			Logger.log(new LogEvent(LOGID, "openPluginView", e));
 		}
@@ -336,30 +364,21 @@ public class UIFunctionsImpl
 
 	// @see com.aelitis.azureus.ui.swt.UIFunctionsSWT#openPluginView(org.gudy.azureus2.ui.swt.plugins.UISWTPluginView)
 	public void openPluginView(UISWTPluginView view) {
-		try {
-			UIFunctionsSWT uiFunctions = mainWindow.getOldUIFunctions(true);
-			if (uiFunctions == null) {
-				return;
-			}
-
-			mainWindow.switchToAdvancedTab();
-			uiFunctions.openPluginView(view);
-
-		} catch (Exception e) {
-			Logger.log(new LogEvent(LOGID, "openPluginView", e));
-		}
-
+		openPluginView(view, view.getPluginViewName());
 	}
 
 	// @see com.aelitis.azureus.ui.UIFunctions#refreshIconBar()
 	public void refreshIconBar() {
 		try {
-			UIFunctionsSWT uiFunctions = mainWindow.getOldUIFunctions(true);
-			if (uiFunctions == null) {
-				return;
+			UIFunctionsSWT uiFunctions = mainWindow.getOldUIFunctions(false);
+			if (uiFunctions != null) {
+				uiFunctions.refreshIconBar();
 			}
 
-			uiFunctions.refreshIconBar();
+			ToolBarView tb = (ToolBarView) SkinViewManager.getByClass(ToolBarView.class);
+			if (tb != null) {
+				tb.refreshCoreToolBarItems();
+			}
 
 		} catch (Exception e) {
 			Logger.log(new LogEvent(LOGID, "refreshIconBar", e));
@@ -370,7 +389,9 @@ public class UIFunctionsImpl
 	// @see com.aelitis.azureus.ui.UIFunctions#refreshLanguage()
 	public void refreshLanguage() {
 		try {
-			UIFunctionsSWT uiFunctions = mainWindow.getOldUIFunctions(true);
+			mainWindow.setSelectedLanguageItem();
+			
+			UIFunctionsSWT uiFunctions = mainWindow.getOldUIFunctions(false);
 			if (uiFunctions == null) {
 				return;
 			}
@@ -386,7 +407,7 @@ public class UIFunctionsImpl
 	// @see com.aelitis.azureus.ui.UIFunctions#removeManagerView(org.gudy.azureus2.core3.download.DownloadManager)
 	public void removeManagerView(DownloadManager dm) {
 		try {
-			UIFunctionsSWT uiFunctions = mainWindow.getOldUIFunctions(true);
+			UIFunctionsSWT uiFunctions = mainWindow.getOldUIFunctions(false);
 			if (uiFunctions == null) {
 				return;
 			}
@@ -402,7 +423,7 @@ public class UIFunctionsImpl
 	// @see com.aelitis.azureus.ui.swt.UIFunctionsSWT#removePluginView(java.lang.String)
 	public void removePluginView(String viewID) {
 		try {
-			UIFunctionsSWT uiFunctions = mainWindow.getOldUIFunctions(true);
+			UIFunctionsSWT uiFunctions = mainWindow.getOldUIFunctions(false);
 			if (uiFunctions == null) {
 				pluginViews_mon.enter();
 				try {
@@ -410,6 +431,7 @@ public class UIFunctionsImpl
 				} finally {
 					pluginViews_mon.exit();
 				}
+				PluginsMenuHelper.getInstance().removePluginViews(viewID);
 				return;
 			}
 
@@ -424,7 +446,7 @@ public class UIFunctionsImpl
 	// @see com.aelitis.azureus.ui.swt.UIFunctionsSWT#removePluginView(org.gudy.azureus2.ui.swt.plugins.UISWTPluginView)
 	public void removePluginView(UISWTPluginView view) {
 		try {
-			UIFunctionsSWT uiFunctions = mainWindow.getOldUIFunctions(true);
+			UIFunctionsSWT uiFunctions = mainWindow.getOldUIFunctions(false);
 			if (uiFunctions == null) {
 				pluginViews_mon.enter();
 				try {
@@ -432,6 +454,7 @@ public class UIFunctionsImpl
 				} finally {
 					pluginViews_mon.exit();
 				}
+				PluginsMenuHelper.getInstance().removePluginView(view, view.getPluginViewName());
 				return;
 			}
 
@@ -443,23 +466,6 @@ public class UIFunctionsImpl
 
 	}
 
-	// @see com.aelitis.azureus.ui.UIFunctions#requestShutdown()
-	public boolean requestShutdown() {
-		try {
-			UIFunctionsSWT uiFunctions = mainWindow.getOldUIFunctions(true);
-			if (uiFunctions == null) {
-				return true;
-			}
-
-			return uiFunctions.requestShutdown();
-
-		} catch (Exception e) {
-			Logger.log(new LogEvent(LOGID, "requestShutdown", e));
-		}
-
-		return false;
-	}
-
 	// @see com.aelitis.azureus.ui.UIFunctions#setStatusText(java.lang.String)
 	public void setStatusText(String string) {
 		// TODO Auto-generated method stub
@@ -479,17 +485,15 @@ public class UIFunctionsImpl
 	public MainStatusBar getMainStatusBar() {
 		return mainWindow.getMainStatusBar();
 	}
-
+	
 	// @see com.aelitis.azureus.ui.UIFunctions#showConfig(java.lang.String)
-	public boolean showConfig(String string) {
+	public boolean showConfig(String section) {
 		try {
-			UIFunctionsSWT uiFunctions = mainWindow.getOldUIFunctions(true);
-			if (uiFunctions == null) {
-				return false;
-			}
-
-			mainWindow.switchToAdvancedTab();
-			return uiFunctions.showConfig(string);
+			/*
+			 * Show in pop-up in Vuze UI's
+			 */
+			ConfigShell.getInstance().open(section);
+			return true;
 
 		} catch (Exception e) {
 			Logger.log(new LogEvent(LOGID, "showConfig", e));
@@ -498,135 +502,117 @@ public class UIFunctionsImpl
 		return false;
 	}
 
-	// @see com.aelitis.azureus.ui.UIFunctions#showMyShares()
-	public void showMyShares() {
-		try {
-			UIFunctionsSWT uiFunctions = mainWindow.getOldUIFunctions(true);
-			if (uiFunctions == null) {
+	public void openView(int viewID, Object data) {
+		if (mainWindow.isOnAdvancedView()) {
+			UIFunctionsSWT uiFunctions = mainWindow.getOldUIFunctions(false);
+			if (uiFunctions != null) {
+				uiFunctions.openView(viewID, data);
 				return;
 			}
-
-			mainWindow.switchToAdvancedTab();
-			uiFunctions.showMyShares();
-
-		} catch (Exception e) {
-			Logger.log(new LogEvent(LOGID, "showMyShares", e));
 		}
 
-	}
+		switch (viewID) {
+			case VIEW_CONSOLE:
+				mainWindow.openView(SideBar.SIDEBAR_SECTION_TOOLS, LoggerView.class,
+						null, data, true);
+				break;
 
-	// @see com.aelitis.azureus.ui.UIFunctions#showMyTorrents()
-	public void showMyTorrents() {
-		try {
-			UIFunctionsSWT uiFunctions = mainWindow.getOldUIFunctions(true);
-			if (uiFunctions == null) {
-				return;
+			case VIEW_ALLPEERS:
+				mainWindow.openView(SideBar.SIDEBAR_SECTION_TOOLS, PeerSuperView.class,
+						null, data, true);
+				break;
+
+			case VIEW_CONFIG:
+				showConfig((data instanceof String) ? (String) data : null);
+				break;
+
+			case VIEW_DM_DETAILS:
+				String id = "DMDetails_";
+				if (data instanceof DownloadManager) {
+					DownloadManager dm = (DownloadManager) data;
+					TOTorrent torrent = dm.getTorrent();
+					if (torrent != null) {
+						try {
+							id += torrent.getHashWrapper().toBase32String();
+						} catch (TOTorrentException e) {
+							e.printStackTrace();
+						}
+					}
+				}
+
+				mainWindow.openView(SideBar.SIDEBAR_SECTION_LIBRARY, ManagerView.class,
+						id, data, true);
+				break;
+
+			case VIEW_DM_MULTI_OPTIONS:
+				mainWindow.openView(SideBar.SIDEBAR_SECTION_LIBRARY,
+						TorrentOptionsView.class, null, data, true);
+				break;
+
+			case VIEW_MYSHARES:
+				mainWindow.openView(SideBar.SIDEBAR_SECTION_LIBRARY,
+						MySharesView.class, null, data, true);
+				break;
+
+			case VIEW_MYTORRENTS: {
+				SideBar sideBar = (SideBar) SkinViewManager.getByClass(SideBar.class);
+
+				if (sideBar != null) {
+					sideBar.showEntryByID(SideBar.SIDEBAR_SECTION_LIBRARY);
+				}
 			}
+				break;
 
-			mainWindow.switchToAdvancedTab();
-			uiFunctions.showMyTorrents();
+			case VIEW_MYTRACKER:
+				mainWindow.openView(SideBar.SIDEBAR_SECTION_TOOLS, MyTrackerView.class,
+						null, data, true);
+				break;
 
-		} catch (Exception e) {
-			Logger.log(new LogEvent(LOGID, "showMyTorrents", e));
-		}
+			case VIEW_STATS:
+				mainWindow.openView(SideBar.SIDEBAR_SECTION_TOOLS, StatsView.class,
+						null, data, true);
+				break;
 
-	}
+			case VIEW_DETAILED_LISTVIEW:
+				mainWindow.openView(SideBar.SIDEBAR_SECTION_TOOLS,
+						DetailedListView.class, null, data, true);
+				break;
 
-	// @see com.aelitis.azureus.ui.UIFunctions#showMyTracker()
-	public void showMyTracker() {
-		try {
-			UIFunctionsSWT uiFunctions = mainWindow.getOldUIFunctions(true);
-			if (uiFunctions == null) {
-				return;
-			}
-
-			mainWindow.switchToAdvancedTab();
-			uiFunctions.showMyTracker();
-
-		} catch (Exception e) {
-			Logger.log(new LogEvent(LOGID, "showMyTracker", e));
-		}
-
-	}
-
-	// @see com.aelitis.azureus.ui.UIFunctions#showStats()
-	public void showStats() {
-		try {
-			UIFunctionsSWT uiFunctions = mainWindow.getOldUIFunctions(true);
-			if (uiFunctions == null) {
-				return;
-			}
-
-			mainWindow.switchToAdvancedTab();
-			uiFunctions.showStats();
-
-		} catch (Exception e) {
-			Logger.log(new LogEvent(LOGID, "showStats", e));
-		}
-
-	}
-
-	// @see com.aelitis.azureus.ui.UIFunctions#showStatsDHT()
-	public void showStatsDHT() {
-		try {
-			UIFunctionsSWT uiFunctions = mainWindow.getOldUIFunctions(true);
-			if (uiFunctions == null) {
-				return;
-			}
-
-			mainWindow.switchToAdvancedTab();
-			uiFunctions.showStatsDHT();
-
-		} catch (Exception e) {
-			Logger.log(new LogEvent(LOGID, "showStatsDHT", e));
+			default:
+				break;
 		}
 	}
 
-	// @see com.aelitis.azureus.ui.UIFunctions#showStatsTransfers()
-	public void showStatsTransfers() {
-		try {
-			UIFunctionsSWT uiFunctions = mainWindow.getOldUIFunctions(true);
-			if (uiFunctions == null) {
-				return;
-			}
-
-			mainWindow.switchToAdvancedTab();
-			uiFunctions.showStatsTransfers();
-		} catch (Exception e) {
-			Logger.log(new LogEvent(LOGID, "", e));
-		}
-	}
-
-	// @see com.aelitis.azureus.ui.UIFunctions#showConsole()
-	public void showConsole() {
-		try {
-			UIFunctionsSWT uiFunctions = mainWindow.getOldUIFunctions(true);
-			if (uiFunctions == null) {
-				return;
-			}
-
-			mainWindow.switchToAdvancedTab();
-			uiFunctions.showConsole();
-		} catch (Exception e) {
-			Logger.log(new LogEvent(LOGID, "ShowConsole", e));
-		}
-	}
 
 	public UISWTInstance getUISWTInstance() {
 		return mainWindow.getUISWTInstanceImpl();
 	}
+	
+	// @see com.aelitis.azureus.ui.UIFunctions#viewURL(java.lang.String, java.lang.String, java.lang.String)
+	public void viewURL(String url, String target, String sourceRef) {
+		ContentNetworkUtils.setSourceRef(target, sourceRef, false);
+		viewURL(url, target, 0, 0, true, false);
+	}
 
 	public boolean viewURL(final String url, final String target, final int w,
 			final int h, final boolean allowResize, final boolean isModal) {
 
 		mainWindow.shell.getDisplay().syncExec(new AERunnable() {
 			public void runSupport() {
+				String realURL = url;
+				ContentNetwork cn = ContentNetworkUtils.getContentNetworkFromTarget(target);
+				if ( !realURL.startsWith( "http" )){
+					realURL = cn.getSiteRelativeURL(realURL, false );
+				}
 				if (target == null) {
-					BrowserWindow window = new BrowserWindow(mainWindow.shell, url, w, h,
-							allowResize, isModal);
+					if (UrlFilter.getInstance().urlCanRPC(realURL)) {
+						realURL = cn.appendURLSuffix(realURL, false, true);
+					}
+					BrowserWindow window = new BrowserWindow(mainWindow.shell, realURL,
+							w, h, allowResize, isModal);
 					window.waitUntilClosed();
 				} else {
-					mainWindow.showURL(url, target);
+					mainWindow.showURL(realURL, target);
 				}
 			}
 		});
@@ -638,20 +624,28 @@ public class UIFunctionsImpl
 
 		mainWindow.shell.getDisplay().syncExec(new AERunnable() {
 			public void runSupport() {
+				String realURL = url;
+				ContentNetwork cn = ContentNetworkUtils.getContentNetworkFromTarget(target);
+				if ( !realURL.startsWith( "http" )){
+					realURL = cn.getSiteRelativeURL(realURL, false );
+				}
 				if (target == null) {
-					BrowserWindow window = new BrowserWindow(mainWindow.shell, url, w, h,
-							allowResize, isModal);
+					if (UrlFilter.getInstance().urlCanRPC(realURL)) {
+						realURL = cn.appendURLSuffix(realURL, false, true);
+					}
+					BrowserWindow window = new BrowserWindow(mainWindow.shell, realURL,
+							w, h, allowResize, isModal);
 					window.waitUntilClosed();
 				} else {
-					mainWindow.showURL(url, target);
+					mainWindow.showURL(realURL, target);
 				}
 			}
 		});
 		return true;
 	}
 
-	protected void oldMainWindowInitialized(MainWindow oldMainWindow) {
-		UIFunctionsSWT uiFunctions = mainWindow.getOldUIFunctions(true);
+	public void oldMainWindowInitialized(MainWindow oldMainWindow) {
+		UIFunctionsSWT uiFunctions = oldMainWindow.getUIFunctions();
 		if (uiFunctions == null) {
 			return;
 		}
@@ -694,70 +688,71 @@ public class UIFunctionsImpl
 	}
 
 	public boolean isGlobalTransferBarShown() {
+		if (!AzureusCoreFactory.isCoreRunning()) {
+			return false;
+		}
 		return AllTransfersBar.getManager().isOpen(
 				AzureusCoreFactory.getSingleton().getGlobalManager());
 	}
 
 	public void showGlobalTransferBar() {
-		Utils.execSWTThread(new AERunnable() {
-			public void runSupport() {
-				AllTransfersBar.open(
-						AzureusCoreFactory.getSingleton().getGlobalManager(),
-						getMainShell());
+		CoreWaiterSWT.waitForCoreRunning(new AzureusCoreRunningListener() {
+			public void azureusCoreRunning(AzureusCore core) {
+				AllTransfersBar.open(core.getGlobalManager(), getMainShell());
 			}
 		});
 	}
 
 	public void closeGlobalTransferBar() {
-		Utils.execSWTThread(new AERunnable() {
-			public void runSupport() {
-				AllTransfersBar.close(AzureusCoreFactory.getSingleton().getGlobalManager());
+		CoreWaiterSWT.waitForCoreRunning(new AzureusCoreRunningListener() {
+			public void azureusCoreRunning(AzureusCore core) {
+				AllTransfersBar.close(core.getGlobalManager());
 			}
 		});
 	}
 
 	public void refreshTorrentMenu() {
 		try {
-			UIFunctionsSWT uiFunctions = mainWindow.getOldUIFunctions(true);
-			if (uiFunctions == null) {
+			if (mainWindow.isOnAdvancedView()) {
+				UIFunctionsSWT uiFunctions = mainWindow.getOldUIFunctions(false);
+
+				if (uiFunctions != null) {
+					uiFunctions.refreshTorrentMenu();
+				}
 				return;
 			}
 
-			uiFunctions.refreshTorrentMenu();
+			Utils.execSWTThread(new AERunnable() {
+				public void runSupport() {
+					final MenuItem torrentItem = MenuFactory.findMenuItem(
+							mainWindow.getMainMenu().getMenu(IMenuConstants.MENU_ID_MENU_BAR),
+							MenuFactory.MENU_ID_TORRENT);
+
+					if (null != torrentItem) {
+
+						DownloadManager[] dms = SelectedContentManager.getDMSFromSelectedContent();
+
+						final DownloadManager[] dm_final = dms;
+						final TableViewSWT tv_final = null;
+						final boolean detailed_view_final = false;
+						if (null == dm_final) {
+							torrentItem.setEnabled(false);
+						} else {
+							torrentItem.setData("downloads", dm_final);
+							torrentItem.setData("TableView", tv_final);
+							torrentItem.setData("is_detailed_view",
+									Boolean.valueOf(detailed_view_final));
+							torrentItem.setEnabled(true);
+						}
+					}
+				}
+			});
 
 		} catch (Exception e) {
 			Logger.log(new LogEvent(LOGID, "refreshTorrentMenu", e));
 		}
 	}
 
-	public void showAllPeersView() {
-		try {
-			UIFunctionsSWT uiFunctions = mainWindow.getOldUIFunctions(true);
-			if (uiFunctions == null) {
-				return;
-			}
-			mainWindow.switchToAdvancedTab();
-			uiFunctions.showAllPeersView();
-
-		} catch (Exception e) {
-			Logger.log(new LogEvent(LOGID, "showAllPeersView", e));
-		}
-	}
-
-	public void showMultiOptionsView(DownloadManager[] dms) {
-		try {
-			UIFunctionsSWT uiFunctions = mainWindow.getOldUIFunctions(true);
-			if (uiFunctions == null) {
-				return;
-			}
-
-			uiFunctions.showMultiOptionsView(dms);
-
-		} catch (Exception e) {
-			Logger.log(new LogEvent(LOGID, "showMultiOptionsView", e));
-		}
-	}
-
 	public IMainMenu createMainMenu(Shell shell) {
 		return new MainMenu(getSkin(), shell);
 	}
@@ -774,4 +769,152 @@ public class UIFunctionsImpl
 		return mainWindow;
 	}
 
+	// @see com.aelitis.azureus.ui.UIFunctions#getUIUpdater()
+	public UIUpdater getUIUpdater() {
+		return UIUpdaterSWT.getInstance();
+	}
+	
+	// @see com.aelitis.azureus.ui.swt.UIFunctionsSWT#closeAllDetails()
+	public void closeAllDetails() {
+		UIFunctionsSWT uiFunctions = mainWindow.getOldUIFunctions(false);
+		if (uiFunctions != null) {
+			uiFunctions.closeAllDetails();
+			return;
+		}
+
+		SkinView sideBarView = SkinViewManager.getByClass(SideBar.class);
+		if (sideBarView instanceof SideBar) {
+			SideBar sideBar = (SideBar) sideBarView;
+			SideBarEntry[] sideBarEntries = sideBar.getEntries();
+			for (int i = 0; i < sideBarEntries.length; i++) {
+				SideBarEntry entry = sideBarEntries[i];
+				String id = entry.getId();
+				if (id != null && id.startsWith("DMDetails_")) {
+					sideBar.closeEntry(id);
+				}
+			}
+		}
+
+	}
+	
+	// @see com.aelitis.azureus.ui.swt.UIFunctionsSWT#hasDetailViews()
+	public boolean hasDetailViews() {
+		UIFunctionsSWT uiFunctions = mainWindow.getOldUIFunctions(false);
+		if (uiFunctions != null) {
+			return uiFunctions.hasDetailViews();
+		}
+		SkinView sideBarView = SkinViewManager.getByClass(SideBar.class);
+		if (sideBarView instanceof SideBar) {
+			SideBar sideBar = (SideBar) sideBarView;
+			SideBarEntry[] sideBarEntries = sideBar.getEntries();
+			for (int i = 0; i < sideBarEntries.length; i++) {
+				SideBarEntry entry = sideBarEntries[i];
+				String id = entry.getId();
+				if (id != null && id.startsWith("DMDetails_")) {
+					return true;
+				}
+			}
+		}
+
+		return false;
+	}
+	
+	public void 
+	performAction(
+		int 			action_id, 
+		Object 			args, 
+		actionListener 	listener )
+	{
+		if ( action_id == ACTION_FULL_UPDATE ){
+			
+			FullUpdateWindow.handleUpdate((String)args, listener );
+			
+		}else{
+			
+			Debug.out( "Unknown action " + action_id );
+		}
+	}
+
+	// @see com.aelitis.azureus.ui.swt.UIFunctionsSWT#showCoreWaitDlg()
+	public Shell showCoreWaitDlg() {
+		final SkinnedDialog closeDialog = new SkinnedDialog(
+				"skin3_dlg_coreloading", "coreloading.body", SWT.TITLE | SWT.BORDER
+				| SWT.APPLICATION_MODAL);
+		
+		closeDialog.setTitle(MessageText.getString("dlg.corewait.title"));
+		SWTSkin skin = closeDialog.getSkin();
+		SWTSkinObjectButton soButton = (SWTSkinObjectButton) skin.getSkinObject("close");
+
+		final SWTSkinObjectText soWaitTask = (SWTSkinObjectText) skin.getSkinObject("task");
+
+		final SWTSkinObject soWaitProgress = skin.getSkinObject("progress");
+		if (soWaitProgress != null) {
+			soWaitProgress.getControl().addPaintListener(new PaintListener() {
+				public void paintControl(PaintEvent e) {
+					Control c = (Control) e.widget;
+					Point size = c.getSize();
+					e.gc.setBackground(ColorCache.getColor(e.display, "#0000ff"));
+					Object data = soWaitProgress.getData("progress");
+					if (data instanceof Long) {
+						int waitProgress = ((Long) data).intValue();
+						int breakX = size.x * waitProgress / 100;
+						e.gc.fillRectangle(0, 0, breakX, size.y);
+						e.gc.setBackground(ColorCache.getColor(e.display, "#808080"));
+						e.gc.fillRectangle(breakX, 0, size.x - breakX, size.y);
+					}
+				}
+			});
+		}
+		
+		AzureusCore core = AzureusCoreFactory.getSingleton();
+		if (!AzureusCoreFactory.isCoreRunning()) {
+			final Initializer initializer = Initializer.getLastInitializer();
+			if (initializer != null) {
+				initializer.addListener(new InitializerListener() {
+					public void reportPercent(final int percent) {
+						Utils.execSWTThread(new AERunnable() {
+							public void runSupport() {
+								if (soWaitProgress != null && !soWaitProgress.isDisposed()) {
+									soWaitProgress.setData("progress", new Long(percent));
+									soWaitProgress.getControl().redraw();
+									soWaitProgress.getControl().update();
+								}
+							}
+						});
+						if (percent > 100) {
+							initializer.removeListener(this);
+						}
+					}
+				
+					public void reportCurrentTask(String currentTask) {
+						if (soWaitTask != null && !soWaitTask.isDisposed()) {
+							soWaitTask.setText(currentTask);
+						}
+					}
+				});
+			}
+		}
+
+		if (soButton != null) {
+			soButton.addSelectionListener(new ButtonListenerAdapter() {
+				public void pressed(SWTSkinButtonUtility buttonUtility,
+						SWTSkinObject skinObject, int stateMask) {
+					closeDialog.close();
+				}
+			});
+		}
+
+		closeDialog.addCloseListener(new SkinnedDialogClosedListener() {
+			public void skinDialogClosed(SkinnedDialog dialog) {
+			}
+		});
+
+		closeDialog.open();
+		return closeDialog.getShell();
+	}
+	
+	// @see com.aelitis.azureus.ui.UIFunctions#doSearch(java.lang.String)
+	public void doSearch(String searchText) {
+		mainWindow.doSearch(searchText);
+	}
 }
