@@ -19,43 +19,30 @@
  */
 package com.aelitis.azureus.ui.swt.skin;
 
-import java.util.ArrayList;
-import java.util.Date;
-import java.util.HashMap;
-import java.util.Iterator;
+import java.util.*;
 import java.util.List;
-import java.util.Map;
+import java.util.concurrent.ConcurrentHashMap;
 
 import org.eclipse.swt.SWT;
 import org.eclipse.swt.browser.Browser;
 import org.eclipse.swt.custom.SashForm;
 import org.eclipse.swt.events.DisposeEvent;
 import org.eclipse.swt.events.DisposeListener;
-import org.eclipse.swt.graphics.Color;
-import org.eclipse.swt.graphics.Cursor;
-import org.eclipse.swt.graphics.Image;
-import org.eclipse.swt.graphics.Point;
-import org.eclipse.swt.graphics.Rectangle;
+import org.eclipse.swt.graphics.*;
 import org.eclipse.swt.layout.FormAttachment;
 import org.eclipse.swt.layout.FormData;
 import org.eclipse.swt.layout.FormLayout;
-import org.eclipse.swt.widgets.Canvas;
-import org.eclipse.swt.widgets.Composite;
-import org.eclipse.swt.widgets.Control;
-import org.eclipse.swt.widgets.Display;
-import org.eclipse.swt.widgets.Event;
-import org.eclipse.swt.widgets.Group;
-import org.eclipse.swt.widgets.Listener;
-import org.eclipse.swt.widgets.Sash;
-import org.eclipse.swt.widgets.Shell;
+import org.eclipse.swt.widgets.*;
+
 import org.gudy.azureus2.core3.util.AEMonitor;
 import org.gudy.azureus2.core3.util.Constants;
 import org.gudy.azureus2.core3.util.Debug;
+import org.gudy.azureus2.ui.swt.Utils;
 
+import com.aelitis.azureus.core.util.CopyOnWriteList;
 import com.aelitis.azureus.ui.IUIIntializer;
 import com.aelitis.azureus.ui.skin.SkinProperties;
-import com.aelitis.azureus.ui.swt.utils.ImageLoader;
-import com.aelitis.azureus.ui.swt.utils.ImageLoaderFactory;
+import com.aelitis.azureus.ui.swt.imageloader.ImageLoader;
 
 /**
  * @author TuxPaper
@@ -64,61 +51,83 @@ import com.aelitis.azureus.ui.swt.utils.ImageLoaderFactory;
  */
 public class SWTSkin
 {
-	static boolean DEBUGLAYOUT = System.getProperty("debuglayout") != null;
+	private static final SWTSkinObjectListener[] NOLISTENERS = new SWTSkinObjectListener[0];
 
-	private Map mapImageLoaders = new HashMap();
+	static boolean DEBUGLAYOUT = System.getProperty("debuglayout") != null;
+	
+	private static int numSkins = 0;
+
+	private Map<SkinProperties, ImageLoader> mapImageLoaders = new ConcurrentHashMap<SkinProperties, ImageLoader>();
 
 	private SWTSkinProperties skinProperties;
 
 	private Listener handCursorListener;
 
-	private Listener ontopPaintListener;
+	//private Listener ontopPaintListener;
 
 	// Key = Skin Object ID; Value = Array of SWTSkinObject
-	private HashMap mapIDsToControls = new HashMap();
+	private HashMap<String, SWTSkinObject[]> mapIDsToSOs = new HashMap<String, SWTSkinObject[]>();
+	
+	private AEMonitor mon_MapIDsToSOs = new AEMonitor("mapIDsToControls");
 
 	// Key = TabSet ID; Value = SWTSkinTabSet
-	private HashMap mapTabSetToControls = new HashMap();
+	private HashMap<String, SWTSkinTabSet> mapTabSetToControls = new HashMap<String, SWTSkinTabSet>();
 
 	// Key = Widget ID; Value = Array of SWTSkinObject
-	private HashMap mapPublicViewIDsToControls = new HashMap();
+	private HashMap<String, SWTSkinObject[]> mapPublicViewIDsToSOs = new HashMap<String, SWTSkinObject[]>();
 
-	private HashMap mapPublicViewIDsToListeners = new HashMap();
+	private AEMonitor mon_mapPublicViewIDsToSOs = new AEMonitor("mapPVIDsToSOs");
+
+	private HashMap<String, ArrayList<SWTSkinObjectListener>> mapPublicViewIDsToListeners = new HashMap<String, ArrayList<SWTSkinObjectListener>>();
 
 	private AEMonitor mapPublicViewIDsToListeners_mon = new AEMonitor(
-			"SWTSkin::mapPublicViewIDsToListeners");
+			"mapPVIDsToListeners");
 
-	private ArrayList ontopImages = new ArrayList();
+	private ArrayList<SWTSkinObjectBasic> ontopImages = new ArrayList<SWTSkinObjectBasic>();
 
 	private Shell shell;
 
 	private boolean bLayoutComplete = false;
 
-	private List listenersLayoutComplete = new ArrayList();
+	private CopyOnWriteList<SWTSkinLayoutCompleteListener> listenersLayoutComplete = new CopyOnWriteList<SWTSkinLayoutCompleteListener>();
 
-	private final ClassLoader classLoader;
-	
 	private boolean ourSkinProperties = false;
 
+	private int currentSkinObjectcreationCount = 0;
+
+	private ImageLoader imageLoader;
+
+	private String startID;
+
 	/**
 	 * 
 	 */
 	public SWTSkin() {
-		this.classLoader = SWTSkin.class.getClassLoader();
 		ourSkinProperties = true;
 		init(new SWTSkinPropertiesImpl());
 	}
 
 	public SWTSkin(ClassLoader classLoader, String skinPath, String mainSkinFile) {
-		this.classLoader = classLoader;
 		ourSkinProperties = true;
 		init(new SWTSkinPropertiesImpl(classLoader, skinPath, mainSkinFile));
 	}
-		
-	private void init(SWTSkinProperties skinProperties) {
-		this.skinProperties = skinProperties;
-		ImageLoaderFactory.createInstance(classLoader, Display.getDefault(), skinProperties);
 
+	private void init(SWTSkinProperties skinProperties) {
+		numSkins++;
+		this.skinProperties = skinProperties;
+		// the first skin gets the default image loader.  We don't add it to
+		// the mapImageLoaders because non-skin objects may use the image loader
+		// mapImageLoaders is used to dispose of images when the skin is disposed
+		if (numSkins == 1) {
+			imageLoader = ImageLoader.getInstance();
+			imageLoader.addSkinProperties(skinProperties);
+		} else {
+			imageLoader = new ImageLoader(Display.getDefault(),
+					skinProperties);
+			mapImageLoaders.put(skinProperties, imageLoader);
+		}
+
+		/*
 		ontopPaintListener = new Listener() {
 			public void handleEvent(Event event) {
 				for (Iterator iter = ontopImages.iterator(); iter.hasNext();) {
@@ -152,16 +161,20 @@ public class SWTSkin
 				}
 			}
 		};
+		*/
 	}
 
 	public ImageLoader getImageLoader(SkinProperties properties) {
-		ImageLoader loader = (ImageLoader) mapImageLoaders.get(properties);
+		if (properties == this.skinProperties) {
+			return imageLoader;
+		}
+		ImageLoader loader = mapImageLoaders.get(properties);
 
 		if (loader != null) {
 			return loader;
 		}
 
-		loader = new ImageLoader(classLoader, Display.getDefault(), properties);
+		loader = new ImageLoader(Display.getDefault(), properties);
 		mapImageLoaders.put(properties, loader);
 
 		return loader;
@@ -172,7 +185,7 @@ public class SWTSkin
 		if (DEBUGLAYOUT) {
 			System.out.println("addToControlMap: " + sID + " : " + skinObject);
 		}
-		addToArrayMap(mapIDsToControls, sID, skinObject);
+		addToSOArrayMap(mapIDsToSOs, mon_MapIDsToSOs, sID, skinObject);
 
 		// For SWT layout -- add a reverse lookup
 		Control control = skinObject.getControl();
@@ -182,45 +195,57 @@ public class SWTSkin
 		}
 	}
 
-	private void addToArrayMap(Map arrayMap, Object key, SWTSkinObject object) {
-		Object existing = arrayMap.get(key);
-		if (existing instanceof SWTSkinObject[]) {
-			SWTSkinObject[] existingObjects = (SWTSkinObject[]) existing;
-
-			boolean bAlreadyPresent = false;
-			for (int i = 0; i < existingObjects.length; i++) {
-				//System.out.println(".." + existingObjects[i]);
-				if (existingObjects[i] != null && existingObjects[i].equals(object)) {
-					bAlreadyPresent = true;
-					System.err.println("already present: " + key + "; " + object
-							+ "; existing: " + existingObjects[i]);
-					break;
-				}
+	private void addToSOArrayMap(Map<String, SWTSkinObject[]> arrayMap,
+			AEMonitor mon, String key, SWTSkinObject object) {
+		if (mon != null) {
+			mon.enter();
+		}
+		try {
+  		Object existing = arrayMap.get(key);
+  		if (existing instanceof SWTSkinObject[]) {
+  			SWTSkinObject[] existingObjects = (SWTSkinObject[]) existing;
+  
+  			boolean bAlreadyPresent = false;
+  			for (int i = 0; i < existingObjects.length; i++) {
+  				//System.out.println(".." + existingObjects[i]);
+  				if (existingObjects[i] != null && existingObjects[i].equals(object)) {
+  					bAlreadyPresent = true;
+  					System.err.println("already present: " + key + "; " + object
+  							+ "; existing: " + existingObjects[i] + " via "
+  							+ Debug.getCompressedStackTrace());
+  					break;
+  				}
+  			}
+  
+  			if (!bAlreadyPresent) {
+  				int length = existingObjects.length;
+  				SWTSkinObject[] newObjects = new SWTSkinObject[length + 1];
+  				System.arraycopy(existingObjects, 0, newObjects, 0, length);
+  				newObjects[length] = object;
+  
+  				arrayMap.put(key, newObjects);
+  				//				System.out.println("addToArrayMap: " + key + " : " + object + " #"
+  				//						+ (length + 1));
+  			}
+  		} else {
+  			arrayMap.put(key, new SWTSkinObject[] {
+  				object
+  			});
+  		}
+		} finally {
+			if (mon != null) {
+				mon.exit();
 			}
-
-			if (!bAlreadyPresent) {
-				int length = existingObjects.length;
-				SWTSkinObject[] newObjects = new SWTSkinObject[length + 1];
-				System.arraycopy(existingObjects, 0, newObjects, 0, length);
-				newObjects[length] = object;
-
-				arrayMap.put(key, newObjects);
-				//				System.out.println("addToArrayMap: " + key + " : " + object + " #"
-				//						+ (length + 1));
-			}
-		} else {
-			arrayMap.put(key, new SWTSkinObject[] {
-				object
-			});
 		}
 	}
 
-	private Object getFromArrayMap(Map arrayMap, Object key, SWTSkinObject parent) {
+	private Object getFromSOArrayMap(Map<String, SWTSkinObject[]> arrayMap,
+			Object key, SWTSkinObject parent) {
 		if (parent == null) {
 			return null;
 		}
 
-		SWTSkinObject[] objects = (SWTSkinObject[]) arrayMap.get(key);
+		SWTSkinObject[] objects = arrayMap.get(key);
 		if (objects == null) {
 			return null;
 		}
@@ -240,9 +265,11 @@ public class SWTSkin
 	}
 
 	private void setSkinObjectViewID(SWTSkinObject skinObject, String sViewID) {
-		addToArrayMap(mapPublicViewIDsToControls, sViewID, skinObject);
+		addToSOArrayMap(mapPublicViewIDsToSOs, mon_mapPublicViewIDsToSOs, sViewID,
+				skinObject);
 	}
 
+	/*
 	public void dumpObjects() {
 		System.out.println("=====");
 		FormData formdata;
@@ -270,29 +297,10 @@ public class SWTSkin
 			}
 		}
 	}
-
-	/**
-	 * Get all the SWTSkinObject of a specific type
-	 * 
-	 * @param sTypeRegEx Type of skin objects to find (container, image, etc)
-	 * @return
-	 *
-	 * @since 3.0.1.3
-	 */
-	public SWTSkinObject[] getSkinObjectsByType(String sTypeRegEx) {
-		List list = new ArrayList();
-		for (Iterator iter = mapIDsToControls.keySet().iterator(); iter.hasNext();) {
-			String sID = (String) iter.next();
-			SWTSkinObject so = getSkinObjectByID(sID);
-			if (so != null && so.getType().matches(sTypeRegEx)) {
-				list.add(so);
-			}
-		}
-		return (SWTSkinObject[]) list.toArray(new SWTSkinObject[0]);
-	}
+	*/
 
 	public SWTSkinObject getSkinObjectByID(String sID) {
-		SWTSkinObject[] objects = (SWTSkinObject[]) mapIDsToControls.get(sID);
+		SWTSkinObject[] objects = mapIDsToSOs.get(sID);
 		if (objects == null) {
 			return null;
 		}
@@ -306,51 +314,103 @@ public class SWTSkin
 			return getSkinObjectByID(sID);
 		}
 
-		return (SWTSkinObject) getFromArrayMap(mapIDsToControls, sID, parent);
+		return (SWTSkinObject) getFromSOArrayMap(mapIDsToSOs, sID, parent);
 	}
 
 	public SWTSkinObject getSkinObject(String sViewID) {
-		SWTSkinObject[] objects = (SWTSkinObject[]) mapPublicViewIDsToControls.get(sViewID);
-		if (objects == null) {
-			return null;
+		SWTSkinObject[] objects = mapPublicViewIDsToSOs.get(sViewID);
+		if (objects == null || objects.length == 0) {
+			if (!Utils.isThisThreadSWT()) {
+				Debug.out("View "
+						+ sViewID
+						+ " does not exist.  Skipping unattach check because not in SWT thread");
+				return null;
+			}
+			return createUnattachedView(sViewID, null);
 		}
 
 		return objects[0];
 	}
 
+	/**
+	 * @param viewID
+	 * @return
+	 *
+	 * @since 3.1.1.1
+	 */
+	private SWTSkinObject createUnattachedView(String viewID, SWTSkinObject parent) {
+		String unattachedView = skinProperties.getStringValue("UnattachedView."
+				+ viewID);
+		if (unattachedView != null) {
+			if (unattachedView.indexOf(',') > 0) {
+				String[] split = unattachedView.split(",");
+				String parentID = split[1];
+				SWTSkinObject soParent = getSkinObjectByID(parentID, parent);
+				if (soParent != null) {
+					String configID = split[0];
+					return createSkinObject(configID, configID, soParent);
+				}
+			}
+
+			SWTSkinObjectListener[] listeners = getSkinObjectListeners(viewID);
+			for (int i = 0; i < listeners.length; i++) {
+				SWTSkinObjectListener l = listeners[i];
+				Object o = l.eventOccured(null,
+						SWTSkinObjectListener.EVENT_CREATE_REQUEST, new String[] {
+							viewID,
+							unattachedView
+						});
+				if (o instanceof SWTSkinObject) {
+					return (SWTSkinObject) o;
+				}
+			}
+		}
+		return null;
+	}
+
 	public SWTSkinObject getSkinObject(String sViewID, SWTSkinObject parent) {
 		if (parent == null) {
 			// XXX Search for parent is shell directly
 			return getSkinObject(sViewID);
 		}
 
-		return (SWTSkinObject) getFromArrayMap(mapPublicViewIDsToControls, sViewID,
-				parent);
+		String parentViewID = parent.getViewID();
+		if (parentViewID != null && parentViewID.equals(sViewID)) {
+			return parent;
+		}
+
+		SWTSkinObject so = (SWTSkinObject) getFromSOArrayMap(
+				mapPublicViewIDsToSOs, sViewID, parent);
+		if (so == null) {
+			so = createUnattachedView(sViewID, parent);
+		}
+
+		return so;
 	}
 
 	public SWTSkinTabSet getTabSet(String sID) {
-		return (SWTSkinTabSet) mapTabSetToControls.get(sID);
+		return mapTabSetToControls.get(sID);
 	}
 
 	public SWTSkinObjectTab activateTab(SWTSkinObject skinObjectInTab) {
 		if (skinObjectInTab == null) {
 			return null;
 		}
-		
+
 		if (skinObjectInTab instanceof SWTSkinObjectTab) {
-			SWTSkinObjectTab tab = (SWTSkinObjectTab)skinObjectInTab;
+			SWTSkinObjectTab tab = (SWTSkinObjectTab) skinObjectInTab;
 			tab.getTabset().setActiveTab(tab);
 			return tab;
 		}
 
-		for (Iterator iter = mapTabSetToControls.values().iterator(); iter.hasNext();) {
-			SWTSkinTabSet tabset = (SWTSkinTabSet) iter.next();
+		for (Iterator<SWTSkinTabSet> iter = mapTabSetToControls.values().iterator(); iter.hasNext();) {
+			SWTSkinTabSet tabset = iter.next();
 
 			SWTSkinObjectTab[] tabs = tabset.getTabs();
 			boolean bHasSkinObject = false;
 			for (int i = 0; i < tabs.length; i++) {
 				SWTSkinObjectTab tab = tabs[i];
-				SWTSkinObject[] activeWidgets = tab.getActiveWidgets();
+				SWTSkinObject[] activeWidgets = tab.getActiveWidgets(true);
 				for (int j = 0; j < activeWidgets.length; j++) {
 					SWTSkinObject object = activeWidgets[j];
 					//System.out.println("check " + tab + ";" + object + " for " + skinObjectInTab);
@@ -408,26 +468,99 @@ public class SWTSkin
 		initialize(shell, startID, null);
 	}
 
-	public void initialize(Shell shell, String startID,
+	public void initialize(final Shell shell, String startID,
 			IUIIntializer uiInitializer) {
 
 		this.shell = shell;
+		this.startID = startID;
 		FormLayout layout = new FormLayout();
 		shell.setLayout(layout);
 		shell.setBackgroundMode(SWT.INHERIT_DEFAULT);
-		
+
 		shell.addDisposeListener(new DisposeListener() {
 			public void widgetDisposed(DisposeEvent e) {
-				for (Iterator iter = mapImageLoaders.values().iterator(); iter.hasNext();) {
-					ImageLoader loader = (ImageLoader) iter.next();
-					loader.unLoadImages();
+				disposeSkin();
+			}
+		});
+
+		Listener l = new Listener() {
+			Control lastControl = null;
+
+			public void handleEvent(Event event) {
+				if (shell.isDisposed() && event.display != null) {
+					event.display.removeFilter(SWT.MouseMove, this);
+					event.display.removeFilter(SWT.MouseExit, this);
+					return;
 				}
-				if (ourSkinProperties) {
-					//skinProperties.dispose();
+				Control cursorControl = shell.getDisplay().getCursorControl();
+				//System.out.println("move from " + (lastControl == null ? null : lastControl.handle) + " to " + (cursorControl == null ? "null" : cursorControl.handle));
+				if (cursorControl != lastControl) {
+					while (lastControl != null && !lastControl.isDisposed()) {
+						SWTSkinObjectBasic so = (SWTSkinObjectBasic) lastControl.getData("SkinObject");
+						if (so != null) {
+							so.switchSuffix("", 3, false, false);
+						}
+
+						lastControl = lastControl.getParent();
+					}
+					lastControl = cursorControl;
+
+					while (cursorControl != null) {
+						SWTSkinObjectBasic so = (SWTSkinObjectBasic) cursorControl.getData("SkinObject");
+						if (so != null) {
+							so.switchSuffix("-over", 3, false, false);
+						}
+
+						cursorControl = cursorControl.getParent();
+					}
+				}
+			}
+		};
+		shell.getDisplay().addFilter(SWT.MouseMove, l);
+		// can't just add a MouseExit listener to the shell, because it doesn't
+		// get fired when a control is on the edge
+		shell.getDisplay().addFilter(SWT.MouseExit, l);
+		shell.addListener(SWT.Deactivate, l);
+		shell.addListener(SWT.Activate, l);
+
+		/****** REPLACED BY MouseMove
+		// When shell activates or deactivates, send a MouseEnter or MouseExit
+		// This fixes the problem where we are hovering over a skinobject with
+		// a "-over" state, we tab away, move the mouse, and tab back again.
+		// Without this code, the skinobject would still be in "-over" state
+		shell.addListener(SWT.Deactivate, new Listener() {
+			public void handleEvent(Event event) {
+				Control cursorControl = shell.getDisplay().getCursorControl();
+				if (cursorControl != null) {
+					while (cursorControl != null) {
+						Event mouseExitEvent = new Event();
+						mouseExitEvent.type = SWT.MouseExit;
+						mouseExitEvent.widget = cursorControl;
+						shell.getDisplay().post(mouseExitEvent);
+						System.out.println(cursorControl.getData("SkinObject"));
+						
+						cursorControl = cursorControl.getParent();
+					}
 				}
 			}
 		});
-		
+		shell.addListener(SWT.Activate, new Listener() {
+			public void handleEvent(Event event) {
+				Control cursorControl = shell.getDisplay().getCursorControl();
+				if (cursorControl != null) {
+					while (cursorControl != null) {
+						Event mouseExitEvent = new Event();
+						mouseExitEvent.type = SWT.MouseEnter;
+						mouseExitEvent.widget = cursorControl;
+						shell.getDisplay().post(mouseExitEvent);
+						
+						cursorControl = cursorControl.getParent();
+					}
+				}
+			}
+		});
+		*/
+
 		Color bg = skinProperties.getColor(startID + ".color");
 		if (bg != null) {
 			shell.setBackground(bg);
@@ -438,6 +571,19 @@ public class SWTSkin
 			shell.setForeground(fg);
 		}
 		
+		
+		int width = skinProperties.getIntValue(startID + ".width", -1);
+		int height = skinProperties.getIntValue(startID + ".height", -1);
+		if (width > 0 && height > 0) {
+			shell.setSize(width, height);
+		}
+		
+		String title = skinProperties.getStringValue(startID + ".title",
+				(String) null);
+		if (title != null) {
+			shell.setText(title);
+		}
+
 		String[] sMainGroups = skinProperties.getStringArray(startID + ".widgets");
 		if (sMainGroups == null) {
 			System.out.println("NO " + startID + ".widgets!!");
@@ -450,7 +596,7 @@ public class SWTSkin
 			if (DEBUGLAYOUT) {
 				System.out.println("Container: " + sID);
 			}
-			
+
 			if (uiInitializer != null) {
 				uiInitializer.increaseProgress();
 			}
@@ -461,7 +607,23 @@ public class SWTSkin
 
 	/**
 	 * 
+	 *
+	 * @since 4.0.0.5
 	 */
+	private void disposeSkin() {
+		numSkins--;
+		for (Iterator<ImageLoader> iter = mapImageLoaders.values().iterator(); iter.hasNext();) {
+			ImageLoader loader = iter.next();
+			loader.unLoadImages();
+		}
+		if (ourSkinProperties) {
+			//skinProperties.dispose();
+		}
+	}
+
+	/**
+	 * 
+	 *
 	private void addPaintListenerToAll(Control control) {
 		// XXX: Bug: When paint listener is set to shell, browser widget will flicker on OSX when resizing
 		if (!(control instanceof Shell)) {
@@ -477,22 +639,24 @@ public class SWTSkin
 			}
 		}
 	}
+	*/
 
 	public void layout() {
 		if (DEBUGLAYOUT) {
 			System.out.println("==== Start Apply Layout");
 		}
 		// Apply layout data from skin
-		for (Iterator iter = mapIDsToControls.keySet().iterator(); iter.hasNext();) {
-			String sID = (String) iter.next();
-			SWTSkinObject[] objects = (SWTSkinObject[]) mapIDsToControls.get(sID);
-
-			if (DEBUGLAYOUT) {
-				System.out.println("Apply Layout for " + objects.length + " " + sID);
-			}
-
-			for (int i = 0; i < objects.length; i++) {
-				attachControl(objects[i]);
+		Object[] values = mapIDsToSOs.values().toArray();
+		for (int i = 0; i < values.length; i++) {
+			SWTSkinObject[] skinObjects = (SWTSkinObject[]) values[i];
+			if (skinObjects != null) {
+				for (int j = 0; j < skinObjects.length; j++) {
+					SWTSkinObject skinObject = skinObjects[j];
+					if (DEBUGLAYOUT) {
+						System.out.println("Apply Layout for " + skinObject);
+					}
+					attachControl(skinObject);
+				}
 			}
 		}
 
@@ -501,29 +665,41 @@ public class SWTSkin
 
 		bLayoutComplete = true;
 
-		Object[] listeners = listenersLayoutComplete.toArray();
-		for (int i = 0; i < listeners.length; i++) {
-			SWTSkinLayoutCompleteListener l = (SWTSkinLayoutCompleteListener) listeners[i];
+		int width = skinProperties.getIntValue(startID + ".width", -1);
+		int height = skinProperties.getIntValue(startID + ".height", -1);
+		if (width > 0 && height <= 0) {
+			shell.pack();
+			shell.setSize(shell.computeSize(width, SWT.DEFAULT));
+		}
+		
+		for (SWTSkinLayoutCompleteListener l : listenersLayoutComplete) {
 			l.skinLayoutCompleted();
 		}
+		listenersLayoutComplete.clear();
 
 		if (DEBUGLAYOUT) {
 			System.out.println("==== End Apply Layout");
 		}
+
+		skinProperties.clearCache();
 	}
 
 	/**
 	 * @param controlToLayout
 	 * @param id
 	 */
-	private void attachControl(SWTSkinObject skinObject) {
+	void attachControl(SWTSkinObject skinObject) {
 		if (skinObject == null) {
 			return;
 		}
 
 		Control controlToLayout = skinObject.getControl();
 
-		if (controlToLayout == null) {
+		if (controlToLayout == null || controlToLayout.isDisposed()) {
+			return;
+		}
+
+		if (controlToLayout.getData("skin.layedout") != null) {
 			return;
 		}
 
@@ -742,19 +918,22 @@ public class SWTSkin
 				newFormData.width);
 		if (!skinObject.getDefaultVisibility()) {
 			if (newFormData.width != 0 && newFormData.height != 0) {
-				controlToLayout.setData("oldSize", new Point(newFormData.width, newFormData.height));
+				controlToLayout.setData("oldSize", new Point(newFormData.width,
+						newFormData.height));
 			}
 			newFormData.width = 0;
 			newFormData.height = 0;
 		}
 		controlToLayout.setLayoutData(newFormData);
+		controlToLayout.setData("skin.layedout", "");
 	}
 
-	private SWTSkinObject createContainer(SWTSkinProperties properties,
+	private SWTSkinObject createContainer(final SWTSkinProperties properties,
 			String sID, final String sConfigID, SWTSkinObject parentSkinObject,
 			boolean bForceCreate, boolean bPropogate, SWTSkinObject intoSkinObject) {
 		String[] sItems = properties.getStringArray(sConfigID + ".widgets");
-		if (sItems == null && !bForceCreate) {
+		final String[] sItemsLater = null; // properties.getStringArray(sConfigID + ".widgets-onshow");
+		if (sItems == null && sItemsLater == null && !bForceCreate) {
 			return null;
 		}
 
@@ -791,30 +970,23 @@ public class SWTSkin
 		}
 
 		if (sItems != null) {
-			String[] paramValues = null;
-			if (properties instanceof SWTSkinPropertiesParam) {
-				paramValues = ((SWTSkinPropertiesParam) properties).getParamValues();
-			}
-			// Cloning is only for one level.  Children get the original properties
-			// object
-			if (properties instanceof SWTSkinPropertiesClone) {
-				properties = ((SWTSkinPropertiesClone) properties).getOriginalProperties();
-			}
-
-			// Propogate any parameter values.
-			// XXX This could get ugly, we should could the # of 
-			//     SWTSkinPropertiesParam to determine if this needs optimizing
-			//     ie. if a top container has paramValues, every child will get a new
-			//         object.  How would this affect memory/performace?
-			if (paramValues != null) {
-				properties = new SWTSkinPropertiesParamImpl(properties, paramValues);
-			}
-
-			for (int i = 0; i < sItems.length; i++) {
-				String sItemID = sItems[i];
-				linkIDtoParent(properties, sItemID, sItemID, skinObject, false, true);
-			}
+			addContainerChildren(skinObject, sItems, properties);
 		}
+		if (sItemsLater != null) {
+			skinObject.addListener(new SWTSkinObjectListener() {
+				public Object eventOccured(SWTSkinObject skinObject, int eventType,
+						Object params) {
+					if (eventType == EVENT_SHOW) {
+						skinObject.removeListener(this);
+						bLayoutComplete = false;
+						addContainerChildren(skinObject, sItemsLater, properties);
+						layout();
+					}
+					return null;
+				}
+			});
+		}
+		
 
 		if (bLayoutComplete) {
 			attachControl(skinObject);
@@ -823,6 +995,33 @@ public class SWTSkin
 		return skinObject;
 	}
 
+	private void addContainerChildren(SWTSkinObject skinObject, String[] sItems,
+			SWTSkinProperties properties) {
+		String[] paramValues = null;
+		if (properties instanceof SWTSkinPropertiesParam) {
+			paramValues = ((SWTSkinPropertiesParam) properties).getParamValues();
+		}
+		// Cloning is only for one level.  Children get the original properties
+		// object
+		if (properties instanceof SWTSkinPropertiesClone) {
+			properties = ((SWTSkinPropertiesClone) properties).getOriginalProperties();
+		}
+
+		// Propogate any parameter values.
+		// XXX This could get ugly, we should could the # of 
+		//     SWTSkinPropertiesParam to determine if this needs optimizing
+		//     ie. if a top container has paramValues, every child will get a new
+		//         object.  How would this affect memory/performace?
+		if (paramValues != null) {
+			properties = new SWTSkinPropertiesParamImpl(properties, paramValues);
+		}
+
+		for (int i = 0; i < sItems.length; i++) {
+			String sItemID = sItems[i];
+			linkIDtoParent(properties, sItemID, sItemID, skinObject, false, true);
+		}
+	}
+
 	private SWTSkinObject createSash(SWTSkinProperties properties, String sID,
 			String sConfigID, SWTSkinObject parentSkinObject, final boolean bVertical) {
 		int style = bVertical ? SWT.VERTICAL : SWT.HORIZONTAL;
@@ -956,7 +1155,7 @@ public class SWTSkin
 
 		String sTabSet = properties.getStringValue(sConfigID + ".tabset", "default");
 
-		SWTSkinTabSet tabset = (SWTSkinTabSet) mapTabSetToControls.get(sTabSet);
+		SWTSkinTabSet tabset = mapTabSetToControls.get(sTabSet);
 		if (tabset == null) {
 			tabset = new SWTSkinTabSet(this, sTabSet);
 			mapTabSetToControls.put(sTabSet, tabset);
@@ -989,10 +1188,9 @@ public class SWTSkin
 
 		return skinObject;
 	}
-	
-	private SWTSkinObject createSlider(SWTSkinProperties properties,
-			String sID, String sConfigID, String[] typeParams,
-			SWTSkinObject parentSkinObject) {
+
+	private SWTSkinObject createSlider(SWTSkinProperties properties, String sID,
+			String sConfigID, String[] typeParams, SWTSkinObject parentSkinObject) {
 		SWTSkinObject skinObject = new SWTSkinObjectSlider(this, properties, sID,
 				sConfigID, typeParams, parentSkinObject);
 		addToControlMap(skinObject);
@@ -1003,7 +1201,6 @@ public class SWTSkin
 
 		return skinObject;
 	}
-	
 
 	public Shell getShell() {
 		return shell;
@@ -1024,6 +1221,7 @@ public class SWTSkin
 	//		return;
 	//	}
 
+	/* Used for dumpObjectsOnly
 	private String getAttachLine(FormAttachment attach) {
 		String s = "";
 		if (attach.control != null) {
@@ -1050,6 +1248,7 @@ public class SWTSkin
 		}
 		return s;
 	}
+	*/
 
 	protected Listener getHandCursorListener(Display display) {
 		if (handCursorListener == null) {
@@ -1069,6 +1268,11 @@ public class SWTSkin
 		return handCursorListener;
 	}
 
+	public SWTSkinObject createSkinObject(String sID, String sConfigID,
+			SWTSkinObject parentSkinObject) {
+		return createSkinObject(sID, sConfigID, parentSkinObject, null);
+	}
+
 	/**
 	 * Create a skin object based off an existing config "template"
 	 *  
@@ -1079,15 +1283,32 @@ public class SWTSkin
 	 * @return new skin object
 	 */
 	public SWTSkinObject createSkinObject(String sID, String sConfigID,
-			SWTSkinObject parentSkinObject) {
-		SWTSkinObject skinObject = linkIDtoParent(skinProperties, sID, sConfigID,
-				parentSkinObject, true, true);
-		if (bLayoutComplete && skinObject != null) {
-			Control control = skinObject.getParent().getControl();
-			if (control instanceof Composite) {
-				((Composite) control).layout(true);
+			SWTSkinObject parentSkinObject, Object creationParams) {
+		SWTSkinObject skinObject = null;
+		Cursor cursor = shell.getCursor();
+		try {
+			shell.setCursor(shell.getDisplay().getSystemCursor(SWT.CURSOR_WAIT));
+
+			boolean b = bLayoutComplete;
+			bLayoutComplete = false;
+
+			skinObject = linkIDtoParent(skinProperties, sID, sConfigID,
+					parentSkinObject, true, true, creationParams);
+			if (b && skinObject != null) {
+				layout();
+				Control control = skinObject.getParent().getControl();
+				if (control instanceof Composite) {
+					((Composite) control).layout(true);
+				}
 			}
+			bLayoutComplete = b;
+		} catch (Exception e) {
+			Debug.out("Trying to create " + sID + "." + sConfigID + " on "
+					+ parentSkinObject, e);
+		} finally {
+			shell.setCursor(cursor);
 		}
+
 		return skinObject;
 	}
 
@@ -1108,22 +1329,43 @@ public class SWTSkin
 	public void removeSkinObject(SWTSkinObject skinObject) {
 		skinObject.triggerListeners(SWTSkinObjectListener.EVENT_DESTROY);
 
-		SWTSkinObject[] objects = (SWTSkinObject[]) mapIDsToControls.get(skinObject.getSkinObjectID());
-		if (objects != null) {
-			for (int i = 0; i < objects.length; i++) {
-				if (objects[i] == skinObject) {
-					objects[i] = null;
-				}
-			}
+		String id = skinObject.getSkinObjectID();
+		mon_MapIDsToSOs.enter();
+		try {
+  		SWTSkinObject[] objects = mapIDsToSOs.get(id);
+  		if (objects != null) {
+  			int x = 0;
+  			for (int i = 0; i < objects.length; i++) {
+  				if (objects[i] != skinObject) {
+  					objects[x++] = objects[i];
+  				}
+  			}
+  
+  			SWTSkinObject[] newObjects = new SWTSkinObject[x];
+  			System.arraycopy(objects, 0, newObjects, 0, x);
+  			mapIDsToSOs.put(id, newObjects);
+  		}
+		} finally {
+			mon_MapIDsToSOs.exit();
 		}
 
-		objects = (SWTSkinObject[]) mapPublicViewIDsToControls.get(skinObject.getViewID());
-		if (objects != null) {
-			for (int i = 0; i < objects.length; i++) {
-				if (objects[i] == skinObject) {
-					objects[i] = null;
-				}
-			}
+		mon_mapPublicViewIDsToSOs.enter();
+		try {
+  		id = skinObject.getViewID();
+  		SWTSkinObject[] objects = mapPublicViewIDsToSOs.get(id);
+  		if (objects != null) {
+  			int x = 0;
+  			for (int i = 0; i < objects.length; i++) {
+  				if (objects[i] != skinObject) {
+  					objects[x++] = objects[i];
+  				}
+  			}
+  			SWTSkinObject[] newObjects = new SWTSkinObject[x];
+  			System.arraycopy(objects, 0, newObjects, 0, x);
+  			mapPublicViewIDsToSOs.put(id, newObjects);
+  		}
+		} finally {
+			mon_mapPublicViewIDsToSOs.exit();
 		}
 
 		skinObject.dispose();
@@ -1132,8 +1374,20 @@ public class SWTSkin
 	private SWTSkinObject linkIDtoParent(SWTSkinProperties properties,
 			String sID, String sConfigID, SWTSkinObject parentSkinObject,
 			boolean bForceCreate, boolean bAddView) {
+		return linkIDtoParent(properties, sID, sConfigID, parentSkinObject,
+				bForceCreate, bAddView, null);
+	}
+
+	private SWTSkinObject linkIDtoParent(SWTSkinProperties properties,
+			String sID, String sConfigID, SWTSkinObject parentSkinObject,
+			boolean bForceCreate, boolean bAddView, Object creationParams) {
+		currentSkinObjectcreationCount++;
+
 		SWTSkinObject skinObject = null;
 		try {
+			if (sConfigID == null) {
+				return null;
+			}
 			String[] sTypeParams = properties.getStringArray(sConfigID + ".type");
 			String sType;
 			if (sTypeParams != null && sTypeParams.length > 0) {
@@ -1208,13 +1462,21 @@ public class SWTSkin
 				return null;
 			} else if (sType.equals("browser")) {
 				skinObject = createBrowser(properties, sID, sConfigID, parentSkinObject);
-			}  else if (sType.equals("separator")) {
+			} else if (sType.equals("separator")) {
 				skinObject = createSeparator(properties, sID, sConfigID, sTypeParams,
 						parentSkinObject);
-			}else {
+			} else if (sType.equals("button")) {
+				skinObject = createButton(properties, sID, sConfigID, sTypeParams,
+						parentSkinObject);
+			} else if (sType.equals("checkbox")) {
+				skinObject = createCheckbox(properties, sID, sConfigID, sTypeParams,
+						parentSkinObject);
+			} else {
 				System.err.println(sConfigID + ": Invalid type of " + sType);
 			}
 
+			skinObject.setData("CreationParams", creationParams);
+
 			if (bAddView) {
 				String sViewID = skinObject.getViewID();
 				if (sViewID != null) {
@@ -1227,6 +1489,50 @@ public class SWTSkin
 			}
 		} catch (Exception e) {
 			e.printStackTrace();
+		} finally {
+			currentSkinObjectcreationCount--;
+		}
+
+		if (skinObject != null) {
+			skinObject.triggerListeners(SWTSkinObjectListener.EVENT_CREATED);
+		}
+
+		return skinObject;
+	}
+
+	/**
+	 * @param properties
+	 * @param sid
+	 * @param configID
+	 * @param typeParams
+	 * @param parentSkinObject
+	 * @return
+	 *
+	 * @since 3.1.1.1
+	 */
+	private SWTSkinObject createButton(SWTSkinProperties properties, String id,
+			String configID, String[] typeParams, SWTSkinObject parentSkinObject) {
+
+		SWTSkinObject skinObject = new SWTSkinObjectButton(this, properties, id,
+				configID, parentSkinObject);
+		addToControlMap(skinObject);
+
+		if (bLayoutComplete) {
+			attachControl(skinObject);
+		}
+
+		return skinObject;
+	}
+
+	private SWTSkinObject createCheckbox(SWTSkinProperties properties, String id,
+			String configID, String[] typeParams, SWTSkinObject parentSkinObject) {
+
+		SWTSkinObject skinObject = new SWTSkinObjectCheckbox(this, properties, id,
+				configID, parentSkinObject);
+		addToControlMap(skinObject);
+
+		if (bLayoutComplete) {
+			attachControl(skinObject);
 		}
 
 		return skinObject;
@@ -1260,8 +1566,8 @@ public class SWTSkin
 			System.err.println("XXXXXXXX " + sID + " has no config ID.."
 					+ Debug.getStackTrace(false, false));
 		}
-		
-				String[] sCloneParams;
+
+		String[] sCloneParams;
 		if (typeParams.length > 1) {
 			int size = typeParams.length - 1;
 			sCloneParams = new String[size];
@@ -1292,6 +1598,7 @@ public class SWTSkin
 			sImageID = sConfigID;
 			if (typeParams.length > 1) {
 				properties.addProperty(sConfigID + ".image", typeParams[1]);
+				sImageID = typeParams[1];
 			}
 		}
 		SWTSkinObjectImage skinObject = new SWTSkinObjectImage(this, properties,
@@ -1318,8 +1625,8 @@ public class SWTSkin
 
 		if (skinObject == null) {
 			if (intoSkinObject == null) {
-				skinObject = new SWTSkinObjectImageContainer(this, properties, sID,
-						sConfigID, parentSkinObject);
+				//skinObject = new SWTSkinObjectImageContainer(this, properties, sID,
+				//		sConfigID, parentSkinObject);
 				addToControlMap(skinObject);
 			} else {
 				skinObject = intoSkinObject;
@@ -1375,7 +1682,7 @@ public class SWTSkin
 	}
 
 	private SWTSkinObject createImageLabel2(SWTSkinProperties properties,
-			String sConfigID, SWTSkinObject parentSkinObject) {
+			final String sConfigID, SWTSkinObject parentSkinObject) {
 		Composite createOn;
 		if (parentSkinObject == null) {
 			createOn = shell;
@@ -1386,10 +1693,22 @@ public class SWTSkin
 		final Canvas drawable = new Canvas(createOn, SWT.NO_BACKGROUND);
 		drawable.setVisible(false);
 
-		ImageLoader imageLoader = getImageLoader(properties);
+		final ImageLoader imageLoader = getImageLoader(properties);
 		Image image = imageLoader.getImage(sConfigID);
 		if (ImageLoader.isRealImage(image)) {
+			imageLoader.releaseImage(sConfigID);
 			image = imageLoader.getImage(sConfigID + ".image");
+			drawable.addDisposeListener(new DisposeListener() {
+				public void widgetDisposed(DisposeEvent e) {
+					imageLoader.releaseImage(sConfigID + ".image");
+				}
+			});
+		} else {
+			drawable.addDisposeListener(new DisposeListener() {
+				public void widgetDisposed(DisposeEvent e) {
+					imageLoader.releaseImage(sConfigID);
+				}
+			});
 		}
 		drawable.setData("image", image);
 
@@ -1402,12 +1721,11 @@ public class SWTSkin
 		return skinObject;
 	}
 
-	
 	private SWTSkinObject createSeparator(SWTSkinProperties properties,
 			String sID, String sConfigID, String[] typeParams,
 			SWTSkinObject parentSkinObject) {
-		SWTSkinObject skinObject = new SWTSkinObjectSeparator(this, properties, sID,
-				sConfigID, parentSkinObject);
+		SWTSkinObject skinObject = new SWTSkinObjectSeparator(this, properties,
+				sID, sConfigID, parentSkinObject);
 		addToControlMap(skinObject);
 
 		if (bLayoutComplete) {
@@ -1416,8 +1734,7 @@ public class SWTSkin
 
 		return skinObject;
 	}
-	
-	
+
 	public SWTSkinProperties getSkinProperties() {
 		return skinProperties;
 	}
@@ -1428,10 +1745,10 @@ public class SWTSkin
 			Object existing = mapPublicViewIDsToListeners.get(viewID);
 
 			if (existing instanceof List) {
-				List list = (List) existing;
+				List<SWTSkinObjectListener> list = (List<SWTSkinObjectListener>) existing;
 				list.add(listener);
 			} else {
-				ArrayList list = new ArrayList();
+				ArrayList<SWTSkinObjectListener> list = new ArrayList<SWTSkinObjectListener>();
 				list.add(listener);
 				mapPublicViewIDsToListeners.put(viewID, list);
 			}
@@ -1440,26 +1757,38 @@ public class SWTSkin
 		}
 	}
 
-	public SWTSkinObjectListener[] getSkinObjectListeners(String viewID) {
-		if (viewID == null) {
-			return new SWTSkinObjectListener[0];
-		}
-
+	public void removeListener(String viewID, SWTSkinObjectListener listener) {
 		mapPublicViewIDsToListeners_mon.enter();
 		try {
 			Object existing = mapPublicViewIDsToListeners.get(viewID);
 
 			if (existing instanceof List) {
-				List list = (List) existing;
-				return (SWTSkinObjectListener[]) list.toArray(new SWTSkinObjectListener[0]);
-			} else {
-				return new SWTSkinObjectListener[0];
+				List<SWTSkinObjectListener> list = (List<SWTSkinObjectListener>) existing;
+				list.remove(listener);
 			}
 		} finally {
 			mapPublicViewIDsToListeners_mon.exit();
 		}
 	}
 
+	public SWTSkinObjectListener[] getSkinObjectListeners(String viewID) {
+		if (viewID == null) {
+			return NOLISTENERS;
+		}
+
+		mapPublicViewIDsToListeners_mon.enter();
+		try {
+			ArrayList<SWTSkinObjectListener> existing = mapPublicViewIDsToListeners.get(viewID);
+
+			if (existing != null) {
+				return existing.toArray(NOLISTENERS);
+			}
+			return NOLISTENERS;
+		} finally {
+			mapPublicViewIDsToListeners_mon.exit();
+		}
+	}
+
 	public boolean isLayoutComplete() {
 		return bLayoutComplete;
 	}
@@ -1483,4 +1812,27 @@ public class SWTSkin
 		System.out.println(d + ";" + then);
 
 	}
+
+	public boolean isCreatingSO() {
+		return currentSkinObjectcreationCount > 0;
+	}
+
+	/**
+	 * 
+	 *
+	 * @since 3.1.1.1
+	 */
+	public void triggerLanguageChange() {
+		Object[] values = mapIDsToSOs.values().toArray();
+		for (int i = 0; i < values.length; i++) {
+			SWTSkinObject[] skinObjects = (SWTSkinObject[]) values[i];
+			if (skinObjects != null) {
+				for (int j = 0; j < skinObjects.length; j++) {
+					SWTSkinObject so = skinObjects[j];
+					so.triggerListeners(SWTSkinObjectListener.EVENT_LANGUAGE_CHANGE);
+				}
+			}
+		}
+	}
+
 }
