@@ -8,7 +8,8 @@ import org.eclipse.swt.widgets.Composite;
 import org.eclipse.swt.widgets.Event;
 import org.eclipse.swt.widgets.Listener;
 
-import org.gudy.azureus2.core3.util.AERunnable;
+import org.gudy.azureus2.core3.internat.MessageText;
+import org.gudy.azureus2.core3.util.*;
 import org.gudy.azureus2.ui.swt.Utils;
 
 /**
@@ -22,10 +23,26 @@ public class SWTSkinButtonUtility
 
 	private final SWTSkinObject skinObject;
 
+	private final String imageViewID;
+
 	public static class ButtonListenerAdapter
 	{
+		public void pressed(SWTSkinButtonUtility buttonUtility,
+				SWTSkinObject skinObject, int stateMask) {
+		}
+
+		/**
+		 * 
+		 * @param buttonUtility
+		 *
+		 * @deprecated
+		 */
 		public void pressed(SWTSkinButtonUtility buttonUtility) {
 		}
+		
+		public boolean held(SWTSkinButtonUtility buttonUtility) {
+			return false;
+		}
 
 		public void disabledStateChanged(SWTSkinButtonUtility buttonUtility,
 				boolean disabled) {
@@ -33,16 +50,53 @@ public class SWTSkinButtonUtility
 	}
 
 	public SWTSkinButtonUtility(SWTSkinObject skinObject) {
+		this(skinObject, null);
+	}
+
+	public SWTSkinButtonUtility(SWTSkinObject skinObject, String imageViewID) {
 		this.skinObject = skinObject;
+		this.imageViewID = imageViewID;
+		
+		if (skinObject instanceof SWTSkinObjectButton) {
+			return;
+		}
+		
 		Listener l = new Listener() {
 			boolean bDownPressed;
+			private TimerEvent timerEvent;
 
 			public void handleEvent(Event event) {
 				if (event.type == SWT.MouseDown) {
+					if (timerEvent == null) {
+						timerEvent = SimpleTimer.addEvent("MouseHold",
+								SystemTime.getOffsetTime(1000), new TimerEventPerformer() {
+									public void perform(TimerEvent event) {
+										timerEvent = null;
+
+										if (!bDownPressed) {
+											return;
+										}
+										bDownPressed = false;
+
+										boolean stillPressed = true;
+										for (Iterator iter = listeners.iterator(); iter.hasNext();) {
+											ButtonListenerAdapter l = (ButtonListenerAdapter) iter.next();
+											stillPressed &= !l.held(SWTSkinButtonUtility.this);
+										}
+										bDownPressed = stillPressed;
+									}
+								});
+					}
 					bDownPressed = true;
 					return;
-				} else if (!bDownPressed) {
-					return;
+				} else {
+					if (timerEvent != null) {
+						timerEvent.cancel();
+						timerEvent = null;
+					}
+					if (!bDownPressed) {
+						return;
+					}
 				}
 
 				bDownPressed = false;
@@ -54,6 +108,8 @@ public class SWTSkinButtonUtility
 				for (Iterator iter = listeners.iterator(); iter.hasNext();) {
 					ButtonListenerAdapter l = (ButtonListenerAdapter) iter.next();
 					l.pressed(SWTSkinButtonUtility.this);
+					l.pressed(SWTSkinButtonUtility.this,
+							SWTSkinButtonUtility.this.skinObject, event.stateMask);
 				}
 			}
 		};
@@ -69,23 +125,37 @@ public class SWTSkinButtonUtility
 	}
 
 	public boolean isDisabled() {
-		return skinObject.getSuffix().equals("-disabled");
+		return skinObject.getSuffix().indexOf("-disabled") >= 0;
 	}
 
+	private boolean inSetDisabled = false;
 	public void setDisabled(boolean disabled) {
-		String suffix = disabled ? "-disabled" : "";
-		if (skinObject.getSuffix().equals(suffix)) {
+		if (inSetDisabled) {
 			return;
 		}
-		skinObject.switchSuffix(suffix, 1, true);
-
-		for (Iterator iter = listeners.iterator(); iter.hasNext();) {
-			ButtonListenerAdapter l = (ButtonListenerAdapter) iter.next();
-			l.disabledStateChanged(SWTSkinButtonUtility.this, disabled);
+		inSetDisabled = true;
+		try {
+  		if (disabled == isDisabled()) {
+  			return;
+  		}
+  		String suffix = disabled ? "-disabled" : "";
+  		skinObject.switchSuffix(suffix, 1, false);
+  
+  		for (Iterator iter = listeners.iterator(); iter.hasNext();) {
+  			ButtonListenerAdapter l = (ButtonListenerAdapter) iter.next();
+  			l.disabledStateChanged(SWTSkinButtonUtility.this, disabled);
+  		}
+		} finally {
+			inSetDisabled = false;
 		}
 	}
 
 	public void addSelectionListener(ButtonListenerAdapter listener) {
+		if (skinObject instanceof SWTSkinObjectButton) {
+			((SWTSkinObjectButton)skinObject).addSelectionListener(listener);
+			return;
+		}
+
 		if (listeners.contains(listener)) {
 			return;
 		}
@@ -97,6 +167,10 @@ public class SWTSkinButtonUtility
 	}
 
 	public void setTextID(final String id) {
+		if (skinObject instanceof SWTSkinObjectButton) {
+			((SWTSkinObjectButton)skinObject).setText(MessageText.getString(id));
+			return;
+		}
 		Utils.execSWTThreadLater(0, new AERunnable() {
 			public void runSupport() {
 				if (skinObject instanceof SWTSkinObjectText) {
@@ -115,8 +189,19 @@ public class SWTSkinButtonUtility
 	}
 
 	public void setImage(final String id) {
+		if (skinObject instanceof SWTSkinObjectButton) {
+			// TODO implement
+			return;
+		}
 		Utils.execSWTThread(new AERunnable() {
 			public void runSupport() {
+				if (imageViewID != null) {
+					SWTSkinObject skinImageObject = skinObject.getSkin().getSkinObject(imageViewID, skinObject);
+					if (skinImageObject instanceof SWTSkinObjectImage) {
+						((SWTSkinObjectImage) skinImageObject).setImageByID(id, null);
+						return;
+					}
+				}
 				if (skinObject instanceof SWTSkinObjectImage) {
 					SWTSkinObjectImage skinImageObject = (SWTSkinObjectImage) skinObject;
 					skinImageObject.setImageByID(id, null);
@@ -132,14 +217,18 @@ public class SWTSkinButtonUtility
 	}
 
 	public void setTooltipID(final String id) {
+		if (skinObject instanceof SWTSkinObjectButton) {
+			// TODO implement
+			return;
+		}
 		if (skinObject instanceof SWTSkinObjectImage) {
 			SWTSkinObjectImage skinImageObject = (SWTSkinObjectImage) skinObject;
-			skinImageObject.setTooltipByID(id);
+			skinImageObject.setTooltipID(id);
 		} else if (skinObject instanceof SWTSkinObjectContainer) {
 			SWTSkinObject[] children = ((SWTSkinObjectContainer) skinObject).getChildren();
 			if (children.length > 0 && children[0] instanceof SWTSkinObjectImage) {
 				SWTSkinObjectImage skinImageObject = (SWTSkinObjectImage) children[0];
-				skinImageObject.setTooltipByID(id);
+				skinImageObject.setTooltipID(id);
 			}
 		}
 	}
