@@ -26,26 +26,25 @@ import java.net.URL;
 import org.eclipse.swt.SWT;
 import org.eclipse.swt.SWTError;
 import org.eclipse.swt.browser.Browser;
-import org.eclipse.swt.widgets.Composite;
-import org.eclipse.swt.widgets.Control;
+import org.eclipse.swt.layout.FormLayout;
+import org.eclipse.swt.widgets.*;
 
-import org.gudy.azureus2.core3.util.AERunnable;
-import org.gudy.azureus2.core3.util.Debug;
-import org.gudy.azureus2.core3.util.SystemTime;
+import org.gudy.azureus2.core3.util.*;
+import org.gudy.azureus2.plugins.PluginInterface;
+import org.gudy.azureus2.pluginsimpl.local.PluginInitializer;
 import org.gudy.azureus2.ui.swt.Utils;
 
-import com.aelitis.azureus.core.AzureusCore;
-import com.aelitis.azureus.core.AzureusCoreFactory;
-import com.aelitis.azureus.core.messenger.config.PlatformConfigMessenger;
+import com.aelitis.azureus.core.cnetwork.ContentNetwork;
+import com.aelitis.azureus.core.cnetwork.ContentNetworkManagerFactory;
+import com.aelitis.azureus.ui.selectedcontent.SelectedContentManager;
 import com.aelitis.azureus.ui.swt.browser.BrowserContext;
+import com.aelitis.azureus.ui.swt.browser.BrowserContext.loadingListener;
 import com.aelitis.azureus.ui.swt.browser.listener.*;
 import com.aelitis.azureus.ui.swt.browser.listener.publish.LocalHoster;
 import com.aelitis.azureus.ui.swt.browser.listener.publish.PublishListener;
 import com.aelitis.azureus.ui.swt.utils.PublishUtils;
-import com.aelitis.azureus.util.Constants;
 import com.aelitis.azureus.util.LocalResourceHTTPServer;
-
-import org.gudy.azureus2.plugins.PluginInterface;
+import com.aelitis.azureus.util.UrlFilter;
 
 /**
  * @author TuxPaper
@@ -59,6 +58,8 @@ public class SWTSkinObjectBrowser
 
 	private Browser browser;
 
+	private Composite cParent;
+
 	private Composite cArea;
 
 	private String sStartURL;
@@ -67,6 +68,12 @@ public class SWTSkinObjectBrowser
 
 	private BrowserContext context;
 
+	private String urlToUse;
+
+	private boolean forceVisibleAfterLoad;
+	
+	private static boolean doneTheUglySWTFocusHack = false;
+
 	/**
 	 * @param skin
 	 * @param properties
@@ -79,17 +86,57 @@ public class SWTSkinObjectBrowser
 			String sID, String sConfigID, SWTSkinObject parent) {
 		super(skin, properties, sID, sConfigID, "browser", parent);
 
-		AzureusCore core = AzureusCoreFactory.getSingleton();
+		cParent = parent == null ? skin.getShell()
+				: (Composite) parent.getControl();
 
-		cArea = parent == null ? skin.getShell() : (Composite) parent.getControl();
+		cArea = cParent;
+		cArea = new Canvas(cParent, SWT.NO_BACKGROUND);
+		cArea.setLayout(new FormLayout());
+
+		setControl(cArea);
+
+		if (cParent.isVisible()) {
+			init();
+		} else {
+			addListener(new SWTSkinObjectListener() {
+				public Object eventOccured(SWTSkinObject skinObject, int eventType,
+						Object params) {
+					if (eventType == SWTSkinObjectListener.EVENT_SHOW) {
+						removeListener(this);
+						init();
+					}
+					return null;
+				}
+			});
+		}
+	}
+
+	public void init() {
+		if (browser != null && !browser.isDisposed()) {
+			return;
+		}
 
 		try {
 			browser = new Browser(cArea, Utils.getInitialBrowserStyle(SWT.NONE));
+
+			browser.setLayoutData(Utils.getFilledFormData());
+			browser.getParent().layout(true);
 		} catch (SWTError e) {
 			System.err.println("Browser: " + e.toString());
 			return;
 		}
 
+		//TODO [SWT] : Remove this stupid code as soon as we update SWT
+		if(Constants.isOSX && ! doneTheUglySWTFocusHack) {
+			doneTheUglySWTFocusHack = true;
+			Shell shell = new Shell(browser.getDisplay(),SWT.NONE);
+			shell.setSize(1,1);
+			shell.setLocation(-2, -2);
+			shell.open();
+			shell.close();
+			browser.setFocus();
+		}
+		
 		Control widgetIndicator = null;
 		String sIndicatorWidgetID = properties.getStringValue(sConfigID
 				+ ".indicator");
@@ -104,53 +151,78 @@ public class SWTSkinObjectBrowser
 		if (browserID == null) {
 			browserID = sID;
 		}
-		
-		context = new BrowserContext(browserID, browser, widgetIndicator,
-				properties.getBooleanValue(sConfigID + ".forceVisibleAfterLoad", true));
 
-		context.addMessageListener(new TorrentListener(core));
+		forceVisibleAfterLoad = properties.getBooleanValue(sConfigID + ".forceVisibleAfterLoad", true);
+		context = new BrowserContext(browserID, browser, widgetIndicator, forceVisibleAfterLoad);
+
+		context.addMessageListener(new TorrentListener());
+		context.addMessageListener(new VuzeListener());
 		context.addMessageListener(new DisplayListener(browser));
 		context.addMessageListener(new ConfigListener(browser));
 		context.addMessageListener(new PublishListener(skin.getShell(), this));
 		context.addMessageListener(new LightBoxBrowserRequestListener());
 		context.addMessageListener(new StatusListener());
 		context.addMessageListener(new BrowserRpcBuddyListener());
-		context.addMessageListener(new MetaSearchListener());
+
+		context.addListener(new loadingListener() {
+			public void browserLoadingChanged(boolean loading, String url) {
+				if (loading && browser.isVisible()) {
+					// hack so search results page doesn't clear cur selected
+					if (UrlFilter.getInstance().urlCanRPC(url)) {
+						SelectedContentManager.clearCurrentlySelectedContent();
+					}
+				}
+				
+			}
+		});
 
 		PublishUtils.setupContext(context);
 
-		setControl(browser);
-		String url = properties.getStringValue(sConfigID + ".url", (String) null);
+		String url = urlToUse != null ? urlToUse : sStartURL != null ? sStartURL
+				: properties.getStringValue(sConfigID + ".url", (String) null);
 		if (url != null) {
 			setURL(url);
 		}
-		
 	}
 
 	public Browser getBrowser() {
+		if (browser == null) {
+			init();
+		}
 		return browser;
 	}
 
 	public void setURL(final String url) {
+		urlToUse = url;
+		if (browser == null) {
+			return;
+		}
 		Utils.execSWTThread(new AERunnable() {
 			public void runSupport() {
 				if (url == null) {
 					browser.setText("");
 				} else {
 					String urlToUse = url;
-					if (PlatformConfigMessenger.urlCanRPC(url)
-							&& urlToUse.indexOf("azid=") < 0) {
-						if (urlToUse.indexOf("?") >= 0) {
-							urlToUse += "&" + Constants.URL_SUFFIX;
-						} else {
-							urlToUse += "?" + Constants.URL_SUFFIX;
+					if (UrlFilter.getInstance().urlCanRPC(url)){
+						ContentNetwork contentNetwork = ContentNetworkManagerFactory.getSingleton().getContentNetwork(
+								context.getContentNetworkID());
+						if (contentNetwork != null) {
+							urlToUse = contentNetwork.appendURLSuffix(urlToUse,
+									false, true);
+						}
+					}
+					if (browser != null) {
+						browser.setUrl(urlToUse);
+						if(browser.isVisible()) {
+							browser.setFocus();
 						}
 					}
-					browser.setUrl(urlToUse);
 				}
 				if (sStartURL == null) {
 					sStartURL = url;
-					browser.setData("StartURL", url);
+					if (browser != null) {
+						browser.setData("StartURL", url);
+					}
 				}
 				//System.out.println(SystemTime.getCurrentTime() + "] Set URL: " + url);
 			}
@@ -177,14 +249,22 @@ public class SWTSkinObjectBrowser
 	 * 
 	 */
 	public void layout() {
-		cArea.layout();
+		cParent.layout();
+	}
+	
+	public void dispose() {
+		if (browser != null && !browser.isDisposed()) {
+			browser.setVisible(false);
+			browser.setUrl("about:blank");
+		}
+		super.dispose();
 	}
 
 	// @see com.aelitis.azureus.ui.swt.browser.listener.publish.LocalHoster#hostFile(java.io.File)
 	public URL hostFile(File f) {
 		if (local_publisher == null) {
 			try {
-				PluginInterface pi = AzureusCoreFactory.getSingleton().getPluginManager().getDefaultPluginInterface();
+				PluginInterface pi = PluginInitializer.getDefaultInterface();
 				local_publisher = new LocalResourceHTTPServer(pi, null);
 			} catch (Throwable e) {
 				Debug.out("Failed to create local resource publisher", e);
@@ -210,7 +290,49 @@ public class SWTSkinObjectBrowser
 	public void setStartURL(String url) {
 		sStartURL = url;
 		if (null != browser) {
+			if (urlToUse == null) {
+				browser.setUrl(url);
+			}
 			browser.setData("StartURL", url);
 		}
 	}
+
+	public boolean isPageLoading() {
+		return context == null ? false : context.isPageLoading();
+	}
+
+	// @see com.aelitis.azureus.ui.swt.skin.SWTSkinObjectBasic#setVisible(boolean)
+	public boolean setIsVisible(final boolean visible, boolean walkup) {
+		boolean changed = super.setIsVisible(visible, walkup);
+
+		if (changed) {
+  		// notify browser after we've fully processed visibility 
+  		Utils.execSWTThreadLater(0, new AERunnable() {
+  			public void runSupport() {
+  				if (!isDisposed() && context != null) {
+  					context.sendBrowserMessage("browser", visible ? "shown" : "hidden");
+  				}
+  			}
+  		});
+		}
+		return changed;
+	}
+
+	public void addListener(loadingListener l) {
+		if (context != null) {
+			context.addListener(l);
+		}
+	}
+
+	public void refresh() {
+		Utils.execSWTThread(new AERunnable() {
+			public void runSupport() {
+				if (browser != null && !browser.isDisposed()) {
+					browser.refresh();
+				}
+			}
+		});
+		
+	}
+
 }
