@@ -39,11 +39,11 @@ public class SWTSkinUtils
 
 	private static Listener imageDownListener;
 
-	private static Listener imageOverListener;
+	//private static Listener imageOverListener;
 
 	static {
-		imageOverListener = new SWTSkinImageChanger("-over", SWT.MouseEnter,
-				SWT.MouseExit);
+		//imageOverListener = new SWTSkinImageChanger("-over", SWT.MouseEnter,
+		//		SWT.MouseExit);
 		imageDownListener = new SWTSkinImageChanger("-down", SWT.MouseDown,
 				SWT.MouseUp);
 	}
@@ -97,13 +97,13 @@ public class SWTSkinUtils
 		return tileMode;
 	}
 
-	static void addMouseImageChangeListeners(Widget widget) {
+	static void addMouseImageChangeListeners(Control widget) {
 		if (widget.getData("hasMICL") != null) {
 			return;
 		}
 
-		widget.addListener(SWT.MouseEnter, imageOverListener);
-		widget.addListener(SWT.MouseExit, imageOverListener);
+		//widget.addListener(SWT.MouseEnter, imageOverListener);
+		//widget.addListener(SWT.MouseExit, imageOverListener);
 		//		new MouseEnterExitListener(widget);
 
 		widget.addListener(SWT.MouseDown, imageDownListener);
@@ -236,53 +236,6 @@ public class SWTSkinUtils
 		}
 	}
 
-	public static void fade(final Composite c, final boolean fadeIn) {
-		UIFunctionsSWT uiFunctions = UIFunctionsManagerSWT.getUIFunctionsSWT();
-		final LightBoxShell lbShell = new LightBoxShell(uiFunctions.getMainShell(),
-				false);
-
-		// assumed: c is on shell
-		Rectangle clientArea = c.getClientArea();
-		lbShell.setInsets(0, c.getShell().getClientArea().height
-				- clientArea.height, 0, 0);
-		lbShell.setStyleMask(LightBoxShell.RESIZE_HORIZONTAL);
-		lbShell.setAlphaLevel(fadeIn ? 255 : 0);
-		lbShell.open();
-		AERunnable runnable = new AERunnable() {
-			public void runSupport() {
-				if (c.isDisposed()) {
-					return;
-				}
-
-				int alphaLevel = lbShell.getAlphaLevel();
-				if (fadeIn) {
-					alphaLevel -= 5;
-					if (alphaLevel < 0) {
-						lbShell.close();
-						return;
-					}
-				} else {
-					alphaLevel += 5;
-					if (alphaLevel > 255) {
-						lbShell.close();
-						return;
-					}
-				}
-				lbShell.setAlphaLevel(alphaLevel);
-
-				final AERunnable r = this;
-				SimpleTimer.addEvent("fade", SystemTime.getCurrentTime() + 10,
-						new TimerEventPerformer() {
-							public void perform(TimerEvent event) {
-								c.getDisplay().asyncExec(r);
-							}
-						});
-			}
-		};
-
-		c.getDisplay().asyncExec(runnable);
-	}
-
 	public static void slide(final SWTSkinObject skinObject, final FormData fd,
 			final Point destSize, final Runnable runOnCompletion) {
 		final Control control = skinObject.getControl();
@@ -396,10 +349,32 @@ public class SWTSkinUtils
 
 			} else {
 				bOver = false;
+				System.out.println(System.currentTimeMillis() + ": " + skinObject
+						+ "-- NOOVER");
 				skinObject.switchSuffix("", 2, true);
 			}
 
 		}
 
 	}
+
+
+	public static SWTSkinObjectBrowser findBrowserSO(SWTSkinObject so) {
+		if (so instanceof SWTSkinObjectBrowser) {
+			return (SWTSkinObjectBrowser) so;
+		}
+		if (so instanceof SWTSkinObjectContainer) {
+			SWTSkinObjectContainer soContainer = (SWTSkinObjectContainer) so;
+			SWTSkinObject[] children = soContainer.getChildren();
+			for (int i = 0; i < children.length; i++) {
+				SWTSkinObject child = children[i];
+				SWTSkinObjectBrowser found = findBrowserSO(child);
+				if (found != null) {
+					return found;
+				}
+			}
+		}
+		return null;
+	}
+
 }
