@@ -127,7 +127,7 @@ public class DCAdManager implements PlatformDCAdManager.GetAdvertDataReplyListen
 
 				downloadManagerAddedHook(new DownloadManager[] {
 						dm
-				});
+				}, 0);
 			}//downloadManagerAdded
 
 		}, false); //addListener
@@ -147,7 +147,7 @@ public class DCAdManager implements PlatformDCAdManager.GetAdvertDataReplyListen
 						&& PlatformTorrentUtils.isContentAdEnabled(dm.getTorrent())) {
 					downloadManagerAddedHook(new DownloadManager[] {
 							dm
-					});
+					}, 0);
 				}
 			}
 		});
@@ -182,15 +182,11 @@ public class DCAdManager implements PlatformDCAdManager.GetAdvertDataReplyListen
 
 
 	/**
-	 * Wait file minutes after start-up then slowly go through the list of
-	 * azpd files to update the ones that are out of date. After the check is
-	 * done this thread goes away.
 	 * @param dms - DownloadManager[]
 	 */
 	private void startLazyAzpdFileCheckThread(final DownloadManager[] dms){
 
 		final String THREAD_NAME = "azpdFileExpireThread";
-		final long CALL_WAIT_TIME = 1000 * 5; //wait 5 seconds between check.
 
 		AEThread2 thread = new AEThread2(THREAD_NAME,true){
 
@@ -225,19 +221,12 @@ public class DCAdManager implements PlatformDCAdManager.GetAdvertDataReplyListen
 						if(f==null)
 							continue;
 
-						debug(THREAD_NAME+": checking "+f.getAbsolutePath());
+						//debug(THREAD_NAME+": checking "+f.getAbsolutePath());
 						if( AzpdFileAccess.isAzpdFileExpired(f) ){
 
 							debug(THREAD_NAME+": found expired azpd file "+f+". Will refresh. ");
 							//refresh this file.
-							downloadManagerAddedHook( new DownloadManager[] { dm } );
-
-							//wait a little to spread out this work.
-							try{ Thread.sleep(CALL_WAIT_TIME); }
-							catch(InterruptedException ie){
-								debug(THREAD_NAME+": interrupted before while running.");
-								return;
-							}
+							downloadManagerAddedHook( new DownloadManager[] { dm }, 1000 );
 						}
 
 					}//if - isAdEnabledContent
@@ -248,7 +237,8 @@ public class DCAdManager implements PlatformDCAdManager.GetAdvertDataReplyListen
 				debug(THREAD_NAME+": finished.");
 			}
 		};
-
+		
+		thread.setPriority(Thread.MIN_PRIORITY);
 		thread.start();
 	}
 
@@ -363,6 +353,11 @@ public class DCAdManager implements PlatformDCAdManager.GetAdvertDataReplyListen
 				return;
 			}
 
+			String thirdPartyImpressionUrl = (String) values.get("thirdPartyImp");
+			if( thirdPartyImpressionUrl == null ) {
+				thirdPartyImpressionUrl = "";
+			}
+			
 			//The vuzeId is used to keep impressions unique.
 			String vuzeId = (String) values.get("vuzeId");
 			if( vuzeId==null ){
@@ -414,8 +409,8 @@ public class DCAdManager implements PlatformDCAdManager.GetAdvertDataReplyListen
 				debug("DM for Ad not found");
 			}
 
-			PlatformDCAdManager.saveImpresssion(impressionID,
-					SystemTime.getCurrentTime(), contentHash, torrentHash, adHash,
+			PlatformDCAdManager.saveImpression(impressionID,   // impressionID == doubleclick impression url
+					SystemTime.getCurrentTime(), contentHash, torrentHash, adHash, thirdPartyImpressionUrl,
 					5000);
 
 		} catch (Exception e) {
@@ -430,8 +425,8 @@ public class DCAdManager implements PlatformDCAdManager.GetAdvertDataReplyListen
 	 * 
 	 * @param dms -
 	 */
-	private void downloadManagerAddedHook(final DownloadManager[] dms) {
-
+	private void downloadManagerAddedHook(final DownloadManager[] dms,
+			final int delay) {
 		AEThread2 thread = new AEThread2("downloadManagerAddedHook", true) {
 			public void run() {
 				debug("enter - downloadManagerAddedHook");
@@ -452,7 +447,7 @@ public class DCAdManager implements PlatformDCAdManager.GetAdvertDataReplyListen
 						return;
 					} else {
 						//Add GetAdvert here.
-						callGetAdvert(adSupportedContentList);
+						callGetAdvert(adSupportedContentList, delay);
 					}
 
 				} finally {
@@ -466,7 +461,7 @@ public class DCAdManager implements PlatformDCAdManager.GetAdvertDataReplyListen
 		thread.run();
 	}//downloadManagerAddedHook
 
-	private void callGetAdvert(List adSupportedContentList) {
+	private void callGetAdvert(List adSupportedContentList, int delay) {
 		try {
 			increaseCheckingForAds();
 			debug("sending ad request for " + adSupportedContentList.size()
@@ -479,7 +474,7 @@ public class DCAdManager implements PlatformDCAdManager.GetAdvertDataReplyListen
 
 				// each reply from getAdvert will in turn decreaseCheckingForAds()
 				increaseCheckingForAds();
-				PlatformDCAdManager.getAdvert(dm, 2000, this);
+				PlatformDCAdManager.getAdvert(dm, delay, this);
 			}
 
 		} catch (Exception e) {
@@ -648,12 +643,29 @@ public class DCAdManager implements PlatformDCAdManager.GetAdvertDataReplyListen
 		if( !isNullAd ){
 			//Find location of the ad(s).
 			try {
-  			File adFile = getAdMediaFromContentDownloadManager(dmContent);
-  			debug("  adPath: "+adFile.getAbsolutePath());
-  			replace(repBuffer,"<##-AD-PATH-##>",adFile.getAbsolutePath());
+  			List<File> ads = getAdMediaFromContentDownloadManager(dmContent);
+  			if (ads.size() > 0) {
+  				File adFile = ads.get(0);
+    			debug("  adPath: "+adFile.getAbsolutePath());
+    			replace(repBuffer,"<##-AD-PATH-##>",adFile.getAbsolutePath());
+    			
+    			for (int i = 0; i < ads.size(); i++) {
+						adFile = ads.get(i);
+						if (adFile != null) {
+							replace(repBuffer, "<##-AD-PATH-" + i + "-##>",
+									adFile.getAbsolutePath());
+						}
+					}
+  			}
 			} catch (Exception e) {
 				debug("reokacASXParams: " + e.toString());
+	  			replace(repBuffer,"<##-AD-PATH-##>","http://www.vuze.com/img43/asx_noad.gif");
 			}
+
+			String newRepBuffer = repBuffer.toString().replaceAll(
+					"<##-AD-PATH-[0-9]+-##>", "http://www.vuze.com/img43/asx_noad.gif");
+			repBuffer.delete(0, repBuffer.length());
+			repBuffer.append(newRepBuffer);
 		}
 
 		//pass the params to the player via the download manager.
@@ -721,36 +733,39 @@ public class DCAdManager implements PlatformDCAdManager.GetAdvertDataReplyListen
 	private static void replace(StringBuffer sb, String param, String value, int startIndex){
 
 		int s = sb.indexOf(param,startIndex);
-		sb.replace(s, s+param.length(), value );
+		if (s >= 0) {
+			sb.replace(s, s+param.length(), value );
+		}
 
 	}//replace
 
-	private File getAdMediaFromContentDownloadManager(DownloadManager contentDM){
+	private List<File> getAdMediaFromContentDownloadManager(DownloadManager contentDM){
 
 		debug("getAdMediaFromContentDownloadManager");
 
 		GlobalManager gm = contentDM.getGlobalManager();
 		Map map = AzpdFileAccess.getPlayerDataMap(contentDM);
 		List adHashList = (List) map.get("ad_hash");
-		String adHash = (String) adHashList.get(0);
 
-		if(adHash==null || adHash.equals("") ){
-			throw new IllegalStateException("No adHash found map="+map);
+		List<File> ads = new ArrayList<File>();
+		
+		for (Iterator iter = adHashList.iterator(); iter.hasNext();) {
+			String adHash = (String) iter.next();
+			if (adHash == null || adHash.equals("")) {
+				continue;
+			}
+
+			HashWrapper adHashWrapper = new HashWrapper(Base32.decode(adHash));
+			DownloadManager dmAd = gm.getDownloadManager(adHashWrapper);
+
+			File adFile;
+			if (dmAd != null) {
+				adFile = dmAd.getDiskManagerFileInfo()[0].getFile(true);
+				ads.add(adFile);
+			}
 		}
 
-		HashWrapper adHashWrapper = new HashWrapper(Base32.decode(adHash));
-		DownloadManager dmAd = gm.getDownloadManager(adHashWrapper); //ToDo: ad a check here for a null result.
-
-		File adFile;
-		if( !(dmAd==null) ){
-			adFile = dmAd.getDiskManagerFileInfo()[0].getFile(true);
-		}else{
-			//what is an alternate method to find the adFile?
-			debug("###  TODO  ### - Need to find an alternate method to get the ad media file location. ");
-			throw new NullPointerException("Need to get an ad to play.");
-		}
-
-		return adFile;
+		return ads;
 	}
 
 	/**
@@ -969,7 +984,9 @@ public class DCAdManager implements PlatformDCAdManager.GetAdvertDataReplyListen
 						DownloadManager dm = (DownloadManager)adsDMList.get(i);
 				
 						// Make sure it's started
-						if (dm.getAssumedComplete() && dm.filesExist()) {
+						// Note: We also used to check dm.filesExist(true)
+						//       but that's too time consuming at startup..
+						if (dm.getAssumedComplete()) { // && dm.filesExist(true)) {
 							dm.setForceStart(false);
 						} else {
 							if (dm.getState() == DownloadManager.STATE_ERROR) {
@@ -999,13 +1016,24 @@ public class DCAdManager implements PlatformDCAdManager.GetAdvertDataReplyListen
 		return adSupportedContentList;
 	}//initDownloadManagerList
 
+
+	/**
+	 * Call when you want to refresh via getAdvert call (non-immediate)
+	 * @param dm - DownloadManager
+	 */
+	public void refreshAd(DownloadManager dm){
+		refreshAd(dm, false);
+	}
+
 	/**
 	 * Call when you want to refresh via getAdvert call.
 	 * @param dm - DownloadManager
 	 */
-	public void refreshAd(DownloadManager dm){
+	public void refreshAd(DownloadManager dm, boolean immediate){
 
-		downloadManagerAddedHook( new DownloadManager[] { dm }  );
+		downloadManagerAddedHook(new DownloadManager[] {
+			dm
+		}, immediate ? 0 : 1000);
 
 	}//refreshAd
 
