@@ -22,15 +22,27 @@
 
 package com.aelitis.azureus.util;
 
+import java.net.URL;
+
 import com.aelitis.azureus.core.AzureusCore;
+import com.aelitis.azureus.core.cnetwork.ContentNetworkManagerFactory;
 import com.aelitis.azureus.core.content.AzureusPlatformContentDirectory;
+import com.aelitis.azureus.core.content.RelatedContent;
+import com.aelitis.azureus.core.content.RelatedContentManager;
+import com.aelitis.azureus.core.content.RelatedContentManagerListener;
+import com.aelitis.azureus.core.devices.DeviceManagerFactory;
 import com.aelitis.azureus.core.download.DownloadManagerEnhancer;
 import com.aelitis.azureus.core.metasearch.MetaSearchManagerFactory;
 import com.aelitis.azureus.core.peer.cache.CacheDiscovery;
+import com.aelitis.azureus.core.subs.Subscription;
+import com.aelitis.azureus.core.subs.SubscriptionManagerFactory;
 import com.aelitis.azureus.core.torrent.PlatformTorrentUtils;
+import com.aelitis.azureus.core.util.AZ3Functions;
+import com.aelitis.azureus.ui.swt.views.skin.TorrentListViewsUtils;
 
 import org.gudy.azureus2.core3.download.DownloadManagerState;
 import org.gudy.azureus2.core3.download.DownloadManagerStateAttributeListener;
+import org.gudy.azureus2.core3.util.Debug;
 import org.gudy.azureus2.plugins.PluginInterface;
 import org.gudy.azureus2.plugins.download.Download;
 import org.gudy.azureus2.plugins.download.DownloadManager;
@@ -38,6 +50,7 @@ import org.gudy.azureus2.plugins.download.DownloadManagerListener;
 import org.gudy.azureus2.plugins.download.DownloadWillBeAddedListener;
 import org.gudy.azureus2.plugins.torrent.Torrent;
 import org.gudy.azureus2.pluginsimpl.local.PluginCoreUtils;
+import org.gudy.azureus2.pluginsimpl.local.PluginInitializer;
 
 public class InitialisationFunctions
 {
@@ -45,19 +58,78 @@ public class InitialisationFunctions
 
 	public static void earlyInitialisation(AzureusCore core) {
 		
-		DownloadUtils.initialise( core );
+		DownloadUtils.initialise();
 		
 		DownloadManagerEnhancer dme = DownloadManagerEnhancer.initialise(core);
 
-		registerTrackerURLExtensions(core);
+		registerTrackerURLExtensions();
 
 		AzureusPlatformContentDirectory.register();
 
 		CacheDiscovery.initialise( dme );
 		
+		ContentNetworkManagerFactory.preInitialise();
+		
 		MetaSearchManagerFactory.preInitialise();
 		
+		SubscriptionManagerFactory.preInitialise();
+		
+		DeviceManagerFactory.preInitialise();
+		
 		NavigationHelper.initialise();
+		
+		RelatedContentManager.preInitialise( core );
+		
+		AZ3Functions.setProvider(
+			new AZ3Functions.provider()
+			{
+				public void 
+				subscribeToRSS(
+					String		name,
+					URL 		url,
+					int			interval,
+					boolean		is_public,
+					String		creator_ref )
+				
+					throws Exception
+				{
+					Subscription subs =
+						SubscriptionManagerFactory.getSingleton().createSingletonRSS(
+						name, url, interval );
+					
+					if ( subs.isPublic() != is_public ){
+						
+						subs.setPublic( is_public );
+					}
+					
+					if ( creator_ref != null ){
+						
+						subs.setCreatorRef( creator_ref );
+					}
+				}
+				
+				public boolean 
+				canShowCDP(
+					org.gudy.azureus2.core3.download.DownloadManager dm ) 
+				{
+					return( TorrentListViewsUtils.canViewDetails( dm ));
+				}
+				
+				public void 
+				showCDP(
+					org.gudy.azureus2.core3.download.DownloadManager	dm,
+					String 												ref )
+				{
+					TorrentListViewsUtils.viewDetails( dm, ref );
+				}
+				
+				public String 
+				getCDPURL(
+					org.gudy.azureus2.core3.download.DownloadManager 	dm ) 
+				{
+					return( TorrentListViewsUtils.getDetailsURL( dm ));
+				}
+			});
 	}
 
 	public static void 
@@ -66,22 +138,31 @@ public class InitialisationFunctions
 	{
 		ExternalStimulusHandler.initialise(core);
 		
-		core.getPluginManager().getDefaultPluginInterface().getUtilities().createDelayedTask(
+		PluginInitializer.getDefaultInterface().getUtilities().createDelayedTask(
 			new Runnable()
 			{
 				public void 
 				run() 
 				{
 					MetaSearchManagerFactory.getSingleton();
+					
+					SubscriptionManagerFactory.getSingleton();
+					
+					try{
+						RelatedContentManager.getSingleton();
+						
+					}catch( Throwable e ){
+						
+						Debug.out( e );
+					}
 				}
 			}).queue();
 	}
 
 	protected static void 
-	registerTrackerURLExtensions(
-		AzureusCore core ) 
+	registerTrackerURLExtensions() 
 	{
-		PluginInterface pi = core.getPluginManager().getDefaultPluginInterface();
+		PluginInterface pi = PluginInitializer.getDefaultInterface();
 
 		DownloadManager	dm = pi.getDownloadManager();
 		
@@ -125,7 +206,7 @@ public class InitialisationFunctions
 					Torrent t = download.getTorrent();
 					if (t == null) {return;}
 					if (!PlatformTorrentUtils.isContent(t, true)) {return;}
-					DownloadUtils.addTrackerExtension(download, EXTENSION_PREFIX, Constants.AZID);	
+					DownloadUtils.addTrackerExtension(download, EXTENSION_PREFIX, ConstantsVuze.AZID);	
 					
 					// allow the tracker to manipulate peer sources for dead/unauthorised torrents
 					download.setFlag(Download.FLAG_ALLOW_PERMITTED_PEER_SOURCE_CHANGES, true);
