@@ -43,6 +43,12 @@ abstract class SyrahPosition implements DHTNetworkPosition {
 		return( (!Float.isNaN( getError())) && getCoords().isValid());
 	}
 	
+	public double[] getLocation() {
+		Coordinate coords = getStableCoords();
+		
+		return( coords.getCoordinates());
+	}
+	
   public String toString () {
     return new String (getCoords()+",er="+getError()+",age="+getAge());
   }
