@@ -26,26 +26,21 @@ package com.aelitis.net.udp.uc.impl;
  *
  */
 
-import java.util.*;
 import java.io.*;
 import java.net.*;
+import java.nio.channels.UnsupportedAddressTypeException;
+import java.util.*;
 
-import org.gudy.azureus2.core3.logging.*;
+import org.bouncycastle.util.encoders.Base64;
+import org.gudy.azureus2.core3.logging.LogAlert;
+import org.gudy.azureus2.core3.logging.LogEvent;
+import org.gudy.azureus2.core3.logging.LogIDs;
+import org.gudy.azureus2.core3.logging.Logger;
 import org.gudy.azureus2.core3.util.*;
 
 import com.aelitis.azureus.core.networkmanager.admin.NetworkAdmin;
 import com.aelitis.azureus.core.networkmanager.admin.NetworkAdminPropertyChangeListener;
-import com.aelitis.net.udp.uc.PRUDPPacket;
-import com.aelitis.net.udp.uc.PRUDPPacketHandler;
-import com.aelitis.net.udp.uc.PRUDPPacketHandlerException;
-import com.aelitis.net.udp.uc.PRUDPPacketHandlerStats;
-import com.aelitis.net.udp.uc.PRUDPPacketReceiver;
-import com.aelitis.net.udp.uc.PRUDPPacketReply;
-import com.aelitis.net.udp.uc.PRUDPPacketRequest;
-import com.aelitis.net.udp.uc.PRUDPPrimordialHandler;
-import com.aelitis.net.udp.uc.PRUDPRequestHandler;
-
-import org.bouncycastle.util.encoders.Base64;
+import com.aelitis.net.udp.uc.*;
 
 public class 
 PRUDPPacketHandlerImpl
@@ -68,10 +63,10 @@ PRUDPPacketHandlerImpl
 	
 	
 	private Map			requests = new LightHashMap();
-	private AEMonitor	requests_mon	= new AEMonitor( "PRUDPPH:req" );
+	private AEMonitor2	requests_mon	= new AEMonitor2( "PRUDPPH:req" );
 	
 	
-	private AEMonitor	send_queue_mon	= new AEMonitor( "PRUDPPH:sd" );
+	private AEMonitor2	send_queue_mon	= new AEMonitor2( "PRUDPPH:sd" );
 	private long		send_queue_data_size;
 	private List[]		send_queues		= new List[]{ new LinkedList(),new LinkedList(),new LinkedList()};
 	private AESemaphore	send_queue_sem	= new AESemaphore( "PRUDPPH:sq" );
@@ -106,6 +101,9 @@ PRUDPPacketHandlerImpl
 
 	private Throwable 	init_error;
 	
+	private PRUDPPacketHandlerImpl altProtocolDelegate; 
+	
+	
 	protected
 	PRUDPPacketHandlerImpl(
 		int				_port,
@@ -120,18 +118,15 @@ PRUDPPacketHandlerImpl
 		
 		final AESemaphore init_sem = new AESemaphore("PRUDPPacketHandler:init");
 		
-		Thread t = new AEThread( "PRUDPPacketReciever:".concat(String.valueOf(port)))
+		new AEThread2( "PRUDPPacketReciever:" + port, true )
 			{
 				public void
-				runSupport()
+				run()
 				{
 					receiveLoop(init_sem);
 				}
-			};
+			}.start();
 		
-		t.setDaemon(true);
-		
-		t.start();
 		
 		final TimerEventPeriodic[]	f_ev = {null};
 		
@@ -168,6 +163,10 @@ PRUDPPacketHandlerImpl
 		}
 		
 		primordial_handler	= handler;
+		
+		PRUDPPacketHandlerImpl delegate = altProtocolDelegate;
+		if(delegate != null)
+			delegate.setPrimordialHandler(handler);
 	}
 	
 	public void
@@ -187,6 +186,10 @@ PRUDPPacketHandlerImpl
 		}
 		
 		request_handler	= _request_handler;
+		
+		PRUDPPacketHandlerImpl delegate = altProtocolDelegate;
+		if(delegate != null)
+			delegate.setRequestHandler(_request_handler);
 	}
 	
 	public PRUDPRequestHandler
@@ -262,10 +265,43 @@ PRUDPPacketHandlerImpl
 	{
 		if ( explicit_bind_ip != null ){
 			
+			if(altProtocolDelegate != null)
+			{
+				altProtocolDelegate.destroy();
+				altProtocolDelegate = null;
+			}
+			
 			target_bind_ip = explicit_bind_ip;
 			
 		}else{
 			
+			InetAddress altAddress = null;
+			NetworkAdmin adm = NetworkAdmin.getSingleton();
+			try
+			{
+				if (default_bind_ip instanceof Inet6Address && !default_bind_ip.isAnyLocalAddress() && adm.hasIPV4Potential())
+					altAddress = adm.getSingleHomedServiceBindAddress(NetworkAdmin.IP_PROTOCOL_VERSION_REQUIRE_V4);
+				else if (default_bind_ip instanceof Inet4Address && adm.hasIPV6Potential())
+					altAddress = adm.getSingleHomedServiceBindAddress(NetworkAdmin.IP_PROTOCOL_VERSION_REQUIRE_V6);
+			} catch (UnsupportedAddressTypeException e)
+			{
+			}
+			
+			if(altProtocolDelegate != null && !altProtocolDelegate.explicit_bind_ip.equals(altAddress))
+			{
+				altProtocolDelegate.destroy();
+				altProtocolDelegate = null;
+			}
+			
+			if(altAddress != null && altProtocolDelegate == null)
+			{
+				altProtocolDelegate = new PRUDPPacketHandlerImpl(port,altAddress);
+				altProtocolDelegate.stats = stats;
+				altProtocolDelegate.primordial_handler = primordial_handler;
+				altProtocolDelegate.request_handler = request_handler;
+			}
+				
+			
 			target_bind_ip = default_bind_ip;
 		}
 	}
@@ -312,19 +348,60 @@ PRUDPPacketHandlerImpl
 				
 				DatagramSocket	new_socket;
 				
-				if ( target_bind_ip == null ){
-					
-					address = new InetSocketAddress("127.0.0.1",port);
-					
-					new_socket = new DatagramSocket( port );
-					
-				}else{
-					
-					address = new InetSocketAddress( target_bind_ip, port );
-					
-					new_socket = new DatagramSocket( address );		
-				}
+				try{
+					if ( target_bind_ip == null ){
 						
+						address = new InetSocketAddress("127.0.0.1",port);
+						
+						new_socket = new DatagramSocket( port );
+						
+					}else{
+						
+						address = new InetSocketAddress( target_bind_ip, port );
+						
+						new_socket = new DatagramSocket( address );		
+					}
+				}catch( BindException e ){
+					
+						// one off attempt to recover by selecting an explicit one.
+						// on  Vista (at least) we sometimes fail with wildcard but succeeed
+						// with explicit (see http://forum.vuze.com/thread.jspa?threadID=77574&tstart=0)
+					
+					if ( target_bind_ip.isAnyLocalAddress()){
+						
+						InetAddress guess = NetworkAdmin.getSingleton().guessRoutableBindAddress();
+						
+						if ( guess != null ){
+							
+							if (Logger.isEnabled())
+								Logger.log(new LogEvent(LOGID,"PRUDPPacketReceiver: retrying with bind IP guess of " + guess ));
+
+							try{
+								
+								InetSocketAddress guess_address = new InetSocketAddress( guess, port );
+								
+								new_socket = new DatagramSocket( guess_address );		
+
+								target_bind_ip 	= guess;
+								address			= guess_address;
+								
+								if (Logger.isEnabled())
+									Logger.log(new LogEvent(LOGID,"PRUDPPacketReceiver: Switched to explicit bind ip " + target_bind_ip + " after initial bind failure with wildcard (" + e.getMessage() + ")" ));
+
+							}catch( Throwable f ){
+								
+								throw( e );
+							}
+						}else{
+							
+							throw( e );
+						}
+					}else{
+						
+						throw( e );
+					}
+				}
+				
 				new_socket.setReuseAddress(true);
 				
 					// short timeout on receive so that we can interrupt a receive fairly quickly
@@ -457,9 +534,12 @@ PRUDPPacketHandlerImpl
 			
 			init_error	= e;
 			
-			Logger.logTextResource(new LogAlert(LogAlert.UNREPEATABLE,
-					LogAlert.AT_ERROR, "Tracker.alert.listenfail"), new String[] { "UDP:"
-					+ port });
+			if (!( e instanceof BindException && Constants.isWindowsVistaOrHigher )){
+				
+				Logger.logTextResource(new LogAlert(LogAlert.UNREPEATABLE,
+						LogAlert.AT_ERROR, "Tracker.alert.listenfail"), new String[] { "UDP:"
+						+ port });
+			}
 			
 			Logger.log(new LogEvent(LOGID, "PRUDPPacketReceiver: "
 					+ "DatagramSocket bind failed on port " + port, e));
@@ -480,6 +560,12 @@ PRUDPPacketHandlerImpl
 					Debug.printStackTrace(e);
 				}
 			}
+
+			// make sure we destroy the delegate too if something happend
+			PRUDPPacketHandlerImpl delegate = altProtocolDelegate;
+			if(delegate != null)
+				delegate.destroy();
+
 			
 			NetworkAdmin.getSingleton().removePropertyChangeListener( prop_listener );
 		}
@@ -562,12 +648,14 @@ PRUDPPacketHandlerImpl
 						
 			stats.packetReceived(packet_len);
 			
+			InetSocketAddress originator = (InetSocketAddress)dg_packet.getSocketAddress();
+			
 			if ( ( packet_data[0]&0x80 ) == 0 ){
 				
 				request_packet	= false;
 				
 				packet = PRUDPPacketReply.deserialiseReply( 
-					this,
+					this, originator, 
 					new DataInputStream(new ByteArrayInputStream( packet_data, 0, packet_len)));
 				
 			}else{
@@ -582,7 +670,7 @@ PRUDPPacketHandlerImpl
 			
 			packet.setSerialisedSize( packet_len );
 			
-			packet.setAddress( (InetSocketAddress)dg_packet.getSocketAddress());
+			packet.setAddress( originator );
 			
 			if ( request_packet ){
 					
@@ -853,6 +941,11 @@ PRUDPPacketHandlerImpl
 			throw( new PRUDPPacketHandlerException( "Transport unavailable" ));
 		}
 		
+		PRUDPPacketHandlerImpl delegate = altProtocolDelegate;
+		if(delegate != null && destination_address.getAddress().getClass().isInstance(delegate.explicit_bind_ip))
+			return delegate.sendAndReceive(auth, request_packet, destination_address, receiver, timeout, priority);		
+
+		
 		try{
 			checkTargetAddress( destination_address );
 			
@@ -1086,7 +1179,7 @@ PRUDPPacketHandlerImpl
 				}else{
 					
 					request.sent();
-					
+					if (dg_packet == null) {throw new NullPointerException("dg_packet is null");}
 					socket.send( dg_packet );
 					
 					// System.out.println( "sent:" + buffer.length );
@@ -1126,9 +1219,24 @@ PRUDPPacketHandlerImpl
 			
 		}catch( Throwable e ){
 			
-			Logger.log(new LogEvent(LOGID,LogEvent.LT_ERROR,
-					"PRUDPPacketHandler: sendAndReceive to " + destination_address + " failed: " + Debug.getNestedExceptionMessage(e))); 
+			// AMC: I've seen this in debug logs - just wonder where it's
+			// coming from.
+			if (e instanceof NullPointerException) {
+				Debug.out(e);
+			}
 			
+			String msg = Debug.getNestedExceptionMessage(e);
+			
+			Logger.log(new LogEvent(LOGID,LogEvent.LT_ERROR,
+					"PRUDPPacketHandler: sendAndReceive to " + destination_address + " failed: " + msg )); 
+			
+			if ( msg.indexOf( "Invalid data length" ) != -1 ){
+				
+				Debug.out( "packet=" + request_packet.getString() + ",auth=" + auth );
+				
+				Debug.out( e );
+			}
+
 			throw( new PRUDPPacketHandlerException( "PRUDPPacketHandler:sendAndReceive failed", e ));
 		}
 	}
@@ -1150,6 +1258,14 @@ PRUDPPacketHandlerImpl
 			throw( new PRUDPPacketHandlerException( "Transport unavailable" ));
 		}
 		
+		PRUDPPacketHandlerImpl delegate = altProtocolDelegate;
+		if(delegate != null && destination_address.getAddress().getClass().isInstance(delegate.explicit_bind_ip))
+		{
+			delegate.send(request_packet, destination_address);
+			return;
+		}
+
+		
 		try{
 			checkTargetAddress( destination_address );
 			
@@ -1203,6 +1319,11 @@ PRUDPPacketHandlerImpl
 			
 			throw( new PRUDPPacketHandlerException( "Invalid port - 0" ));
 		}
+		
+		if ( address.getAddress() == null ){
+			
+			throw( new PRUDPPacketHandlerException( "Unresolved host '" + address.getHostName() + "'" ));
+		}
 	}
 	
 	public void
@@ -1222,6 +1343,12 @@ PRUDPPacketHandlerImpl
 			
 			queued_request_timeout = 5000;
 		}
+		
+		PRUDPPacketHandlerImpl delegate = altProtocolDelegate;
+		if(delegate != null)
+			delegate.setDelays(_send_delay, _receive_delay, _queued_request_timeout);
+		
+		
 	}
 	
 	public long
@@ -1231,13 +1358,22 @@ PRUDPPacketHandlerImpl
 		for (int i=0;i<send_queues.length;i++){
 			res += send_queues[i].size();
 		}
+
+		PRUDPPacketHandlerImpl delegate = altProtocolDelegate;
+		if(delegate != null)
+			res += delegate.getSendQueueLength();
+
 		return(res);
 	}
 	
 	public long
 	getReceiveQueueLength()
 	{
-		return( recv_queue.size());
+		long size = recv_queue.size(); 
+		PRUDPPacketHandlerImpl delegate = altProtocolDelegate;
+		if(delegate != null)
+			size += delegate.getReceiveQueueLength();
+		return size;
 	}
 	
 	public void
@@ -1257,6 +1393,14 @@ PRUDPPacketHandlerImpl
 			throw( new PRUDPPacketHandlerException( "Transport unavailable" ));
 		}
 		
+		PRUDPPacketHandlerImpl delegate = altProtocolDelegate;
+		if(delegate != null && target.getAddress().getClass().isInstance(delegate.explicit_bind_ip))
+		{
+			delegate.primordialSend(buffer, target);
+			return;
+		}
+
+		
 		try{
 			checkTargetAddress( target );
 			
@@ -1290,6 +1434,10 @@ PRUDPPacketHandlerImpl
 	{
 		destroyed	= true;
 		
+		PRUDPPacketHandlerImpl delegate = altProtocolDelegate;
+		if(delegate != null)
+			delegate.destroy();
+
 		destroy_sem.reserve();
 	}
 }
