@@ -36,7 +36,9 @@ import com.aelitis.azureus.core.util.Java15Utils;
 import org.gudy.azureus2.core3.util.AEMonitor;
 import org.gudy.azureus2.core3.util.AERunnable;
 import org.gudy.azureus2.core3.util.Debug;
+import org.gudy.azureus2.core3.util.FileUtil;
 import org.gudy.azureus2.core3.util.ThreadPool;
+import org.gudy.azureus2.core3.util.TorrentUtils;
 
 import org.gudy.azureus2.plugins.utils.resourcedownloader.ResourceDownloader;
 import org.gudy.azureus2.plugins.utils.resourcedownloader.ResourceDownloaderAdapter;
@@ -85,7 +87,7 @@ UPnPImpl
 	private UPnPAdapter				adapter;
 	private SSDPIGD					ssdp;
 	
-	private Map			root_locations	= new HashMap();
+	private Map<String,UPnPRootDeviceImpl>			root_locations	= new HashMap<String, UPnPRootDeviceImpl>();
 	
 	private List		log_listeners		= new ArrayList();
 	private List		log_history			= new ArrayList();
@@ -126,7 +128,6 @@ UPnPImpl
 		final InetAddress			local_address,
 		final String				usn,
 		final URL					location )
-
 	{
 		
 			// we need to take this operation off the main thread as it can take some time. This is a single
@@ -335,6 +336,19 @@ UPnPImpl
 		reset();
 	}
 	
+	public void
+	search()
+	{
+		ssdp.searchNow();
+	}
+	
+	public void
+	search(
+		String[]	STs )
+	{
+		ssdp.searchNow( STs );
+	}
+	
 	public void
 	reset()
 	{
@@ -480,11 +494,12 @@ UPnPImpl
 	
 	public SimpleXMLParserDocument
 	downloadXML(
-		URL				url )
+		UPnPRootDeviceImpl	root,
+		URL					url )
 	
 		throws UPnPException
 	{
-		return( downloadXML( null, url ));
+		return( downloadXMLSupport( null, url ));
 	}
 	
 	public SimpleXMLParserDocument
@@ -502,7 +517,7 @@ UPnPImpl
 				device.restoreRelativeBaseURL();
 			}
 			
-			return( downloadXMLSupport( device, url ));
+			return( downloadXMLSupport( device.getFriendlyName(), url ));
 			
 		}catch( UPnPException e ){
 			
@@ -510,7 +525,7 @@ UPnPImpl
 				
 				device.clearRelativeBaseURL();
 				
-				return( downloadXMLSupport( device, url ));
+				return( downloadXMLSupport( device.getFriendlyName(), url ));
 			}
 			
 			throw( e );
@@ -519,12 +534,14 @@ UPnPImpl
 	
 	protected SimpleXMLParserDocument
 	downloadXMLSupport(
-		UPnPDeviceImpl	device,
+		String			friendly_name,
 		URL				url )
 	
 		throws UPnPException
 	{
 		try{
+			TorrentUtils.setTLSDescription( "UPnP Device" + ( friendly_name==null?"":( ": " + friendly_name )));
+			
 			ResourceDownloaderFactory rdf = adapter.getResourceDownloaderFactory();
 				
 			ResourceDownloader rd = rdf.getRetryDownloader( rdf.create( url, true ), 3 );
@@ -550,7 +567,11 @@ UPnPImpl
 			}
 			
 			throw( new UPnPException( "Root device location '" + url + "' - data read failed", e ));
-		}	
+			
+		}finally{
+			
+			TorrentUtils.setTLSDescription( null );
+		}
 	}
 	
 	protected boolean
@@ -623,67 +644,105 @@ UPnPImpl
 		
 		adapter.trace( "UPnP:Request: -> " + control + "," + request );
 
-		if (use_http_connection) {
-			HttpURLConnection	con = (HttpURLConnection)Java15Utils.openConnectionForceNoProxy(control);
-				
-			con.setRequestProperty( "SOAPAction", "\""+ soap_action + "\"");
-				
-			con.setRequestProperty( "Content-Type", "text/xml; charset=\"utf-8\"" );
-				
-			con.setRequestProperty( "User-Agent", "Azureus (UPnP/1.0)" );
-				
-			con.setRequestMethod( "POST" );
-				
-			con.setDoInput( true );
-			con.setDoOutput( true );
-				
-			OutputStream	os = con.getOutputStream();
-				
-			PrintWriter	pw = new PrintWriter( new OutputStreamWriter(os, "UTF-8" ));
-							
-			pw.println( request );
-				
-			pw.flush();
-		
-			con.connect();
-				
-			if ( con.getResponseCode() == 405 || con.getResponseCode() == 500 ){
+		if ( use_http_connection ){
+			
+			try{
+				TorrentUtils.setTLSDescription( "UPnP Device: " + service.getDevice().getFriendlyName());
+			
+				HttpURLConnection	con1 = (HttpURLConnection)Java15Utils.openConnectionForceNoProxy(control);
 					
-				// gotta retry with M-POST method
-									
-				con = (HttpURLConnection)control.openConnection();
+				con1.setRequestProperty( "SOAPAction", "\""+ soap_action + "\"");
 					
-				con.setRequestProperty( "Content-Type", "text/xml; charset=\"utf-8\"" );
+				con1.setRequestProperty( "Content-Type", "text/xml; charset=\"utf-8\"" );
 					
-				con.setRequestMethod( "M-POST" );
+				con1.setRequestProperty( "User-Agent", "Azureus (UPnP/1.0)" );
 					
-				con.setRequestProperty( "MAN", "\"http://schemas.xmlsoap.org/soap/envelope/\"; ns=01" );
-		
-				con.setRequestProperty( "01-SOAPACTION", "\""+ soap_action + "\"");
+				con1.setRequestMethod( "POST" );
 					
-				con.setDoInput( true );
-				con.setDoOutput( true );
+				con1.setDoInput( true );
+				con1.setDoOutput( true );
 					
-				os = con.getOutputStream();
+				OutputStream	os = con1.getOutputStream();
 					
-				pw = new PrintWriter( new OutputStreamWriter(os, "UTF-8" ));
+				PrintWriter	pw = new PrintWriter( new OutputStreamWriter(os, "UTF-8" ));
 								
 				pw.println( request );
 					
 				pw.flush();
-		
-				con.connect();
-				
-				return( parseXML(con.getInputStream()));	
-					
-			}else{
-					
-				return( parseXML(con.getInputStream()));
-			}
-		}
-		else {
-			Socket	socket = new Socket(control.getHost(), control.getPort());
 			
+				con1.connect();
+					
+				if ( con1.getResponseCode() == 405 || con1.getResponseCode() == 500 ){
+						
+						// gotta retry with M-POST method
+						
+					try{
+						HttpURLConnection con2 = (HttpURLConnection)control.openConnection();
+							
+						con2.setRequestProperty( "Content-Type", "text/xml; charset=\"utf-8\"" );
+							
+						con2.setRequestMethod( "M-POST" );
+							
+						con2.setRequestProperty( "MAN", "\"http://schemas.xmlsoap.org/soap/envelope/\"; ns=01" );
+				
+						con2.setRequestProperty( "01-SOAPACTION", "\""+ soap_action + "\"");
+							
+						con2.setDoInput( true );
+						con2.setDoOutput( true );
+							
+						os = con2.getOutputStream();
+							
+						pw = new PrintWriter( new OutputStreamWriter(os, "UTF-8" ));
+										
+						pw.println( request );
+							
+						pw.flush();
+				
+						con2.connect();
+						
+						return( parseXML(con2.getInputStream()));	
+						
+					}catch( Throwable e ){
+						
+					}
+					
+					InputStream es = con1.getErrorStream();
+					
+					String	info = null;
+					
+					try{
+						info = FileUtil.readInputStreamAsString( es, 512 );
+						
+					}catch( Throwable e ){
+					}
+					
+					String error = "SOAP RPC failed: " + con1.getResponseCode() + " " + con1.getResponseMessage();
+					
+					if ( info != null ){
+						
+						error += " - " + info;
+					}
+
+					throw( new IOException ( error ));
+					
+				}else{
+						
+					return( parseXML(con1.getInputStream()));
+				}
+			}finally{
+				
+				TorrentUtils.setTLSDescription( null );
+			}
+		}else{
+			final int CONNECT_TIMEOUT 	= 15*1000;
+			final int READ_TIMEOUT		= 30*1000;
+			
+			Socket	socket = new Socket();
+			
+			socket.connect( new InetSocketAddress( control.getHost(), control.getPort()), CONNECT_TIMEOUT );
+			
+			socket.setSoTimeout( READ_TIMEOUT );
+				
 			try{
 				PrintWriter	pw = new PrintWriter(new OutputStreamWriter( socket.getOutputStream(), "UTF8" ));
 
@@ -707,7 +766,7 @@ UPnPImpl
 				
 				pw.flush();
 				
-				InputStream	is = HTTPUtils.decodeChunkedEncoding( socket.getInputStream());
+				InputStream	is = HTTPUtils.decodeChunkedEncoding( socket );
 				
 				return( parseXML( is ));
 				
@@ -719,10 +778,10 @@ UPnPImpl
 				}catch( Throwable e ){
 					
 					Debug.printStackTrace(e);
-				} // end catch
-			} // end finally
-		} // end else
-	} // end method
+				}
+			}
+		} 
+	}
 	
 	protected File
 	getTraceFile()
@@ -870,6 +929,20 @@ UPnPImpl
 		log_listeners.remove( l );
 	}
 	
+	public UPnPRootDevice[]
+	getRootDevices()
+	{
+		try{
+			this_mon.enter();
+
+			return( root_locations.values().toArray( new UPnPRootDevice[ root_locations.size()] ));
+			
+		}finally{
+			
+			this_mon.exit();
+		}
+	}
+	
 	public void
 	addRootDeviceListener(
 		UPnPListener	l )
