@@ -202,6 +202,10 @@ public class ConfigurationDefaults {
     def.put("Seeding Piece Check Recheck Enable", TRUE );
     def.put("priorityExtensions", "");
     def.put("priorityExtensionsIgnoreCase", FALSE);
+    
+    def.put("Rename Incomplete Files", FALSE );
+    def.put("Rename Incomplete Files Extension", ".az!" );
+    
     def.put("Ip Filter Enabled", TRUE);
     def.put("Ip Filter Allow",FALSE);
     def.put("Ip Filter Enable Banning", TRUE);
@@ -372,6 +376,7 @@ public class ConfigurationDefaults {
     def.put( "Tracker TCP NonBlocking Restrict Request Types", TRUE);
     def.put( "Tracker TCP NonBlocking Conc Max", new Long(2048));
     def.put( "Tracker Client Scrape Enable", TRUE);
+    def.put( "Tracker Client Scrape Total Disable", FALSE );
     def.put( "Tracker Client Scrape Stopped Enable", TRUE);
     def.put( "Tracker Client Scrape Single Only", FALSE);
     def.put( "Tracker Server Full Scrape Enable", TRUE );
@@ -454,6 +459,7 @@ public class ConfigurationDefaults {
     def.put( "peercontrol.udp.probe.enable", FALSE );
     def.put( "peercontrol.hide.piece", FALSE );
     def.put( "peercontrol.scheduler.use.priorities", TRUE );
+    def.put( "peercontrol.prefer.udp", FALSE );
     	
     def.put( "File.truncate.if.too.large", FALSE);
     def.put( "Enable System Tray", TRUE);
@@ -461,17 +467,22 @@ public class ConfigurationDefaults {
     def.put( "Ignore.peer.ports", "0" );
     def.put( "Security.JAR.tools.dir", "" );
     def.put( "network.max.simultaneous.connect.attempts", new Long( Constants.isWindows?8:24 ));
+    def.put( "network.tcp.max.connections.outstanding", new Long( 2048 ));
     def.put( "network.tcp.mtu.size", new Long(1500) );
     def.put( "network.udp.mtu.size", new Long(1500) );
     def.put( "network.tcp.socket.SO_SNDBUF", ZERO );
     def.put( "network.tcp.socket.SO_RCVBUF", ZERO );
     def.put( "network.tcp.socket.IPDiffServ", "" );
     def.put( "network.tcp.read.select.time", new Long(25));
+    def.put( "network.tcp.read.select.min.time", new Long(0));
     def.put( "network.tcp.write.select.time", new Long(25));
+    def.put( "network.tcp.write.select.min.time", new Long(0));
     def.put( "network.control.write.idle.time", new Long(50));
     def.put( "network.control.write.aggressive", FALSE );
     def.put( "network.control.read.idle.time", new Long(50));
     def.put( "network.control.read.aggressive", FALSE );
+    def.put( "network.control.read.processor.count", new Long(1));
+    def.put( "network.control.write.processor.count", new Long(1));
     def.put( "peermanager.schedule.time", new Long(100));
     def.put( "confirm_torrent_removal", FALSE );
     def.put( "add_torrents_silently", FALSE );
@@ -533,27 +544,40 @@ public class ConfigurationDefaults {
     def.put("FileBrowse.usePathFinder", FALSE);
     	
     //temp section for SpeedManagerAlgorithmProviderV2
-    def.put(SpeedManagerAlgorithmProviderV2.SETTING_DOWNLOAD_MAX_LIMIT, new Long(SMConst.START_DOWNLOAD_RATE_MAX) );
-    def.put(SpeedManagerAlgorithmProviderV2.SETTING_UPLOAD_MAX_LIMIT, new Long(SMConst.START_UPLOAD_RATE_MAX) );
-
-    def.put(SpeedManagerAlgorithmProviderV2.SETTING_DHT_GOOD_SET_POINT, new Long(50) );
-    def.put(SpeedManagerAlgorithmProviderV2.SETTING_DHT_GOOD_TOLERANCE, new Long(100) );
-    def.put(SpeedManagerAlgorithmProviderV2.SETTING_DHT_BAD_SET_POINT, new Long(900) );
-    def.put(SpeedManagerAlgorithmProviderV2.SETTING_DHT_BAD_TOLERANCE, new Long(500) );
-
-    	//**** NOTE! This default can be overridden in ConfigurationChecker 
-    def.put(SpeedManagerImpl.CONFIG_VERSION, new Long(2) );	// 1 == classic, 2 == beta
     
-    def.put( SpeedLimitMonitor.DOWNLOAD_CONF_LIMIT_SETTING, SpeedLimitConfidence.NONE.getString() );
-    def.put( SpeedLimitMonitor.UPLOAD_CONF_LIMIT_SETTING, SpeedLimitConfidence.NONE.getString() );
-    def.put( SpeedLimitMonitor.UPLOAD_CHOKE_PING_COUNT, new Long(1) );
-
-    //default V2 algorithm seeding and download mode usage, stored as an Int
-    def.put( SpeedLimitMonitor.USED_UPLOAD_CAPACITY_SEEDING_MODE, new Long(90) );  
-    def.put( SpeedLimitMonitor.USED_UPLOAD_CAPACITY_DOWNLOAD_MODE, new Long(60) );
-
-    def.put( SpeedManagerAlgorithmProviderV2.SETTING_WAIT_AFTER_ADJUST, TRUE );
-    def.put( SpeedManagerAlgorithmProviderV2.SETTING_INTERVALS_BETWEEN_ADJUST, new Long(2) );
+    try{
+	    def.put(SpeedManagerAlgorithmProviderV2.SETTING_DOWNLOAD_MAX_LIMIT, new Long(SMConst.START_DOWNLOAD_RATE_MAX) );
+	    def.put(SpeedManagerAlgorithmProviderV2.SETTING_UPLOAD_MAX_LIMIT, new Long(SMConst.START_UPLOAD_RATE_MAX) );
+	
+	    def.put(SpeedManagerAlgorithmProviderV2.SETTING_DHT_GOOD_SET_POINT, new Long(50) );
+	    def.put(SpeedManagerAlgorithmProviderV2.SETTING_DHT_GOOD_TOLERANCE, new Long(100) );
+	    def.put(SpeedManagerAlgorithmProviderV2.SETTING_DHT_BAD_SET_POINT, new Long(900) );
+	    def.put(SpeedManagerAlgorithmProviderV2.SETTING_DHT_BAD_TOLERANCE, new Long(500) );
+	
+	    	//**** NOTE! This default can be overridden in ConfigurationChecker 
+	    def.put(SpeedManagerImpl.CONFIG_VERSION, new Long(2) );	// 1 == classic, 2 == beta
+	    
+	    def.put( SpeedLimitMonitor.DOWNLOAD_CONF_LIMIT_SETTING, SpeedLimitConfidence.NONE.getString() );
+	    def.put( SpeedLimitMonitor.UPLOAD_CONF_LIMIT_SETTING, SpeedLimitConfidence.NONE.getString() );
+	    def.put( SpeedLimitMonitor.UPLOAD_CHOKE_PING_COUNT, new Long(1) );
+	
+	    //default V2 algorithm seeding and download mode usage, stored as an Int
+	    def.put( SpeedLimitMonitor.USED_UPLOAD_CAPACITY_SEEDING_MODE, new Long(90) );  
+	    def.put( SpeedLimitMonitor.USED_UPLOAD_CAPACITY_DOWNLOAD_MODE, new Long(60) );
+	
+	    def.put( SpeedManagerAlgorithmProviderV2.SETTING_WAIT_AFTER_ADJUST, TRUE );
+	    def.put( SpeedManagerAlgorithmProviderV2.SETTING_INTERVALS_BETWEEN_ADJUST, new Long(2) );
+	    
+    }catch( Throwable e ){
+    	
+    }
+    
+    	// subscriptions
+    
+    def.put( "subscriptions.max.non.deleted.results", new Long( 512 ));
+    def.put( "subscriptions.auto.start.downloads", TRUE );
+    def.put( "subscriptions.auto.start.min.mb", ZERO );
+    def.put( "subscriptions.auto.start.max.mb", ZERO );
 }
   
   protected 
