@@ -39,10 +39,12 @@ import com.aelitis.azureus.core.util.CaseSensitiveFileMap;
 public interface 
 DownloadManagerState 
 {
+	public static final String AT_VERSION					= "version";
 	public static final String AT_CATEGORY					= "category";
 	public static final String AT_NETWORKS					= "networks";
 	public static final String AT_USER						= "user";
 	public static final String AT_PEER_SOURCES				= "peersources";
+	public static final String AT_PEER_SOURCES_DENIED		= "peersourcesdenied";
 	public static final String AT_TRACKER_CLIENT_EXTENSIONS	= "trackerclientextensions";
 	public static final String AT_FILE_LINKS				= "filelinks";
 	public static final String AT_FILE_STORE_TYPES			= "storetypes";
@@ -52,22 +54,30 @@ DownloadManagerState
 	public static final String AT_DISPLAY_NAME              = "displayname";
 	public static final String AT_USER_COMMENT              = "comment";
 	public static final String AT_RELATIVE_SAVE_PATH        = "relativepath";
-	public static final String AT_CONTENT_MAP 				= "contentmap";
 	public static final String AT_SECRETS				 	= "secrets";
 	public static final String AT_RESUME_STATE		 		= "resumecomplete";
 	public static final String AT_PRIMARY_FILE		 		= "primaryfile";
 	public static final String AT_TIME_SINCE_DOWNLOAD		= "timesincedl";
 	public static final String AT_TIME_SINCE_UPLOAD			= "timesinceul";
+	public static final String AT_AVAIL_BAD_TIME			= "badavail";
+	public static final String AT_TIME_STOPPED				= "timestopped";
+	public static final String AT_INCOMP_FILE_SUFFIX		= "incompfilesuffix";
 	
 	public static Object[][] ATTRIBUTE_DEFAULTS = {
+		{ AT_VERSION,								new Integer( -1 )},
 		{ AT_TIME_SINCE_DOWNLOAD,					new Integer( -1 )},
 		{ AT_TIME_SINCE_UPLOAD,						new Integer( -1 )},
+		{ AT_AVAIL_BAD_TIME,						new Long( -1 )},
 	};
 	
-	public static final long FLAG_ONLY_EVER_SEEDED			= Download.FLAG_ONLY_EVER_SEEDED;
-	public static final long FLAG_SCAN_INCOMPLETE_PIECES	= Download.FLAG_SCAN_INCOMPLETE_PIECES;
-	public static final long FLAG_DISABLE_AUTO_FILE_MOVE    = Download.FLAG_DISABLE_AUTO_FILE_MOVE;
-	public static final long FLAG_MOVE_ON_COMPLETION_DONE   = Download.FLAG_MOVE_ON_COMPLETION_DONE;
+	public static final long FLAG_ONLY_EVER_SEEDED						= Download.FLAG_ONLY_EVER_SEEDED;
+	public static final long FLAG_SCAN_INCOMPLETE_PIECES				= Download.FLAG_SCAN_INCOMPLETE_PIECES;
+	public static final long FLAG_DISABLE_AUTO_FILE_MOVE    			= Download.FLAG_DISABLE_AUTO_FILE_MOVE;
+	public static final long FLAG_MOVE_ON_COMPLETION_DONE   			= Download.FLAG_MOVE_ON_COMPLETION_DONE;
+	public static final long FLAG_LOW_NOISE								= Download.FLAG_LOW_NOISE;
+	public static final long FLAG_ALLOW_PERMITTED_PEER_SOURCE_CHANGES	= Download.FLAG_ALLOW_PERMITTED_PEER_SOURCE_CHANGES;
+	public static final long FLAG_DO_NOT_DELETE_DATA_ON_REMOVE  		= Download.FLAG_DO_NOT_DELETE_DATA_ON_REMOVE;
+	
 	
 	public static final String	PARAM_MAX_PEERS							= "max.peers";
 	public static final String	PARAM_MAX_PEERS_WHEN_SEEDING			= "max.peers.when.seeding";
@@ -78,8 +88,10 @@ DownloadManagerState
 	public static final String	PARAM_MAX_UPLOADS_WHEN_SEEDING_ENABLED	= "max.uploads.when.seeding.enabled";
 	public static final String	PARAM_STATS_COUNTED						= "stats.counted";
 	public static final String	PARAM_DOWNLOAD_ADDED_TIME				= "stats.download.added.time";
+	public static final String	PARAM_DOWNLOAD_COMPLETED_TIME			= "stats.download.completed.time";
 	public static final String	PARAM_MAX_UPLOAD_WHEN_BUSY				= "max.upload.when.busy";
 	public static final String  PARAM_DND_FLAGS							= "dndflags";
+	public static final String  PARAM_RANDOM_SEED						= "rand";
 	
 	public static final int DEFAULT_MAX_UPLOADS	= 4;
 	public static final int MIN_MAX_UPLOADS		= 2;
@@ -94,8 +106,10 @@ DownloadManagerState
 		{ PARAM_MAX_UPLOADS_WHEN_SEEDING_ENABLED, 	new Boolean( false ) },
 		{ PARAM_STATS_COUNTED, 						new Boolean( false ) },
 		{ PARAM_DOWNLOAD_ADDED_TIME,				new Long( 0 ) },
+		{ PARAM_DOWNLOAD_COMPLETED_TIME, 			new Long( 0 ) },
 		{ PARAM_MAX_UPLOAD_WHEN_BUSY,				new Long( 0 ) },
-		{ PARAM_DND_FLAGS, new Long ( 0 ) },
+		{ PARAM_DND_FLAGS, 							new Long( 0 ) },
+		{ PARAM_RANDOM_SEED, 						new Long( 0 ) },
 	};
 	
 	public TOTorrent
@@ -105,8 +119,7 @@ DownloadManagerState
 	getDownloadManager();
 	
 	public File 
-	getStateFile(
-		String	name );
+	getStateFile( );
 	
 	public void
 	setFlag(
@@ -190,7 +203,6 @@ DownloadManagerState
 	public void setUserComment(String name);
 	
 	public String getRelativeSavePath();
-	public void setRelativeSavePath(String path);
 
 	public void setPrimaryFile(String fileFullPath);
 	public String getPrimaryFile();
@@ -223,7 +235,12 @@ DownloadManagerState
 	
 	public boolean
 	isPeerSourcePermitted(
-		String	peerSource );
+		String		peerSource );
+	
+	public void
+	setPeerSourcePermitted(
+		String		peerSource,
+		boolean		permitted );
 	
 	public boolean
 	isPeerSourceEnabled(
@@ -273,6 +290,7 @@ DownloadManagerState
 	public Map getMapAttribute(String name);
 	public void	setListAttribute(String	name, String[] values);
 	public String[]	getListAttribute(String	name);
+	public String getListAttribute(String name, int idx);
 	public void setIntAttribute(String name, int value);
 	public int getIntAttribute(String name);
 	public void setLongAttribute(String name, long value);
@@ -281,6 +299,12 @@ DownloadManagerState
 	public boolean getBooleanAttribute(String name);	
 	public boolean hasAttribute(String name);
 	
+	public void
+	setActive(
+		boolean	active );
+	
+	public void discardFluff();
+	
 	public void
 	save();
 	
@@ -306,4 +330,18 @@ DownloadManagerState
 	boolean parameterExists(String name);
 	
 	public void generateEvidence(IndentWriter writer);
+	
+	/**
+	 * This method should only be invoked in matching try-finally pairs. If it is invoked with true
+	 * multiple times it must be invoked with false the equal amount of times to reallow state
+	 * writes
+	 * 
+	 * @param suppress
+	 *            when set to true prevents flushing of the state/increments the internal nesting
+	 *            counter, decrements/allows flush otherwise
+	 */
+	public void suppressStateSave(boolean suppress);
+	
+	public void addListener(DownloadManagerStateAttributeListener l, String attribute, int event_type);
+	public void removeListener(DownloadManagerStateAttributeListener l, String attribute, int event_type);
 }
