@@ -25,10 +25,8 @@ package org.gudy.azureus2.core3.download.impl;
 import java.io.File;
 import java.io.IOException;
 import java.net.InetSocketAddress;
-import java.util.ArrayList;
-import java.util.HashMap;
-import java.util.List;
-import java.util.Map;
+import java.util.*;
+
 
 import org.gudy.azureus2.core3.config.COConfigurationManager;
 import org.gudy.azureus2.core3.disk.*;
@@ -63,7 +61,6 @@ import com.aelitis.azureus.core.peermanager.peerdb.PeerItemFactory;
 import com.aelitis.azureus.core.util.bloom.BloomFilter;
 import com.aelitis.azureus.core.util.bloom.BloomFilterFactory;
 import com.aelitis.azureus.plugins.extseed.ExternalSeedPlugin;
-import com.aelitis.azureus.plugins.net.buddy.tracker.BuddyPluginTracker;
 
 public class 
 DownloadManagerController 
@@ -86,8 +83,10 @@ DownloadManagerController
 			ext_seed_plugin_tried	= true;
 			
 			try {
-				ext_seed_plugin = (ExternalSeedPlugin)AzureusCoreFactory.getSingleton().getPluginManager().getPluginInterfaceByClass(
-						ExternalSeedPlugin.class).getPlugin();
+				org.gudy.azureus2.plugins.PluginInterface ext_pi = AzureusCoreFactory.getSingleton().getPluginManager().getPluginInterfaceByClass(ExternalSeedPlugin.class);
+				if (ext_pi != null) {
+					ext_seed_plugin = (ExternalSeedPlugin)ext_pi.getPlugin();
+				}
 	
 			}catch (Throwable e){
 	
@@ -191,6 +190,8 @@ DownloadManagerController
 	
 	private boolean	 piece_checking_enabled	= true;
 	
+	private long		priority_connection_count;
+	
 	
 	protected
 	DownloadManagerController(
@@ -496,6 +497,27 @@ DownloadManagerController
 								dms.setPeerSourcePermitted( s, false );
 							}
 						}
+						
+						PEPeerManager pm = getPeerManager();
+						
+						if ( pm != null ){
+							
+							Set<String>	allowed = new HashSet<String>();
+							
+							allowed.addAll( Arrays.asList( allowed_sources ));
+							
+							Iterator<PEPeer> it = pm.getPeers().iterator();
+						
+							while( it.hasNext()){
+								
+								PEPeer peer = it.next();
+								
+								if ( !allowed.contains( peer.getPeerSource())){
+									
+									pm.removePeer( peer, "Peer source not permitted" );
+								}
+							}
+						}
 					}
 					
 					public boolean
@@ -1144,7 +1166,7 @@ DownloadManagerController
 	      
   		if ( call_filesExist ){
   			
-  			filesExist();
+  			filesExist( true );
   		}
   		
   		if ( _inform_changed ){
@@ -1541,8 +1563,17 @@ DownloadManagerController
 
 	
 	public boolean 
-	filesExist() 
+	filesExist(
+		boolean	expected_to_be_allocated )
 	{
+		if ( !expected_to_be_allocated ){
+			
+			if ( !download_manager.isDataAlreadyAllocated()){
+				
+				return( false );
+			}
+		}
+		
 		DiskManager dm = getDiskManager();
 
 		if (dm != null) {
@@ -2049,10 +2080,30 @@ DownloadManagerController
 		}
 	}
 	
-	public boolean
-	hasBuddies()
+	public void
+	priorityConnectionChanged(
+		boolean	added )
 	{
-		return( download_manager.getUserData( BuddyPluginTracker.DOWNLOAD_KEY ) != null );
+		synchronized( this ){
+			
+			if ( added ){
+				
+				priority_connection_count++;
+				
+			}else{
+				
+				priority_connection_count--;
+			}
+		}
+	}
+	
+	public boolean
+	hasPriorityConnection()
+	{
+		synchronized( this ){
+			
+			return( priority_connection_count > 0 ); 
+		}
 	}
 	
 	public String 
@@ -2099,7 +2150,6 @@ DownloadManagerController
 	
 	protected class FileInfoFacadeSet implements DiskManagerFileInfoSet {
 
-		AEMonitor	facade_mon	= new AEMonitor( "DownloadManagerController:Facade" );
 		DiskManagerFileInfoSet delegate;
 		fileInfoFacade[] facadeFiles = new fileInfoFacade[0];	// default before torrent avail
 
@@ -2108,6 +2158,9 @@ DownloadManagerController
 		}
 
 		public int nbFiles() {
+			if (delegate == null) {
+				return 0;
+			}
 			return delegate.nbFiles();
 		}
 
@@ -2132,7 +2185,7 @@ DownloadManagerController
 			final List delayed_prio_changes = new ArrayList();
 			
 			try {
-				facade_mon.enter();
+				this_mon.enter();
 				if (files_facade_destroyed)	return;
 
 				DiskManager dm = DownloadManagerController.this.getDiskManager();
@@ -2184,7 +2237,7 @@ DownloadManagerController
 				delegate = active;
 				
 			} finally {
-				facade_mon.exit();
+				this_mon.exit();
 			}
 			
 			fileFacadeSet.facadeFiles = info;
@@ -2213,7 +2266,7 @@ DownloadManagerController
 		
 		protected void destroyFileInfo() {
 			try {
-				facade_mon.enter();
+				this_mon.enter();
 				if (fileFacadeSet == null || files_facade_destroyed)
 					return;
 
@@ -2222,7 +2275,7 @@ DownloadManagerController
 				for (int i = 0; i < facadeFiles.length; i++)
 					facadeFiles[i].close();
 			} finally {
-				facade_mon.exit();
+				this_mon.exit();
 			}
 		}
 	}
@@ -2405,7 +2458,15 @@ DownloadManagerController
 		
 			throws	Exception
 		{
-			delegate.flushCache();
+			try{
+				this_mon.enter();
+				
+				delegate.flushCache();
+				
+			}finally{
+				
+				this_mon.exit();
+			}
 		}
 		
 		public DirectByteBuffer
@@ -2415,13 +2476,29 @@ DownloadManagerController
 		
 			throws IOException
 		{
-			return( delegate.read( offset, length ));
+			try{
+				this_mon.enter();
+			
+				return( delegate.read( offset, length ));
+				
+			}finally{
+				
+				this_mon.exit();
+			}
 		}
 		
 		public void
 		close()
 		{
-			delegate.close();
+			try{
+				this_mon.enter();
+
+				delegate.close();
+				
+			}finally{
+				
+				this_mon.exit();
+			}
 		}
 		
 		public void
@@ -2465,7 +2542,7 @@ DownloadManagerController
 				writer.println("Force Start");
 			}
 
-			writer.println("FilesExist? " + filesExist());
+			writer.println("FilesExist? " + filesExist(download_manager.isDataAlreadyAllocated()));
 			
 		} finally {
 			writer.exdent();
