@@ -1300,10 +1300,21 @@ DownloadManagerImpl
 	}
 	*/
 	
+	/**
+	 * @deprecated
+	 */
+	
 	public boolean 
-	filesExist() 
+	filesExist()
 	{
-		return( controller.filesExist());
+		return( filesExist( true ));
+	}
+	
+	public boolean 
+	filesExist(
+		boolean expected_to_be_allocated )
+	{
+		return( controller.filesExist( expected_to_be_allocated ));
 	}
 	
 	
@@ -1436,7 +1447,7 @@ DownloadManagerImpl
 		// If a torrent that is assumed complete, verify that it actually has the
 		// files, before we create the diskManager, which will try to allocate
 		// disk space.
-		if (assumedComplete && !filesExist()) {
+		if (assumedComplete && !filesExist( true )) {
 			// filesExist() has set state to error for us
 
 			// If the user wants to re-download the missing files, they must
@@ -1766,7 +1777,8 @@ DownloadManagerImpl
 		
 		if ( cl == null ){
 			
-			Debug.out( "setAnnounceResult called when download not running" );
+			// this can happen due to timing issues - not work debug spew for
+			// Debug.out( "setAnnounceResult called when download not running" );
 			
 			return;
 		}
@@ -1972,36 +1984,31 @@ DownloadManagerImpl
 		}
 		
 		if (response != null) {
-			if (response.isValid()) {
-				int state = getState();
-				if (state == STATE_ERROR || state == STATE_STOPPED) {
-					long minNextScrape = SystemTime.getCurrentTime()
-							+ (state == STATE_ERROR ? SCRAPE_DELAY_ERROR_TORRENTS
-									: SCRAPE_DELAY_STOPPED_TORRENTS);
-					if (response.getNextScrapeStartTime() < minNextScrape) {
-						response.setNextScrapeStartTime(minNextScrape);
-					}
-				}
-			} else if (response.getStatus() == TRTrackerScraperResponse.ST_INITIALIZING) {
+			int state = getState();
+			if (state == STATE_ERROR || state == STATE_STOPPED) {
 				long minNextScrape;
-				int state = getState();
-				if (state == STATE_ERROR || state == STATE_STOPPED) {
-					// Delay initial scrape if torrent is stopped or in error.
-					// Save excessive thread creation (by the scraper) and spreads out
-					// CPU usage at startup time
+				if (response.getStatus() == TRTrackerScraperResponse.ST_INITIALIZING) {
 					minNextScrape = SystemTime.getCurrentTime()
 							+ (state == STATE_ERROR ? SCRAPE_INITDELAY_ERROR_TORRENTS
 									: SCRAPE_INITDELAY_STOPPED_TORRENTS);
 				} else {
-					// Spread the scrapes out a bit.  This is extremely helpfull on large
-					// torrent lists, and trackers that do not support multi-scrapes.
-					// For trackers that do support multi-scrapes, it will really delay
-					// the scrape for all torrent in the tracker to the one that has
-					// the lowest share ratio.
-					int sr = getStats().getShareRatio();
 					minNextScrape = SystemTime.getCurrentTime()
-							+ ((sr > 10000 ? 10000 : sr + 1000) * 60);
+							+ (state == STATE_ERROR ? SCRAPE_DELAY_ERROR_TORRENTS
+									: SCRAPE_DELAY_STOPPED_TORRENTS);
 				}
+				if (response.getNextScrapeStartTime() < minNextScrape) {
+					response.setNextScrapeStartTime(minNextScrape);
+				}
+			} else if (!response.isValid() && response.getStatus() == TRTrackerScraperResponse.ST_INITIALIZING) {
+				long minNextScrape;
+				// Spread the scrapes out a bit.  This is extremely helpfull on large
+				// torrent lists, and trackers that do not support multi-scrapes.
+				// For trackers that do support multi-scrapes, it will really delay
+				// the scrape for all torrent in the tracker to the one that has
+				// the lowest share ratio.
+				int sr = getStats().getShareRatio();
+				minNextScrape = SystemTime.getCurrentTime()
+						+ ((sr > 10000 ? 10000 : sr + 1000) * 60);
 
 				if (response.getNextScrapeStartTime() < minNextScrape) {
 					response.setNextScrapeStartTime(minNextScrape);
@@ -2269,6 +2276,14 @@ DownloadManagerImpl
 	public void
 	addListener(
 		DownloadManagerListener	listener )
+	{
+		addListener(listener, true);
+	}
+
+	public void
+	addListener(
+		DownloadManagerListener	listener,
+		boolean triggerStateChange )
 	{
 		if (listener == null) {
 			Debug.out("Warning: null listener");
@@ -2280,7 +2295,9 @@ DownloadManagerImpl
 
 			listeners.addListener(listener);
 				
-			listener.stateChanged( this, getState());
+			if (triggerStateChange) {
+				listener.stateChanged( this, getState());
+			}
 
 				// we DON'T dispatch a downloadComplete event here as this event is used to mark the
 				// transition between downloading and seeding, NOT purely to inform of seeding status
@@ -2399,7 +2416,7 @@ DownloadManagerImpl
 				
 				// an active torrent changed its position, scheduling needs to be updated
 				if(getState() == DownloadManager.STATE_SEEDING || getState() == DownloadManager.STATE_DOWNLOADING)
-					PeerControlSchedulerFactory.getSingleton().updateScheduleOrdering();
+					PeerControlSchedulerFactory.updateScheduleOrdering();
 			}
 		}finally{
 			
@@ -3346,12 +3363,6 @@ DownloadManagerImpl
 	  }
   }
   
-  public File[] calculateDefaultPaths() {
-	  SaveLocationChange slc = DownloadManagerMoveHandler.recalculatePath(this); 
-	  if (slc == null) {return null;}
-	  return new File[] {slc.download_location, slc.torrent_location};
-  }
-    
   public boolean isInDefaultSaveDir() {
 	  return DownloadManagerDefaultPaths.isInDefaultDownloadDir(this);
   }
@@ -3385,6 +3396,83 @@ DownloadManagerImpl
 	  controller.removeRateLimiter( group, upload );  
   }
 	
+  public boolean
+  isTrackerError()
+  {
+		TRTrackerAnnouncer announcer = getTrackerClient();
+		
+		if ( announcer != null ){
+			
+			TRTrackerAnnouncerResponse resp = announcer.getLastResponse();
+			
+			if ( resp != null ){
+				
+				if ( resp.getStatus() == TRTrackerAnnouncerResponse.ST_REPORTED_ERROR ){
+					
+					return( true );
+				}
+			}
+		}else{
+			
+			TRTrackerScraperResponse resp = getTrackerScrapeResponse();
+			
+			if ( resp != null ){
+				
+				if ( resp.getStatus() == TRTrackerScraperResponse.ST_ERROR ){
+					
+					return( true );
+				}
+			}
+		}  
+		
+		return( false );
+  }
+  
+  public boolean
+  isUnauthorisedOnTracker()
+  {
+		TRTrackerAnnouncer announcer = getTrackerClient();
+
+		String	status_str = null;
+		
+		if ( announcer != null ){
+			
+			TRTrackerAnnouncerResponse resp = announcer.getLastResponse();
+			
+			if ( resp != null ){
+				
+				if ( resp.getStatus() == TRTrackerAnnouncerResponse.ST_REPORTED_ERROR ){
+					
+					status_str = resp.getStatusString();
+				}
+			}
+		}else{
+			
+			TRTrackerScraperResponse resp = getTrackerScrapeResponse();
+			
+			if ( resp != null ){
+				
+				if ( resp.getStatus() == TRTrackerScraperResponse.ST_ERROR ){
+					
+					status_str = resp.getStatusString();
+				}
+			}
+		}
+		
+		if ( status_str != null ){
+			
+			status_str = status_str.toLowerCase();
+			
+			if ( 	status_str.contains( "not authorised" ) ||
+					status_str.contains( "not authorized" )){
+				
+				return( true );
+			}
+		}
+		
+		return( false );
+  }
+  
   private byte[]
   getIdentity()
   {
@@ -3631,7 +3719,7 @@ DownloadManagerImpl
 	
 	public int[] getStorageType(DiskManagerFileInfo[] info) {
 		String[] types = DiskManagerImpl.getStorageTypes(this);
-		int[] result = new int[types.length];
+		int[] result = new int[info.length];
 		boolean is_linear;
 		for (int i=0; i<info.length; i++) {
 			is_linear = types[info[i].getIndex()].equals("L");
