@@ -35,6 +35,7 @@ import java.util.*;
 import org.gudy.azureus2.core3.category.Category;
 import org.gudy.azureus2.core3.category.CategoryManager;
 import org.gudy.azureus2.core3.config.COConfigurationManager;
+import org.gudy.azureus2.core3.disk.DiskManagerFileInfo;
 import org.gudy.azureus2.core3.download.*;
 import org.gudy.azureus2.core3.download.impl.DownloadManagerAdapter;
 import org.gudy.azureus2.core3.global.*;
@@ -144,7 +145,7 @@ public class GlobalManagerImpl
 				}
 			});
 	
-	private List 		managers_cow	= new ArrayList();
+	private List<DownloadManager> 		managers_cow	= new ArrayList<DownloadManager>();
 	private AEMonitor	managers_mon	= new AEMonitor( "GM:Managers" );
 	
 	private Map		manager_map			= new HashMap();
@@ -155,7 +156,10 @@ public class GlobalManagerImpl
 	private GlobalManagerStatsImpl		stats;
     private long last_swarm_stats_calc_time		= 0;
     private long last_swarm_stats				= 0;
-	    
+    
+    // Set this flag to disable interaction with downloads.config.
+    // Do *NOT* change this - only the constructor should set it once.
+	private boolean cripple_downloads_config;
 
 	private TRTrackerScraper 			trackerScraper;
 	private GlobalManagerStatsWriter 	stats_writer;
@@ -208,9 +212,10 @@ public class GlobalManagerImpl
     int loopFactor;
     private static final int waitTime = 10*1000;
     // 5 minutes save resume data interval (default)
-    private int saveResumeLoopCount = 5*60*1000 / waitTime;
-    private int natCheckLoopCount	= 30*1000 / waitTime;
-    private int seedPieceCheckCount	= 30*1000 / waitTime;
+    private int saveResumeLoopCount 	= 5*60*1000 / waitTime;
+    private int initSaveResumeLoopCount = 60*1000 / waitTime;
+    private int natCheckLoopCount		= 30*1000 / waitTime;
+    private int seedPieceCheckCount		= 30*1000 / waitTime;
            
     private AESemaphore	run_sem = new AESemaphore( "GM:Checker:run");
     
@@ -238,7 +243,8 @@ public class GlobalManagerImpl
 	        
 	        determineSaveResumeDataInterval();
 	        
-	        if ((loopFactor % saveResumeLoopCount == 0) || needsSaving) {
+	        if (	( loopFactor % saveResumeLoopCount == 0 ) || 
+	        		( needsSaving && loadingComplete && loopFactor > initSaveResumeLoopCount )) {
 	          	
 	        	saveDownloads( true );
 	        }
@@ -308,9 +314,10 @@ public class GlobalManagerImpl
   	long 							existingTorrentLoadDelay)
   {
     //Debug.dumpThreadsLoop("Active threads");
-  	
 	progress_listener = listener;
 	
+	this.cripple_downloads_config = "1".equals(System.getProperty("azureus.disabledownloads"));
+	
   	AEDiagnostics.addEvidenceGenerator( this );
 	
     stats = new GlobalManagerStatsImpl( this );
@@ -1297,7 +1304,7 @@ public class GlobalManagerImpl
 	
 	long	lastListenerUpdate = 0;
 	
-	List	managers = managers_cow;
+	List<DownloadManager> managers = sortForStop();
 	
 	int nbDownloads = managers.size();
 	
@@ -1401,8 +1408,9 @@ public class GlobalManagerImpl
   pauseDownloads(
 	boolean	tag_only )
   {
-    for( Iterator i = managers_cow.iterator(); i.hasNext(); ) {
-      DownloadManager manager = (DownloadManager)i.next();
+	List<DownloadManager> managers = sortForStop();
+	
+    for( DownloadManager manager: managers ){
       
       if ( manager.getTorrent() == null ) {
         continue;
@@ -1607,10 +1615,48 @@ public class GlobalManagerImpl
     return false;
   }
   
-  
+  	private List<DownloadManager>
+  	sortForStop()
+	{
+  		List<DownloadManager>	managers = new ArrayList<DownloadManager>( managers_cow );
+  		
+  		Collections.sort(
+  			managers,
+  			new Comparator<DownloadManager>()
+  			{
+  				public int 
+  				compare(
+  					DownloadManager o1, 
+  					DownloadManager o2) 
+  				{
+  					int s1 = o1.getState();
+  					int s2 = o2.getState();
+  					
+  					if ( s2 == DownloadManager.STATE_QUEUED ){
+  						
+  						return( 1 );
+  						
+  					}else if ( s1 == DownloadManager.STATE_QUEUED ){
+
+  						return( -1 );
+  					}
+  					
+  					return( 0 );
+  				}
+  			});
+  		
+  		return( managers );
+	}
   
   private void loadDownloads() 
   {
+	  if (this.cripple_downloads_config) {
+		  loadingComplete = true;
+		  loadingSem.releaseForever();
+		  return;
+	  }
+
+	  
 	  try{
 		  DownloadManagerStateFactory.loadGlobalStateCache();
 		  
@@ -1862,6 +1908,9 @@ public class GlobalManagerImpl
     //    if(Boolean.getBoolean("debug")) return;
 
 	  needsSaving = false;
+	  if (this.cripple_downloads_config) {
+		  return;
+	  }
 	  
   	try{
   		managers_mon.enter();
@@ -1997,8 +2046,15 @@ public class GlobalManagerImpl
 	{
 		return( stats );
 	}
-	
-  public int getIndexOf(DownloadManager manager) {
+
+	public boolean contains(DownloadManager manager) {
+    if (managers_cow != null && manager != null) {
+      return managers_cow.contains(manager);
+    }
+    return false;
+  }
+
+	public int getIndexOf(DownloadManager manager) {
     if (managers_cow != null && manager != null)
       return managers_cow.indexOf(manager);
     return -1;
@@ -2356,7 +2412,7 @@ public class GlobalManagerImpl
 		  
 		  //System.out.println( "force_start_exists->" + force_start_non_seed_exists );
 		  
-		  PeerControlSchedulerFactory.getSingleton().overrideWeightedPriorities( force_start_non_seed_exists  );
+		  PeerControlSchedulerFactory.overrideWeightedPriorities( force_start_non_seed_exists  );
 	  }
   }
   
@@ -2370,6 +2426,7 @@ public class GlobalManagerImpl
   checkSeedingOnlyStateSupport()
   {
     boolean seeding 			= false;
+    boolean seeding_set 			= false;
     boolean	potentially_seeding	= false;
     
     List managers = managers_cow;
@@ -2391,19 +2448,34 @@ public class GlobalManagerImpl
         		if ( dm.isDownloadComplete( false )){
         			
         			potentially_seeding = true;
+        		} else {
+
+        			// Queued and Incomplete means we aren't "only seeding".  This
+        			// download WILL start, it just hasn't yet for various reasons
+        			// (1st Priority, etc)
+          		seeding 			= false;
+          		
+          		// can't break out, because we still need to calculate
+          		// potentially_seeding.  Set a flag so we don't set "seeding"
+          		// back to true
+          		seeding_set = true;
+
         		}
         	}
         	
         	continue;
         }
-                
+        
         if ( state == DownloadManager.STATE_DOWNLOADING ){
         	
         	if (!pm.hasDownloadablePiece()){
         		
         			// complete DND file
-        		
-        		seeding = true;
+
+        		if (!seeding_set) {
+        			
+        			seeding = true;
+        		}
         		
         	}else{
         		
@@ -2414,7 +2486,10 @@ public class GlobalManagerImpl
         	}
         }else if ( state == DownloadManager.STATE_SEEDING ){
         	
-        	seeding = true;
+      		if (!seeding_set) {
+
+      			seeding = true;
+      		}
         }
     }
     
@@ -2647,6 +2722,56 @@ public class GlobalManagerImpl
 								Debug.printStackTrace(e);
 							}
 						}
+						
+						if ( COConfigurationManager.getBooleanParameter( "Rename Incomplete Files")){
+							
+							String	ext = COConfigurationManager.getStringParameter( "Rename Incomplete Files Extension" ).trim();
+							
+							DownloadManagerState state = manager.getDownloadState();
+							
+							String existing_ext = state.getAttribute( DownloadManagerState.AT_INCOMP_FILE_SUFFIX );
+
+							if ( ext.length() > 0 && existing_ext == null ){
+									
+								ext = FileUtil.convertOSSpecificChars( ext, false );
+								
+								DiskManagerFileInfo[] fileInfos = manager.getDiskManagerFileInfo();
+								
+								try{
+									state.suppressStateSave(true);
+																	
+									for ( int i=0; i<fileInfos.length; i++ ){
+										
+										DiskManagerFileInfo fileInfo = fileInfos[i];
+										
+										File base_file = fileInfo.getFile( false );
+										
+										File existing_link = state.getFileLink( base_file );
+										
+										if ( existing_link == null || !existing_link.exists()){
+											
+											File	new_link;
+											
+											if ( existing_link == null ){
+												
+												new_link = new File( base_file.getParentFile(), base_file.getName() + ext );
+												
+											}else{
+												
+												new_link = new File( existing_link.getParentFile(), existing_link.getName() + ext );
+											}
+											
+											state.setFileLink( base_file,new_link );
+										}
+									}
+								}finally{
+									
+									state.setAttribute( DownloadManagerState.AT_INCOMP_FILE_SUFFIX, ext );
+									
+									state.suppressStateSave(false);
+								}
+							}
+						}
 					}
 				});
 	}
