@@ -173,22 +173,39 @@ OverallStatsImpl
 				Set		types,
 				Map		values )
 			{	
-				if ( types.contains( AzureusCoreStats.ST_XFER_UPLOADED_PROTOCOL_BYTES )){
-					
-					values.put( AzureusCoreStats.ST_XFER_UPLOADED_PROTOCOL_BYTES, new Long( totalProtocolUploaded ));
-				}
-				if ( types.contains( AzureusCoreStats.ST_XFER_UPLOADED_DATA_BYTES )){
-					
-					values.put( AzureusCoreStats.ST_XFER_UPLOADED_DATA_BYTES, new Long( totalDataUploaded ));
-				}
-				if ( types.contains( AzureusCoreStats.ST_XFER_DOWNLOADED_PROTOCOL_BYTES )){
-					
-					values.put( AzureusCoreStats.ST_XFER_DOWNLOADED_PROTOCOL_BYTES, new Long( totalProtocolDownloaded ));
-				}
-				if ( types.contains( AzureusCoreStats.ST_XFER_DOWNLOADED_DATA_BYTES )){
-					
-					values.put( AzureusCoreStats.ST_XFER_DOWNLOADED_DATA_BYTES, new Long( totalDataDownloaded ));
-				}
+			  	try{
+				    GlobalManagerStats stats = core.getGlobalManager().getStats();
+
+			  		this_mon.enter();
+			  		
+					if ( types.contains( AzureusCoreStats.ST_XFER_UPLOADED_PROTOCOL_BYTES )){
+						
+						values.put( 
+							AzureusCoreStats.ST_XFER_UPLOADED_PROTOCOL_BYTES, 
+							new Long( totalProtocolUploaded + ( stats.getTotalProtocolBytesSent() - lastProtocolUploaded )));
+					}
+					if ( types.contains( AzureusCoreStats.ST_XFER_UPLOADED_DATA_BYTES )){
+						
+						values.put( 
+							AzureusCoreStats.ST_XFER_UPLOADED_DATA_BYTES, 
+							new Long( totalDataUploaded + ( stats.getTotalDataBytesSent() - lastDataUploaded )));
+					}
+					if ( types.contains( AzureusCoreStats.ST_XFER_DOWNLOADED_PROTOCOL_BYTES )){
+						
+						values.put( 
+							AzureusCoreStats.ST_XFER_DOWNLOADED_PROTOCOL_BYTES, 
+							new Long( totalProtocolDownloaded + ( stats.getTotalProtocolBytesReceived() - lastProtocolDownloaded )));
+					}
+					if ( types.contains( AzureusCoreStats.ST_XFER_DOWNLOADED_DATA_BYTES )){
+						
+						values.put( 
+							AzureusCoreStats.ST_XFER_DOWNLOADED_DATA_BYTES, 
+							new Long( totalDataDownloaded + ( stats.getTotalDataBytesReceived() - lastDataDownloaded )));
+					}
+			  	}finally{
+			  	  	
+			  		this_mon.exit();
+			  	}
 			}
 		});
 	
