@@ -24,14 +24,15 @@
 
 package org.gudy.azureus2.core3.torrentdownloader.impl;
 
-import java.io.InputStream;
-import java.io.File;
-import java.io.FileOutputStream;
-import java.io.IOException;
+import java.io.*;
 
 import java.net.HttpURLConnection;
+import java.net.Inet4Address;
+import java.net.InetAddress;
 import java.net.URL;
 import java.net.URLDecoder;
+import java.util.Iterator;
+import java.util.Map;
 
 import javax.net.ssl.*;
 
@@ -49,8 +50,10 @@ import org.gudy.azureus2.core3.torrent.*;
  */
 public class TorrentDownloaderImpl extends AEThread implements TorrentDownloader {
 
+  private String	original_url; 
   private String 	url_str;
   private String	referrer;
+  private Map		request_properties;
   private String 	file_str;
   
   private URL url;
@@ -67,6 +70,7 @@ public class TorrentDownloaderImpl extends AEThread implements TorrentDownloader
   private byte[] buf = new byte[1020];
   private int bufBytes = 0;
   private boolean deleteFileOnCancel = true;
+  private boolean ignoreReponseCode = false;
   
 
   private AEMonitor this_mon 	= new AEMonitor( "TorrentDownloader" );
@@ -82,10 +86,13 @@ public class TorrentDownloaderImpl extends AEThread implements TorrentDownloader
   		TorrentDownloaderCallBackInterface	_iface, 
 		String 								_url,
 		String								_referrer,
+		Map									_request_properties,
 		String								_file )
   {
     this.iface = _iface;
     
+    original_url = _url;
+    
     //clean up accidental left-facing slashes
     _url = _url.replace( (char)92, (char)47 );
     
@@ -94,9 +101,10 @@ public class TorrentDownloaderImpl extends AEThread implements TorrentDownloader
 
     setName("TorrentDownloader: " + _url);
     
-    url_str 	= _url;
-    referrer	= _referrer;
-    file_str	= _file;
+    url_str 			= _url;
+    referrer			= _referrer;
+    request_properties	= _request_properties;
+    file_str			= _file;
   }
 
   public void notifyListener() {
@@ -151,7 +159,7 @@ public class TorrentDownloaderImpl extends AEThread implements TorrentDownloader
     	// hack here - the magnet download process requires an additional paramter to cause it to
     	// stall on error so the error can be reported
 	  
-    	if ( protocol.equals( "magnet" )){
+    	if ( protocol.equals( "magnet" ) || protocol.equals( "dht" )){
 		  
     		url = AddressUtils.adjustURL( new URL(url_str+"&pause_on_error=true"));
     	}
@@ -194,6 +202,26 @@ public class TorrentDownloaderImpl extends AEThread implements TorrentDownloader
 	      	con.setRequestProperty( "Referer", referrer );
 	      }
 	      
+	      if ( request_properties != null ){
+	    	  
+	    	  Iterator it = request_properties.entrySet().iterator();
+	    	  
+	    	  while( it.hasNext()){
+	    		
+	    		  Map.Entry	entry = (Map.Entry)it.next();
+	    		  
+	    		  String	key 	= (String)entry.getKey();
+	    		  String	value	= (String)entry.getValue();
+	    		  
+	    		  	// currently this code doesn't support gzip/deflate...
+	    		  
+	    		  if ( !key.equalsIgnoreCase( "Accept-Encoding" )){
+	    			  	    			    			  
+	    			  con.setRequestProperty( key, value );
+	    		  }
+	    	  }
+	      }
+	      
 	      this.con.connect();
 	      
 	      break;
@@ -211,23 +239,51 @@ public class TorrentDownloaderImpl extends AEThread implements TorrentDownloader
       		}
 
       		throw( e );
+      		
+      	}catch( IOException e ){
+      		
+      		if ( i == 0 ){
+      			
+      			URL retry_url = UrlUtils.getIPV4Fallback( url );
+      			
+      			if ( retry_url != null ){
+      				
+      				url = retry_url;
+      				
+      			}else{
+      				
+      				throw( e );
+      			}
+      		}
       	}
       }
       
-      int response = this.con.getResponseCode();
-      if ((response != HttpURLConnection.HTTP_ACCEPTED) && (response != HttpURLConnection.HTTP_OK)) {
-        this.error(response, Integer.toString(response) + ": " + this.con.getResponseMessage());
-        return;
-      }
+  		int response = this.con.getResponseCode();
+    	if (!ignoreReponseCode) {
+        if ((response != HttpURLConnection.HTTP_ACCEPTED) && (response != HttpURLConnection.HTTP_OK)) {
+          this.error(response, Integer.toString(response) + ": " + this.con.getResponseMessage());
+          return;
+        }
+    	}
 
-      this.filename = this.con.getHeaderField("Content-Disposition");
-      if ((this.filename!=null) && this.filename.toLowerCase().matches(".*attachment.*")) // Some code to handle b0rked servers.
-        while (this.filename.toLowerCase().charAt(0)!='a')
-          this.filename = this.filename.substring(1);
-      if ((this.filename == null) || !this.filename.toLowerCase().startsWith("attachment") || (this.filename.indexOf('=') == -1)) {
+      /*
+      Map headerFields = this.con.getHeaderFields();
+      
+      System.out.println("Header of download of " + url_str);
+      for (Iterator iter = headerFields.keySet().iterator(); iter.hasNext();) {
+				String s = (String) iter.next();
+				System.out.println(s + ":" + headerFields.get(s));
+				
+			}
+	*/
+      filename = this.con.getHeaderField("Content-Disposition");
+      if ((filename!=null) && filename.toLowerCase().matches(".*attachment.*")) // Some code to handle b0rked servers.
+        while (filename.toLowerCase().charAt(0)!='a')
+          filename = filename.substring(1);
+      if ((filename == null) || !filename.toLowerCase().startsWith("attachment") || (filename.indexOf('=') == -1)) {
         String tmp = this.url.getFile();
         if (tmp.length() == 0 || tmp.equals("/")) {
-        	this.filename = url.getHost();
+        	filename = url.getHost();
         }
         else if ( tmp.startsWith("?")){
         
@@ -252,19 +308,19 @@ public class TorrentDownloaderImpl extends AEThread implements TorrentDownloader
     			
     			if ( p2 == -1 ){
     				
-    				this.filename = query.substring(pos);
+    				filename = query.substring(pos);
     				
     			}else{
     				
-    				this.filename = query.substring(pos,p2);
+    				filename = query.substring(pos,p2);
     			}
         	}else{
         		
-        		this.filename = "Torrent" + (long)(Math.random()*Long.MAX_VALUE);
+        		filename = "Torrent" + (long)(Math.random()*Long.MAX_VALUE);
         	}
     		
     		
-    		this.filename += ".tmp";
+    		filename += ".tmp";
     		
         }else{
 	        if (tmp.lastIndexOf('/') != -1)
@@ -278,20 +334,25 @@ public class TorrentDownloaderImpl extends AEThread implements TorrentDownloader
 	          tmp = tmp.substring(0,param_pos);
 	        }
 	        
-	        this.filename = URLDecoder.decode(tmp, Constants.DEFAULT_ENCODING );
+	        filename = URLDecoder.decode(tmp, Constants.DEFAULT_ENCODING );
         }
       } else {
-        this.filename = this.filename.substring(this.filename.indexOf('=') + 1);
-        if (this.filename.startsWith("\"") && this.filename.endsWith("\""))
-          this.filename = this.filename.substring(1, this.filename.lastIndexOf('\"'));
-        File temp = new File(this.filename);
-        this.filename = temp.getName();
+        filename = filename.substring(filename.indexOf('=') + 1);
+        if (filename.startsWith("\"") && filename.endsWith("\""))
+          filename = filename.substring(1, filename.lastIndexOf('\"'));
+        
+        filename = URLDecoder.decode(filename, Constants.DEFAULT_ENCODING );
+        
+        	// not sure of this piece of logic here but I'm not changing it at the moment
+        
+        File temp = new File(filename);
+        filename = temp.getName();
       }
 
-      this.filename = FileUtil.convertOSSpecificChars( this.filename );
+      filename = FileUtil.convertOSSpecificChars( filename, false );
       
-      this.directoryname = COConfigurationManager.getDirectoryParameter("General_sDefaultTorrent_Directory");
-      boolean useTorrentSave = COConfigurationManager.getBooleanParameter("Save Torrent Files", true);
+      directoryname = COConfigurationManager.getDirectoryParameter("General_sDefaultTorrent_Directory");
+      boolean useTorrentSave = COConfigurationManager.getBooleanParameter("Save Torrent Files");
 
       if (file_str != null) {
       	// not completely sure about the whole logic in this block
@@ -408,6 +469,15 @@ public class TorrentDownloaderImpl extends AEThread implements TorrentDownloader
 			
 		try{
 			in = this.con.getInputStream();
+		
+		} catch (FileNotFoundException e) {
+			if (ignoreReponseCode) {
+
+				in = this.con.getErrorStream();
+			} else {
+
+				throw e;
+			}
 				
 		}finally{
 			
@@ -424,9 +494,21 @@ public class TorrentDownloaderImpl extends AEThread implements TorrentDownloader
 			
 	    if ( this.state != STATE_ERROR ){
 		    	
-	    	this.file = new File(this.directoryname, this.filename);
-	        
-	    	this.file.createNewFile();
+	    	this.file = new File(this.directoryname, filename);
+
+	    	boolean useTempFile = false;
+	    	try {
+	    		this.file.createNewFile();
+	    		useTempFile = !this.file.exists();
+	    	} catch (Throwable t) {
+	    		useTempFile = true;
+	    	}
+	    	
+	    	if (useTempFile) {
+	    		this.file = File.createTempFile("AZU", ".torrent", new File(
+							this.directoryname));
+	    		this.file.createNewFile();
+	    	}
 	        
 	        FileOutputStream fileout = new FileOutputStream(this.file, false);
 	        
@@ -501,6 +583,8 @@ public class TorrentDownloaderImpl extends AEThread implements TorrentDownloader
 	          	Debug.printStackTrace( e );
 	          }
 	          
+	          TorrentUtils.setObtainedFrom( file, original_url );
+
 	          this.state = STATE_FINISHED;
 	        }
 	        this.notifyListener();
@@ -509,7 +593,7 @@ public class TorrentDownloaderImpl extends AEThread implements TorrentDownloader
     	  
     	if ( !cancel ){
     		
-    		Debug.out("'" + this.directoryname + "' '" +  this.filename + "'", e);
+    		Debug.out("'" + this.directoryname + "' '" +  filename + "'", e);
     	}
       	
         this.error(0, "Exception while downloading '" + this.url.toString() + "':" + e.getMessage());
@@ -588,7 +672,7 @@ public class TorrentDownloaderImpl extends AEThread implements TorrentDownloader
   
   public java.io.File getFile() {
     if ((!this.isAlive()) || (this.file == null))
-      this.file = new File(this.directoryname, this.filename);
+      this.file = new File(this.directoryname, filename);
     return this.file;
   }
 
@@ -620,7 +704,7 @@ public class TorrentDownloaderImpl extends AEThread implements TorrentDownloader
       if (path != null)
         this.directoryname = path;
       if (file != null)
-        this.filename = file;
+        filename = file;
     }
   }
 
@@ -651,4 +735,13 @@ public class TorrentDownloaderImpl extends AEThread implements TorrentDownloader
   public boolean getDeleteFileOnCancel() {
   	return deleteFileOnCancel;
   }
+
+  public boolean isIgnoreReponseCode() {
+		return ignoreReponseCode;
+	}
+
+	public void setIgnoreReponseCode(boolean ignoreReponseCode) {
+		this.ignoreReponseCode = ignoreReponseCode;
+	}
+
 }
