@@ -88,6 +88,10 @@ TRTrackerBTAnnouncerImpl
 	
 	public static String 	UDP_REALM = "UDP Tracker";
 	
+	private static int userMinInterval = 0;
+	private static int userMaxNumwant = 100;
+	private static boolean udpAnnounceEnabled = true;
+
     static{
 	  	PRUDPTrackerCodecs.registerCodecs();
 	  	COConfigurationManager.addAndFireParameterListeners(
@@ -124,11 +128,7 @@ TRTrackerBTAnnouncerImpl
 	private boolean				manual_control;
   
 	private long min_interval = 0;
-	private static int userMinInterval = 0;
-	private static int userMaxNumwant = 100;
-	private static boolean udpAnnounceEnabled = true;
 	
-  
 	private int  failure_added_time = 0;
     private long failure_time_last_updated = 0;
 	
@@ -166,6 +166,8 @@ TRTrackerBTAnnouncerImpl
 	
 	
 	private int announceCount;
+	private int announceFailCount;
+	
 	private byte autoUDPprobeEvery = 1;
 	
   
@@ -474,9 +476,9 @@ TRTrackerBTAnnouncerImpl
 			
 			percentage = 100;
 			
-		}else if ( percentage < 0 ){
+		}else if ( percentage < 50 ){
 			
-			percentage	= 0;
+			percentage	= 50;
 		}
 		
 		long	now = SystemTime.getCurrentTime();
@@ -992,12 +994,16 @@ TRTrackerBTAnnouncerImpl
 			 	}
 			 				
 			 }else{
-			  			  	
+			  		
+				 announceFailCount++;
+				 
 				 last_failure_resp = resp;
 			 }
 			 
 		  }catch( MalformedURLException e ){
 		  	
+			announceFailCount++;
+			
 		  	Debug.printStackTrace( e );
 		  	
 		  	last_failure_resp = 
@@ -1008,8 +1014,10 @@ TRTrackerBTAnnouncerImpl
 						getErrorRetryInterval(), 
 						"malformed URL '" + (request_url==null?"<null>":request_url.toString()) + "'" );
 		  	
-		  }catch( Exception e ){
-		  			  	
+		  }catch( Throwable e ){
+		  		
+			announceFailCount++;
+
 		  	last_failure_resp = 
 		  		new TRTrackerAnnouncerResponseImpl(
 		  				original_url,
@@ -1067,151 +1075,206 @@ TRTrackerBTAnnouncerImpl
 	{
    			// set context in case authentication dialog is required
     	
-    	TorrentUtils.setTLSTorrentHash( torrent_hash );
-    	
- 			// loop to possibly retry update on SSL certificate install
- 		
- 		for (int i=0;i<2;i++){	
- 		
-	  		String	failure_reason = null;
-	  	
-			try{  
+ 		try{
+	    	TorrentUtils.setTLSTorrentHash( torrent_hash );
+	    	
+	 			// loop to possibly retry update on SSL certificate install
+	 		
+	 		for (int i=0;i<2;i++){	
+	 		
+		  		String	failure_reason = null;
+
 				String	protocol = reqUrl.getProtocol();
-				
-				if (Logger.isEnabled()){
-					Logger.log(new LogEvent(torrent, LOGID,
-							"Tracker Announcer is Requesting: " + reqUrl));
-				}
-				
-		  		ByteArrayOutputStream message = new ByteArrayOutputStream();
-		  		
-		  		
-		  		URL udpAnnounceURL = null;
-		  		
-		  		boolean	udp_probe = false;
-		  		
-		  		if (protocol.equalsIgnoreCase("udp") && udpAnnounceEnabled)
-				{
-					udpAnnounceURL = reqUrl;
-				} else if (protocol.equalsIgnoreCase("http") && !az_tracker
-					&& announceCount % autoUDPprobeEvery == 0 && udpAnnounceEnabled)
-				{
-					// if we don't know this tracker supports UDP then don't probe on
-					// first announce as we don't want a large delay on torrent startup
-					// also if we are stopping we don't want to initiate a probe as
-					// we want the stop instruction to get to tracker if possible
-					if ((stopped || announceCount == 0) && !TRTrackerUtils.isUDPProbeOK(reqUrl))
-					{
-						// skip probe
-					} else
-					{
-						udpAnnounceURL = new URL(reqUrl.toString().replaceFirst("^http", "udp"));
-						udp_probe = true;
-					}
-				}
-		  				
-		  		if (udpAnnounceURL != null)
-				{
-					failure_reason = announceUDP(reqUrl, message);
-					if ((failure_reason != null || message.size() == 0) && udp_probe)
-					{
-						// automatic UDP probe failed, use HTTP again
-						udpAnnounceURL = null;
-						if (autoUDPprobeEvery < 16)
-							autoUDPprobeEvery <<= 1;
-						else // unregister in case the tracker somehow changed its capabilities
-							TRTrackerUtils.setUDPProbeResult(reqUrl, false);
-						if (Logger.isEnabled())
-							Logger.log(new LogEvent(torrent, LOGID, LogEvent.LT_INFORMATION, "redirection of http announce [" + tracker_url[0] + "] to udp failed, will retry in " + autoUDPprobeEvery + " announces"));
-						
-					} else if (failure_reason == null && udp_probe)
-					{
-						TRTrackerUtils.setUDPProbeResult(reqUrl, true);
-						if (Logger.isEnabled())
-							Logger.log(new LogEvent(torrent, LOGID, LogEvent.LT_INFORMATION, "redirection of http announce [" + tracker_url[0] + "] to udp successful"));
-						autoUDPprobeEvery = 1;
-					}
-				}
-		  		
-		  		announceCount++;
-		  		
-		  		if ( udpAnnounceURL == null)
-		  			failure_reason = announceHTTP( tracker_url, reqUrl, message );
-
-		  		// if we've got some kind of response then return it
-				if ( message.size() > 0 )
-					return( message.toByteArray());
 
+				try{  
 					
-				if ( failure_reason == null ){
-					failure_reason = "No data received from tracker";
-				}
-				
-	
-			}catch( SSLException e ){
-				
-				// e.printStackTrace();
+					if (Logger.isEnabled()){
+						Logger.log(new LogEvent(torrent, LOGID,
+								"Tracker Announcer is Requesting: " + reqUrl));
+					}
+					
+			  		ByteArrayOutputStream message = new ByteArrayOutputStream();
+			  		
+			  		
+			  		URL udpAnnounceURL = null;
+			  		
+			  		boolean	udp_probe = false;
+			  		
+			  			// if we have multiple tracker URLs then do something sensible about
+			  		
+			  		if ( protocol.equalsIgnoreCase("udp") && udpAnnounceEnabled ){
+			  			
+						udpAnnounceURL = reqUrl;
+						
+					}else if (	protocol.equalsIgnoreCase("http") && 
+								!az_tracker	&& 
+								announceCount % autoUDPprobeEvery == 0 && 
+								udpAnnounceEnabled ){
+						
+							// if we don't know this tracker supports UDP then don't probe on
+							// first announce as we don't want a large delay on torrent startup
+							// also if we are stopping we don't want to initiate a probe as
+							// we want the stop instruction to get to tracker if possible
+						
+						if (	( 	stopped || 
+									announceCount == 0 ||
+									( announceCount < trackerUrlLists.size() && announceFailCount == announceCount )) && 
+								!TRTrackerUtils.isUDPProbeOK(reqUrl)){
+							
+							// skip probe
+							
+						}else{
+							udpAnnounceURL = new URL(reqUrl.toString().replaceFirst("^http", "udp"));
+							
+							udp_probe = true;
+						}
+					}
+			  				
+			  		if ( udpAnnounceURL != null ){
+			  			
+						failure_reason = announceUDP( reqUrl, message, udp_probe );
+						
+						if ((failure_reason != null || message.size() == 0) && udp_probe){
+							
+								// automatic UDP probe failed, use HTTP again
+							
+							udpAnnounceURL = null;
+							
+							if ( autoUDPprobeEvery < 16 ){
+						
+								autoUDPprobeEvery <<= 1;
+							}else{
 								
-					// try and install certificate regardless of error (as this changed in JDK1.5
-					// and broke this...)
-				
-				if ( i == 0 ){//&& e.getMessage().indexOf("No trusted certificate found") != -1 ){
-					
-					if ( SESecurityManager.installServerCertificates( reqUrl ) != null ){
+									// unregister in case the tracker somehow changed its capabilities
 						
-							// certificate has been installed
-						
-						continue;	// retry with new certificate
+								TRTrackerUtils.setUDPProbeResult(reqUrl, false);
+							}
+							
+							if (Logger.isEnabled()){
+								Logger.log(new LogEvent(torrent, LOGID, LogEvent.LT_INFORMATION, "redirection of http announce [" + tracker_url[0] + "] to udp failed, will retry in " + autoUDPprobeEvery + " announces"));
+							}	
+						}else if (failure_reason == null && udp_probe){
+							
+							TRTrackerUtils.setUDPProbeResult(reqUrl, true);
+							
+							if (Logger.isEnabled()){
+								Logger.log(new LogEvent(torrent, LOGID, LogEvent.LT_INFORMATION, "redirection of http announce [" + tracker_url[0] + "] to udp successful"));
+							}
+							
+							autoUDPprobeEvery = 1;
+						}
+					}
+			  		
+			  		announceCount++;
+			  		
+			  		if ( udpAnnounceURL == null){
+			  			
+			  			failure_reason = announceHTTP( tracker_url, reqUrl, message );
+			  		}
+	
+			  			// if we've got some kind of response then return it
+			  		
+					if ( message.size() > 0 ){
 						
+						return( message.toByteArray());
 					}
 						
-					failure_reason = exceptionToString( e );
+					if ( failure_reason == null ){
+						
+						failure_reason = "No data received from tracker";
+					}
+				}catch( SSLException e ){
 					
-				}else{
+					// e.printStackTrace();
+									
+						// try and install certificate regardless of error (as this changed in JDK1.5
+						// and broke this...)
 					
-					failure_reason = exceptionToString( e );
+					if ( i == 0 ){//&& e.getMessage().indexOf("No trusted certificate found") != -1 ){
+						
+						if ( SESecurityManager.installServerCertificates( reqUrl ) != null ){
+							
+								// certificate has been installed
+							
+							continue;	// retry with new certificate
+							
+						}
+							
+						failure_reason = exceptionToString( e );
+						
+					}else{
+						
+						failure_reason = exceptionToString( e );
+						
+					}
+				}catch( IOException e ){
 					
+		     		if ( i == 0 && protocol.toLowerCase().startsWith( "http" )){
+		      			
+		      			URL retry_url = UrlUtils.getIPV4Fallback( reqUrl );
+		      			
+		      			if ( retry_url != null ){
+		      				
+		      				reqUrl = retry_url;
+		      				
+		      				continue;
+		      			}
+		     		}
+		     		
+		     		failure_reason = exceptionToString( e );
+		     		
+				}catch (Exception e){
+			  
+			  		// e.printStackTrace();
+			  
+			  		failure_reason = exceptionToString( e );
 				}
-			}catch (Exception e){
-		  
-		  		// e.printStackTrace();
-		  
-		  		failure_reason = exceptionToString( e );
-			}
-				
-			if ( failure_reason.indexOf("401" ) != -1 ){
 					
-				failure_reason = "Tracker authentication failed";
-			}
-		
-			if (Logger.isEnabled())
-				Logger.log(new LogEvent(torrent, LOGID, LogEvent.LT_ERROR,
-						"Exception while processing the Tracker Request for " + reqUrl + ": "
-								+ failure_reason));
+				if ( failure_reason != null && failure_reason.indexOf("401" ) != -1 ){
+						
+					failure_reason = "Tracker authentication failed";
+				}
 			
-			throw( new Exception( failure_reason));
- 		}
- 		
- 			// should never get here as second iteration of loop will always cause an exit
- 		
- 		throw( new Exception( "Internal Error: should never get here" ));
+				if (Logger.isEnabled())
+					Logger.log(new LogEvent(torrent, LOGID, LogEvent.LT_ERROR,
+							"Exception while processing the Tracker Request for " + reqUrl + ": "
+									+ failure_reason));
+				
+				throw( new Exception( failure_reason));
+	 		}
+	 		
+	 			// should never get here as second iteration of loop will always cause an exit
+	 		
+	 		throw( new Exception( "Internal Error: should never get here" ));
+	 		
+		}finally{
+			
+			TorrentUtils.setTLSTorrentHash( null );
+		}
   	}
   
  	
  	protected String
  	announceHTTP(
  		URL[]					tracker_url,	// overwritten if redirected
- 		URL						reqUrl,
+ 		URL						original_reqUrl,
  		ByteArrayOutputStream	message )
  	
  		throws IOException
  	{ 		
- 		TRTrackerUtils.checkForBlacklistedURLs( reqUrl );
+ 		TRTrackerUtils.checkForBlacklistedURLs( original_reqUrl );
  		
- 		reqUrl = TRTrackerUtils.adjustURLForHosting( reqUrl );
+ 		URL reqUrl = TRTrackerUtils.adjustURLForHosting( original_reqUrl );
  		
  		reqUrl = AddressUtils.adjustURL( reqUrl );
  		
+ 		if ( reqUrl != original_reqUrl ){
+			if (Logger.isEnabled()){
+				Logger.log(new LogEvent(torrent, LOGID,
+						"    HTTP: url adjusted to " + reqUrl ));
+			}
+ 		}
+ 		
  		String	failure_reason = null;
  		
  		HttpURLConnection con;
@@ -1411,20 +1474,29 @@ TRTrackerBTAnnouncerImpl
  	
  	protected String
  	announceUDP(
- 		URL						reqUrl,
-		ByteArrayOutputStream	message )
+ 		URL						original_reqUrl,
+		ByteArrayOutputStream	message,
+		boolean                 is_probe )
  	
  		throws IOException
  	{
- 		reqUrl = TRTrackerUtils.adjustURLForHosting( reqUrl );
+ 		long timeout = is_probe?10000:PRUDPPacket.DEFAULT_UDP_TIMEOUT;
+ 		
+ 		URL reqUrl = TRTrackerUtils.adjustURLForHosting( original_reqUrl );
 
+ 		if ( reqUrl != original_reqUrl ){
+			if (Logger.isEnabled()){
+				Logger.log(new LogEvent(torrent, LOGID,
+						"    UDP: url adjusted to " + reqUrl ));
+			}
+ 		}
+ 		
  		String	failure_reason = null;
 		
  		PasswordAuthentication	auth = null;	
  		
  		try{
- 			if ( reqUrl.getQuery().toLowerCase().indexOf("auth") != -1 ){
- 				
+ 			if ( (!is_probe) && UrlUtils.queryHasParameter(reqUrl.getQuery(), "auth", false)) {
  				 auth = SESecurityManager.getPasswordAuthentication( UDP_REALM, reqUrl );
  			}
  						
@@ -1438,7 +1510,7 @@ TRTrackerBTAnnouncerImpl
  			
 		 			PRUDPPacket connect_request = new PRUDPPacketRequestConnect();
 		 			
-		 			PRUDPPacket reply = handler.sendAndReceive( auth, connect_request, destination );
+		 			PRUDPPacket reply = handler.sendAndReceive( auth, connect_request, destination, timeout );
 		 			
 		 			if ( reply.getAction() == PRUDPPacketTracker.ACT_REPLY_CONNECT ){
 		 			
@@ -2407,6 +2479,11 @@ TRTrackerBTAnnouncerImpl
 
 						int peers_length = meta_peers.size();
 
+				  	 if (Logger.isEnabled()) {
+								Logger.log(new LogEvent(torrent, LOGID,
+										"ANNOUNCE CompactPeers2: num=" + peers_length));
+					   }
+
 						if ( peers_length > 1 ){
 								// calculate average rtt to use for those with no rtt
 		 					 
@@ -2700,7 +2777,12 @@ TRTrackerBTAnnouncerImpl
 							//for every peer
 						
 						int peers_length = meta_peers.size();
-							
+
+				  	 if (Logger.isEnabled()) {
+								Logger.log(new LogEvent(torrent, LOGID,
+										"ANNOUNCE old style non-compact: num=" + peers_length));
+					   }
+						
 						if ( crypto_flags != null && peers_length != crypto_flags.length ){
 							
 							crypto_flags = null;
@@ -2816,6 +2898,12 @@ TRTrackerBTAnnouncerImpl
 
 				    	int	peer_number = 0;
 				    	
+					  	 if (Logger.isEnabled()) {
+									Logger.log(new LogEvent(torrent, LOGID,
+											"ANNOUNCE CompactPeers: num=" + (meta_peers.length/entry_size)));
+						   }
+
+				    	
 				    	for (int i=0;i<meta_peers.length;i+=entry_size){
 				    		
 				    		peer_number++;
@@ -2897,36 +2985,55 @@ TRTrackerBTAnnouncerImpl
 							throw( new IOException( "peers missing from response" ));
 				    	}
 				    }else if (!metaData.containsKey("peers6")){ 
-				    	// we got nothing useful under peers and no peers6 either
+				    	
+				    		// 	we got nothing useful under peers and no peers6 either
+				    	
 						throw( new IOException( "peers missing from response" ));
 				    }
 			    	
 			    	
 			    	final byte[] v6peers = (byte[])metaData.get("peers6");
-				    if (v6peers != null)
-					{
-				    	// 16 bytes for v6 + 2 bytes for port
+			    	
+				    if ( v6peers != null ){
+				    	
+				    		// 16 bytes for v6 + 2 bytes for port
+				    	
 						final int entry_size = 18;
-
-						
+		
 						final byte[] rawAddr = new byte[16]; 
 						
-						for (int i = 0; i <= v6peers.length - entry_size; i += entry_size)
-						{
-							System.arraycopy(v6peers, i, rawAddr, 0, 16);
+						for (int i=0; i<v6peers.length; i+=entry_size ){
+							
+							System.arraycopy( v6peers, i, rawAddr, 0, 16 );
+							
 							String ip = InetAddress.getByAddress(rawAddr).getHostAddress();
-							int tcp_port = rawAddr[i+16] * 256 + rawAddr[i+17];
-							if (tcp_port < 0 || tcp_port > 65535)
-							{
-								if (Logger.isEnabled())
+							
+				    		int	po1 = 0xFF & v6peers[i+16];
+				    		int	po2 = 0xFF & v6peers[i+17];
+
+							int tcp_port = po1*256 + po2;
+							
+							if (tcp_port < 0 || tcp_port > 65535){
+							
+								if (Logger.isEnabled()){
+									
 									Logger.log(new LogEvent(torrent, LOGID, LogEvent.LT_ERROR, "Invalid compactv6 peer port given: " + ip + ": " + tcp_port));
+								}
+								
 								continue;
 							}
-							byte[] peer_peer_id = getAnonymousPeerId(ip, tcp_port);
+							
+							byte[] peer_peer_id = getAnonymousPeerId( ip, tcp_port );
+							
 							short protocol = DownloadAnnounceResultPeer.PROTOCOL_NORMAL;
+							
 							TRTrackerAnnouncerResponsePeerImpl peer = new TRTrackerAnnouncerResponsePeerImpl(PEPeerSource.PS_BT_TRACKER, peer_peer_id, ip, tcp_port, 0, 0, protocol, TRTrackerAnnouncer.AZ_TRACKER_VERSION_1, (short) 0);
-							if (Logger.isEnabled())
+							
+							if (Logger.isEnabled()){
+								
 								Logger.log(new LogEvent(torrent, LOGID, "COMPACTv6 PEER: " + peer.getString()));
+							}
+							
 							valid_meta_peers.add(peer);
 						}
 					}
@@ -3207,7 +3314,8 @@ TRTrackerBTAnnouncerImpl
 			
 			List	l_peers = new ArrayList(ext_peers.length);
 				
-			boolean ps_enabled = announce_data_provider.isPeerSourceEnabled( PEPeerSource.PS_BT_TRACKER );
+			boolean ps_enabled = 	announce_data_provider != null && 
+									announce_data_provider.isPeerSourceEnabled( PEPeerSource.PS_BT_TRACKER );
 						
 			for (int i=0;i<ext_peers.length;i++){
 				
