@@ -231,6 +231,15 @@ TRHostExternalTorrent
 		return( hash_wrapper );
 	}
 	
+   	public void 
+	setHashOverride(
+		byte[] hash ) 
+	
+		throws TOTorrentException 
+	{
+		throw( new TOTorrentException( "Not supported", TOTorrentException.RT_HASH_FAILS ));
+	}
+   	
 	public boolean
 	getPrivate()
 	{
