@@ -22,9 +22,13 @@
 
 package org.gudy.azureus2.core3.util;
 
-import java.util.*;
 import java.io.*;
-import java.nio.*;
+import java.nio.ByteBuffer;
+import java.nio.CharBuffer;
+import java.util.ArrayList;
+import java.util.Iterator;
+import java.util.List;
+import java.util.Map;
 
 /**
  * A set of utility methods to decode a bencoded array of byte into a Map.
@@ -35,8 +39,13 @@ import java.nio.*;
  */
 public class BDecoder 
 {
-	private boolean recovery_mode;
+	private static final int MAX_BYTE_ARRAY_SIZE	= 8*1024*1024;
+	private static final int MAX_MAP_KEY_SIZE		= 64*1024;
 	
+	private static final boolean TRACE	= false;
+	
+	private boolean recovery_mode;
+
 	public static Map
 	decode(
 		byte[]	data )
@@ -91,14 +100,24 @@ public class BDecoder
 	{ 
 		return( decode(new BDecoderInputStreamArray(data, offset, length )));
 	}
-	
+
 	public Map 
 	decodeStream(
 		BufferedInputStream data )  
 
 		throws IOException 
 	{
-		Object	res = decodeInputStream(new BDecoderInputStreamStream(data), 0);
+		return decodeStream(data, true);
+	}
+
+	public Map 
+	decodeStream(
+		BufferedInputStream data,
+		boolean internKeys)  
+
+		throws IOException 
+	{
+		Object	res = decodeInputStream(data, 0, internKeys);
 
 		if ( res == null ){
 
@@ -114,11 +133,11 @@ public class BDecoder
 
 	private Map 
 	decode(
-		BDecoderInputStream data ) 
+		InputStream data ) 
 
 		throws IOException 
 	{
-		Object res = decodeInputStream(data, 0);
+		Object res = decodeInputStream(data, 0, true);
 
 		if ( res == null ){
 
@@ -134,8 +153,9 @@ public class BDecoder
 
 	private Object 
 	decodeInputStream(
-		BDecoderInputStream dbis,
-		int			nesting ) 
+		InputStream dbis,
+		int			nesting,
+		boolean internKeys) 
 
 		throws IOException 
 	{
@@ -165,11 +185,19 @@ public class BDecoder
 				
 				byte[] tempByteArray = null;
 
-				while ((tempByteArray = (byte[]) decodeInputStream(dbis, nesting+1)) != null) {
+				while ((tempByteArray = (byte[]) decodeInputStream(dbis, nesting+1,internKeys)) != null) {
 
+					if ( tempByteArray.length > MAX_MAP_KEY_SIZE ){
+						
+						String msg = "dictionary key is too large, max=" + MAX_MAP_KEY_SIZE + ": value=" + new String( tempByteArray, 0, 128 );
+						
+						System.err.println( msg );
+						
+						throw( new IOException( msg ));
+					}
 						//decode some more
 
-					Object value = decodeInputStream(dbis,nesting+1);
+					Object value = decodeInputStream(dbis,nesting+1,internKeys);
 					
 						// value interning is too CPU-intensive, let's skip that for now
 						//if(value instanceof byte[] && ((byte[])value).length < 17)
@@ -186,10 +214,32 @@ public class BDecoder
 
 						key = new String(cb.array(),0,cb.limit());
 
-						key = StringInterner.intern( key );
+						if (internKeys) {
+							key = StringInterner.intern( key );
+						}
 					}
 
-					tempMap.put( key, value);
+					if ( TRACE ){
+						System.out.println( key + "->" + value + ";" );
+					}
+					
+						// recover from some borked encodings that I have seen whereby the value has
+						// not been encoded. This results in, for example, 
+						// 18:azureus_propertiesd0:e
+						// we only get null back here if decoding has hit an 'e' or end-of-file
+						// that is, there is no valid way for us to get a null 'value' here
+					
+					if ( value == null ){
+						
+						System.err.println( "Invalid encoding - value not serialsied for '" + key + "' - ignoring" );
+						
+						break;
+					}
+				
+					if ( tempMap.put( key, value) != null ){
+						
+						Debug.out( "BDecoder: key '" + key + "' already exists!" );
+					}
 				}
 
 				/*	
@@ -235,7 +285,7 @@ public class BDecoder
 					//create the key
 				
 				Object tempElement = null;
-				while ((tempElement = decodeInputStream(dbis, nesting+1)) != null) {
+				while ((tempElement = decodeInputStream(dbis, nesting+1, internKeys)) != null) {
 						//add the element
 					tempList.add(tempElement);
 				}
@@ -332,23 +382,27 @@ public class BDecoder
   }
 	 */
 
+	/** only create the array once per decoder instance (no issues with recursion as it's only used in a leaf method)
+	 */
+	private final char[] numberChars = new char[32];
+	
 	private long 
 	getNumberFromStream(
-		BDecoderInputStream 	dbis, 
+		InputStream 	dbis, 
 		char 					parseChar) 
 
 		throws IOException 
 	{
-		final char[]	chars = new char[32];
+		
 
 		int tempByte = dbis.read();
 
 		int pos = 0;
 
 		while ((tempByte != parseChar) && (tempByte >= 0)) {
-			chars[pos++] = (char)tempByte;
-			if ( pos == chars.length ){
-				throw( new NumberFormatException( "Number too large: " + new String(chars,0,pos) + "..." ));
+			numberChars[pos++] = (char)tempByte;
+			if ( pos == numberChars.length ){
+				throw( new NumberFormatException( "Number too large: " + new String(numberChars,0,pos) + "..." ));
 			}
 			tempByte = dbis.read();
 		}
@@ -365,25 +419,31 @@ public class BDecoder
 			return(0);
 		}
 
-		return( parseLong( chars, 0, pos ));
+		return( parseLong( numberChars, 0, pos ));
 	}
 
+	// This is similar to Long.parseLong(String) source
 	public static long
 	parseLong(
 		char[]	chars,
 		int		start,
 		int		length )
 	{
-		long result = 0;
-
-		boolean negative = false;
-
-		int 	i 	= start;
-		int	max = start + length;
-
-		long limit;
-
 		if ( length > 0 ){
+			// Short Circuit: We don't support octal parsing, so if it 
+			// starts with 0, it's 0
+			if (chars[start] == '0') {
+
+				return 0;
+			}
+
+			long result = 0;
+
+			boolean negative = false;
+
+			int 	i 	= start;
+
+			long limit;
 
 			if ( chars[i] == '-' ){
 
@@ -394,10 +454,27 @@ public class BDecoder
 				i++;
 
 			}else{
+				// Short Circuit: If we are only processing one char,
+				// and it wasn't a '-', just return that digit instead
+				// of doing the negative junk
+				if (length == 1) {
+					int digit = chars[i] - '0';
+
+					if ( digit < 0 || digit > 9 ){
+
+						throw new NumberFormatException(new String(chars,start,length));
+
+					}else{
+
+						return digit;
+					}
+				}
 
 				limit = -Long.MAX_VALUE;
 			}
 
+			int	max = start + length;
+
 			if ( i < max ){
 
 				int digit = chars[i++] - '0';
@@ -439,25 +516,26 @@ public class BDecoder
 
 				result -= digit;
 			}
+
+			if ( negative ){
+
+				if ( i > start+1 ){
+
+					return result;
+
+				}else{	/* Only got "-" */
+
+					throw new NumberFormatException(new String(chars,start,length));
+				}
+			}else{
+
+				return -result;
+			}  
 		}else{
 
 			throw new NumberFormatException(new String(chars,start,length));
 		}
 
-		if ( negative ){
-
-			if ( i > start+1 ){
-
-				return result;
-
-			}else{	/* Only got "-" */
-
-				throw new NumberFormatException(new String(chars,start,length));
-			}
-		}else{
-
-			return -result;
-		}  
 	}
 
 
@@ -509,7 +587,7 @@ public class BDecoder
 
 	private byte[] 
 	getByteArrayFromStream(
-		BDecoderInputStream dbis )
+		InputStream dbis )
 		
 		throws IOException 
 	{
@@ -522,7 +600,7 @@ public class BDecoder
 		// note that torrent hashes can be big (consider a 55GB file with 2MB pieces
 		// this generates a pieces hash of 1/2 meg
 
-		if ( length > 8*1024*1024 ){
+		if ( length > MAX_BYTE_ARRAY_SIZE ){
 
 			throw( new IOException( "Byte array length too large (" + length + ")"));
 		}
@@ -549,6 +627,21 @@ public class BDecoder
 		recovery_mode	= r;
 	}
 
+	public static void
+	print(
+		Object		obj )
+	{
+		StringWriter 	sw = new StringWriter();
+		
+		PrintWriter		pw = new PrintWriter( sw );
+		
+		print( pw, obj );
+		
+		pw.flush();
+		
+		System.out.println( sw.toString());
+	}
+	
 	public static void
 	print(
 		PrintWriter	writer,
@@ -577,7 +670,7 @@ public class BDecoder
 			if ( b.length==20 ){
 				writer.println( use_indent + " { "+ ByteFormatter.nicePrint( b )+ " }" );
 			}else if ( b.length < 64 ){
-				writer.println( new String(b) );
+				writer.println( new String(b) + " [" + ByteFormatter.encodeString( b ) + "]" );
 			}else{
 				writer.println( "[byte array length " + b.length );
 			}
@@ -652,7 +745,7 @@ public class BDecoder
 
 				}catch( Throwable e ){
 
-					Debug.printStackTrace(e);
+					System.err.println(e);
 				}
 			}else if ( value instanceof Map ){
 
@@ -688,7 +781,7 @@ public class BDecoder
 
 				}catch( Throwable e ){
 
-					Debug.printStackTrace(e);
+					System.err.println(e);
 				}
 			}else if ( value instanceof Map ){
 
@@ -724,7 +817,7 @@ public class BDecoder
 			e.printStackTrace();
 		}
 	}
-
+/*
 	private interface
 	BDecoderInputStream
 	{
@@ -836,11 +929,11 @@ public class BDecoder
 			is.reset();
 		}
 	}
-
+*/
 	private class
 	BDecoderInputStreamArray
 	
-		implements BDecoderInputStream
+		extends InputStream
 	{
 		final private byte[]		buffer;
 		final private int			count;
@@ -950,7 +1043,7 @@ public class BDecoder
 	main(
 			String[]	args )
 	{	  
-		print( 	new File( "C:\\Temp\\xxx.torrent" ),
-				new File( "C:\\Temp\\xxx.txt" ));
+		print( 	new File( "C:\\Temp\\tables.config" ),
+				new File( "C:\\Temp\\tables.txt" ));
 	}
 }
