@@ -29,6 +29,7 @@ package org.gudy.azureus2.core3.util;
 import java.util.Arrays;
 import java.util.Calendar;
 import java.util.Date;
+import java.text.DecimalFormatSymbols;
 import java.text.SimpleDateFormat;
 import java.text.NumberFormat;
 
@@ -81,6 +82,7 @@ DisplayFormatters
     
     private static boolean	separate_prot_data_stats;
     private static boolean	data_stats_only;
+		private static char decimalSeparator;
     
 	static{
 		use_si_units = COConfigurationManager.getBooleanParameter("config.style.useSIUnits");
@@ -240,6 +242,8 @@ DisplayFormatters
 	percentage_format = NumberFormat.getPercentInstance();
 	percentage_format.setMinimumFractionDigits(1);
 	percentage_format.setMaximumFractionDigits(1);
+	
+		 decimalSeparator = new DecimalFormatSymbols().getDecimalSeparator();
    }
   
 	private static String
@@ -270,7 +274,9 @@ DisplayFormatters
 	private static String	ManagerItem_queued;
 	private static String	ManagerItem_error;
 	private static String	ManagerItem_forced;
-
+	private static String	yes;
+	private static String	no;
+	
 	public static void
 	loadMessages()
 	{
@@ -293,6 +299,8 @@ DisplayFormatters
 		ManagerItem_queued				= getResourceString( "ManagerItem.queued", "queued" );
 		ManagerItem_error				= getResourceString( "ManagerItem.error", "error" );
 		ManagerItem_forced				= getResourceString( "ManagerItem.forced", "forced" );
+		yes								= getResourceString( "GeneralView.yes", "Yes" );
+		no								= getResourceString( "GeneralView.no", "No" );
 	}
 	
 	private static String
@@ -326,6 +334,13 @@ DisplayFormatters
 		}
 	}
 
+	public static String
+	getYesNo(
+		boolean	b )
+	{
+		return( b?yes:no );
+	}
+	
 	public static String
 	getRateUnit(
 		int		unit_size )
@@ -349,6 +364,12 @@ DisplayFormatters
 		return units_base10[unit_size];
 	}
 
+	public static boolean
+	isRateUsingBits()
+	{
+		return( use_units_rate_bits );
+	}
+	
 	public static String
 	formatByteCountToKiBEtc(int n)
 	{
@@ -645,7 +666,7 @@ DisplayFormatters
 				tmp = ManagerItem_downloading;
 				break;
 
-			case DownloadManager.STATE_SEEDING:
+			case DownloadManager.STATE_SEEDING:{
 
 				DiskManager diskManager = manager.getDiskManager();
 
@@ -668,7 +689,7 @@ DisplayFormatters
 					tmp = ManagerItem_seeding;
 				}
 				break;
-
+			}
 			case DownloadManager.STATE_STOPPED:
 				tmp = manager.isPaused() ? ManagerItem_paused : ManagerItem_stopped;
 				break;
@@ -689,10 +710,14 @@ DisplayFormatters
 				tmp = ManagerItem_initializing;
 				break;
 
-			case DownloadManager.STATE_ALLOCATING:
+			case DownloadManager.STATE_ALLOCATING:{
 				tmp = ManagerItem_allocating;
+				DiskManager diskManager = manager.getDiskManager();
+				if (diskManager != null){		
+					tmp += ": " + formatPercentFromThousands( diskManager.getPercentDone());
+				}
 				break;
-
+			}
 			case DownloadManager.STATE_CHECKING:
 				tmp = ManagerItem_checking + ": "
 						+ formatPercentFromThousands(manager.getStats().getCompleted());
@@ -795,6 +820,32 @@ DisplayFormatters
 		return( tmp );
 	}
 
+	public static String
+	trimDigits(
+		String		str,
+		int			num_digits )
+	{
+		char[] 	chars 	= str.toCharArray();
+		String 	res 	= "";
+		int		digits 	= 0;
+		
+		for (int i=0;i<chars.length;i++){
+			char c = chars[i];
+			if ( Character.isDigit(c)){
+				digits++;
+				if ( digits <= num_digits ){
+					res += c;
+				}
+			}else if ( c == '.' && digits >= 3 ){
+									
+			}else{
+				res += c;
+			}
+		}
+		
+		return( res );
+	}
+	
   public static String formatPercentFromThousands(int thousands) {
  
     return percentage_format.format(thousands / 1000.0);
@@ -1046,4 +1097,8 @@ DisplayFormatters
 			System.out.println("123456:" + DisplayFormatters.formatDecimal(123456.999, 0));
 			System.out.println(DisplayFormatters.formatDecimal(0.0/0, 3));
 		}
+
+	public static char getDecimalSeparator() {
+		return decimalSeparator;
+	}
 }
\ No newline at end of file
