@@ -24,6 +24,7 @@ package org.gudy.azureus2.core3.util;
 
 import java.io.*;
 import java.lang.reflect.Method;
+import java.net.SocketTimeoutException;
 import java.net.URI;
 import java.net.URL;
 import java.util.*;
@@ -490,6 +491,18 @@ public class FileUtil {
   	}
   }
   
+  	public static boolean
+  	resilientConfigFileExists(
+  		String		name )
+  	{
+ 		File parent_dir = new File(SystemProperties.getUserPath());
+
+ 		boolean use_backups = COConfigurationManager.getBooleanParameter("Use Config File Backups" );
+
+ 		return( new File( parent_dir, name ).exists() ||
+ 				( use_backups && new File( parent_dir, name + ".bak" ).exists()));
+  	}
+  
 	public static Map
 	readResilientConfigFile(
 		String		file_name )
@@ -527,7 +540,7 @@ public class FileUtil {
 	readResilientFile(
 		File		file )
 	{
-		return( readResilientFile( file.getParentFile(),file.getName(),false));
+		return( readResilientFile( file.getParentFile(),file.getName(),false, true));
 	}
 	
  	public static Map
@@ -535,6 +548,16 @@ public class FileUtil {
 		File		parent_dir,
 		String		file_name,
 		boolean		use_backup )
+ 	{
+ 		return readResilientFile(parent_dir, file_name, use_backup, true);
+ 	}
+ 	
+ 	public static Map
+	readResilientFile(
+		File		parent_dir,
+		String		file_name,
+		boolean		use_backup,
+		boolean		intern_keys )
  	{
 		File	backup_file = new File( parent_dir, file_name + ".bak" );
 		 
@@ -546,13 +569,13 @@ public class FileUtil {
  			// if we've got a backup, don't attempt recovery here as the .bak file may be
  			// fully OK
  		
- 		Map	res = readResilientFileSupport( parent_dir, file_name, !use_backup );
+ 		Map	res = readResilientFileSupport( parent_dir, file_name, !use_backup, intern_keys );
  		
  		if ( res == null && use_backup ){
  				
  				// try backup without recovery
  			
- 		 	res = readResilientFileSupport( parent_dir, file_name + ".bak", false );
+ 		 	res = readResilientFileSupport( parent_dir, file_name + ".bak", false, intern_keys );
  		 		
 	 		if ( res != null ){
 	 			
@@ -569,7 +592,7 @@ public class FileUtil {
 	 			
 	 				// neither main nor backup file ok, retry main file with recovery
 	 			
-	 			res = readResilientFileSupport( parent_dir, file_name, true );
+	 			res = readResilientFileSupport( parent_dir, file_name, true, true );
 	 		}
  		}
  		
@@ -587,7 +610,8 @@ public class FileUtil {
 	readResilientFileSupport(
 		File		parent_dir,
 		String		file_name,
-		boolean		attempt_recovery )
+		boolean		attempt_recovery,
+		boolean		intern_keys )
 	{
    		try{
   			class_mon.enter();
@@ -598,7 +622,7 @@ public class FileUtil {
 	  			Map	res = null;
 	  			
 	  			try{
-	  				res = readResilientFile( file_name, parent_dir, file_name, 0, false );
+	  				res = readResilientFile( file_name, parent_dir, file_name, 0, false, intern_keys );
 	  			
 	  			}catch( Throwable e ){
 	  				
@@ -607,7 +631,7 @@ public class FileUtil {
 	  			
 	  			if ( res == null && attempt_recovery ){
 	  				
-	  				res = readResilientFile( file_name, parent_dir, file_name, 0, true );
+	  				res = readResilientFile( file_name, parent_dir, file_name, 0, true, intern_keys );
 	  				
 	  				if ( res != null ){
 	  					Logger.log(new LogAlert(LogAlert.UNREPEATABLE, LogAlert.AT_WARNING,
@@ -640,7 +664,8 @@ public class FileUtil {
 		File		parent_dir,
 		String		file_name,
 		int			fail_count,
-		boolean		recovery_mode )
+		boolean		recovery_mode,
+		boolean		skip_key_intern)
 	{	  
 			// logging in here is only done during "non-recovery" mode to prevent subsequent recovery
 			// attempts logging everything a second time.
@@ -687,7 +712,7 @@ public class FileUtil {
 //								+ file_name + "' failed, " + "file not found or 0-sized."));
   			}
   			
-  			return( readResilientFile( original_file_name, parent_dir, file_name + ".saving", 0, recovery_mode ));
+  			return( readResilientFile( original_file_name, parent_dir, file_name + ".saving", 0, recovery_mode, true ));
   		}
 
   		BufferedInputStream bin = null;
@@ -723,7 +748,7 @@ public class FileUtil {
   				decoder.setRecoveryMode( true );
   			}
   			
-	    	Map	res = decoder.decodeStream(bin);
+	    	Map	res = decoder.decodeStream(bin, !skip_key_intern);
 	    	
 	    	if ( using_backup && !recovery_mode ){
   		
@@ -795,7 +820,7 @@ public class FileUtil {
 	    		return( null );
 	    	}
 	    	
-	    	return( readResilientFile( original_file_name, parent_dir, file_name + ".saving", 1, recovery_mode ));
+	    	return( readResilientFile( original_file_name, parent_dir, file_name + ".saving", 1, recovery_mode, true ));
  			 
 	    }finally{
 	    	
@@ -812,7 +837,24 @@ public class FileUtil {
 	    }
 	}
   
-
+	public static void
+	deleteResilientFile(
+		File		file )
+	{
+		file.delete();
+		new File( file.getParentFile(), file.getName() + ".bak" ).delete();
+	}
+	
+	public static void
+	deleteResilientConfigFile(
+		String		name )
+	{
+		File parent_dir = new File(SystemProperties.getUserPath());
+		
+		new File( parent_dir, name ).delete();
+		new File( parent_dir, name + ".bak" ).delete();
+	}
+	
 	private static void
 	getReservedFileHandles()
 	{
@@ -956,6 +998,10 @@ public class FileUtil {
       }
     }
     
+    public static void copyFileWithException( final File _source, final File _dest ) throws IOException{
+         copyFile( new FileInputStream( _source ), new FileOutputStream( _dest ) );
+    }
+    
     public static boolean copyFile( final File _source, final OutputStream _dest, boolean closeInputStream ) {
         try {
           copyFile( new FileInputStream( _source ), _dest, closeInputStream );
@@ -1010,6 +1056,42 @@ public class FileUtil {
     	}
     }
     
+    public static void 
+    copyFile( 
+    	final InputStream 	_source, 
+    	final File 			_dest,
+    	boolean				_close_input_stream )
+    
+    	throws IOException
+    {
+    	FileOutputStream	dest = null;
+        	
+    	boolean	close_input = _close_input_stream;
+    	
+    	try{
+    		dest = new FileOutputStream(_dest);
+       		
+    		close_input = false;
+    		
+    		copyFile( _source, dest, close_input );
+    		
+    	}finally{
+    		
+       		try{
+    			if( close_input ){
+    				
+    				_source.close();
+    			}
+    		}catch( IOException e ){
+     		}
+    		
+    		if ( dest != null ){
+    			
+    			dest.close();
+    		}
+    	}
+    }
+    
     public static void 
     copyFile( 
       InputStream   is,
@@ -1451,20 +1533,43 @@ public class FileUtil {
     
     
     
-    public static void writeBytesAsFile( String filename, byte[] file_data ) {
-      try{
-        File file = new File( filename );
-        
-        FileOutputStream out = new FileOutputStream( file );
-        
-        out.write( file_data );
-        
-        out.close();
-      }
-      catch( Throwable t ) {
-        Debug.out( "writeBytesAsFile:: error: ", t );
-      }
+    public static void 
+    writeBytesAsFile( 
+    	String filename, 
+    	byte[] file_data ) 
+    {
+    		// pftt, this is used by emp so can't fix signature to make more useful
+    	
+    	writeBytesAsFile2( filename, file_data );
+    }
+    
+    public static boolean 
+    writeBytesAsFile2( 
+    	String filename, 
+    	byte[] file_data ) 
+    {
+    	try{
+    		File file = new File( filename );
 
+    		if ( !file.getParentFile().exists()){
+    			
+    			file.getParentFile().mkdirs();
+    		}
+    		
+    		FileOutputStream out = new FileOutputStream( file );
+
+    		out.write( file_data );
+
+    		out.close();
+    		
+    		return( true );
+    		
+    	}catch( Throwable t ){
+    		
+    		Debug.out( "writeBytesAsFile:: error: ", t );
+    	
+    		return( false );
+    	}
     }
     
 	public static boolean
@@ -1623,6 +1728,42 @@ public class FileUtil {
 		return (result.toString());
 	}
 
+	public static String
+	readInputStreamAsStringWithTruncation(
+		InputStream 	is,
+		int				size_limit )
+	
+		throws IOException
+	{
+		StringBuffer result = new StringBuffer(1024);
+
+		byte[] buffer = new byte[1024];
+
+		try{
+			while (true) {
+	
+				int len = is.read(buffer);
+	
+				if (len <= 0) {
+	
+					break;
+				}
+	
+				result.append(new String(buffer, 0, len, "ISO-8859-1"));
+	
+				if (size_limit >= 0 && result.length() > size_limit) {
+	
+					result.setLength(size_limit);
+	
+					break;
+				}
+			}
+		}catch( SocketTimeoutException e ){
+		}
+
+		return (result.toString());
+	}
+	
 	public static String
 	readFileEndAsString(
 		File	file,
@@ -1668,6 +1809,64 @@ public class FileUtil {
 		}
 	}
 	
+	public static byte[]
+	readInputStreamAsByteArray(
+		InputStream		is )
+	
+		throws IOException
+	{
+		ByteArrayOutputStream	baos = new ByteArrayOutputStream(32*1024);
+		
+		byte[]	buffer = new byte[32*1024];
+		
+		while( true ){
+			
+			int	len = is.read( buffer );
+			
+			if ( len <= 0 ){
+				
+				break;
+			}
+			
+			baos.write( buffer, 0, len );
+		}
+		
+		return( baos.toByteArray());
+	}
+	
+	public static byte[]
+   	readFileAsByteArray(
+   		File		file )
+   	
+   		throws IOException
+   	{
+   		ByteArrayOutputStream	baos = new ByteArrayOutputStream((int)file.length());
+   		
+   		byte[]	buffer = new byte[32*1024];
+   		
+   		InputStream is = new FileInputStream( file );
+   		
+   		try{
+	   		while( true ){
+	   			
+	   			int	len = is.read( buffer );
+	   			
+	   			if ( len <= 0 ){
+	   				
+	   				break;
+	   			}
+	   			
+	   			baos.write( buffer, 0, len );
+	   		}
+	   		
+	   		return( baos.toByteArray());
+	   		
+   		}finally{
+   			
+   			is.close();
+   		}
+   	}
+	
 	public final static boolean getUsableSpaceSupported()
 	{
 		return reflectOnUsableSpace != null;
