@@ -33,6 +33,7 @@ package org.gudy.azureus2.core3.util;
  * It has been introduced to reduce the likelyhood of such deadlocks
  */
 
+import java.lang.reflect.Method;
 import java.util.*;
 
 import org.gudy.azureus2.core3.logging.LogEvent;
@@ -41,43 +42,45 @@ import org.gudy.azureus2.core3.logging.Logger;
 
 
 public class 
-ListenerManager
+ListenerManager<T>
 {
-	public static ListenerManager
+	private static final boolean TIME_LISTENERS = false;
+	
+	public static <T>ListenerManager<T>
 	createManager(
-		String							name,
-		ListenerManagerDispatcher		target )
+		String										name,
+		ListenerManagerDispatcher<T>				target )
 	{
-		return( new ListenerManager( name, target, false ));
+		return( new ListenerManager<T>( name, target, false ));
 	}
 	
-	public static ListenerManager
+	public static <T>ListenerManager<T>
 	createAsyncManager(
 		String							name,
-		ListenerManagerDispatcher		target )
+		ListenerManagerDispatcher<T>	target )
 	{
-		return( new ListenerManager( name, target, true ));
+		return( new ListenerManager<T>( name, target, true ));
 	}
 	
 	
 	protected String	name;
 	
-	protected ListenerManagerDispatcher					target;
+	protected ListenerManagerDispatcher<T>				target;
 	protected ListenerManagerDispatcherWithException	target_with_exception;
 	
 	protected boolean	async;
-	protected Thread	async_thread;
+	protected AEThread2	async_thread;
 	
-	protected List		listeners		= new ArrayList();
+	protected List<T>			listeners		= new ArrayList<T>();
 	
-	protected List			dispatch_queue;
-	protected AESemaphore	dispatch_sem;
+	protected List<Object[]>	dispatch_queue;
+	protected AESemaphore		dispatch_sem;
 	
 	protected
 	ListenerManager(
-		String							_name,
-		ListenerManagerDispatcher		_target,
-		boolean							_async )
+		String								_name,
+		ListenerManagerDispatcher<T>		_target,
+		boolean								_async )
 	{
 		name	= _name;
 		target	= _target;
@@ -91,7 +94,7 @@ ListenerManager
 		if ( async ){
 			
 			dispatch_sem	= new AESemaphore("ListenerManager::"+name);
-			dispatch_queue 	= new LinkedList();
+			dispatch_queue 	= new LinkedList<Object[]>();
 			
 			if ( target_with_exception != null ){
 				
@@ -102,11 +105,11 @@ ListenerManager
 	
 	public void
 	addListener(
-		Object		listener )
+		T		listener )
 	{
 		synchronized( this ){
 			
-			ArrayList	new_listeners	= new ArrayList( listeners );
+			ArrayList<T>	new_listeners	= new ArrayList<T>( listeners );
 			
 			if (new_listeners.contains(listener)) {
 				Logger.log(new LogEvent(LogIDs.CORE, LogEvent.LT_WARNING,
@@ -125,17 +128,15 @@ ListenerManager
 			
 			if ( async && async_thread == null ){
 				
-				async_thread = new AEThread( name )
+				async_thread = new AEThread2( name, true )
 					{
 						public void
-						runSupport()
+						run()
 						{
 							dispatchLoop();
 						}
 					};
-					
-				async_thread.setDaemon( true );
-				
+									
 				async_thread.start();
 			}
 		}
@@ -147,7 +148,7 @@ ListenerManager
 	{
 		synchronized( this ){
 			
-			ArrayList	new_listeners = new ArrayList( listeners );
+			ArrayList<T>	new_listeners = new ArrayList<T>( listeners );
 			
 			new_listeners.remove( listener );
 			
@@ -164,12 +165,22 @@ ListenerManager
 		}
 	}
 	
+	public boolean
+	hasListener(
+		T		listener )
+	{
+		synchronized( this ){
+
+			return( listeners.contains( listener ));
+		}
+	}
+	
 	public void
 	clear()
 	{
 		synchronized( this ){
 									
-			listeners	= new ArrayList();
+			listeners	= new ArrayList<T>();
 			
 			if ( async ){
 				
@@ -182,7 +193,7 @@ ListenerManager
 		}
 	}
 	
-	public List
+	public List<T>
 	getListenersCopy()
 	{
 			// we can just return the listeners as we copy on update
@@ -241,7 +252,7 @@ ListenerManager
 				throw( new RuntimeException( "call dispatchWithException, not dispatch"));
 			}
 			
-			List	listeners_ref;
+			List<T>	listeners_ref;
 			
 			synchronized( this ){
 				
@@ -265,7 +276,7 @@ ListenerManager
 	
 		throws Throwable
 	{
-		List	listeners_ref;
+		List<T>	listeners_ref;
 		
 		synchronized( this ){
 			
@@ -277,7 +288,7 @@ ListenerManager
 	
 	public void
 	dispatch(
-		Object	listener,
+		T		listener,
 		int		type,
 		Object	value )
 	{
@@ -286,7 +297,7 @@ ListenerManager
 	
 	public void
 	dispatch(
-		Object	listener,
+		T		listener,
 		int		type,
 		Object	value,
 		boolean	blocking )
@@ -308,17 +319,15 @@ ListenerManager
 				
 				if ( async_thread == null ){
 					
-					async_thread = new AEThread( name )
+					async_thread = new AEThread2( name, true )
 						{
 							public void
-							runSupport()
+							run()
 							{
 								dispatchLoop();
 							}
 						};
-						
-					async_thread.setDaemon( true );
-					
+											
 					async_thread.start();
 				}
 			}
@@ -336,19 +345,96 @@ ListenerManager
 				throw( new RuntimeException( "call dispatchWithException, not dispatch"));
 			}
 			
-			try{
-				target.dispatch( listener, type, value );
-				
-			}catch( Throwable e ){
-				
-				Debug.printStackTrace( e );
-			}
+			doDispatch( listener, type, value );
 		}
 	}
 
+	protected String
+	getListenerName( 
+		T	 listener )
+	{
+		Class listener_class = listener.getClass();
+		
+		String	res = listener_class.getName();
+		
+		try{
+			Method getString = listener_class.getMethod( "getString", new Class[0]);
+			
+			if ( getString != null ){
+				
+				String s = (String)getString.invoke( listener, new Object[0] );
+				
+				res += " (" + s + ")";
+			}
+		}catch( Throwable e ){
+			
+		}
+		
+		return( res );
+	}
+	
+	protected void
+	doDispatch(
+		T			listener,
+		int			type,
+		Object		value )
+	{
+		try{
+			if ( TIME_LISTENERS ){
+
+				long	start = SystemTime.getCurrentTime();
+				
+				try{
+
+					target.dispatch( listener, type, value );
+
+				}finally{
+					
+					long duration = SystemTime.getCurrentTime() - start;
+					
+					System.out.println( name + "/" + type + ": " + getListenerName( listener ) + " - " + duration );
+				}
+			}else{
+			
+				target.dispatch( listener, type, value );
+			}
+			
+		}catch( Throwable e ){
+			
+			Debug.printStackTrace( e );
+		}
+	}
+	
+	protected void
+	doDispatchWithException(
+		T			listener,
+		int			type,
+		Object		value )
+	
+		throws Throwable
+	{
+		if ( TIME_LISTENERS ){
+
+			long	start = SystemTime.getCurrentTime();
+			
+			try{
+				target_with_exception.dispatchWithException( listener, type, value );
+
+			}finally{
+				
+				long duration = SystemTime.getCurrentTime() - start;
+				
+				System.out.println( name + "/" + type + ": " + getListenerName( listener ) + " - " + duration );
+			}
+		}else{
+		
+			target_with_exception.dispatchWithException( listener, type, value );
+		}
+	}
+	
 	protected void
 	dispatchInternal(
-		List		listeners_ref,
+		List<T>		listeners_ref,
 		int			type,
 		Object		value )
 	
@@ -364,26 +450,18 @@ ListenerManager
 					// DON'T catch and handle exceptions here are they are permitted to
 					// occur!
 				
-				target_with_exception.dispatchWithException( listeners_ref.get(i), type, value );
+				doDispatchWithException( listeners_ref.get(i), type, value );
 					
 			}else{
 			
-				try{
-						// System.out.println( name + ":dispatch" );
-						
-					target.dispatch( listeners_ref.get(i), type, value );
-					
-				}catch( Throwable e ){
-					
-					Debug.printStackTrace( e );
-				}
+				doDispatch( listeners_ref.get(i), type, value );
 			}
 		}
 	}
 	
 	protected void
 	dispatchInternal(
-		Object		listener,
+		T			listener,
 		int			type,
 		Object		value )
 	
@@ -396,19 +474,11 @@ ListenerManager
 				// DON'T catch and handle exceptions here are they are permitted to
 				// occur!
 
-			target_with_exception.dispatchWithException( listener, type, value );
+			doDispatchWithException( listener, type, value );
 				
 		}else{
-			try{
-				
-					// System.out.println( name + ":dispatch" );
-				
-				target.dispatch( listener, type, value );
 			
-			}catch( Throwable e ){
-					
-				Debug.printStackTrace( e );
-			}
+			doDispatch( listener, type, value );
 		}
 	}
 	
@@ -425,7 +495,7 @@ ListenerManager
 			
 			synchronized( this ){
 				
-				if ( async_thread != Thread.currentThread()){
+				if ( async_thread == null || !async_thread.isCurrentThread()){
 					
 						// we've been asked to close. this sem reservation must be
 						// "returned" to the pool in case it represents a valid  entry
@@ -447,11 +517,11 @@ ListenerManager
 				try{						
 					if ( data.length == 4 ){
 					
-						dispatchInternal((List)data[0], ((Integer)data[1]).intValue(), data[2] );
+						dispatchInternal((List<T>)data[0], ((Integer)data[1]).intValue(), data[2] );
 						
 					}else{
 						
-						dispatchInternal( data[0], ((Integer)data[1]).intValue(), data[2] );
+						dispatchInternal((T)data[0], ((Integer)data[1]).intValue(), data[2] );
 					}
 					
 				}catch( Throwable e ){
@@ -471,13 +541,13 @@ ListenerManager
 		// System.out.println( "ListenerManager::dispatch thread '" + Thread.currentThread() + "' ends");
 	}
 	
-	public static void
+	public static <T>void
 	dispatchWithTimeout(
-		List							_listeners,
-		final ListenerManagerDispatcher	_dispatcher,
-		long							_timeout )
+		List<T>								_listeners,
+		final ListenerManagerDispatcher<T>	_dispatcher,
+		long								_timeout )
 	{
-		final List	listeners = new ArrayList( _listeners );
+		final List<T>	listeners = new ArrayList<T>( _listeners );
 		
 		final boolean[]	completed = new boolean[listeners.size()];
 		
@@ -487,9 +557,9 @@ ListenerManager
 			
 			final int f_i	= i;
 						
-			new AEThread( "ListenerManager:dwt:dispatcher", true ){
+			new AEThread2( "ListenerManager:dwt:dispatcher", true ){
 				public void
-				runSupport()
+				run()
 				{
 					try{
 						_dispatcher.dispatch( listeners.get(f_i), -1, null );
