@@ -31,7 +31,7 @@ import java.net.*;
 import java.util.*;
 
 import com.aelitis.azureus.core.*;
-import com.aelitis.azureus.plugins.dht.DHTPlugin;
+import com.aelitis.azureus.core.util.CopyOnWriteList;
 
 import org.gudy.azureus2.core3.config.COConfigurationManager;
 import org.gudy.azureus2.core3.config.ParameterListener;
@@ -40,7 +40,6 @@ import org.gudy.azureus2.core3.logging.LogRelation;
 import org.gudy.azureus2.core3.torrent.*;
 import org.gudy.azureus2.core3.disk.*;
 import org.gudy.azureus2.core3.download.*;
-import org.gudy.azureus2.plugins.PluginInterface;
 
 
 public class 
@@ -53,6 +52,8 @@ TorrentUtils
 	private static final String		TORRENT_AZ_PROP_TORRENT_FLAGS			= "torrent_flags";
 	private static final String		TORRENT_AZ_PROP_PLUGINS					= "plugins";
 	
+	public static final String		TORRENT_AZ_PROP_OBTAINED_FROM			= "obtained_from";
+	
 	private static final String		MEM_ONLY_TORRENT_PATH		= "?/\\!:mem_only:!\\/?";
 	
 	
@@ -73,6 +74,8 @@ TorrentUtils
 	
 	private static boolean bSaveTorrentBackup;
 	
+	private static CopyOnWriteList	torrent_attribute_listeners = new CopyOnWriteList();
+	
 	static {
 		COConfigurationManager.addAndFireParameterListener("Save Torrent Backup",
 				new ParameterListener() {
@@ -481,11 +484,11 @@ TorrentUtils
 		return( errorDetail );
 	}
 	
-	public static List
+	public static List<List<String>>
 	announceGroupsToList(
 		TOTorrent	torrent )
 	{
-		List	groups = new ArrayList();
+		List<List<String>>	groups = new ArrayList<List<String>>();
 		
 		TOTorrentAnnounceURLGroup group = torrent.getAnnounceURLGroup();
 		
@@ -493,7 +496,7 @@ TorrentUtils
 		
 		if ( sets.length == 0 ){
 		
-			List	s = new ArrayList();
+			List<String>	s = new ArrayList<String>();
 			
 			s.add( torrent.getAnnounceURL().toString());
 			
@@ -502,7 +505,7 @@ TorrentUtils
 			
 			for (int i=0;i<sets.length;i++){
 			
-				List	s = new ArrayList();
+				List<String>	s = new ArrayList<String>();
 								
 				TOTorrentAnnounceURLSet	set = sets[i];
 				
@@ -525,8 +528,8 @@ TorrentUtils
 	
 	public static void
 	listToAnnounceGroups(
-		List		groups,
-		TOTorrent	torrent )
+		List<List<String>>		groups,
+		TOTorrent				torrent )
 	{
 		try{
 			TOTorrentAnnounceURLGroup tg = torrent.getAnnounceURLGroup();
@@ -922,6 +925,39 @@ TorrentUtils
 		return( null );
 	}
 	
+	public static void
+	setTLSDescription(
+		String		desc )
+	{
+		((Map)tls.get()).put( "desc", desc );
+	}
+	
+	public static String
+	getTLSDescription()
+	{
+		return((String)((Map)tls.get()).get( "desc" ));
+	}
+	
+		/**
+		 * get tls for cloning onto another thread
+		 * @return
+		 */
+	
+	public static Object
+	getTLS()
+	{
+		return( tls.get());
+	}
+	
+	public static void
+	setTLS(
+		Object	obj )
+	{
+		Map	m = (Map)obj;
+		
+		((Map)tls.get()).putAll(m);
+	}
+	
 	public static URL
 	getDecentralisedEmptyURL()
 	{
@@ -992,6 +1028,83 @@ TorrentUtils
 		return( m );
 	}
 	
+	private static Map
+	getAzureusPrivateProperties(
+		TOTorrent	torrent )
+	{
+		Map	m = torrent.getAdditionalMapProperty( TOTorrent.AZUREUS_PRIVATE_PROPERTIES );
+		
+		if ( m == null ){
+			
+			m = new HashMap();
+			
+			torrent.setAdditionalMapProperty( TOTorrent.AZUREUS_PRIVATE_PROPERTIES, m );
+		}
+		
+		return( m );
+	}
+	
+	public static void
+	setObtainedFrom(
+		File			file,
+		String			str )
+	{
+		try{
+			TOTorrent	torrent = readFromFile( file, false, false );
+			
+			setObtainedFrom( torrent, str );
+			
+			writeToFile( torrent );
+			
+		} catch (TOTorrentException e) {
+			// ignore, file probably not torrent
+			
+		}catch( Throwable e ){
+			
+			Debug.out( e );
+		}
+	}
+	
+	public static void
+	setObtainedFrom(
+		TOTorrent		torrent,
+		String			str )
+	{
+		Map	m = getAzureusPrivateProperties( torrent );
+			
+		try{
+			m.put( TORRENT_AZ_PROP_OBTAINED_FROM, str.getBytes( "UTF-8" ));
+			
+			fireAttributeListener( torrent, TORRENT_AZ_PROP_OBTAINED_FROM, str );
+			
+		}catch( Throwable e ){
+			
+			Debug.printStackTrace(e);
+		}
+	}
+	
+	public static String
+	getObtainedFrom(
+		TOTorrent		torrent )
+	{
+		Map	m = getAzureusPrivateProperties( torrent );
+
+		byte[]	from = (byte[])m.get( TORRENT_AZ_PROP_OBTAINED_FROM );
+		
+		if ( from != null ){
+			
+			try{
+				return( new String( from, "UTF-8" ));
+				
+			}catch( Throwable e ){
+			
+				Debug.printStackTrace(e);
+			}
+		}
+		
+		return( null );
+	}
+	
 	public static void
 	setFlag(
 		TOTorrent		torrent,
@@ -1204,8 +1317,8 @@ TorrentUtils
 			return( false );
 		}	
 		
-		String announceURL = torrent.getAnnounceURL().toExternalForm();
-		if(announceURL.matches(".*[0-9a-z]{20,40}.*")) {
+		if ( UrlUtils.containsPasskey( torrent.getAnnounceURL())){
+				
 			return torrent.getPrivate();
 		}
 		
@@ -1704,6 +1817,15 @@ TorrentUtils
 			return( delegate.getHashWrapper());
 		}
 		
+	   	public void 
+    	setHashOverride(
+    		byte[] hash ) 
+    	
+    		throws TOTorrentException 
+    	{
+    		throw( new TOTorrentException( "Not supported", TOTorrentException.RT_HASH_FAILS ));
+    	}
+	   	
 		public boolean
 		getPrivate()
 		{
@@ -2343,4 +2465,55 @@ TorrentUtils
 			}
 		}
 	}
+	
+	public static String
+	getMagnetURI(
+		byte[]		hash )
+	{
+		return( "magnet:?xt=urn:btih:" + Base32.encode( hash ));
+	}
+	
+	private static void
+	fireAttributeListener(
+		TOTorrent		torrent,	
+		String			attribute,
+		Object			value )
+	{
+		Iterator it = torrent_attribute_listeners.iterator();
+		
+		while( it.hasNext()){
+			
+			try{
+				((torrentAttributeListener)it.next()).attributeSet(torrent, attribute, value);
+				
+			}catch( Throwable e ){
+				
+				Debug.printStackTrace(e);
+			}
+		}
+	}
+		
+	public static void
+	addTorrentAttributeListener(
+		torrentAttributeListener	listener )
+	{
+		torrent_attribute_listeners.add( listener );
+	}
+	
+	public static void
+	removeTorrentAttributeListener(
+		torrentAttributeListener	listener )
+	{
+		torrent_attribute_listeners.remove( listener );
+	}
+	
+	public interface
+	torrentAttributeListener
+	{
+		public void
+		attributeSet(
+			TOTorrent	torrent,
+			String		attribute,
+			Object		value );
+	}
 }
