@@ -30,7 +30,9 @@ import java.util.regex.Pattern;
 
 import org.bouncycastle.util.encoders.Base64;
 import org.gudy.azureus2.core3.config.COConfigurationManager;
+import org.gudy.azureus2.core3.torrent.TOTorrent;
 import org.gudy.azureus2.plugins.utils.resourcedownloader.ResourceDownloader;
+import org.gudy.azureus2.plugins.utils.resourceuploader.ResourceUploader;
 
 import com.aelitis.azureus.core.util.Java15Utils;
 import com.aelitis.net.magneturi.MagnetURIHandler;
@@ -52,10 +54,11 @@ public class UrlUtils
 			"http://",
 			"https://",
 			"ftp://",
+			"dht://",
 			"magnet:?",
 			"magnet://?" };
 
-	private static int MAGNETURL_STARTS_AT = 3;
+	private static int MAGNETURL_STARTS_AT = 3;	// dht:// is a form of magnet URL
 	
 	private static final Object[] XMLescapes = new Object[] {
 		new String[] { "&", "&amp;" },
@@ -84,6 +87,13 @@ public class UrlUtils
 		return parseTextForURL(text, accept_magnets, true);
 	}
 
+	public static String
+	getURL(
+		String	text )
+	{
+		return( parseTextForURL(text, false, false ));
+	}
+	
 	public static String parseTextForURL(String text, boolean accept_magnets,
 			boolean guess) {
 
@@ -242,6 +252,17 @@ public class UrlUtils
 		}
 	}
 	
+	public static String decode(String s) {
+		if (s == null) {
+			return "";
+		}
+		try {
+			return( URLDecoder.decode(s, "UTF-8"));
+		} catch (UnsupportedEncodingException e) {
+			return( URLDecoder.decode(s));
+		}
+	}
+	
 	public static String escapeXML(String s) {
 		if (s == null) {
 			return "";
@@ -395,6 +416,14 @@ public class UrlUtils
 	
 	private static final String default_headers = "SG9zdDogbG9jYWxob3N0OjQ1MTAwClVzZXItQWdlbnQ6IE1vemlsbGEvNS4wIChXaW5kb3dzOyBVOyBXaW5kb3dzIE5UIDUuMTsgZW4tVVM7IHJ2OjEuOC4xLjE0KSBHZWNrby8yMDA4MDQwNCBGaXJlZm94LzIuMC4wLjE0CkFjY2VwdDogdGV4dC94bWwsYXBwbGljYXRpb24veG1sLGFwcGxpY2F0aW9uL3hodG1sK3htbCx0ZXh0L2h0bWw7cT0wLjksdGV4dC9wbGFpbjtxPTAuOCxpbWFnZS9wbmcsKi8qO3E9MC41CkFjY2VwdC1MYW5ndWFnZTogZW4tdXMsZW47cT0wLjUKQWNjZXB0LUVuY29kaW5nOiBnemlwLGRlZmxhdGUKQWNjZXB0LUNoYXJzZXQ6IElTTy04ODU5LTEsdXRmLTg7cT0wLjcsKjtxPTAuNwpLZWVwLUFsaXZlOiAzMDAKQ29ubmVjdGlvbjoga2VlcC1hbGl2ZQ==";
 
+	public static void
+	setBrowserHeaders(
+		ResourceDownloader		rd,
+		String					referer )
+	{
+		setBrowserHeaders( rd, null, referer );
+	}
+	
 	public static void
 	setBrowserHeaders(
 		ResourceDownloader		rd,
@@ -427,7 +456,7 @@ public class UrlUtils
 				}
 			}
 			
-			if ( referer != null ){
+			if ( referer != null && referer.length() > 0 ){
 				
 				rd.setProperty( "URL_Referer", referer );
 			}
@@ -435,6 +464,46 @@ public class UrlUtils
 		}
 	}
 	
+	public static void
+	setBrowserHeaders(
+		ResourceUploader		ru,
+		String					encoded_headers,
+		String					referer )
+	{
+		String	headers_to_use = getBrowserHeadersToUse( encoded_headers );
+		
+		try{
+			String header_string = new String( Base64.decode( headers_to_use ), "UTF-8" );
+		
+			String[]	headers = header_string.split( "\n" );
+			
+			for (int i=0;i<headers.length;i++ ){
+			
+				String	header = headers[i];
+				
+				int	pos = header.indexOf( ':' );
+				
+				if ( pos != -1 ){
+					
+					String	lhs = header.substring(0,pos).trim();
+					String	rhs	= header.substring(pos+1).trim();
+					
+					if ( !( lhs.equalsIgnoreCase( "Host") || 
+							lhs.equalsIgnoreCase( "Referer" ))){
+						
+						ru.setProperty( "URL_" + lhs, rhs );
+					}
+				}
+			}
+			
+			if ( referer != null && referer.length() > 0 ){
+				
+				ru.setProperty( "URL_Referer", referer );
+			}
+		}catch( Throwable e ){	
+		}
+	}
+	
 	public static void
 	setBrowserHeaders(
 		URLConnection			connection,
@@ -476,7 +545,7 @@ public class UrlUtils
 				}
 			}
 			
-			if ( referer != null ){
+			if ( referer != null && referer.length() > 0 ){
 				
 				connection.setRequestProperty( "Referer", referer );
 			}
@@ -517,7 +586,7 @@ public class UrlUtils
 				}
 			}
 			
-			if ( referer != null ){
+			if ( referer != null && referer.length() > 0){
 				
 				result.put( "Referer", referer );
 			}
@@ -558,4 +627,178 @@ public class UrlUtils
 		
 		return( headers_to_use );
 	}
+	
+	public static boolean queryHasParameter(String query_string, String param_name, boolean case_sensitive) {
+		if (!case_sensitive) {
+			query_string = query_string.toLowerCase();
+			param_name = param_name.toLowerCase();
+		}
+		if (query_string.charAt(0) == '?') {
+			query_string = '&' + query_string.substring(1);
+		}
+		else if (query_string.charAt(0) != '&') {
+			query_string = '&' + query_string;
+		}
+		
+		return query_string.indexOf("&" + param_name + "=") != -1;
+	}
+	
+	public static boolean 
+	containsPasskey(
+		URL		url )
+	{
+		String url_str = url.toExternalForm();
+		
+		return( url_str.matches(".*[0-9a-z]{20,40}.*"));
+	}
+	
+	public static URL
+	setPort(
+		URL		u,
+		int		port )
+	{
+		StringBuffer result = new StringBuffer();
+		result.append(u.getProtocol());
+		result.append(":");
+		String authority=u.getAuthority();
+		if (authority != null && authority.length() > 0) {
+			result.append("//");
+			int pos = authority.indexOf( '@' );
+			if ( pos != -1 ){
+				result.append(authority.substring(0,pos+1));
+				authority = authority.substring(pos+1);
+			}
+			pos = authority.lastIndexOf(':');
+			if ( pos == -1 ){
+				result.append(authority + ":" + port );
+			}else{
+				result.append(authority.substring(0,pos+1) + port );				
+			}
+		}
+		if (u.getPath() != null) {
+			result.append(u.getPath());
+		}
+		if (u.getQuery() != null) {
+			result.append('?');
+			result.append(u.getQuery());
+		}
+		if (u.getRef() != null) {
+			result.append("#");
+			result.append(u.getRef());
+		}
+		try{
+			return( new URL( result.toString()));
+		}catch( Throwable e ){
+			Debug.out(e);
+			return(u);
+		}
+	}
+	
+	public static URL
+	setHost(
+		URL			u,
+		String		host )
+	{
+		StringBuffer result = new StringBuffer();
+		result.append(u.getProtocol());
+		result.append(":");
+		String authority=u.getAuthority();
+		if (authority != null && authority.length() > 0) {
+			result.append("//");
+			int pos = authority.indexOf( '@' );
+			if ( pos != -1 ){
+				result.append(authority.substring(0,pos+1));
+				authority = authority.substring(pos+1);
+			}
+			pos = authority.lastIndexOf(':');
+			if ( pos == -1 ){
+				result.append(host );
+			}else{
+				result.append(host + authority.substring(pos));				
+			}
+		}
+		if (u.getPath() != null) {
+			result.append(u.getPath());
+		}
+		if (u.getQuery() != null) {
+			result.append('?');
+			result.append(u.getQuery());
+		}
+		if (u.getRef() != null) {
+			result.append("#");
+			result.append(u.getRef());
+		}
+		try{
+			return( new URL( result.toString()));
+		}catch( Throwable e ){
+			Debug.out(e);
+			return(u);
+		}
+	}
+	
+		/**
+		 * Returns an explicit IPv4 url if the supplied one has both IPv6 and IPv4 addresses
+		 * @param url
+		 * @return
+		 */
+	
+	public static URL
+	getIPV4Fallback(
+		URL	url )
+	{
+		try{
+			InetAddress[] addresses = InetAddress.getAllByName( url.getHost());
+			
+			if ( addresses.length > 0 ){
+				
+				InetAddress	ipv4	= null;
+				InetAddress	ipv6	= null;
+				
+				for ( InetAddress a: addresses ){
+					
+					if ( a instanceof Inet4Address ){
+						
+						ipv4 = a;
+						
+					}else{
+						
+						ipv6 = a;
+					}
+				}
+				
+				if ( ipv4 != null && ipv6 != null ){
+					
+					url = UrlUtils.setHost( url, ipv4.getHostAddress());
+					
+					return( url );
+				}
+			}
+		}catch( Throwable f ){
+		}	
+		
+		return( null );
+	}
+	
+	public static long
+	getContentLength(
+		HttpURLConnection	con )
+	{
+		long res = con.getContentLength();
+		
+		if ( res == -1 ){
+			
+			try{
+				String	str = con.getHeaderField( "content-length" );
+				
+				if ( str != null ){
+					
+					res = Long.parseLong( str );
+				}
+			}catch( Throwable e ){
+				
+			}
+		}
+		
+		return( res );
+	}
 }
