@@ -22,25 +22,36 @@
 
 package org.gudy.azureus2.platform.macosx;
 
+import java.io.BufferedReader;
+import java.io.File;
+import java.io.IOException;
+import java.io.InputStream;
+import java.io.InputStreamReader;
+import java.net.InetAddress;
+import java.text.MessageFormat;
+import java.util.HashSet;
+
 import org.gudy.azureus2.core3.config.COConfigurationManager;
 import org.gudy.azureus2.core3.config.ParameterListener;
-import org.gudy.azureus2.core3.logging.*;
-import org.gudy.azureus2.core3.util.*;
+import org.gudy.azureus2.core3.logging.LogAlert;
+import org.gudy.azureus2.core3.logging.LogEvent;
+import org.gudy.azureus2.core3.logging.LogIDs;
+import org.gudy.azureus2.core3.logging.Logger;
+import org.gudy.azureus2.core3.util.AEDiagnostics;
+import org.gudy.azureus2.core3.util.AEDiagnosticsEvidenceGenerator;
+import org.gudy.azureus2.core3.util.AEMonitor;
+import org.gudy.azureus2.core3.util.Constants;
+import org.gudy.azureus2.core3.util.Debug;
+import org.gudy.azureus2.core3.util.IndentWriter;
+import org.gudy.azureus2.core3.util.SystemProperties;
 import org.gudy.azureus2.platform.PlatformManager;
 import org.gudy.azureus2.platform.PlatformManagerCapabilities;
 import org.gudy.azureus2.platform.PlatformManagerListener;
 import org.gudy.azureus2.platform.PlatformManagerPingCallback;
 import org.gudy.azureus2.platform.macosx.access.jnilib.OSXAccess;
-
 import org.gudy.azureus2.plugins.platform.PlatformManagerException;
 
-import java.io.BufferedReader;
-import java.io.File;
-import java.io.IOException;
-import java.io.InputStreamReader;
-import java.net.InetAddress;
-import java.text.MessageFormat;
-import java.util.HashSet;
+import com.apple.cocoa.application.NSApplication;
 
 
 /**
@@ -62,6 +73,9 @@ public class PlatformManagerImpl implements PlatformManager, AEDiagnosticsEviden
     //T: PlatformManagerCapabilities
     private final HashSet capabilitySet = new HashSet();
 
+    private volatile String		computer_name;
+    private volatile boolean	computer_name_tried;
+    
     /**
      * Gets the platform manager singleton, which was already initialized
      */
@@ -118,6 +132,7 @@ public class PlatformManagerImpl implements PlatformManager, AEDiagnosticsEviden
         capabilitySet.add(PlatformManagerCapabilities.GetUserDataDirectory);
         capabilitySet.add(PlatformManagerCapabilities.UseNativeScripting);
         capabilitySet.add(PlatformManagerCapabilities.PlaySystemAlert);
+        capabilitySet.add(PlatformManagerCapabilities.RequestUserAttention);
         
         if (OSXAccess.isLoaded()) {
 	        capabilitySet.add(PlatformManagerCapabilities.GetVersion);
@@ -147,23 +162,41 @@ public class PlatformManagerImpl implements PlatformManager, AEDiagnosticsEviden
     	return OSXAccess.getVersion();
     }
 
+    protected PListEditor
+    getPList()
+    
+    	throws IOException
+    {
+		String	plist = 
+			System.getProperty("user.dir") +
+			SystemProperties.SEP+ SystemProperties.getApplicationName() + ".app/Contents/Info.plist";
+
+		if (!new File(plist).exists()) {
+			Debug.out("WARNING: plist not found: " + plist);
+			return null;
+		}
+		PListEditor editor = new PListEditor( plist );
+	
+		return( editor );
+    }
+    
     protected void
     checkPList()
     {
     	try{
-			String	plist = 
-				System.getProperty("user.dir") +
-				SystemProperties.SEP+ SystemProperties.getApplicationName() + ".app/Contents/Info.plist";
-
-    		PListEditor editor = new PListEditor( plist );
-		
+    		PListEditor editor = getPList();
+    		if (editor == null) {
+    			return;
+    		}
+    		
     		editor.setFileTypeExtensions(new String[] {"torrent","tor","vuze","vuz"});
     		editor.setSimpleStringValue("CFBundleName", "Vuze");
 			editor.setSimpleStringValue("CFBundleTypeName", "Vuze Download");
 			editor.setSimpleStringValue("CFBundleGetInfoString","Vuze");
 			editor.setSimpleStringValue("CFBundleShortVersionString",Constants.AZUREUS_VERSION);
 			editor.setSimpleStringValue("CFBundleVersion",Constants.AZUREUS_VERSION);
-    		
+			editor.setArrayValues("CFBundleURLSchemes", "string", new String[] { "magnet", "dht", "vuze"});
+			
     	}catch( Throwable e ){
     		
     		Debug.out( "Failed to update plist", e );
@@ -171,6 +204,20 @@ public class PlatformManagerImpl implements PlatformManager, AEDiagnosticsEviden
 	
     }
     
+    protected void
+    touchPList()
+    {
+       	try{
+    		PListEditor editor = getPList();
+  	
+    		editor.touchFile();
+    		
+       	}catch( Throwable e ){
+    		
+    		Debug.out( "Failed to touch plist", e );
+    	}
+    }
+    
     /**
      * {@inheritDoc}
      * @see org.gudy.azureus2.core3.util.SystemProperties#getUserPath()
@@ -183,6 +230,114 @@ public class PlatformManagerImpl implements PlatformManager, AEDiagnosticsEviden
     			+ SystemProperties.SEP;
     }
 
+	public String 
+	getComputerName() 
+	{
+		if ( computer_name_tried ){
+			
+			return( computer_name );
+		}
+		
+		try{
+			String result = null;
+			
+			String	hostname = System.getenv( "HOSTNAME" );
+			
+			if ( hostname != null && hostname.length() > 0 ){
+				
+				result = hostname;
+			}
+			
+			if ( result == null ){
+				
+				String	host = System.getenv( "HOST" );
+				
+				if ( host != null && host.length() > 0 ){
+					
+					result = host;
+				}
+			}
+			
+			if ( result == null ){
+			
+				try{				
+					String[] to_run = new String[3];
+					
+				  	to_run[0] = "/bin/sh";
+				  	to_run[1] = "-c";
+				  	to_run[2] = "echo $HOSTNAME";
+				  	
+					Process p = Runtime.getRuntime().exec( to_run );
+					
+					if ( p.waitFor() == 0 ){
+						
+						String	output = "";
+						
+						InputStream is = p.getInputStream();
+						
+						while( true ){
+							
+							byte[] buffer = new byte[1024];
+							
+							int len = is.read( buffer );
+							
+							if ( len <= 0 ){
+								
+								break;
+							}
+							
+							output += new String( buffer, 0, len );
+							
+							if ( output.length() > 64 ){
+								
+								break;
+							}
+						}
+						
+						if ( output.length() > 0 ){
+							
+							result = output.trim();
+							
+							int pos = result.indexOf(' ');
+							
+							if ( pos != -1 ){
+								
+								result = result.substring( 0, pos ).trim();
+							}
+						}
+					}
+				}catch( Throwable e ){
+				}
+			}
+			
+			if ( result != null ){
+			
+				int	pos = result.lastIndexOf( '.' );
+				
+				if ( pos != -1 ){
+					
+					result = result.substring( 0, pos );
+				}
+				
+				if ( result.length() > 0 ){
+					
+					if ( result.length() > 32 ){
+						
+						result = result.substring( 0, 32 );
+					}
+					
+					computer_name = result;
+				}
+			}
+						
+			return( computer_name );
+			
+		}finally{
+			
+			computer_name_tried = true;
+		}
+	}
+	
 	public File
 	getLocation(
 		long	location_id )
@@ -282,12 +437,9 @@ public class PlatformManagerImpl implements PlatformManager, AEDiagnosticsEviden
 	   throw new PlatformManagerException("Unsupported capability called on platform manager");
 	}
 	
-    /**
-     * Not implemented; does nothing
-     */
     public void registerApplication() throws PlatformManagerException
     {
-        // handled by LaunchServices and/0r user interaction
+    	touchPList();
     }
 
     /**
@@ -736,6 +888,9 @@ public class PlatformManagerImpl implements PlatformManager, AEDiagnosticsEviden
 				} else {
 					writer.println("Not loaded");
 				}
+				
+				writer.println("Computer Name: " + getComputerName());
+
 			} finally {
 				writer.exdent();
 			}
@@ -746,4 +901,30 @@ public class PlatformManagerImpl implements PlatformManager, AEDiagnosticsEviden
 		throw new PlatformManagerException(
 				"Unsupported capability called on platform manager");
 	}
+
+	/**
+	 * If the application is not active causes the application icon at the bottom to bounce until the application becomes active
+	 * If the application is already active then this method does nothing.
+	 * 
+	 * Note: This is an undocumented feature from Apple so it's behavior may change without warning
+	 * 
+	 * @param type one of USER_REQUEST_INFO, USER_REQUEST_WARNING
+	 */
+	public void requestUserAttention(int type, Object data)
+			throws PlatformManagerException {
+		try {
+			NSApplication app = NSApplication.sharedApplication();
+			if (type == USER_REQUEST_INFO) {
+				app.requestUserAttention(NSApplication.UserAttentionRequestInformational);
+			} else if (type == USER_REQUEST_WARNING) {
+				app.requestUserAttention(NSApplication.UserAttentionRequestCritical);
+			} else if (type == USER_REQUEST_QUESTION) {
+				// not applicable
+			}
+
+		} catch (Exception e) {
+			throw new PlatformManagerException("Failed to request user attention", e);
+		}
+
+	}
 }
