@@ -127,11 +127,12 @@ PlatformManagerImpl
 		}
 	}
 	
-	protected AEWin32Access		access;
+	private final AEWin32Access		access;
 
-	protected String			app_exe_name;
-	protected File				az_exe;
-	protected boolean			az_exe_checked;
+	private final String			app_name;
+	private final String			app_exe_name;
+	private File					az_exe;
+	private boolean					az_exe_checked;
 
 	protected
 	PlatformManagerImpl()
@@ -142,7 +143,9 @@ PlatformManagerImpl
 		
 		access.addListener( this );
 		
-		app_exe_name	= SystemProperties.getApplicationName() + ".exe";
+		app_name		= SystemProperties.getApplicationName();
+		
+		app_exe_name	= app_name + ".exe";
 		
         initializeCapabilities();
 	}
@@ -322,7 +325,7 @@ PlatformManagerImpl
 
 				if (!az_exe.exists()) {
 					try {
-						az_home = access.getApplicationInstallDir(SystemProperties.getApplicationName());
+						az_home = access.getApplicationInstallDir( app_name );
 
 						az_exe = new File(az_home + File.separator + app_exe_name).getAbsoluteFile();
 
@@ -337,7 +340,7 @@ PlatformManagerImpl
 								
 				if ( !az_exe.exists()){
 					
-					String	msg = app_exe_name + " not found in " + az_home + " - can't check file associations. Please re-install " + SystemProperties.getApplicationName();
+					String	msg = app_exe_name + " not found in " + az_home + " - can't check file associations. Please re-install " + app_name;
 					
 					az_exe = null;
 					
@@ -370,8 +373,7 @@ PlatformManagerImpl
 		throws PlatformManagerException
 	{
 		try{
-			return access.getUserAppData() + SystemProperties.SEP
-					+ SystemProperties.APPLICATION_NAME + SystemProperties.SEP;
+			return access.getUserAppData() + SystemProperties.SEP + app_name + SystemProperties.SEP;
 			
 		}catch( Throwable e ){
 			
@@ -379,6 +381,19 @@ PlatformManagerImpl
 		}		
 	}
 	
+	public String 
+	getComputerName() 
+	{
+		String	host = System.getenv( "COMPUTERNAME" );
+		
+		if ( host != null && host.length() > 0 ){
+			
+			return( host );
+		}
+		
+		return( null );
+	}
+	
 	public File
 	getLocation(
 		long	location_id )
@@ -442,20 +457,35 @@ PlatformManagerImpl
 	{
 			// all this stuff needs the exe location so bail out early if unavailable
 		
-		getApplicationEXELocation();
+		File exe_loc = getApplicationEXELocation();
+		
+		if ( exe_loc.exists()){
+			
+			checkExeKey( exe_loc );
+		}
 		
 		try{
 				// always trigger magnet reg here if not owned so old users get it...
+					
+			registerMagnet( false );
 			
-			if ( getAdditionalFileTypeRegistrationDetails( "Magnet", ".magnet" ) == RT_NONE ){
-		
-				registerMagnet();
-			}
 		}catch( Throwable e ){
 			
 			Debug.printStackTrace(e);
 		}
 		
+		try{
+				// always trigger magnet reg here if not owned so old users get it...
+			
+			if ( getAdditionalFileTypeRegistrationDetails( "DHT", ".dht" ) == RT_NONE ){
+		
+				registerDHT();
+			}
+		}catch( Throwable e ){
+			
+			Debug.printStackTrace(e);
+		}
+	
 		if ( isAdditionalFileTypeRegistered( OLD_MAIN_ASS0C, ".torrent" )){
 			
 			unregisterAdditionalFileType( OLD_MAIN_ASS0C, ".torrent" );
@@ -488,6 +518,38 @@ PlatformManagerImpl
 		return( reg );
 	}
 	
+	protected void
+	checkExeKey(
+		File		exe )
+	{
+		checkExeKey( AEWin32Access.HKEY_CURRENT_USER, exe );
+		checkExeKey( AEWin32Access.HKEY_LOCAL_MACHINE, exe );
+	}
+	
+	protected void
+	checkExeKey(
+		int			hkey,
+		File		exe )
+	{
+		String	exe_str = exe.getAbsolutePath();
+		
+		String str = null;
+		
+		try{
+			str = access.readStringValue( hkey, "software\\" + app_name, "exec" );
+
+		}catch( Throwable e ){
+		}
+		
+		try{
+			if ( str == null || !str.equals( exe_str )){
+				
+				access.writeStringValue( hkey, "software\\" + app_name,	"exec",	exe_str );
+			}
+		}catch( Throwable e ){
+		}
+	}
+	
 	public boolean
 	isAdditionalFileTypeRegistered(
 		String		name,
@@ -611,7 +673,9 @@ PlatformManagerImpl
 	
 		throws PlatformManagerException
 	{
-		registerMagnet();
+		registerMagnet( true );
+		
+		registerDHT();
 		
 		registerAdditionalFileType( NEW_MAIN_ASSOC, Constants.APP_NAME + " Download", ".torrent", "application/x-bittorrent" );
 		
@@ -619,14 +683,93 @@ PlatformManagerImpl
 	}
 	
 	protected void
-	registerMagnet()
+	registerMagnet(
+		boolean		force )
+	{
+
+		try{
+				// see http://forum.vuze.com/thread.jspa?threadID=79692&tstart=0
+			
+			String	az_exe_string	= getApplicationEXELocation().toString();
+			
+			boolean	magnet_exe_managing = false;
+			
+			try{
+				String existing = access.readStringValue( AEWin32Access.HKEY_CLASSES_ROOT, "magnet\\shell\\open\\command", "" );
+
+				magnet_exe_managing = existing.toLowerCase().indexOf( "\\magnet.exe" ) != -1;
+				
+			}catch( Throwable e ){
+			}
+			
+			if ( !magnet_exe_managing ){
+				
+				if ( force || getAdditionalFileTypeRegistrationDetails( "Magnet", ".magnet" ) == RT_NONE ){
+
+					try{
+						registerAdditionalFileType( 
+							"Magnet", 
+							"Magnet URI", 
+							".magnet", 
+							"application/x-magnet",
+							true );
+						
+					}catch( Throwable e ){
+						
+						Debug.printStackTrace(e);
+					}
+				}
+			}
+			
+				// we always write this hierarchy in case magnet.exe installed in the future
+			
+			createKey( AEWin32Access.HKEY_LOCAL_MACHINE, "Software\\magnet" );
+			createKey( AEWin32Access.HKEY_LOCAL_MACHINE, "Software\\magnet\\Handlers" );
+			createKey( AEWin32Access.HKEY_LOCAL_MACHINE, "Software\\magnet\\Handlers\\Azureus" );
+	
+			access.writeStringValue( AEWin32Access.HKEY_LOCAL_MACHINE, "Software\\magnet\\Handlers\\Azureus", "DefaultIcon", "\"" + az_exe_string + "," + getIconIndex() + "\"" );
+			access.writeStringValue( AEWin32Access.HKEY_LOCAL_MACHINE, "Software\\magnet\\Handlers\\Azureus", "Description", "Download with Vuze (formerly Azureus)" );
+			access.writeStringValue( AEWin32Access.HKEY_LOCAL_MACHINE, "Software\\magnet\\Handlers\\Azureus", "ShellExecute", "\"" + az_exe_string + "\" %URL" );
+			
+			access.writeWordValue( AEWin32Access.HKEY_LOCAL_MACHINE, "Software\\magnet\\Handlers\\Azureus\\Type", "urn:btih", 0 );
+
+		}catch( Throwable e ){		
+		}
+	}
+	
+	protected boolean
+	createKey(
+		int		type,
+		String	key )
+	{
+		try{
+			access.readStringValue( type, key, "");
+		
+			return( true );
+			
+		}catch( Throwable e ){
+			
+			try{
+				access.writeStringValue( type, key, "", "" );
+				
+				return( true );
+				
+			}catch( Throwable f ){
+				
+				return( false );
+			}
+		}
+	}
+	
+	protected void
+	registerDHT()
 	{
 		try{
 			registerAdditionalFileType( 
-				"Magnet", 
-				"Magnet URI", 
-				".magnet", 
-				"application/x-magnet",
+				"DHT", 
+				"DHT URI", 
+				".dht", 
+				"application/x-dht",
 				true );
 			
 		}catch( Throwable e ){
@@ -634,6 +777,7 @@ PlatformManagerImpl
 			Debug.printStackTrace(e);
 		}
 	}
+	
 	public void
 	registerAdditionalFileType(
 		String		name,				// e.g. "Azureus"
@@ -674,6 +818,16 @@ PlatformManagerImpl
 					"",
 					name );
 		
+			writeStringToHKCRandHKCU(
+					type,
+					"Content Type",
+					content_type );
+		
+			writeStringToHKCRandHKCU(
+					"MIME\\Database\\Content Type\\" + content_type,
+					"Extension",
+					type );
+		
 			writeStringToHKCRandHKCU(
 					name,
 					"",
@@ -695,8 +849,8 @@ PlatformManagerImpl
 					"\"" + az_exe_string + "\" \"%1\"" );
 					
 			writeStringToHKCRandHKCU(
-					name + "\\Content Type" ,
-					"",
+					name,
+					"Content Type",
 					content_type );
 			
 			if ( url_protocol ){
@@ -722,7 +876,7 @@ PlatformManagerImpl
 		try {
 			access.writeStringValue(AEWin32Access.HKEY_CLASSES_ROOT, subkey, name, value);
 		} catch (Throwable e) {
-			if (!Constants.isWindowsVista) {
+			if (!Constants.isWindowsVistaOrHigher) {
 				Debug.out(e);
 			}
 		}
@@ -1053,15 +1207,14 @@ PlatformManagerImpl
 		String cid = null;
 		try {
 			cid = access.readStringValue(AEWin32Access.HKEY_LOCAL_MACHINE,
-					"SOFTWARE\\Azureus", "CID");
+					"SOFTWARE\\" + app_name, "CID");
 		} catch (Exception e) {
 		}
 
 		if (cid == null || cid.length() == 0) {
 			needWrite = true;
 			try {
-				File commonPath = new File(access.getCommonAppData(),
-						SystemProperties.APPLICATION_NAME);
+				File commonPath = new File(access.getCommonAppData(),app_name);
 				if (commonPath.isDirectory()) {
 					File fCID = new File(commonPath, "azCID.txt");
 					if (fCID.exists()) {
@@ -1094,7 +1247,7 @@ PlatformManagerImpl
 	private void setAzComputerID(String cid) {
 		try {
 			access.writeStringValue(AEWin32Access.HKEY_LOCAL_MACHINE,
-					"SOFTWARE\\Azureus", "CID", cid);
+					"SOFTWARE\\" + app_name, "CID", cid);
 		} catch (Exception e) {
 			Debug.out("Could not write CID: " + e.getMessage());
 		}
@@ -1104,7 +1257,7 @@ PlatformManagerImpl
 			if (sCommonAppData != null && sCommonAppData.length() > 0) {
 				File commonPath = new File(sCommonAppData);
 				if (commonPath.isDirectory()) {
-					commonPath = new File(commonPath, SystemProperties.APPLICATION_NAME);
+					commonPath = new File(commonPath, app_name);
 					FileUtil.mkdirs(commonPath);
 
 					File fCID = new File(commonPath, "azCID.txt");
@@ -1127,4 +1280,8 @@ PlatformManagerImpl
 		}
 		
 	}
+
+	public void requestUserAttention(int type, Object data) throws PlatformManagerException {
+		throw new PlatformManagerException("Unsupported capability called on platform manager");
+	}
 }
