@@ -227,6 +227,8 @@ PlatformManagerUpdateChecker
 						alternate_rdl,
 						Update.RESTART_REQUIRED_YES );
 				
+				update.setDescriptionURL(sf_details.getInfoURL());
+				
 				alternate_rdl.addListener( 
 						new ResourceDownloaderAdapter()
 						{
@@ -245,6 +247,8 @@ PlatformManagerUpdateChecker
 			
 			Debug.printStackTrace( e );
 			
+			checker.reportProgress( "Failed to load plugin details for the platform manager: " + Debug.getNestedExceptionMessage(e));
+			
 			checker.failed();
 			
 		}finally{
