@@ -1,7 +1,7 @@
 /*
  * Created on 21-Jul-2004
  * Created by Paul Gardner
- * Copyright (C) 2004 Aelitis, All Rights Reserved.
+ * Copyright (C) 2004, 2005, 2006 Aelitis, All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,7 +15,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  * 
- * AELITIS, SARL au capital de 30,000 euros
+ * AELITIS, SAS au capital de 46,603.30 euros
  * 8 Allee Lenotre, La Grille Royale, 78600 Le Mesnil le Roi, France.
  *
  */
@@ -23,18 +23,21 @@
 package org.gudy.azureus2.plugins.peers;
 
 /**
- * @author parg
- * @deprecated never implemented
+ * Listener for peer events.
+ * @deprecated use PeerListener2
  */
-
-public interface 
-PeerListener 
-{
+public interface PeerListener {
+  
   /**
-   * @deprecated never implemented
-   * @param event
+   * The peer has changed to the given state.
+   * @param new_state of peer
    */
-	public void
-	eventOccurred(
-		PeerEvent	event );
+  public void stateChanged( int new_state );
+  
+  /**
+   * The peer has sent us a bad piece data chunk.
+   * @param piece_num piece that failed hash check
+   * @param total_bad_chunks total number of bad chunks sent by this peer so far
+   */
+  public void sentBadChunk( int piece_num, int total_bad_chunks );
 }
