@@ -27,6 +27,9 @@ package org.gudy.azureus2.plugins.tracker.web;
  */
 
 import java.io.InputStream;
+import java.net.InetSocketAddress;
+import java.net.URL;
+import java.util.Map;
 
 import org.gudy.azureus2.plugins.tracker.*;
 
@@ -39,15 +42,46 @@ TrackerWebPageRequest
 	public String
 	getClientAddress();
 	
+	public InetSocketAddress
+	getClientAddress2();
+	
+	public InetSocketAddress
+	getLocalAddress();
+	
+	public String
+	getUser();
+	
+		/**
+		 * This gives the relative URL of the request (e.g. /fred.html)
+		 * @return
+		 */
+	
 	public String
 	getURL();
 	
 	public String
 	getHeader();
 	
+		/**
+		 * Returns a map containing the separate headers. Keys are lowercase
+		 * @return
+		 * @since 2.3.0.7
+		 */
+	
+	public Map 
+	getHeaders();
+	
 	public InputStream
 	getInputStream();
 	
+		/**
+		 * Absolute URL including protocol and port e.g. https://a.b.c:1235/fred.html
+		 * @return
+		 */
+	
+	public URL
+	getAbsoluteURL();
+	
 	public TrackerWebContext
 	getContext();
 }
