@@ -14,6 +14,10 @@
 
 package org.gudy.azureus2.plugins.ui.tables;
 
+import org.gudy.azureus2.plugins.download.Download;
+import org.gudy.azureus2.plugins.download.DownloadTypeComplete;
+import org.gudy.azureus2.plugins.download.DownloadTypeIncomplete;
+
 /** Allows plugins to manage Azureus UI tables.
  *
  * @author TuxPaper
@@ -25,6 +29,21 @@ public interface TableManager
   public static final String TABLE_MYTORRENTS_COMPLETE   = "MySeeders";
   /** Visible for Incompleted Torrents table */
   public static final String TABLE_MYTORRENTS_INCOMPLETE = "MyTorrents";
+  
+  public static final String TABLE_MYTORRENTS_UNOPENED = "Unopened";
+  /** Visible for Completed Torrents table (big version)*/
+  public static final String TABLE_MYTORRENTS_COMPLETE_BIG   = "MySeeders.big";
+  /** Visible for Incompleted Torrents table (big version)*/
+  public static final String TABLE_MYTORRENTS_INCOMPLETE_BIG = "MyTorrents.big";
+  
+  public static final String TABLE_MYTORRENTS_UNOPENED_BIG = "Unopened.big";
+
+  public static final String TABLE_MYTORRENTS_ALL_BIG   = "MyLibrary.big";
+
+  public static final String TABLE_ACTIVITY = "Activity";
+  
+  public static final String TABLE_ACTIVITY_BIG = "Activity.big";
+
   /** Visible for Torrent Peers table */
   public static final String TABLE_TORRENT_PEERS         = "Peers";
   /** Visible for Torrent Pieces table */
@@ -35,6 +54,8 @@ public interface TableManager
   public static final String TABLE_MYTRACKER             = "MyTracker";
   /** Visible for My Shares table */
   public static final String TABLE_MYSHARES              = "MyShares";
+  /** Visible for All Peers table */
+  public static final String TABLE_ALL_PEERS             = "AllPeers";
 
   /** Creates a column for a UI table.
    * In order for this object to be displayed in an Azureus UI table, the
@@ -44,7 +65,7 @@ public interface TableManager
    * some TableColumn functions are not available or act differently after the 
    * column had been added.
    * <p>
-   * In order to the plugin to display correctly the column name, your are 
+   * In order to the plugin to display correctly the column name, you are 
    * required to create a key in your language file consisting of the
    * {@link TableManager} Table ID of the table you are adding the column to,
    * plus ".column." plus the logical name of your column. 
@@ -65,6 +86,41 @@ public interface TableManager
    * @return an interface object allowing modification of the table column.
    */
   public TableColumn createColumn(String tableID, String cellID);
+  
+  /**
+   * Register a column for a specific data source type.  When the column
+   * is created, the listener will be fired.
+   * <p>
+   * This method is independent of {@link #createColumn(String, String)} and
+   * {@link #addColumn(TableColumn)}.  This method improves upon them by
+   * saving you from creating and adding the column to every potential table
+   * it could be displayed in.  It allows for your column to be added to future
+   * tables without and code changes on your end. 
+   * 
+   * @param forDataSourceType Class of datasource you want to add a column to,
+   *                          such as {@link Download}, {@link DownloadTypeComplete},
+   *                          {@link DownloadTypeIncomplete}
+   *
+   * @param cellID The logical name of the column.
+   * 
+   * @param listener
+   *                 
+   * @return an interface object allowing modification of the table column.
+   *
+   * @since 3.1.1.1
+   */
+  public void registerColumn(Class forDataSourceType, String cellID,
+			TableColumnCreationListener listener);
+
+  /**
+   * unregister a listener added by @link {@link #registerColumn(Class, String, TableColumnCreationListener)}
+   * @param forDataSourceType
+   * @param cellID
+   * @param listener
+   * @since 4005
+   */
+  public void unregisterColumn(Class forDataSourceType, String cellID,
+			TableColumnCreationListener listener);
 
   /** Adds a column to an Azureus UI table.
    *
