@@ -30,26 +30,23 @@ import java.net.URLClassLoader;
 import java.net.URLConnection;
 import java.util.*;
 
-import com.aelitis.azureus.core.*;
-
-import org.gudy.azureus2.core3.config.COConfigurationManager;
 import org.gudy.azureus2.core3.download.DownloadManager;
 import org.gudy.azureus2.core3.global.GlobalManager;
 import org.gudy.azureus2.core3.global.GlobalManagerListener;
 import org.gudy.azureus2.core3.internat.MessageText;
-import org.gudy.azureus2.core3.util.*;
 import org.gudy.azureus2.core3.logging.*;
-
-
+import org.gudy.azureus2.core3.util.*;
 import org.gudy.azureus2.plugins.*;
 import org.gudy.azureus2.pluginsimpl.local.launch.PluginLauncherImpl;
 import org.gudy.azureus2.pluginsimpl.local.ui.UIManagerImpl;
-import org.gudy.azureus2.pluginsimpl.local.update.*;
+import org.gudy.azureus2.pluginsimpl.local.update.UpdateManagerImpl;
 import org.gudy.azureus2.pluginsimpl.local.utils.UtilitiesImpl;
-
+import org.gudy.azureus2.pluginsimpl.local.utils.UtilitiesImpl.runnableWithException;
 import org.gudy.azureus2.update.UpdaterUpdateChecker;
 import org.gudy.azureus2.update.UpdaterUtils;
 
+import com.aelitis.azureus.core.*;
+
 
 
 /**
@@ -74,68 +71,75 @@ PluginInitializer
 		// plugin id
 		// plugin key for prefixing config data
 		// report if not present
+	// force re-enable if disabled by config
 	
   private String[][]	builtin_plugins = { 
    			{	 PluginManagerDefaults.PID_START_STOP_RULES, 
    					"com.aelitis.azureus.plugins.startstoprules.defaultplugin.StartStopRulesDefaultPlugin", 
    					"azbpstartstoprules", 
    					"", 
-   					"true" },
+   					"true",
+   					"true"},
    			{	 PluginManagerDefaults.PID_REMOVE_RULES, 
    					"com.aelitis.azureus.plugins.removerules.DownloadRemoveRulesPlugin", 
    					"azbpremovalrules", 
    					"",
-					"true" },
+					"true",
+					"false"},
     		{	 PluginManagerDefaults.PID_SHARE_HOSTER, 
    					"com.aelitis.azureus.plugins.sharing.hoster.ShareHosterPlugin", 
    					"azbpsharehoster", 
    					"ShareHoster",
-					"true" },
+					"true",
+					"false"},
    			{	 PluginManagerDefaults.PID_PLUGIN_UPDATE_CHECKER, 
    					"org.gudy.azureus2.pluginsimpl.update.PluginUpdatePlugin", 
    					"azbppluginupdate", 
    					"PluginUpdate",
-					"true" },
-			{	 PluginManagerDefaults.PID_CLIENT_ID, 
-				    "com.aelitis.azureus.plugins.clientid.ClientIDPlugin", 
-				    "azbpclientid", 
-				    "Client ID",
-					"true" },
+					"true",
+					"true"},
 			{	 PluginManagerDefaults.PID_UPNP, 
 				    "com.aelitis.azureus.plugins.upnp.UPnPPlugin", 
 				    "azbpupnp", 
 				    "UPnP",
-					"true" },
+					"true",
+					"false"},
 			{	 PluginManagerDefaults.PID_DHT, 
 					"com.aelitis.azureus.plugins.dht.DHTPlugin", 
 					"azbpdht", 
 					"DHT",
-					"true" },
+					"true",
+					"false"},
 			{	 PluginManagerDefaults.PID_DHT_TRACKER, 
 					"com.aelitis.azureus.plugins.tracker.dht.DHTTrackerPlugin", 
 					"azbpdhdtracker", 
 					"DHT Tracker",
-					"true" },
+					"true",
+					"false"},
 			{	 PluginManagerDefaults.PID_MAGNET, 
 					"com.aelitis.azureus.plugins.magnet.MagnetPlugin", 
 					"azbpmagnet", 
 					"Magnet URI Handler",
-					"true" },
+					"true",
+					"false"},
 			{	 PluginManagerDefaults.PID_CORE_UPDATE_CHECKER, 
    					"org.gudy.azureus2.update.CoreUpdateChecker", 
    					"azbpcoreupdater", 
    					"CoreUpdater",
-					"true" },
+					"true",
+					"true"},
 			{	 PluginManagerDefaults.PID_CORE_PATCH_CHECKER, 
    					"org.gudy.azureus2.update.CorePatchChecker", 
    					"azbpcorepatcher", 
    					"CorePatcher",
-					"true" },
+					"true",
+					"true"},
 	   		{	 PluginManagerDefaults.PID_PLATFORM_CHECKER, 
    					"org.gudy.azureus2.platform.PlatformManagerPluginDelegate", 
    					"azplatform2", 
    					"azplatform2",
-					"true" },
+					"true",
+					"false"},
 	   		//{	 PluginManagerDefaults.PID_JPC, 
 				//	"com.aelitis.azureus.plugins.jpc.JPCPlugin", 
 				//	"azjpc", 
@@ -145,57 +149,51 @@ PluginInitializer
 					"com.aelitis.azureus.plugins.extseed.ExternalSeedPlugin", 
 					"azextseed", 
 					"azextseed",
-	   				"true" },
+	   				"true",
+	   				"false"},
 	   		{	 PluginManagerDefaults.PID_LOCAL_TRACKER, 
 	   				"com.aelitis.azureus.plugins.tracker.local.LocalTrackerPlugin", 
 	   				"azlocaltracker", 
 	   				"azlocaltracker",
-					"true" },
+					"true",
+					"false"},
 			{	 PluginManagerDefaults.PID_NET_STATUS, 
 		   			"com.aelitis.azureus.plugins.net.netstatus.NetStatusPlugin", 
 		   			"aznetstat", 
 		   			"aznetstat",
-					"true" },
+					"true",
+					"false"},
 			{	 PluginManagerDefaults.PID_BUDDY, 
 					"com.aelitis.azureus.plugins.net.buddy.BuddyPlugin", 
 					"azbuddy", 
 					"azbuddy",
-					"true" },
+					"true",
+					"false"},
 			/* disable until we can get some tracker admins to work on this
 	   		{	 PluginManagerDefaults.PID_TRACKER_PEER_AUTH, 
 					"com.aelitis.azureus.plugins.tracker.peerauth.TrackerPeerAuthPlugin", 
 					"aztrackerpeerauth", 
 					"aztrackerpeerauth",
-					"true" },
+					"true",
+					"false" },
 			*/
-        };
- 
+        }; 
   
   	// these can be removed one day
   
   private static String[][]default_version_details =
   {
-  		{ "org.gudy.azureus2.ui.webplugin.remoteui.servlet.RemoteUIServlet", 	
-  				"webui", 			"Swing Web Interface",	"1.2.3" },
-  		{ "org.ludo.plugins.azureus.AzureusIpFilterExporter", 					
-  				"safepeer", 		"SafePeer",				"1.2.4" },
-  		{ "org.gudy.azureus2.countrylocator.Plugin", 
-  				"CountryLocator", 	"Country Locator",		"1.0" },
-		{ "org.cneclipse.bdcc.BDCCPlugin", 
-  				"bdcc", 			"BitTorrent IRC Bot",	"2.1" },
 		{ "org.cneclipse.multiport.MultiPortPlugin", 
   				"multi-ports", 		"Mutli-Port Trackers",	"1.0" },
-		{ "i18nPlugin.i18nPlugin", 
-  				"i18nAZ", 			"i18nAZ",				"1.0" },
-		{ "info.baeker.markus.plugins.azureus.RSSImport", 
-  				"RSSImport", 		"RSS Importer", 		"1.0" },
   };
   
   private static PluginInitializer	singleton;
   private static AEMonitor			class_mon	= new AEMonitor( "PluginInitializer");
 
   private static List		registration_queue 	= new ArrayList();
-   
+  
+  private static List		initThreads = new ArrayList(1);
+  
   private AzureusCoreOperation core_operation;
   
   private AzureusCore		azureus_core;
@@ -212,7 +210,6 @@ PluginInitializer
   private List		s_plugins				= new ArrayList();
   private List		s_plugin_interfaces		= new ArrayList();
   
-  private List		initThreads;
   private boolean	initialisation_complete;
   
   private volatile boolean	plugins_initialised;
@@ -253,7 +250,7 @@ PluginInitializer
 	  	}else{
 	  		
 	  		try{
-	  			singleton.initializePluginFromClass( _class, INTERNAL_PLUGIN_ID, _class.getName());
+	  			singleton.initializePluginFromClass( _class, INTERNAL_PLUGIN_ID, _class.getName(), false, false, true);
 	  			
 			}catch(PluginException e ){
 	  				
@@ -282,8 +279,9 @@ PluginInitializer
 	  		try{
 	  			singleton.initializePluginFromInstance( plugin, id, plugin.getClass().getName());
 	  			
-			}catch(PluginException e ){
+			}catch( Throwable e ){
 	  				
+				Debug.out( e );
 	  		}
 	  	}
 	}finally{
@@ -298,6 +296,25 @@ PluginInitializer
 	  return( loading_builtin );
   }
   
+  public static void
+  checkAzureusVersion(
+	  String name,
+	  Properties props,
+	  boolean alert_on_fail
+  ) throws PluginException {
+	  
+	  String required_version = (String)props.get("plugin.azureus.min_version");
+	  if (required_version == null) {return;}
+	  if (Constants.compareVersions(Constants.AZUREUS_VERSION, required_version) < 0) {
+		  String plugin_name_bit = name.length() > 0 ? (name+" "):"";
+		  String msg = "Plugin " + plugin_name_bit + "requires " + Constants.APP_NAME + " version " + required_version + " or higher";
+		  if (alert_on_fail) {
+			  Logger.log(new LogAlert(LogAlert.REPEATABLE, LogAlert.AT_ERROR, msg));
+		  }
+		  throw new PluginException(msg);
+	  }
+  }
+  
   public static void
   checkJDKVersion(
 	String		name,
@@ -404,6 +421,24 @@ PluginInitializer
        
     plugin_manager = PluginManagerImpl.getSingleton( this );
     
+    String	dynamic_plugins = System.getProperty( "azureus.dynamic.plugins", null );
+    
+    if ( dynamic_plugins != null ){
+    	
+    	String[]	classes = dynamic_plugins.split( ";" );
+    	
+    	for ( String c: classes ){
+    		
+    		try{
+    			queueRegistration( Class.forName( c ));
+    			
+    		}catch( Throwable e ){
+    			
+    			Debug.out( "Registration of dynamic plugin '" + c + "' failed", e );
+    		}
+    	}
+    }
+    
     UpdaterUtils.checkBootstrapPlugins();
   }
   
@@ -414,16 +449,53 @@ PluginInitializer
 	  azureus_core.triggerLifeCycleComponentCreated( pi );
   }
   
+  protected void
+  fireOperational(
+	PluginInterfaceImpl	pi,
+	boolean				op )
+  {
+	  fireEventSupport( op?PluginEvent.PEV_PLUGIN_OPERATIONAL:PluginEvent.PEV_PLUGIN_NOT_OPERATIONAL, pi );
+  }
+  
+  public static void
+  addInitThread()
+  {
+	  synchronized( initThreads ){
+		  
+		  if ( initThreads.contains( Thread.currentThread())){
+			  
+			  Debug.out( "Already added" );
+		  }
+		  
+		  initThreads.add( Thread.currentThread());
+	  }
+  }
+  
+  public static void
+  removeInitThread()
+  {
+	  synchronized( initThreads ){
+		  
+		  initThreads.remove( Thread.currentThread());
+	  }
+  }
+  
   protected boolean
   isInitialisationThread()
   {
-	  return initThreads != null && initThreads.contains(Thread.currentThread());
+	  synchronized( initThreads ){
+
+		  return initThreads.contains(Thread.currentThread());
+	  }
   }
   
   	public List 
 	loadPlugins(
 		AzureusCore		core,
-		boolean bSkipAlreadyLoaded) 
+		boolean bSkipAlreadyLoaded,
+		boolean load_external_plugins,
+		boolean loading_for_startup,
+		boolean initialise_plugins) 
   	{
   		if ( bSkipAlreadyLoaded ){
   		
@@ -479,29 +551,36 @@ PluginInitializer
 	    
 	    	// user ones first so they override app ones if present
 	    
-	    pluginLoaded.addAll(loadPluginsFromDir(user_dir, 0, user_plugins
-				+ app_plugins, bSkipAlreadyLoaded));
-	    
-	    if ( !user_dir.equals( app_dir )){
-	    	
-	    	pluginLoaded.addAll(loadPluginsFromDir(app_dir, user_plugins,
-					user_plugins + app_plugins, bSkipAlreadyLoaded));
+	    if (load_external_plugins) {
+		    pluginLoaded.addAll(loadPluginsFromDir(user_dir, 0, user_plugins
+					+ app_plugins, bSkipAlreadyLoaded, loading_for_startup, initialise_plugins));
+		    
+		    if ( !user_dir.equals( app_dir )){
+		    	
+		    	pluginLoaded.addAll(loadPluginsFromDir(app_dir, user_plugins,
+						user_plugins + app_plugins, bSkipAlreadyLoaded, loading_for_startup, initialise_plugins));
+		    }
+	    }
+	    else {
+	    	if (Logger.isEnabled()) {
+	    		Logger.log(new LogEvent(LOGID, "Loading of external plugins skipped"));
+	    	}
 	    }
       
 	    if (Logger.isEnabled())
 	    	Logger.log(new LogEvent(LOGID, "Loading built-in plugins"));
 	    
-	    if (core_operation != null) {
-	    	core_operation.reportCurrentTask(MessageText.getString("splash.plugin")
-	    			+ MessageText.getString("ConfigView.pluginlist.column.type.builtIn"));
-	    }
-
   		PluginManagerDefaults	def = PluginManager.getDefaults();
     
   		for (int i=0;i<builtin_plugins.length;i++){
     		
   			if ( def.isDefaultPluginEnabled( builtin_plugins[i][0])){
     		
+  			    if (core_operation != null) {
+  			    	core_operation.reportCurrentTask(MessageText.getString("splash.plugin")
+  			    			+ builtin_plugins[i][0]);
+  			    }
+
   				try{
   					loading_builtin	= true;
   					
@@ -638,7 +717,9 @@ PluginInitializer
   	File	pluginDirectory,
 	int		plugin_offset,
 	int		plugin_total,
-	boolean bSkipAlreadyLoaded)
+	boolean bSkipAlreadyLoaded,
+	boolean loading_for_startup,
+	boolean initialise)
   {
   	List dirLoadedPIs = new ArrayList();
   	
@@ -677,7 +758,7 @@ PluginInitializer
 	      
 	    try{
 	    
-	    	List	loaded_pis = loadPluginFromDir(pluginsDirectory[i], bSkipAlreadyLoaded);
+	    	List	loaded_pis = loadPluginFromDir(pluginsDirectory[i], bSkipAlreadyLoaded, loading_for_startup, initialise);
 	      	
 	    		// save details for later initialisation
 	    	
@@ -701,7 +782,9 @@ PluginInitializer
   private List 
   loadPluginFromDir(
   	File directory,
-  	boolean bSkipAlreadyLoaded)
+  	boolean bSkipAlreadyLoaded,
+  	boolean loading_for_startup,
+  	boolean initialise) // initialise setting is used if loading_for_startup isnt
   
   	throws PluginException
   {
@@ -852,6 +935,7 @@ PluginInitializer
       }
 
       checkJDKVersion( pluginName, props, true );
+      checkAzureusVersion(pluginName, props, true);
       
       plugin_class_string = (String)props.get( "plugin.class");
       
@@ -1040,12 +1124,8 @@ PluginInitializer
 							pid,
 							plugin_version[0] );
 	      
-	      // Must use getPluginID() instead of pid, because they may differ
-	      boolean bEnabled = COConfigurationManager
-							.getBooleanParameter("PluginInfo."
-									+ plugin_interface.getPluginID() + ".enabled", true);
-	      
-	      plugin_interface.setDisabled(!bEnabled);
+	      boolean bEnabled = (loading_for_startup) ? plugin_interface.getPluginState().isLoadedAtStartup() : initialise;
+	      plugin_interface.getPluginState().setDisabled(!bEnabled);
 
 	      try{
 	      
@@ -1063,9 +1143,9 @@ PluginInitializer
 	      loaded_pis.add( plugin_interface );
 	      
 	      if ( load_failure != null ){
-	      		  
-	    	  	// don't complain about our internal one
+	    	  plugin_interface.setAsFailed();
 	    	  
+	    	// don't complain about our internal one
 	    	  if ( !pid.equals(UpdaterUpdateChecker.getPluginID())){
 	    		  
 		      	String msg = "Error loading plugin '" + pluginName + "' / '" + plugin_class_string + "'";
@@ -1114,9 +1194,7 @@ PluginInitializer
   initialisePlugins() 
   {
 	  try{
-		  if(initThreads == null)
-			  initThreads = new ArrayList();
-		  initThreads.add(Thread.currentThread());
+		  addInitThread();
 		  
 		  final LinkedList initQueue = new LinkedList();
 		  
@@ -1148,7 +1226,7 @@ PluginInitializer
 			
 							}
 			
-						} catch (PluginException e) {
+						} catch ( Throwable e ){
 							// already handled
 						} finally {
 							if (core_operation != null) {
@@ -1196,7 +1274,7 @@ PluginInitializer
 									Logger.log(new LogEvent(LOGID, "Initializing built-in plugin '"
 											+ builtin_plugins[idx][2] + "'" ));
 			
-								initializePluginFromClass(cla, id, key);
+								initializePluginFromClass(cla, id, key, "true".equals(builtin_plugins[idx][5]), true, true);
 			
 								if (Logger.isEnabled())
 								Logger.log(new LogEvent(LOGID, LogEvent.LT_WARNING,
@@ -1204,7 +1282,7 @@ PluginInitializer
 							} catch (Throwable e) {
 								try {
 									// replace it with a "broken" plugin instance
-									initializePluginFromClass(FailedPlugin.class, id, key);
+									initializePluginFromClass(FailedPlugin.class, id, key, false, false, true);
 			
 								} catch (Throwable f) {
 								}
@@ -1249,7 +1327,7 @@ PluginInitializer
 								Class cla = (Class) entry;
 			
 								singleton.initializePluginFromClass(cla, INTERNAL_PLUGIN_ID, cla
-										.getName());
+										.getName(), false, true, true);
 			
 							} else {
 								Object[] x = (Object[]) entry;
@@ -1265,38 +1343,66 @@ PluginInitializer
 				});
 			}
 			
-			AEThread2 secondaryInitializer = new AEThread2("2nd PluginInitializer Thread",true)
-			{
-				public void run() {
-					initThreads.add(Thread.currentThread());
-					
-					while(true)
+			AEThread2 secondaryInitializer = 
+				new AEThread2("2nd PluginInitializer Thread",true)
+				{
+					public void run() 
 					{
-						Runnable toRun;
-						synchronized (initQueue)
-						{
-							if(initQueue.isEmpty())
-								break;
-							toRun = (Runnable)initQueue.remove(0);
+						
+						try{
+							addInitThread();
+							
+							while( true ){
+							
+								Runnable toRun;
+								
+								synchronized (initQueue){
+									
+									if (initQueue.isEmpty()){
+										
+										break;
+									}
+									
+									toRun = (Runnable)initQueue.remove(0);
+								}
+								
+								try{
+									toRun.run();
+									
+								}catch( Throwable e ){
+									
+									Debug.out(e);
+								}
+							}
+						}finally{
+							
+							removeInitThread();
 						}
-						toRun.run();
 					}
-					
-					initThreads.remove(Thread.currentThread());
-				}
-			};
+				};
 			secondaryInitializer.start();
 			
-			while(true)
-			{
+			while(true){
+			
 				Runnable toRun;
-				synchronized (initQueue)
-				{
-					if(initQueue.isEmpty())
+				
+				synchronized( initQueue ){
+					
+					if( initQueue.isEmpty()){
+						
 						break;
+					}
+					
 					toRun = (Runnable)initQueue.remove(0);
 				}
-				toRun.run();
+				
+				try{
+					toRun.run();
+					
+				}catch( Throwable e ){
+					
+					Debug.out(e);
+				}
 			}
 			
 			secondaryInitializer.join();
@@ -1306,9 +1412,10 @@ PluginInitializer
 			plugins_initialised = true;
 			
 			fireEvent( PluginEvent.PEV_ALL_PLUGINS_INITIALISED );
+			
 	  }finally{
 		  
-		  initThreads = null;
+		  removeInitThread();
 	  }
 	}
   
@@ -1331,37 +1438,48 @@ PluginInitializer
   		
   		for (int i=0;i<l.size();i++){
   	
-  			PluginInterfaceImpl	plugin_interface = (PluginInterfaceImpl)l.get(i);
+  			final PluginInterfaceImpl	plugin_interface = (PluginInterfaceImpl)l.get(i);
   			
-  			if (plugin_interface.isDisabled()) {
+  			if (plugin_interface.getPluginState().isDisabled()) {
   				
   				synchronized( s_plugin_interfaces ){
-  				
+  					
   					s_plugin_interfaces.add( plugin_interface );
   				}
   				
   				continue;
   			}
-  	
-  			Plugin	plugin = plugin_interface.getPlugin();
-  			
-  			if (plugin_interface.isOperational())
+  	  			
+  			if ( plugin_interface.getPluginState().isOperational()){
+  				
   				continue;
+  			}
   			
   			Throwable	load_failure = null;
-  			
+  	
+ 			final Plugin	plugin = plugin_interface.getPlugin();
+
   			try{
       	
-				UtilitiesImpl.setPluginThreadContext( plugin_interface );
+  				UtilitiesImpl.callWithPluginThreadContext(
+  					plugin_interface,
+  					new runnableWithException<PluginException>()
+  					{
+  						public void 
+  						run() 
+  							throws PluginException 
+  						{
+  							fireCreated( plugin_interface );
 				
-				fireCreated( plugin_interface );
-				
-  				plugin.initialize(plugin_interface);
+  							plugin.initialize(plugin_interface);
       	  				
-  				if (!(plugin instanceof FailedPlugin)){
+  							if (!(plugin instanceof FailedPlugin)){
   					
-  					plugin_interface.setOperational( true );
-  				}
+  								plugin_interface.getPluginStateImpl().setOperational( true, false );
+  							}
+  						}
+  					});
+  				
   			}catch( Throwable e ){
       	
   				load_failure	= e;
@@ -1396,9 +1514,12 @@ PluginInitializer
   
   protected void 
   initializePluginFromClass(
-  	Class 	plugin_class,
-	String	plugin_id,
-	String	plugin_config_key )
+  	final Class 	plugin_class,
+	final String	plugin_id,
+	String			plugin_config_key,
+	boolean 		force_enabled,
+	boolean 		loading_for_startup,
+	boolean 		initialise)
   
   	throws PluginException
   {
@@ -1415,7 +1536,7 @@ PluginInitializer
   	    
   	try{
   	
-  		Plugin plugin = (Plugin) plugin_class.newInstance();
+  		final Plugin plugin = (Plugin) plugin_class.newInstance();
   		
   		String	plugin_name;
 
@@ -1441,7 +1562,7 @@ PluginInitializer
   		
   		properties.put( "plugin.name", plugin_name );
   		
-  		PluginInterfaceImpl plugin_interface = 
+  		final PluginInterfaceImpl plugin_interface = 
   			new PluginInterfaceImpl(
   						plugin, 
 						this,
@@ -1453,47 +1574,65 @@ PluginInitializer
 						plugin_id,
 						null );
 
-	      // Must use getPluginID() instead of pid, because they may differ
-	      boolean bEnabled = COConfigurationManager
-							.getBooleanParameter("PluginInfo."
-									+ plugin_interface.getPluginID() + ".enabled", true);
+  		boolean bEnabled = (loading_for_startup) ? plugin_interface.getPluginState().isLoadedAtStartup() : initialise; 
 	      
-	      plugin_interface.setDisabled(!bEnabled);
+	      /**
+	       * For some plugins, override any config setting which disables the plugin.
+	       */
+	      if (force_enabled && !bEnabled) {
+	    	  plugin_interface.getPluginState().setLoadedAtStartup(true);
+	    	  bEnabled = true;
+	    	  Logger.log(new LogAlert(false, LogAlert.AT_WARNING, MessageText.getString(
+	    	      "plugins.init.force_enabled", new String[] {plugin_id}
+	    	  )));
+	      }
+	      
+	      plugin_interface.getPluginState().setDisabled(!bEnabled);
   		
-		UtilitiesImpl.setPluginThreadContext( plugin_interface );
+	      final boolean f_enabled = bEnabled;
+	      
+	      UtilitiesImpl.callWithPluginThreadContext(
+	    		 plugin_interface,
+	    		 new runnableWithException<PluginException>()
+	    		 {
+	    			 public void
+	    			 run() 
+	    			 	throws PluginException 
+	    			 {
+	    				 try{
 
-		try{
-			
-			Method	load_method = plugin_class.getMethod( "load", new Class[]{ PluginInterface.class });
-		      	
-		   	load_method.invoke( plugin, new Object[]{ plugin_interface });
-		      	
-		 }catch( NoSuchMethodException e ){
-		      	
-		 }catch( Throwable e ){
-		      	
-			Debug.printStackTrace( e );
-			
-			Logger.log(new LogAlert(LogAlert.UNREPEATABLE,
-						"Load of built in plugin '" + plugin_id + "' fails", e));
-		}
-		 
-		 if (bEnabled) {
-			 
-			 if ( core_operation != null ){
-				 
-				 core_operation.reportCurrentTask(MessageText.getString("splash.plugin.init") + " " + plugin_interface.getPluginName());
-			 }
-	
-			 fireCreated( plugin_interface );
-			 
-	  		plugin.initialize(plugin_interface);
-	  		
-	  		if (!(plugin instanceof FailedPlugin)){
-	  			
-	  			plugin_interface.setOperational( true );
-	  		}
-		 }
+	    					 Method	load_method = plugin_class.getMethod( "load", new Class[]{ PluginInterface.class });
+
+	    					 load_method.invoke( plugin, new Object[]{ plugin_interface });
+
+	    				 }catch( NoSuchMethodException e ){
+
+	    				 }catch( Throwable e ){
+
+	    					 Debug.printStackTrace( e );
+
+	    					 Logger.log(new LogAlert(LogAlert.UNREPEATABLE,
+	    							 "Load of built in plugin '" + plugin_id + "' fails", e));
+	    				 }
+
+	    				 if (f_enabled) {
+
+	    					 if ( core_operation != null ){
+
+	    						 core_operation.reportCurrentTask(MessageText.getString("splash.plugin.init") + " " + plugin_interface.getPluginName());
+	    					 }
+
+	    					 fireCreated( plugin_interface );
+
+	    					 plugin.initialize(plugin_interface);
+
+	    					 if (!(plugin instanceof FailedPlugin)){
+
+	    						 plugin_interface.getPluginStateImpl().setOperational( true, false );
+	    					 }
+	    				 }
+	    			 }
+	    		 });
   		
 		 synchronized( s_plugin_interfaces ){
    		
@@ -1517,14 +1656,14 @@ PluginInitializer
   
   protected void
   initializePluginFromInstance(
-  	Plugin		plugin,
-	String		plugin_id,
-	String		plugin_config_key )
+  	final Plugin		plugin,
+	String				plugin_id,
+	String				plugin_config_key )
   
   	throws PluginException
   {
   	try{  		
-  		PluginInterfaceImpl plugin_interface = 
+  		final PluginInterfaceImpl plugin_interface = 
   			new PluginInterfaceImpl(
   						plugin, 
 						this,
@@ -1536,17 +1675,26 @@ PluginInitializer
 						plugin_id,
 						null );
   		
-		UtilitiesImpl.setPluginThreadContext( plugin_interface );
-
-		fireCreated( plugin_interface );
-		
-  		plugin.initialize(plugin_interface);
-  		
-  		if (!(plugin instanceof FailedPlugin)){
-  			
-  			plugin_interface.setOperational( true );
-  		}
-  		
+		UtilitiesImpl.callWithPluginThreadContext(
+			plugin_interface,
+			new UtilitiesImpl.runnableWithException<PluginException>()
+			{
+				public void
+				run()
+				
+					throws PluginException
+				{
+					fireCreated( plugin_interface );
+					
+			  		plugin.initialize(plugin_interface);
+			  		
+			  		if (!(plugin instanceof FailedPlugin)){
+			  			
+			  			plugin_interface.getPluginStateImpl().setOperational( true, false );
+			  		}
+				}
+			});
+			  		
   		synchronized( s_plugin_interfaces ){
    		
   			s_plugins.add( plugin );
@@ -1596,28 +1744,30 @@ PluginInitializer
   	}
   }
   
-  protected void
-  reloadPlugin(
-  	PluginInterfaceImpl		pi )
+  protected void reloadPlugin(PluginInterfaceImpl pi) throws PluginException {
+	  reloadPlugin(pi, false, true);
+  }
   
-  	throws PluginException
-  {
-  	unloadPlugin( pi );
+	protected void reloadPlugin(
+			PluginInterfaceImpl pi,
+			boolean loading_for_startup,
+			boolean initialise) throws PluginException {
   	
-  	Object key 			= pi.getInitializerKey();
-  	String config_key	= pi.getPluginConfigKey();
+	  unloadPlugin( pi );
   	
-  	if ( key instanceof File ){
+	  Object key 			= pi.getInitializerKey();
+	  String config_key	= pi.getPluginConfigKey();
+  	
+	  if ( key instanceof File ){
   		
-  		List	pis = loadPluginFromDir( (File)key, false );
+  		List	pis = loadPluginFromDir( (File)key, false, loading_for_startup, initialise);
   		
   		initialisePlugin( pis );
   		
-  	}else{
-  		
-  		initializePluginFromClass( (Class) key, pi.getPluginID(), config_key );
-  	}
-  }
+	  }else{
+  		initializePluginFromClass( (Class) key, pi.getPluginID(), config_key, false, loading_for_startup, initialise );
+	  }
+	}
  
   protected AzureusCore
   getAzureusCore()
@@ -1631,12 +1781,16 @@ PluginInitializer
   	return( azureus_core.getGlobalManager() );
   }
   
-  public static PluginInterface
-  getDefaultInterface()
-  {
+	public static PluginInterface
+	getDefaultInterface()
+	{
+  	if (singleton == null) {
+  		throw new AzureusCoreException(
+					"PluginInitializer not instantiated by AzureusCore.create yet");
+  	}
   	return( singleton.getDefaultInterfaceSupport());
-  }
-  
+	}
+
   protected PluginInterface
   getDefaultInterfaceSupport()
   {
@@ -1927,4 +2081,5 @@ PluginInitializer
 			writer.exdent();
 		}
 	}
+
 }
