@@ -63,7 +63,7 @@ DDBaseImpl
 	
 	private static DDBaseTTTorrent	torrent_transfer;
 	
-	public static DistributedDatabase
+	public static DDBaseImpl
 	getSingleton(
 		AzureusCore	azureus_core )
 	{
@@ -94,11 +94,17 @@ DDBaseImpl
 	{
 		azureus_core	= _azureus_core;
 		
-		torrent_transfer =  new DDBaseTTTorrent( _azureus_core, this );
+		torrent_transfer =  new DDBaseTTTorrent( this );
 		
 		grabDHT();
 	}
 	
+	public DDBaseTTTorrent
+	getTTTorrent()
+	{
+		return( torrent_transfer );
+	}
+	
 	protected DHTPlugin
 	grabDHT()
 	{
@@ -738,6 +744,13 @@ DDBaseImpl
 		{
 		}
 		
+		public void 
+		starts(
+			byte[] 	_key ) 
+		{
+			listener.event( new dbEvent( DistributedDatabaseEvent.ET_OPERATION_STARTS, key ));
+		}
+		
 		public void
 		valueRead(
 			DHTPluginContact	originator,
