@@ -68,6 +68,8 @@ PeerForeignDelegate
 	
 	private BitFlags	bit_flags;
 	
+	private boolean		priority_connection;
+	
 	private Map			data;
 
 	private HashMap		peer_listeners;
@@ -95,7 +97,7 @@ PeerForeignDelegate
 	{
 		NetworkManager.getSingleton().startTransferProcessing( network_connection );
 		
-		NetworkManager.getSingleton().upgradeTransferProcessing( network_connection );
+		NetworkManager.getSingleton().upgradeTransferProcessing( network_connection, manager.getPartitionID());
 	}
 	
 	protected void
@@ -473,8 +475,10 @@ PeerForeignDelegate
 		return( foreign.isSeed());
 	}
 	
-	public boolean isRelativeSeed() {
-		return isSeed(); // lazy check
+	public boolean 
+	isRelativeSeed() 
+	{
+		return( false );
 	}
 
  
@@ -900,6 +904,19 @@ PeerForeignDelegate
 	{
 	}
 	
+	public void
+	setPriorityConnection(
+		boolean		is_priority )
+	{
+		priority_connection = is_priority;
+	}
+	
+	public boolean 
+	isPriorityConnection() 
+	{
+		return( priority_connection );
+	}
+	
 	public void
 	generateEvidence(
 		IndentWriter	writer )
