@@ -26,18 +26,24 @@ package org.gudy.azureus2.pluginsimpl.local.sharing;
  *
  */
 
-import java.io.*;
-import java.net.*;
+import java.io.File;
+import java.io.IOException;
+import java.net.URL;
 import java.util.*;
 
-import org.gudy.azureus2.plugins.torrent.*;
-import org.gudy.azureus2.pluginsimpl.local.torrent.*;
-import org.gudy.azureus2.plugins.sharing.*;
-import org.gudy.azureus2.core3.util.*;
-import org.gudy.azureus2.core3.config.*;
-import org.gudy.azureus2.core3.logging.*;
+import org.gudy.azureus2.core3.config.COConfigurationListener;
+import org.gudy.azureus2.core3.config.COConfigurationManager;
+import org.gudy.azureus2.core3.config.ParameterListener;
+import org.gudy.azureus2.core3.logging.LogEvent;
+import org.gudy.azureus2.core3.logging.LogIDs;
+import org.gudy.azureus2.core3.logging.Logger;
 import org.gudy.azureus2.core3.torrent.*;
 import org.gudy.azureus2.core3.tracker.util.TRTrackerUtils;
+import org.gudy.azureus2.core3.util.*;
+import org.gudy.azureus2.plugins.sharing.*;
+import org.gudy.azureus2.plugins.torrent.*;
+import org.gudy.azureus2.pluginsimpl.local.PluginInitializer;
+import org.gudy.azureus2.pluginsimpl.local.torrent.TorrentImpl;
 
 import com.aelitis.azureus.core.AzureusCoreFactory;
 
@@ -92,6 +98,7 @@ ShareManagerImpl
 	private Map					shares 		= new HashMap();
 	
 	private shareScanner		current_scanner;
+	private boolean				scanning;
 	
 	private List				listeners	= new ArrayList();
 	
@@ -137,9 +144,7 @@ ShareManagerImpl
 						config.suspendSaving();
 					
 						config.loadConfig(this);
-				
-						checkConsistency();
-						
+										
 					}finally{
 					
 						Iterator it = shares.values().iterator();
@@ -172,6 +177,21 @@ ShareManagerImpl
 				}finally{
 					
 					initialising	= false;
+					
+					new AEThread2( "ShareManager:initScan", true )
+					{
+						public void
+						run()
+						{
+							try{
+								scanShares();
+								
+							}catch( Throwable e ){
+								
+								Debug.printStackTrace(e);
+							}
+						}
+					}.start();
 				}
 			}
 		}finally{
@@ -672,7 +692,7 @@ ShareManagerImpl
 				old_resource.delete();
 			}
 
-			ShareResourceDirContents new_resource = new ShareResourceDirContentsImpl( this, dir, recursive );
+			ShareResourceDirContents new_resource = new ShareResourceDirContentsImpl( this, dir, recursive, true );
 						
 			shares.put( name, new_resource );
 			
@@ -751,11 +771,40 @@ ShareManagerImpl
 	
 		throws ShareException
 	{
-		if (Logger.isEnabled())
-			Logger.log(new LogEvent(LOGID,
-					"ShareManager: scanning resources for changes"));
+		try{
+			this_mon.enter();
 
-		checkConsistency();
+			if ( scanning ){
+				
+				return;
+			}
+			
+			scanning = true;
+			
+		}finally{
+			
+			this_mon.exit();
+		}
+		
+		try{
+			if (Logger.isEnabled())
+				Logger.log(new LogEvent(LOGID,
+						"ShareManager: scanning resources for changes"));
+	
+			checkConsistency();
+			
+		}finally{
+			
+			try{
+				this_mon.enter();
+
+				scanning = false;
+				
+			}finally{
+				
+				this_mon.exit();
+			}
+		}
 	}
 	
 		// bit of a hack this, but to do it properly would require extensive rework to decouple the
@@ -872,8 +921,17 @@ ShareManagerImpl
 			}
 			
 			Iterator	it = share_map.iterator();
+	
+			// We don't need GlobalManager, so isCoreRunning isn't needed
+			// Hopefully all the things we need are avail on core create
+			if (!AzureusCoreFactory.isCoreAvailable()) {
+				// could probably log some stuff below, but for now
+				// be safe and lazy and just exit
+				writer.println("No Core");
+				return;
+			}
 			
-			TorrentManager tm = AzureusCoreFactory.getSingleton().getPluginManager().getDefaultPluginInterface().getTorrentManager();
+			TorrentManager tm = PluginInitializer.getDefaultInterface().getTorrentManager();
 
 			TorrentAttribute	category_attribute 	= tm.getAttribute( TorrentAttribute.TA_CATEGORY );
 			TorrentAttribute	props_attribute 	= tm.getAttribute( TorrentAttribute.TA_SHARE_PROPERTIES );
@@ -971,42 +1029,36 @@ ShareManagerImpl
 		{
 			current_scanner	= this;
 			
-			Thread t = 
-				new AEThread( "ShareManager::scanner" )
+			new AEThread2( "ShareManager::scanner", true )
+			{
+				public void
+				run()
 				{
-					public void
-					runSupport()
-					{
-						while( current_scanner == shareScanner.this ){
-						
-							try{
-								
-								int		scan_period		= COConfigurationManager.getIntParameter( "Sharing Rescan Period" );
+					while( current_scanner == shareScanner.this ){
+					
+						try{
+							
+							int		scan_period		= COConfigurationManager.getIntParameter( "Sharing Rescan Period" );
 
-								if ( scan_period < 1 ){
-									
-									scan_period	= 1;
-								}
+							if ( scan_period < 1 ){
 								
-								Thread.sleep( scan_period * 1000 );
-								
-								if ( current_scanner == shareScanner.this ){
-									
-									scanShares();
-								}
-
-							}catch( Throwable e ){
-								
-								Debug.printStackTrace(e);
+								scan_period	= 1;
 							}
+							
+							Thread.sleep( scan_period * 1000 );
+							
+							if ( current_scanner == shareScanner.this ){
+								
+								scanShares();
+							}
+
+						}catch( Throwable e ){
+							
+							Debug.printStackTrace(e);
 						}
 					}
-				};
-				
-			t.setDaemon(true);
-			
-			t.start();
-			
+				}
+			}.start();
 		}
 	}
 }
