@@ -139,17 +139,28 @@ UpdateManagerImpl
 		}
 	}
 	
-	public UpdateCheckInstance
+	public UpdateCheckInstanceImpl
 	createEmptyUpdateCheckInstance(
 		int			type,
 		String		name )
+	{
+		return( createEmptyUpdateCheckInstance( type, name, false ));
+	}
+	
+	public UpdateCheckInstanceImpl
+	createEmptyUpdateCheckInstance(
+		int			type,
+		String		name,
+		boolean		low_noise )
 	{
 		try{
 			this_mon.enter();
 	
 			UpdatableComponentImpl[]	comps = new UpdatableComponentImpl[0];
 			
-			UpdateCheckInstance	res = new UpdateCheckInstanceImpl( this, type, name, comps );
+			UpdateCheckInstanceImpl	res = new UpdateCheckInstanceImpl( this, type, name, comps );
+			
+			res.setLowNoise( low_noise );
 			
 			for (int i=0;i<listeners.size();i++){
 				
