@@ -42,12 +42,16 @@ import org.gudy.azureus2.plugins.*;
 import org.gudy.azureus2.plugins.utils.*;
 import org.gudy.azureus2.plugins.utils.resourcedownloader.*;
 import org.gudy.azureus2.plugins.utils.resourceuploader.ResourceUploaderFactory;
+import org.gudy.azureus2.plugins.utils.search.Search;
 import org.gudy.azureus2.plugins.utils.search.SearchException;
+import org.gudy.azureus2.plugins.utils.search.SearchInitiator;
+import org.gudy.azureus2.plugins.utils.search.SearchListener;
 import org.gudy.azureus2.plugins.utils.search.SearchProvider;
 import org.gudy.azureus2.plugins.utils.security.SESecurityManager;
 import org.gudy.azureus2.plugins.utils.xml.rss.RSSFeed;
 import org.gudy.azureus2.plugins.utils.xml.simpleparser.SimpleXMLParserDocumentException;
 import org.gudy.azureus2.plugins.utils.xml.simpleparser.SimpleXMLParserDocumentFactory;
+import org.gudy.azureus2.pluginsimpl.local.PluginInitializer;
 import org.gudy.azureus2.pluginsimpl.local.utils.resourcedownloader.*;
 import org.gudy.azureus2.pluginsimpl.local.utils.resourceuploader.ResourceUploaderFactoryImpl;
 import org.gudy.azureus2.pluginsimpl.local.utils.security.*;
@@ -61,6 +65,7 @@ import org.gudy.azureus2.core3.ipchecker.extipchecker.ExternalIPCheckerServiceLi
 import org.gudy.azureus2.core3.logging.LogEvent;
 import org.gudy.azureus2.core3.logging.LogIDs;
 import org.gudy.azureus2.core3.logging.Logger;
+import org.gudy.azureus2.core3.util.AEDiagnostics;
 import org.gudy.azureus2.core3.util.AEMonitor;
 import org.gudy.azureus2.core3.util.AESemaphore;
 import org.gudy.azureus2.core3.util.AEThread2;
@@ -75,6 +80,7 @@ import org.gudy.azureus2.core3.util.IPToHostNameResolverListener;
 import org.gudy.azureus2.core3.util.SystemProperties;
 import org.gudy.azureus2.core3.util.DirectByteBufferPool;
 import org.gudy.azureus2.core3.util.SystemTime;
+import org.gudy.azureus2.core3.util.TimeFormatter;
 import org.gudy.azureus2.core3.util.Timer;
 import org.gudy.azureus2.core3.util.TimerEvent;
 import org.gudy.azureus2.core3.util.TimerEventPerformer;
@@ -92,10 +98,10 @@ UtilitiesImpl
 	private AzureusCore				core;
 	private PluginInterface			pi;
 	
-	private static ThreadLocal		tls	= 
-		new ThreadLocal()
+	private static ThreadLocal<PluginInterface>		tls	= 
+		new ThreadLocal<PluginInterface>()
 		{
-			public Object
+			public PluginInterface
 			initialValue()
 			{
 				return( null );
@@ -103,8 +109,8 @@ UtilitiesImpl
 		};
 		
 		
-	private static List		search_managers 	= new ArrayList();
-	private static List		search_providers	= new ArrayList();
+	private static List<searchManager>		search_managers 	= new ArrayList<searchManager>();
+	private static List<Object[]>			search_providers	= new ArrayList<Object[]>();
 	
 	public
 	UtilitiesImpl(
@@ -285,9 +291,7 @@ UtilitiesImpl
 				public void
 				run()
 				{
-					setPluginThreadContext( pi );
-					
-					target.run();
+					callWithPluginThreadContext( pi, target );
 				}
 			};
 			
@@ -786,10 +790,79 @@ UtilitiesImpl
 	}
 	
 	public static final void
-	setPluginThreadContext(
-		PluginInterface		pi )
+	callWithPluginThreadContext(
+		PluginInterface				pi,
+		Runnable					target )
 	{
-		tls.set( pi );
+		PluginInterface existing = tls.get();
+		
+		try{
+			tls.set( pi );
+			
+			target.run();
+			
+		}finally{
+			
+			tls.set( existing );
+		}
+	}
+	
+	public static final <T extends Exception> void
+	callWithPluginThreadContext(
+		PluginInterface					pi,
+		runnableWithException<T>		target )
+	
+		throws T
+	{
+		PluginInterface existing = tls.get();
+		
+		try{
+			tls.set( pi );
+			
+			target.run();
+			
+		}finally{
+			
+			tls.set( existing );
+		}
+	}
+	
+	public static final <T> T
+	callWithPluginThreadContext(
+		PluginInterface				pi,
+		runnableWithReturn<T>			target )
+	{
+		PluginInterface existing = tls.get();
+		
+		try{
+			tls.set( pi );
+			
+			return( target.run());
+			
+		}finally{
+			
+			tls.set( existing );
+		}
+	}
+	
+	public static final <T,S extends Exception> T
+	callWithPluginThreadContext(
+		PluginInterface							pi,
+		runnableWithReturnAndException<T,S>		target )
+	
+		throws S
+	{
+		PluginInterface existing = tls.get();
+		
+		try{
+			tls.set( pi );
+			
+			return( target.run());
+			
+		}finally{
+			
+			tls.set( existing );
+		}
 	}
 	
 	public static PluginInterface
@@ -833,7 +906,7 @@ UtilitiesImpl
 	private static AESemaphore	delayed_tasks_sem	= new AESemaphore( "Utilities:delayedTask" );
 	private static AEThread2	delayed_task_thread;
 	
-	public static DelayedTaskImpl addDelayedTask(String	name, Runnable r) {
+	public static DelayedTask addDelayedTask(String	name, Runnable r) {
 		DelayedTaskImpl res = new DelayedTaskImpl(name);
 		res.setTask(r);
 		return res;
@@ -841,11 +914,12 @@ UtilitiesImpl
 	
 	private static void
 	queueTask(
-		DelayedTaskImpl		task )
+		DelayedTaskImpl		task,
+		int pos)
 	{
 		synchronized( delayed_tasks ){
 			
-			delayed_tasks.add( task );
+			delayed_tasks.add( pos == -1 ? delayed_tasks.size() : pos, task );
 			
 			delayed_tasks_sem.release();
 			
@@ -857,30 +931,39 @@ UtilitiesImpl
 						public void
 						run()
 						{
-							while( true ){
-								
-								if ( !delayed_tasks_sem.reserve( 5*1000 )){
+							try{
+								PluginInitializer.addInitThread();
+							
+								while( true ){
 									
-									synchronized( delayed_tasks ){
+									if ( !delayed_tasks_sem.reserve( 5*1000 )){
 										
-										if ( delayed_tasks.isEmpty()){
+										synchronized( delayed_tasks ){
 											
-											delayed_task_thread	= null;
-											
-											break;
+											if ( delayed_tasks.isEmpty()){
+												
+												delayed_task_thread	= null;
+												
+												break;
+											}
 										}
-									}
-								}else{
-									
-									DelayedTaskImpl	task;
-									
-									synchronized( delayed_tasks ){
+									}else{
 										
-										task = (DelayedTaskImpl)delayed_tasks.remove(0);
+										DelayedTaskImpl	task;
+										
+										synchronized( delayed_tasks ){
+											
+											task = (DelayedTaskImpl)delayed_tasks.remove(0);
+										}
+										
+										//System.out.println( TimeFormatter.milliStamp() + ": Running delayed task: " + task.getName());
+										
+										task.run();
 									}
-									
-									task.run();
 								}
+							}finally{
+								
+								PluginInitializer.removeInitThread();
 							}
 						}
 					};
@@ -898,13 +981,13 @@ UtilitiesImpl
 	
 		throws SearchException
 	{
-		List	managers;
+		List<searchManager>	managers;
 		
 		synchronized( UtilitiesImpl.class ){
 			
 			search_providers.add( new Object[]{ pi, provider  });
 			
-			managers = new ArrayList( search_managers );
+			managers = new ArrayList<searchManager>( search_managers );
 		}
 		
 		for (int i=0;i<managers.size();i++){
@@ -913,6 +996,26 @@ UtilitiesImpl
 		}
 	}
 	
+	public SearchInitiator 
+	getSearchInitiator() 
+	
+		throws SearchException 
+	{
+		List<searchManager>	managers;
+		
+		synchronized( UtilitiesImpl.class ){
+						
+			managers = new ArrayList<searchManager>( search_managers );
+		}
+		
+		if ( managers.size() == 0 ){
+			
+			throw( new SearchException( "No search managers registered - try later" ));
+		}
+		
+		return( managers.get(0));
+	}
+	
 	public static void
 	addSearchManager(
 		searchManager		manager )
@@ -936,12 +1039,39 @@ UtilitiesImpl
 	
 	public interface
 	searchManager
+		extends SearchInitiator
 	{
 		public void
 		addProvider( 
 			PluginInterface		pi,
 			SearchProvider		provider );
 	}
+		
+	
+	public interface
+	runnableWithReturn<T>
+	{
+		public T
+		run();
+	}
+	
+	public interface
+	runnableWithException<T extends Exception>
+	{
+		public void
+		run()
+		
+			throws T;
+	}
+	
+	public interface
+	runnableWithReturnAndException<T,S extends Exception>
+	{
+		public T
+		run()
+		
+			throws S;
+	}
 	
 	static class
 	DelayedTaskImpl
@@ -975,32 +1105,48 @@ UtilitiesImpl
 				throw( new RuntimeException( "Target must be set before queueing" ));
 			}	
 			
-			queueTask( this );
+			queueTask( this, -1 );
 		}
-		
+
+		public void
+		queueFirst()
+		{
+			if ( target == null ){
+				
+				throw( new RuntimeException( "Target must be set before queueing" ));
+			}	
+			
+			queueTask( this, 0 );
+		}
+
 		protected void
 		run()
 		{
-			run_time = SystemTime.getCurrentTime();
-			
-			target.run();
-			
-			long now = SystemTime.getCurrentTime();
-			
-			if (Logger.isEnabled()) {
-	     		Logger.log(	
-	     			new LogEvent(
-	     				LogIDs.PLUGIN, LogEvent.LT_INFORMATION,
-	     				"Delayed task '" + getName() + 
-	     					"': queue_time=" + ( run_time - create_time ) +
-	     					", exec_time=" + ( now - run_time )));
+			try{
+				run_time = SystemTime.getCurrentTime();
+				
+				target.run();
+				
+				long now = SystemTime.getCurrentTime();
+				
+				if (Logger.isEnabled()) {
+		     		Logger.log(	
+		     			new LogEvent(
+		     				LogIDs.PLUGIN, LogEvent.LT_INFORMATION,
+		     				"Delayed task '" + getName() + 
+		     					"': queue_time=" + ( run_time - create_time ) +
+		     					", exec_time=" + ( now - run_time )));
+				}
+			}catch( Throwable e ){
+				
+				Debug.out( "Initialisation task " + getName() + " failed to complete", e );
 			}
 		}
 		
 		protected String
 		getName()
 		{
-			return( name );
+			return( name + " (" + target.getClass() + ")" );
 		}
 
 	}
