@@ -174,10 +174,12 @@ ResourceDownloaderAlternateImpl
 		}
 	}
 	
-	protected void
+	public void
 	setProperty(
 		String	name,
 		Object	value )
+	
+		throws ResourceDownloaderException
 	{
 		setPropertySupport( name, value );
 		
@@ -272,7 +274,7 @@ ResourceDownloaderAlternateImpl
 		try{
 			this_mon.enter();
 		
-			result	= new ResourceDownloaderException( "Download cancelled");
+			result	= new ResourceDownloaderCancelledException();
 			
 			cancelled	= true;
 			
