@@ -33,7 +33,6 @@ import org.gudy.azureus2.plugins.disk.DiskManager;
 import org.gudy.azureus2.plugins.disk.DiskManagerFileInfo;
 import org.gudy.azureus2.plugins.download.*;
 import org.gudy.azureus2.plugins.download.savelocation.*;
-import org.gudy.azureus2.plugins.download.session.SessionAuthenticator;
 import org.gudy.azureus2.plugins.peers.PeerManager;
 import org.gudy.azureus2.plugins.torrent.*;
 
@@ -947,10 +946,6 @@ RPDownload
 		notSupported();
 	}
     
-    public void setSessionAuthenticator( SessionAuthenticator auth ) {
-      notSupported();  //TODO ?
-    }
-
 		/* (non-Javadoc)
 		 * @see org.gudy.azureus2.plugins.download.Download#setSeedingRank(int)
 		 */
