@@ -81,4 +81,14 @@ SFPluginDetails
 	
 		throws SFPluginDetailsException;
 
+	public String
+	getRelativeURLBase();
+
+	/**
+	 * @return
+	 *
+	 * @since 3.0.1.7
+	 */
+	String
+	getInfoURL();
 }
