@@ -30,8 +30,10 @@ import java.net.SocketException;
 import java.util.*;
 
 import com.aelitis.azureus.core.*;
+import com.aelitis.azureus.core.impl.AzureusCoreSingleInstanceClient;
 import com.aelitis.azureus.ui.UIFunctions;
 import com.aelitis.azureus.ui.UIFunctionsManager;
+import com.aelitis.azureus.ui.swt.UIFunctionsSWT;
 
 import org.gudy.azureus2.core3.util.*;
 import org.gudy.azureus2.core3.config.COConfigurationManager;
@@ -47,7 +49,7 @@ public class
 StartServer
 {
 	private static final LogIDs LOGID = LogIDs.GUI;
-  public static final String ACCESS_STRING = "Azureus Start Server Access";
+
   private ServerSocket socket;
   private int state;
 
@@ -63,7 +65,10 @@ StartServer
   StartServer()
     {
     try {
-        socket = new ServerSocket(6880, 50, InetAddress.getByName("127.0.0.1")); //NOLAR: only bind to localhost
+    	// DON'T USE LOGGER HERE DUE TO COMMENTS BELOW - IF AZ ALREADY RUNNING THEN THE SERVERSOCKET
+    	// CALL WILL THROW AN EXCEPTION 
+    	
+    	socket = new ServerSocket(6880, 50, InetAddress.getByName("127.0.0.1")); //NOLAR: only bind to localhost
         
         state = STATE_LISTENING;    
         
@@ -72,11 +77,15 @@ StartServer
         			+ "127.0.0.1:6880 for passed torrent info"));
     
     }catch (Throwable t) {
+    	
+		// DON'T USE LOGGER here as we DON't want to initialise all the logger stuff
+		// and in particular AEDiagnostics config dirty stuff!!!!
+
       state = STATE_FAULTY;
       String reason = t.getMessage() == null ? "<>" : t.getMessage();
-      Logger.log(new LogEvent(LOGID, LogEvent.LT_ERROR,
-					"StartServer ERROR: unable" + " to bind to 127.0.0.1:6880 listening"
-							+ " for passed torrent info: " + reason));
+
+      System.out.println( "StartServer ERROR: unable" + " to bind to 127.0.0.1:6880 listening"
+							+ " for passed torrent info: " + reason);
     }
   }
 
@@ -88,11 +97,15 @@ StartServer
         azureus_core.addLifecycleListener(
 		    	new AzureusCoreLifecycleAdapter()
 				{
-		    		public void
-					started(
-						AzureusCore		core )
-		    		{
-		    			openQueuedTorrents( azureus_core );
+					public void
+					componentCreated(
+						AzureusCore 			core, 
+						AzureusCoreComponent	component) 
+					{
+						if ( component instanceof UIFunctionsSWT ){
+		    			
+							openQueuedTorrents();
+						}
 		    		}
 				});
         
@@ -104,7 +117,7 @@ StartServer
 		    		public void 
 		    		runSupport()
 					{
-		    			pollForConnectionsSupport( azureus_core );
+		    			pollForConnectionsSupport();
 		    		}
 				};
 		    
@@ -115,8 +128,7 @@ StartServer
 	}
     
   private void 
-  pollForConnectionsSupport(
-	AzureusCore		azureus_core ) 
+  pollForConnectionsSupport() 
   {
     bContinue = true;
     while (bContinue) {
@@ -134,33 +146,15 @@ StartServer
 								+ line + "'"));
       	 
           if (line != null) {
-            StringTokenizer st = new StringTokenizer(line, ";");           
-            int i = 0;
-            if(st.countTokens() > 1) {
-            	String args[] = new String[st.countTokens() - 1];
-            	String checker = st.nextToken();
-            	if(checker.equals(ACCESS_STRING)) {
-                	
-            		String debug_str = "";
-                  	
-            		while (st.hasMoreElements()) {              
-            			String bit = st.nextToken().replaceAll("&;", ";").replaceAll("&&", "&");
-                    
-            			debug_str += (debug_str.length()==0?"":" ; ") + bit;
-                    
-            			args[i++] = bit;
-            		}
-                  
-            		Logger.log(new LogEvent(LOGID,
-										"Main::startServer: decoded to '" + debug_str + "'"));
-                  
-            		if( !COConfigurationManager.getBooleanParameter( "add_torrents_silently" ) ) {
-            			showMainWindow();
-                  }
-              	                  
-                  processArgs(azureus_core,args);
-                }
-            }
+        	  String [] args = parseArgs(line);
+        	  if (args != null && args.length > 0) {
+        		  String debug_str = args[0];
+        		  for (int i=1; i<args.length; i++) {
+        			  debug_str += " ; " + args[i];
+        		  }
+        		  Logger.log(new LogEvent(LOGID, "Main::startServer: decoded to '" + debug_str + "'"));
+        		  processArgs(args);
+        	  }
           }
         }
         sck.close();
@@ -179,18 +173,42 @@ StartServer
       }
     }
   }
+  
+  private static String[] parseArgs(String line) {
+	  if (!line.startsWith(AzureusCoreSingleInstanceClient.ACCESS_STRING + ";")) {return null;}
 
+	  // I'm sure there's a lovely regex which could do this, but I can't be bothered to figure
+	  // it out.
+	  ArrayList parts = new ArrayList();
+	  StringBuffer buf = new StringBuffer();
+	  boolean escape_mode = false;
+	  char c;
+	  for (int i=AzureusCoreSingleInstanceClient.ACCESS_STRING.length() + 1; i<line.length(); i++) {
+		  c = line.charAt(i);
+		  if (escape_mode) {buf.append(c); escape_mode = false;}
+		  else if (c == '&') {escape_mode = true;}
+		  else if (c == ';') {parts.add(buf.toString()); buf.setLength(0);}
+		  else {buf.append(c);}
+	  }
+	  if (buf.length() > 0) {parts.add(buf.toString());}
+	  return (String[])parts.toArray(new String[parts.size()]);
+		  
+  }
   
   protected void 
   processArgs(
-	AzureusCore		azureus_core,
    	String 			args[]) 
   {
     if (args.length < 1 || !args[0].equals( "args" )){
     	
     	return;
     }
-           	
+    
+    if (args.length == 1
+				|| !COConfigurationManager.getBooleanParameter("add_torrents_silently")) {
+			showMainWindow();
+		}
+
     boolean	open	= true;
     
     for (int i = 1; i < args.length; i++) {
@@ -203,7 +221,7 @@ StartServer
 	
 	  	    	UIFunctions uiFunctions = UIFunctionsManager.getUIFunctions();
 	  	    	if (uiFunctions != null) {
-	  	    		uiFunctions.requestShutdown();
+	  	    		uiFunctions.dispose(false, false);
 	  	    	}
 	  	    	
 	  	    	return;
@@ -222,7 +240,48 @@ StartServer
   	    
         String file_name = arg;
         
-        if( file_name.toUpperCase().startsWith( "HTTP:" ) || file_name.toUpperCase().startsWith( "MAGNET:" ) ) {
+        File file = new File(file_name);
+        
+        if ( !file.exists() && !isURI( file_name )){
+        	
+        		// handle hex info hashes
+	        
+	        if ( file_name.length() == 40 ){
+	        
+	        	byte[]	hash = null;
+	        	
+	        	try{
+	        		hash = ByteFormatter.decodeString( file_name );
+	        		
+	        	}catch( Throwable e ){
+	        	}
+	        	
+	        	if ( hash != null && hash.length == 20 ){
+	        		
+	        		file_name = "magnet:?xt=urn:btih:" + Base32.encode( hash );
+	        	}
+	        }
+	        
+	        	// handle base32 info hash
+	       
+	        if ( file_name.length() == 32 ){
+	            
+	        	byte[]	hash = null;
+	        	
+	        	try{
+	        		hash = Base32.decode( file_name );
+	        		
+	        	}catch( Throwable e ){
+	        	}
+	        	
+	        	if ( hash != null && hash.length == 20 ){
+	        		
+	        		file_name = "magnet:?xt=urn:btih:" +file_name;
+	        	}
+	        }
+        }
+        
+        if ( isURI( file_name )) {
         	
         	if (Logger.isEnabled())
 						Logger.log(new LogEvent(LOGID, "StartServer: args[" + i
@@ -231,7 +290,7 @@ StartServer
         }else{
 
             try {
-              File file = new File(file_name);
+              
 
               if (!file.exists()) {
 
@@ -269,14 +328,25 @@ StartServer
 
         if ( !queued ){
  
-        	handleFile( azureus_core, file_name, open );
+        	handleFile( file_name, open );
         }
       }
   }
   
+  protected boolean
+  isURI(
+	String	file_name )
+  {
+      String file_name_lower = file_name.toLowerCase();
+      
+	  return( file_name_lower.startsWith( "http:" ) || 
+			  file_name_lower.startsWith( "https:" ) || 
+			  file_name_lower.startsWith( "magnet:" ) ||
+			  file_name_lower.startsWith( "dht:" ));
+  }
+  
   protected void
   handleFile(
-	AzureusCore	azureus_core,
 	String		file_name,
 	boolean		open )
   {
@@ -292,11 +362,11 @@ StartServer
       		
       		if ( f.isDirectory()){
       			
-      			ShareUtils.shareDir( azureus_core, file_name );
+      			ShareUtils.shareDir( file_name );
       			
       		}else{
       			
-         		ShareUtils.shareFile( azureus_core, file_name );            		 
+         		ShareUtils.shareFile( file_name );            		 
       		}
       	}
       } catch (Throwable e) {
@@ -306,8 +376,7 @@ StartServer
   }
   
   protected void
-  openQueuedTorrents(
-	AzureusCore		azureus_core )
+  openQueuedTorrents()
   {
     try{
       	this_mon.enter();
@@ -326,7 +395,7 @@ StartServer
     	String	file_name 	= (String)entry[0];
     	boolean	open		= ((Boolean)entry[1]).booleanValue();
     	
-    	handleFile( azureus_core, file_name, open );
+    	handleFile( file_name, open );
     }
   }
   
@@ -352,5 +421,39 @@ StartServer
   public int getState() {
     return state;
   }
+  
+  // Test argument parsing code.
+  public static void main(String [] args) {
+	  String[] input_tests = new String[] {
+		  "a;b;c",
+		  "test",
+		  AzureusCoreSingleInstanceClient.ACCESS_STRING + ";b;c;d", // Simple test
+		  AzureusCoreSingleInstanceClient.ACCESS_STRING + ";b;c&;d;e", // Less simple test
+		  AzureusCoreSingleInstanceClient.ACCESS_STRING + ";b;c&&;d;e", // Even less simple test
+		  AzureusCoreSingleInstanceClient.ACCESS_STRING + ";b;c&&&;d;e", // Awkward test
+	  };
+	  
+	  String[][] output_results = new String[][] {
+        null,
+        null,
+        new String[] {"b", "c", "d"},
+        new String[] {"b", "c;d", "e"},
+        new String[] {"b", "c&", "d", "e"},
+        new String[] {"b", "c&;d", "e"},
+	  };
+	  
+	  for (int i=0; i<input_tests.length; i++) {
+		  System.out.println("Testing: " + input_tests[i]);
+		  String[] result = parseArgs(input_tests[i]);
+		  if (result == output_results[i]) {continue;}
+		  if (Arrays.equals(result, output_results[i])) {continue;}
+		  System.out.println("TEST FAILED");
+		  System.out.println("  Expected: " + Arrays.asList(output_results[i]));
+		  System.out.println("  Decoded : " + Arrays.asList(result));
+		  System.exit(1);
+	  }
+	  
+	  System.out.println("Done.");
+  }
 
 }
