@@ -24,7 +24,8 @@ import java.io.File;
 import java.io.IOException;
 import java.net.InetAddress;
 import java.net.URLEncoder;
-import java.util.Arrays;
+import java.util.*;
+import java.util.List;
 
 import org.eclipse.swt.SWT;
 import org.eclipse.swt.events.SelectionAdapter;
@@ -34,6 +35,7 @@ import org.eclipse.swt.widgets.*;
 import org.gudy.azureus2.core3.category.Category;
 import org.gudy.azureus2.core3.category.CategoryManager;
 import org.gudy.azureus2.core3.config.COConfigurationManager;
+import org.gudy.azureus2.core3.disk.DiskManagerFileInfo;
 import org.gudy.azureus2.core3.download.DownloadManager;
 import org.gudy.azureus2.core3.download.DownloadManagerState;
 import org.gudy.azureus2.core3.internat.MessageText;
@@ -41,8 +43,7 @@ import org.gudy.azureus2.core3.logging.LogAlert;
 import org.gudy.azureus2.core3.logging.Logger;
 import org.gudy.azureus2.core3.peer.PEPeerManager;
 import org.gudy.azureus2.core3.peer.PEPeerSource;
-import org.gudy.azureus2.core3.torrent.TOTorrent;
-import org.gudy.azureus2.core3.torrent.TOTorrentFactory;
+import org.gudy.azureus2.core3.torrent.*;
 import org.gudy.azureus2.core3.tracker.client.TRTrackerAnnouncer;
 import org.gudy.azureus2.core3.tracker.util.TRTrackerUtils;
 import org.gudy.azureus2.core3.util.*;
@@ -52,7 +53,6 @@ import org.gudy.azureus2.ui.swt.mainwindow.TorrentOpener;
 import org.gudy.azureus2.ui.swt.maketorrent.MultiTrackerEditor;
 import org.gudy.azureus2.ui.swt.maketorrent.TrackerEditorListener;
 import org.gudy.azureus2.ui.swt.minibar.DownloadBar;
-import org.gudy.azureus2.ui.swt.shells.InputShell;
 import org.gudy.azureus2.ui.swt.shells.MessageBoxShell;
 import org.gudy.azureus2.ui.swt.views.ViewUtils;
 import org.gudy.azureus2.ui.swt.views.utils.ManagerUtils;
@@ -168,10 +168,6 @@ public class TorrentUtil {
 					Debug.printStackTrace(ex);
 				}
 
-				if (dm.getTrackerClient() == null) {
-					changeUrl = false;
-				}
-
 				if (barsOpened && !DownloadBar.getManager().isOpen(dm)) {
 					barsOpened = false;
 				}
@@ -202,8 +198,7 @@ public class TorrentUtil {
 
 				}
 				int state = dm.getState();
-				bChangeDir &= (state == DownloadManager.STATE_ERROR || state == DownloadManager.STATE_STOPPED || state == DownloadManager.STATE_QUEUED)
-						&& dm.isDownloadComplete(false);
+				bChangeDir &= (state == DownloadManager.STATE_ERROR || state == DownloadManager.STATE_STOPPED || state == DownloadManager.STATE_QUEUED);;
 
 				/**
 				 * Only perform a test on disk if:
@@ -212,7 +207,7 @@ public class TorrentUtil {
 				 *       amounts of files across multiple torrents before we generate a menu.
 				 */
 				if (bChangeDir && dms.length == 1) {
-					bChangeDir = !dm.filesExist();
+					bChangeDir = dm.isDataAlreadyAllocated() && !dm.filesExist( true );
 				}
 
 				boolean scan = dm.getDownloadState().getFlag(DownloadManagerState.FLAG_SCAN_INCOMPLETE_PIECES);
@@ -290,7 +285,7 @@ public class TorrentUtil {
 				public void run(DownloadManager dm) {
 					UIFunctions uiFunctions = UIFunctionsManager.getUIFunctions();
 					if (uiFunctions != null) {
-						uiFunctions.openManagerView(dm);
+						uiFunctions.openView(UIFunctions.VIEW_DM_DETAILS, dm);
 					}
 				}
 			});
@@ -385,9 +380,10 @@ public class TorrentUtil {
 		Messages.setLanguageText(itemChangeTracker, "MyTorrentsView.menu.changeTracker"); //$NON-NLS-1$
 		Utils.setMenuItemImage(itemChangeTracker, "add_tracker");
 		itemChangeTracker.addListener(SWT.Selection, new DMTask(dms) {
-			public void run(DownloadManager dm) {
-				TRTrackerAnnouncer tc = dm.getTrackerClient();
-				if (tc != null) new TrackerChangerWindow(composite.getDisplay(), tc);
+			public void run(DownloadManager[] dms) {
+				if ( dms.length > 0 ){
+					new TrackerChangerWindow(composite.getDisplay(), dms);
+				}
 			}
 		});
 		itemChangeTracker.setEnabled(changeUrl);
@@ -396,29 +392,87 @@ public class TorrentUtil {
 		Messages.setLanguageText(itemEditTracker, "MyTorrentsView.menu.editTracker");
 		Utils.setMenuItemImage(itemEditTracker, "edit_trackers");
 		itemEditTracker.addListener(SWT.Selection, new DMTask(dms) {
-			public void run(final DownloadManager dm) {
-				if (dm.getTorrent() != null) {
-					final TOTorrent torrent = dm.getTorrent();
-
-					java.util.List group = TorrentUtils.announceGroupsToList(torrent);
-
-					new MultiTrackerEditor(null, group, new TrackerEditorListener() {
-						public void trackersChanged(String str, String str2, java.util.List group) {
-							TorrentUtils.listToAnnounceGroups(group, torrent);
-
-							try {
-								TorrentUtils.writeToFile(torrent);
-							}
-							catch (Throwable e) {
-								Debug.printStackTrace(e);
-							}
-
-							if (dm.getTrackerClient() != null) dm.getTrackerClient().resetTrackerUrl(true);
+			public void 
+			run(
+				final DownloadManager[] dms ) 
+			{
+				Map<String,List<DownloadManager>>	same_map = new HashMap<String, List<DownloadManager>>();
+				
+				for ( DownloadManager dm: dms ){
+					
+					TOTorrent torrent = dm.getTorrent();
+					
+					if ( torrent == null ){
+						
+						continue;
+					}
+					
+					List<List<String>> group = TorrentUtils.announceGroupsToList(torrent);
+					
+					String str = "";
+					
+					for ( List<String> l: group ){
+						str += "[[";
+						
+						for ( String s:l ){
+							str += s + ", ";
 						}
-					}, true);
+					}
+					
+					List<DownloadManager> dl = same_map.get( str );
+					
+					if ( dl == null ){
+						
+						dl = new ArrayList<DownloadManager>();
+						
+						same_map.put( str, dl );
+					}
+					
+					dl.add(dm );
 				}
-			} // run
+				
+				for ( final List<DownloadManager> set: same_map.values()){
+										
+					TOTorrent torrent = set.get(0).getTorrent();
+
+					List<List<String>> group = TorrentUtils.announceGroupsToList(torrent);
+
+					new MultiTrackerEditor(
+						null, 
+						group, 
+						new TrackerEditorListener() 
+						{
+							public void 
+							trackersChanged(
+								String 				str, 
+								String 				str2, 
+								List<List<String>> 	group ) 
+							{
+								for ( DownloadManager dm: set ){
+									
+									TOTorrent torrent = dm.getTorrent();
+									
+									TorrentUtils.listToAnnounceGroups(group, torrent);
+		
+									try{
+										TorrentUtils.writeToFile(torrent);
+										
+									}catch (Throwable e){
+										
+										Debug.printStackTrace(e);
+									}
+		
+									if ( dm.getTrackerClient() != null){
+										
+										dm.getTrackerClient().resetTrackerUrl(true);
+									}
+								}
+							}
+						}, true);
+				}
+			} 
 		});
+		
 		itemEditTracker.setEnabled(hasSelection);
 
 		final MenuItem itemManualUpdate = new MenuItem(menuTracker, SWT.PUSH);
@@ -542,7 +596,7 @@ public class TorrentUtil {
 		Messages.setLanguageText(itemCheckFilesExist, "MyTorrentsView.menu.checkfilesexist");
 		itemCheckFilesExist.addListener(SWT.Selection, new DMTask(dms) {
 			public void run(DownloadManager dm) {
-				dm.filesExist();
+				dm.filesExist( true );
 			}
 		});
 
@@ -557,6 +611,20 @@ public class TorrentUtil {
 		itemFileRescan.setSelection(allScanSelected);
 		itemFileRescan.setEnabled(fileRescan);
 
+			// clear allocation
+		
+		MenuItem itemFileClearAlloc = new MenuItem(menuFiles, SWT.PUSH);
+		Messages.setLanguageText(itemFileClearAlloc, "MyTorrentsView.menu.clear_alloc_data");
+		itemFileClearAlloc.addListener(SWT.Selection, new DMTask(dms) {
+			public void run(DownloadManager dm) {
+				dm.setDataAlreadyAllocated( false );
+			}
+		});
+		
+		itemFileClearAlloc.setEnabled(allStopped);
+		
+			// clear resume
+		
 		MenuItem itemFileClearResume = new MenuItem(menuFiles, SWT.PUSH);
 		Messages.setLanguageText(itemFileClearResume, "MyTorrentsView.menu.clear_resume_data");
 		itemFileClearResume.addListener(SWT.Selection, new DMTask(dms) {
@@ -620,9 +688,9 @@ public class TorrentUtil {
 				final boolean change_displayed_name = ((Boolean) mi.getData("display_name")).booleanValue();
 				final boolean change_save_name = ((Boolean) mi.getData("save_name")).booleanValue();
 				String msg_key_prefix = "MyTorrentsView.menu.rename." + (String) mi.getData("msg_key") + ".enter.";
-				SimpleTextEntryWindow text_entry = new SimpleTextEntryWindow(composite.getDisplay());
+				SimpleTextEntryWindow text_entry = new SimpleTextEntryWindow();
 				text_entry.setTitle(msg_key_prefix + "title");
-				text_entry.setMessages(new String[] { msg_key_prefix + "message", msg_key_prefix + "message.2" });
+				text_entry.setMessage(msg_key_prefix + "message");
 				text_entry.setPreenteredText(suggested, false);
 				text_entry.prompt();
 				if (text_entry.hasSubmittedInput()) {
@@ -680,7 +748,7 @@ public class TorrentUtil {
 			itemExportXML.addListener(SWT.Selection, new DMTask(dms) {
 				public void run(DownloadManager[] dms) {
 					DownloadManager dm = dms[0]; // First only.
-					if (dm != null) new ExportTorrentWizard(azureus_core, itemExportXML.getDisplay(), dm);
+					if (dm != null) new ExportTorrentWizard(itemExportXML.getDisplay(), dm);
 				}
 			});
 
@@ -842,7 +910,7 @@ public class TorrentUtil {
 				public void run(DownloadManager[] dms) {
 					UIFunctions uiFunctions = UIFunctionsManager.getUIFunctions();
 					
-					uiFunctions.showMultiOptionsView( dms );
+					uiFunctions.openView(UIFunctions.VIEW_DM_MULTI_OPTIONS, dms);
 				}
 			});
 		}
@@ -969,11 +1037,15 @@ public class TorrentUtil {
 				"MyTorrentsView.menu.reposition.manual");
 		itemPositionManual.addSelectionListener(new SelectionAdapter() {
 			public void widgetSelected(SelectionEvent e) {
-				InputShell is = new InputShell(
+				SimpleTextEntryWindow entryWindow = new SimpleTextEntryWindow(
 						"MyTorrentsView.dialog.setPosition.title",
 						"MyTorrentsView.dialog.setPosition.text");
+				entryWindow.prompt();
+				if (!entryWindow.hasSubmittedInput()) {
+					return;
+				}
+				String sReturn = entryWindow.getSubmittedInput();
 
-				String sReturn = is.open();
 				if (sReturn == null)
 					return;
 
@@ -1097,31 +1169,6 @@ public class TorrentUtil {
 
 		addCategorySubMenu(dms, menuCategory, composite);
 		
-		// Rename
-		final MenuItem itemRenameAll = new MenuItem(menu, SWT.CASCADE);
-		Messages.setLanguageText(itemRenameAll, "MyTorrentsView.menu.rename");
-		itemRenameAll.setEnabled(hasSelection);
-		if (itemRenameAll.isEnabled()) {
-			itemRenameAll.setData("suggested_text", first_selected.getDisplayName());
-			itemRenameAll.setData("display_name", Boolean.FALSE);
-			itemRenameAll.setData("save_name", Boolean.FALSE);
-			itemRenameAll.setData("rename_all", Boolean.TRUE);
-			itemRenameAll.setData("msg_key", "displayed");
-		}
-
-		itemRenameAll.addListener(SWT.Selection, rename_listener);
-		
-		// Edit Comment
-		final MenuItem itemEditComment = new MenuItem(menu, SWT.CASCADE);
-		Messages.setLanguageText(itemEditComment, "MyTorrentsView.menu.edit_comment");
-		itemEditComment.setEnabled(dms.length > 0);
-
-		itemEditComment.addListener(SWT.Selection, new DMTask(dms) {
-			public void run(DownloadManager[] dms) {
-				promptUserForComment(dms);
-			}
-		});
-
 		// ---
 		new MenuItem(menu, SWT.SEPARATOR);
 
@@ -1343,14 +1390,25 @@ public class TorrentUtil {
 		dDialog.setMessage(MessageText.getString("MainWindow.dialog.choose.savepath"));
 		String sSavePath = dDialog.open();
 		if (sSavePath != null) {
+			File fSavePath = new File(sSavePath);
 			for (int i = 0; i < dms.length; i++) {
 				DownloadManager dm = dms[i];
 				if (dm.getState() == DownloadManager.STATE_ERROR) {
-
+					
 					dm.setTorrentSaveDir(sSavePath);
+					
+					boolean found = dm.filesExist(true);
+					if (!found && dm.getTorrent() != null
+							&& !dm.getTorrent().isSimpleTorrent()) {
+						sSavePath = fSavePath.getParent();
+						if (sSavePath != null) {
+							dm.setTorrentSaveDir(sSavePath);
+							found = dm.filesExist(true);
+						}
+					}
 
-					if (dm.filesExist()) {
 
+					if (found) {
 						dm.stopIt(DownloadManager.STATE_STOPPED, false, false);
 
 						ManagerUtils.queue(dm, shell);
@@ -1379,7 +1437,7 @@ public class TorrentUtil {
 		task.go();
 		UIFunctions uiFunctions = UIFunctionsManager.getUIFunctions();
 		if (uiFunctions != null) {
-			uiFunctions.showMyTracker();
+			uiFunctions.openView(UIFunctions.VIEW_MYTRACKER, null);
 		}
 	}
 
@@ -1393,7 +1451,7 @@ public class TorrentUtil {
 		task.go();
 		UIFunctions uiFunctions = UIFunctionsManager.getUIFunctions();
 		if (uiFunctions != null) {
-			uiFunctions.showMyTracker();
+			uiFunctions.openView(UIFunctions.VIEW_MYTRACKER, null);
 		}
 	}
 
@@ -1455,7 +1513,7 @@ public class TorrentUtil {
 		// Create dialog box.
 		String suggested = dm.getDownloadState().getUserComment(); 
 		String msg_key_prefix = "MyTorrentsView.menu.edit_comment.enter.";
-		SimpleTextEntryWindow text_entry = new SimpleTextEntryWindow(Display.getCurrent());
+		SimpleTextEntryWindow text_entry = new SimpleTextEntryWindow();
 		text_entry.setTitle(msg_key_prefix + "title");
 		text_entry.setMessage(msg_key_prefix + "message");
 		text_entry.setPreenteredText(suggested, false);
