@@ -21,84 +21,33 @@
 
 package org.gudy.azureus2.ui.swt;
 
-import java.util.ArrayList;
-import java.util.Arrays;
+import java.util.*;
 import java.util.List;
-import java.util.StringTokenizer;
 
 import org.eclipse.swt.SWT;
 import org.eclipse.swt.SWTException;
 import org.eclipse.swt.custom.ScrolledComposite;
-import org.eclipse.swt.dnd.Clipboard;
-import org.eclipse.swt.dnd.DND;
-import org.eclipse.swt.dnd.DropTarget;
-import org.eclipse.swt.dnd.DropTargetAdapter;
-import org.eclipse.swt.dnd.DropTargetEvent;
-import org.eclipse.swt.dnd.DropTargetListener;
-import org.eclipse.swt.dnd.FileTransfer;
-import org.eclipse.swt.dnd.HTMLTransfer;
-import org.eclipse.swt.dnd.TextTransfer;
-import org.eclipse.swt.dnd.Transfer;
-import org.eclipse.swt.events.ControlAdapter;
-import org.eclipse.swt.events.ControlEvent;
-import org.eclipse.swt.events.DisposeEvent;
-import org.eclipse.swt.events.DisposeListener;
-import org.eclipse.swt.graphics.Color;
-import org.eclipse.swt.graphics.Cursor;
-import org.eclipse.swt.graphics.Device;
-import org.eclipse.swt.graphics.Font;
-import org.eclipse.swt.graphics.FontData;
-import org.eclipse.swt.graphics.GC;
-import org.eclipse.swt.graphics.Image;
-import org.eclipse.swt.graphics.ImageData;
-import org.eclipse.swt.graphics.PaletteData;
-import org.eclipse.swt.graphics.Point;
-import org.eclipse.swt.graphics.RGB;
-import org.eclipse.swt.graphics.Rectangle;
-import org.eclipse.swt.graphics.Region;
-import org.eclipse.swt.graphics.Resource;
-import org.eclipse.swt.graphics.TextLayout;
-import org.eclipse.swt.layout.FormAttachment;
-import org.eclipse.swt.layout.FormData;
-import org.eclipse.swt.layout.GridData;
-import org.eclipse.swt.layout.GridLayout;
+import org.eclipse.swt.dnd.*;
+import org.eclipse.swt.events.*;
+import org.eclipse.swt.graphics.*;
+import org.eclipse.swt.layout.*;
 import org.eclipse.swt.program.Program;
-import org.eclipse.swt.widgets.Composite;
-import org.eclipse.swt.widgets.Control;
-import org.eclipse.swt.widgets.Display;
-import org.eclipse.swt.widgets.Event;
-import org.eclipse.swt.widgets.Label;
-import org.eclipse.swt.widgets.Listener;
-import org.eclipse.swt.widgets.MenuItem;
-import org.eclipse.swt.widgets.Monitor;
-import org.eclipse.swt.widgets.ScrollBar;
-import org.eclipse.swt.widgets.Shell;
-import org.eclipse.swt.widgets.Table;
-import org.eclipse.swt.widgets.TableItem;
-import org.eclipse.swt.widgets.Text;
-import org.eclipse.swt.widgets.Widget;
+import org.eclipse.swt.widgets.*;
+
 import org.gudy.azureus2.core3.config.COConfigurationManager;
 import org.gudy.azureus2.core3.internat.MessageText;
-import org.gudy.azureus2.core3.util.AEDiagnostics;
-import org.gudy.azureus2.core3.util.AEDiagnosticsLogger;
-import org.gudy.azureus2.core3.util.AERunnable;
-import org.gudy.azureus2.core3.util.AERunnableBoolean;
-import org.gudy.azureus2.core3.util.AERunnableObject;
-import org.gudy.azureus2.core3.util.AERunnableWithCallback;
-import org.gudy.azureus2.core3.util.AESemaphore;
-import org.gudy.azureus2.core3.util.Constants;
-import org.gudy.azureus2.core3.util.Debug;
-import org.gudy.azureus2.core3.util.SystemTime;
-import org.gudy.azureus2.core3.util.UrlUtils;
+import org.gudy.azureus2.core3.util.*;
+import org.gudy.azureus2.core3.util.Timer;
 import org.gudy.azureus2.ui.swt.mainwindow.Colors;
 import org.gudy.azureus2.ui.swt.mainwindow.SWTThread;
 import org.gudy.azureus2.ui.swt.mainwindow.TorrentOpener;
 import org.gudy.azureus2.ui.swt.shells.MessageBoxShell;
+import org.gudy.azureus2.ui.swt.views.table.TableViewSWT;
 import org.gudy.azureus2.ui.swt.views.utils.VerticalAligner;
 
-import com.aelitis.azureus.core.impl.AzureusCoreImpl;
 import com.aelitis.azureus.ui.swt.UIFunctionsManagerSWT;
 import com.aelitis.azureus.ui.swt.UIFunctionsSWT;
+import com.aelitis.azureus.ui.swt.imageloader.ImageLoader;
 
 /**
  * @author Olivier
@@ -106,7 +55,7 @@ import com.aelitis.azureus.ui.swt.UIFunctionsSWT;
  */
 public class Utils
 {
-	private static final String GOOD_STRING = "(/|,jI~`gy";
+	public static final String GOOD_STRING = "(/|,jI~`gy";
 
 	public static final boolean isGTK = SWT.getPlatform().equals("gtk");
 
@@ -121,7 +70,7 @@ public class Utils
 	private static final boolean DIRECT_SETCHECKED = !Constants.isOSX
 			|| SWT.getVersion() >= 3212;
 
-	public static final boolean SWT32_TABLEPAINT = false; //SWT.getVersion() >= 3200;
+	public static final boolean SWT32_TABLEPAINT = SWT.getVersion() >= 3200;
 
 	/**
 	 * Debug/Diagnose SWT exec calls.  Provides usefull information like how
@@ -135,6 +84,17 @@ public class Utils
 
 	private static AEDiagnosticsLogger diag_logger;
 
+	private static Image[] shellIcons = null;
+
+	private final static String[] shellIconNames = {
+		"azureus",
+		"azureus32",
+		"azureus64",
+		"azureus128"
+	};
+
+	public static final Rectangle EMPTY_RECT = new Rectangle(0, 0, 0, 0);
+
 	static {
 		if (DEBUG_SWTEXEC) {
 			queue = new ArrayList();
@@ -193,6 +153,9 @@ public class Utils
 	}
 
 	public static void disposeSWTObjects(Object[] disposeList) {
+		if (disposeList == null) {
+			return;
+		}
 		boolean bResourceObjectExists = SWT.getVersion() >= 3129;
 
 		for (int i = 0; i < disposeList.length; i++) {
@@ -416,8 +379,7 @@ public class Utils
 
 		public void drop(DropTargetEvent event) {
 			if (url == null || url.isDisposed()) {
-				TorrentOpener.openDroppedTorrents(AzureusCoreImpl.getSingleton(),
-						event, bAllowShareAdd);
+				TorrentOpener.openDroppedTorrents(event, bAllowShareAdd);
 			} else {
 				if (event.data instanceof URLTransfer.URLType) {
 					if (((URLTransfer.URLType) event.data).linkURL != null)
@@ -534,7 +496,10 @@ public class Utils
 		}
 	}
 
-	public static void alternateTableBackground(Table table) {
+		// Yes, this is actually used by the RSSFeed plugin...
+		// so don't make private until this is fixed
+	
+    public static void alternateTableBackground(Table table) {
 		if (table == null || table.isDisposed())
 			return;
 
@@ -545,6 +510,9 @@ public class Utils
 		}
 
 		int iTopIndex = table.getTopIndex();
+		if (iTopIndex < 0) {
+			return;
+		}
 		int iBottomIndex = getTableBottomIndex(table, iTopIndex);
 
 		Color[] colors = {
@@ -576,8 +544,17 @@ public class Utils
 	 * @see <a href="http://developer.apple.com/documentation/UserExperience/Conceptual/OSXHIGuidelines/XHIGMenus/chapter_7_section_3.html#//apple_ref/doc/uid/TP30000356/TPXREF116">Apple HIG</a>
 	 */
 	public static void setMenuItemImage(final MenuItem item, final String repoKey) {
-		if (!Constants.isOSX)
-			item.setImage(ImageRepository.getImage(repoKey));
+		if (Constants.isOSX || repoKey == null) {
+			return;
+		}
+		ImageLoader imageLoader = ImageLoader.getInstance();
+		item.setImage(imageLoader.getImage(repoKey));
+		item.addDisposeListener(new DisposeListener() {
+			public void widgetDisposed(DisposeEvent e) {
+				ImageLoader imageLoader = ImageLoader.getInstance();
+				imageLoader.releaseImage(repoKey);
+			}
+		});
 	}
 
 	public static void setMenuItemImage(final MenuItem item, final Image image) {
@@ -592,40 +569,31 @@ public class Utils
 	 * @param shell
 	 */
 	public static void setShellIcon(Shell shell) {
-		final String[] sImageNames = {
-			"azureus",
-			"azureus32",
-			"azureus64",
-			"azureus128"
-		};
-
-		if (Constants.isOSX)
+		if (Constants.isOSX) {
 			return;
+		}
 
 		try {
-			ArrayList list = new ArrayList();
-			Image[] images = new Image[] {
-				ImageRepository.getImage("azureus"),
-				ImageRepository.getImage("azureus32"),
-				ImageRepository.getImage("azureus64"),
-				ImageRepository.getImage("azureus128")
-			};
+			if (shellIcons == null) {
 
-			for (int i = 0; i < images.length; i++) {
-				Image image = ImageRepository.getImage(sImageNames[i]);
-				if (image != null)
-					list.add(image);
+				ArrayList<Image> listShellIcons = new ArrayList<Image>(
+						shellIconNames.length);
+
+				ImageLoader imageLoader = ImageLoader.getInstance();
+				for (int i = 0; i < shellIconNames.length; i++) {
+					// never release images since they are always used and stored
+					// in an array
+					Image image = imageLoader.getImage(shellIconNames[i]);
+					if (ImageLoader.isRealImage(image)) {
+						listShellIcons.add(image);
+					}
+				}
+				shellIcons = (Image[]) listShellIcons.toArray(new Image[listShellIcons.size()]);
 			}
 
-			if (list.size() == 0)
-				return;
-
-			shell.setImages((Image[]) list.toArray(new Image[0]));
+			shell.setImages(shellIcons);
 		} catch (NoSuchMethodError e) {
 			// SWT < 3.0
-			Image image = ImageRepository.getImage(sImageNames[0]);
-			if (image != null)
-				shell.setImage(image);
 		}
 	}
 
@@ -698,12 +666,13 @@ public class Utils
 	 * @since 3.0.4.3
 	 */
 	private static boolean execSWTThread(final Runnable code, final int msLater) {
-		Display display = getDisplay();
+		final Display display = getDisplay();
 		if (display == null || code == null) {
 			return false;
 		}
 
-		if (msLater < 0 && display.getThread() == Thread.currentThread()) {
+		boolean isSWTThread = display.getThread() == Thread.currentThread();
+		if (msLater < 0 && isSWTThread) {
 			if (queue == null) {
 				code.run();
 			} else {
@@ -723,12 +692,23 @@ public class Utils
 					if (msLater <= 0) {
 						display.asyncExec(code);
 					} else {
-						display.timerExec(msLater, code);
+						if(isSWTThread) {
+							display.timerExec(msLater, code);
+						} else {
+  						SimpleTimer.addEvent("execSWTThreadLater",
+  								SystemTime.getOffsetTime(msLater), new TimerEventPerformer() {
+  									public void perform(TimerEvent event) {
+  										if (!display.isDisposed()) {
+  											display.asyncExec(code);
+  										}
+  									}
+  								});
+						}
 					}
 				} else {
 					queue.add(code);
 
-					diag_logger.log(SystemTime.getCurrentTime() + "] + QUEUE. size= "
+					diag_logger.log(SystemTime.getCurrentTime() + "] + Q. size= "
 							+ queue.size() + "; add " + code + " via "
 							+ Debug.getCompressedStackTrace());
 					final long lStart = SystemTime.getCurrentTime();
@@ -757,22 +737,35 @@ public class Utils
 									code.run();
 								}
 							} finally {
-								wait = SystemTime.getCurrentTime() - lStartTimeRun;
-								if (wait > 500) {
+								long runTIme = SystemTime.getCurrentTime() - lStartTimeRun;
+								if (runTIme > 500) {
 									diag_logger.log(SystemTime.getCurrentTime() + "] took "
-											+ wait + "ms to run " + code);
+											+ runTIme + "ms to run " + code);
 								}
 
-								diag_logger.log(SystemTime.getCurrentTime()
-										+ "] - QUEUE. size=" + queue.size());
 								queue.remove(code);
+
+								diag_logger.log(SystemTime.getCurrentTime()
+										+ "] - Q. size=" + queue.size() + ";wait:" + wait
+										+ "ms;run:" + runTIme + "ms");
 							}
 						}
 					};
 					if (msLater <= 0) {
 						display.asyncExec(runnableWrapper);
 					} else {
-						display.timerExec(msLater, runnableWrapper);
+						if(isSWTThread) {
+							display.timerExec(msLater, code);
+						} else {
+  						SimpleTimer.addEvent("execSWTThreadLater",
+  								SystemTime.getOffsetTime(msLater), new TimerEventPerformer() {
+  									public void perform(TimerEvent event) {
+  										if (!display.isDisposed()) {
+  											display.asyncExec(code);
+  										}
+  									}
+  								});
+						}
 					}
 				}
 			} catch (NullPointerException e) {
@@ -786,7 +779,7 @@ public class Utils
 
 		return true;
 	}
-
+	
 	/**
 	 * Execute code in the Runnable object using SWT's thread.  If current
 	 * thread it already SWT's thread, the code will run immediately.  If the
@@ -975,17 +968,20 @@ public class Utils
 	}
 
 	/**
-	 * Bottom Index may be negative
+	 * Bottom Index may be negative. Returns bottom index even if invisible.
 	 */
 	public static int getTableBottomIndex(Table table, int iTopIndex) {
 		// on Linux, getItemHeight is slow AND WRONG. so is getItem(x).getBounds().y 
 		// getItem(Point) is slow on OSX
+		
+		// On Windows, in rare cases, getItem(Point(2,y)) doesn't return the item
+		// (such as within a paint event)
 
 		int itemCount = table.getItemCount();
-		if (!table.isVisible() || iTopIndex >= itemCount)
+		if (iTopIndex >= itemCount || iTopIndex < 0)
 			return -1;
 
-		if (Constants.isOSX) {
+		if (Constants.isOSX || Constants.isWindows) {
 			try {
 				TableItem item = table.getItem(iTopIndex);
 				Rectangle bounds = item.getBounds();
@@ -1024,7 +1020,7 @@ public class Utils
 	}
 
 	public static void launch(String sFile) {
-		if (sFile == null) {
+		if (sFile == null || sFile.trim().length() == 0) {
 			return;
 		}
 
@@ -1188,18 +1184,18 @@ public class Utils
 		return ret;
 	}
 
-	public static int pixelsToPoint(double pixels, int dpi) {
+    private static int pixelsToPoint(double pixels, int dpi) {
 		int ret = (int) Math.round((pixels * 72.0) / dpi);
 		return ret;
 	}
 
-	public static boolean drawImage(GC gc, Image image, Rectangle dstRect,
+    private static boolean drawImage(GC gc, Image image, Rectangle dstRect,
 			Rectangle clipping, int hOffset, int vOffset, boolean clearArea) {
 		return drawImage(gc, image, new Point(0, 0), dstRect, clipping, hOffset,
 				vOffset, clearArea);
 	}
 
-	public static boolean drawImage(GC gc, Image image, Rectangle dstRect,
+    private static boolean drawImage(GC gc, Image image, Rectangle dstRect,
 			Rectangle clipping, int hOffset, int vOffset) {
 		return drawImage(gc, image, new Point(0, 0), dstRect, clipping, hOffset,
 				vOffset, false);
@@ -1345,7 +1341,7 @@ public class Utils
 	 * @param control Control that had it's sized changed and needs more room
 	 */
 	public static void relayout(Control control, boolean expandOnly) {
-		if (control == null || control.isDisposed()) {
+		if (control == null || control.isDisposed() || !control.isVisible()) {
 			return;
 		}
 
@@ -1356,9 +1352,13 @@ public class Utils
 			return;
 		}
 
+		int fixedWidth = -1;
+		int fixedHeight = -1;
 		Object layoutData = control.getLayoutData();
 		if (layoutData instanceof FormData) {
 			FormData fd = (FormData) layoutData;
+			fixedHeight = fd.height;
+			fixedWidth = fd.width;
 			if (fd.width != SWT.DEFAULT && fd.height != SWT.DEFAULT) {
 				parent.layout();
 				return;
@@ -1378,7 +1378,8 @@ public class Utils
 
 			//System.out.println("new=" + newSize + ";target=" + targetSize);
 
-			if (newSize.y >= targetSize.y && newSize.x >= targetSize.x) {
+			if ((fixedHeight > -1 || (newSize.y >= targetSize.y))
+					&& (fixedWidth > -1 || (newSize.x >= targetSize.x))) {
 				break;
 			}
 		}
@@ -1500,6 +1501,11 @@ public class Utils
 	}
 
 	public static Font getFontWithHeight(Font baseFont, GC gc, int heightInPixels) {
+		return getFontWithHeight(baseFont, gc, heightInPixels, SWT.DEFAULT);
+	}
+
+	public static Font getFontWithHeight(Font baseFont, GC gc,
+			int heightInPixels, int style) {
 		Font font = null;
 		Device device = baseFont.getDevice();
 
@@ -1525,6 +1531,9 @@ public class Utils
 					font.dispose();
 				}
 				fontData[0].setHeight(size);
+				if (style != SWT.DEFAULT) {
+					fontData[0].setStyle(style);
+				}
 
 				font = new Font(device, fontData);
 
@@ -1725,7 +1734,7 @@ public class Utils
 		return gridData;
 	}
 
-	public static Image createAlphaImage(Device device, int width, int height) {
+    private static Image createAlphaImage(Device device, int width, int height) {
 		return createAlphaImage(device, width, height, (byte) 0);
 	}
 
@@ -1742,7 +1751,7 @@ public class Utils
 		return image;
 	}
 
-	public static Image blitImage(Device device, Image srcImage,
+  public static Image blitImage(Device device, Image srcImage,
 			Rectangle srcArea, Image dstImage, Point dstPos) {
 		if (srcArea == null) {
 			srcArea = srcImage.getBounds();
@@ -1754,28 +1763,17 @@ public class Utils
 			dstBounds.x = dstPos.x;
 			dstBounds.y = dstPos.y;
 		}
-		srcArea.intersect(dstBounds);
-
-		// draw the image with no mask! :(
-		GC gc = new GC(dstImage);
-		try {
-			gc.drawImage(srcImage, srcArea.x, srcArea.y, srcArea.width,
-					srcArea.height, dstPos.x, dstPos.y, srcArea.width, srcArea.height);
-
-		} finally {
-			gc.dispose();
-		}
 
 		ImageData dstImageData = dstImage.getImageData();
 		ImageData srcImageData = srcImage.getImageData();
 		int yPos = dstPos.y;
+		int[] pixels = new int[srcArea.width];
+		byte[] alphas = new byte[srcArea.width];
 		for (int y = 0; y < srcArea.height; y++) {
-			int xPos = dstPos.x;
-			for (int x = 0; x < srcArea.width; x++) {
-				dstImageData.setAlpha(xPos, yPos, srcImageData.getAlpha(x + srcArea.x,
-						y + srcArea.y));
-				xPos++;
-			}
+			srcImageData.getPixels(srcArea.x, y + srcArea.y, srcArea.width, pixels, 0);
+			dstImageData.setPixels(dstPos.x, yPos, srcArea.width, pixels, 0);
+			srcImageData.getAlphas(srcArea.x, y + srcArea.y, srcArea.width, alphas, 0);
+			dstImageData.setAlphas(dstPos.x, yPos, srcArea.width, alphas, 0);
 			yPos++;
 		}
 
@@ -2059,7 +2057,7 @@ public class Utils
 	 * @param y
 	 * @return the monitor if found; otherwise returns <code>null</code>
 	 */
-	public static Monitor getMonitor(int x, int y) {
+    private static Monitor getMonitor(int x, int y) {
 		return getMonitor(new Point(x, y));
 	}
 
@@ -2138,4 +2136,260 @@ public class Utils
 			}
 		});
 	}
+	
+	public static final long IMMEDIATE_ADDREMOVE_DELAY = 150;
+
+	private static final long IMMEDIATE_ADDREMOVE_MAXDELAY = 2000;
+
+	private static Timer timerProcessDataSources = new Timer("Process Data Sources");
+
+	private static TimerEvent timerEventProcessDS;
+
+	private static List processDataSourcesOutstanding = new ArrayList();
+	
+
+	public static boolean
+	addDataSourceAggregated(
+		addDataSourceCallback		callback )
+	{
+		if ( callback == null ){
+			
+			return( true );
+		}
+		
+		boolean processQueueImmediately = false;
+		
+		List	to_do_now = null;
+		
+		synchronized( timerProcessDataSources ){
+						
+			if ( timerEventProcessDS != null && !timerEventProcessDS.hasRun()){
+				
+					// Push timer forward, unless we've pushed it forward for over x seconds
+				
+				long now = SystemTime.getCurrentTime();
+				
+				if (now - timerEventProcessDS.getCreatedTime() < IMMEDIATE_ADDREMOVE_MAXDELAY) {
+					
+					long lNextTime = now + IMMEDIATE_ADDREMOVE_DELAY;
+										
+					timerProcessDataSources.adjustAllBy( lNextTime - timerEventProcessDS.getWhen());
+					
+					if ( !processDataSourcesOutstanding.contains( callback )){
+						
+						processDataSourcesOutstanding.add( callback );
+					}
+				}else{
+										
+					timerEventProcessDS.cancel();
+					
+					timerEventProcessDS = null;
+
+					processQueueImmediately = true;
+					
+					to_do_now = processDataSourcesOutstanding;
+					
+					processDataSourcesOutstanding = new ArrayList();
+				}
+			}else{
+				
+				if ( !processDataSourcesOutstanding.contains( callback )){
+					
+					processDataSourcesOutstanding.add( callback );
+				}
+
+				timerEventProcessDS = 
+					timerProcessDataSources.addEvent(
+						SystemTime.getCurrentTime() + IMMEDIATE_ADDREMOVE_DELAY,
+						new TimerEventPerformer() 
+						{
+							public void 
+							perform(
+								TimerEvent event ) 
+							{
+								List	to_do;
+																
+								synchronized( timerProcessDataSources ){
+								
+									timerEventProcessDS = null;
+
+									to_do = processDataSourcesOutstanding;
+									
+									processDataSourcesOutstanding = new ArrayList();
+								}
+								
+								for (int i=0;i<to_do.size();i++){
+									
+									try{
+										
+										addDataSourceCallback this_callback = (addDataSourceCallback)to_do.get(i);
+														
+										if (TableViewSWT.DEBUGADDREMOVE ) {
+											this_callback.debug("processDataSourceQueue after "
+													+ (SystemTime.getCurrentTime() - event.getCreatedTime())
+													+ "ms");
+										}
+										
+										this_callback.process();
+										
+									}catch( Throwable e ){
+										
+										Debug.printStackTrace(e);
+									}
+								}
+							}
+						});
+			}
+			
+			if ( to_do_now != null ){
+								
+					// process outside the synchronized block, otherwise we'll end up with deadlocks
+
+				to_do_now.remove( callback );
+				
+				for (int i=0;i<to_do_now.size();i++){
+					
+					try{
+						
+						addDataSourceCallback this_callback = (addDataSourceCallback)to_do_now.get(i);
+
+						if ( TableViewSWT.DEBUGADDREMOVE ){
+							
+							this_callback.debug("Over immediate delay limit, processing queue now");
+						}
+						
+						this_callback.process();
+						
+					}catch( Throwable e ){
+						
+						Debug.printStackTrace(e);
+					}
+				}
+			}
+		}
+		
+		return( processQueueImmediately );
+	}
+	
+	public interface
+	addDataSourceCallback
+	{
+		public void
+		process();
+		
+		public void
+		debug(
+			String		str );
+	}
+	
+	
+	private static Map truncatedTextCache = new HashMap();
+	
+	private static class TruncatedTextResult {
+		String text;
+		int maxWidth;
+		
+		public TruncatedTextResult() {
+		}
+	}
+	
+	public synchronized static String truncateText(GC gc,String text, int maxWidth,boolean cache) {
+		if(cache) {
+			TruncatedTextResult result = (TruncatedTextResult) truncatedTextCache.get(text);
+			if(result != null && result.maxWidth == maxWidth) {
+				return result.text;
+			}
+		}
+		StringBuffer sb = new StringBuffer(text);
+		String append = "...";
+		int appendWidth = gc.textExtent(append).x;
+		boolean needsAppend = false;
+		while(gc.textExtent(sb.toString()).x > maxWidth) {
+			//Remove characters until they fit into the maximum width
+			sb.deleteCharAt(sb.length()-1);
+			needsAppend = true;
+			if(sb.length() == 1) {
+				break;
+			}
+		}
+		
+		if(needsAppend) {
+			while(gc.textExtent(sb.toString()).x + appendWidth > maxWidth) {
+				//Remove characters until they fit into the maximum width
+				sb.deleteCharAt(sb.length()-1);
+				needsAppend = true;
+				if(sb.length() == 1) {
+					break;
+				}
+			}
+			sb.append(append);
+		}
+		
+		
+		
+		if(cache) {
+			TruncatedTextResult ttR = new TruncatedTextResult();
+			ttR.text = sb.toString();
+			ttR.maxWidth = maxWidth;
+			
+			truncatedTextCache.put(text, ttR);
+		}
+		
+		return sb.toString();
+	}
+
+	/**
+	 * @param bg
+	 * @return
+	 *
+	 * @since 3.1.1.1
+	 */
+	public static String toColorHexString(Color bg) {
+		StringBuffer sb = new StringBuffer();
+		twoHex(sb, bg.getRed());
+		twoHex(sb, bg.getGreen());
+		twoHex(sb, bg.getBlue());
+		return sb.toString();
+	}
+	
+	private static void twoHex(StringBuffer sb, int h) {
+		if (h <= 15) {
+			sb.append('0');
+		}
+		sb.append(Integer.toHexString(h));
+	}
+	
+	public static String
+	getWidgetBGColorURLParam()
+	{
+		Color bg = findAnyShell().getDisplay().getSystemColor(SWT.COLOR_WIDGET_BACKGROUND);
+		
+		byte[] color = new byte[3];
+		
+		color[0] = (byte) bg.getRed();
+		color[1] = (byte) bg.getGreen();
+		color[2] = (byte) bg.getBlue();
+
+		return( "bg_color=" + ByteFormatter.nicePrint(color));
+	}
+	
+	public static void
+	reportError(
+		Throwable e )
+	{
+		MessageBoxShell mb = 
+			new MessageBoxShell(
+				findAnyShell(),
+				MessageText.getString("ConfigView.section.security.op.error.title"),
+				MessageText.getString("ConfigView.section.security.op.error",
+						new String[] {
+							Debug.getNestedExceptionMessage(e)
+						}), 
+				new String[] {
+					MessageText.getString("Button.ok"),
+				},
+				0 );
+		
+		mb.open();
+	}
 }
