@@ -45,6 +45,7 @@ import org.gudy.azureus2.core3.security.*;
 import org.bouncycastle.util.encoders.Base64;
 
 import com.aelitis.azureus.core.networkmanager.admin.NetworkAdmin;
+import com.aelitis.azureus.ui.common.RememberedDecisionsManager;
 
 public class 
 AuthenticatorWindow 
@@ -222,8 +223,8 @@ AuthenticatorWindow
 	
 			// System.out.println( "auth req for " + realm + " - " + tracker );
 			
-			if ( bind_ip == null){
-		
+			if ( bind_ip == null || bind_ip.isAnyLocalAddress()){
+
 				self_addr = "127.0.0.1";
 		
 			}else{
@@ -299,7 +300,7 @@ AuthenticatorWindow
 	protected String[]
 	getAuthenticationDialog(
 		final String		realm,
-		final String		tracker )
+		final String		location )
 	{
 		final Display	display = SWTThread.getInstance().getDisplay();
 		
@@ -314,15 +315,19 @@ AuthenticatorWindow
 		
 		TOTorrent		torrent = TorrentUtils.getTLSTorrent();
 		
-		final	String	torrent_name;
+		final	boolean	is_tracker;
+		final	String	details;
 		
 		if ( torrent == null ){
 			
-			torrent_name	= null;
+			is_tracker 	= false;
+			
+			details		= TorrentUtils.getTLSDescription();
 			
 		}else{
 			
-			torrent_name	= TorrentUtils.getLocalisedName( torrent );
+			details		= TorrentUtils.getLocalisedName( torrent );
+			is_tracker	= true;
 		}
 		
 		try{
@@ -332,7 +337,7 @@ AuthenticatorWindow
 					public void
 					runSupport()
 					{
-						dialog[0] = new authDialog( sem, display, realm, tracker, torrent_name );
+						dialog[0] = new authDialog( sem, display, realm, is_tracker, location, details );
 					}
 				});
 		}catch( Throwable e ){
@@ -341,6 +346,7 @@ AuthenticatorWindow
 			
 			return( null );
 		}
+		
 		sem.reserve();
 		
 		String	user 	= dialog[0].getUsername();
@@ -359,7 +365,7 @@ AuthenticatorWindow
 	authDialog
 	{
 		private Shell			shell;
-		private AESemaphore	sem;
+		private AESemaphore		sem;
 		
 		private String		username;
 		private String		password;
@@ -370,8 +376,9 @@ AuthenticatorWindow
 			AESemaphore		_sem,
 			Display			display,
 			String			realm,
-			String			tracker,
-			String			torrent_name )
+			boolean			is_tracker,
+			String			target,
+			String			details )
 		{
 			sem	= _sem;
 			
@@ -382,6 +389,18 @@ AuthenticatorWindow
 				return;
 			}
 			
+			final String ignore_key = "IgnoreAuth:" + realm + ":" + target + ":" + details;
+			
+			if ( RememberedDecisionsManager.getRememberedDecision( ignore_key ) == 1 ){
+				
+				Debug.out( "Authentication for " + realm + "/" + target + "/" + details + " ignored as told not to ask again" );
+				
+				sem.release();
+				
+				return;
+			}
+			
+				
 	 		shell = new Shell (display,SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL);
 	 	
 	 		Utils.setShellIcon(shell);
@@ -408,33 +427,33 @@ AuthenticatorWindow
 			gridData.horizontalSpan = 2;
 			realm_value.setLayoutData(gridData);
 	    
-	    		// tracker
+	    		// target
 			
-			Label tracker_label = new Label(shell,SWT.NULL);
-			Messages.setLanguageText(tracker_label, "authenticator.tracker");
+			Label target_label = new Label(shell,SWT.NULL);
+			Messages.setLanguageText(target_label, is_tracker?"authenticator.tracker":"authenticator.location");
 			gridData = new GridData(GridData.FILL_BOTH);
 			gridData.horizontalSpan = 1;
-			tracker_label.setLayoutData(gridData);
+			target_label.setLayoutData(gridData);
 			
-			Label tracker_value = new Label(shell,SWT.NULL);
-			tracker_value.setText(tracker.replaceAll("&", "&&"));
+			Label target_value = new Label(shell,SWT.NULL);
+			target_value.setText(target.replaceAll("&", "&&"));
 			gridData = new GridData(GridData.FILL_BOTH);
 			gridData.horizontalSpan = 2;
-			tracker_value.setLayoutData(gridData);
+			target_value.setLayoutData(gridData);
 	    		
-			if ( torrent_name != null ){
+			if ( details != null ){
 				
-				Label torrent_label = new Label(shell,SWT.NULL);
-				Messages.setLanguageText(torrent_label, "authenticator.torrent");
+				Label details_label = new Label(shell,SWT.NULL);
+				Messages.setLanguageText(details_label, is_tracker?"authenticator.torrent":"authenticator.details");
 				gridData = new GridData(GridData.FILL_BOTH);
 				gridData.horizontalSpan = 1;
-				torrent_label.setLayoutData(gridData);
+				details_label.setLayoutData(gridData);
 				
-				Label torrent_value = new Label(shell,SWT.NULL);
-				torrent_value.setText(torrent_name.replaceAll("&", "&&"));
+				Label details_value = new Label(shell,SWT.NULL);
+				details_value.setText(details.replaceAll("&", "&&"));
 				gridData = new GridData(GridData.FILL_BOTH);
 				gridData.horizontalSpan = 2;
-				torrent_value.setLayoutData(gridData);
+				details_value.setLayoutData(gridData);
 			}
 	    		// user
 	    		
@@ -485,7 +504,7 @@ AuthenticatorWindow
 		    final Button checkBox = new Button(shell, SWT.CHECK);
 		    checkBox.setText(MessageText.getString( "authenticator.savepassword" ));
 			gridData = new GridData(GridData.FILL_BOTH);
-			gridData.horizontalSpan = 2;
+			gridData.horizontalSpan = 1;
 			checkBox.setLayoutData(gridData);
 			checkBox.addListener(SWT.Selection,new Listener() {
 		  		public void handleEvent(Event e) {
@@ -493,6 +512,17 @@ AuthenticatorWindow
 		   		}
 			 });
 			
+		    final Button dontAsk = new Button(shell, SWT.CHECK);
+		    dontAsk.setText(MessageText.getString( "general.dont.ask.again" ));
+			gridData = new GridData(GridData.FILL_BOTH);
+			gridData.horizontalSpan = 1;
+			dontAsk.setLayoutData(gridData);
+			dontAsk.addListener(SWT.Selection,new Listener() {
+		  		public void handleEvent(Event e) {
+		  			RememberedDecisionsManager.setRemembered( ignore_key, dontAsk.getSelection()?1:0 );
+		   		}
+			 });
+			
 			// line
 			
 			Label labelSeparator = new Label(shell,SWT.SEPARATOR | SWT.HORIZONTAL);
