@@ -25,9 +25,11 @@ import org.eclipse.swt.graphics.GC;
 import org.eclipse.swt.graphics.Image;
 import org.eclipse.swt.graphics.Rectangle;
 import org.eclipse.swt.graphics.Point;
+import org.eclipse.swt.widgets.Display;
 import org.eclipse.swt.widgets.Table;
 import org.gudy.azureus2.ui.swt.Utils;
 import org.gudy.azureus2.ui.swt.components.BufferedTableRow;
+import org.gudy.azureus2.ui.swt.views.table.impl.TableCellImpl;
 import org.gudy.azureus2.ui.swt.views.utils.VerticalAligner;
 
 /** Draws an image at a column in a row of a table using direct paints to the 
@@ -76,7 +78,8 @@ public abstract class BufferedGraphicTableItem1 extends BufferedTableItemImpl
   
   /* Sets image to be drawn.
    * @param img Image to be stored & drawn
-   * @return true - image was changed.  false = image was the same
+   * @return true - image was changed.<br>  
+   *         false = image was the same (doesn't mean image bits were the same)
    */
   public boolean setGraphic(Image img) {
     boolean bImageSet = (image != img);
@@ -90,11 +93,9 @@ public abstract class BufferedGraphicTableItem1 extends BufferedTableItemImpl
         bDoRedraw = true;
       image = img;
     }
+    //doPaint(bDoRedraw);
 
-    doPaint(bDoRedraw);
-
-    // can't assume image bits haven't changed
-    return true;
+    return bImageSet;
   }
 
   public boolean needsPainting() {
@@ -172,25 +173,27 @@ public abstract class BufferedGraphicTableItem1 extends BufferedTableItemImpl
       return;
     }
     
-    if (orientation == SWT.FILL) {
-      if (imageBounds.width != bounds.width
-					|| imageBounds.height != bounds.height) {
+    boolean fits = (imageBounds.width == bounds.width
+				&& imageBounds.height == bounds.height);
+    if (orientation == SWT.FILL || fits) {
+      if (fits) {
         //System.out.println("doPaint() sizewrong #"+row.getIndex()+ ".  Image="+imageBounds +";us="+bounds);
 /**/
         // Enable this for semi-fast visual update with some flicker
         boolean ourGC = (gc == null);
-        if (ourGC)
+        if (ourGC) {
           gc = new GC(table);
+        }
         if (gc != null) {
           int iAdj = VerticalAligner.getTableAdjustVerticalBy(table);
           bounds.y += iAdj;
           iAdj = VerticalAligner.getTableAdjustHorizontallyBy(table);
           bounds.x += iAdj;
 
-          gc.drawImage(image, 0, 0, imageBounds.width, imageBounds.height, 
-                       bounds.x, bounds.y, bounds.width, bounds.height);
-          if (ourGC)
+          gc.drawImage(image, bounds.x, bounds.y);
+          if (ourGC) {
             gc.dispose();
+          }
         }
         // _OR_ enable refresh() for slower visual update with lots of flicker
         //refresh();
@@ -199,7 +202,7 @@ public abstract class BufferedGraphicTableItem1 extends BufferedTableItemImpl
         
         // TODO: make config option to choose
 /**/
-        invalidate();
+        //invalidate();
         return;
       }
     } else {
@@ -253,15 +256,17 @@ public abstract class BufferedGraphicTableItem1 extends BufferedTableItemImpl
     if (ourGC) {
       gc = new GC(table);
     }
+    try {
 
-    Point srcStart = new Point(clipping.x - bounds.x, clipping.y - bounds.y); 
-    Rectangle dstRect = new Rectangle(clipping.x, clipping.y, 
-    		imageBounds.width - srcStart.x, imageBounds.height - srcStart.y);
-
-    Utils.drawImage(gc, image, srcStart, dstRect, clipping, 0, 0, false);
-    
-    if (ourGC) {
-      gc.dispose();
+      Point srcStart = new Point(clipping.x - bounds.x, clipping.y - bounds.y); 
+      Rectangle dstRect = new Rectangle(clipping.x, clipping.y, 
+      		imageBounds.width - srcStart.x, imageBounds.height - srcStart.y);
+  
+      Utils.drawImage(gc, image, srcStart, dstRect, clipping, 0, 0, false);
+    } finally {
+      if (ourGC) {
+        gc.dispose();
+      }
     }
   }
 
@@ -321,4 +326,40 @@ public abstract class BufferedGraphicTableItem1 extends BufferedTableItemImpl
   public void setOrientation(int orientation) {
   	this.orientation = orientation;
   }
+  
+  public Image getBackgroundImage() {
+  	Image imageRowBG = row.getBackgroundImage();
+  	if (imageRowBG != null) {
+  		Rectangle bounds = getBounds();
+  		
+  		int wInside = bounds.width - (marginWidth * 2);
+  		int hInside = bounds.height - (marginHeight * 2);
+  		Image imageCellBG = new Image(Display.getDefault(), wInside, hInside);
+  		GC gc = new GC(imageCellBG);
+  		gc.drawImage(imageRowBG, bounds.x + marginWidth, 0 + marginHeight,
+					wInside, hInside, 0, 0, wInside, hInside);
+  		gc.dispose();
+  		
+  		return imageCellBG;
+  	}
+  	
+		Table table = row.getTable();
+		
+		Rectangle bounds = getBounds();
+		
+		if (bounds.isEmpty()) {
+			return null;
+		}
+		
+		Image image = new Image(Display.getDefault(), bounds.width
+				- (marginWidth * 2), bounds.height - (marginHeight * 2));
+		
+		GC gc = new GC(image);
+		gc.setForeground(getBackground());
+		gc.setBackground(getBackground());
+		gc.fillRectangle(0, 0, bounds.width, bounds.height);
+		gc.dispose();
+		
+		return image;
+	}
 }
