@@ -26,6 +26,7 @@ import org.eclipse.swt.widgets.Display;
 import org.eclipse.swt.widgets.Table;
 
 import org.gudy.azureus2.core3.util.AERunnable;
+import org.gudy.azureus2.core3.util.Debug;
 import org.gudy.azureus2.ui.swt.Utils;
 
 /**
@@ -211,8 +212,11 @@ public abstract class BufferedTableItemImpl implements BufferedTableItem
 		
 		Image image = new Image(table.getDisplay(), bounds.width, bounds.height);
 		
-		GC gc = new GC(table);
-		gc.copyArea(image, bounds.x, bounds.y);
+		GC gc = new GC(image);
+		gc.setForeground(getBackground());
+		gc.setBackground(getBackground());
+		gc.fillRectangle(0, 0, bounds.width, bounds.height);
+		//gc.copyArea(image, bounds.x, bounds.y);
 		gc.dispose();
 		
 		return image;
@@ -220,6 +224,18 @@ public abstract class BufferedTableItemImpl implements BufferedTableItem
 
   // @see org.gudy.azureus2.ui.swt.components.BufferedTableItem#redraw()
   public void redraw() {
+  	Utils.execSWTThread(new AERunnable() {
+			public void runSupport() {
+				Rectangle bounds = getBounds();
+				if (bounds != null) {
+					Table table = row.getTable();
+					Rectangle dirty = table.getClientArea().intersection(bounds);
+					if (!dirty.isEmpty()) {
+						table.redraw(dirty.x, dirty.y, dirty.width, dirty.height, false);
+					}
+				}
+			}
+		});
   }
   
   // @see org.gudy.azureus2.ui.swt.components.BufferedTableItem#getMaxLines()
@@ -246,10 +262,13 @@ public abstract class BufferedTableItemImpl implements BufferedTableItem
   // @see org.gudy.azureus2.ui.swt.components.BufferedTableItem#isMouseOver()
   public boolean isMouseOver() {
 		Table table = row.getTable();
-		Point pt = Display.getCurrent().getCursorLocation();
+		if (table == null || table.isDisposed()) {
+			return false;
+		}
+		Point pt = table.getDisplay().getCursorLocation();
 		pt = table.toControl(pt);
 
 		Rectangle bounds = getBounds();
-		return bounds.contains(pt);
+		return bounds == null ? false : bounds.contains(pt);
   }
 }
