@@ -27,8 +27,6 @@ import org.gudy.azureus2.ui.swt.shells.MessageBoxShell;
 import org.gudy.azureus2.ui.swt.wizard.IWizardPanel;
 import org.gudy.azureus2.ui.swt.wizard.Wizard;
 
-import com.aelitis.azureus.core.AzureusCore;
-
 /**
  * @author Olivier
  * 
@@ -45,6 +43,8 @@ public class ConfigureWizard extends Wizard {
   //Server / NAT Settings
   int serverTCPListenPort = COConfigurationManager.getIntParameter( "TCP.Listen.Port" );
   //Files / Torrents
+  private String 	_dataPath;
+  private boolean 	_dataPathChanged;
   String torrentPath;
   
   boolean completed = false;
@@ -52,16 +52,24 @@ public class ConfigureWizard extends Wizard {
 
   public 
   ConfigureWizard(
-  	AzureusCore		azureus_core,
 		boolean modal) 
   {
-    super(azureus_core,"configureWizard.title",modal);
+    super("configureWizard.title",modal);
     IWizardPanel panel = new LanguagePanel(this,null);
     try  {
       torrentPath = COConfigurationManager.getDirectoryParameter("General_sDefaultTorrent_Directory");
     } catch(Exception e) {
       torrentPath = ""; 
     }
+    
+    if ( COConfigurationManager.getBooleanParameter( "Use default data dir" )){
+    
+    	_dataPath = COConfigurationManager.getStringParameter( "Default save path" );
+    	
+    }else{
+    	_dataPath = "";
+    }
+    
     this.setFirstPanel(panel);
   }
   
@@ -87,4 +95,24 @@ public class ConfigureWizard extends Wizard {
 
 		super.onClose();
 	}
+  
+  protected String
+  getDataPath()
+  {
+	  return( _dataPath );
+  }
+  
+  protected void
+  setDataPath(
+	String	s )
+  {
+	  _dataPath 		= s;
+	  _dataPathChanged 	= true;
+  }
+  
+  protected boolean
+  hasDataPathChanged()
+  {
+	  return( _dataPathChanged );
+  }
 }
