@@ -36,8 +36,12 @@ import org.gudy.azureus2.core3.logging.Logger;
 import org.gudy.azureus2.core3.logging.impl.FileLogging;
 import org.gudy.azureus2.core3.util.*;
 import org.gudy.azureus2.platform.PlatformManagerFactory;
+import org.gudy.azureus2.ui.swt.SimpleTextEntryWindow;
 import org.gudy.azureus2.ui.swt.Utils;
-import org.gudy.azureus2.ui.swt.shells.InputShell;
+import org.gudy.azureus2.ui.swt.shells.CoreWaiterSWT;
+import org.gudy.azureus2.ui.swt.shells.CoreWaiterSWT.TriggerInThread;
+
+import com.aelitis.azureus.core.*;
 
 /**
  * @author TuxPaper
@@ -61,7 +65,7 @@ public class UIDebugGenerator
 			return;
 		}
 
-		File path = new File(SystemProperties.getUserPath(), "debug");
+		final File path = new File(SystemProperties.getUserPath(), "debug");
 		if (!path.isDirectory()) {
 			path.mkdir();
 		} else {
@@ -113,12 +117,14 @@ public class UIDebugGenerator
 			}
 		}
 
-		InputShell inputShell = new InputShell("UIDebugGenerator.messageask.title",
+		SimpleTextEntryWindow entryWindow = new SimpleTextEntryWindow(
+				"UIDebugGenerator.messageask.title",
 				"UIDebugGenerator.messageask.text", true);
-		String message = inputShell.open();
-		if (inputShell.isCanceled()) {
+		entryWindow.prompt();
+		if (!entryWindow.hasSubmittedInput()) {
 			return;
-		} 
+		}
+		String message = entryWindow.getSubmittedInput();
 
 		if (message == null || message.length() == 0) {
 			Utils.openMessageBox(Utils.findAnyShell(), SWT.OK,
@@ -139,19 +145,33 @@ public class UIDebugGenerator
 			e.printStackTrace();
 		}
 
-		try {
-			File fEvidence = new File(path, "evidence.log");
-			FileWriter fw;
-			fw = new FileWriter(fEvidence);
-			PrintWriter pw = new PrintWriter(fw);
+		CoreWaiterSWT.waitForCore(TriggerInThread.ANY_THREAD,
+				new AzureusCoreRunningListener() {
 
-			AEDiagnostics.generateEvidence(pw);
+					public void azureusCoreRunning(AzureusCore core) {
+						core.createOperation(AzureusCoreOperation.OP_PROGRESS,
+								new AzureusCoreOperationTask() {
+									public void run(AzureusCoreOperation operation) {
+										try {
+
+											File fEvidence = new File(path, "evidence.log");
+											FileWriter fw;
+											fw = new FileWriter(fEvidence);
+											PrintWriter pw = new PrintWriter(fw);
+
+											AEDiagnostics.generateEvidence(pw);
+
+											fw.close();
+
+										} catch (IOException e) {
+
+											Debug.printStackTrace(e);
+										}
+									}
+								});
+					}
+				});
 
-			fw.close();
-		} catch (IOException e) {
-			// TODO Auto-generated catch block
-			e.printStackTrace();
-		}
 
 		try {
 			File outFile = new File(SystemProperties.getUserPath(), "debug.zip");
