@@ -26,14 +26,12 @@ import org.eclipse.swt.SWT;
 import org.eclipse.swt.widgets.Menu;
 import org.eclipse.swt.widgets.MenuItem;
 import org.eclipse.swt.widgets.Shell;
+
 import org.gudy.azureus2.core3.config.COConfigurationManager;
 import org.gudy.azureus2.core3.util.Constants;
 import org.gudy.azureus2.core3.util.Debug;
 import org.gudy.azureus2.core3.util.SystemProperties;
 
-import com.aelitis.azureus.core.AzureusCore;
-import com.aelitis.azureus.core.AzureusCoreFactory;
-
 /**
  * @author Olivier Chalouhi
  */
@@ -59,8 +57,6 @@ public class MainMenu
 
 	private Menu menuBar;
 
-	private AzureusCore core;
-
 	/**
 	 * <p>Creates the main menu bar and attaches it to the given shell</p>
 	 * @param shell A shell
@@ -80,10 +76,6 @@ public class MainMenu
 
 	private void createMenus(final Shell parent) {
 
-		if (core == null) {
-			core = AzureusCoreFactory.getSingleton();
-		}
-
 		//The Main Menu
 		menuBar = new Menu(parent, SWT.BAR);
 		parent.setMenuBar(menuBar);
@@ -122,7 +114,7 @@ public class MainMenu
 			DebugMenuHelper.createDebugMenuItem(menuBar);
 		}
 
-		addHelpMenu(parent);
+		addV2HelpMenu(parent);
 
 		/*
 		 * Enabled/disable menus based on what ui mode we're in
@@ -227,6 +219,9 @@ public class MainMenu
 			indent(MenuFactory.addViewToolbarMenuItem(viewMenu));
 			indent(MenuFactory.addTransferBarToMenu(viewMenu));
 			indent(MenuFactory.addAllPeersMenuItem(viewMenu));
+			if (Constants.isCVSVersion()) {
+				indent(MenuFactory.addDetailedListMenuItem(viewMenu));
+			}
 
 			/*
 			 * These 2 menus resides on the Tools menu on non-OSX platforms;
@@ -292,7 +287,7 @@ public class MainMenu
 	 * Creates the Help menu and all its children
 	 * @param parent
 	 */
-	private void addHelpMenu(final Shell parent) {
+	private void addV2HelpMenu(final Shell parent) {
 		MenuItem helpItem = MenuFactory.createHelpMenuItem(menuBar);
 
 		Menu helpMenu = helpItem.getMenu();
@@ -306,7 +301,7 @@ public class MainMenu
 		MenuFactory.addReleaseNotesMenuItem(helpMenu);
 		MenuFactory.addWhatsNewMenuItem(helpMenu);
 		
-		MenuFactory.addFAQMenuItem(helpMenu, Constants.AZUREUS_WIKI);
+		MenuFactory.addWikiMenuItem(helpMenu);
 		MenuFactory.addGetPluginsMenuItem(helpMenu);
 
 		MenuFactory.addSeparatorMenuItem(helpMenu);
@@ -314,6 +309,7 @@ public class MainMenu
 		if (false == SystemProperties.isJavaWebStartInstance()) {
 			MenuFactory.addCheckUpdateMenuItem(helpMenu);
 		}
+		MenuFactory.addDonationMenuItem(helpMenu);
 
 		MenuFactory.addSeparatorMenuItem(helpMenu);
 		MenuFactory.addDebugHelpMenuItem(helpMenu);
