@@ -23,11 +23,11 @@ import java.text.NumberFormat;
 
 import org.eclipse.swt.SWT;
 import org.eclipse.swt.custom.CLabel;
-import org.eclipse.swt.events.MouseAdapter;
-import org.eclipse.swt.events.MouseEvent;
+import org.eclipse.swt.events.*;
 import org.eclipse.swt.graphics.*;
 import org.eclipse.swt.layout.*;
 import org.eclipse.swt.widgets.*;
+
 import org.gudy.azureus2.core3.config.COConfigurationManager;
 import org.gudy.azureus2.core3.config.ParameterListener;
 import org.gudy.azureus2.core3.config.impl.TransferSpeedValidator;
@@ -42,30 +42,39 @@ import org.gudy.azureus2.plugins.PluginInterface;
 import org.gudy.azureus2.plugins.PluginManager;
 import org.gudy.azureus2.plugins.network.ConnectionManager;
 import org.gudy.azureus2.plugins.ui.config.ConfigSection;
+import org.gudy.azureus2.pluginsimpl.local.PluginInitializer;
 import org.gudy.azureus2.ui.swt.*;
 import org.gudy.azureus2.ui.swt.progress.*;
+import org.gudy.azureus2.ui.swt.shells.CoreWaiterSWT;
 import org.gudy.azureus2.ui.swt.update.UpdateWindow;
 
 import com.aelitis.azureus.core.AzureusCore;
 import com.aelitis.azureus.core.AzureusCoreFactory;
+import com.aelitis.azureus.core.AzureusCoreRunningListener;
 import com.aelitis.azureus.core.dht.DHT;
 import com.aelitis.azureus.core.networkmanager.NetworkManager;
 import com.aelitis.azureus.plugins.dht.DHTPlugin;
 import com.aelitis.azureus.ui.UIFunctions;
 import com.aelitis.azureus.ui.UIFunctionsManager;
 import com.aelitis.azureus.ui.UIStatusTextClickListener;
+import com.aelitis.azureus.ui.common.updater.UIUpdatable;
+import com.aelitis.azureus.ui.swt.UIFunctionsManagerSWT;
+import com.aelitis.azureus.ui.swt.imageloader.ImageLoader;
 
 /**
  * Moved from MainWindow and GUIUpdater
  */
 public class MainStatusBar
+	implements UIUpdatable
 {
 	/**
 	 * Warning status icon identifier
 	 */
 	private static final String STATUS_ICON_WARN = "sb_warning";
+	
+	private static final String ID = "MainStatusBar";
 
-	private AEMonitor this_mon = new AEMonitor("MainStatusBar");
+	private AEMonitor this_mon = new AEMonitor(ID);
 
 	private UpdateWindow updateWindow;
 
@@ -77,6 +86,8 @@ public class MainStatusBar
 
 	private String statusImageKey = null;
 
+	private Image statusImage = null;
+
 	private AZProgressBar progressBar;
 
 	private CLabel ipBlocked;
@@ -102,6 +113,8 @@ public class MainStatusBar
 
 	private int lastNATstatus = -1;
 
+	private String lastNATimageID = null;
+
 	private int lastDHTstatus = -1;
 
 	private long lastDHTcount = -1;
@@ -114,10 +127,6 @@ public class MainStatusBar
 
 	private DHTPlugin dhtPlugin;
 
-	private GlobalManager globalManager;
-
-	private AzureusCore azureusCore;
-
 	private UIFunctions uiFunctions;
 
 	private UIStatusTextClickListener clickListener;
@@ -149,13 +158,7 @@ public class MainStatusBar
 	 */
 	private CLabelPadding progressViewerImageLabel;
 
-	private Image progress_error_img = null;
-
-	private Image progress_info_img = null;
-
-	private Image progress_viewer_img = null;
-
-	private Image currentProgressImage = null;
+	private String lastProgressImageID = null;
 
 	private boolean updateProgressBarDisplayQueued = false;
 
@@ -163,30 +166,34 @@ public class MainStatusBar
 
 	protected AEMonitor latestReport_mon = new AEMonitor("latestReport");
 
+	private String lastSRimageID = null;
+
 	/**
 	 * 
 	 */
 	public MainStatusBar() {
 		numberFormat = NumberFormat.getInstance();
-		overall_stats = StatsFactory.getStats();
-		PluginManager pm = AzureusCoreFactory.getSingleton().getPluginManager();
-		connection_manager = pm.getDefaultPluginInterface().getConnectionManager();
-		PluginInterface dht_pi = pm.getPluginInterfaceByClass(DHTPlugin.class);
-		if (dht_pi != null) {
-			dhtPlugin = (DHTPlugin) dht_pi.getPlugin();
-		}
+		// Proably need to wait for core to be running to make sure dht plugin is fully avail
+		AzureusCoreFactory.addCoreRunningListener(new AzureusCoreRunningListener() {
+			public void azureusCoreRunning(AzureusCore core) {
+				PluginManager pm = core.getPluginManager();
+				connection_manager = PluginInitializer.getDefaultInterface().getConnectionManager();
+				PluginInterface dht_pi = pm.getPluginInterfaceByClass(DHTPlugin.class);
+				if (dht_pi != null) {
+					dhtPlugin = (DHTPlugin) dht_pi.getPlugin();
+				}
+			}
+		});
 	}
 
 	/**
 	 * 
 	 * @return composite holding the statusbar
 	 */
-	public Composite initStatusBar(final AzureusCore core,
-			final GlobalManager globalManager, Display display, final Composite parent) {
-		this.display = display;
-		this.globalManager = globalManager;
-		this.azureusCore = core;
+	public Composite initStatusBar(final Composite parent) {
+		this.display = parent.getDisplay();
 		this.uiFunctions = UIFunctionsManager.getUIFunctions();
+		ImageLoader imageLoader = ImageLoader.getInstance();
 
 		FormData formData;
 
@@ -195,7 +202,7 @@ public class MainStatusBar
 		statusBar = new Composite(parent, SWT.NONE);
 		statusBar.setForeground(fgColor);
 		isAZ3 = "az3".equalsIgnoreCase(COConfigurationManager.getStringParameter("ui"));
-
+		
 		GridLayout layout_status = new GridLayout();
 		layout_status.numColumns = 20;
 		layout_status.horizontalSpacing = 0;
@@ -220,13 +227,11 @@ public class MainStatusBar
 		statusText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL
 				| GridData.VERTICAL_ALIGN_FILL));
 
-		// This is the highest image displayed on the statusbar
-		Image image = ImageRepository.getImage(STATUS_ICON_WARN);
-		int imageHeight = (image == null) ? 20 : image.getBounds().height;
+		addStatusBarMenu(statusText);
 
 		GC gc = new GC(statusText);
 		// add 6, because CLabel forces a 3 pixel indent
-		int height = Math.max(imageHeight, gc.getFontMetrics().getHeight()) + 6;
+		int height = Math.max(16, gc.getFontMetrics().getHeight()) + 6;
 		gc.dispose();
 
 		formData = new FormData();
@@ -264,12 +269,19 @@ public class MainStatusBar
 		progressGridData.widthHint = 5;
 		progressBar.setLayoutData(progressGridData);
 
+		if ( isAZ3 ){
+		
+			try{
+				addFeedBack();
+			}catch( Throwable e ){
+				
+				Debug.printStackTrace(e);
+			}
+		}
+		
 		/*
 		 * Progress reporting window image label
 		 */
-		progress_error_img = ImageRepository.getImage("progress_error");
-		progress_info_img = ImageRepository.getImage("progress_info");
-		progress_viewer_img = ImageRepository.getImage("progress_viewer");
 
 		progressViewerImageLabel = new CLabelPadding(statusBar, SWT.NONE);
 		// image set below after adding listener
@@ -306,6 +318,14 @@ public class MainStatusBar
 				}
 			}
 		});
+		progressViewerImageLabel.addDisposeListener(new DisposeListener() {
+			public void widgetDisposed(DisposeEvent e) {
+				ImageLoader imageLoader = ImageLoader.getInstance();
+				imageLoader.releaseImage("progress_error");
+				imageLoader.releaseImage("progress_info");
+				imageLoader.releaseImage("progress_viewer");
+			}
+		});
 
 		statusBar.layout();
 
@@ -363,9 +383,9 @@ public class MainStatusBar
 		ipBlocked = new CLabelPadding(statusBar, borderFlag);
 		ipBlocked.setText("{} IPs:"); //$NON-NLS-1$
 		Messages.setLanguageText(ipBlocked, "MainWindow.IPs.tooltip");
-		ipBlocked.addMouseListener(new MouseAdapter() {
-			public void mouseDoubleClick(MouseEvent arg0) {
-				BlockedIpsWindow.showBlockedIps(azureusCore, parent.getShell());
+		ipBlocked.addListener(SWT.MouseDoubleClick, new ListenerNeedingCoreRunning() {
+			public void handleEvent(AzureusCore core, Event event) {
+				BlockedIpsWindow.showBlockedIps(core, parent.getShell());
 			}
 		});
 
@@ -378,19 +398,19 @@ public class MainStatusBar
 				});
 
 		statusDown = new CLabelPadding(statusBar, borderFlag);
-		statusDown.setImage(ImageRepository.getImage("down"));
+		statusDown.setImage(imageLoader.getImage("down"));
 		statusDown.setText(/*MessageText.getString("ConfigView.download.abbreviated") +*/"n/a");
 		Messages.setLanguageText(statusDown,
 				"MainWindow.status.updowndetails.tooltip");
 
 		Listener lStats = new Listener() {
 			public void handleEvent(Event e) {
-				uiFunctions.showStats();
+				uiFunctions.openView(UIFunctions.VIEW_STATS, null);
 			}
 		};
 
 		statusUp = new CLabelPadding(statusBar, borderFlag);
-		statusUp.setImage(ImageRepository.getImage("up"));
+		statusUp.setImage(imageLoader.getImage("up"));
 		statusUp.setText(/*MessageText.getString("ConfigView.upload.abbreviated") +*/"n/a");
 		Messages.setLanguageText(statusUp,
 				"MainWindow.status.updowndetails.tooltip");
@@ -400,7 +420,7 @@ public class MainStatusBar
 
 		Listener lDHT = new Listener() {
 			public void handleEvent(Event e) {
-				uiFunctions.showStatsDHT();
+				uiFunctions.openView(UIFunctions.VIEW_STATS, "dht");
 			}
 		};
 
@@ -409,9 +429,13 @@ public class MainStatusBar
 		Listener lSR = new Listener() {
 			public void handleEvent(Event e) {
 
-				uiFunctions.showStatsTransfers();
+				uiFunctions.openView(UIFunctions.VIEW_STATS, "transfers");
 
 				OverallStats stats = StatsFactory.getStats();
+				
+				if (stats == null) {
+					return;
+				}
 
 				long ratio = (1000 * stats.getUploadedBytes() / (stats.getDownloadedBytes() + 1));
 
@@ -424,11 +448,12 @@ public class MainStatusBar
 
 		srStatus.addListener(SWT.MouseDoubleClick, lSR);
 
-		Listener lNAT = new Listener() {
-			public void handleEvent(Event e) {
-				uiFunctions.showConfig(ConfigSection.SECTION_CONNECTION);
+		Listener lNAT = new ListenerNeedingCoreRunning() {
+			public void handleEvent(AzureusCore core, Event e) {
+				uiFunctions.openView(UIFunctions.VIEW_CONFIG,
+						ConfigSection.SECTION_CONNECTION);
 
-				if (azureusCore.getPluginManager().getDefaultPluginInterface().getConnectionManager().getNATStatus() != ConnectionManager.NAT_OK) {
+				if (PluginInitializer.getDefaultInterface().getConnectionManager().getNATStatus() != ConnectionManager.NAT_OK) {
 					Utils.launch(Constants.AZUREUS_WIKI + "NAT_problem");
 				}
 			}
@@ -443,6 +468,12 @@ public class MainStatusBar
 			final Menu menuUpSpeed = new Menu(statusBar.getShell(), SWT.POP_UP);
 			menuUpSpeed.addListener(SWT.Show, new Listener() {
 				public void handleEvent(Event e) {
+					if (!AzureusCoreFactory.isCoreRunning()) {
+						return;
+					}
+					AzureusCore core = AzureusCoreFactory.getSingleton();
+					GlobalManager globalManager = core.getGlobalManager();
+					
 					SelectableSpeedMenu.generateMenuItems(menuUpSpeed, core,
 							globalManager, true);
 				}
@@ -462,9 +493,9 @@ public class MainStatusBar
 					event.button = e.button;
 					e.widget.getDisplay().post(event);
 
-					Utils.execSWTThread(new AERunnable() {
-						public void runSupport() {
-							SelectableSpeedMenu.invokeSlider(true);
+					CoreWaiterSWT.waitForCoreRunning(new AzureusCoreRunningListener() {
+						public void azureusCoreRunning(AzureusCore core) {
+							SelectableSpeedMenu.invokeSlider(core, true);
 						}
 					});
 				}
@@ -475,6 +506,12 @@ public class MainStatusBar
 			final Menu menuDownSpeed = new Menu(statusBar.getShell(), SWT.POP_UP);
 			menuDownSpeed.addListener(SWT.Show, new Listener() {
 				public void handleEvent(Event e) {
+					if (!AzureusCoreFactory.isCoreRunning()) {
+						return;
+					}
+					AzureusCore core = AzureusCoreFactory.getSingleton();
+					GlobalManager globalManager = core.getGlobalManager();
+
 					SelectableSpeedMenu.generateMenuItems(menuDownSpeed, core,
 							globalManager, false);
 				}
@@ -493,9 +530,9 @@ public class MainStatusBar
 					event.button = e.button;
 					e.widget.getDisplay().post(event);
 
-					Utils.execSWTThread(new AERunnable() {
-						public void runSupport() {
-							SelectableSpeedMenu.invokeSlider(false);
+					CoreWaiterSWT.waitForCoreRunning(new AzureusCoreRunningListener() {
+						public void azureusCoreRunning(AzureusCore core) {
+							SelectableSpeedMenu.invokeSlider(core, false);
 						}
 					});
 				}
@@ -504,16 +541,136 @@ public class MainStatusBar
 
 		PRManager.addListener(new ProgressListener());
 		setProgressImage();
+		
+		uiFunctions.getUIUpdater().addUpdater(this);
+		
 		return statusBar;
 	}
 
+	private void addFeedBack() {
+		AzureusCoreFactory.addCoreRunningListener(new AzureusCoreRunningListener() {
+			public void azureusCoreRunning(AzureusCore core) {
+				Utils.execSWTThread(new AERunnable() {
+					public void runSupport() {
+						_addFeedBack();
+					}
+				});
+			}
+		});
+	}
+
+	private void _addFeedBack() {
+		/*
+		 * Feedback
+		 * 
+		 */
+
+		// only show after restart after 15 mins uptime
+		OverallStats stats = StatsFactory.getStats();
+
+		long secs_uptime = stats.getTotalUpTime();
+
+		long last_uptime = COConfigurationManager.getLongParameter(
+				"statusbar.feedback.uptime", 0);
+
+		if (last_uptime == 0) {
+
+			COConfigurationManager.setParameter("statusbar.feedback.uptime",
+					secs_uptime);
+
+		} else if (secs_uptime - last_uptime > 15 * 60) {
+
+			CLabel feedback = createStatusEntry(new CLabelUpdater() {
+				public void update(CLabel label) {
+				}
+			});
+			
+			feedback.setText(MessageText.getString("statusbar.feedback"));
+
+			Listener feedback_listener = new Listener() {
+				public void handleEvent(Event e) {
+
+					String url = "feedback?" + Utils.getWidgetBGColorURLParam()
+							+ "&fromWeb=false&os.name=" + UrlUtils.encode(Constants.OSName)
+							+ "&os.version="
+							+ UrlUtils.encode(System.getProperty("os.version"))
+							+ "&java.version=" + UrlUtils.encode(Constants.JAVA_VERSION);
+
+					// Utils.launch( url );
+
+					UIFunctionsManagerSWT.getUIFunctionsSWT().viewURL(url, null, 600,
+							520, true, false);
+				}
+			};
+			
+			feedback.setToolTipText(MessageText.getString("statusbar.feedback.tooltip"));
+			feedback.setCursor(Cursors.handCursor);
+			feedback.setForeground(Colors.blue);
+			feedback.addListener(SWT.MouseUp, feedback_listener);
+			feedback.addListener(SWT.MouseDoubleClick, feedback_listener);
+			
+			feedback.setVisible(true);
+		}
+	}
+
+	/**
+	 * @param statusBar2
+	 *
+	 * @since 4.0.0.1
+	 */
+	private void addStatusBarMenu(Composite cSB) {
+		if (!Constants.isCVSVersion()) {
+			return;
+		}
+		Menu menu = new Menu(cSB);
+		cSB.setMenu(menu);
+
+		MenuItem itemShow = new MenuItem(menu, SWT.CASCADE);
+		itemShow.setText("Show");
+		Menu menuShow = new Menu(itemShow);
+		itemShow.setMenu(menuShow);
+
+		final String[] statusAreaLangs = {
+			"ConfigView.section.style.status.show_sr",
+			"ConfigView.section.style.status.show_nat",
+			"ConfigView.section.style.status.show_ddb",
+			"ConfigView.section.style.status.show_ipf",
+		};
+		final String[] statusAreaConfig = {
+			"Status Area Show SR",
+			"Status Area Show NAT",
+			"Status Area Show DDB",
+			"Status Area Show IPF",
+		};
+
+		for (int i = 0; i < statusAreaConfig.length; i++) {
+			final String configID = statusAreaConfig[i];
+			String langID = statusAreaLangs[i];
+
+			final MenuItem item = new MenuItem(menuShow, SWT.CHECK);
+			Messages.setLanguageText(item, langID);
+			item.addListener(SWT.Selection, new Listener() {
+				public void handleEvent(Event event) {
+					COConfigurationManager.setParameter(configID,
+							!COConfigurationManager.getBooleanParameter(configID));
+				}
+			});
+			menuShow.addListener(SWT.Show, new Listener() {
+				public void handleEvent(Event event) {
+					item.setSelection(COConfigurationManager.getBooleanParameter(configID));
+				}
+			});
+		}
+		
+	}
+
 	/**
 	 * 
 	 * @param keyedSentence
 	 */
 	public void setStatusText(String keyedSentence) {
 		this.statusTextKey = keyedSentence == null ? "" : keyedSentence;
-		statusImageKey = null;
+		setStatusImageKey(null);
 		this.clickListener = null;
 		if (statusTextKey.length() == 0) { // reset
 			resetStatus();
@@ -521,15 +678,31 @@ public class MainStatusBar
 
 		updateStatusText();
 	}
+	
+	private void setStatusImageKey(String newStatusImageKey) {
+		if (("" + statusImageKey).equals("" + newStatusImageKey)) {
+			return;
+		}
+		ImageLoader imageLoader = ImageLoader.getInstance();
+		if (statusImageKey != null) {
+			imageLoader.releaseImage(statusImageKey);
+		}
+		statusImageKey = newStatusImageKey;
+		if (statusImageKey != null) {
+			statusImage = imageLoader.getImage(statusImageKey);
+		} else {
+			statusImage = null;
+		}
+	}
 
 	private void resetStatus() {
 		if (Constants.isCVSVersion()) {
 			statusTextKey = "MainWindow.status.unofficialversion ("
 					+ Constants.AZUREUS_VERSION + ")";
-			statusImageKey = STATUS_ICON_WARN;
+			setStatusImageKey(STATUS_ICON_WARN);
 		} else if (!Constants.isOSX) { //don't show official version numbers for OSX L&F
 			statusTextKey = Constants.APP_NAME + " " + Constants.AZUREUS_VERSION;
-			statusImageKey = null;
+			setStatusImageKey(null);
 		}
 
 	}
@@ -549,12 +722,12 @@ public class MainStatusBar
 
 		this.clickListener = l;
 		if (statustype == UIFunctions.STATUSICON_WARNING) {
-			statusImageKey = STATUS_ICON_WARN;
+			setStatusImageKey(STATUS_ICON_WARN);
 		}
 		if (statustype == UIFunctions.STATUSICON_WARNING) {
-			statusImageKey = STATUS_ICON_WARN;
+			setStatusImageKey(STATUS_ICON_WARN);
 		} else {
-			statusImageKey = null;
+			setStatusImageKey(null);
 		}
 
 		updateStatusText();
@@ -577,8 +750,7 @@ public class MainStatusBar
 			public void runSupport() {
 				if (statusText != null && !statusText.isDisposed()) {
 					statusText.setText(MessageText.getStringForSentence(text));
-					statusText.setImage((statusImageKey == null) ? null
-							: ImageRepository.getImage(statusImageKey));
+					statusText.setImage(statusImage);
 				}
 			}
 		});
@@ -610,10 +782,10 @@ public class MainStatusBar
 		}
 	}
 
-	/**
-	 */
-	public void refreshStatusBar() {
-		if (ipBlocked.isDisposed()) {
+	// @see com.aelitis.azureus.ui.common.updater.UIUpdatable#updateUI()
+	public void updateUI() {
+		if (statusBar.isDisposed()) {
+			uiFunctions.getUIUpdater().removeUpdater(this);
 			return;
 		}
 
@@ -625,144 +797,76 @@ public class MainStatusBar
 			}
 		}
 
-		// IP Filter Status Section
-		IpFilter ip_filter = azureusCore.getIpFilterManager().getIPFilter();
-
-		ipBlocked.setText("IPs: "
-				+ numberFormat.format(ip_filter.getNbRanges())
-				+ " - "
-				+ numberFormat.format(ip_filter.getNbIpsBlockedAndLoggable())
-				+ "/"
-				+ numberFormat.format(ip_filter.getNbBannedIps())
-				+ "/"
-				+ numberFormat.format(azureusCore.getIpFilterManager().getBadIps().getNbBadIps()));
-		ipBlocked.setToolTipText(MessageText.getString("MainWindow.IPs.tooltip",
-				new String[] {
-					DisplayFormatters.formatDateShort(ip_filter.getLastUpdateTime())
-				}));
-
-		// SR status section
-
-		long ratio = (1000 * overall_stats.getUploadedBytes() / (overall_stats.getDownloadedBytes() + 1));
-
-		int sr_status;
-
-		if (ratio < 500) {
-
-			sr_status = 0;
-
-		} else if (ratio < 900) {
-
-			sr_status = 1;
-
-		} else {
-
-			sr_status = 2;
+		if (ipBlocked.isVisible()) {
+			updateIPBlocked();
+		}
+		
+		if (srStatus.isVisible()) {
+			updateShareRatioStatus();
 		}
 
-		if (sr_status != last_sr_status) {
 
-			String imgID;
-
-			switch (sr_status) {
-				case 2:
-					imgID = "greenled";
-					break;
-
-				case 1:
-					imgID = "yellowled";
-					break;
-
-				default:
-					imgID = "redled";
-					break;
-			}
-
-			srStatus.setImage(ImageRepository.getImage(imgID));
-
-			last_sr_status = sr_status;
+		if (natStatus.isVisible()) {
+			updateNatStatus();
+		}
+		
+		if (dhtStatus.isVisible()) {
+			updateDHTStatus();
 		}
 
-		if (ratio != last_sr_ratio) {
 
-			String tooltipID;
+		// UL/DL Status Sections
+		if (AzureusCoreFactory.isCoreRunning()) {
+			AzureusCore core = AzureusCoreFactory.getSingleton();
+			GlobalManager gm = core.getGlobalManager();
+			GlobalManagerStats stats = gm.getStats();
 
-			switch (sr_status) {
-				case 2:
-					tooltipID = "MainWindow.sr.status.tooltip.ok";
-					break;
+			int dl_limit = NetworkManager.getMaxDownloadRateBPS() / 1024;
 
-				case 1:
-					tooltipID = "MainWindow.sr.status.tooltip.poor";
-					break;
+			statusDown.setText((dl_limit == 0 ? "" : "[" + dl_limit + "K] ")
+					+ DisplayFormatters.formatDataProtByteCountToKiBEtcPerSec(
+							stats.getDataReceiveRate(), stats.getProtocolReceiveRate()));
 
-				default:
-					tooltipID = "MainWindow.sr.status.tooltip.bad";
-					break;
+			boolean auto_up = COConfigurationManager.getBooleanParameter(TransferSpeedValidator.getActiveAutoUploadParameter(gm))
+					&& TransferSpeedValidator.isAutoUploadAvailable(core);
+
+			int ul_limit_norm = NetworkManager.getMaxUploadRateBPSNormal() / 1024;
+
+			String seeding_only;
+			if (NetworkManager.isSeedingOnlyUploadRate()) {
+				int ul_limit_seed = NetworkManager.getMaxUploadRateBPSSeedingOnly() / 1024;
+				if (ul_limit_seed == 0) {
+					seeding_only = "+" + Constants.INFINITY_STRING + "K";
+				} else {
+					int diff = ul_limit_seed - ul_limit_norm;
+					seeding_only = (diff >= 0 ? "+" : "") + diff + "K";
+				}
+			} else {
+				seeding_only = "";
 			}
 
-			String ratio_str = "";
-
-			String partial = "" + ratio % 1000;
-
-			while (partial.length() < 3) {
-
-				partial = "0" + partial;
-			}
-
-			ratio_str = (ratio / 1000) + "." + partial;
-
-			srStatus.setToolTipText(MessageText.getString(tooltipID, new String[] {
-				ratio_str
-			}));
-
-			last_sr_ratio = ratio;
+			statusUp.setText((ul_limit_norm == 0 ? "" : "[" + ul_limit_norm + "K"
+					+ seeding_only + "]")
+					+ (auto_up ? "* " : " ")
+					+ DisplayFormatters.formatDataProtByteCountToKiBEtcPerSec(
+							stats.getDataSendRate(), stats.getProtocolSendRate()));
 		}
 
-		// NAT status Section
+		// End of Status Sections
+		statusBar.layout();
+	}
 
-		int nat_status = connection_manager.getNATStatus();
-
-		if (lastNATstatus != nat_status) {
-			String imgID;
-			String tooltipID;
-			String statusID;
-
-			switch (nat_status) {
-				case ConnectionManager.NAT_UNKNOWN:
-					imgID = "grayled";
-					tooltipID = "MainWindow.nat.status.tooltip.unknown";
-					statusID = "MainWindow.nat.status.unknown";
-					break;
-
-				case ConnectionManager.NAT_OK:
-					imgID = "greenled";
-					tooltipID = "MainWindow.nat.status.tooltip.ok";
-					statusID = "MainWindow.nat.status.ok";
-					break;
-
-				case ConnectionManager.NAT_PROBABLY_OK:
-					imgID = "yellowled";
-					tooltipID = "MainWindow.nat.status.tooltip.probok";
-					statusID = "MainWindow.nat.status.probok";
-					break;
-
-				default:
-					imgID = "redled";
-					tooltipID = "MainWindow.nat.status.tooltip.bad";
-					statusID = "MainWindow.nat.status.bad";
-					break;
-			}
-
-			natStatus.setImage(ImageRepository.getImage(imgID));
-			natStatus.setToolTipText(MessageText.getString(tooltipID));
-			natStatus.setText(MessageText.getString(statusID));
-			lastNATstatus = nat_status;
+	/**
+	 * 
+	 *
+	 * @since 3.1.1.1
+	 */
+	private void updateDHTStatus() {
+		if (dhtPlugin == null) {
+			return;
 		}
-
 		// DHT Status Section
-		int dht_status = (dhtPlugin == null) ? DHTPlugin.STATUS_DISABLED
-				: dhtPlugin.getStatus();
+		int dht_status = dhtPlugin.getStatus();
 		long dht_count = -1;
 		//boolean	reachable = false;
 		if (dht_status == DHTPlugin.STATUS_RUNNING) {
@@ -776,7 +880,8 @@ public class MainStatusBar
 		}
 
 		if (lastDHTstatus != dht_status || lastDHTcount != dht_count) {
-			Image img = ImageRepository.getImage("sb_count");
+			boolean hasImage = dhtStatus.getImage() != null;
+			boolean needImage = true;
 			switch (dht_status) {
 				case DHTPlugin.STATUS_RUNNING:
 
@@ -816,60 +921,228 @@ public class MainStatusBar
 					break;
 
 				default:
-					img = null;
+					needImage = false;
 					break;
 			}
 
-			dhtStatus.setImage(img);
+			if (hasImage != needImage) {
+				ImageLoader imageLoader = ImageLoader.getInstance();
+				if (needImage) {
+					Image img = imageLoader.getImage("sb_count");
+					dhtStatus.setImage(img);
+				} else {
+					imageLoader.releaseImage("sb_count");
+					dhtStatus.setImage(null);
+				}
+			}
 			lastDHTstatus = dht_status;
 			lastDHTcount = dht_count;
 		}
+	}
 
-		// UL/DL Status Sections
-
-		int dl_limit = NetworkManager.getMaxDownloadRateBPS() / 1024;
-
-		GlobalManagerStats stats = globalManager.getStats();
-
-		statusDown.setText((dl_limit == 0 ? "" : "[" + dl_limit + "K] ")
-				+ DisplayFormatters.formatDataProtByteCountToKiBEtcPerSec(
-						stats.getDataReceiveRate(), stats.getProtocolReceiveRate()));
-
-		boolean auto_up = COConfigurationManager.getBooleanParameter(TransferSpeedValidator.getActiveAutoUploadParameter(globalManager))
-				&& TransferSpeedValidator.isAutoUploadAvailable(azureusCore);
-
-		int ul_limit_norm = NetworkManager.getMaxUploadRateBPSNormal() / 1024;
-
-		String seeding_only;
-		if (NetworkManager.isSeedingOnlyUploadRate()) {
-			int ul_limit_seed = NetworkManager.getMaxUploadRateBPSSeedingOnly() / 1024;
-			if (ul_limit_seed == 0) {
-				seeding_only = "+" + Constants.INFINITY_STRING + "K";
-			} else {
-				int diff = ul_limit_seed - ul_limit_norm;
-				seeding_only = (diff >= 0 ? "+" : "") + diff + "K";
-			}
-		} else {
-			seeding_only = "";
+	/**
+	 * 
+	 *
+	 * @since 3.1.1.1
+	 */
+	private void updateNatStatus() {
+		// NAT status Section
+		if (connection_manager == null) {
+			return;
 		}
 
-		statusUp.setText((ul_limit_norm == 0 ? "" : "[" + ul_limit_norm + "K"
-				+ seeding_only + "]")
-				+ (auto_up ? "* " : " ")
-				+ DisplayFormatters.formatDataProtByteCountToKiBEtcPerSec(
-						stats.getDataSendRate(), stats.getProtocolSendRate()));
+		int nat_status = connection_manager.getNATStatus();
 
-		// End of Status Sections
-		statusBar.layout();
+		if (lastNATstatus != nat_status) {
+			String imgID;
+			String tooltipID;
+			String statusID;
+
+			switch (nat_status) {
+				case ConnectionManager.NAT_UNKNOWN:
+					imgID = "grayled";
+					tooltipID = "MainWindow.nat.status.tooltip.unknown";
+					statusID = "MainWindow.nat.status.unknown";
+					break;
+
+				case ConnectionManager.NAT_OK:
+					imgID = "greenled";
+					tooltipID = "MainWindow.nat.status.tooltip.ok";
+					statusID = "MainWindow.nat.status.ok";
+					break;
+
+				case ConnectionManager.NAT_PROBABLY_OK:
+					imgID = "yellowled";
+					tooltipID = "MainWindow.nat.status.tooltip.probok";
+					statusID = "MainWindow.nat.status.probok";
+					break;
+
+				default:
+					imgID = "redled";
+					tooltipID = "MainWindow.nat.status.tooltip.bad";
+					statusID = "MainWindow.nat.status.bad";
+					break;
+			}
+			
+			if (!imgID.equals(lastNATimageID)) {
+				ImageLoader imageLoader = ImageLoader.getInstance();
+				natStatus.setImage(imageLoader.getImage(imgID));
+
+				if (lastNATimageID != null) {
+					imageLoader.releaseImage(lastNATimageID);
+				}
+				lastNATimageID = imgID;
+			}
+
+			natStatus.setToolTipText(MessageText.getString(tooltipID));
+			natStatus.setText(MessageText.getString(statusID));
+			lastNATstatus = nat_status;
+		}
+	}
+
+	/**
+	 * 
+	 *
+	 * @since 3.1.1.1
+	 */
+	private void updateShareRatioStatus() {
+		// SR status section
+		
+		if (overall_stats == null) {
+			overall_stats = StatsFactory.getStats();
+			
+			if (overall_stats == null) {
+				return;
+			}
+		}
+
+		long ratio = (1000 * overall_stats.getUploadedBytes() / (overall_stats.getDownloadedBytes() + 1));
+
+		int sr_status;
+
+		if (ratio < 500) {
+
+			sr_status = 0;
+
+		} else if (ratio < 900) {
+
+			sr_status = 1;
+
+		} else {
+
+			sr_status = 2;
+		}
+
+		if (sr_status != last_sr_status) {
+
+			String imgID;
+
+			switch (sr_status) {
+				case 2:
+					imgID = "greenled";
+					break;
+
+				case 1:
+					imgID = "yellowled";
+					break;
+
+				default:
+					imgID = "redled";
+					break;
+			}
+
+			if (!imgID.equals(lastSRimageID)) {
+				ImageLoader imageLoader = ImageLoader.getInstance();
+				srStatus.setImage(imageLoader.getImage(imgID));
+				if (lastSRimageID != null) {
+					imageLoader.releaseImage(lastSRimageID);
+				}
+				lastSRimageID  = imgID;
+			}
+			
+			last_sr_status = sr_status;
+		}
+
+		if (ratio != last_sr_ratio) {
+
+			String tooltipID;
+
+			switch (sr_status) {
+				case 2:
+					tooltipID = "MainWindow.sr.status.tooltip.ok";
+					break;
+
+				case 1:
+					tooltipID = "MainWindow.sr.status.tooltip.poor";
+					break;
+
+				default:
+					tooltipID = "MainWindow.sr.status.tooltip.bad";
+					break;
+			}
+
+			String ratio_str = "";
+
+			String partial = "" + ratio % 1000;
+
+			while (partial.length() < 3) {
+
+				partial = "0" + partial;
+			}
+
+			ratio_str = (ratio / 1000) + "." + partial;
+
+			srStatus.setToolTipText(MessageText.getString(tooltipID, new String[] {
+				ratio_str
+			}));
+
+			last_sr_ratio = ratio;
+		}
+	}
+
+	/**
+	 * 
+	 *
+	 * @since 3.1.1.1
+	 */
+	private void updateIPBlocked() {
+		if (!AzureusCoreFactory.isCoreRunning()) {
+			return;
+		}
+		
+		AzureusCore azureusCore = AzureusCoreFactory.getSingleton();
+
+		// IP Filter Status Section
+		IpFilter ip_filter = azureusCore.getIpFilterManager().getIPFilter();
+
+		ipBlocked.setText("IPs: "
+				+ numberFormat.format(ip_filter.getNbRanges())
+				+ " - "
+				+ numberFormat.format(ip_filter.getNbIpsBlockedAndLoggable())
+				+ "/"
+				+ numberFormat.format(ip_filter.getNbBannedIps())
+				+ "/"
+				+ numberFormat.format(azureusCore.getIpFilterManager().getBadIps().getNbBadIps()));
+		ipBlocked.setToolTipText(MessageText.getString("MainWindow.IPs.tooltip",
+				new String[] {
+					DisplayFormatters.formatDateShort(ip_filter.getLastUpdateTime())
+				}));
 	}
 
 	/**
 	 * @param string
 	 */
 	public void setDebugInfo(String string) {
-		if (!statusText.isDisposed())
+		if (statusText != null && !statusText.isDisposed())
 			statusText.setToolTipText(string);
 	}
+	
+	public boolean isMouseOver() {
+		if (statusText == null || statusText.isDisposed()) {
+			return false;
+		}
+		return statusText.getDisplay().getCursorControl() == statusText;
+	}
 
 	public static interface CLabelUpdater
 	{
@@ -915,19 +1188,24 @@ public class MainStatusBar
 			if (!isVisible()) {
 				return (new Point(0, 0));
 			}
-			Point pt = super.computeSize(wHint, hHint, changed);
-			pt.x += 4;
-
-			long now = System.currentTimeMillis();
-			if (lastWidth > pt.x && now - widthSetOn < KEEPWIDTHFOR_MS) {
-				pt.x = lastWidth;
-			} else {
-				if (lastWidth != pt.x)
-					lastWidth = pt.x;
-				widthSetOn = now;
+			try {
+				Point pt = super.computeSize(wHint, hHint, changed);
+  			pt.x += 4;
+  
+  			long now = System.currentTimeMillis();
+  			if (lastWidth > pt.x && now - widthSetOn < KEEPWIDTHFOR_MS) {
+  				pt.x = lastWidth;
+  			} else {
+  				if (lastWidth != pt.x)
+  					lastWidth = pt.x;
+  				widthSetOn = now;
+  			}
+  
+  			return pt;
+			} catch (Throwable t) {
+				Debug.out("Error while computing size for CLabel with text:" + getText());
+				return new Point(0, 0);
 			}
-
-			return pt;
 		}
 	}
 
@@ -1057,19 +1335,29 @@ public class MainStatusBar
 	}
 
 	private void setProgressImage() {
-		Image newProgressImage;
+		String imageID;
+
 		if (PRManager.getReporterCount(ProgressReportingManager.COUNT_ERROR) > 0) {
-			newProgressImage = progress_error_img;
+			imageID = "progress_error";
 		} else if (PRManager.getReporterCount(ProgressReportingManager.COUNT_ALL) > 0) {
-			newProgressImage = progress_info_img;
+			imageID = "progress_info";
 		} else {
-			newProgressImage = progress_viewer_img;
+			imageID = "progress_viewer";
 		}
-		if (currentProgressImage != newProgressImage) {
-			currentProgressImage = newProgressImage;
+
+		if (!imageID.equals(lastProgressImageID)) {
+			final String fImageID = imageID;
 			Utils.execSWTThread(new AERunnable() {
 				public void runSupport() {
-					progressViewerImageLabel.setImage(currentProgressImage);
+					if (progressViewerImageLabel.isDisposed()) {
+						return;
+					}
+					ImageLoader imageLoader = ImageLoader.getInstance();
+					progressViewerImageLabel.setImage(imageLoader.getImage(fImageID));
+					if (lastProgressImageID != null) {
+						imageLoader.releaseImage(lastProgressImageID);
+					}
+					lastProgressImageID  = fImageID;
 				}
 			});
 		}
@@ -1197,4 +1485,8 @@ public class MainStatusBar
 		return null;
 	}
 
+	// @see com.aelitis.azureus.ui.common.updater.UIUpdatable#getUpdateUIName()
+	public String getUpdateUIName() {
+		return ID;
+	}
 }
\ No newline at end of file
