@@ -28,17 +28,17 @@ import java.util.Map;
 
 import org.eclipse.swt.SWT;
 import org.eclipse.swt.SWTError;
-import org.eclipse.swt.custom.*;
 import org.eclipse.swt.events.*;
 import org.eclipse.swt.graphics.*;
 import org.eclipse.swt.layout.FormAttachment;
 import org.eclipse.swt.layout.FormData;
 import org.eclipse.swt.layout.FormLayout;
 import org.eclipse.swt.widgets.*;
+
 import org.gudy.azureus2.core3.config.COConfigurationManager;
 import org.gudy.azureus2.core3.config.ParameterListener;
 import org.gudy.azureus2.core3.download.DownloadManager;
-import org.gudy.azureus2.core3.download.impl.DownloadManagerAdapter;
+import org.gudy.azureus2.core3.download.DownloadManagerState;
 import org.gudy.azureus2.core3.global.GlobalManager;
 import org.gudy.azureus2.core3.global.GlobalManagerAdapter;
 import org.gudy.azureus2.core3.internat.MessageText;
@@ -49,17 +49,18 @@ import org.gudy.azureus2.core3.util.*;
 import org.gudy.azureus2.plugins.*;
 import org.gudy.azureus2.plugins.sharing.ShareException;
 import org.gudy.azureus2.plugins.sharing.ShareManager;
+import org.gudy.azureus2.pluginsimpl.local.PluginCoreUtils;
+import org.gudy.azureus2.pluginsimpl.local.PluginInitializer;
 import org.gudy.azureus2.ui.swt.*;
 import org.gudy.azureus2.ui.swt.associations.AssociationChecker;
-import org.gudy.azureus2.ui.swt.components.ColorUtils;
 import org.gudy.azureus2.ui.swt.components.shell.ShellManager;
 import org.gudy.azureus2.ui.swt.config.wizard.ConfigureWizard;
 import org.gudy.azureus2.ui.swt.debug.ObfusticateImage;
 import org.gudy.azureus2.ui.swt.debug.ObfusticateShell;
 import org.gudy.azureus2.ui.swt.debug.ObfusticateTab;
+import org.gudy.azureus2.ui.swt.donations.DonationWindow;
 import org.gudy.azureus2.ui.swt.maketorrent.NewTorrentWizard;
 import org.gudy.azureus2.ui.swt.minibar.AllTransfersBar;
-import org.gudy.azureus2.ui.swt.minibar.DownloadBar;
 import org.gudy.azureus2.ui.swt.minibar.MiniBarManager;
 import org.gudy.azureus2.ui.swt.plugins.UISWTView;
 import org.gudy.azureus2.ui.swt.plugins.UISWTViewEventListener;
@@ -70,6 +71,7 @@ import org.gudy.azureus2.ui.swt.views.*;
 import org.gudy.azureus2.ui.swt.views.stats.StatsView;
 import org.gudy.azureus2.ui.swt.views.table.TableViewSWT;
 import org.gudy.azureus2.ui.swt.views.table.impl.TableViewTab;
+import org.gudy.azureus2.ui.swt.views.table.utils.TableColumnCreator;
 import org.gudy.azureus2.ui.swt.views.table.utils.TableColumnManager;
 import org.gudy.azureus2.ui.swt.welcome.WelcomeWindow;
 import org.gudy.azureus2.ui.systray.SystemTraySWT;
@@ -77,10 +79,12 @@ import org.gudy.azureus2.ui.systray.SystemTraySWT;
 import com.aelitis.azureus.core.AzureusCore;
 import com.aelitis.azureus.core.AzureusCoreException;
 import com.aelitis.azureus.core.AzureusCoreFactory;
+import com.aelitis.azureus.core.versioncheck.VersionCheckClient;
 import com.aelitis.azureus.ui.UIFunctionsManager;
 import com.aelitis.azureus.ui.UIStatusTextClickListener;
 import com.aelitis.azureus.ui.swt.UIFunctionsManagerSWT;
 import com.aelitis.azureus.ui.swt.UIFunctionsSWT;
+import com.aelitis.azureus.util.MapUtils;
 
 /**
  * @author Olivier
@@ -98,8 +102,6 @@ public class MainWindow
 
 	private Initializer initializer;
 
-	private GUIUpdater updater;
-
 	private AzureusCore azureus_core;
 
 	private GlobalManager globalManager;
@@ -119,8 +121,6 @@ public class MainWindow
 
 	private IconBar iconBar;
 
-	private boolean useCustomTab;
-
 	private Composite folder;
 
 	/** 
@@ -136,21 +136,23 @@ public class MainWindow
 
 	private AEMonitor downloadViews_mon = new AEMonitor("MainWindow:dlviews");
 
-	private Tab mytorrents;
+	private Item mytorrents;
+	
+	private Item detailed_list;
 
-	private Tab all_peers;
+	private Item all_peers;
 
-	private Tab my_tracker_tab;
+	private Item my_tracker_tab;
 
-	private Tab my_shares_tab;
+	private Item my_shares_tab;
 
-	private Tab stats_tab;
+	private Item stats_tab;
 
-	private Tab console;
+	private Item console;
 
-	private Tab multi_options_tab;
+	private Item multi_options_tab;
 
-	private Tab config;
+	private Item config;
 
 	private ConfigView config_view;
 
@@ -168,6 +170,8 @@ public class MainWindow
 
 	private boolean bSettingVisibility = false;
 
+	private Tab mainTabSet;
+
 	public MainWindow(AzureusCore _azureus_core, Initializer _initializer,
 			ArrayList events) {
 		bShowMainWindow = true;
@@ -233,6 +237,36 @@ public class MainWindow
 			Debug.printStackTrace(e);
 		}
 	}
+	
+	public MainWindow() {
+		bShowMainWindow = true;
+
+		try {
+			if (Logger.isEnabled())
+				Logger.log(new LogEvent(LOGID, "MainWindow start"));
+
+			AEDiagnostics.addEvidenceGenerator(this);
+
+			azureus_core = AzureusCoreFactory.getSingleton();
+
+			globalManager = azureus_core.getGlobalManager();
+
+			display = SWTThread.getInstance().getDisplay();
+
+			window = this;
+
+		} catch (AzureusCoreException e) {
+
+			Debug.printStackTrace(e);
+		}
+	}
+	
+	public void init(Composite parent, UISWTInstanceImpl swtInstance) {
+		this.parent = parent;
+		this.shell = parent.getShell();
+		uiSWTInstanceImpl = swtInstance;
+	}
+
 
 	public void setShowMainWindow(boolean b) {
 		bShowMainWindow = b;
@@ -253,9 +287,6 @@ public class MainWindow
 
 			globalManager.loadExistingTorrentsNow(true);
 
-			useCustomTab = COConfigurationManager.getBooleanParameter("useCustomTab");
-			Tab.setUseCustomTab(useCustomTab);
-
 			COConfigurationManager.addParameterListener("config.style.useSIUnits",
 					this);
 
@@ -361,8 +392,7 @@ public class MainWindow
 				attachToTopOf = separator;
 
 				mainStatusBar = new MainStatusBar();
-				Composite statusBar = mainStatusBar.initStatusBar(azureus_core,
-						globalManager, display, shell);
+				Composite statusBar = mainStatusBar.initStatusBar(shell);
 
 				controlAboveFolder = attachToTopOf;
 				controlBelowFolder = statusBar;
@@ -379,32 +409,8 @@ public class MainWindow
 				Logger.log(new LogEvent(LOGID, "Drag and Drop not available", e));
 			}
 
-			if (!useCustomTab) {
-				folder = new TabFolder(parent, SWT.V_SCROLL);
-			} else {
-				folder = new CTabFolder(parent, SWT.CLOSE | SWT.BORDER);
-				final Color bg = ColorUtils.getShade(folder.getBackground(),
-						(Constants.isOSX) ? -25 : -6);
-				final Color fg = ColorUtils.getShade(folder.getForeground(),
-						(Constants.isOSX) ? 25 : 6);
-				folder.setBackground(bg);
-				folder.setForeground(fg);
-				//((CTabFolder)folder).setBorderVisible(false);
-				folder.addDisposeListener(new DisposeListener() {
-					public void widgetDisposed(DisposeEvent event) {
-						bg.dispose();
-						fg.dispose();
-					}
-				});
-
-				((CTabFolder) folder).addCTabFolder2Listener(new CTabFolder2Adapter() {
-					public void close(CTabFolderEvent event) {
-						if (!Tab.closed((Item) event.item)) {
-							event.doit = false;
-						}
-					}
-				});
-			}
+			mainTabSet = new Tab(this);
+			folder = mainTabSet.createFolderWidget(parent);
 
 			formData = new FormData();
 			if (controlAboveFolder == null) {
@@ -422,9 +428,7 @@ public class MainWindow
 			formData.right = new FormAttachment(100, 0); // 2 params for Pre SWT 3.0
 			folder.setLayoutData(formData);
 
-			Tab.initialize(this, folder);
-
-			folder.getDisplay().addFilter(SWT.KeyDown, new Listener() {
+			display.addFilter(SWT.KeyDown, new Listener() {
 				public void handleEvent(Event event) {
 					// Another window has control, skip filter
 					Control focus_control = display.getFocusControl();
@@ -436,26 +440,7 @@ public class MainWindow
 							&& event.character > 0)
 						key += 'a' - 1;
 
-					// ESC or CTRL+F4 closes current Tab
-					if (key == SWT.ESC
-							|| (event.keyCode == SWT.F4 && event.stateMask == SWT.CTRL)) {
-						Tab.closeCurrent();
-						event.doit = false;
-					} else if (event.keyCode == SWT.F6
-							|| (event.character == SWT.TAB && (event.stateMask & SWT.CTRL) != 0)) {
-						// F6 or Ctrl-Tab selects next Tab
-						// On Windows the tab key will not reach this filter, as it is
-						// processed by the traversal TRAVERSE_TAB_NEXT.  It's unknown
-						// what other OSes do, so the code is here in case we get TAB
-						if ((event.stateMask & SWT.SHIFT) == 0) {
-							event.doit = false;
-							Tab.selectNextTab(true);
-							// Shift+F6 or Ctrl+Shift+Tab selects previous Tab
-						} else if (event.stateMask == SWT.SHIFT) {
-							Tab.selectNextTab(false);
-							event.doit = false;
-						}
-					} else if (key == 'l' && (event.stateMask & SWT.MOD1) != 0) {
+					if (key == 'l' && (event.stateMask & SWT.MOD1) != 0) {
 						// Ctrl-L: Open URL
 						OpenTorrentWindow.invokeURLPopup(shell, globalManager);
 						event.doit = false;
@@ -463,92 +448,6 @@ public class MainWindow
 				}
 			});
 
-			SelectionAdapter selectionAdapter = new SelectionAdapter() {
-				public void widgetSelected(final SelectionEvent event) {
-					if (display != null && !display.isDisposed())
-						Utils.execSWTThread(new AERunnable() {
-							public void runSupport() {
-								if (useCustomTab) {
-									CTabItem item = (CTabItem) event.item;
-									if (item != null && !item.isDisposed()
-											&& !folder.isDisposed()) {
-										try {
-											((CTabFolder) folder).setSelection(item);
-											Control control = item.getControl();
-											if (control != null) {
-												control.setVisible(true);
-												control.setFocus();
-											}
-										} catch (Throwable e) {
-											Debug.printStackTrace(e);
-											//Do nothing
-										}
-									}
-								}
-								if (iconBar != null) {
-									iconBar.setCurrentEnabler(MainWindow.this);
-								}
-								refreshTorrentMenu();
-							}
-
-						});
-				}
-			};
-
-			if (!useCustomTab) {
-				((TabFolder) folder).addSelectionListener(selectionAdapter);
-			} else {
-				try {
-					((CTabFolder) folder).setMinimumCharacters(75);
-				} catch (Exception e) {
-					Logger.log(new LogEvent(LOGID, "Can't set MIN_TAB_WIDTH", e));
-				}
-				//try {
-				///  TabFolder2ListenerAdder.add((CTabFolder)folder);
-				//} catch (NoClassDefFoundError e) {
-				((CTabFolder) folder).addCTabFolderListener(new CTabFolderAdapter() {
-					public void itemClosed(CTabFolderEvent event) {
-						if (!event.doit) {
-							return;
-						}
-						Tab.closed((CTabItem) event.item);
-						event.doit = true;
-						((CTabItem) event.item).dispose();
-					}
-				});
-				//}
-
-				((CTabFolder) folder).addSelectionListener(selectionAdapter);
-
-				try {
-					((CTabFolder) folder).setSelectionBackground(new Color[] {
-						display.getSystemColor(SWT.COLOR_LIST_BACKGROUND),
-						display.getSystemColor(SWT.COLOR_LIST_BACKGROUND),
-						display.getSystemColor(SWT.COLOR_WIDGET_BACKGROUND)
-					}, new int[] {
-						10,
-						90
-					}, true);
-				} catch (NoSuchMethodError e) {
-					/** < SWT 3.0M8 **/
-					((CTabFolder) folder).setSelectionBackground(new Color[] {
-						display.getSystemColor(SWT.COLOR_LIST_BACKGROUND)
-					}, new int[0]);
-				}
-				((CTabFolder) folder).setSelectionForeground(display.getSystemColor(SWT.COLOR_LIST_FOREGROUND));
-
-				try {
-					/* Pre 3.0M8 doesn't have Simple-mode (it's always simple mode)
-					   in 3.0M9, it was called setSimpleTab(boolean)
-					   in 3.0RC1, it's called setSimple(boolean)
-					   Prepare for the future, and use setSimple()
-					 */
-					((CTabFolder) folder).setSimple(!COConfigurationManager.getBooleanParameter("GUI_SWT_bFancyTab"));
-				} catch (NoSuchMethodError e) {
-					/** < SWT 3.0RC1 **/
-				}
-			}
-
 			if (Logger.isEnabled())
 				Logger.log(new LogEvent(LOGID, "Initializing GUI complete"));
 
@@ -568,7 +467,7 @@ public class MainWindow
 
 			if (!COConfigurationManager.getBooleanParameter("Wizard Completed")) {
 				// returns after the wizard is done
-				new ConfigureWizard(getAzureusCore(), true);
+				new ConfigureWizard(true);
 			}
 
 			plugin_manager.firePluginEvent(PluginEvent.PEV_CONFIGURATION_WIZARD_COMPLETES);
@@ -577,6 +476,8 @@ public class MainWindow
 			// Must be done before initializing views, since plugins may register
 			// table columns and other objects
 			if (uiSWTInstanceImpl == null) {
+				TableColumnCreator.initCoreColumns();
+
 				uiSWTInstanceImpl = new UISWTInstanceImpl(azureus_core);
 				uiSWTInstanceImpl.init(initializer);
 
@@ -606,7 +507,7 @@ public class MainWindow
 			initializer.nextTask();
 		}
 
-		if (azureus_core.getTrackerHost().getTorrents().length > 0) {
+		if (!Constants.isSafeMode && azureus_core.getTrackerHost().getTorrents().length > 0) {
 			Utils.execSWTThreadLater(delay += delayInc, new Runnable() {
 				public void run() {
 					showMyTracker();
@@ -614,11 +515,9 @@ public class MainWindow
 			});
 		}
 
-		PluginManager plugin_manager = azureus_core.getPluginManager();
-
 		// share manager init is async so we need to deal with this
 
-		PluginInterface default_pi = plugin_manager.getDefaultPluginInterface();
+		PluginInterface default_pi = PluginInitializer.getDefaultInterface();
 
 		try {
 			final ShareManager share_manager = default_pi.getShareManager();
@@ -654,7 +553,7 @@ public class MainWindow
 			Debug.out(e);
 		}
 
-		if (COConfigurationManager.getBooleanParameter("Open MyTorrents")) {
+		if (!Constants.isSafeMode && COConfigurationManager.getBooleanParameter("Open MyTorrents")) {
 			Utils.execSWTThreadLater(delay += delayInc, new Runnable() {
 				public void run() {
 					showMyTorrents();
@@ -666,7 +565,7 @@ public class MainWindow
 
 		new ProgressWindow();
 
-		if (COConfigurationManager.getBooleanParameter("Open Console")) {
+		if (!Constants.isSafeMode && COConfigurationManager.getBooleanParameter("Open Console")) {
 			Utils.execSWTThreadLater(delay += delayInc, new Runnable() {
 				public void run() {
 					showConsole();
@@ -675,7 +574,7 @@ public class MainWindow
 		}
 		events = null;
 
-		if (COConfigurationManager.getBooleanParameter("Open Config")) {
+		if (Constants.isSafeMode || COConfigurationManager.getBooleanParameter("Open Config")) {
 			Utils.execSWTThreadLater(delay += delayInc, new Runnable() {
 				public void run() {
 					showConfig();
@@ -683,7 +582,7 @@ public class MainWindow
 			});
 		}
 
-		if (COConfigurationManager.getBooleanParameter("Open Stats On Start")) {
+		if (!Constants.isSafeMode && COConfigurationManager.getBooleanParameter("Open Stats On Start")) {
 			Utils.execSWTThreadLater(delay += delayInc, new Runnable() {
 				public void run() {
 					showStats();
@@ -691,7 +590,7 @@ public class MainWindow
 			});
 		}
 
-		if (COConfigurationManager.getBooleanParameter("Open Transfer Bar On Start")) {
+		if (!Constants.isSafeMode && COConfigurationManager.getBooleanParameter("Open Transfer Bar On Start")) {
 			Utils.execSWTThreadLater(delay += delayInc, new Runnable() {
 				public void run() {
 					uiFunctions.showGlobalTransferBar();
@@ -699,11 +598,6 @@ public class MainWindow
 			});
 		}
 
-		COConfigurationManager.addParameterListener("GUI_SWT_bFancyTab", this);
-
-		updater = new GUIUpdater(this);
-		updater.start();
-
 		COConfigurationManager.addAndFireParameterListener("IconBar.enabled",
 				new ParameterListener() {
 					public void parameterChanged(String parameterName) {
@@ -834,20 +728,25 @@ public class MainWindow
 	
 		AssociationChecker.checkAssociations();
 
+		// Donation stuff
+		Map map = VersionCheckClient.getSingleton().getMostRecentVersionCheckData();
+		DonationWindow.setInitialAskHours(MapUtils.getMapInt(map,
+				"donations.askhrs", DonationWindow.getInitialAskHours()));
+
 		azureus_core.triggerLifeCycleComponentCreated(uiFunctions);
 	}
 
 	protected void showMyTracker() {
 		if (my_tracker_tab == null) {
-			my_tracker_tab = new Tab(new MyTrackerView(azureus_core));
-			my_tracker_tab.getView().getComposite().addDisposeListener(
+			my_tracker_tab = mainTabSet.createTabItem(new MyTrackerView(), true);
+			mainTabSet.getView(my_tracker_tab).getComposite().addDisposeListener(
 					new DisposeListener() {
 						public void widgetDisposed(DisposeEvent e) {
 							my_tracker_tab = null;
 						}
 					});
 		} else {
-			my_tracker_tab.setFocus();
+			mainTabSet.setFocus(my_tracker_tab);
 			refreshIconBar();
 			refreshTorrentMenu();
 		}
@@ -855,15 +754,15 @@ public class MainWindow
 
 	protected void showMyShares() {
 		if (my_shares_tab == null) {
-			my_shares_tab = new Tab(new MySharesView(azureus_core));
-			my_shares_tab.getView().getComposite().addDisposeListener(
+			my_shares_tab = mainTabSet.createTabItem(new MySharesView(), true);
+			mainTabSet.getView(my_shares_tab).getComposite().addDisposeListener(
 					new DisposeListener() {
 						public void widgetDisposed(DisposeEvent e) {
 							my_shares_tab = null;
 						}
 					});
 		} else {
-			my_shares_tab.setFocus();
+			mainTabSet.setFocus(my_shares_tab);
 			refreshIconBar();
 			refreshTorrentMenu();
 		}
@@ -871,16 +770,33 @@ public class MainWindow
 
 	protected void showMyTorrents() {
 		if (mytorrents == null) {
-			MyTorrentsSuperView view = new MyTorrentsSuperView(azureus_core);
-			mytorrents = new Tab(view);
-			mytorrents.getView().getComposite().addDisposeListener(
+			MyTorrentsSuperView view = new MyTorrentsSuperView();
+			mytorrents = mainTabSet.createTabItem(view, true);
+			mainTabSet.getView(mytorrents).getComposite().addDisposeListener(
 					new DisposeListener() {
 						public void widgetDisposed(DisposeEvent e) {
 							mytorrents = null;
 						}
 					});
 		} else {
-			mytorrents.setFocus();
+			mainTabSet.setFocus(mytorrents);
+		}
+		refreshIconBar();
+		refreshTorrentMenu();
+	}
+	
+	protected void showDetailedListView() {
+		if (detailed_list == null) {
+			DetailedListView view = new DetailedListView(azureus_core);
+			detailed_list = mainTabSet.createTabItem(view, true);
+			mainTabSet.getView(detailed_list).getComposite().addDisposeListener(
+					new DisposeListener() {
+						public void widgetDisposed(DisposeEvent e) {
+							detailed_list = null;
+						}
+					});
+		} else {
+			mainTabSet.setFocus(detailed_list);
 		}
 		refreshIconBar();
 		refreshTorrentMenu();
@@ -888,16 +804,16 @@ public class MainWindow
 
 	protected void showAllPeersView() {
 		if (all_peers == null) {
-			PeerSuperView view = new PeerSuperView(azureus_core.getGlobalManager());
-			all_peers = new Tab(view);
-			all_peers.getView().getComposite().addDisposeListener(
+			PeerSuperView view = new PeerSuperView();
+			all_peers = mainTabSet.createTabItem(view, true);
+			mainTabSet.getView(all_peers).getComposite().addDisposeListener(
 					new DisposeListener() {
 						public void widgetDisposed(DisposeEvent e) {
 							all_peers = null;
 						}
 					});
 		} else {
-			all_peers.setFocus();
+			mainTabSet.setFocus(all_peers);
 		}
 		refreshIconBar();
 		refreshTorrentMenu();
@@ -910,7 +826,7 @@ public class MainWindow
 
 		TorrentOptionsView view = new TorrentOptionsView(managers);
 
-		multi_options_tab = new Tab(view);
+		multi_options_tab = mainTabSet.createTabItem(view, true);
 
 		view.getComposite().addDisposeListener(new DisposeListener() {
 			public void widgetDisposed(DisposeEvent e) {
@@ -942,41 +858,19 @@ public class MainWindow
 		if (mainStatusBar != null)
 			mainStatusBar.refreshStatusText();
 
-		if (folder != null) {
-			if (useCustomTab) {
-				((CTabFolder) folder).update();
-			} else {
-				((TabFolder) folder).update();
-			}
+		if (mainTabSet != null) {
+			mainTabSet.update();
 		}
 	}
 
-	protected boolean destroyRequest() {
-		Logger.log(new LogEvent(LOGID, "MainWindow::destroyRequest"));
-
-		if (COConfigurationManager.getBooleanParameter("Password enabled")) {
-
-			if (!PasswordWindow.showPasswordWindow(display)) {
-				Logger.log(new LogEvent(LOGID, "    denied - password is enabled"));
-
-				return false;
-			}
-		}
-
-		Utils.execSWTThread(new Runnable() {
-			public void run() {
-				dispose(false, false);
-			}
-		});
-		return true;
-	}
-
 	private void downloadManagerAdded(DownloadManager created) {
-		created.addListener(new DownloadManagerAdapter() {
-			public void stateChanged(DownloadManager manager, int state) {
-				downloadManagerStateChanged(manager, state);
-			}
-		});
+		// only do donation check if swt instance is initialized, which means
+		// initial download managers are loaded and this main window is really
+		// the main window
+		if (uiSWTInstanceImpl != null
+				&& !created.getDownloadState().getFlag(DownloadManagerState.FLAG_LOW_NOISE)) {
+			DonationWindow.checkForDonationPopup();
+		}
 	}
 
 	protected void openManagerView(DownloadManager downloadManager) {
@@ -984,12 +878,14 @@ public class MainWindow
 			downloadViews_mon.enter();
 
 			if (downloadViews.containsKey(downloadManager)) {
-				Tab tab = (Tab) downloadViews.get(downloadManager);
-				tab.setFocus();
+				mainTabSet.setFocus((Item) downloadViews.get(downloadManager));
 				refreshIconBar();
 				refreshTorrentMenu();
 			} else {
-				Tab tab = new Tab(new ManagerView(azureus_core, downloadManager));
+				Item tab = openPluginView(null, "DMView", new ManagerView(),
+						downloadManager, true, true);
+//				Item tab = mainTabSet.createTabItem(new ManagerView(azureus_core,
+//						downloadManager), true);
 				downloadViews.put(downloadManager, tab);
 			}
 		} finally {
@@ -1014,13 +910,13 @@ public class MainWindow
 			downloadViews_mon.enter();
 
 			if (downloadViews.containsKey(removed)) {
-				final Tab tab = (Tab) downloadViews.get(removed);
+				final Item tab = (Item) downloadViews.get(removed);
 				Utils.execSWTThread(new AERunnable() {
 					public void runSupport() {
 						if (display == null || display.isDisposed())
 							return;
 
-						tab.dispose();
+						mainTabSet.dispose(tab);
 					}
 				});
 
@@ -1160,13 +1056,17 @@ public class MainWindow
 		 * We can't rely that the normal mechanism for doing this won't fail (which it usually does)
 		 * when the GUI is being disposed of.
 		 */
-		AllTransfersBar transfer_bar = AllTransfersBar.getBarIfOpen(AzureusCoreFactory.getSingleton().getGlobalManager());
-		if (transfer_bar != null) {
-			transfer_bar.forceSaveLocation();
+		try {
+  		AllTransfersBar transfer_bar = AllTransfersBar.getBarIfOpen(AzureusCoreFactory.getSingleton().getGlobalManager());
+  		if (transfer_bar != null) {
+  			transfer_bar.forceSaveLocation();
+  		}
+		} catch (Exception e) {
+			// ignore
 		}
 
 		// close all tabs
-		Tab.closeAllTabs();
+		mainTabSet.closeAllTabs();
 
 		isAlreadyDead = true; //NICO try to never die twice...
 		/*
@@ -1174,11 +1074,6 @@ public class MainWindow
 		  SysTrayMenu.dispose();
 		*/
 
-		if (updater != null) {
-
-			updater.stopIt();
-		}
-
 		if (initializer != null) {
 			initializer.stopIt(for_restart, close_already_in_progress);
 		}
@@ -1191,7 +1086,6 @@ public class MainWindow
 		COConfigurationManager.removeParameterListener("config.style.useSIUnits",
 				this);
 		COConfigurationManager.removeParameterListener("Show Download Basket", this);
-		COConfigurationManager.removeParameterListener("GUI_SWT_bFancyTab", this);
 
 		UIExitUtilsSWT.uiShutdown();
 
@@ -1218,24 +1112,27 @@ public class MainWindow
 
 	Map pluginTabs = new HashMap();
 
-	protected void openPluginView(String sParentID, String sViewID,
-			UISWTViewEventListener l, Object dataSource, boolean bSetFocus) {
+	protected Item openPluginView(String sParentID, String sViewID,
+			UISWTViewEventListener l, Object dataSource, boolean bSetFocus,
+			boolean useCoreDS) {
 
 		UISWTViewImpl view = null;
 		try {
 			view = new UISWTViewImpl(sParentID, sViewID, l);
 		} catch (Exception e) {
-			Tab tab = (Tab) pluginTabs.get(sViewID);
+			Item tab = (Item) pluginTabs.get(sViewID);
 			if (tab != null) {
-				tab.setFocus();
+				mainTabSet.setFocus(tab);
 			}
-			return;
+			return tab;
 		}
+		view.setUseCoreDataSource(useCoreDS);
 		view.dataSourceChanged(dataSource);
 
-		Tab tab = new Tab(view, bSetFocus);
+		Item tab = mainTabSet.createTabItem(view, bSetFocus);
 
 		pluginTabs.put(sViewID, tab);
+		return tab;
 	}
 
 	/**
@@ -1244,28 +1141,9 @@ public class MainWindow
 	 * @param sViewID
 	 */
 	protected void closePluginViews(String sViewID) {
-		Item[] items;
-
-		if (folder instanceof CTabFolder)
-			items = ((CTabFolder) folder).getItems();
-		else if (folder instanceof TabFolder)
-			items = ((TabFolder) folder).getItems();
-		else
-			return;
-
-		for (int i = 0; i < items.length; i++) {
-			IView view = Tab.getView(items[i]);
-			if (view instanceof UISWTViewImpl) {
-				String sID = ((UISWTViewImpl) view).getViewID();
-				if (sID != null && sID.equals(sViewID)) {
-					try {
-						closePluginView(view);
-					} catch (Exception e) {
-						Debug.printStackTrace(e);
-					}
-				}
-			}
-		} // for
+		if (mainTabSet != null) {
+			mainTabSet.closePluginViews(sViewID);
+		}
 	}
 
 	/**
@@ -1274,23 +1152,17 @@ public class MainWindow
 	 * @return open plugin views
 	 */
 	protected UISWTView[] getPluginViews() {
-		Item[] items;
-
-		if (folder instanceof CTabFolder)
-			items = ((CTabFolder) folder).getItems();
-		else if (folder instanceof TabFolder)
-			items = ((TabFolder) folder).getItems();
-		else
-			return new UISWTView[0];
+		IView[] allViews = mainTabSet.getAllViews();
 
 		ArrayList views = new ArrayList();
 
-		for (int i = 0; i < items.length; i++) {
-			IView view = Tab.getView(items[i]);
+		for (int i = 0; i < allViews.length; i++) {
+			IView view = allViews[i];
+
 			if (view instanceof UISWTViewImpl) {
 				views.add(view);
 			}
-		} // for
+		}
 
 		return (UISWTView[]) views.toArray(new UISWTView[0]);
 	}
@@ -1298,11 +1170,11 @@ public class MainWindow
 	protected void openPluginView(final AbstractIView view, final String name) {
 		Utils.execSWTThread(new AERunnable() {
 			public void runSupport() {
-				Tab tab = (Tab) pluginTabs.get(name);
+				Item tab = (Item) pluginTabs.get(name);
 				if (tab != null) {
-					tab.setFocus();
+					mainTabSet.setFocus(tab);
 				} else {
-					tab = new Tab(view);
+					tab = mainTabSet.createTabItem(view, true);
 					pluginTabs.put(name, tab);
 				}
 			}
@@ -1310,11 +1182,11 @@ public class MainWindow
 	}
 
 	protected void closePluginView(IView view) {
-		Item tab = Tab.getTab(view);
+		Item tab = mainTabSet.getTab(view);
 
 		if (tab != null) {
 
-			Tab.closed(tab);
+			mainTabSet.closed(tab);
 		}
 	}
 
@@ -1336,16 +1208,6 @@ public class MainWindow
 			}
 		}
 
-		if (parameterName.equals("GUI_SWT_bFancyTab")
-				&& folder instanceof CTabFolder && folder != null
-				&& !folder.isDisposed()) {
-			try {
-				((CTabFolder) folder).setSimple(!COConfigurationManager.getBooleanParameter("GUI_SWT_bFancyTab"));
-			} catch (NoSuchMethodError e) {
-				/** < SWT 3.0RC1 **/
-			}
-		}
-
 		if (parameterName.equals("config.style.useSIUnits")) {
 			updateComponents();
 		}
@@ -1358,6 +1220,12 @@ public class MainWindow
 		if (itemKey.equals("new"))
 			return true;
 		IView currentView = getCurrentView();
+		if (currentView instanceof UISWTViewImpl) {
+			UISWTViewEventListener eventListener = ((UISWTViewImpl) currentView).getEventListener();
+			if (eventListener instanceof IconBarEnabler) {
+				return ((IconBarEnabler)eventListener).isEnabled(itemKey);
+			}
+		}
 		if (currentView != null)
 			return currentView.isEnabled(itemKey);
 		return false;
@@ -1375,27 +1243,26 @@ public class MainWindow
 			return;
 		}
 		if (itemKey.equals("new")) {
-			new NewTorrentWizard(getAzureusCore(), display);
+			new NewTorrentWizard(display);
 			return;
 		}
 		IView currentView = getCurrentView();
+		if (currentView instanceof UISWTViewImpl) {
+			UISWTViewEventListener eventListener = ((UISWTViewImpl) currentView).getEventListener();
+			if (eventListener instanceof IconBarEnabler) {
+				((IconBarEnabler)eventListener).itemActivated(itemKey);
+				return;
+			}
+		}
 		if (currentView != null)
 			currentView.itemActivated(itemKey);
 	}
 
 	IView getCurrentView() {
-		try {
-			if (!useCustomTab) {
-				TabItem[] selection = ((TabFolder) folder).getSelection();
-				if (selection.length > 0) {
-					return Tab.getView(selection[0]);
-				}
-				return null;
-			}
-			return Tab.getView(((CTabFolder) folder).getSelection());
-		} catch (Exception e) {
-			return null;
+		if (mainTabSet != null) {
+			return mainTabSet.getCurrentView();
 		}
+		return null;
 	}
 
 	protected void refreshIconBar() {
@@ -1418,6 +1285,19 @@ public class MainWindow
 				((ManagerView) currentView).getDownload(),
 			};
 			detailed_view = true;
+		} else if (currentView instanceof UISWTView) {
+			UISWTView current_swt_view = (UISWTView)currentView;
+			Object core_object = PluginCoreUtils.convert(current_swt_view.getDataSource(), true);
+			if (core_object instanceof DownloadManager) {
+				dm = new DownloadManager[] {(DownloadManager)core_object};
+				
+				// We should be using a constant somewhere!
+				detailed_view = "DMView".equals(current_swt_view.getViewID());
+			}
+			else {
+				dm = null;
+				detailed_view = false;
+			}
 		} else if (currentView instanceof MyTorrentsSuperView) {
 			dm = ((MyTorrentsSuperView) this.getCurrentView()).getSelectedDownloads();
 			detailed_view = false;
@@ -1464,15 +1344,15 @@ public class MainWindow
 
 	protected void closeViewOrWindow() {
 		if (getCurrentView() != null)
-			Tab.closeCurrent();
+			mainTabSet.closeCurrent();
 		else
 			close();
 	}
 
 	protected ConfigView showConfig() {
 		if (config == null) {
-			config_view = new ConfigView(azureus_core);
-			config = new Tab(config_view);
+			config_view = new ConfigView();
+			config = mainTabSet.createTabItem(config_view, true);
 			config_view.getComposite().addDisposeListener(new DisposeListener() {
 				public void widgetDisposed(DisposeEvent e) {
 					config = null;
@@ -1480,7 +1360,7 @@ public class MainWindow
 				}
 			});
 		} else {
-			config.setFocus();
+			mainTabSet.setFocus(config);
 			refreshIconBar();
 			refreshTorrentMenu();
 		}
@@ -1507,15 +1387,15 @@ public class MainWindow
 
 	protected void showConsole() {
 		if (console == null) {
-			console = new Tab(new LoggerView(events));
-			console.getView().getComposite().addDisposeListener(
+			console = mainTabSet.createTabItem(new LoggerView(events), true);
+			mainTabSet.getView(console).getComposite().addDisposeListener(
 					new DisposeListener() {
 						public void widgetDisposed(DisposeEvent e) {
 							console = null;
 						}
 					});
 		} else {
-			console.setFocus();
+			mainTabSet.setFocus(console);
 			refreshIconBar();
 			refreshTorrentMenu();
 		}
@@ -1523,15 +1403,15 @@ public class MainWindow
 
 	protected void showStats() {
 		if (stats_tab == null) {
-			stats_tab = new Tab(new StatsView(globalManager, azureus_core));
-			stats_tab.getView().getComposite().addDisposeListener(
+			stats_tab = mainTabSet.createTabItem(new StatsView(), true);
+			mainTabSet.getView(stats_tab).getComposite().addDisposeListener(
 					new DisposeListener() {
 						public void widgetDisposed(DisposeEvent e) {
 							stats_tab = null;
 						}
 					});
 		} else {
-			stats_tab.setFocus();
+			mainTabSet.setFocus(stats_tab);
 			refreshIconBar();
 			refreshTorrentMenu();
 		}
@@ -1542,7 +1422,10 @@ public class MainWindow
 		if (stats_tab == null) {
 			return;
 		}
-		((StatsView) stats_tab.getView()).showDHT();
+		IView view = mainTabSet.getView(stats_tab);
+		if (view instanceof StatsView) {
+			((StatsView) view).showDHT();
+		}
 	}
 
 	protected void showStatsTransfers() {
@@ -1550,7 +1433,10 @@ public class MainWindow
 		if (stats_tab == null) {
 			return;
 		}
-		((StatsView) stats_tab.getView()).showTransfers();
+		IView view = mainTabSet.getView(stats_tab);
+		if (view instanceof StatsView) {
+			((StatsView) view).showTransfers();
+		}
 	}
 
 	protected void setSelectedLanguageItem() {
@@ -1568,18 +1454,14 @@ public class MainWindow
 			}
 
 			if (folder != null) {
-				if (useCustomTab) {
-					((CTabFolder) folder).update();
-				} else {
-					((TabFolder) folder).update();
-				}
+				folder.update();
 			}
 
 			if (downloadBasket != null) {
 				downloadBasket.updateLanguage();
 			}
 
-			Tab.updateLanguage();
+			mainTabSet.updateLanguage();
 
 			if (mainStatusBar != null) {
 				mainStatusBar.updateStatusText();
@@ -1618,34 +1500,6 @@ public class MainWindow
 		mainMenu = menu;
 	}
 
-	private void downloadManagerStateChanged(final DownloadManager manager,
-			int state) {
-		// if state == STARTED, then open the details window (according to config)
-		if (state == DownloadManager.STATE_DOWNLOADING
-				|| state == DownloadManager.STATE_SEEDING) {
-			if (display != null && !display.isDisposed()) {
-				Utils.execSWTThread(new AERunnable() {
-					public void runSupport() {
-						if (display == null || display.isDisposed())
-							return;
-
-						if (COConfigurationManager.getBooleanParameter("Open Details")) {
-							openManagerView(manager);
-						}
-
-						boolean complete = manager.isDownloadComplete(false);
-
-						if (((!complete) && COConfigurationManager.getBooleanParameter("Open Bar Incomplete"))
-								|| (complete && COConfigurationManager.getBooleanParameter("Open Bar Complete"))) {
-
-							DownloadBar.open(manager, shell);
-						}
-					}
-				});
-			}
-		}
-	}
-
 	protected AzureusCore getAzureusCore() {
 		return (azureus_core);
 	}
@@ -1656,48 +1510,8 @@ public class MainWindow
 
 		try {
 			writer.indent();
-
-			writer.println("MyTorrents");
-
-			Tab t = mytorrents;
-			if (t != null) {
-				try {
-					writer.indent();
-
-					t.generateDiagnostics(writer);
-				} finally {
-
-					writer.exdent();
-				}
-			}
-
-			t = my_tracker_tab;
-			if (t != null) {
-				writer.println("MyTracker");
-
-				try {
-					writer.indent();
-
-					t.generateDiagnostics(writer);
-				} finally {
-
-					writer.exdent();
-				}
-			}
-
-			t = my_shares_tab;
-			if (t != null) {
-				writer.println("MyShares");
-
-				try {
-					writer.indent();
-
-					t.generateDiagnostics(writer);
-				} finally {
-
-					writer.exdent();
-				}
-			}
+			
+			mainTabSet.generateDiagnostics(writer);
 
 			TableColumnManager.getInstance().generateDiagnostics(writer);
 		} finally {
@@ -1792,12 +1606,12 @@ public class MainWindow
 	public Image generateObfusticatedImage() {
 		Image image;
 
-		IView[] allViews = Tab.getAllViews();
+		IView[] allViews = mainTabSet.getAllViews();
 		for (int i = 0; i < allViews.length; i++) {
 			IView view = allViews[i];
 
 			if (view instanceof ObfusticateTab) {
-				Item tab = Tab.getTab(view);
+				Item tab = mainTabSet.getTab(view);
 				tab.setText(((ObfusticateTab) view).getObfusticatedHeader());
 				folder.update();
 			}
@@ -1841,9 +1655,15 @@ public class MainWindow
 		boolean isMaximized = COConfigurationManager.getBooleanParameter(
 				"window.maximized", false);
 		if (isMaximized) {
-			Display current = Display.getCurrent();
-			if (current != null) {
-				Rectangle clientArea = current.getClientArea();
+			Monitor monitor = SWTThread.getInstance().getPrimaryMonitor();
+			if (Utils.isThisThreadSWT()) {
+				if (window != null && window.getShell() != null
+						&& !window.getShell().isDisposed()) {
+					monitor = window.getShell().getMonitor();
+				}
+			}
+			if (monitor != null) {
+				Rectangle clientArea = monitor.getClientArea();
 				size = new Point(clientArea.width, clientArea.height);
 				return size;
 			}
@@ -1866,7 +1686,6 @@ public class MainWindow
 
 	public static void addToVersionCheckMessage(final Map map) {
 		try {
-			if (window == null || window.shell == null || window.shell.isDisposed()) {
 				Point size = getStoredWindowSize();
 				if (size == null) {
 					return;
@@ -1874,31 +1693,6 @@ public class MainWindow
 
 				map.put("mainwindow.w", new Long(size.x));
 				map.put("mainwindow.h", new Long(size.y));
-				return;
-			}
-
-			Utils.execSWTThread(new AERunnable() {
-				public void runSupport() {
-					Point size = null;
-
-					if (window != null) {
-						final Shell shell = window.getShell();
-						if (shell != null && !shell.isDisposed() && !shell.getMinimized()) {
-							size = shell.getSize();
-						}
-					}
-
-					if (size == null) {
-						size = getStoredWindowSize();
-						if (size == null) {
-							return;
-						}
-					}
-					map.put("mainwindow.w", new Long(size.x));
-					map.put("mainwindow.h", new Long(size.y));
-				}
-
-			}, false);
 		} catch (Throwable e) {
 			e.printStackTrace();
 		}
@@ -1939,7 +1733,7 @@ public class MainWindow
 
 			return mainStatusBar.getBounds();
 
-		} else if (windowElement == IMainWindow.WINDOW_ELEMENT_SEARCHBAR) {
+		} else if (windowElement == IMainWindow.WINDOW_ELEMENT_TOPBAR) {
 
 			//KN: No search bar in classic UI
 
@@ -1967,4 +1761,31 @@ public class MainWindow
 		return new Rectangle(0, 0, 0, 0);
 	}
 
+	/**
+	 * 
+	 *
+	 * @since 3.1.1.1
+	 */
+	public void closeAllDetails() {
+		if (mainTabSet != null) {
+			mainTabSet.closeAllDetails();
+		}
+	}
+
+	/**
+	 * @return
+	 *
+	 * @since 3.1.1.1
+	 */
+	public boolean hasDetailViews() {
+		if (mainTabSet != null) {
+			return( mainTabSet.hasDetails());
+		}
+		return false;
+	}
+
+	public Tab getMainTabSet() {
+		return mainTabSet;
+	}
+
 }
