@@ -5,6 +5,7 @@ import java.util.Iterator;
 import org.eclipse.swt.SWT;
 import org.eclipse.swt.events.*;
 import org.eclipse.swt.widgets.*;
+
 import org.gudy.azureus2.core3.config.COConfigurationManager;
 import org.gudy.azureus2.core3.config.ParameterListener;
 import org.gudy.azureus2.core3.download.DownloadManager;
@@ -13,12 +14,10 @@ import org.gudy.azureus2.core3.logging.LogEvent;
 import org.gudy.azureus2.core3.logging.LogIDs;
 import org.gudy.azureus2.core3.logging.Logger;
 import org.gudy.azureus2.core3.util.AERunnable;
-import org.gudy.azureus2.core3.util.AESemaphore;
 import org.gudy.azureus2.core3.util.Constants;
-import org.gudy.azureus2.plugins.ui.tables.TableCell;
-import org.gudy.azureus2.plugins.ui.tables.TableManager;
-import org.gudy.azureus2.plugins.ui.tables.TableRow;
-import org.gudy.azureus2.plugins.ui.tables.TableRowMouseListener;
+import org.gudy.azureus2.core3.util.Debug;
+import org.gudy.azureus2.plugins.ui.menus.MenuManager;
+import org.gudy.azureus2.plugins.ui.tables.*;
 import org.gudy.azureus2.plugins.update.UpdateCheckInstance;
 import org.gudy.azureus2.plugins.update.UpdateCheckInstanceListener;
 import org.gudy.azureus2.pluginsimpl.local.download.DownloadManagerImpl;
@@ -27,6 +26,7 @@ import org.gudy.azureus2.ui.swt.*;
 import org.gudy.azureus2.ui.swt.components.shell.ShellManager;
 import org.gudy.azureus2.ui.swt.config.wizard.ConfigureWizard;
 import org.gudy.azureus2.ui.swt.debug.UIDebugGenerator;
+import org.gudy.azureus2.ui.swt.donations.DonationWindow;
 import org.gudy.azureus2.ui.swt.exporttorrent.wizard.ExportTorrentWizard;
 import org.gudy.azureus2.ui.swt.help.AboutWindow;
 import org.gudy.azureus2.ui.swt.help.HealthHelpWindow;
@@ -84,8 +84,14 @@ public class MenuFactory
 
 		transferMenu.addMenuListener(new MenuListener() {
 			public void menuShown(MenuEvent menu) {
-				itemPause.setEnabled(getCore().getGlobalManager().canPauseDownloads());
-				itemResume.setEnabled(getCore().getGlobalManager().canResumeDownloads());
+				if (!AzureusCoreFactory.isCoreRunning()) {
+					itemPause.setEnabled(false);
+					itemResume.setEnabled(false);
+				} else {
+					AzureusCore core = AzureusCoreFactory.getSingleton();
+					itemPause.setEnabled(core.getGlobalManager().canPauseDownloads());
+					itemResume.setEnabled(core.getGlobalManager().canResumeDownloads());
+				}
 			}
 
 			public void menuHidden(MenuEvent menu) {
@@ -99,6 +105,10 @@ public class MenuFactory
 		return createTopLevelMenuItem(menuParent, MENU_ID_VIEW);
 	}
 
+	public static MenuItem createPublishMenuItem(Menu menuParent) {
+		return createTopLevelMenuItem(menuParent, MENU_ID_PUBLISH);
+	}
+
 	public static MenuItem createAdvancedMenuItem(Menu menuParent) {
 		return createTopLevelMenuItem(menuParent, MENU_ID_ADVANCED);
 	}
@@ -114,30 +124,27 @@ public class MenuFactory
 		 */
 		MenuBuildUtils.addMaintenanceListenerForMenu(torrentItem.getMenu(),
 				new MenuBuildUtils.MenuBuilder() {
-					public void buildMenu(Menu menu) {
+					public void buildMenu(Menu menu, MenuEvent menuEvent) {
 						DownloadManager[] current_dls = (DownloadManager[]) torrentItem.getData("downloads");
 						if (current_dls == null) {
 							return;
 						}
-						boolean is_detailed_view = ((Boolean) torrentItem.getData("is_detailed_view")).booleanValue();
-						TableViewSWT tv = (TableViewSWT) torrentItem.getData("TableView");
 
-						TorrentUtil.fillTorrentMenu(menu, current_dls, getCore(),
-								menuParent.getShell(), !is_detailed_view, 0, tv);
+						if (AzureusCoreFactory.isCoreRunning()) {
+							boolean is_detailed_view = ((Boolean) torrentItem.getData("is_detailed_view")).booleanValue();
+							TableViewSWT tv = (TableViewSWT) torrentItem.getData("TableView");
+							AzureusCore core = AzureusCoreFactory.getSingleton();
+
+							TorrentUtil.fillTorrentMenu(menu, current_dls, core,
+									menuParent.getShell(), !is_detailed_view, 0, tv);
+						}
 
-						/*
-						 * KN: This is a reference to a plugins class from a core class;
-						 * maybe MenuItem should be moved to core?
-						 * 
-						 * AMC: No, because we need to add menu items that plugins have
-						 * registered to add.
-						 */
 						org.gudy.azureus2.plugins.ui.menus.MenuItem[] menu_items;
 
 						menu_items = MenuItemManager.getInstance().getAllAsArray(
 								new String[] {
-									"torrentmenu",
-									"download_context"
+									MenuManager.MENU_TORRENT_MENU,
+									MenuManager.MENU_DOWNLOAD_CONTEXT
 								});
 
 						final Object[] plugin_dls = DownloadManagerImpl.getDownloadStatic(current_dls);
@@ -213,7 +220,7 @@ public class MenuFactory
 				MENU_ID_PLUGINS);
 		MenuBuildUtils.addMaintenanceListenerForMenu(pluginsMenuItem.getMenu(),
 				new MenuBuildUtils.MenuBuilder() {
-					public void buildMenu(Menu menu) {
+					public void buildMenu(Menu menu, MenuEvent menuEvent) {
 						PluginsMenuHelper.getInstance().buildPluginMenu(menu,
 								menuParent.getShell(), includeGetPluginsMenu);
 					}
@@ -234,7 +241,7 @@ public class MenuFactory
 		MenuItem file_create = addMenuItem(menuParent, MENU_ID_CREATE,
 				new Listener() {
 					public void handleEvent(Event e) {
-						new NewTorrentWizard(getCore(), getDisplay());
+						new NewTorrentWizard(e.display);
 					}
 				});
 		return file_create;
@@ -319,7 +326,7 @@ public class MenuFactory
 		MenuItem file_share_file = addMenuItem(menuParent, MENU_ID_SHARE_FILE,
 				new Listener() {
 					public void handleEvent(Event e) {
-						ShareUtils.shareFile(getCore(), menuParent.getShell());
+						ShareUtils.shareFile(menuParent.getShell());
 					}
 				});
 		return file_share_file;
@@ -329,7 +336,7 @@ public class MenuFactory
 		MenuItem file_share_dir = addMenuItem(menuParent, MENU_ID_SHARE_DIR,
 				new Listener() {
 					public void handleEvent(Event e) {
-						ShareUtils.shareDir(getCore(), menuParent.getShell());
+						ShareUtils.shareDir(menuParent.getShell());
 					}
 				});
 		return file_share_dir;
@@ -339,7 +346,7 @@ public class MenuFactory
 		MenuItem file_share_dircontents = addMenuItem(menuParent,
 				MENU_ID_SHARE_DIR_CONTENT, new Listener() {
 					public void handleEvent(Event e) {
-						ShareUtils.shareDirContents(getCore(), menuParent.getShell(), false);
+						ShareUtils.shareDirContents(menuParent.getShell(), false);
 					}
 				});
 		return file_share_dircontents;
@@ -350,7 +357,7 @@ public class MenuFactory
 		MenuItem file_share_dircontents_rec = addMenuItem(menuParent,
 				MENU_ID_SHARE_DIR_CONTENT_RECURSE, new Listener() {
 					public void handleEvent(Event e) {
-						ShareUtils.shareDirContents(getCore(), menuParent.getShell(), true);
+						ShareUtils.shareDirContents(menuParent.getShell(), true);
 					}
 				});
 		return file_share_dircontents_rec;
@@ -360,7 +367,7 @@ public class MenuFactory
 		MenuItem file_import = addMenuItem(menuParent, MENU_ID_IMPORT,
 				new Listener() {
 					public void handleEvent(Event e) {
-						new ImportTorrentWizard(getCore(), getDisplay());
+						new ImportTorrentWizard();
 					}
 				});
 		return file_import;
@@ -370,7 +377,7 @@ public class MenuFactory
 		MenuItem file_export = addMenuItem(menuParent, MENU_ID_EXPORT,
 				new Listener() {
 					public void handleEvent(Event e) {
-						new ExportTorrentWizard(getCore(), getDisplay());
+						new ExportTorrentWizard();
 					}
 				});
 		return file_export;
@@ -406,7 +413,7 @@ public class MenuFactory
 		final MenuItem item = addMenuItem(menu, MENU_ID_CLOSE_ALL_DETAIL,
 				new Listener() {
 					public void handleEvent(Event e) {
-						Tab.closeAllDetails();
+						UIFunctionsManagerSWT.getUIFunctionsSWT().closeAllDetails();
 					}
 				});
 
@@ -414,16 +421,14 @@ public class MenuFactory
 			public void handleEvent(Event event) {
 				if (true == MenuFactory.isEnabledForCurrentMode(item)) {
 					if (false == item.isDisposed() && false == event.widget.isDisposed()) {
-						item.setEnabled(Tab.hasDetails());
+						boolean hasDetails = UIFunctionsManagerSWT.getUIFunctionsSWT().hasDetailViews();
+						item.setEnabled(hasDetails);
 					}
 				}
 			}
 		};
 
 		menu.addListener(SWT.Show, enableHandler);
-		//		shell.addListener(SWT.FocusIn, enableHandler);
-		Tab.addTabAddedListener(enableHandler);
-		Tab.addTabRemovedListener(enableHandler);
 
 		return item;
 	}
@@ -493,9 +498,10 @@ public class MenuFactory
 	}
 
 	public static MenuItem addStartAllMenuItem(Menu menu) {
-		return addMenuItem(menu, MENU_ID_START_ALL_TRANSFERS, new Listener() {
-			public void handleEvent(Event event) {
-				getCore().getGlobalManager().startAllDownloads();
+		return addMenuItem(menu, MENU_ID_START_ALL_TRANSFERS,
+				new ListenerNeedingCoreRunning() {
+					public void handleEvent(AzureusCore core, Event e) {
+						core.getGlobalManager().startAllDownloads();
 				/*
 				 * KN: Not sure why we can not use the call below as opposed to the line above
 				 *  which was the exiting code
@@ -534,18 +540,29 @@ public class MenuFactory
 			public void handleEvent(Event e) {
 				UIFunctions uiFunctions = UIFunctionsManager.getUIFunctions();
 				if (uiFunctions != null) {
-					uiFunctions.showMyTorrents();
+					uiFunctions.openView(UIFunctions.VIEW_MYTORRENTS, null);
 				}
 			}
 		});
 	}
 
+	public static MenuItem addDetailedListMenuItem(Menu menu) {
+		return addMenuItem(menu, MENU_ID_DETAILED_LIST, new Listener() {
+			public void handleEvent(Event e) {
+				UIFunctions uiFunctions = UIFunctionsManager.getUIFunctions();
+				if (uiFunctions != null) {
+					uiFunctions.openView(UIFunctions.VIEW_DETAILED_LISTVIEW, null);
+				}
+			}
+		});
+	}
+	
 	public static MenuItem addAllPeersMenuItem(Menu menu) {
 		return addMenuItem(menu, MENU_ID_ALL_PEERS, new Listener() {
 			public void handleEvent(Event e) {
 				UIFunctions uiFunctions = UIFunctionsManager.getUIFunctions();
 				if (uiFunctions != null) {
-					uiFunctions.showAllPeersView();
+					uiFunctions.openView(UIFunctions.VIEW_ALLPEERS, null);
 				}
 			}
 		});
@@ -556,7 +573,7 @@ public class MenuFactory
 			public void handleEvent(Event e) {
 				UIFunctions uiFunctions = UIFunctionsManager.getUIFunctions();
 				if (uiFunctions != null) {
-					uiFunctions.showMyTracker();
+					uiFunctions.openView(UIFunctions.VIEW_MYTRACKER, null);
 				}
 			}
 		});
@@ -567,7 +584,7 @@ public class MenuFactory
 			public void handleEvent(Event e) {
 				UIFunctions uiFunctions = UIFunctionsManager.getUIFunctions();
 				if (uiFunctions != null) {
-					uiFunctions.showMyShares();
+					uiFunctions.openView(UIFunctions.VIEW_MYSHARES, null);
 				}
 			}
 		});
@@ -606,13 +623,13 @@ public class MenuFactory
 
 	public static MenuItem addTransferBarToMenu(final Menu menu) {
 		final MenuItem item = addMenuItem(menu, SWT.CHECK, MENU_ID_TRANSFER_BAR,
-				new Listener() {
-					public void handleEvent(Event e) {
+				new ListenerNeedingCoreRunning() {
+					public void handleEvent(AzureusCore core, Event e) {
 						if (AllTransfersBar.getManager().isOpen(
-								getCore().getGlobalManager())) {
-							AllTransfersBar.close(getCore().getGlobalManager());
+								core.getGlobalManager())) {
+							AllTransfersBar.close(core.getGlobalManager());
 						} else {
-							AllTransfersBar.open(getCore().getGlobalManager(),
+							AllTransfersBar.open(core.getGlobalManager(),
 									menu.getShell());
 						}
 					}
@@ -627,12 +644,9 @@ public class MenuFactory
 	}
 
 	public static MenuItem addBlockedIPsMenuItem(Menu menu) {
-		return addMenuItem(menu, MENU_ID_IP_FILTER, new Listener() {
-			public void handleEvent(Event event) {
-				//				if (MainWindow.isAlreadyDead) {
-				//					return;
-				//				}
-				BlockedIpsWindow.showBlockedIps(getCore(),
+		return addMenuItem(menu, MENU_ID_IP_FILTER, new ListenerNeedingCoreRunning() {
+			public void handleEvent(AzureusCore core, Event e) {
+				BlockedIpsWindow.showBlockedIps(core,
 						getUIFunctionSWT().getMainShell());
 			}
 		});
@@ -643,7 +657,7 @@ public class MenuFactory
 			public void handleEvent(Event e) {
 				UIFunctions uiFunctions = UIFunctionsManager.getUIFunctions();
 				if (uiFunctions != null) {
-					uiFunctions.showConsole();
+					uiFunctions.openView(UIFunctions.VIEW_CONSOLE, null);
 				}
 			}
 		});
@@ -654,7 +668,7 @@ public class MenuFactory
 			public void handleEvent(Event e) {
 				UIFunctions uiFunctions = UIFunctionsManager.getUIFunctions();
 				if (uiFunctions != null) {
-					uiFunctions.showStats();
+					uiFunctions.openView(UIFunctions.VIEW_STATS, null);
 				}
 			}
 		});
@@ -671,7 +685,7 @@ public class MenuFactory
 	public static MenuItem addSpeedTestMenuItem(Menu menu) {
 		return addMenuItem(menu, MENU_ID_SPEED_TEST, new Listener() {
 			public void handleEvent(Event e) {
-				new SpeedTestWizard(getCore(), getDisplay());
+				new SpeedTestWizard();
 			}
 		});
 	}
@@ -679,7 +693,7 @@ public class MenuFactory
 	public static MenuItem addConfigWizardMenuItem(Menu menu) {
 		return addMenuItem(menu, MENU_ID_CONFIGURE, new Listener() {
 			public void handleEvent(Event e) {
-				new ConfigureWizard(getCore(), false);
+				new ConfigureWizard(false);
 			}
 		});
 	}
@@ -689,23 +703,8 @@ public class MenuFactory
 			public void handleEvent(Event e) {
 				UIFunctions uiFunctions = UIFunctionsManager.getUIFunctions();
 				if (uiFunctions != null) {
-					uiFunctions.showConfig(null);
+					uiFunctions.openView(UIFunctions.VIEW_CONFIG, null);
 				}
-
-				/* ==== Code for opening the Preferences in a pop-up dialog ===
-				Shell shell = ShellFactory.createShell(SWT.RESIZE | SWT.DIALOG_TRIM);
-				shell.setLayout(new GridLayout());
-				shell.setText(MessageText.getString(MessageText.resolveLocalizationKey("ConfigView.title.full")));
-				Utils.setShellIcon(shell);
-				ConfigView cView = new ConfigView(getCore());
-				cView.initialize(shell);
-				shell.pack();
-				UIFunctionsSWT uiFunctions = getUIFunctionSWT();
-				if (uiFunctions != null) {
-					Utils.centerWindowRelativeTo(shell, uiFunctions.getMainShell());
-				}
-				shell.open();
-				*/
 			}
 		});
 	}
@@ -763,6 +762,9 @@ public class MenuFactory
 
 		final Listener enableHandler = new Listener() {
 			public void handleEvent(Event event) {
+				if (item.isDisposed()) {
+					return;
+				}
 				Iterator iter = ShellManager.sharedManager().getWindows();
 				boolean hasNonMaximizedShell = false;
 				while (iter.hasNext()) {
@@ -924,16 +926,10 @@ public class MenuFactory
 		});
 	}
 
-	/**
-	 * Add the FAQ menu item to the given menu
-	 * @param menu
-	 * @param faq_url the fully qualified url to the appropriate FAQ (Vuze uses a different url than Classic)
-	 * @return
-	 */
-	public static MenuItem addFAQMenuItem(Menu menu, final String faq_url) {
-		return addMenuItem(menu, MENU_ID_FAQ, new Listener() {
+	public static MenuItem addWikiMenuItem(Menu menu) {
+		return addMenuItem(menu, MENU_ID_COMMUNITY_WIKI, new Listener() {
 			public void handleEvent(Event e) {
-				Utils.launch(faq_url);
+				Utils.launch(Constants.AZUREUS_WIKI);
 			}
 		});
 	}
@@ -946,6 +942,24 @@ public class MenuFactory
 		});
 	}
 
+	
+	public static MenuItem addHelpSupportMenuItem(Menu menu, final String support_url ) {
+		return addMenuItem(menu, MENU_ID_HELP_SUPPORT, new Listener() {
+			public void handleEvent(Event e) {
+				Utils.launch( support_url );
+			}
+		});
+	}
+	
+	public static MenuItem addDonationMenuItem(Menu menu) {
+		return addMenuItem(menu, MENU_ID_DONATE, new Listener() {
+      public void handleEvent(Event e) {
+        DonationWindow.open(true, "menu");
+      }
+    });
+	}
+
+	
 	public static MenuItem addGetPluginsMenuItem(Menu menu) {
 		return addMenuItem(menu, MENU_ID_PLUGINS_HELP, new Listener() {
 			public void handleEvent(Event e) {
@@ -963,13 +977,13 @@ public class MenuFactory
 	}
 
 	public static MenuItem addCheckUpdateMenuItem(final Menu menu) {
-		return addMenuItem(menu, MENU_ID_UPDATE_CHECK, new Listener() {
-			public void handleEvent(Event e) {
+		return addMenuItem(menu, MENU_ID_UPDATE_CHECK, new ListenerNeedingCoreRunning() {
+			public void handleEvent(AzureusCore core, Event e) {
 				UIFunctionsSWT uiFunctions = UIFunctionsManagerSWT.getUIFunctionsSWT();
 				if (uiFunctions != null) {
 					uiFunctions.bringToFront();
 				}
-				UpdateMonitor.getSingleton(getCore()).performCheck(true, false, false,
+				UpdateMonitor.getSingleton(core).performCheck(true, false, false,
 						new UpdateCheckInstanceListener() {
 							public void cancelled(UpdateCheckInstance instance) {
 							}
@@ -991,17 +1005,19 @@ public class MenuFactory
 	}
 
 	public static MenuItem addPluginInstallMenuItem(Menu menuParent) {
-		return addMenuItem(menuParent, MENU_ID_PLUGINS_INSTALL, new Listener() {
-			public void handleEvent(Event e) {
-				new InstallPluginWizard(getCore(), getDisplay());
+		return addMenuItem(menuParent, MENU_ID_PLUGINS_INSTALL,
+				new Listener() {
+					public void handleEvent(Event e) {
+						new InstallPluginWizard();
 			}
 		});
 	}
 
 	public static MenuItem addPluginUnInstallMenuItem(Menu menuParent) {
-		return addMenuItem(menuParent, MENU_ID_PLUGINS_UNINSTALL, new Listener() {
-			public void handleEvent(Event e) {
-				new UnInstallPluginWizard(getCore(), getDisplay());
+		return addMenuItem(menuParent, MENU_ID_PLUGINS_UNINSTALL,
+				new Listener() {
+					public void handleEvent(Event e) {
+						new UnInstallPluginWizard(getDisplay());
 			}
 		});
 	}
@@ -1026,7 +1042,7 @@ public class MenuFactory
 		return new MenuItem(menuParent, SWT.SEPARATOR);
 	}
 
-	private static MenuItem createTopLevelMenuItem(Menu menuParent,
+	public static MenuItem createTopLevelMenuItem(Menu menuParent,
 			String localizationKey) {
 		Menu menu = new Menu(menuParent.getShell(), SWT.DROP_DOWN);
 		MenuItem menuItem = new MenuItem(menuParent, SWT.CASCADE);
@@ -1067,7 +1083,15 @@ public class MenuFactory
 
 	public static final MenuItem addMenuItem(Menu menu, int style,
 			String localizationKey, Listener selListener) {
-		MenuItem menuItem = new MenuItem(menu, style);
+		return addMenuItem(menu, style, -1, localizationKey, selListener);
+	}
+
+	public static final MenuItem addMenuItem(Menu menu, int style, int index,
+			String localizationKey, Listener selListener) {
+		if (index < 0 || index > menu.getItemCount()) {
+			index = menu.getItemCount();
+		}
+		MenuItem menuItem = new MenuItem(menu, style, index);
 		Messages.setLanguageText(menuItem, localizationKey);
 		KeyBindings.setAccelerator(menuItem, localizationKey);
 		menuItem.addListener(SWT.Selection, selListener);
@@ -1088,10 +1112,6 @@ public class MenuFactory
 				"No instance of UIFunctionsSWT found; the UIFunctionsManager might not have been initialized properly");
 	}
 
-	private static AzureusCore getCore() {
-		return AzureusCoreFactory.getSingleton();
-	}
-
 	private static Display getDisplay() {
 		return SWTThread.getInstance().getDisplay();
 	}
@@ -1112,6 +1132,8 @@ public class MenuFactory
 				KeyBindings.setAccelerator(item,
 						MessageText.resolveAcceleratorKey(localizationKey));
 				updateMenuText(item.getMenu());
+			} else {
+				Messages.updateLanguageForControl(item);
 			}
 		}
 	}
@@ -1322,6 +1344,8 @@ public class MenuFactory
 		}
 	}
 
+	private static final boolean DEBUG_SET_FOREGROUND = System.getProperty("debug.setforeground") != null;
+	
 	private static TableRow wrapAsRow(final Object o, final String table_name) {
 		return new TableRow() {
 			  public Object getDataSource() {return o;}
@@ -1330,11 +1354,17 @@ public class MenuFactory
 			  private void notSupported() {
 				  throw new RuntimeException("method is not supported - table row is a \"virtual\" one, only getDataSource and getTableID are supported.");
 			  }
+			  
+			  private void setForegroundDebug() {
+				  if (DEBUG_SET_FOREGROUND) {
+					  Debug.out("setForeground on fake TableRow");
+				  }
+			  }
 
 			  // Everything below is unsupported.
-			  public void setForeground(int red, int green, int blue) {notSupported();}
-			  public void setForeground(int[] rgb) {notSupported();}
-			  public void setForegroundToErrorColor() {notSupported();}
+			  public void setForeground(int red, int green, int blue) {setForegroundDebug(); notSupported();}
+			  public void setForeground(int[] rgb) {setForegroundDebug(); notSupported();}
+			  public void setForegroundToErrorColor() {setForegroundDebug(); notSupported();}
 			  public boolean isValid() {notSupported(); return false;}
 			  public TableCell getTableCell(String sColumnName) {notSupported(); return null;}
 			  public boolean isSelected()  {notSupported(); return false;}
