@@ -26,14 +26,7 @@ import org.eclipse.swt.SWT;
 import org.eclipse.swt.custom.StyledText;
 import org.eclipse.swt.layout.GridData;
 import org.eclipse.swt.layout.GridLayout;
-import org.eclipse.swt.widgets.Button;
-import org.eclipse.swt.widgets.Composite;
-import org.eclipse.swt.widgets.Display;
-import org.eclipse.swt.widgets.Event;
-import org.eclipse.swt.widgets.Label;
-import org.eclipse.swt.widgets.Listener;
-import org.eclipse.swt.widgets.Shell;
-import org.eclipse.swt.widgets.Text;
+import org.eclipse.swt.widgets.*;
 import org.gudy.azureus2.core3.config.COConfigurationManager;
 import org.gudy.azureus2.core3.internat.MessageText;
 import org.gudy.azureus2.core3.ipchecker.natchecker.NatChecker;
@@ -42,6 +35,11 @@ import org.gudy.azureus2.core3.util.AEThread;
 import org.gudy.azureus2.ui.swt.Messages;
 import org.gudy.azureus2.ui.swt.Utils;
 import org.gudy.azureus2.ui.swt.components.shell.ShellFactory;
+import org.gudy.azureus2.ui.swt.shells.CoreWaiterSWT;
+import org.gudy.azureus2.ui.swt.shells.CoreWaiterSWT.TriggerInThread;
+
+import com.aelitis.azureus.core.AzureusCore;
+import com.aelitis.azureus.core.AzureusCoreRunningListener;
 import com.aelitis.azureus.core.AzureusCoreFactory;
 import com.aelitis.azureus.core.networkmanager.admin.NetworkAdmin;
 
@@ -67,7 +65,7 @@ public class NatTestWindow {
 
     public void runSupport() {
           printMessage(MessageText.getString("configureWizard.nat.testing") + " " + TCPListenPort + " ... ");
-          NatChecker checker = new NatChecker(AzureusCoreFactory.getSingleton(), NetworkAdmin.getSingleton().getMultiHomedOutgoingRoundRobinBindAddress(), TCPListenPort, false);          
+          NatChecker checker = new NatChecker(AzureusCoreFactory.getSingleton(), NetworkAdmin.getSingleton().getMultiHomedServiceBindAddresses(true)[0], TCPListenPort, false);          
           switch (checker.getResult()) {
           case NatChecker.NAT_OK :
             printMessage(MessageText.getString("configureWizard.nat.ok") + "\n" + checker.getAdditionalInfo());
@@ -92,7 +90,7 @@ public class NatTestWindow {
   public NatTestWindow() {
     serverTCPListenPort = COConfigurationManager.getIntParameter( "TCP.Listen.Port" );
     
-    final Shell shell = ShellFactory.createShell(SWT.BORDER | SWT.TITLE | SWT.CLOSE);        
+    final Shell shell = ShellFactory.createShell(Utils.findAnyShell(), SWT.BORDER | SWT.TITLE | SWT.CLOSE);        
     shell.setText(MessageText.getString("configureWizard.nat.title"));
     Utils.setShellIcon(shell);
 
@@ -171,9 +169,14 @@ public class NatTestWindow {
       public void handleEvent(Event event) {
         bTest.setEnabled(false);
         textResults.setText("");
-        
-        checker = new Checker(serverTCPListenPort);
-        checker.start();
+
+        CoreWaiterSWT.waitForCore(TriggerInThread.ANY_THREAD,
+						new AzureusCoreRunningListener() {
+							public void azureusCoreRunning(AzureusCore core) {
+						checker = new Checker(serverTCPListenPort);
+						checker.start();
+					}
+				});
       }
     });
         
