@@ -27,6 +27,7 @@ package org.gudy.azureus2.ui.swt.pluginsimpl;
  *
  */
 
+import java.lang.ref.WeakReference;
 import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.Iterator;
@@ -36,10 +37,14 @@ import java.util.Map;
 import org.eclipse.swt.SWT;
 import org.eclipse.swt.layout.GridData;
 import org.eclipse.swt.layout.GridLayout;
+import org.eclipse.swt.layout.RowLayout;
 import org.eclipse.swt.widgets.Composite;
 import org.eclipse.swt.widgets.Control;
+import org.eclipse.swt.widgets.Event;
 import org.eclipse.swt.widgets.Group;
 import org.eclipse.swt.widgets.Label;
+import org.eclipse.swt.widgets.Listener;
+import org.gudy.azureus2.core3.config.COConfigurationManager;
 import org.gudy.azureus2.core3.internat.MessageText;
 import org.gudy.azureus2.core3.util.AERunnable;
 import org.gudy.azureus2.core3.util.Debug;
@@ -52,22 +57,26 @@ import org.gudy.azureus2.pluginsimpl.local.ui.config.*;
 import org.gudy.azureus2.ui.swt.Messages;
 import org.gudy.azureus2.ui.swt.Utils;
 import org.gudy.azureus2.ui.swt.config.*;
+import org.gudy.azureus2.ui.swt.components.LinkLabel;
 import org.gudy.azureus2.ui.swt.plugins.UISWTConfigSection;
+import org.gudy.azureus2.ui.swt.plugins.UISWTParameterContext;
 
 public class 
 BasicPluginConfigImpl
 	implements UISWTConfigSection
 {
-	protected BasicPluginConfigModel	model;
+	protected WeakReference<BasicPluginConfigModel>		model_ref;
 	
 	protected String					parent_section;
 	protected String					section;
 		
 	public
 	BasicPluginConfigImpl(
-		BasicPluginConfigModel	_model )
+		WeakReference<BasicPluginConfigModel>	_model_ref )
 	{
-		model			= _model;
+		model_ref			= _model_ref;
+		
+		BasicPluginConfigModel	model = model_ref.get();
 		
 		parent_section	= model.getParentSection();
 		section			= model.getSection();
@@ -103,13 +112,36 @@ BasicPluginConfigImpl
 	{
 		
 	}
+	
+	public int 
+	maxUserMode() 
+	{
+		BasicPluginConfigModel	model = model_ref.get();
+		
+		org.gudy.azureus2.plugins.ui.config.Parameter[] parameters = model.getParameters();
+		
+		int	max_mode = 0;
+		
+		for (int i=0;i<parameters.length;i++){
+			
+			final ParameterImpl	param = 	(ParameterImpl)parameters[i];
+		
+			if ( param.getMinimumRequiredUserMode() > max_mode ){
+				
+				max_mode = param.getMinimumRequiredUserMode();
+			}
+		}
+		
+		return( max_mode );
+	}
 
 
 	public Composite 
 	configSectionCreate(
 		final Composite parent ) 
 	{
-		
+		int userMode = COConfigurationManager.getIntParameter("User Mode");
+
 			// main tab set up
 		
 		Composite main_tab = new Composite(parent, SWT.NULL);
@@ -132,12 +164,19 @@ BasicPluginConfigImpl
 		
 		Composite current_composite	= main_tab;
 		
+		BasicPluginConfigModel	model = model_ref.get();
+
 		org.gudy.azureus2.plugins.ui.config.Parameter[] parameters = model.getParameters();
 		
 		for (int i=0;i<parameters.length;i++){
 			
 			final ParameterImpl	param = 	(ParameterImpl)parameters[i];
 		
+			if ( param.getMinimumRequiredUserMode() > userMode ){
+				
+				continue;
+			}
+			
 			ParameterGroupImpl	pg = param.getGroup();
 			
 			if ( pg == null ){
@@ -181,17 +220,49 @@ BasicPluginConfigImpl
 					label_text.indexOf('\t') != -1 ||
 					!(param instanceof BooleanParameterImpl)) {
 				
-				label = new Label(current_composite, param instanceof LabelParameterImpl
-							? SWT.WRAP : SWT.NULL);
+				String hyperlink = null;
+				if (param instanceof HyperlinkParameterImpl) {
+					hyperlink = ((HyperlinkParameterImpl)param).getHyperlink();
+				}
+
+				label = new Label(current_composite, (param instanceof LabelParameterImpl) ? SWT.WRAP : SWT.NULL);
 	
-				if ( label_key == null ){
-					
+				if ( label_key == null ){	
 					label.setText( param.getLabelText());
-					
 				}else{
-					
 					Messages.setLanguageText(label, label_key );
 				}
+				
+				if (hyperlink != null) {
+					LinkLabel.makeLinkedLabel(label, hyperlink);
+				}
+				
+				if (param instanceof HyperlinkParameterImpl) {
+					
+					final Label f_label = label;
+					
+					param.addListener(
+							new ParameterListener()
+							{
+								public void
+								parameterChanged(
+									org.gudy.azureus2.plugins.ui.config.Parameter	p )
+								{
+									if ( f_label.isDisposed()){
+							
+										param.removeListener( this );								
+									}else{
+										
+										String hyperlink = ((HyperlinkParameterImpl)param).getHyperlink();
+										
+										if (hyperlink != null) {
+											LinkLabel.updateLinkedLabel(f_label, hyperlink);
+										}
+									}
+								}
+							});				
+					}
+
 			}
 	
 			String	key = param.getKey();
@@ -235,12 +306,14 @@ BasicPluginConfigImpl
 				
 			}else if ( param instanceof IntParameterImpl ){
 						
-				swt_param = 
-					new IntParameter(
-						current_composite, 
-						key, 
-						((IntParameterImpl)param).getDefaultValue(),
-						false );	// don't want intermediate values
+				IntParameterImpl int_param = (IntParameterImpl)param;
+				swt_param = new IntParameter(current_composite, key,
+						int_param.getDefaultValue());
+				
+				if (int_param.isLimited()) {
+					((IntParameter)swt_param).setMinimumValue(int_param.getMinValue());
+					((IntParameter)swt_param).setMaximumValue(int_param.getMaxValue());
+				}
 				
 				param.addListener(
 						new ParameterListener()
@@ -265,14 +338,65 @@ BasicPluginConfigImpl
 				gridData.widthHint = 100;
 				
 				swt_param.setLayoutData( gridData );
-							
+				
+			}else if ( param instanceof ColorParameterImpl ) {
+				final Composite area = new Composite(current_composite, SWT.NULL);
+				//GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.FILL_HORIZONTAL );
+				GridData gridData = new GridData();
+				area.setLayoutData(gridData);
+				layout = new GridLayout();
+				layout.numColumns 	= 2;
+				layout.marginHeight = 0;
+				layout.marginWidth 	= 0;
+				area.setLayout(layout);
+				
+				final ButtonParameter[] reset_button_holder = new ButtonParameter[1];
+				final ColorParameterImpl color_param = (ColorParameterImpl)param;
+				swt_param = new org.gudy.azureus2.ui.swt.config.ColorParameter(
+						area, key, color_param.getRedValue(),
+						color_param.getGreenValue(), color_param.getBlueValue()) {
+					
+					public void newColorSet() {
+						color_param.reloadParamDataFromConfig(true);
+						if (reset_button_holder[0] == null) {return;}
+						reset_button_holder[0].getControl().setEnabled(true);
+					}
+				};
+				
+				// Reuse the same label as defined for Azureus UI reset buttons.
+				reset_button_holder[0] = new ButtonParameter(area, "ConfigView.section.style.colorOverrides.reset");
+				reset_button_holder[0].getControl().setEnabled(color_param.isOverridden());
+				reset_button_holder[0].getControl().addListener(SWT.Selection, new Listener(){
+					public void handleEvent(Event event) {
+						reset_button_holder[0].getControl().setEnabled(false);
+						color_param.resetToDefault();
+						color_param.reloadParamDataFromConfig(false);
+					}
+				});
+			
+				gridData = new GridData();
+				gridData.widthHint = 50;
+				
+				swt_param.setLayoutData( gridData );
 			}else if ( param instanceof StringParameterImpl ){
 				
 				GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
 				
 				gridData.widthHint = 150;
 
-				swt_param = new StringParameter(current_composite, key, ((StringParameterImpl)param).getDefaultValue());
+				StringParameterImpl s_param = (StringParameterImpl)param;
+				
+				swt_param = new StringParameter(current_composite, key, s_param.getDefaultValue(), s_param.getGenerateIntermediateEvents());
+				
+				swt_param.setLayoutData( gridData );
+				
+			}else if ( param instanceof InfoParameterImpl ){
+				
+				GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
+				
+				gridData.widthHint = 150;
+				
+				swt_param = new InfoParameter(current_composite, key, "" );
 				
 				swt_param.setLayoutData( gridData );
 				
@@ -284,7 +408,7 @@ BasicPluginConfigImpl
 				
 				gridData.widthHint = 150;
 
-				swt_param = new StringListParameter(current_composite, key, sl_param.getDefaultValue(), sl_param.getValues(), sl_param.getLabels());
+				swt_param = new StringListParameter(current_composite, key, sl_param.getDefaultValue(), sl_param.getLabels(), sl_param.getValues());
 				
 				swt_param.setLayoutData( gridData );
 				
@@ -298,7 +422,7 @@ BasicPluginConfigImpl
 				
 				swt_param.setLayoutData( gridData );
 				
-			}else if ( param instanceof DirectoryParameterImpl ){
+			}else if ( param instanceof DirectoryParameterImpl || param instanceof org.gudy.azureus2.pluginsimpl.local.ui.config.FileParameter){
 				
 				Composite area = new Composite(current_composite, SWT.NULL);
 
@@ -312,10 +436,16 @@ BasicPluginConfigImpl
 				layout.marginHeight = 0;
 				layout.marginWidth 	= 0;
 				
-				area.setLayout(layout);				
+				area.setLayout(layout);
+				
+				if (param instanceof DirectoryParameterImpl) {
+					swt_param = new DirectoryParameter(area, key, ((DirectoryParameterImpl)param).getDefaultValue());
+				}
+				else {
+					org.gudy.azureus2.pluginsimpl.local.ui.config.FileParameter fp = (org.gudy.azureus2.pluginsimpl.local.ui.config.FileParameter)param;
+					swt_param = new org.gudy.azureus2.ui.swt.config.FileParameter(area, key, fp.getDefaultValue(), fp.getFileExtensions());
+				}
 				
-				swt_param = new DirectoryParameter(area, key, ((DirectoryParameterImpl)param).getDefaultValue());
-		
 			}else if ( param instanceof ActionParameterImpl ){
 				
 				ActionParameterImpl	_param = (ActionParameterImpl)param;
@@ -330,7 +460,7 @@ BasicPluginConfigImpl
 				}
 				
 				swt_param.addChangeListener(
-						new ParameterChangeListener()
+						new ParameterChangeAdapter()
 						{
 							public void
 							parameterChanged(
@@ -344,6 +474,31 @@ BasicPluginConfigImpl
 								}
 							}
 						});
+			} else if ( param instanceof UIParameterImpl) {
+				if (((UIParameterImpl)param).getContext() instanceof UISWTParameterContext) {
+					UISWTParameterContext context = (UISWTParameterContext)((UIParameterImpl)param).getContext();
+					Composite internal_composite = new Composite(current_composite, SWT.NULL);
+					GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
+					internal_composite.setLayoutData(gridData);
+					boolean initialised_component = true;
+					try {context.create(internal_composite);}
+					catch (Exception e) {Debug.printStackTrace(e); initialised_component = false;}
+					
+					if (initialised_component) {
+						swt_param = new UISWTParameter(internal_composite, param.getKey());
+					}
+					else {
+						swt_param = null;
+						
+						// We're only expecting this for plugins which screw up their component generation,
+						// so we don't care that this text is not localised.
+						if (label != null) {label.setText("Error while generating UI component.");}
+					}
+				}
+				else {
+					swt_param = null;
+				}
+				
 			}else{
 				
 					// label
