@@ -25,6 +25,7 @@ package org.gudy.azureus2.ui.swt.pluginsimpl;
 import java.awt.*;
 import java.io.File;
 import java.io.InputStream;
+import java.lang.ref.WeakReference;
 import java.net.URL;
 import java.util.HashMap;
 import java.util.Map;
@@ -46,14 +47,25 @@ import org.gudy.azureus2.core3.internat.MessageText;
 import org.gudy.azureus2.core3.util.AERunnable;
 import org.gudy.azureus2.core3.util.AETemporaryFileHandler;
 import org.gudy.azureus2.core3.util.Debug;
+import org.gudy.azureus2.plugins.PluginInterface;
+import org.gudy.azureus2.plugins.download.Download;
+import org.gudy.azureus2.plugins.torrent.Torrent;
+import org.gudy.azureus2.plugins.ui.*;
+import org.gudy.azureus2.plugins.ui.menus.MenuItem;
+import org.gudy.azureus2.plugins.ui.model.BasicPluginConfigModel;
 import org.gudy.azureus2.plugins.ui.model.BasicPluginViewModel;
+import org.gudy.azureus2.plugins.ui.tables.TableColumn;
+import org.gudy.azureus2.plugins.ui.tables.TableColumnCreationListener;
+import org.gudy.azureus2.plugins.ui.tables.TableContextMenuItem;
+import org.gudy.azureus2.pluginsimpl.local.PluginInitializer;
+import org.gudy.azureus2.pluginsimpl.local.download.DownloadImpl;
+import org.gudy.azureus2.pluginsimpl.local.ui.UIManagerImpl;
 import org.gudy.azureus2.ui.common.util.MenuItemManager;
 import org.gudy.azureus2.ui.swt.*;
 import org.gudy.azureus2.ui.swt.mainwindow.*;
 import org.gudy.azureus2.ui.swt.minibar.AllTransfersBar;
 import org.gudy.azureus2.ui.swt.minibar.DownloadBar;
 import org.gudy.azureus2.ui.swt.plugins.*;
-import org.gudy.azureus2.ui.swt.pluginsimpl.UIMessageImpl;
 import org.gudy.azureus2.ui.swt.shells.MessageBoxShell;
 import org.gudy.azureus2.ui.swt.views.table.utils.TableColumnManager;
 import org.gudy.azureus2.ui.swt.views.table.utils.TableContextMenuManager;
@@ -61,35 +73,26 @@ import org.gudy.azureus2.ui.swt.views.utils.ManagerUtils;
 
 import com.aelitis.azureus.core.AzureusCore;
 import com.aelitis.azureus.ui.IUIIntializer;
+import com.aelitis.azureus.ui.UIFunctions;
 import com.aelitis.azureus.ui.UIFunctionsManager;
+import com.aelitis.azureus.ui.common.table.TableColumnCore;
+import com.aelitis.azureus.ui.common.table.TableStructureEventDispatcher;
 import com.aelitis.azureus.ui.common.table.impl.TableColumnImpl;
 import com.aelitis.azureus.ui.swt.UIFunctionsManagerSWT;
 import com.aelitis.azureus.ui.swt.UIFunctionsSWT;
 
-import org.gudy.azureus2.plugins.PluginInterface;
-import org.gudy.azureus2.plugins.download.Download;
-import org.gudy.azureus2.plugins.torrent.Torrent;
-import org.gudy.azureus2.plugins.ui.*;
-import org.gudy.azureus2.plugins.ui.menus.MenuItem;
-import org.gudy.azureus2.plugins.ui.model.BasicPluginConfigModel;
-import org.gudy.azureus2.plugins.ui.tables.TableColumn;
-import org.gudy.azureus2.plugins.ui.tables.TableContextMenuItem;
-
-import org.gudy.azureus2.pluginsimpl.local.download.DownloadImpl;
-import org.gudy.azureus2.pluginsimpl.local.ui.UIManagerImpl;
-
 public class 
 UISWTInstanceImpl
 	implements UIInstanceFactory, UISWTInstance, UIManagerEventListener
 {
 	private AzureusCore		core;
 	
-	private Map awt_view_map 	= new WeakHashMap();
-	private Map config_view_map = new WeakHashMap();
+	private Map<UISWTAWTPluginView,UISWTPluginView> 			awt_view_map 	= new WeakHashMap<UISWTAWTPluginView,UISWTPluginView>();
+	private Map<BasicPluginConfigModel,BasicPluginConfigImpl> 	config_view_map = new WeakHashMap<BasicPluginConfigModel,BasicPluginConfigImpl>();
 	
-	private Map views = new HashMap();
+	private Map<String,Map<String,UISWTViewEventListener>> views = new HashMap<String,Map<String,UISWTViewEventListener>>();
 	
-	private Map	plugin_map	= new WeakHashMap();
+	private Map<PluginInterface,UIInstance>	plugin_map = new WeakHashMap<PluginInterface,UIInstance>();
 	
 	private boolean bUIAttaching;
 
@@ -105,7 +108,7 @@ UISWTInstanceImpl
 	}
 	
 	public void init(IUIIntializer init) {
-		UIManager ui_manager = core.getPluginManager().getDefaultPluginInterface().getUIManager();
+		UIManager ui_manager = PluginInitializer.getDefaultInterface().getUIManager();
 		ui_manager.addUIEventListener(this);
 		
 		bUIAttaching = true;
@@ -275,7 +278,7 @@ UISWTInstanceImpl
 						if (auto_download) {
 							Shell shell = uiFunctions.getMainShell();
 							if (shell != null) {
-								new FileDownloadWindow(core, shell, target.toString(),
+								new FileDownloadWindow(shell, target.toString(),
 										referrer == null ? null : referrer.toString(),
 										request_properties );
 							}
@@ -326,7 +329,7 @@ UISWTInstanceImpl
 					
 					BasicPluginConfigModel	model = (BasicPluginConfigModel)data;
 					
-					BasicPluginConfigImpl view = new BasicPluginConfigImpl(model);
+					BasicPluginConfigImpl view = new BasicPluginConfigImpl( new WeakReference<BasicPluginConfigModel>( model ));
 					   
 					config_view_map.put( model, view );
 					
@@ -365,9 +368,13 @@ UISWTInstanceImpl
 			}
 			case UIManagerEvent.ET_CREATE_TABLE_COLUMN:{
 				
-	 			String[]	args = (String[])data;
-	 			
-	 			event.setResult( new TableColumnImpl(args[0], args[1]));
+				if (data instanceof TableColumn) {
+					event.setResult((TableColumn) data);
+				} else {
+					String[] args = (String[]) data;
+
+					event.setResult(new TableColumnImpl(args[0], args[1]));
+				}
 	 			
 	 			break;
 			} 
@@ -377,7 +384,11 @@ UISWTInstanceImpl
 				
 				if ( _col instanceof TableColumnImpl ){
 					
-					TableColumnManager.getInstance().addColumn((TableColumnImpl)_col);
+					TableColumnManager.getInstance().addColumns(new TableColumnCore[] {	(TableColumnCore) _col });
+					
+					TableStructureEventDispatcher tsed = TableStructureEventDispatcher.getInstance(_col.getTableID());
+					
+					tsed.tableStructureChanged(true, _col.getForDataSourceType());
 					
 				}else{
 					
@@ -386,6 +397,57 @@ UISWTInstanceImpl
 				
 				break;
 			} 
+			case UIManagerEvent.ET_REGISTER_COLUMN:{
+				
+				Object[] params = (Object[])data;
+				
+				TableColumnManager tcManager = TableColumnManager.getInstance();
+				
+				Class 	dataSource 	= (Class)params[0];
+				String	columnName	= (String)params[1];
+				
+				tcManager.registerColumn(dataSource, columnName, (TableColumnCreationListener)params[2]);
+				
+				String[] tables = tcManager.getTableIDs();
+				
+				for ( String tid: tables ){
+					
+						// we don't know which tables are affected at this point to refresh all. 
+						// if this proves to be a performance issue then we would have to use the
+						// datasource to derive affected tables somehow
+					
+					TableStructureEventDispatcher tsed = TableStructureEventDispatcher.getInstance( tid );
+							
+					tsed.tableStructureChanged(true, dataSource);
+				}
+				
+				break;
+			} 
+			case UIManagerEvent.ET_UNREGISTER_COLUMN:{
+				
+				Object[] params = (Object[])data;
+				
+				TableColumnManager tcManager = TableColumnManager.getInstance();
+				
+				Class 	dataSource 	= (Class)params[0];
+				String	columnName	= (String)params[1];
+
+				tcManager.unregisterColumn(dataSource, columnName, (TableColumnCreationListener)params[2]);
+				
+				String[] tables = tcManager.getTableIDs();
+				
+				for ( String tid: tables ){
+					
+					TableColumnCore col = tcManager.getTableColumnCore( tid, columnName );
+					
+					if ( col != null ){
+					
+						col.remove();
+					}
+				}
+				
+				break;
+			} 
 			case UIManagerEvent.ET_ADD_TABLE_CONTEXT_MENU_ITEM:{
 				TableContextMenuItem	item = (TableContextMenuItem)data;
 				TableContextMenuManager.getInstance().addContextMenuItem(item);
@@ -412,7 +474,9 @@ UISWTInstanceImpl
 				if (!(data instanceof String))
 					break;
 
-	    	event.setResult(new Boolean(uiFunctions.showConfig((String)data)));
+	    	event.setResult(Boolean.TRUE);
+	    	
+	    	uiFunctions.openView(UIFunctions.VIEW_CONFIG, data);
 
 				break;
 			}
@@ -479,6 +543,7 @@ UISWTInstanceImpl
 
 	/** @deprecated */
 	public void addView(final UISWTPluginView view, boolean bAutoOpen) {
+		// Currently used by firefrog.
 		try {
 			uiFunctions.addPluginView(view);
 			if (bAutoOpen) {
@@ -501,9 +566,11 @@ UISWTInstanceImpl
 	/** @deprecated */
 	public void
 	addView(
-		final UISWTAWTPluginView	view,
+		UISWTAWTPluginView			view,
 		boolean						auto_open )
 	{
+		final WeakReference<UISWTAWTPluginView> view_ref = new WeakReference<UISWTAWTPluginView>( view );
+		
 		UISWTPluginView	v = 
 			new UISWTPluginView()
 			{
@@ -515,13 +582,13 @@ UISWTInstanceImpl
 				public String
 				getPluginViewName()
 				{
-					return( view.getPluginViewName());
+					return( view_ref.get().getPluginViewName());
 				}
 				
 				public String 
 				getFullTitle() 
 				{
-					return( view.getPluginViewName());
+					return( view_ref.get().getPluginViewName());
 				}
 				 
 				public void 
@@ -554,7 +621,7 @@ UISWTInstanceImpl
 										
 										first_paint	= false;
 											
-										view.open( component );
+										view_ref.get().open( component );
 									}
 								}
 							}
@@ -564,7 +631,7 @@ UISWTInstanceImpl
 	
 					f.add( pan );
 							
-					component	= view.create();
+					component	= view_ref.get().create();
 					
 					pan.add( component, BorderLayout.CENTER );
 				}
@@ -580,7 +647,7 @@ UISWTInstanceImpl
 				{
 					super.delete();
 					
-					view.delete( component );
+					view_ref.get().delete( component );
 				}
 			};
 			
@@ -612,9 +679,9 @@ UISWTInstanceImpl
 
 	public void addView(String sParentID, final String sViewID,
 			final UISWTViewEventListener l) {
-		Map subViews = (Map) views.get(sParentID);
+		Map<String,UISWTViewEventListener> subViews = views.get(sParentID);
 		if (subViews == null) {
-			subViews = new HashMap();
+			subViews = new HashMap<String,UISWTViewEventListener>();
 			views.put(sParentID, subViews);
 		}
 
@@ -635,7 +702,7 @@ UISWTInstanceImpl
 	
 	// TODO: Remove views from PeersView, etc
 	public void removeViews(String sParentID, final String sViewID) {
-		Map subViews = (Map) views.get(sParentID);
+		Map<String,UISWTViewEventListener> subViews = views.get(sParentID);
 		if (subViews == null)
 			return;
 
@@ -662,12 +729,12 @@ UISWTInstanceImpl
 	
 	public boolean openView(final String sParentID, final String sViewID,
 			final Object dataSource, final boolean setfocus) {
-		Map subViews = (Map) views.get(sParentID);
+		Map<String,UISWTViewEventListener> subViews = views.get(sParentID);
 		if (subViews == null) {
 			return false;
 		}
 
-		final UISWTViewEventListener l = (UISWTViewEventListener) subViews.get(sViewID);
+		final UISWTViewEventListener l = subViews.get(sViewID);
 		if (l == null) {
 			return false;
 		}
@@ -753,12 +820,21 @@ UISWTInstanceImpl
 	// Core Functions
 	// ==============
 	
-	public Map getViewListeners(String sParentID) {
-		return (Map)views.get(sParentID);
+	public Map<String,UISWTViewEventListener> getViewListeners(String sParentID) {
+		return views.get(sParentID);
+	}
+	
+	/**
+	 * @return  Map(key=parentid, value=Map(key=id, value=UISWTViewEventListener))
+	 *
+	 * @since 3.1.1.1
+	 */
+	public Map<String,Map<String,UISWTViewEventListener>> getAllViews() {
+		return views;
 	}
 	
 	public UIInputReceiver getInputReceiver() {
-		return new SimpleTextEntryWindow(getDisplay());
+		return new SimpleTextEntryWindow();
 	}
 	
 	public UIMessage createMessage() {
@@ -799,7 +875,7 @@ UISWTInstanceImpl
 	public void openConfig(final BasicPluginConfigModel model) {
 		Utils.execSWTThread(new Runnable() {
 			public void run() {
-				uiFunctions.showConfig(model.getSection());
+	    	uiFunctions.openView(UIFunctions.VIEW_CONFIG, model.getSection());
 			}
 		});
 	}
@@ -808,15 +884,15 @@ UISWTInstanceImpl
 	instanceWrapper
 		implements UISWTInstance
 	{
-		private PluginInterface			pi;
-		private UISWTInstanceImpl		delegate;
+		private WeakReference<PluginInterface>		pi_ref;
+		private UISWTInstanceImpl					delegate;
 		
 		protected
 		instanceWrapper(
 			PluginInterface		_pi,
 			UISWTInstanceImpl	_delegate )
 		{
-			pi			= _pi;
+			pi_ref		= new WeakReference<PluginInterface>(_pi );
 			delegate	= _delegate;
 		}
 		
@@ -838,6 +914,13 @@ UISWTInstanceImpl
 		loadImage(
 			String	resource )
 		{
+			PluginInterface pi = pi_ref.get();
+			
+			if ( pi == null ){
+				
+				return( null );
+			}
+			
 			return( delegate.loadImage( pi, resource ));
 		}
 		
