@@ -20,7 +20,8 @@
  */
 package org.gudy.azureus2.ui.swt.shells;
 
-import java.util.*;
+import java.util.ArrayList;
+import java.util.Iterator;
 import java.util.List;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
@@ -32,7 +33,9 @@ import org.eclipse.swt.graphics.*;
 import org.eclipse.swt.layout.*;
 import org.eclipse.swt.widgets.*;
 
-import org.gudy.azureus2.ui.swt.ImageRepository;
+import org.gudy.azureus2.core3.util.Debug;
+
+import com.aelitis.azureus.ui.swt.imageloader.ImageLoader;
 
 /**
  * @author Olivier Chalouhi
@@ -44,13 +47,13 @@ public class GCStringPrinter
 
 	private static final String GOOD_STRING = "(/|,jI~`gy";
 
-	private static final int FLAG_SKIPCLIP = 1;
+	public static final int FLAG_SKIPCLIP = 1;
 
-	private static final int FLAG_FULLLINESONLY = 2;
+	public static final int FLAG_FULLLINESONLY = 2;
 
-	private static final int FLAG_NODRAW = 4;
+	public static final int FLAG_NODRAW = 4;
 
-	private static final int FLAG_KEEP_URL_INFO = 8;
+	public static final int FLAG_KEEP_URL_INFO = 8;
 
 	private static final Pattern patHREF = Pattern.compile(
 			"<\\s*?a\\s.*?href\\s*?=\\s*?\"(.+?)\".*?>(.*?)<\\s*?/a\\s*?>",
@@ -100,11 +103,13 @@ public class GCStringPrinter
 		public String text;
 
 		public Color urlColor;
+		
+		public Color dropShadowColor;
 
 		int relStartPos;
 
 		// We could use a region, but that uses a resource that requires disposal
-		List hitAreas = null;
+		public List hitAreas = null;
 
 		int titleLength;
 
@@ -114,6 +119,8 @@ public class GCStringPrinter
 
 		public String target;
 
+		public boolean urlUnderline;
+
 		// @see java.lang.Object#toString()
 		public String toString() {
 			return super.toString() + ": relStart=" + relStartPos + ";url=" + url
@@ -184,6 +191,31 @@ public class GCStringPrinter
 		return false;
 	}
 
+	private boolean _printString() {
+		boolean b = false;
+		try {
+			boolean wasAdvanced = gc.getAdvanced();
+			Rectangle clipping = null;
+			// With Advanced ontext antialias in SWT.DEFAULT is not the system's default
+			// Turn off Advanced while drawing text so it antialiases based on 
+			// system prefs.
+			if (gc.getAdvanced() && gc.getTextAntialias() == SWT.DEFAULT
+					&& gc.getAlpha() == 255) {
+				clipping = gc.getClipping();
+				gc.setAdvanced(false);
+				gc.setClipping(clipping);
+			}
+			b = __printString();
+			if (wasAdvanced) {
+				gc.setAdvanced(true);
+				gc.setClipping(clipping);
+			}
+		} catch (Throwable t) {
+			Debug.out(t);
+		}
+		return b;
+	}
+
 	/**
 	 * @param gc
 	 * @param string
@@ -194,7 +226,7 @@ public class GCStringPrinter
 	 *
 	 * @since 3.0.4.3
 	 */
-	private boolean _printString() {
+	private boolean __printString() {
 		size = new Point(0, 0);
 
 		if (string == null) {
@@ -419,10 +451,6 @@ public class GCStringPrinter
 				}
 			}
 		} finally {
-			if (!skipClip && !noDraw) {
-				gc.setClipping(oldClipping);
-			}
-
 			if (lines.size() > 0) {
 				// rebuild full text to get the exact y-extent of the output
 				// this may be different (but shouldn't be!) than the height of each
@@ -440,7 +468,7 @@ public class GCStringPrinter
 
 				for (Iterator iter = lines.iterator(); iter.hasNext();) {
 					LineInfo lineInfo = (LineInfo) iter.next();
-					size.x += Math.max(lineInfo.width, size.x);
+					size.x = Math.max(lineInfo.width, size.x);
 					size.y += lineInfo.height;
 				}
 				
@@ -462,10 +490,16 @@ public class GCStringPrinter
 					}
 				}
 			}
+
+			if (!skipClip && !noDraw) {
+				gc.setClipping(oldClipping);
+			}
+
 		}
 		
 
-		return size.y <= printArea.height;
+		cutoff |= size.y > printArea.height;
+		return !cutoff;
 	}
 
 	/**
@@ -564,7 +598,12 @@ public class GCStringPrinter
 		}
 
 		if (!wrap && hasMoreElements && excessPos >= 0) {
-			outputLine.replace(outputLine.length() - 1, outputLine.length(), "..");
+			int len = outputLine.length();
+			if (len > 2) {
+				len -= 2;
+			}
+			outputLine.setLength(len);
+			outputLine.append("\u2026");
 			cutoff = true;
 		}
 		//drawLine(gc, outputLine, swtFlags, rectDraw);
@@ -607,6 +646,9 @@ public class GCStringPrinter
 
 				if (newWidth > printArea.width) {
 					if (bounds.width + spaceExtent.x < printArea.width || lineInfo.width > 0) {
+						//outputLine.append(space);
+						//outputLine.append(word, 0, 2);
+						//System.out.println("w1 = " + lineInfo.width + ";h=" + lineInfo.height);
 						return 0;
 					}
 				}
@@ -615,22 +657,43 @@ public class GCStringPrinter
 					lineInfo.imageIndexes = new int[] { imgIdx };
 				}
 				
+				
+				int targetWidth = lineInfo.width + newWidth;
+				
 				lineInfo.width = newWidth;
 				lineInfo.height = Math.max(bounds.height, lineInfo.height);
 
-				outputLine.append(space);
-				outputLine.append(word);
-				if (space.length() > 0) {
-					space.delete(0, space.length());
+				Point ptWordSize = gc.stringExtent(word.substring(2) + " ");
+				if (lineInfo.width + ptWordSize.x > printArea.width) {
+					outputLine.append(space);
+					outputLine.append(word.substring(0,2));
+					//System.out.println("w8 = " + lineInfo.width + ";h=" + lineInfo.height);
+					return 2;
 				}
-				space.append(' ');
 				
-				return -1;
+				outputLine.append(space);
+				space.setLength(0);
+				outputLine.append(word.substring(0, 2));
+				word = word.substring(2);
+				//outputLine.append(word);
+				//if (space.length() > 0) {
+				//	space.delete(0, space.length());
+				//}
+				//space.append(' ');
+				
+				//System.out.println("w2 = " + lineInfo.width + ";h=" + lineInfo.height);
+				//return -1;
 			}
 		}
 
 		Point ptWordSize = gc.stringExtent(word + " ");
 		boolean bWordLargerThanWidth = ptWordSize.x > printArea.width;
+		// This will split put a word that is longer than a full line onto a new
+		// line (when the existing line has text).
+		if (bWordLargerThanWidth && lineInfo.width > 0) {
+			//System.out.println("w3 = " + lineInfo.width + ";h=" + lineInfo.height);
+			return 0;
+		}
 		int targetWidth = lineInfo.width + ptWordSize.x;
 		if (targetWidth > printArea.width) {
 			// word is longer than space avail, split
@@ -702,8 +765,12 @@ public class GCStringPrinter
 					} else if (sLine.length() > 0) {
 						outputLine.append(sLine.charAt(0));
 					}
-				} else if (len > 1) {
-					outputLine.replace(outputLine.length() - 1, outputLine.length(), "..");
+				} else {
+					if (len > 2) {
+						len -= 2;
+					}
+					outputLine.setLength(len);
+					outputLine.append("\u2026");
 					cutoff = true;
 				}
 			}
@@ -711,6 +778,7 @@ public class GCStringPrinter
 			if (DEBUG) {
 				System.out.println("excess " + word.substring(endIndex));
 			}
+			//System.out.println("w9 = " + lineInfo.width + ";h=" + lineInfo.height);
 			return endIndex;
 		}
 
@@ -728,12 +796,18 @@ public class GCStringPrinter
 					} else if (sLine.length() > 0) {
 						outputLine.append(sLine.charAt(0));
 					}
-				} else if (len > 1) {
-					outputLine.replace(outputLine.length() - 1, outputLine.length(), "..");
+				} else {
+					if (len > 2) {
+						len -= 2;
+					}
+					outputLine.setLength(len);
+					outputLine.append("\u2026");
 					cutoff = true;
 				}
+				//System.out.println("w5 = " + lineInfo.width + ";h=" + lineInfo.height);
 				return -1;
 			} else {
+				//System.out.println("w6 = " + lineInfo.width + ";h=" + lineInfo.height);
 				return 0;
 			}
 			//drawLine(gc, outputLine, swtFlags, rectDraw);
@@ -748,6 +822,7 @@ public class GCStringPrinter
 		}
 		space.append(' ');
 
+		//System.out.println("w4 = " + lineInfo.width + ";h=" + lineInfo.height);
 		return -1;
 	}
 
@@ -849,6 +924,12 @@ public class GCStringPrinter
 				Color fgColor = null;
 				if (!noDraw) {
 					fgColor = gc.getForeground();
+
+					if (urlInfo.dropShadowColor != null) {
+						gc.setForeground(urlInfo.dropShadowColor);
+						drawText(gc, s, x0 + 1, y0 + 1, lineInfo.height, null, noDraw);
+					}
+
 					if (urlInfo.urlColor != null) {
 						gc.setForeground(urlInfo.urlColor);
 					} else if (urlColor != null) {
@@ -860,6 +941,9 @@ public class GCStringPrinter
 				}
 				pt = drawText(gc, s, x0, y0, lineInfo.height, urlInfo.hitAreas, noDraw);
 				if (!noDraw) {
+					if (urlInfo.urlUnderline) {
+						gc.drawLine(x0, y0 + pt.y - 1, x0 + pt.x - 1, y0 + pt.y - 1); 
+					}
 					gc.setForeground(fgColor);
 				}
 
@@ -998,13 +1082,13 @@ public class GCStringPrinter
 		final Display display = Display.getDefault();
 		final Shell shell = new Shell(display, SWT.SHELL_TRIM);
 
-		ImageRepository.loadImagesForSplashWindow(display);
+		ImageLoader imageLoader = ImageLoader.getInstance();
 		
 		final Image[] images = {
-			ImageRepository.getImage("azureus32"),
-			ImageRepository.getImage("azureus64"),
-			ImageRepository.getImage("azureus"),
-			ImageRepository.getImage("azureus128"),
+			imageLoader.getImage("azureus32"),
+			imageLoader.getImage("azureus64"),
+			imageLoader.getImage("azureus"),
+			imageLoader.getImage("azureus128"),
 		};
 
 		//final String text = "Opil Wrir, Na Poys Iysk, Yann Only. test of the string printer averlongwordthisisyesindeed";
@@ -1075,6 +1159,10 @@ public class GCStringPrinter
 		btnWrap.setText("Wrap");
 		btnWrap.setSelection(true);
 		btnWrap.addListener(SWT.Selection, l);
+		
+		final Label lblInfo = new Label(shell, SWT.WRAP);
+		lblInfo.setText("Welcome");
+		
 
 		Listener l2 = new Listener() {
 			URLInfo lastHitInfo = null;
@@ -1103,7 +1191,9 @@ public class GCStringPrinter
 						if (url1.equals(url2)) {
 							return;
 						}
+						cPaint.redraw();
 						lastHitInfo = hitUrl;
+						return;
 					}
 
 					Rectangle bounds = cPaint.getClientArea();
@@ -1117,9 +1207,14 @@ public class GCStringPrinter
 					sp.setUrlColor(colorURL);
 					URLInfo hitUrl = sp.getHitUrl(pt.x, pt.y);
 					if (hitUrl != null) {
+						shell.setCursor(shell.getDisplay().getSystemCursor(SWT.CURSOR_HAND));
 						hitUrl.urlColor = colorURL2;
+					} else {
+						shell.setCursor(null);
 					}
-					sp.printString();
+					boolean fit = sp.printString();
+					
+					lblInfo.setText(fit ? "fit" : "no fit");
 
 					bounds.width--;
 					bounds.height--;
@@ -1140,6 +1235,8 @@ public class GCStringPrinter
 			}
 
 			private GCStringPrinter buildSP(GC gc) {
+				//gc.setFont(Utils.getFontWithHeight(shell.getFont(), gc, 15));
+				//gc.setTextAntialias(SWT.ON);
 				Rectangle bounds = cPaint.getClientArea();
 
 				int style = btnWrap.getSelection() ? SWT.WRAP : 0;
@@ -1299,6 +1396,15 @@ public class GCStringPrinter
 		this.imageScales = imageScales;
 	}
 
+	/**
+	 * @return
+	 *
+	 * @since 4.0.0.1
+	 */
+	public String getText() {
+		return string;
+	}
+
 	/*
 	private Point stringExtent(GC gc, String s) {
 		Matcher m = patOver1000.matcher(s);
