@@ -20,7 +20,6 @@
 package org.gudy.azureus2.ui.swt.shells;
 
 import java.util.ArrayList;
-import java.util.regex.Pattern;
 
 import org.eclipse.swt.SWT;
 import org.eclipse.swt.custom.StyledText;
@@ -42,6 +41,7 @@ import org.gudy.azureus2.ui.swt.mainwindow.ClipboardCopy;
 import org.gudy.azureus2.ui.swt.shells.GCStringPrinter.URLInfo;
 
 import com.aelitis.azureus.ui.swt.*;
+import com.aelitis.azureus.ui.swt.imageloader.ImageLoader;
 import com.aelitis.azureus.ui.swt.utils.ColorCache;
 
 /**
@@ -68,8 +68,6 @@ public class MessageSlideShell
 
 	private static final boolean DEBUG = false;
 
-	private final static String REGEX_URLHTML = "<A HREF=\"(.+?)\">(.+?)</A>";
-
 	/** Slide until there's this much gap between shell and edge of screen */
 	private final static int EDGE_GAP = 0;
 
@@ -135,11 +133,8 @@ public class MessageSlideShell
 
 	private Image imgPopup;
 
-	/** Forces the timer feature to be active; overriding the default behavior */
-	private boolean forceTimer = true;
-
 	protected Color colorURL;
-	
+
 	private Color colorFG;
 
 	private int shellWidth;
@@ -157,16 +152,9 @@ public class MessageSlideShell
 	 * @note Display moved to end to remove conflict in constructors
 	 */
 	public MessageSlideShell(Display display, int iconID, String keyPrefix,
-			String details, String[] textParams) {
+			String details, String[] textParams, int timeoutSecs ) {
 		this(display, iconID, MessageText.getString(keyPrefix + ".title"),
-				MessageText.getString(keyPrefix + ".text", textParams), details);
-	}
-
-	public MessageSlideShell(Display display, int iconID, String keyPrefix,
-			String details, String[] textParams, Object[] relatedObjects) {
-		this(display, iconID, MessageText.getString(keyPrefix + ".title"),
-				MessageText.getString(keyPrefix + ".text", textParams), details,
-				relatedObjects, false);
+				MessageText.getString(keyPrefix + ".text", textParams), details, timeoutSecs);
 	}
 
 	/**
@@ -177,19 +165,19 @@ public class MessageSlideShell
 	 * @param details
 	 * @param textParams
 	 * @param relatedObjects
-	 * @param forceTimer Forces the timer feature to be active; overriding the default logic
+	 * @param timeoutSecs = -1 -> use default timeout, 0 -> no timeout, other -> timeout in secs 
 	 */
 	public MessageSlideShell(Display display, int iconID, String keyPrefix,
 			String details, String[] textParams, Object[] relatedObjects,
-			boolean forceTimer) {
+			int timeoutSecs ) {
 		this(display, iconID, MessageText.getString(keyPrefix + ".title"),
 				MessageText.getString(keyPrefix + ".text", textParams), details,
-				relatedObjects, forceTimer);
+				relatedObjects, timeoutSecs);
 	}
 
 	public MessageSlideShell(Display display, int iconID, String title,
-			String text, String details) {
-		this(display, iconID, title, text, details, null, false);
+			String text, String details, int timeoutSecs) {
+		this(display, iconID, title, text, details, null, timeoutSecs);
 	}
 
 	/**
@@ -201,17 +189,15 @@ public class MessageSlideShell
 	 * @param text Text to put in the body
 	 * @param details Text displayed when the Details button is pressed.  Null
 	 *                 for disabled Details button.
-	 * @param forceTimer Forces the timer feature to be active; overriding the default logic
+	 * @param timeoutSecs = -1 -> use default timeout, 0 -> no timeout, other -> timeout in secs 
 	 */
 	public MessageSlideShell(Display display, int iconID, String title,
-			String text, String details, Object[] relatedObjects, boolean forceTimer) {
+			String text, String details, Object[] relatedObjects, int timeoutSecs ) {
 		try {
 			monitor.enter();
 
-			this.forceTimer = forceTimer;
-
 			PopupParams popupParams = new PopupParams(iconID, title, text, details,
-					relatedObjects);
+					relatedObjects, timeoutSecs );
 			historyList.add(popupParams);
 			if (currentPopupIndex < 0) {
 				create(display, popupParams, true);
@@ -255,10 +241,10 @@ public class MessageSlideShell
 	 * @param relatedTo 
 	 */
 	public static void recordMessage(int iconID, String title, String text,
-			String details, Object[] relatedTo) {
+			String details, Object[] relatedTo, int timeoutSecs ) {
 		try {
 			monitor.enter();
-			historyList.add(new PopupParams(iconID, title, text, details, relatedTo));
+			historyList.add(new PopupParams(iconID, title, text, details, relatedTo, timeoutSecs));
 			if (firstUnreadMessage == -1) {
 				firstUnreadMessage = historyList.size() - 1;
 			}
@@ -311,37 +297,18 @@ public class MessageSlideShell
 		sDetails = popupParams.details;
 
 		// Load Images
-		// Disable BG Image on OSX
-		if (imgPopup == null) {
-			if (Constants.isOSX && (SWT.getVersion() < 3221 || !USE_SWT32_BG_SET)) {
-				USE_SWT32_BG_SET = false;
-				imgPopup = null;
-			} else {
-				imgPopup = ImageRepository.getImage("popup");
-			}
-		}
-		Rectangle imgPopupBounds;
-		if (imgPopup != null) {
-			shellWidth = imgPopup.getBounds().width;
-			imgPopupBounds = imgPopup.getBounds();
-		} else {
-			shellWidth = SHELL_DEF_WIDTH;
-			imgPopupBounds = null;
-		}
-		Image imgIcon = popupParams.iconID <= 0 ? null : display.getSystemImage(popupParams.iconID);
+		Image imgIcon = popupParams.iconID <= 0 ? null
+				: display.getSystemImage(popupParams.iconID);
 
 		/*
 		 * If forceTimer is true then we always show the counter for auto-closing the shell;
 		 * otherwise proceed to the more fine-grained logic
 		 */
-		if (true == forceTimer) {
-			bDelayPaused = false;
-		} else {
 			// if there's a link, or the info is non-information,
 			// disable timer and mouse watching
-			bDelayPaused = UrlUtils.parseHTMLforURL(popupParams.text) != null
-					|| popupParams.iconID != SWT.ICON_INFORMATION || !bSlide;
-		}
+
+		bDelayPaused = popupParams.iconID != SWT.ICON_INFORMATION || !bSlide;
+
 		// Pause the auto-close delay when mouse is over slidey
 		// This will be applies to every control
 		final MouseTrackAdapter mouseAdapter = bDelayPaused ? null
@@ -380,6 +347,38 @@ public class MessageSlideShell
 		Utils.setShellIcon(shell);
 		shell.setText(popupParams.title);
 
+		// Disable BG Image on OSX
+		if (imgPopup == null) {
+			if (Constants.isOSX && (SWT.getVersion() < 3221 || !USE_SWT32_BG_SET)) {
+				USE_SWT32_BG_SET = false;
+				imgPopup = null;
+			} else {
+				imgPopup = ImageLoader.getInstance().getImage("popup");
+				shell.addDisposeListener(new DisposeListener() {
+					public void widgetDisposed(DisposeEvent e) {
+						ImageLoader.getInstance().releaseImage("popup");
+					}
+				});
+			}
+		}
+		Rectangle imgPopupBounds;
+		if (imgPopup != null) {
+			shellWidth = imgPopup.getBounds().width;
+			
+			/*
+			 * KN: The buttons at the bottom of the shell has an automatic horizontal spacing on OSX;
+			 * compensating the shellWidth so that the buttons will not overlap the image at the bottom left
+			 * of the shell
+			 */
+			if (true == Constants.isOSX) {
+				shellWidth += 30;
+			}
+			imgPopupBounds = imgPopup.getBounds();
+		} else {
+			shellWidth = SHELL_DEF_WIDTH;
+			imgPopupBounds = null;
+		}
+
 		UISkinnableSWTListener[] listeners = UISkinnableManagerSWT.getInstance().getSkinnableListeners(
 				MessageSlideShell.class.toString());
 		for (int i = 0; i < listeners.length; i++) {
@@ -389,7 +388,7 @@ public class MessageSlideShell
 				Debug.out(e);
 			}
 		}
-		
+
 		if (colorFG == null) {
 			colorFG = display.getSystemColor(SWT.COLOR_BLACK);
 		}
@@ -756,7 +755,18 @@ public class MessageSlideShell
 			}
 		}
 
-		runPopup(endBounds, idxHistory, bSlide);
+		int	timeoutSecs;
+		
+		if ( popupParams.timeoutSecs < 0 ){
+		
+			timeoutSecs = COConfigurationManager.getIntParameter("Message Popup Autoclose in Seconds");
+			
+		}else{
+			
+			timeoutSecs = popupParams.timeoutSecs;
+		}
+		
+		runPopup(endBounds, idxHistory, bSlide, timeoutSecs );
 	}
 
 	/**
@@ -771,8 +781,8 @@ public class MessageSlideShell
 			public Point computeSize(int wHint, int hHint, boolean changed) {
 				Rectangle area = new Rectangle(0, 0, shellWidth, 5000);
 				GC gc = new GC(this);
-				GCStringPrinter sp = new GCStringPrinter(gc, popupParams.text, area, true,
-						false, SWT.WRAP | SWT.TOP);
+				GCStringPrinter sp = new GCStringPrinter(gc, popupParams.text, area,
+						true, false, SWT.WRAP | SWT.TOP);
 				sp.calculateMetrics();
 				gc.dispose();
 				Point size = sp.getCalculatedSize();
@@ -812,7 +822,7 @@ public class MessageSlideShell
 				} else if (e.type == SWT.MouseUp) {
 					if (sp != null) {
 						URLInfo hitUrl = sp.getHitUrl(e.x, e.y);
-						if (hitUrl != null) {
+						if (hitUrl != null && !hitUrl.url.startsWith(":")) {
 							Utils.launch(hitUrl.url);
 						}
 					}
@@ -938,7 +948,7 @@ public class MessageSlideShell
 	 * @param bSlide Whether to slide in, or show immediately
 	 */
 	private void runPopup(final Rectangle endBounds, final int idx,
-			final boolean bSlide) {
+			final boolean bSlide, final int timeoutSecs ) {
 		if (shell == null || shell.isDisposed())
 			return;
 
@@ -966,7 +976,7 @@ public class MessageSlideShell
 					});
 				}
 
-				int delayLeft = COConfigurationManager.getIntParameter("Message Popup Autoclose in Seconds") * 1000;
+				int delayLeft = timeoutSecs * 1000;
 				final boolean autohide = (delayLeft != 0);
 
 				long lastDelaySecs = 0;
@@ -1065,11 +1075,6 @@ public class MessageSlideShell
 		});
 	}
 
-	public static String stripOutHyperlinks(String message) {
-		return Pattern.compile(REGEX_URLHTML, Pattern.CASE_INSENSITIVE).matcher(
-				message).replaceAll("$2");
-	}
-
 	/**
 	 * Waits until all slideys are closed before returning to caller.
 	 */
@@ -1098,17 +1103,20 @@ public class MessageSlideShell
 
 		Object[] relatedTo;
 
+		int	timeoutSecs;
+		
 		/**
 		 * @param iconID
 		 * @param title
 		 * @param text
 		 * @param details
 		 */
-		public PopupParams(int iconID, String title, String text, String details) {
+		public PopupParams(int iconID, String title, String text, String details, int timeoutSecs ) {
 			this.iconID = iconID;
 			this.title = title;
 			this.text = text;
 			this.details = details;
+			this.timeoutSecs = timeoutSecs;
 			addedOn = System.currentTimeMillis();
 		}
 
@@ -1120,8 +1128,8 @@ public class MessageSlideShell
 		 * @param relatedTo
 		 */
 		public PopupParams(int iconID, String title, String text, String details,
-				Object[] relatedTo) {
-			this(iconID, title, text, details);
+				Object[] relatedTo, int timeoutSecs ) {
+			this(iconID, title, text, details, timeoutSecs );
 			this.relatedTo = relatedTo;
 		}
 	}
@@ -1153,8 +1161,6 @@ public class MessageSlideShell
 
 	public static void test(Display display) {
 
-		ImageRepository.loadImages(display);
-
 		String title = "This is the title that never ends, never ends!";
 		String text = "This is a very long message with lots of information and "
 				+ "stuff you really should read.  Are you still reading? Good, because "
@@ -1177,24 +1183,24 @@ public class MessageSlideShell
 		//				MessagePopupShell.ICON_INFO, "Title", text, "Details");
 
 		new MessageSlideShell(display, SWT.ICON_INFORMATION,
-				"Simple. . . . . . . . . . . . . . . . . . .", "Simple", (String) null);
+				"Simple. . . . . . . . . . . . . . . . . . .", "Simple", (String) null, -1);
 
 		new MessageSlideShell(display, SWT.ICON_INFORMATION, title + "1", text,
-				"Details: " + text);
+				"Details: " + text, -1);
 
 		new MessageSlideShell(display, SWT.ICON_INFORMATION, "ShortTitle2",
-				"ShortText", "Details");
+				"ShortText", "Details", -1);
 		MessageSlideShell.waitUntilClosed();
 
 		new MessageSlideShell(display, SWT.ICON_INFORMATION, "ShortTitle3",
-				"ShortText", (String) null);
+				"ShortText", (String) null, -1);
 		for (int x = 0; x < 10; x++)
 			text += "\n\n\n\n\n\n\n\nWow";
 		new MessageSlideShell(display, SWT.ICON_INFORMATION, title + "4", text,
-				"Details");
+				"Details", -1);
 
 		new MessageSlideShell(display, SWT.ICON_ERROR, title + "5", text,
-				(String) null);
+				(String) null, -1);
 
 		MessageSlideShell.waitUntilClosed();
 	}
