@@ -33,6 +33,7 @@ import org.gudy.azureus2.core3.logging.LogEvent;
 import org.gudy.azureus2.core3.logging.LogIDs;
 import org.gudy.azureus2.core3.logging.Logger;
 import org.gudy.azureus2.core3.util.*;
+import org.gudy.azureus2.ui.swt.Utils;
 import org.gudy.azureus2.ui.swt.components.StringListChooser;
 import org.gudy.azureus2.ui.swt.progress.IProgressReport;
 import org.gudy.azureus2.ui.swt.progress.IProgressReportConstants;
@@ -49,7 +50,10 @@ import com.aelitis.azureus.ui.swt.UIFunctionsManagerSWT;
 
 import org.gudy.azureus2.plugins.PluginInterface;
 import org.gudy.azureus2.plugins.update.*;
+import org.gudy.azureus2.plugins.utils.DelayedTask;
 import org.gudy.azureus2.plugins.utils.resourcedownloader.ResourceDownloader;
+import org.gudy.azureus2.pluginsimpl.local.PluginInitializer;
+import org.gudy.azureus2.pluginsimpl.local.utils.UtilitiesImpl;
 
 /**
  * @author Olivier Chalouhi
@@ -96,7 +100,7 @@ public class UpdateMonitor
 	protected UpdateMonitor(AzureusCore _azureus_core) {
 		azCore = _azureus_core;
 
-		PluginInterface defPI = azCore.getPluginManager().getDefaultPluginInterface();
+		PluginInterface defPI = PluginInitializer.getDefaultInterface();
 		UpdateManager um = defPI.getUpdateManager();
 
 		um.addListener(new UpdateManagerListener() {
@@ -157,52 +161,69 @@ public class UpdateMonitor
 					}
 				});
 
-		// wait a bit before starting check to give rest of AZ time to initialise 
-		new DelayedEvent("UpdateMon:wait", 2500, new AERunnable() {
-			public void runSupport() {
+		DelayedTask delayed_task = 
+			UtilitiesImpl.addDelayedTask(
+				"Update Check", 
+				new Runnable()
+				{
+					public void
+					run()
+					{
+						// check for non-writeable app dir on non-vista platforms (vista we've got a chance of
+						// elevating perms when updating) and warn user. Particularly useful on OSX when
+						// users haven't installed properly
+		
+						if ( !( Constants.isWindowsVistaOrHigher || SystemProperties.isJavaWebStartInstance())){
+		
+							String	app_str = SystemProperties.getApplicationPath();
+		
+							if ( !new File(app_str).canWrite()){
+		
+								final UIFunctions uiFunctions = UIFunctionsManager.getUIFunctions();
+		
+								if ( uiFunctions != null ){
+		
+									if ( app_str.endsWith( File.separator )){
+		
+										app_str = app_str.substring(0, app_str.length()-1);
+									}
+									
+									final String f_app_str = app_str;
+									
+									Utils.execSWTThread(
+										new Runnable()
+										{
+											public void
+											run()
+											{
+												UIFunctionsUserPrompter prompt = 
+													uiFunctions.getUserPrompter(
+														MessageText.getString("updater.cant.write.to.app.title"), 
+														MessageText.getString("updater.cant.write.to.app.details", new String[]{f_app_str}), 
+														new String[]{ MessageText.getString( "Button.ok" )}, 
+														0 );
 				
-					// check for non-writeable app dir on non-vista platforms (vista we've got a chance of
-					// elevating perms when updating) and warn user. Particularly useful on OSX when
-					// users haven't installed properly
-			
-				if ( !( Constants.isWindowsVista || SystemProperties.isJavaWebStartInstance())){
-				
-					String	app_str = SystemProperties.getApplicationPath();
+												//prompt.setHtml( "http://a.b.c/" );
 					
-					if ( !new File(app_str).canWrite()){
-						
-						UIFunctions uiFunctions = UIFunctionsManager.getUIFunctions();
-						
-						if ( uiFunctions != null ){
-							
-							if ( app_str.endsWith( File.separator )){
-								
-								app_str = app_str.substring(0, app_str.length()-1);
+												prompt.setIconResource( "warning" );
+					
+												prompt.setRememberID( "UpdateMonitor.can.not.write.to.app.dir.2", false );
+					
+												prompt.setRememberText( MessageText.getString( "MessageBoxWindow.nomoreprompting" ));
+					
+												prompt.open();
+											}
+										},
+										true );
+								}
 							}
-							
-							UIFunctionsUserPrompter prompt = 
-								uiFunctions.getUserPrompter(
-									MessageText.getString("updater.cant.write.to.app.title"), 
-									MessageText.getString("updater.cant.write.to.app.details", new String[]{app_str}), 
-									new String[]{ MessageText.getString( "Button.ok" )}, 
-									0 );
-							
-							//prompt.setHtml( "http://a.b.c/" );
-							
-							prompt.setIconResource( "warning" );
-							
-							prompt.setRememberID( "UpdateMonitor.can.not..write.to.app.dir", false );
-							
-							prompt.setRememberText( MessageText.getString( "MessageBoxWindow.nomoreprompting" ));
-							
-							prompt.open();
 						}
+		
+						performAutoCheck(true);
 					}
-				}
-				
-				performAutoCheck(true);
-			}
-		});
+				});
+		
+		delayed_task.queue();
 	}
 
 	protected class updateStatusChanger
@@ -457,7 +478,7 @@ public class UpdateMonitor
 
 		AEThread2 t = new AEThread2("UpdateMonitor:kickoff", true) {
 			public void run() {
-				UpdateManager um = azCore.getPluginManager().getDefaultPluginInterface().getUpdateManager();
+				UpdateManager um = PluginInitializer.getDefaultInterface().getUpdateManager();
 
 				current_update_instance = um.createUpdateCheckInstance(bForce
 						? UpdateCheckInstance.UCI_INSTALL : UpdateCheckInstance.UCI_UPDATE,
@@ -478,7 +499,7 @@ public class UpdateMonitor
 		t.start();
 	}
 
-	public void complete(UpdateCheckInstance instance) {
+	public void complete( final UpdateCheckInstance instance) {
 		
 		if ( instance.isLowNoise()){
 			
@@ -486,20 +507,11 @@ public class UpdateMonitor
 			
 			return;
 		}
-		// we can get here for either update actions (triggered above) or for plugin
-		// install actions (triggered by the plugin installer)
-
-		boolean update_action = instance.getType() == UpdateCheckInstance.UCI_UPDATE;
-
-		UIFunctions uiFunctions = UIFunctionsManager.getUIFunctions();
-		if (uiFunctions != null) {
-			uiFunctions.setStatusText("");
-		}
+		
+		boolean hasDownloads = false;
 
 		Update[] us = instance.getUpdates();
 
-		boolean hasDownloads = false;
-
 		// updates with zero-length downloaders exist for admin purposes
 		// and shoudn't cause the window to be shown if only they exist
 
@@ -513,6 +525,33 @@ public class UpdateMonitor
 			}
 		}
 
+		try{
+			int ui = (Integer)instance.getProperty( UpdateCheckInstance.PT_UI_STYLE );
+		
+			if ( ui == UpdateCheckInstance.PT_UI_STYLE_SIMPLE ){
+				
+				new SimpleInstallUI( this, instance );
+				
+				return;
+			}
+			
+		}catch( Throwable e ){
+			
+			Debug.printStackTrace(e);
+		}
+		
+		// we can get here for either update actions (triggered above) or for plugin
+		// install actions (triggered by the plugin installer)
+
+		boolean update_action = instance.getType() == UpdateCheckInstance.UCI_UPDATE;
+
+		UIFunctions uiFunctions = UIFunctionsManager.getUIFunctions();
+		if (uiFunctions != null) {
+			uiFunctions.setStatusText("");
+		}
+
+
+
 		// this controls whether or not the update window is displayed
 		// note that we just don't show the window if this is set, we still do the
 		// update check (as amongst other things we want ot know the latest
@@ -553,7 +592,18 @@ public class UpdateMonitor
 			} else {
 				if (autoDownload) {
 					new UpdateAutoDownloader(us, new UpdateAutoDownloader.cbCompletion() {
-						public void allUpdatesComplete(boolean requiresRestart, boolean bHadMandatoryUpdates) {
+						public void 
+						allUpdatesComplete(
+							boolean requiresRestart, 
+							boolean bHadMandatoryUpdates) 
+						{
+							Boolean b = (Boolean)instance.getProperty( UpdateCheckInstance.PT_CLOSE_OR_RESTART_ALREADY_IN_PROGRESS );
+							
+							if ( b != null && b ){
+								
+								return;
+							}
+							
 							if (requiresRestart) {
 								handleRestart();
 							}else if ( bHadMandatoryUpdates ){
@@ -611,7 +661,7 @@ public class UpdateMonitor
 	}
 		
 	protected void
-	handleLowNoise(
+	addDecisionHandler(
 		UpdateCheckInstance		instance )
 	{
 		instance.addDecisionListener(
@@ -656,6 +706,13 @@ public class UpdateMonitor
 		  				return( null );
 		  			}
 		  		});		
+	}
+	
+	protected void
+	handleLowNoise(
+		UpdateCheckInstance		instance )
+	{
+		addDecisionHandler( instance );
 				
 		Update[] updates = instance.getUpdates();
 		
