@@ -2,7 +2,7 @@
  * Created on 7 mai 2004
  * Created by Olivier Chalouhi
  * 
- * Copyright (C) 2004 Aelitis SARL, All rights Reserved
+ * Copyright (C) 2004, 2005, 2006 Aelitis SAS, All rights Reserved
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,51 +17,42 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  * 
- * AELITIS, SARL au capital de 30,000 euros,
+ * AELITIS, SAS au capital de 46,603.30 euros,
  * 8 Allee Lenotre, La Grille Royale, 78600 Le Mesnil le Roi, France.
  */
 package org.gudy.azureus2.ui.swt.update;
 
 import java.io.InputStream;
-import java.util.ArrayList;
-import java.util.HashMap;
-import java.util.Iterator;
+import java.util.*;
 import java.util.List;
-import java.util.Map;
 
 import org.eclipse.swt.SWT;
+import org.eclipse.swt.browser.Browser;
 import org.eclipse.swt.custom.SashForm;
-import org.eclipse.swt.custom.StyledText;
-import org.eclipse.swt.layout.FormAttachment;
-import org.eclipse.swt.layout.FormData;
-import org.eclipse.swt.layout.FormLayout;
-import org.eclipse.swt.widgets.Button;
-import org.eclipse.swt.widgets.Display;
-import org.eclipse.swt.widgets.Event;
-import org.eclipse.swt.widgets.Label;
-import org.eclipse.swt.widgets.Listener;
-import org.eclipse.swt.widgets.ProgressBar;
-import org.eclipse.swt.widgets.Shell;
-import org.eclipse.swt.widgets.Table;
-import org.eclipse.swt.widgets.TableColumn;
-import org.eclipse.swt.widgets.TableItem;
-
-import com.aelitis.azureus.core.*;
+import org.eclipse.swt.layout.*;
+import org.eclipse.swt.widgets.*;
 
 import org.gudy.azureus2.core3.config.COConfigurationManager;
 import org.gudy.azureus2.core3.internat.MessageText;
-import org.gudy.azureus2.core3.util.AERunnable;
-import org.gudy.azureus2.core3.util.Constants;
-import org.gudy.azureus2.core3.util.DisplayFormatters;
+import org.gudy.azureus2.core3.util.*;
+import org.gudy.azureus2.ui.swt.Messages;
+import org.gudy.azureus2.ui.swt.Utils;
+import org.gudy.azureus2.ui.swt.components.LinkArea;
+import org.gudy.azureus2.ui.swt.components.StringListChooser;
+import org.gudy.azureus2.ui.swt.components.shell.ShellFactory;
+import org.gudy.azureus2.ui.swt.mainwindow.MainStatusBar;
+import org.gudy.azureus2.ui.swt.mainwindow.SWTThread;
+
+import com.aelitis.azureus.core.AzureusCore;
+import com.aelitis.azureus.ui.swt.UIFunctionsManagerSWT;
+import com.aelitis.azureus.ui.swt.UIFunctionsSWT;
+
 import org.gudy.azureus2.plugins.update.Update;
 import org.gudy.azureus2.plugins.update.UpdateCheckInstance;
+import org.gudy.azureus2.plugins.update.UpdateManagerDecisionListener;
 import org.gudy.azureus2.plugins.utils.resourcedownloader.ResourceDownloader;
 import org.gudy.azureus2.plugins.utils.resourcedownloader.ResourceDownloaderException;
 import org.gudy.azureus2.plugins.utils.resourcedownloader.ResourceDownloaderListener;
-import org.gudy.azureus2.ui.swt.ImageRepository;
-import org.gudy.azureus2.ui.swt.Messages;
-import org.gudy.azureus2.ui.swt.mainwindow.MainWindow;
-import org.gudy.azureus2.ui.swt.mainwindow.SWTThread;
 
 /**
  * @author Olivier Chalouhi
@@ -69,10 +60,9 @@ import org.gudy.azureus2.ui.swt.mainwindow.SWTThread;
  */
 public class 
 UpdateWindow
-	extends 	AERunnable
 	implements 	ResourceDownloaderListener{
   
-  private AzureusCore			azureus_core;
+  private UpdateMonitor			update_monitor;
   private UpdateCheckInstance	check_instance;
   private int					check_type;
   
@@ -80,7 +70,7 @@ UpdateWindow
   
   Shell updateWindow;
   Table table;
-  StyledText stDescription;
+  LinkArea link_area;
   ProgressBar progress;
   Label status;
   
@@ -92,13 +82,13 @@ UpdateWindow
   
   
   
-  boolean askingForShow;
-  
-  boolean restartRequired;
+  boolean	hasMandatoryUpdates;
+  boolean 	restartRequired;
   
   private long totalDownloadSize;
   private List downloaders;
   private Iterator iterDownloaders;
+	private Browser browser;
   
   private static final int COL_NAME = 0;
   private static final int COL_VERSION = 1;
@@ -107,40 +97,88 @@ UpdateWindow
   
   public 
   UpdateWindow(
-  		AzureusCore			_azureus_core,
-  		UpdateCheckInstance	_check_instance )
+	UpdateMonitor		_update_monitor,
+  	AzureusCore			_azureus_core,
+  	UpdateCheckInstance	_check_instance )
   {
-  	azureus_core	= _azureus_core;
+	update_monitor	= _update_monitor;
   	check_instance 	= _check_instance;
   	
   	check_type = check_instance.getType();
   	
-    this.display = SWTThread.getInstance().getDisplay();
+  	check_instance.addDecisionListener(
+	  		new UpdateManagerDecisionListener()
+	  		{
+	  			public Object
+	  			decide(
+	  				Update		update,
+	  				int			decision_type,
+	  				String		decision_name,
+	  				String		decision_description,
+	  				Object		decision_data )
+	  			{
+	  				if ( decision_type == UpdateManagerDecisionListener.DT_STRING_ARRAY_TO_STRING ){
+	  					
+	  					String[]	options = (String[])decision_data;
+  					
+	  					Shell	shell = updateWindow;
+	  					
+	  					if ( shell == null ){
+	  						
+	  						Debug.out( "Shell doesn't exist" );
+	  						
+	  						return( null );
+	  					}
+	  					
+	  					StringListChooser chooser = new StringListChooser( shell );
+	  					
+	  					chooser.setTitle( decision_name );
+	  					chooser.setText( decision_description );
+	  					
+	  					for (int i=0;i<options.length;i++){
+	  						
+	  						chooser.addOption( options[i] );
+	  					}
+	  					
+	  					String	result = chooser.open();
+	  					
+	  					return( result );
+	  				}
+	  				
+	  				return( null );
+	  			}
+	  		});
+  	
     this.updateWindow = null;
-    this.askingForShow =false;
-    if(display != null && !display.isDisposed())
-      display.asyncExec(this);
+    this.display = SWTThread.getInstance().getDisplay();
+    
+    Utils.execSWTThreadWithBool("UpdateWindow", new AERunnableBoolean() {
+			public boolean runSupport() {
+				buildWindow();
+				return true;
+			}
+		});
   }
   
   //The Shell creation process
-  public void runSupport() {
+  public void buildWindow() {
     if(display == null || display.isDisposed())
       return;
+
+    Utils.waitForModals();
     
     //Do not use ~SWT.CLOSE cause on some linux/GTK platform it
     //forces the window to be only 200x200
     //catch close event instead, and never do it
-    updateWindow = new Shell(display,(SWT.DIALOG_TRIM | SWT.RESIZE) );
+		updateWindow = ShellFactory.createMainShell(SWT.DIALOG_TRIM | SWT.RESIZE);
     
     updateWindow.addListener(SWT.Close,new Listener() {
       public void handleEvent(Event e) {
-        e.doit = false;
+        dispose();
       }
     });
     
-    if(! Constants.isOSX) {
-      updateWindow.setImage(ImageRepository.getImage("azureus"));
-    }
+    Utils.setShellIcon(updateWindow);
     
     String	res_prefix = "swt.";
     
@@ -154,7 +192,7 @@ UpdateWindow
     	
     }else{
     	
-    	res_prefix += "update.window";
+    	res_prefix = "UpdateWindow";
     }
     
     Messages.setLanguageText(updateWindow, res_prefix + ".title");
@@ -180,10 +218,10 @@ UpdateWindow
        
     table = new Table(sash,SWT.CHECK | SWT.BORDER | SWT.SINGLE | SWT.FULL_SELECTION);
     String[] names = {"name" , "version" , "size"};
-    int[] sizes = {220,80,80};
+    int[] sizes = {350,100,100};
     for(int i = 0 ; i < names.length ; i++) {
-      TableColumn column = new TableColumn(table,SWT.LEFT);
-      Messages.setLanguageText(column,"swt.update.window.columns." + names[i]);
+      TableColumn column = new TableColumn(table, i == 0 ? SWT.LEFT : SWT.RIGHT);
+      Messages.setLanguageText(column,"UpdateWindow.columns." + names[i]);
       column.setWidth(sizes[i]);
     }
     table.setHeaderVisible(true);    
@@ -194,9 +232,28 @@ UpdateWindow
       }
     });
     
-    stDescription = new StyledText(sash,SWT.BORDER | SWT.READ_ONLY | SWT.H_SCROLL | SWT.V_SCROLL);    
-    stDescription.setWordWrap(true);
+    Composite cInfoArea = new Composite(sash, SWT.NONE);
+    cInfoArea.setLayout(new FormLayout());
     
+    link_area = new LinkArea(cInfoArea);
+    FormData fd = new FormData();
+    fd.top = new FormAttachment(0, 0);
+    fd.bottom = new FormAttachment(100, 0);
+    fd.right = new FormAttachment(100, 0);
+    fd.left = new FormAttachment(0, 0);
+    link_area.getComponent().setLayoutData(fd);
+    
+    try {
+    	browser = new Browser(cInfoArea, Utils.getInitialBrowserStyle(SWT.BORDER));
+      fd = new FormData();
+      fd.top = new FormAttachment(0, 0);
+      fd.bottom = new FormAttachment(100, 0);
+      fd.right = new FormAttachment(100, 0);
+      fd.left = new FormAttachment(0, 0);
+      browser.setLayoutData(fd);
+    } catch (Throwable t) {
+    }
+
     progress = new ProgressBar(updateWindow,SWT.NULL);
     progress.setMinimum(0);
     progress.setMaximum(100);
@@ -205,8 +262,12 @@ UpdateWindow
     status = new Label(updateWindow,SWT.NULL);
     
  
+    Composite cButtons = new Composite(updateWindow, SWT.NONE);
+		FillLayout fl = new FillLayout(SWT.HORIZONTAL);
+		fl.spacing = 3;
+		cButtons.setLayout(fl);
     
-    btnOk = new Button(updateWindow,SWT.PUSH);
+    btnOk = new Button(cButtons,SWT.PUSH);
     Messages.setLanguageText(btnOk,res_prefix + ".ok" );
     
     updateWindow.setDefaultButton( btnOk );
@@ -219,9 +280,9 @@ UpdateWindow
     btnOk.addListener(SWT.Selection, lOk);
     btnOk.setEnabled( false );
     
-    btnCancel = new Button(updateWindow,SWT.PUSH);
+    btnCancel = new Button(cButtons,SWT.PUSH);
     
-    Messages.setLanguageText(btnCancel,"swt.update.window.cancel");
+    Messages.setLanguageText(btnCancel,"UpdateWindow.cancel");
     
     lCancel = new Listener() {
 	      public void handleEvent(Event e) {
@@ -256,22 +317,17 @@ UpdateWindow
     formData = new FormData();
     formData.left = new FormAttachment(0,0);
     formData.right = new FormAttachment(100,0);
-    formData.bottom = new FormAttachment(btnCancel);
+    formData.bottom = new FormAttachment(cButtons);
     status.setLayoutData(formData);
     
     formData = new FormData();
-    formData.width = 100;
     formData.right = new FormAttachment(100,0);
     formData.bottom = new FormAttachment(100,0);
-    btnCancel.setLayoutData(formData);
+    cButtons.setLayoutData(formData);
     
-    formData = new FormData();
-    formData.width = 100;
-    formData.right = new FormAttachment(btnCancel);
-    formData.bottom = new FormAttachment(100,0);
-    btnOk.setLayoutData(formData);
+    sash.setWeights(new int[] { 25, 75 });
     
-    updateWindow.setSize(400,400);
+    updateWindow.setSize(600,450);
     //updateWindow.open();
   }
   
@@ -282,23 +338,62 @@ UpdateWindow
     checkRestartNeeded();
     TableItem[] items = table.getSelection();
     if(items.length == 0) return;
-    Update update = (Update) items[0].getData();        
-    String[] descriptions = update.getDescription();
-    stDescription.setText("");
-    for(int i = 0 ; i < descriptions.length ; i++) {
-      stDescription.append(descriptions[i] + "\n");
+    Update update = (Update) items[0].getData();
+    
+    String desciptionURL = update.getDesciptionURL();
+    if (desciptionURL != null && browser != null) {
+    	browser.setUrl(desciptionURL);
+    	browser.setVisible(true);
+    	link_area.getComponent().setVisible(false);
+    } else {
+    	if (browser != null) {
+    		browser.setVisible(false);
+    	}
+    	link_area.getComponent().setVisible(true);
+    
+      String[] descriptions = update.getDescription();
+      
+      link_area.reset();
+      
+      link_area.setRelativeURLBase( update.getRelativeURLBase());
+      
+      for(int i = 0 ; i < descriptions.length ; i++) {
+  
+      	link_area.addLine( descriptions[i] );
+      }
     }
   }
   
+  public Shell
+  getShell()
+  {
+	  return( updateWindow );
+  }
+  
   public void dispose() {
-    updateWindow.dispose();
-    MainWindow.getWindow().setUpdateNeeded(null);
+  	Utils.execSWTThread(new AERunnable() {
+			public void runSupport() {
+		    updateWindow.dispose();
+      	UIFunctionsSWT functionsSWT = UIFunctionsManagerSWT.getUIFunctionsSWT();
+      	if (functionsSWT != null) {
+      		MainStatusBar mainStatusBar = functionsSWT.getMainStatusBar();
+      		if (mainStatusBar != null) {
+      			mainStatusBar.setUpdateNeeded(null);
+      		}
+      	}
+			}
+		});
   }
   
   public void addUpdate(final Update update) {
     if(display == null || display.isDisposed())
       return;
   
+    if ( update.isMandatory()){
+    	
+    	hasMandatoryUpdates = true;
+    }
+    
     display.asyncExec(new AERunnable() {
       public void runSupport() {
         if(table == null || table.isDisposed())
@@ -306,8 +401,8 @@ UpdateWindow
         
         final TableItem item = new TableItem(table,SWT.NULL);
         item.setData(update);
-        item.setText(COL_NAME,update.getName());  
-        item.setText(COL_VERSION,update.getNewVersion());
+        item.setText(COL_NAME,update.getName()==null?"Unknown":update.getName());  
+        item.setText(COL_VERSION,update.getNewVersion()==null?"Unknown":update.getNewVersion());
         ResourceDownloader[] rds = update.getDownloaders();
         long totalLength = 0;
         for(int i = 0 ; i < rds.length ; i++) {
@@ -339,7 +434,13 @@ UpdateWindow
         	
         }else{
         	
-          MainWindow.getWindow().setUpdateNeeded(UpdateWindow.this);
+        	UIFunctionsSWT functionsSWT = UIFunctionsManagerSWT.getUIFunctionsSWT();
+        	if (functionsSWT != null) {
+        		MainStatusBar mainStatusBar = functionsSWT.getMainStatusBar();
+        		if (mainStatusBar != null) {
+        			mainStatusBar.setUpdateNeeded(UpdateWindow.this);
+        		}
+        	}
         }
       }
     });
@@ -362,8 +463,11 @@ UpdateWindow
   }
   
   public void show() {
-    if(updateWindow == null || updateWindow.isDisposed())
+    if(updateWindow == null || updateWindow.isDisposed()) {
       return;
+    }
+    
+    Utils.centreWindow( updateWindow );
     updateWindow.open();
     updateWindow.forceActive();       
   }
@@ -392,9 +496,9 @@ UpdateWindow
       }
     }
     if(restartRequired) {
-        status.setText(MessageText.getString("swt.update.window.status.restartNeeded"));
+        status.setText(MessageText.getString("UpdateWindow.status.restartNeeded"));
     }else if(restartMaybeRequired) {
-        status.setText(MessageText.getString("swt.update.window.status.restartMaybeNeeded"));
+        status.setText(MessageText.getString("UpdateWindow.status.restartMaybeNeeded"));
     }else{
       status.setText("");
     }
@@ -402,9 +506,15 @@ UpdateWindow
   
   private void update() {
     btnOk.setEnabled(false);    
-    Messages.setLanguageText(btnCancel,"swt.update.window.cancel");
+    Messages.setLanguageText(btnCancel,"UpdateWindow.cancel");
     table.setEnabled(false);
-    stDescription.setText("");
+    
+    link_area.reset();
+    if (browser != null) {
+    	browser.setVisible(false);
+    }
+  	link_area.getComponent().setVisible(true);
+    
     TableItem[] items = table.getItems();
     
     totalDownloadSize = 0;   
@@ -420,7 +530,7 @@ UpdateWindow
         try {
           totalDownloadSize += rds[j].getSize();
         } catch (Exception e) {
-          stDescription.append(MessageText.getString("swt.update.window.no_size") + rds[j].getName() +"\n");
+          link_area.addLine(MessageText.getString("UpdateWindow.no_size") + rds[j].getName());
         }        
       }
     }
@@ -445,31 +555,40 @@ UpdateWindow
     
     display.asyncExec(new AERunnable(){
       public void runSupport() {
+    	  
+		Boolean b = (Boolean)check_instance.getProperty( UpdateCheckInstance.PT_CLOSE_OR_RESTART_ALREADY_IN_PROGRESS );
+			
+		if ( b != null && b ){
+			
+			finishUpdate(false, true);
+			
+			return;
+		}
+		
       	checkRestartNeeded();	// gotta recheck coz a maybe might have got to yes
         progress.setSelection(100);
-        status.setText(MessageText.getString("swt.update.window.status.done"));
+        status.setText(MessageText.getString("UpdateWindow.status.done"));
         btnOk.removeListener(SWT.Selection,lOk);
         btnOk.setEnabled(true);
         btnOk.addListener(SWT.Selection,new Listener() {
           public void handleEvent(Event e) {
-            finishUpdate(true);
+            finishUpdate(true, false);
           }
         });
         if(restartRequired) {
-          ((FormData)btnOk.getLayoutData()).width = 150;
-          ((FormData)btnCancel.getLayoutData()).width = 150;
-          updateWindow.layout();
-          Messages.setLanguageText(btnOk,"swt.update.window.restart");
+          Messages.setLanguageText(btnOk,"UpdateWindow.restart");
           btnCancel.removeListener(SWT.Selection,lCancel);
-          Messages.setLanguageText(btnCancel,"swt.update.window.restartLater");
+          Messages.setLanguageText(btnCancel,"UpdateWindow.restartLater");
           btnCancel.addListener(SWT.Selection, new Listener() {
             public void handleEvent(Event e) {
-              finishUpdate(false);
+              finishUpdate(false,false);
             }
           });
+          updateWindow.layout();
         } else {
-          Messages.setLanguageText(btnOk,"swt.update.window.close");
+          Messages.setLanguageText(btnOk,"UpdateWindow.close");
           btnCancel.setEnabled(false);
+          updateWindow.layout();
         }
       }
     });
@@ -480,6 +599,13 @@ UpdateWindow
     setProgressSelection(percentage);   
   }
   
+  public void
+  reportAmountComplete(
+	 ResourceDownloader	downloader,
+	 long				amount )
+  {  
+  }
+  
   private void setProgressSelection(final int percent) {
     if(display == null || display.isDisposed())
       return;
@@ -505,7 +631,7 @@ UpdateWindow
   public void failed(ResourceDownloader downloader,
       ResourceDownloaderException e) {
     downloader.removeListener(this);
-    setStatusText(MessageText.getString("swt.update.window.status.failed"));
+    setStatusText(MessageText.getString("UpdateWindow.status.failed"));
     
     String	msg = downloader.getName() + " : " + e;
     
@@ -540,29 +666,43 @@ UpdateWindow
     
     display.asyncExec(new AERunnable(){
       public void runSupport() {
-        if(stDescription != null && !stDescription.isDisposed())
-          stDescription.append(text + "\n");
+    	  link_area.addLine( text );
       }
     });  
   }
   
   
-  private void finishUpdate(boolean restartNow) {
+  private void finishUpdate(boolean restartNow, boolean just_close ) {
     //When completing, remove the link in mainWindow :
-    MainWindow.getWindow().setUpdateNeeded(null);
+  	UIFunctionsSWT functionsSWT = UIFunctionsManagerSWT.getUIFunctionsSWT();
+  	if (functionsSWT != null) {
+  		MainStatusBar mainStatusBar = functionsSWT.getMainStatusBar();
+  		if (mainStatusBar != null) {
+  			mainStatusBar.setUpdateNeeded(null);
+  		}
+  	}
     
-    //If restart is required, then restart
-    if( restartRequired && restartNow) {
-    	// this HAS to be done this way around else the restart inherits
-    	// the 6880 port listen. However, this is a general problem....
-    	
-      if ( !MainWindow.getWindow().dispose(true,false)){
-           	
-      	updateWindow.dispose(); 
-      }
-    }else{
-    	
-      updateWindow.dispose();      
+  	boolean bDisposeUpdateWindow = true;
+
+  	if ( !just_close ){
+	    //If restart is required, then restart
+	    if( restartRequired && restartNow) {
+	    	// this HAS to be done this way around else the restart inherits
+	    	// the 6880 port listen. However, this is a general problem....
+	    	UIFunctionsSWT uiFunctions = UIFunctionsManagerSWT.getUIFunctionsSWT();
+	    	if (uiFunctions != null && uiFunctions.dispose(true, false)) {
+	   			bDisposeUpdateWindow = false;
+				}
+	    }else if ( hasMandatoryUpdates && !restartRequired ){
+	    	
+	    		// run a further update check as we can immediately install non-mandatory updates now
+	    	
+	    	update_monitor.requestRecheck();
+	    }
+  	}
+
+    if (bDisposeUpdateWindow) {
+      updateWindow.dispose();
     }
   }
   
