@@ -25,10 +25,13 @@ package org.gudy.azureus2.ui.swt.views;
 import java.io.File;
 import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.List;
 
 import org.eclipse.swt.SWT;
+import org.eclipse.swt.dnd.*;
 import org.eclipse.swt.graphics.Point;
 import org.eclipse.swt.widgets.*;
+
 import org.gudy.azureus2.core3.config.COConfigurationManager;
 import org.gudy.azureus2.core3.config.ParameterListener;
 import org.gudy.azureus2.core3.disk.DiskManagerFileInfo;
@@ -36,12 +39,15 @@ import org.gudy.azureus2.core3.download.DownloadManager;
 import org.gudy.azureus2.core3.download.DownloadManagerState;
 import org.gudy.azureus2.core3.download.DownloadManagerStateAttributeListener;
 import org.gudy.azureus2.core3.internat.MessageText;
+import org.gudy.azureus2.core3.logging.LogEvent;
+import org.gudy.azureus2.core3.logging.LogIDs;
+import org.gudy.azureus2.core3.logging.Logger;
+import org.gudy.azureus2.core3.torrent.TOTorrent;
+import org.gudy.azureus2.core3.util.AERunnable;
+import org.gudy.azureus2.core3.util.Debug;
 import org.gudy.azureus2.core3.util.FileUtil;
 import org.gudy.azureus2.plugins.ui.tables.TableManager;
-import org.gudy.azureus2.ui.swt.MessageBoxWindow;
-import org.gudy.azureus2.ui.swt.Messages;
-import org.gudy.azureus2.ui.swt.SimpleTextEntryWindow;
-import org.gudy.azureus2.ui.swt.Utils;
+import org.gudy.azureus2.ui.swt.*;
 import org.gudy.azureus2.ui.swt.views.file.FileInfoView;
 import org.gudy.azureus2.ui.swt.views.table.TableViewSWT;
 import org.gudy.azureus2.ui.swt.views.table.TableViewSWTMenuFillListener;
@@ -66,6 +72,7 @@ public class FilesView
 	TableLifeCycleListener
 {
 	boolean refreshing = false;
+  private DragSource dragSource = null;
 
   private static final TableColumnCore[] basicItems = {
     new NameItem(),
@@ -103,15 +110,17 @@ public class FilesView
   
   private MenuItem path_item;
 
-  private TableViewSWT tv;
+  private TableViewSWT<DiskManagerFileInfo> tv;
   
 
   /**
    * Initialize 
    */
 	public FilesView() {
-		tv = new TableViewSWTImpl(TableManager.TABLE_TORRENT_FILES, "FilesView",
-				basicItems, "firstpiece", SWT.MULTI | SWT.FULL_SELECTION | SWT.VIRTUAL);
+		tv = new TableViewSWTImpl<DiskManagerFileInfo>(
+				org.gudy.azureus2.plugins.disk.DiskManagerFileInfo.class,
+				TableManager.TABLE_TORRENT_FILES, "FilesView", basicItems,
+				"firstpiece", SWT.MULTI | SWT.FULL_SELECTION | SWT.VIRTUAL);
 		setTableView(tv);
 		tv.setRowDefaultIconSize(new Point(16, 16));
 		tv.setEnableTabViews(true);
@@ -159,7 +168,7 @@ public class FilesView
 	}
 
 	// @see com.aelitis.azureus.ui.common.table.TableSelectionListener#defaultSelected(com.aelitis.azureus.ui.common.table.TableRowCore[])
-	public void defaultSelected(TableRowCore[] rows) {
+	public void defaultSelected(TableRowCore[] rows, int stateMask) {
 		DiskManagerFileInfo fileInfo = (DiskManagerFileInfo) tv.getFirstSelectedDataSource();
 		if (fileInfo != null
 				&& fileInfo.getAccessMode() == DiskManagerFileInfo.READ)
@@ -167,10 +176,10 @@ public class FilesView
 	}
 
 	// @see org.gudy.azureus2.ui.swt.views.TableViewSWTMenuFillListener#fillMenu(org.eclipse.swt.widgets.Menu)
-	public void fillMenu(final Menu menu) {
+	public void fillMenu(String sColumnName, final Menu menu) {
 		Shell shell = menu.getShell();
-		Object[] infos = tv.getSelectedDataSources();
-		boolean hasSelection = (infos.length > 0);
+		Object[] data_sources = tv.getSelectedDataSources().toArray();
+		boolean hasSelection = (data_sources.length > 0);
 
     final MenuItem itemOpen = new MenuItem(menu, SWT.PUSH);
     Messages.setLanguageText(itemOpen, "FilesView.menu.open");
@@ -184,7 +193,7 @@ public class FilesView
 	Messages.setLanguageText(itemExplore, "MyTorrentsView.menu." + (use_open_containing_folder ? "open_parent_folder" : "explore"));
 	itemExplore.addListener(SWT.Selection, new Listener() {
 		public void handleEvent(Event event) {
-		    Object[] dataSources = tv.getSelectedDataSources();
+		    Object[] dataSources = tv.getSelectedDataSources().toArray();
 		    for (int i = dataSources.length - 1; i >= 0; i--) {
 		    	DiskManagerFileInfo info = (DiskManagerFileInfo)dataSources[i];
 		    	if (info != null) {
@@ -253,69 +262,71 @@ public class FilesView
 	boolean	all_priority		= true;
 	boolean	all_not_priority	= true;
 		
-	DiskManagerFileInfo[] dmi_array = new DiskManagerFileInfo[infos.length];
-	System.arraycopy(infos, 0, dmi_array, 0, infos.length);
+	DiskManagerFileInfo[] dmi_array = new DiskManagerFileInfo[data_sources.length];
+	
+	System.arraycopy(data_sources, 0, dmi_array, 0, data_sources.length);
+	
 	int[] storage_types = manager.getStorageType(dmi_array);
 		
-		for (int i = 0; i < infos.length; i++) {
-			
-			DiskManagerFileInfo file_info = (DiskManagerFileInfo) infos[i];
+	for (int i = 0; i < dmi_array.length; i++) {
 
-			if (open && file_info.getAccessMode() != DiskManagerFileInfo.READ) {
+		DiskManagerFileInfo file_info = dmi_array[i];
 
-				open = false;
-			}
+		if (open && file_info.getAccessMode() != DiskManagerFileInfo.READ) {
 
-			if (all_compact && storage_types[i] != DiskManagerFileInfo.ST_COMPACT) {
-				all_compact = false;
-			}
+			open = false;
+		}
 
-			if (all_skipped || all_priority || all_not_priority) {
-				if ( file_info.isSkipped()){
-					all_priority		= false;
-					all_not_priority	= false;
-				}else{
-					all_skipped = false;
-					
-					// Only do this check if we need to.
-					if (all_not_priority || all_priority) {
-						if (file_info.isPriority()){
-							all_not_priority = false;
-						}else{
-							all_priority = false;
-						}
+		if (all_compact && storage_types[i] != DiskManagerFileInfo.ST_COMPACT) {
+			all_compact = false;
+		}
+
+		if (all_skipped || all_priority || all_not_priority) {
+			if ( file_info.isSkipped()){
+				all_priority		= false;
+				all_not_priority	= false;
+			}else{
+				all_skipped = false;
+
+				// Only do this check if we need to.
+				if (all_not_priority || all_priority) {
+					if (file_info.isPriority()){
+						all_not_priority = false;
+					}else{
+						all_priority = false;
 					}
 				}
 			}
 		}
+	}
 
-		// we can only open files if they are read-only
+	// we can only open files if they are read-only
 
-		itemOpen.setEnabled(open);
+	itemOpen.setEnabled(open);
 
-		// can't rename files for non-persistent downloads (e.g. shares) as these
-		// are managed "externally"
+	// can't rename files for non-persistent downloads (e.g. shares) as these
+	// are managed "externally"
 
-		itemRenameOrRetarget.setEnabled(manager.isPersistent());
-		itemRename.setEnabled(manager.isPersistent());
-		itemRetarget.setEnabled(manager.isPersistent());
+	itemRenameOrRetarget.setEnabled(manager.isPersistent());
+	itemRename.setEnabled(manager.isPersistent());
+	itemRetarget.setEnabled(manager.isPersistent());
 
-		itemSkipped.setEnabled( !all_skipped );
-	
-		itemHigh.setEnabled( !all_priority );
-		
-		itemLow.setEnabled( !all_not_priority );
-		
-		itemDelete.setEnabled( !all_compact );
+	itemSkipped.setEnabled( !all_skipped );
 
-    itemOpen.addListener(SWT.Selection, new TableSelectedRowsListener(tv) {
-      public void run(TableRowCore row) {
-        DiskManagerFileInfo fileInfo = (DiskManagerFileInfo)row.getDataSource(true);
-        if (fileInfo.getAccessMode() == DiskManagerFileInfo.READ) {
-        	Utils.launch(fileInfo.getFile(true).toString());
-        }
-      }
-    });
+	itemHigh.setEnabled( !all_priority );
+
+	itemLow.setEnabled( !all_not_priority );
+
+	itemDelete.setEnabled( !all_compact );
+
+	itemOpen.addListener(SWT.Selection, new TableSelectedRowsListener(tv) {
+		public void run(TableRowCore row) {
+			DiskManagerFileInfo fileInfo = (DiskManagerFileInfo)row.getDataSource(true);
+			if (fileInfo.getAccessMode() == DiskManagerFileInfo.READ) {
+				Utils.launch(fileInfo.getFile(true).toString());
+			}
+		}
+	});
     
     Listener rename_listener = new Listener() {
     	public void handleEvent(Event event) {
@@ -343,9 +354,8 @@ public class FilesView
 	}
 
 	private String askForRenameFilename(DiskManagerFileInfo fileInfo) {
-		SimpleTextEntryWindow dialog = new SimpleTextEntryWindow(Display.getDefault());
-		dialog.setTitle("FilesView.rename.filename.title");
-		dialog.setMessage("FilesView.rename.filename.text");
+		SimpleTextEntryWindow dialog = new SimpleTextEntryWindow(
+				"FilesView.rename.filename.title", "FilesView.rename.filename.text");
 		dialog.setPreenteredText(fileInfo.getFile(true).getName(), false); // false -> it's not "suggested", it's a previous value
 		dialog.allowEmptyInput(false);
 		dialog.prompt();
@@ -524,15 +534,19 @@ public class FilesView
 	int linearCount = 0;
 
 
+	if(infos.length > 1)
+	{
+		
+	}
 	// This should hopefully reduce the number of "exists" checks.
 	File save_location = manager.getAbsoluteSaveLocation();
-	boolean root_exists = save_location.isDirectory();
+	boolean root_exists = save_location.isDirectory() || (infos.length <= 1 && save_location.exists());
 	
 	boolean type_has_been_changed = false;
 	boolean requires_pausing = false;
 
 	for (int i=0; i<infos.length; i++) {
-		int existing_storage_type = existing_storage_types[infos[i].getIndex()];  
+		int existing_storage_type = existing_storage_types[i];  
 		int new_storage_type = DiskManagerFileInfo.ST_LINEAR;
 		if (skipped) {
 
@@ -619,29 +633,29 @@ public class FilesView
 	    if (files != null && (this.force_refresh || !doAllExist(files))) {
 	    	this.force_refresh = false;
 
-	    	Object[] datasources = tv.getDataSources();
-	    	if(datasources.length == files.length)
+	    	List<DiskManagerFileInfo> datasources = tv.getDataSources();
+	    	if(datasources.size() == files.length)
 	    	{
 	    		// check if we actually have to replace anything
-	    		ArrayList toAdd = new ArrayList(Arrays.asList(files));
-		    	ArrayList toRemove = new ArrayList();
-		    	for(int i = 0;i < datasources.length;i++)
+	    		ArrayList<DiskManagerFileInfo> toAdd = new ArrayList<DiskManagerFileInfo>(Arrays.asList(files));
+		    	ArrayList<DiskManagerFileInfo> toRemove = new ArrayList<DiskManagerFileInfo>();
+		    	for(int i = 0;i < datasources.size();i++)
 		    	{
-		    		DiskManagerFileInfo info = (DiskManagerFileInfo)datasources[i];
+		    		DiskManagerFileInfo info = datasources.get(i);
 		    		
 		    		if(files[info.getIndex()] == info)
 		    			toAdd.set(info.getIndex(), null);
 		    		else
 		    			toRemove.add(info);
 		    	}
-		    	tv.removeDataSources(toRemove.toArray());
-		    	tv.addDataSources(toAdd.toArray());
+		    	tv.removeDataSources(toRemove.toArray(new DiskManagerFileInfo[toRemove.size()]));
+		    	tv.addDataSources(toAdd.toArray(new DiskManagerFileInfo[toAdd.size()]));
 		    	((TableViewSWTImpl)tv).tableInvalidate();
 	    	} else
 	    	{
 		    	tv.removeAllTableRows();
 	    		
-			    Object filesCopy[] = new Object[files.length]; 
+		    	DiskManagerFileInfo filesCopy[] = new DiskManagerFileInfo[files.length]; 
 			    System.arraycopy(files, 0, filesCopy, 0, files.length);
 
 			    tv.addDataSources(filesCopy);
@@ -722,9 +736,25 @@ public class FilesView
 	  this.force_refresh = true;
   }
   
-  public void tableViewInitialized() {}
+  public void tableViewInitialized() {
+    createDragDrop();
+  }
+  
   public void tableViewDestroyed() {
-	  if (manager != null) {
+  	Utils.execSWTThread(new AERunnable() {
+			public void runSupport() {
+				try {
+					Utils.disposeSWTObjects(new Object[] {
+						dragSource,
+					});
+					dragSource = null;
+				} catch (Exception e) {
+					Debug.out(e);
+				}
+			}
+		});
+
+  	if (manager != null) {
 		  manager.getDownloadState().removeListener(this, DownloadManagerState.AT_FILE_LINKS, DownloadManagerStateAttributeListener.WRITTEN);
 	  }
   }
@@ -737,4 +767,56 @@ public class FilesView
 	// @see com.aelitis.azureus.ui.common.table.TableSelectionListener#mouseExit(com.aelitis.azureus.ui.common.table.TableRowCore)
 	public void mouseExit(TableRowCore row) {
 	}
+
+	private void createDragDrop() {
+		try {
+
+			Transfer[] types = new Transfer[] { TextTransfer.getInstance() };
+
+			if (dragSource != null && !dragSource.isDisposed()) {
+				dragSource.dispose();
+			}
+
+			dragSource = tv.createDragSource(DND.DROP_MOVE | DND.DROP_COPY);
+			if (dragSource != null) {
+				dragSource.setTransfer(types);
+				dragSource.addDragListener(new DragSourceAdapter() {
+					private String eventData;
+
+					public void dragStart(DragSourceEvent event) {
+						TableRowCore[] rows = tv.getSelectedRows();
+						if (rows.length != 0 && manager != null
+								&& manager.getTorrent() != null) {
+							event.doit = true;
+						} else {
+							event.doit = false;
+							return;
+						}
+
+						// Build eventData here because on OSX, selection gets cleared
+						// by the time dragSetData occurs
+						Object[] selectedDownloads = tv.getSelectedDataSources().toArray();
+						eventData = "DiskManagerFileInfo\n";
+						TOTorrent torrent = manager.getTorrent();
+						for (int i = 0; i < selectedDownloads.length; i++) {
+							DiskManagerFileInfo fi = (DiskManagerFileInfo) selectedDownloads[i];
+							
+							try {
+								eventData += torrent.getHashWrapper().toBase32String() + ";"
+										+ fi.getIndex() + "\n";
+							} catch (Exception e) {
+							}
+						}
+					}
+
+					public void dragSetData(DragSourceEvent event) {
+						// System.out.println("DragSetData");
+						event.data = eventData;
+					}
+				});
+			}
+		} catch (Throwable t) {
+			Logger.log(new LogEvent(LogIDs.GUI, "failed to init drag-n-drop", t));
+		}
+	}
 }
