@@ -35,8 +35,16 @@ import org.eclipse.swt.widgets.*;
 import org.gudy.azureus2.core3.category.Category;
 import org.gudy.azureus2.core3.category.CategoryManager;
 import org.gudy.azureus2.core3.download.DownloadManager;
-import org.gudy.azureus2.core3.global.GlobalManager;
 import org.gudy.azureus2.core3.util.Debug;
+import org.gudy.azureus2.plugins.PluginInterface;
+import org.gudy.azureus2.plugins.download.Download;
+import org.gudy.azureus2.plugins.sharing.*;
+import org.gudy.azureus2.plugins.torrent.Torrent;
+import org.gudy.azureus2.plugins.torrent.TorrentAttribute;
+import org.gudy.azureus2.plugins.tracker.Tracker;
+import org.gudy.azureus2.plugins.tracker.TrackerTorrent;
+import org.gudy.azureus2.plugins.ui.tables.TableManager;
+import org.gudy.azureus2.pluginsimpl.local.PluginInitializer;
 import org.gudy.azureus2.pluginsimpl.local.torrent.TorrentManagerImpl;
 import org.gudy.azureus2.ui.swt.*;
 import org.gudy.azureus2.ui.swt.views.table.TableViewSWTMenuFillListener;
@@ -47,19 +55,12 @@ import org.gudy.azureus2.ui.swt.views.tableitems.myshares.NameItem;
 import org.gudy.azureus2.ui.swt.views.tableitems.myshares.TypeItem;
 
 import com.aelitis.azureus.core.AzureusCore;
+import com.aelitis.azureus.core.AzureusCoreRunningListener;
+import com.aelitis.azureus.core.AzureusCoreFactory;
 import com.aelitis.azureus.ui.UIFunctions;
 import com.aelitis.azureus.ui.UIFunctionsManager;
 import com.aelitis.azureus.ui.common.table.*;
 
-import org.gudy.azureus2.plugins.PluginInterface;
-import org.gudy.azureus2.plugins.download.Download;
-import org.gudy.azureus2.plugins.sharing.*;
-import org.gudy.azureus2.plugins.torrent.Torrent;
-import org.gudy.azureus2.plugins.torrent.TorrentAttribute;
-import org.gudy.azureus2.plugins.tracker.Tracker;
-import org.gudy.azureus2.plugins.tracker.TrackerTorrent;
-import org.gudy.azureus2.plugins.ui.tables.TableManager;
-
 /**
  * @author parg
  * @author TuxPaper
@@ -70,7 +71,7 @@ public class MySharesView
 extends TableViewTab
 implements ShareManagerListener,
 		TableLifeCycleListener, TableViewSWTMenuFillListener,
-		TableRefreshListener
+		TableRefreshListener, TableSelectionListener
 {
   private static final TableColumnCore[] basicItems = {
     new NameItem(),
@@ -81,27 +82,20 @@ implements ShareManagerListener,
 	protected static final TorrentAttribute	category_attribute = 
 		TorrentManagerImpl.getSingleton().getAttribute( TorrentAttribute.TA_CATEGORY );
 
-  	private AzureusCore		azureus_core;
-  	
-	private GlobalManager	global_manager;
-	
 	private Menu			menuCategory;
 
-	private TableViewSWTImpl tv;
-	
+	private TableViewSWTImpl<ShareResource> tv;
+
 	public 
-	MySharesView(
-		AzureusCore	_azureus_core )
+	MySharesView()
 	{	
-		tv = new TableViewSWTImpl(TableManager.TABLE_MYSHARES, "MySharesView",
-				basicItems, "name", SWT.MULTI | SWT.FULL_SELECTION | SWT.BORDER
-						| SWT.VIRTUAL);
+		tv = new TableViewSWTImpl<ShareResource>(ShareResource.class, TableManager.TABLE_MYSHARES,
+				"MySharesView", basicItems, "name", SWT.MULTI | SWT.FULL_SELECTION
+						| SWT.BORDER | SWT.VIRTUAL);
 		setTableView(tv);
-		azureus_core	= _azureus_core;
-		global_manager = azureus_core.getGlobalManager();
 
 		tv.addSelectionListener(new TableSelectionAdapter() {
-			public void defaultSelected(TableRowCore[] rows) {
+			public void defaultSelected(TableRowCore[] rows, int stateMask) {
 				MySharesView.this.defaultSelected(rows);
 			}
 		
@@ -110,15 +104,19 @@ implements ShareManagerListener,
 		tv.addLifeCycleListener(this);
 		tv.addMenuFillListener(this);
 		tv.addRefreshListener(this, false);
+		tv.addSelectionListener(this, false);
 	}
 	
 	private void defaultSelected(TableRowCore[] rows) {
-		ShareResource share = (ShareResource) tv.getFirstSelectedDataSource();
+		ShareResource share = tv.getFirstSelectedDataSource();
 		if (share == null) {
 			return;
 		}
+		
+		// if a row was selected that means it was added, which 
+		// required a core, so we assume there's a core here
 
-		List dms = global_manager.getDownloadManagers();
+		List dms = AzureusCoreFactory.getSingleton().getGlobalManager().getDownloadManagers();
 
 		for (int i = 0; i < dms.size(); i++) {
 			DownloadManager dm = (DownloadManager) dms.get(i);
@@ -139,7 +137,7 @@ implements ShareManagerListener,
 
 					UIFunctions uiFunctions = UIFunctionsManager.getUIFunctions();
 					if (uiFunctions != null) {
-						uiFunctions.openManagerView(dm);
+						uiFunctions.openView(UIFunctions.VIEW_DM_DETAILS, dm);
 					}
 
 					break;
@@ -151,22 +149,27 @@ implements ShareManagerListener,
 	}
 	
 	public void tableViewInitialized() {
-		createRows();
+		AzureusCoreFactory.addCoreRunningListener(new AzureusCoreRunningListener() {
+			public void azureusCoreRunning(AzureusCore core) {
+				createRows(core);
+			}
+		});
 	}
 	
 	public void tableViewDestroyed() {
 		try {
-			azureus_core.getPluginManager().getDefaultPluginInterface().getShareManager().removeListener(
+			PluginInitializer.getDefaultInterface().getShareManager().removeListener(
 					this);
 		} catch (ShareException e) {
 			Debug.printStackTrace(e);
+		} catch (Throwable ignore) {
 		}
 	}
 
-  private void createRows() {
+  private void createRows(AzureusCore core) {
 		try{
 
-			ShareManager	sm = azureus_core.getPluginManager().getDefaultPluginInterface().getShareManager();
+			ShareManager	sm = core.getPluginManager().getDefaultPluginInterface().getShareManager();
 			
 			ShareResource[]	shares = sm.getShares();
 			
@@ -185,7 +188,7 @@ implements ShareManagerListener,
 
   public void 
   fillMenu(
-  	final Menu menu) 
+  	String sColumnName, final Menu menu) 
   {
   	Shell shell = menu.getShell();
 		/*
@@ -213,7 +216,7 @@ implements ShareManagerListener,
 	   Utils.setMenuItemImage(itemRemove, "delete");
 
 
-	   Object[] shares = tv.getSelectedDataSources();
+	   Object[] shares = tv.getSelectedDataSources().toArray();
 
 	   itemRemove.setEnabled(shares.length > 0);
 
@@ -341,11 +344,15 @@ implements ShareManagerListener,
   {
     start = stop = remove = false;
     
+    if (!AzureusCoreFactory.isCoreRunning()) {
+    	return;
+    }
+    
 	List	items = getSelectedItems();
 	
     if (items.size() > 0) {
     
-  	  PluginInterface pi = azureus_core.getPluginManager().getDefaultPluginInterface();
+  	  PluginInterface pi = PluginInitializer.getDefaultInterface();
 
   	  org.gudy.azureus2.plugins.download.DownloadManager	dm 		= pi.getDownloadManager();
 
@@ -392,7 +399,7 @@ implements ShareManagerListener,
       return stop;
     if(itemKey.equals("remove"))
       return remove;
-    return false;
+    return super.isEnabled(itemKey);
   }
   
 
@@ -402,15 +409,18 @@ implements ShareManagerListener,
       return;
     }else if ( itemKey.equals( "stop" )){
     	stopSelectedShares();
+    	return;
     }else if ( itemKey.equals( "start" )){
     	startSelectedShares();
+    	return;
     }
+    super.itemActivated(itemKey);
   }
   
   private List
   getSelectedItems()
   {
-	  Object[] shares = tv.getSelectedDataSources();
+	  Object[] shares = tv.getSelectedDataSources().toArray();
 	    
 	  List	items = new ArrayList();
 	  
@@ -497,8 +507,11 @@ implements ShareManagerListener,
 	boolean	do_stop )
   {
 	  List	items = getSelectedItems();
+	  if (items.size() == 0) {
+	  	return;
+	  }
 	
-	  PluginInterface pi = azureus_core.getPluginManager().getDefaultPluginInterface();
+	  PluginInterface pi = PluginInitializer.getDefaultInterface();
 	    
 	  org.gudy.azureus2.plugins.download.DownloadManager	dm 		= pi.getDownloadManager();
 	    
@@ -578,7 +591,7 @@ implements ShareManagerListener,
   removeSelectedShares()
   {
 	stopSelectedShares();
-    Object[] shares = tv.getSelectedDataSources();
+    Object[] shares = tv.getSelectedDataSources().toArray();
     for (int i = 0; i < shares.length; i++) {
     	try{
     		((ShareResource)shares[i]).delete();
@@ -587,13 +600,50 @@ implements ShareManagerListener,
     		
     	  Alerts.showErrorMessageBoxUsingResourceString(
 						new Object[] { shares[i] },
-    	  		"globalmanager.download.remove.veto", e );
+    	  		"globalmanager.download.remove.veto", e, 0 );
     	}
     }
   }
 
 	public void addThisColumnSubMenu(String columnName, Menu menuThisColumn) {
-		// TODO Auto-generated method stub
-		
+	}
+
+	// @see com.aelitis.azureus.ui.common.table.TableSelectionListener#defaultSelected(com.aelitis.azureus.ui.common.table.TableRowCore[], int)
+	public void defaultSelected(TableRowCore[] rows, int stateMask) {
+	}
+
+	// @see com.aelitis.azureus.ui.common.table.TableSelectionListener#deselected(com.aelitis.azureus.ui.common.table.TableRowCore[])
+	public void deselected(TableRowCore[] rows) {
+		computePossibleActions();
+  	UIFunctions uiFunctions = UIFunctionsManager.getUIFunctions();
+  	if (uiFunctions != null) {
+  		uiFunctions.refreshIconBar();
+  	}
+	}
+
+	// @see com.aelitis.azureus.ui.common.table.TableSelectionListener#focusChanged(com.aelitis.azureus.ui.common.table.TableRowCore)
+	public void focusChanged(TableRowCore focus) {
+		computePossibleActions();
+  	UIFunctions uiFunctions = UIFunctionsManager.getUIFunctions();
+  	if (uiFunctions != null) {
+  		uiFunctions.refreshIconBar();
+  	}
+	}
+
+	// @see com.aelitis.azureus.ui.common.table.TableSelectionListener#mouseEnter(com.aelitis.azureus.ui.common.table.TableRowCore)
+	public void mouseEnter(TableRowCore row) {
+	}
+
+	// @see com.aelitis.azureus.ui.common.table.TableSelectionListener#mouseExit(com.aelitis.azureus.ui.common.table.TableRowCore)
+	public void mouseExit(TableRowCore row) {
+	}
+
+	// @see com.aelitis.azureus.ui.common.table.TableSelectionListener#selected(com.aelitis.azureus.ui.common.table.TableRowCore[])
+	public void selected(TableRowCore[] row) {
+		computePossibleActions();
+  	UIFunctions uiFunctions = UIFunctionsManager.getUIFunctions();
+  	if (uiFunctions != null) {
+  		uiFunctions.refreshIconBar();
+  	}
 	}
 }
