@@ -44,11 +44,13 @@ import org.gudy.azureus2.core3.tracker.host.TRHostListener;
 import org.gudy.azureus2.core3.tracker.host.TRHostTorrent;
 import org.gudy.azureus2.core3.tracker.host.TRHostTorrentRemovalVetoException;
 import org.gudy.azureus2.core3.util.AERunnable;
+import org.gudy.azureus2.core3.util.AsyncController;
 import org.gudy.azureus2.core3.util.Debug;
 import org.gudy.azureus2.core3.util.TorrentUtils;
 import org.gudy.azureus2.ui.swt.*;
 import org.gudy.azureus2.ui.swt.mainwindow.Colors;
 import org.gudy.azureus2.ui.swt.mainwindow.SWTThread;
+import org.gudy.azureus2.ui.swt.shells.CoreWaiterSWT;
 import org.gudy.azureus2.ui.swt.views.table.TableRowSWT;
 import org.gudy.azureus2.ui.swt.views.table.TableViewSWT;
 import org.gudy.azureus2.ui.swt.views.table.TableViewSWTMenuFillListener;
@@ -57,11 +59,14 @@ import org.gudy.azureus2.ui.swt.views.table.impl.TableViewTab;
 import org.gudy.azureus2.ui.swt.views.tableitems.mytracker.*;
 
 import com.aelitis.azureus.core.AzureusCore;
+import com.aelitis.azureus.core.AzureusCoreRunningListener;
+import com.aelitis.azureus.core.AzureusCoreFactory;
 import com.aelitis.azureus.ui.UIFunctions;
 import com.aelitis.azureus.ui.UIFunctionsManager;
 import com.aelitis.azureus.ui.common.table.*;
 
 import org.gudy.azureus2.plugins.torrent.TorrentAttribute;
+import org.gudy.azureus2.plugins.tracker.TrackerTorrent;
 import org.gudy.azureus2.plugins.ui.tables.TableManager;
 
 import org.gudy.azureus2.pluginsimpl.local.torrent.TorrentManagerImpl;
@@ -79,43 +84,44 @@ public class MyTrackerView
 	implements TRHostListener, CategoryManagerListener, TableLifeCycleListener,
 	TableSelectionListener, TableViewSWTMenuFillListener, TableRefreshListener
 {
-  private static final TableColumnCore[] basicItems = {
-    new NameItem(),
-    new TrackerItem(),
-    new StatusItem(),
-    new CategoryItem(),
-    new PassiveItem(),
-    new SeedCountItem(),
-    new PeerCountItem(),
-    new BadNATCountItem(),
-	new AnnounceCountItem(),
-    new ScrapeCountItem(),
-    new CompletedCountItem(),
-    new UploadedItem(),
-    new DownloadedItem(),
-    new LeftItem(),
-    new TotalBytesInItem(),
-    new AverageBytesInItem(),
-    new TotalBytesOutItem(),
-    new AverageBytesOutItem(),
-    new DateAddedItem(),
-  };
+  private static TableColumnCore[] basicItems = null;
 
 	protected static final TorrentAttribute	category_attribute = 
 		TorrentManagerImpl.getSingleton().getAttribute( TorrentAttribute.TA_CATEGORY );
 
-	private AzureusCore	azureus_core;
-
 	private Menu			menuCategory;
 
-	private TableViewSWT tv;
+	private TableViewSWT<TRHostTorrent> tv;
 
-	public MyTrackerView(AzureusCore _azureus_core) {
-		tv = new TableViewSWTImpl(TableManager.TABLE_MYTRACKER, "MyTrackerView",
-				basicItems, "name", SWT.MULTI | SWT.FULL_SELECTION | SWT.BORDER
-						| SWT.VIRTUAL);
+	public MyTrackerView() {
+		if (basicItems == null) {
+			basicItems = new TableColumnCore[] {
+				new NameItem(),
+				new TrackerItem(),
+				new StatusItem(),
+				new CategoryItem(),
+				new PassiveItem(),
+				new SeedCountItem(),
+				new PeerCountItem(),
+				new BadNATCountItem(),
+				new AnnounceCountItem(),
+				new ScrapeCountItem(),
+				new CompletedCountItem(),
+				new UploadedItem(),
+				new DownloadedItem(),
+				new LeftItem(),
+				new TotalBytesInItem(),
+				new AverageBytesInItem(),
+				new TotalBytesOutItem(),
+				new AverageBytesOutItem(),
+				new DateAddedItem(),
+			};
+		}
+
+		tv = new TableViewSWTImpl<TRHostTorrent>(TrackerTorrent.class,
+				TableManager.TABLE_MYTRACKER, "MyTrackerView", basicItems, "name",
+				SWT.MULTI | SWT.FULL_SELECTION | SWT.BORDER | SWT.VIRTUAL);
 		setTableView(tv);
-		azureus_core = _azureus_core;
 		tv.addLifeCycleListener(this);
 		tv.addSelectionListener(this, false);
 		tv.addMenuFillListener(this);
@@ -124,30 +130,42 @@ public class MyTrackerView
 	
 	// @see com.aelitis.azureus.ui.common.table.TableLifeCycleListener#tableViewInitialized()
 	public void tableViewInitialized() {
-		azureus_core.getTrackerHost().addListener( this );
+		AzureusCoreFactory.addCoreRunningListener(new AzureusCoreRunningListener() {
+			public void azureusCoreRunning(AzureusCore core) {
+				core.getTrackerHost().addListener(MyTrackerView.this);
+			}
+		});
 	}
 	
 	// @see com.aelitis.azureus.ui.common.table.TableLifeCycleListener#tableViewDestroyed()
 	public void tableViewDestroyed() {
-  	azureus_core.getTrackerHost().removeListener( this );
+		try {
+			AzureusCoreFactory.getSingleton().getTrackerHost().removeListener( this );
+		} catch (Exception ignore) {
+		}
 	}
 
 	// @see com.aelitis.azureus.ui.common.table.TableSelectionListener#defaultSelected(com.aelitis.azureus.ui.common.table.TableRowCore[])
-	public void defaultSelected(TableRowCore[] rows) {
-		TRHostTorrent torrent = (TRHostTorrent) tv.getFirstSelectedDataSource();
+	public void defaultSelected(TableRowCore[] rows, int stateMask) {
+		final TRHostTorrent torrent = tv.getFirstSelectedDataSource();
 		if (torrent == null)
 			return;
-		DownloadManager dm = azureus_core.getGlobalManager().getDownloadManager(
-				torrent.getTorrent());
-		if (dm != null) {
-			UIFunctions uiFunctions = UIFunctionsManager.getUIFunctions();
-			if (uiFunctions != null) {
-				uiFunctions.openManagerView(dm);
+		CoreWaiterSWT.waitForCoreRunning(new AzureusCoreRunningListener() {
+		
+			public void azureusCoreRunning(AzureusCore core) {
+				DownloadManager dm = core.getGlobalManager().getDownloadManager(
+						torrent.getTorrent());
+				if (dm != null) {
+					UIFunctions uiFunctions = UIFunctionsManager.getUIFunctions();
+					if (uiFunctions != null) {
+						uiFunctions.openView(UIFunctions.VIEW_DM_DETAILS, dm);
+					}
+				}
 			}
-		}
+		});
 	}
     
-  public void fillMenu(final Menu menu) {	  
+  public void fillMenu(String sColumnName, final Menu menu) {	  
 	    menuCategory = new Menu(getComposite().getShell(), SWT.DROP_DOWN);
 	    final MenuItem itemCategory = new MenuItem(menu, SWT.CASCADE);
 	    Messages.setLanguageText(itemCategory, "MyTorrentsView.menu.setCategory"); //$NON-NLS-1$
@@ -170,7 +188,7 @@ public class MyTrackerView
 	   Messages.setLanguageText(itemRemove, "MyTorrentsView.menu.remove"); //$NON-NLS-1$
 	   Utils.setMenuItemImage(itemRemove, "delete");
 
-	   Object[] hostTorrents = tv.getSelectedDataSources();
+	   Object[] hostTorrents = tv.getSelectedDataSources().toArray();
 
 	   itemStart.setEnabled(false);
 	   itemStop.setEnabled(false);
@@ -264,7 +282,8 @@ public class MyTrackerView
 		URL					absolute_url,
 		String				header,
 		InputStream			is,
-		OutputStream		os )
+		OutputStream		os,
+		AsyncController		async )
 	
 		throws IOException
 	{
@@ -328,7 +347,7 @@ public class MyTrackerView
   
   private void computePossibleActions() {
     start = stop = remove = false;
-    Object[] hostTorrents = tv.getSelectedDataSources();
+    Object[] hostTorrents = tv.getSelectedDataSources().toArray();
     if (hostTorrents.length > 0) {
       remove = true;
       for (int i = 0; i < hostTorrents.length; i++) {
@@ -413,8 +432,8 @@ public class MyTrackerView
       	}catch( TRHostTorrentRemovalVetoException f ){
       		
       		Alerts.showErrorMessageBoxUsingResourceString(
-							new Object[] { torrent },
-      				"globalmanager.download.remove.veto", f );
+					new Object[] { torrent },
+      				"globalmanager.download.remove.veto", f, 0 );
       	}
       }
     });
@@ -514,14 +533,23 @@ public class MyTrackerView
   }
   
   private void assignSelectedToCategory(final Category category) {
+		CoreWaiterSWT.waitForCoreRunning(new AzureusCoreRunningListener() {
+			public void azureusCoreRunning(final AzureusCore core) {
+				assignSelectedToCategory(core, category);
+			}
+		});
+  }
+  
+  private void assignSelectedToCategory(final AzureusCore core,
+			final Category category) {
     tv.runForSelectedRows(new TableGroupRowRunner() {
       public void run(TableRowCore row) {
       	
 	    TRHostTorrent	tr_torrent = (TRHostTorrent)row.getDataSource(true);
 		
-		TOTorrent	torrent = tr_torrent.getTorrent();
+		final TOTorrent	torrent = tr_torrent.getTorrent();
 		
-		DownloadManager dm = azureus_core.getGlobalManager().getDownloadManager( torrent );
+		DownloadManager dm = core.getGlobalManager().getDownloadManager( torrent );
 
 		if ( dm != null ){
 			
@@ -561,14 +589,29 @@ public class MyTrackerView
 
 	// @see com.aelitis.azureus.ui.common.table.TableSelectionListener#deselected(com.aelitis.azureus.ui.common.table.TableRowCore[])
 	public void deselected(TableRowCore[] rows) {
+		computePossibleActions();
+  	UIFunctions uiFunctions = UIFunctionsManager.getUIFunctions();
+  	if (uiFunctions != null) {
+  		uiFunctions.refreshIconBar();
+  	}
 	}
 
 	// @see com.aelitis.azureus.ui.common.table.TableSelectionListener#focusChanged(com.aelitis.azureus.ui.common.table.TableRowCore)
 	public void focusChanged(TableRowCore focus) {
+		computePossibleActions();
+  	UIFunctions uiFunctions = UIFunctionsManager.getUIFunctions();
+  	if (uiFunctions != null) {
+  		uiFunctions.refreshIconBar();
+  	}
 	}
 
 	// @see com.aelitis.azureus.ui.common.table.TableSelectionListener#selected(com.aelitis.azureus.ui.common.table.TableRowCore[])
 	public void selected(TableRowCore[] rows) {
+		computePossibleActions();
+  	UIFunctions uiFunctions = UIFunctionsManager.getUIFunctions();
+  	if (uiFunctions != null) {
+  		uiFunctions.refreshIconBar();
+  	}
 	}
 
 	// @see com.aelitis.azureus.ui.common.table.TableSelectionListener#mouseEnter(com.aelitis.azureus.ui.common.table.TableRowCore)
