@@ -32,6 +32,7 @@ import org.eclipse.swt.SWT;
 import org.eclipse.swt.events.MouseAdapter;
 import org.eclipse.swt.events.MouseEvent;
 import org.eclipse.swt.graphics.Image;
+import org.eclipse.swt.layout.FillLayout;
 import org.eclipse.swt.layout.GridData;
 import org.eclipse.swt.layout.GridLayout;
 import org.eclipse.swt.widgets.*;
@@ -44,19 +45,18 @@ import org.gudy.azureus2.core3.logging.LogAlert;
 import org.gudy.azureus2.core3.logging.Logger;
 import org.gudy.azureus2.core3.util.AERunnable;
 import org.gudy.azureus2.core3.util.DisplayFormatters;
-import org.gudy.azureus2.ui.swt.ImageRepository;
 import org.gudy.azureus2.ui.swt.Messages;
 import org.gudy.azureus2.ui.swt.Utils;
 import org.gudy.azureus2.ui.swt.config.*;
 import org.gudy.azureus2.ui.swt.plugins.UISWTConfigSection;
 
 import com.aelitis.azureus.core.AzureusCore;
+import com.aelitis.azureus.core.AzureusCoreFactory;
+import com.aelitis.azureus.ui.swt.imageloader.ImageLoader;
 
 import org.gudy.azureus2.plugins.ui.config.ConfigSection;
 
 public class ConfigSectionIPFilter implements UISWTConfigSection {
-  AzureusCore	azureus_core;
-  
   IpFilter filter;
   Table table;
   boolean noChange;
@@ -105,10 +105,8 @@ public class ConfigSectionIPFilter implements UISWTConfigSection {
 	private IPFilterListener filterListener;
   
   public
-  ConfigSectionIPFilter(
-  	AzureusCore		_azureus_core )
+  ConfigSectionIPFilter()
   {
-  	azureus_core	= _azureus_core;
   	comparator = new FilterComparator();
   }
   
@@ -137,7 +135,7 @@ public class ConfigSectionIPFilter implements UISWTConfigSection {
 
   public void configSectionDelete() {
   	if (bIsCachingDescriptions) {
-	    IpFilterManager ipFilterManager = azureus_core.getIpFilterManager();
+	    IpFilterManager ipFilterManager = AzureusCoreFactory.getSingleton().getIpFilterManager();
 	  	ipFilterManager.clearDescriptionCache();
 	  	bIsCachingDescriptions = false;
   	}
@@ -145,17 +143,31 @@ public class ConfigSectionIPFilter implements UISWTConfigSection {
   	if (filter != null) {
   		filter.removeListener(filterListener);
   	}
+		ImageLoader imageLoader = ImageLoader.getInstance();
+		imageLoader.releaseImage("openFolderButton");
+		imageLoader.releaseImage("subitem");
   }
 
   public Composite configSectionCreate(final Composite parent) {
-    Image imgOpenFolder = ImageRepository.getImage("openFolderButton");
-    String sCurConfigID;
+
+  	if (!AzureusCoreFactory.isCoreRunning()) {
+      Composite cSection = new Composite(parent, SWT.NULL);
+    	cSection.setLayout(new FillLayout());
+    	Label lblNotAvail = new Label(cSection, SWT.WRAP);
+    	Messages.setLanguageText(lblNotAvail, "core.not.available");
+    	return cSection;
+    }
+
+		ImageLoader imageLoader = ImageLoader.getInstance();
+		Image imgOpenFolder = imageLoader.getImage("openFolderButton");			
+
+		String sCurConfigID;
 
     GridData gridData;
 
     int userMode = COConfigurationManager.getIntParameter("User Mode");
 
-    final IpFilterManager ipFilterManager = azureus_core.getIpFilterManager();
+    final IpFilterManager ipFilterManager = AzureusCoreFactory.getSingleton().getIpFilterManager();
     filter = ipFilterManager.getIPFilter();
 
     Composite gFilter = new Composite(parent, SWT.NULL);
@@ -239,9 +251,8 @@ public class ConfigSectionIPFilter implements UISWTConfigSection {
     layout.marginWidth = 0;
     cIndent.setLayout(layout);
     
+		Image img = imageLoader.getImage("subitem");
 		Label label = new Label(cIndent, SWT.NULL);
-		Image img = ImageRepository.getImage("subitem");
-		img.setBackground(label.getBackground());
 		gridData = new GridData(GridData.VERTICAL_ALIGN_BEGINNING);
 		label.setLayoutData(gridData);
 		label.setImage(img);
@@ -568,13 +579,13 @@ public class ConfigSectionIPFilter implements UISWTConfigSection {
   }
 
   public void editRange(IpRange range) {
-    new IpFilterEditor(azureus_core,table.getShell(), range);
+    new IpFilterEditor(AzureusCoreFactory.getSingleton(),table.getShell(), range);
     noChange = false;
     //refresh();
   }
 
   public void addRange() {
-    new IpFilterEditor(azureus_core,table.getShell(), null);
+    new IpFilterEditor(AzureusCoreFactory.getSingleton(),table.getShell(), null);
     //noChange = false;
     //refresh();
   }
