@@ -30,6 +30,7 @@ import org.eclipse.swt.events.ModifyListener;
 import org.eclipse.swt.layout.GridData;
 import org.eclipse.swt.layout.GridLayout;
 import org.eclipse.swt.widgets.*;
+import org.gudy.azureus2.core3.config.impl.StringListImpl;
 import org.gudy.azureus2.core3.config.COConfigurationManager;
 import org.gudy.azureus2.core3.config.ParameterListener;
 import org.gudy.azureus2.core3.internat.MessageText;
@@ -41,8 +42,11 @@ import org.gudy.azureus2.platform.PlatformManagerCapabilities;
 import org.gudy.azureus2.plugins.platform.PlatformManagerException;
 import org.gudy.azureus2.plugins.ui.config.ConfigSection;
 import org.gudy.azureus2.ui.swt.Messages;
+import org.gudy.azureus2.ui.swt.Utils;
+import org.gudy.azureus2.ui.swt.components.LinkLabel;
 import org.gudy.azureus2.ui.swt.config.*;
 import org.gudy.azureus2.ui.swt.plugins.UISWTConfigSection;
+import org.gudy.azureus2.core3.util.TrackersUtil;
 
 import java.util.HashMap;
 
@@ -51,8 +55,6 @@ public class ConfigSectionInterface implements UISWTConfigSection {
 
 	private final static String LBLKEY_PREFIX = "ConfigView.label.";
 
-	Label passwordMatch;
-
 	private ParameterListener decisions_parameter_listener;
 
 	public String configSectionGetParentSection() {
@@ -74,6 +76,11 @@ public class ConfigSectionInterface implements UISWTConfigSection {
 					"MessageBoxWindow.decisions", decisions_parameter_listener);
 		}
 	}
+	
+	public int maxUserMode() {
+		return 0;
+	}
+
 
 	public Composite configSectionCreate(final Composite parent) {
 		GridData gridData;
@@ -87,20 +94,49 @@ public class ConfigSectionInterface implements UISWTConfigSection {
 		cDisplay.setLayoutData(gridData);
 		layout = new GridLayout();
 		layout.numColumns = 1;
+		layout.marginWidth = 0;
+		layout.marginHeight = 0;
 		cDisplay.setLayout(layout);
 
-		new BooleanParameter(cDisplay, "Open Details", LBLKEY_PREFIX
-				+ "opendetails");
-		new BooleanParameter(cDisplay, "Open Bar", false, LBLKEY_PREFIX + "openbar");
+		/////////////
+    Group gAutoOpen = new Group(cDisplay, SWT.NULL);
+    Messages.setLanguageText(gAutoOpen, LBLKEY_PREFIX + "autoopen");
+    layout = new GridLayout(3, false);
+    gAutoOpen.setLayout(layout);
+    gAutoOpen.setLayoutData(new GridData());
+
+
+		label = new Label(gAutoOpen, SWT.NULL);
+		Messages.setLanguageText(label, LBLKEY_PREFIX + "autoopen.detailstab");
+		new BooleanParameter(gAutoOpen, "Open Details", LBLKEY_PREFIX
+				+ "autoopen.dl");
+		new BooleanParameter(gAutoOpen, "Open Seeding Details", LBLKEY_PREFIX
+				+ "autoopen.cd");
+
+
+		label = new Label(gAutoOpen, SWT.NULL);
+		Messages.setLanguageText(label, LBLKEY_PREFIX + "autoopen.downloadbars");
+		new BooleanParameter(gAutoOpen, "Open Bar Incomplete", LBLKEY_PREFIX + "autoopen.dl");
+		new BooleanParameter(gAutoOpen, "Open Bar Complete", LBLKEY_PREFIX + "autoopen.cd");
+		
+		/////////////
+		
+		new BooleanParameter(cDisplay, "Remember transfer bar location", LBLKEY_PREFIX + "transferbar.remember_location");
 
 		if (!Constants.isOSX || SWT.getVersion() >= 3300) {
 
-			BooleanParameter est = new BooleanParameter(cDisplay,
+	    Group gSysTray = new Group(cDisplay, SWT.NULL);
+	    Messages.setLanguageText(gSysTray, LBLKEY_PREFIX + "systray");
+	    layout = new GridLayout();
+	    gSysTray.setLayout(layout);
+	    gSysTray.setLayoutData(new GridData());
+
+			BooleanParameter est = new BooleanParameter(gSysTray,
 					"Enable System Tray", KEY_PREFIX + "enabletray");
 
-			BooleanParameter ctt = new BooleanParameter(cDisplay, "Close To Tray",
+			BooleanParameter ctt = new BooleanParameter(gSysTray, "Close To Tray",
 					LBLKEY_PREFIX + "closetotray");
-			BooleanParameter mtt = new BooleanParameter(cDisplay, "Minimize To Tray",
+			BooleanParameter mtt = new BooleanParameter(gSysTray, "Minimize To Tray",
 					LBLKEY_PREFIX + "minimizetotray");
 
 			est.setAdditionalActionPerformer(new ChangeSelectionActionPerformer(ctt
@@ -117,8 +153,10 @@ public class ConfigSectionInterface implements UISWTConfigSection {
         Messages.setLanguageText(limit_group, LBLKEY_PREFIX + "set_ui_transfer_speeds");
         layout = new GridLayout();
         limit_group.setLayout(layout);
+        limit_group.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
         
         Label limit_group_label = new Label(limit_group, SWT.WRAP);
+        limit_group_label.setLayoutData(Utils.getWrappableLabelGridData(1, GridData.GRAB_HORIZONTAL));
         Messages.setLanguageText(limit_group_label, LBLKEY_PREFIX + "set_ui_transfer_speeds.description");
         
         String[] limit_types = new String[] {"download", "upload"};
@@ -139,7 +177,7 @@ public class ConfigSectionInterface implements UISWTConfigSection {
         	bp.setAdditionalActionPerformer(iaap);
         }
 
-		new BooleanParameter(cDisplay, "Send Version Info", true, LBLKEY_PREFIX
+		new BooleanParameter(cDisplay, "Send Version Info", LBLKEY_PREFIX
 				+ "allowSendVersion");
 
 		Composite cArea = new Composite(cDisplay, SWT.NULL);
@@ -149,6 +187,9 @@ public class ConfigSectionInterface implements UISWTConfigSection {
 		layout.numColumns = 2;
 		cArea.setLayout(layout);
 		cArea.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
+		
+		new LinkLabel(cArea, LBLKEY_PREFIX + "version.info.link",
+				"http://www.azureuswiki.com/index.php/Version.azureusplatform.com");
 
 		if (!Constants.isOSX) {
 
@@ -193,6 +234,32 @@ public class ConfigSectionInterface implements UISWTConfigSection {
 			}
 		});
 
+		final Label clear_tracker_label = new Label(cArea, SWT.NULL);
+		Messages.setLanguageText(clear_tracker_label, KEY_PREFIX + "cleartrackers");
+
+		final Button clear_tracker_button = new Button(cArea, SWT.PUSH);
+		Messages.setLanguageText(clear_tracker_button, KEY_PREFIX
+				+ "cleartrackersbutton");
+		
+		clear_tracker_button.addListener(SWT.Selection, new Listener() {
+			public void handleEvent(Event event) {
+				TrackersUtil.getInstance().clearAllTrackers(true);
+			}
+		});
+		
+		final Label clear_save_path_label = new Label(cArea, SWT.NULL);
+		Messages.setLanguageText(clear_save_path_label, KEY_PREFIX + "clearsavepaths");
+
+		final Button clear_save_path_button = new Button(cArea, SWT.PUSH);
+		Messages.setLanguageText(clear_save_path_button, KEY_PREFIX
+				+ "clearsavepathsbutton");
+		
+		clear_save_path_button.addListener(SWT.Selection, new Listener() {
+			public void handleEvent(Event event) {
+				COConfigurationManager.setParameter("saveTo_list", new StringListImpl());
+			}
+		});
+		
 		decisions_parameter_listener = new ParameterListener() {
 			public void parameterChanged(String parameterName) {
 				if (clear_decisions.isDisposed()) {
@@ -218,48 +285,6 @@ public class ConfigSectionInterface implements UISWTConfigSection {
 		COConfigurationManager.addParameterListener("MessageBoxWindow.decisions",
 				decisions_parameter_listener);
 
-		// password
-
-		label = new Label(cArea, SWT.NULL);
-		Messages.setLanguageText(label, LBLKEY_PREFIX + "password");
-
-		gridData = new GridData();
-		gridData.widthHint = 150;
-		PasswordParameter pw1 = new PasswordParameter(cArea, "Password");
-		pw1.setLayoutData(gridData);
-		Text t1 = (Text) pw1.getControl();
-
-		//password confirm
-
-		label = new Label(cArea, SWT.NULL);
-		Messages.setLanguageText(label, LBLKEY_PREFIX + "passwordconfirm");
-		gridData = new GridData();
-		gridData.widthHint = 150;
-		PasswordParameter pw2 = new PasswordParameter(cArea, "Password Confirm");
-		pw2.setLayoutData(gridData);
-		Text t2 = (Text) pw2.getControl();
-
-		// password activated
-
-		label = new Label(cArea, SWT.NULL);
-		Messages.setLanguageText(label, LBLKEY_PREFIX + "passwordmatch");
-		passwordMatch = new Label(cArea, SWT.NULL);
-		gridData = new GridData();
-		gridData.widthHint = 150;
-		passwordMatch.setLayoutData(gridData);
-		refreshPWLabel();
-
-		t1.addModifyListener(new ModifyListener() {
-			public void modifyText(ModifyEvent e) {
-				refreshPWLabel();
-			}
-		});
-		t2.addModifyListener(new ModifyListener() {
-			public void modifyText(ModifyEvent e) {
-				refreshPWLabel();
-			}
-		});
-
 		// drag-drop
 
 		label = new Label(cArea, SWT.NULL);
@@ -278,8 +303,33 @@ public class ConfigSectionInterface implements UISWTConfigSection {
 			dropLabels[i] = MessageText.getString(drop_options[i]);
 			dropValues[i] = "" + i;
 		}
-		new StringListParameter(cArea, "config.style.dropdiraction", "1",
+		new StringListParameter(cArea, "config.style.dropdiraction",
 				dropLabels, dropValues);
+		
+		// double-click
+		
+    if (COConfigurationManager.getStringParameter("ui").equals("az3")) {
+  		label = new Label(cArea, SWT.NULL);
+  		Messages.setLanguageText(label, LBLKEY_PREFIX + "dm.dblclick");
+  
+  		String[] dblclickOptions = {
+  			"ConfigView.option.dm.dblclick.play",
+  			"ConfigView.option.dm.dblclick.details",
+  			"ConfigView.option.dm.dblclick.show",
+  		};
+  
+  		String dblclickLabels[] = new String[dblclickOptions.length];
+  		String dblclickValues[] = new String[dblclickOptions.length];
+  
+  		for (int i = 0; i < dblclickOptions.length; i++) {
+  
+  			dblclickLabels[i] = MessageText.getString(dblclickOptions[i]);
+  			dblclickValues[i] = "" + i;
+  		}
+  		new StringListParameter(cArea, "list.dm.dblclick", dblclickLabels,
+  				dblclickValues);
+    }
+		
 
 		// reset associations
 
@@ -325,38 +375,4 @@ public class ConfigSectionInterface implements UISWTConfigSection {
 		return cDisplay;
 	}
 
-	private void refreshPWLabel() {
-
-		if (passwordMatch == null || passwordMatch.isDisposed())
-			return;
-		byte[] password = COConfigurationManager.getByteParameter("Password", ""
-				.getBytes());
-		COConfigurationManager.setParameter("Password enabled", false);
-		if (password.length == 0) {
-			passwordMatch.setText(MessageText.getString(LBLKEY_PREFIX
-					+ "passwordmatchnone"));
-		} else {
-			byte[] confirm = COConfigurationManager.getByteParameter(
-					"Password Confirm", "".getBytes());
-			if (confirm.length == 0) {
-				passwordMatch.setText(MessageText.getString(LBLKEY_PREFIX
-						+ "passwordmatchno"));
-			} else {
-				boolean same = true;
-				for (int i = 0; i < password.length; i++) {
-					if (password[i] != confirm[i])
-						same = false;
-				}
-				if (same) {
-					passwordMatch.setText(MessageText.getString(LBLKEY_PREFIX
-							+ "passwordmatchyes"));
-					COConfigurationManager.setParameter("Password enabled", true);
-				} else {
-					passwordMatch.setText(MessageText.getString(LBLKEY_PREFIX
-							+ "passwordmatchno"));
-				}
-			}
-		}
-	}
-
 }
