@@ -27,24 +27,23 @@ package org.gudy.azureus2.ui.swt.views.configsections;
 import java.io.File;
 
 import org.eclipse.swt.SWT;
-import org.eclipse.swt.widgets.Composite;
-import org.eclipse.swt.widgets.Control;
 import org.eclipse.swt.layout.GridData;
 import org.eclipse.swt.layout.GridLayout;
-import org.eclipse.swt.widgets.Label;
-import org.eclipse.swt.widgets.Button;
-import org.eclipse.swt.widgets.Event;
-import org.eclipse.swt.widgets.Listener;
+import org.eclipse.swt.widgets.*;
 
-import org.gudy.azureus2.plugins.ui.config.ConfigSection;
-import org.gudy.azureus2.ui.swt.config.*;
-import org.gudy.azureus2.ui.swt.plugins.UISWTConfigSection;
-import org.gudy.azureus2.ui.swt.Messages;
-import org.gudy.azureus2.ui.swt.Utils;
+import org.gudy.azureus2.core3.config.COConfigurationManager;
 import org.gudy.azureus2.core3.internat.MessageText;
+import org.gudy.azureus2.core3.util.Constants;
 import org.gudy.azureus2.core3.util.Debug;
 import org.gudy.azureus2.core3.util.FileUtil;
-import org.gudy.azureus2.core3.util.Constants;
+import org.gudy.azureus2.platform.PlatformManagerCapabilities;
+import org.gudy.azureus2.platform.PlatformManagerFactory;
+import org.gudy.azureus2.ui.swt.Messages;
+import org.gudy.azureus2.ui.swt.Utils;
+import org.gudy.azureus2.ui.swt.config.*;
+import org.gudy.azureus2.ui.swt.plugins.UISWTConfigSection;
+
+import org.gudy.azureus2.plugins.ui.config.ConfigSection;
 
 public class ConfigSectionInterfaceDisplay implements UISWTConfigSection {
 	private final static String MSG_PREFIX = "ConfigView.section.style.";
@@ -63,9 +62,17 @@ public class ConfigSectionInterfaceDisplay implements UISWTConfigSection {
 
 	public void configSectionDelete() {
 	}
+	
+	public int maxUserMode() {
+		return 2;
+	}
+
 
 	public Composite configSectionCreate(final Composite parent) {
-		// "Display" Sub-Section:
+    int userMode = COConfigurationManager.getIntParameter("User Mode");
+		boolean isAZ3 = COConfigurationManager.getStringParameter("ui").equals("az3");
+
+    // "Display" Sub-Section:
 		// ----------------------
 		// Any Look & Feel settings that don't really change the way the user 
 		// normally interacts
@@ -78,20 +85,13 @@ public class ConfigSectionInterfaceDisplay implements UISWTConfigSection {
 		layout.numColumns = 1;
 		cLook.setLayout(layout);
 
-		BooleanParameter bpCustomTab = new BooleanParameter(cLook, "useCustomTab", MSG_PREFIX + "useCustomTabs");
-		Control cFancyTab = new BooleanParameter(cLook, "GUI_SWT_bFancyTab", 
-				MSG_PREFIX + "useFancyTabs").getControl();
-
-		Control[] controls = { cFancyTab };
-		bpCustomTab
-				.setAdditionalActionPerformer(new ChangeSelectionActionPerformer(
-						controls));
-
 		new BooleanParameter(cLook, "Show Download Basket", MSG_PREFIX
 				+ "showdownloadbasket");
 
-		new BooleanParameter(cLook, "IconBar.enabled", MSG_PREFIX
-				+ "showiconbar");
+		if (!isAZ3) {
+  		new BooleanParameter(cLook, "IconBar.enabled", MSG_PREFIX
+  				+ "showiconbar");
+		}
 
 		Composite cStatusBar = new Composite(cLook, SWT.NULL);
 		layout = new GridLayout();
@@ -111,6 +111,10 @@ public class ConfigSectionInterfaceDisplay implements UISWTConfigSection {
 		new BooleanParameter(cLook, "Add URL Silently", MSG_PREFIX	+ "addurlsilently");
 		new BooleanParameter(cLook, "add_torrents_silently", "ConfigView.section.interface.display.add_torrents_silently");
 
+		new BooleanParameter(cLook, "suppress_file_download_dialog", "ConfigView.section.interface.display.suppress.file.download.dialog");
+
+		new BooleanParameter(cLook, "show_torrents_menu", "Menu.show.torrent.menu");
+
 		if (Constants.isWindowsXP) {
 			final Button enableXPStyle = new Button(cLook, SWT.CHECK);
 			Messages.setLanguageText(enableXPStyle, MSG_PREFIX + "enableXPStyle");
@@ -162,31 +166,46 @@ public class ConfigSectionInterfaceDisplay implements UISWTConfigSection {
 
 		if (Utils.isGTK) {
 			// See Eclipse Bug #42416 ([Platform Inconsistency] GC(Table) has wrong origin)
-			new BooleanParameter(cLook, "SWT_bGTKTableBug", true, MSG_PREFIX
+			new BooleanParameter(cLook, "SWT_bGTKTableBug", MSG_PREFIX
 					+ "verticaloffset");
 		}
 
 		if (Constants.isOSX) {
-			new BooleanParameter(cLook, "enable_small_osx_fonts", true, MSG_PREFIX
+			new BooleanParameter(cLook, "enable_small_osx_fonts", MSG_PREFIX
 					+ "osx_small_fonts");
 		}
 
-		new BooleanParameter(cLook, "GUI_SWT_bAlternateTablePainting", MSG_PREFIX
-				+ "alternateTablePainting");
-
-		new BooleanParameter(cLook, "config.style.useSIUnits", MSG_PREFIX
-				+ "useSIUnits");
-		new BooleanParameter(cLook, "config.style.useUnitsRateBits",
-				MSG_PREFIX + "useUnitsRateBits");
-		new BooleanParameter(cLook, "config.style.doNotUseGB", false, MSG_PREFIX
-				+ "doNotUseGB");
-
-		new BooleanParameter(cLook, "config.style.dataStatsOnly", MSG_PREFIX
-				+ "dataStatsOnly");
-
-		new BooleanParameter(cLook, "config.style.separateProtDataStats", MSG_PREFIX
-				+ "separateProtDataStats");
+		if (userMode > 1) {
+  		new BooleanParameter(cLook, "GUI_SWT_bAlternateTablePainting", MSG_PREFIX
+  				+ "alternateTablePainting");
+		}
 
+		if (userMode > 0) {
+  		new BooleanParameter(cLook, "config.style.useSIUnits", MSG_PREFIX
+  				+ "useSIUnits");
+  		new BooleanParameter(cLook, "config.style.useUnitsRateBits",
+  				MSG_PREFIX + "useUnitsRateBits");
+  		new BooleanParameter(cLook, "config.style.doNotUseGB", MSG_PREFIX
+  				+ "doNotUseGB");
+  
+  		new BooleanParameter(cLook, "config.style.dataStatsOnly", MSG_PREFIX
+  				+ "dataStatsOnly");
+  
+  		new BooleanParameter(cLook, "config.style.separateProtDataStats", MSG_PREFIX
+  				+ "separateProtDataStats");
+		}
+		
+		
+    if( userMode > 1 ) {
+  		final BooleanParameter fMoz = new BooleanParameter(cLook, "swt.forceMozilla",MSG_PREFIX + "forceMozilla");
+  		Composite pArea = new Composite(cLook,SWT.NULL);
+  		pArea.setLayout(new GridLayout(3,false));
+  		pArea.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
+  		Messages.setLanguageText(new Label(pArea,SWT.NONE), MSG_PREFIX+"xulRunnerPath");
+  		final Parameter xulDir = new DirectoryParameter(pArea, "swt.xulRunner.path","");
+  		fMoz.setAdditionalActionPerformer(new ChangeSelectionActionPerformer(xulDir.getControls(), false));
+    }
+		
 		Composite cArea = new Composite(cLook, SWT.NULL);
 		layout = new GridLayout();
 		layout.marginHeight = 0;
@@ -194,19 +213,25 @@ public class ConfigSectionInterfaceDisplay implements UISWTConfigSection {
 		layout.numColumns = 2;
 		cArea.setLayout(layout);
 		cArea.setLayoutData(new GridData());
-
+		
 		label = new Label(cArea, SWT.NULL);
 		Messages.setLanguageText(label, MSG_PREFIX + "guiUpdate");
 		int[] values = { 100, 250, 500, 1000, 2000, 5000, 10000, 15000 };
 		String[] labels = { "100 ms", "250 ms", "500 ms", "1 s", "2 s", "5 s", "10 s", "15 s" };
 		new IntListParameter(cArea, "GUI Refresh", 1000, labels, values);
+		
+		label = new Label(cArea, SWT.NULL);
+		Messages.setLanguageText(label, MSG_PREFIX + "inactiveUpdate");
+		gridData = new GridData();
+		gridData.widthHint = 15;
+		IntParameter inactiveUpdate = new IntParameter(cArea, "Refresh When Inactive", 1,	-1);
+		inactiveUpdate.setLayoutData(gridData);
 
 		label = new Label(cArea, SWT.NULL);
 		Messages.setLanguageText(label, MSG_PREFIX + "graphicsUpdate");
 		gridData = new GridData();
 		gridData.widthHint = 15;
-		IntParameter graphicUpdate = new IntParameter(cArea, "Graphics Update", 1,
-				-1);
+		IntParameter graphicUpdate = new IntParameter(cArea, "Graphics Update", 1,	-1);
 		graphicUpdate.setLayoutData(gridData);
 
 		label = new Label(cArea, SWT.NULL);
@@ -226,15 +251,29 @@ public class ConfigSectionInterfaceDisplay implements UISWTConfigSection {
 		new IntListParameter(cArea, "config.style.table.defaultSortOrder",
 				sortOrderLabels, sortOrderValues);
 		
-		new BooleanParameter(cArea, "FilesView.separate_rename_and_retarget", MSG_PREFIX
-				+ "SeparateFilesViewRenameRetarget");
-
 		new BooleanParameter(cLook, "NameColumn.showProgramIcon", MSG_PREFIX
 				+ "showProgramIcon");
 
 		new BooleanParameter(cLook, "DND Always In Incomplete", MSG_PREFIX
 				+ "DNDalwaysInIncomplete");
 		
+		new BooleanParameter(cLook, "MyTorrentsView.alwaysShowHeader", "ConfigView.label.alwaysShowLibraryHeader");
+		
+		// Reuse the labels of the other menu actions.
+		if (PlatformManagerFactory.getPlatformManager().hasCapability(PlatformManagerCapabilities.ShowFileInBrowser)) {
+			BooleanParameter bp = new BooleanParameter(cLook, "MyTorrentsView.menu.show_parent_folder_enabled", MSG_PREFIX
+					+ "use_show_parent_folder");
+			Messages.setLanguageText(bp.getControl(), "ConfigView.section.style.use_show_parent_folder", new String[] {
+				MessageText.getString("MyTorrentsView.menu.open_parent_folder"),
+				MessageText.getString("MyTorrentsView.menu.explore"),
+			});
+			
+			if (Constants.isOSX) {
+				new BooleanParameter(cLook, "FileBrowse.usePathFinder", 
+						MSG_PREFIX + "usePathFinder");
+			}
+		}
+		
 		return cLook;
 	}
 }
