@@ -28,6 +28,8 @@ import java.util.List;
 
 import org.eclipse.swt.SWT;
 import org.eclipse.swt.events.*;
+import org.eclipse.swt.graphics.Image;
+import org.eclipse.swt.layout.FillLayout;
 import org.eclipse.swt.layout.GridData;
 import org.eclipse.swt.layout.GridLayout;
 import org.eclipse.swt.widgets.*;
@@ -36,8 +38,8 @@ import org.gudy.azureus2.core3.config.COConfigurationManager;
 import org.gudy.azureus2.core3.config.ParameterListener;
 import org.gudy.azureus2.core3.internat.MessageText;
 import org.gudy.azureus2.core3.util.AERunnable;
+import org.gudy.azureus2.core3.util.Debug;
 import org.gudy.azureus2.core3.util.FileUtil;
-import org.gudy.azureus2.ui.swt.ImageRepository;
 import org.gudy.azureus2.ui.swt.Messages;
 import org.gudy.azureus2.ui.swt.Utils;
 import org.gudy.azureus2.ui.swt.config.DualChangeSelectionActionPerformer;
@@ -49,6 +51,8 @@ import org.gudy.azureus2.ui.swt.plugins.UISWTConfigSection;
 import org.gudy.azureus2.ui.swt.views.ConfigView;
 
 import com.aelitis.azureus.core.AzureusCore;
+import com.aelitis.azureus.core.AzureusCoreFactory;
+import com.aelitis.azureus.ui.swt.imageloader.ImageLoader;
 
 import org.gudy.azureus2.plugins.PluginException;
 import org.gudy.azureus2.plugins.PluginInterface;
@@ -81,14 +85,16 @@ public class ConfigSectionPlugins implements UISWTConfigSection, ParameterListen
 
 	private ConfigView configView;
 
-	private AzureusCore azureusCore;
-
 	FilterComparator comparator;
 	
 	List pluginIFs;
 
 	private Table table;
 	
+	private Image imgRedLed;
+	
+	private Image imgGreenLed;
+	
 	class FilterComparator implements Comparator {
 		boolean ascending = true;
 
@@ -129,11 +135,15 @@ public class ConfigSectionPlugins implements UISWTConfigSection, ParameterListen
 
 			switch (field) {
 				case FIELD_LOAD: {
-					boolean b0 = COConfigurationManager.getBooleanParameter("PluginInfo."
-							+ if0.getPluginID() + ".enabled", true);
-					boolean b1 = COConfigurationManager.getBooleanParameter("PluginInfo."
-							+ if1.getPluginID() + ".enabled", true);
-					result = (b0 == b1 ? 0 : (b0 ? 1 : -1));
+					boolean b0 = if0.getPluginState().isLoadedAtStartup();
+					boolean b1 = if1.getPluginState().isLoadedAtStartup();
+					result = (b0 == b1 ? 0 : (b0 ? -1 : 1));
+					
+					// Use the plugin ID name to sort by instead.
+					if (result == 0) {
+						result = if0.getPluginID().compareToIgnoreCase(
+								if1.getPluginID());
+					}
 					break;
 				}
 
@@ -156,9 +166,9 @@ public class ConfigSectionPlugins implements UISWTConfigSection, ParameterListen
 				}
 				
 				case FIELD_UNLOADABLE: {
-					boolean b0 = if0.isUnloadable();
-					boolean b1 = if1.isUnloadable();
-					result = (b0 == b1 ? 0 : (b0 ? 1 : -1));
+					boolean b0 = if0.getPluginState().isUnloadable();
+					boolean b1 = if1.getPluginState().isUnloadable();
+					result = (b0 == b1 ? 0 : (b0 ? -1 : 1));
 					break;
 				}
 			}
@@ -233,7 +243,7 @@ public class ConfigSectionPlugins implements UISWTConfigSection, ParameterListen
 				
 				case FIELD_UNLOADABLE: {
 					return MessageText.getString("Button."
-							+ (pluginIF.isUnloadable() ? "yes" : "no")).replaceAll("&", "");
+							+ (pluginIF.getPluginState().isUnloadable() ? "yes" : "no")).replaceAll("&", "");
 				}
 			} // switch
 
@@ -245,9 +255,8 @@ public class ConfigSectionPlugins implements UISWTConfigSection, ParameterListen
 	 * Initialize
 	 * @param _configView
 	 */
-	public ConfigSectionPlugins(ConfigView _configView, AzureusCore _azureusCore) {
+	public ConfigSectionPlugins(ConfigView _configView) {
 		configView = _configView;
-		azureusCore = _azureusCore;
 		comparator = new FilterComparator();
 	}
 
@@ -271,13 +280,28 @@ public class ConfigSectionPlugins implements UISWTConfigSection, ParameterListen
 
 
 	public void configSectionDelete() {
+		ImageLoader imageLoader = ImageLoader.getInstance();
+		imageLoader.releaseImage("redled");
+		imageLoader.releaseImage("greenled");
 	}
 
 	public Composite configSectionCreate(final Composite parent) {
+  	if (!AzureusCoreFactory.isCoreRunning()) {
+      Composite cSection = new Composite(parent, SWT.NULL);
+    	cSection.setLayout(new FillLayout());
+    	Label lblNotAvail = new Label(cSection, SWT.WRAP);
+    	Messages.setLanguageText(lblNotAvail, "core.not.available");
+    	return cSection;
+    }
+
 		GridLayout layout;
 		GridData gridData;
 
 		Label label;
+		
+		ImageLoader imageLoader = ImageLoader.getInstance();
+		imgRedLed = imageLoader.getImage("redled");
+		imgGreenLed = imageLoader.getImage("greenled");
 
 		Composite infoGroup = new Composite(parent, SWT.NULL);
 		gridData = new GridData(GridData.VERTICAL_ALIGN_FILL
@@ -445,10 +469,10 @@ public class ConfigSectionPlugins implements UISWTConfigSection, ParameterListen
 					int index = items[i];
 					if (index >= 0 && index < pluginIFs.size()) {
 						PluginInterface pluginIF = (PluginInterface) pluginIFs.get(index);
-						if (pluginIF.isOperational()) {
-							if (pluginIF.isUnloadable()) {
+						if (pluginIF.getPluginState().isOperational()) {
+							if (pluginIF.getPluginState().isUnloadable()) {
 								try {
-									pluginIF.unload();
+									pluginIF.getPluginState().unload();
 								} catch (PluginException e1) {
 									// TODO Auto-generated catch block
 									e1.printStackTrace();
@@ -474,19 +498,24 @@ public class ConfigSectionPlugins implements UISWTConfigSection, ParameterListen
 				for (int i = 0; i < items.length; i++) {
 					int index = items[i];
 					if (index >= 0 && index < pluginIFs.size()) {
+						
 						PluginInterface pluginIF = (PluginInterface) pluginIFs.get(index);
+						if (pluginIF.getPluginState().isOperational()) {continue;} // Already loaded. 
 
-						if (pluginIF.isDisabled()) {
-							try {
-								COConfigurationManager.setParameter("PluginInfo."
-										+ pluginIF.getPluginID() + ".enabled", true );
-								pluginIF.setDisabled( false );
-								pluginIF.reload();
-							} catch (PluginException e1) {
-								// TODO Auto-generated catch block
-								e1.printStackTrace();
-							}
+						// Re-enable disabled plugins, as long as they haven't failed on
+						// initialise.
+						if (pluginIF.getPluginState().isDisabled()) {
+							if (pluginIF.getPluginState().hasFailed()) {continue;}
+							pluginIF.getPluginState().setDisabled(false);
 						}
+						
+						try {
+							pluginIF.getPluginState().reload();
+						} catch (PluginException e1) {
+							// TODO Auto-generated catch block
+							Debug.printStackTrace(e1);
+						}
+						
 						pluginIFs = rebuildPluginIFs();
 						table.setItemCount(pluginIFs.size());
 						Collections.sort(pluginIFs, comparator);
@@ -503,7 +532,7 @@ public class ConfigSectionPlugins implements UISWTConfigSection, ParameterListen
 		Messages.setLanguageText(btnScan, "ConfigView.pluginlist.scan");
 		btnScan.addSelectionListener(new SelectionAdapter() {
 			public void widgetSelected(SelectionEvent e) {
-				azureusCore.getPluginManager().refreshPluginList();
+				AzureusCoreFactory.getSingleton().getPluginManager().refreshPluginList(false);
 				pluginIFs = rebuildPluginIFs();
 				table.setItemCount(pluginIFs.size());
 				Collections.sort(pluginIFs, comparator);
@@ -520,8 +549,8 @@ public class ConfigSectionPlugins implements UISWTConfigSection, ParameterListen
 
 				for (int i = 0; i < COLUMN_HEADERS.length; i++) {
 					if (i == FilterComparator.FIELD_NAME)
-						item.setImage(i, ImageRepository.getImage(pluginIF.isOperational()
-								? "greenled" : "redled")); 
+						item.setImage(i, pluginIF.getPluginState().isOperational()
+								? imgGreenLed : imgRedLed); 
 					
 					String sText = comparator.getFieldValue(i, pluginIF);
 					if (sText == null)
@@ -529,9 +558,8 @@ public class ConfigSectionPlugins implements UISWTConfigSection, ParameterListen
 					item.setText(i, sText);
 				}
 
-				item.setGrayed(pluginIF.isMandatory());
-				boolean bEnabled = COConfigurationManager.getBooleanParameter("PluginInfo."
-						+ pluginIF.getPluginID() + ".enabled", true);
+				item.setGrayed(pluginIF.getPluginState().isMandatory());
+				boolean bEnabled = pluginIF.getPluginState().isLoadedAtStartup();
 		    Utils.setCheckedInSetData(item, bEnabled);
 				item.setData("PluginID", pluginIF.getPluginID());
 				Utils.alternateRowBackground(item);
@@ -552,16 +580,12 @@ public class ConfigSectionPlugins implements UISWTConfigSection, ParameterListen
 						return;
 					}
 
-					pluginIF.setDisabled(!item.getChecked());
-					COConfigurationManager.setParameter("PluginInfo."
-							+ item.getData("PluginID") + ".enabled", item.getChecked());
+					pluginIF.getPluginState().setDisabled(!item.getChecked());
+					pluginIF.getPluginState().setLoadedAtStartup(item.getChecked());
 				}
 				
-				btnUnload.setEnabled( pluginIF.isUnloadable());
-				boolean bEnabled = COConfigurationManager.getBooleanParameter("PluginInfo."
-						+ pluginIF.getPluginID() + ".enabled", true);
-				btnLoad.setEnabled(pluginIF.isDisabled() && pluginIF.isOperational()
-						&& !bEnabled);
+				btnUnload.setEnabled(pluginIF.getPluginState().isOperational() && pluginIF.getPluginState().isUnloadable()); 
+				btnLoad.setEnabled(!pluginIF.getPluginState().isOperational() && !pluginIF.getPluginState().hasFailed());
 			}
 		});
 
@@ -577,7 +601,7 @@ public class ConfigSectionPlugins implements UISWTConfigSection, ParameterListen
 	 * @since 3.0.5.3
 	 */
 	private List rebuildPluginIFs() {
-		List pluginIFs = Arrays.asList(azureusCore.getPluginManager().getPlugins());
+		List pluginIFs = Arrays.asList(AzureusCoreFactory.getSingleton().getPluginManager().getPlugins());
 		for (Iterator iter = pluginIFs.iterator(); iter.hasNext();) {
 			PluginInterface pi = (PluginInterface) iter.next();
 			// COConfigurationManager will not add the same listener twice
