@@ -25,35 +25,26 @@
 package org.gudy.azureus2.ui.swt.views.configsections;
 
 import org.eclipse.swt.SWT;
-import org.eclipse.swt.widgets.Composite;
-import org.eclipse.swt.widgets.Control;
-import org.eclipse.swt.layout.GridData;
-import org.eclipse.swt.layout.GridLayout;
-import org.eclipse.swt.widgets.Label;
-import org.eclipse.swt.widgets.Button;
-import org.eclipse.swt.widgets.Event;
-import org.eclipse.swt.widgets.Listener;
-import org.eclipse.swt.widgets.Group;
-import org.eclipse.swt.widgets.Display;
 import org.eclipse.swt.events.MouseAdapter;
 import org.eclipse.swt.events.MouseEvent;
+import org.eclipse.swt.layout.GridData;
+import org.eclipse.swt.layout.GridLayout;
+import org.eclipse.swt.widgets.*;
 
-import com.aelitis.azureus.core.*;
-
-import org.gudy.azureus2.plugins.ui.config.ConfigSection;
-import org.gudy.azureus2.ui.swt.config.*;
-import org.gudy.azureus2.ui.swt.Messages;
-import org.gudy.azureus2.ui.swt.Utils;
 import org.gudy.azureus2.core3.config.COConfigurationManager;
 import org.gudy.azureus2.core3.internat.MessageText;
 import org.gudy.azureus2.core3.util.AENetworkClassifier;
 import org.gudy.azureus2.core3.util.AERunnable;
+import org.gudy.azureus2.plugins.ui.config.ConfigSection;
+import org.gudy.azureus2.ui.swt.Messages;
+import org.gudy.azureus2.ui.swt.Utils;
+import org.gudy.azureus2.ui.swt.auth.CertificateCreatorWindow;
+import org.gudy.azureus2.ui.swt.config.*;
 import org.gudy.azureus2.ui.swt.ipchecker.IpCheckerWizard;
 import org.gudy.azureus2.ui.swt.ipchecker.IpSetterCallBack;
 import org.gudy.azureus2.ui.swt.mainwindow.Colors;
 import org.gudy.azureus2.ui.swt.mainwindow.Cursors;
 import org.gudy.azureus2.ui.swt.plugins.UISWTConfigSection;
-import org.gudy.azureus2.ui.swt.auth.*;
 
 public class 
 ConfigSectionTrackerServer 
@@ -61,15 +52,14 @@ ConfigSectionTrackerServer
 {
 	private static final String	CFG_PREFIX = "ConfigView.section.";
 	
-	protected	AzureusCore	azureus_core;
-	
 	private final int REQUIRED_MODE = 1;
 	
-	public
-	ConfigSectionTrackerServer(
-		AzureusCore		_azureus_core )
-	{
-		azureus_core	= _azureus_core;
+	public int maxUserMode() {
+		return 2;
+	}
+
+	
+	public ConfigSectionTrackerServer() {
 	}
 	
   public String configSectionGetParentSection() {
@@ -155,7 +145,7 @@ ConfigSectionTrackerServer
 
     check_button.addListener(SWT.Selection, new Listener() {
         public void handleEvent(Event event) {
-          IpCheckerWizard wizard = new IpCheckerWizard(azureus_core, display);
+          IpCheckerWizard wizard = new IpCheckerWizard();
           wizard.setIpSetterCallBack(new IpSetterCallBack() {
               public void setIp(final String ip) {
                 if(display == null || display.isDisposed())
