@@ -45,8 +45,11 @@ import org.gudy.azureus2.ui.swt.Utils;
 import org.gudy.azureus2.ui.swt.components.graphics.SpeedGraphic;
 import org.gudy.azureus2.ui.swt.views.AbstractIView;
 
+import com.aelitis.azureus.core.AzureusCore;
+import com.aelitis.azureus.core.AzureusCoreRunningListener;
 import com.aelitis.azureus.core.AzureusCoreFactory;
 import com.aelitis.azureus.core.dht.DHT;
+import com.aelitis.azureus.core.dht.DHTStorageAdapter;
 import com.aelitis.azureus.core.dht.control.DHTControlActivity;
 import com.aelitis.azureus.core.dht.control.DHTControlListener;
 import com.aelitis.azureus.core.dht.control.DHTControlStats;
@@ -78,7 +81,7 @@ public class DHTView extends AbstractIView {
   Label lblNodes,lblLeaves;
   Label lblContacts,lblReplacements,lblLive,lblUnknown,lblDying;
   Label lblSkew, lblRendezvous, lblReachable;
-  Label lblKeys,lblValues;
+  Label lblKeys,lblValues,lblSize;
   Label lblLocal,lblDirect,lblIndirect;
   Label lblDivFreq,lblDivSize;
   
@@ -101,16 +104,23 @@ public class DHTView extends AbstractIView {
   DHTControlActivity[] activities;
   
   private final int dht_type;
+	protected AzureusCore core;
   
 
   public DHTView( int dht_type ) {
     this.dht_type = dht_type;
-    init();
+  	AzureusCoreFactory.addCoreRunningListener(new AzureusCoreRunningListener() {
+
+			public void azureusCoreRunning(AzureusCore core) {
+				DHTView.this.core = core;
+				init(core);
+			}
+		});
   }
   
-  private void init() {
+  private void init(AzureusCore core) {
     try {
-      PluginInterface dht_pi = AzureusCoreFactory.getSingleton().getPluginManager().getPluginInterfaceByClass( DHTPlugin.class );
+      PluginInterface dht_pi = core.getPluginManager().getPluginInterfaceByClass( DHTPlugin.class );
         
       if ( dht_pi == null ){
       	   
@@ -282,8 +292,10 @@ public class DHTView extends AbstractIView {
     lblValues.setLayoutData(new GridData(SWT.FILL,SWT.TOP,true,false));
     
     label = new Label(gDB,SWT.NONE);
-    label = new Label(gDB,SWT.NONE);
+    Messages.setLanguageText(label,"TableColumn.header.size");    
     
+    lblSize = new Label(gDB,SWT.NONE);
+    lblSize.setLayoutData(new GridData(SWT.FILL,SWT.TOP,true,false));    
     
     label = new Label(gDB,SWT.NONE);
     Messages.setLanguageText(label,"DHTView.db.local");    
@@ -562,10 +574,13 @@ public class DHTView extends AbstractIView {
   }
   
   public void refresh() {    
-    if(dht == null) { 
-      init();
-      return;
-    }
+  	if (dht == null) {
+  		if (core != null) {
+  			// keep trying until dht is avail
+  			init(core);
+  		}
+			return;
+  	}
     
     inGraph.refresh();
     outGraph.refresh();
@@ -584,7 +599,8 @@ public class DHTView extends AbstractIView {
     DHTTransportStats transportStats = transport.getStats();
     lblUpTime.setText(TimeFormatter.format(controlStats.getRouterUptime() / 1000));
     lblNumberOfUsers.setText("" + controlStats.getEstimatedDHTSize());
-    lblReachable.setText(transport.isReachable()?yes_str:no_str);
+    int percent = transportStats.getRouteablePercentage();
+    lblReachable.setText((transport.isReachable()?yes_str:no_str) + (percent==-1?"":(" " + percent+"%")));
     
     DHTNATPuncher puncher = dht.getNATPuncher();
     
@@ -616,14 +632,29 @@ public class DHTView extends AbstractIView {
   private void refreshDB() {    
     if(refreshIter == 0) {
 	  DHTDBStats    dbStats = dht.getDataBase().getStats();
-      lblKeys.setText("" + dbStats.getKeyCount());  
+      lblKeys.setText("" + dbStats.getKeyCount() + " (" + dbStats.getLocalKeyCount() + ")" );  
       int[] stats = dbStats.getValueDetails();
       lblValues.setText("" + stats[DHTDBStats.VD_VALUE_COUNT]);
-      lblDirect.setText("" + stats[DHTDBStats.VD_DIRECT_SIZE]);
-      lblIndirect.setText("" + stats[DHTDBStats.VD_INDIRECT_SIZE]);
-      lblLocal.setText("" + stats[DHTDBStats.VD_LOCAL_SIZE]);
-      lblDivFreq.setText("" + stats[DHTDBStats.VD_DIV_FREQ]);
-      lblDivSize.setText("" + stats[DHTDBStats.VD_DIV_SIZE]);
+      lblSize.setText(DisplayFormatters.formatByteCountToKiBEtc(dbStats.getSize()));
+      lblDirect.setText(DisplayFormatters.formatByteCountToKiBEtc( stats[DHTDBStats.VD_DIRECT_SIZE]));
+      lblIndirect.setText(DisplayFormatters.formatByteCountToKiBEtc( stats[DHTDBStats.VD_INDIRECT_SIZE]));
+      lblLocal.setText(DisplayFormatters.formatByteCountToKiBEtc( stats[DHTDBStats.VD_LOCAL_SIZE]));
+      
+      DHTStorageAdapter sa = dht.getStorageAdapter();
+      
+      String rem_freq;
+      String rem_size;
+      
+      if ( sa == null ){
+    	  rem_freq = "?";
+    	  rem_size = "?";
+      }else{
+    	  rem_freq = "" + sa.getRemoteFreqDivCount();
+    	  rem_size = "" + sa.getRemoteSizeDivCount(); 
+      }
+      
+      lblDivFreq.setText("" + stats[DHTDBStats.VD_DIV_FREQ] + " (" + rem_freq + ")");
+      lblDivSize.setText("" + stats[DHTDBStats.VD_DIV_SIZE] + " (" + rem_size + ")");
     } else {
       refreshIter++;
       if(refreshIter == 100) refreshIter = 0;
