@@ -44,9 +44,7 @@ import org.gudy.azureus2.ui.swt.debug.ObfusticateCellText;
 import org.gudy.azureus2.ui.swt.mainwindow.Colors;
 import org.gudy.azureus2.ui.swt.plugins.UISWTGraphic;
 import org.gudy.azureus2.ui.swt.pluginsimpl.UISWTGraphicImpl;
-import org.gudy.azureus2.ui.swt.views.table.TableCellSWT;
-import org.gudy.azureus2.ui.swt.views.table.TableRowSWT;
-import org.gudy.azureus2.ui.swt.views.table.TableViewSWT;
+import org.gudy.azureus2.ui.swt.views.table.*;
 import org.gudy.azureus2.ui.swt.views.table.utils.TableColumnSWTUtils;
 
 import com.aelitis.azureus.ui.common.table.*;
@@ -102,11 +100,13 @@ public class TableCellImpl
 	private ArrayList cellMouseListeners;
 	private ArrayList cellMouseMoveListeners;
 	private ArrayList cellVisibilityListeners;
+	private ArrayList cellSWTPaintListeners;
   private TableColumnCore tableColumn;
   private byte refreshErrLoopCount;
   private byte tooltipErrLoopCount;
   private byte loopFactor;
   private Object oToolTip;
+  private Object defaultToolTip;
 	private int iCursorID = -1;
 	private Graphic graphic = null;
 	
@@ -129,7 +129,9 @@ public class TableCellImpl
 	private byte restartRefresh = 0;
 	private boolean bInRefreshAsync = false;
 	
-  static {
+	private int textAlpha = 255;
+	
+	static {
   	COConfigurationManager.addAndFireParameterListener(CFG_PAINT,
 				new ParameterListener() {
 					public void parameterChanged(String parameterName) {
@@ -143,7 +145,7 @@ public class TableCellImpl
       int position, BufferedTableItem item) {
     this.tableColumn = _tableColumn;
     this.tableRow = _tableRow;
-    flags = FLAG_SORTVALUEISTEXT | FLAG_UPTODATE;
+    flags = FLAG_SORTVALUEISTEXT;
     refreshErrLoopCount = 0;
     tooltipErrLoopCount = 0;
     loopFactor = 0;
@@ -180,6 +182,7 @@ public class TableCellImpl
 	        }
 	        public void invalidate() {
 	        	clearFlag(FLAG_VALID);
+	        	redraw();
 	        }
 	      };
 	    } else {
@@ -189,6 +192,7 @@ public class TableCellImpl
 	        }
 	        public void invalidate() {
 	        	clearFlag(FLAG_VALID);
+	        	redraw();
 	        }
 	      };
 	    }
@@ -474,6 +478,11 @@ public class TableCellImpl
   	
   	tableColumn.setLastSortValueChange(SystemTime.getCurrentTime());
     sortValue = valueToSort;
+    
+  	if (cellSWTPaintListeners != null
+				|| tableColumn.hasCellOtherListeners("SWTPaint")) {
+  		redraw();
+  	}
 
     return true;
   }
@@ -521,6 +530,13 @@ public class TableCellImpl
     return oToolTip;
   }
 
+  public Object getDefaultToolTip() {
+	return defaultToolTip;
+  }
+  public void setDefaultToolTip(Object tt) {
+	defaultToolTip = tt;
+  }
+  
 	public boolean isDisposed() {
 		return hasFlag(FLAG_DISPOSED);
 	}
@@ -574,6 +590,7 @@ public class TableCellImpl
     return pt.y;
   }
 
+  // @see org.gudy.azureus2.ui.swt.views.table.TableCellSWT#setGraphic(org.eclipse.swt.graphics.Image)
   public boolean setGraphic(Image img) {
   	checkCellForSetting();
 
@@ -584,10 +601,12 @@ public class TableCellImpl
     boolean b = ((BufferedGraphicTableItem)bufferedTableItem).setGraphic(img);
     if (b) {
     	setFlag(FLAG_VISUALLY_CHANGED_SINCE_REFRESH);
+			bufferedTableItem.redraw();
     }
     return b;
   }
 
+  // @see org.gudy.azureus2.plugins.ui.tables.TableCell#setGraphic(org.gudy.azureus2.plugins.ui.Graphic)
   public boolean setGraphic(Graphic img) {
   	if (img != null){
   		checkCellForSetting();
@@ -607,6 +626,7 @@ public class TableCellImpl
       boolean b = ((BufferedGraphicTableItem)bufferedTableItem).setGraphic(null);
       if (b) {
       	setFlag(FLAG_VISUALLY_CHANGED_SINCE_REFRESH);
+  			bufferedTableItem.redraw();
       }
     }
 
@@ -615,14 +635,14 @@ public class TableCellImpl
     	boolean b = ((BufferedGraphicTableItem)bufferedTableItem).setGraphic(imgSWT);
       if (b) {
       	setFlag(FLAG_VISUALLY_CHANGED_SINCE_REFRESH);
+  			bufferedTableItem.redraw();
       }
-    }
-    
-    if (img instanceof UISWTGraphic){
+    } else if (img instanceof UISWTGraphic){
     	Image imgSWT = ((UISWTGraphic)img).getImage();
     	boolean b = ((BufferedGraphicTableItem)bufferedTableItem).setGraphic(imgSWT);
       if (b) {
       	setFlag(FLAG_VISUALLY_CHANGED_SINCE_REFRESH);
+  			bufferedTableItem.redraw();
       }
     }
     
@@ -864,6 +884,59 @@ public class TableCellImpl
 		}
 	}
 
+	/**
+	 * @param listenerObject
+	 *
+	 * @since 3.1.1.1
+	 */
+	private void addSWTPaintListener(TableCellSWTPaintListener listener) {
+		try {
+			this_mon.enter();
+
+			if (cellSWTPaintListeners == null)
+				cellSWTPaintListeners = new ArrayList(1);
+
+			cellSWTPaintListeners.add(listener);
+
+		} finally {
+			this_mon.exit();
+		}
+	}
+
+	public void invokeSWTPaintListeners(GC gc) {
+  	if (tableColumn != null) {
+			Object[] swtPaintListeners = tableColumn.getCellOtherListeners("SWTPaint");
+			if (swtPaintListeners != null) { 
+  			for (int i = 0; i < swtPaintListeners.length; i++) {
+  				try {
+  					TableCellSWTPaintListener l = (TableCellSWTPaintListener) swtPaintListeners[i];
+  
+  					l.cellPaint(gc, this);
+  
+  				} catch (Throwable e) {
+  					Debug.printStackTrace(e);
+  				}
+  			}
+			}
+		}
+
+		if (cellSWTPaintListeners == null) {
+			return;
+		}
+		
+
+		for (int i = 0; i < cellSWTPaintListeners.size(); i++) {
+			try {
+				TableCellSWTPaintListener l = (TableCellSWTPaintListener) (cellSWTPaintListeners.get(i));
+
+				l.cellPaint(gc, this);
+
+			} catch (Throwable e) {
+				Debug.printStackTrace(e);
+			}
+		}
+	}
+	
 	public void addListeners(Object listenerObject) {
 		if (listenerObject instanceof TableCellDisposeListener) {
 			addDisposeListener((TableCellDisposeListener)listenerObject);
@@ -885,6 +958,9 @@ public class TableCellImpl
 
 		if (listenerObject instanceof TableCellVisibilityListener)
 			addVisibilityListener((TableCellVisibilityListener)listenerObject);
+
+		if (listenerObject instanceof TableCellSWTPaintListener)
+			addSWTPaintListener((TableCellSWTPaintListener)listenerObject);
 	}
 
 	/**
@@ -900,6 +976,12 @@ public class TableCellImpl
 	/* Start of Core-Only function */
   //////////////////////////////////
 	
+	public void redraw() {
+		if (bufferedTableItem != null) {
+			bufferedTableItem.redraw();
+		}
+	}
+	
   public void invalidate(final boolean bMustRefresh) {
   	if (!hasFlag(FLAG_VALID)) {
   		if (bMustRefresh) {
@@ -978,6 +1060,9 @@ public class TableCellImpl
   // @see com.aelitis.azureus.ui.common.table.TableCellCore#refresh(boolean, boolean, boolean)
   public boolean refresh(boolean bDoGraphics, boolean bRowVisible,  boolean bCellVisible)
   {
+  	if (tableColumn == null) {
+  		return false;
+  	}
 	  boolean ret = getVisuallyChangedSinceRefresh();
   	clearFlag(FLAG_VISUALLY_CHANGED_SINCE_REFRESH);
 
@@ -1052,6 +1137,10 @@ public class TableCellImpl
 		  {
 			  boolean bWasValid = isValid();
 
+			  if (hasFlag(FLAG_MUSTREFRESH)) {
+			  	clearFlag(FLAG_MUSTREFRESH);
+			  }
+
 			  if (bDebug)
 				  debug("invoke refresh; wasValid? " + bWasValid);
 
@@ -1071,11 +1160,8 @@ public class TableCellImpl
 
 			  // Change to valid only if we weren't valid before the listener calls
 			  // This is in case the listeners set valid to false when it was true
-			  if (!bWasValid) 
+			  if (!bWasValid && !hasFlag(FLAG_MUSTREFRESH)) {
 			  	setFlag(FLAG_VALID);
-
-			  if (hasFlag(FLAG_MUSTREFRESH)) {
-			  	clearFlag(FLAG_MUSTREFRESH);
 			  }
 		  }
 		  loopFactor++;
@@ -1161,6 +1247,9 @@ public class TableCellImpl
   }
 
   public boolean needsPainting() {
+  	if (cellSWTPaintListeners != null || tableColumn.hasCellOtherListeners("SWTPaint")) {
+  		return true;
+  	}
   	if (bufferedTableItem == null) {
   		return false;
   	}
@@ -1181,6 +1270,9 @@ public class TableCellImpl
 		if (bDebug) {
 			debug("doPaint up2date:" + hasFlag(FLAG_UPTODATE) + ";v:" + hasFlag(FLAG_VALID) + ";rl=" + refreshListeners);
 		}
+		
+		invokeSWTPaintListeners(gc);
+		
     bufferedTableItem.doPaint(gc);
     
     if (childCells != null) {
@@ -1217,7 +1309,8 @@ public class TableCellImpl
 		return "TableCell {"
 				+ tableColumn.getName()
 				+ ","
-				+ (bufferedTableItem == null ? "null" : ""
+				+ (tableRow == null ? "" : "r" + tableRow.getIndex())
+				+ (bufferedTableItem == null ? "c?" : "c"
 						+ bufferedTableItem.getPosition()) + "," + getText() + ","
 				+ getSortValue() + "}";
 	}
@@ -1416,13 +1509,13 @@ public class TableCellImpl
 	
 	public void setCursorID(int cursorID) {
 		iCursorID = cursorID;
-		if (isMouseOver()) {
-			Utils.execSWTThread(new AERunnable() {
-				public void runSupport() {
+		Utils.execSWTThread(new AERunnable() {
+			public void runSupport() {
+				if (isMouseOver()) {
 					bufferedTableItem.setCursor(iCursorID);
 				}
-			});
-		}
+			}
+		});
 	}
 	
 	public boolean isMouseOver() {
@@ -1461,4 +1554,13 @@ public class TableCellImpl
 		//TODO: childCell.setParentCell(this);
 		childCells.add(childCell);
 	}
+
+  public int getTextAlpha() {
+		return textAlpha;
+	}
+
+	public void setTextAlpha(int textOpacity) {
+		this.textAlpha = textOpacity;
+	}
+
 }
