@@ -24,9 +24,12 @@
  
 package org.gudy.azureus2.ui.swt.views.tableitems.peers;
 import org.gudy.azureus2.core3.peer.PEPeer;
-import org.gudy.azureus2.plugins.ui.tables.*;
 import org.gudy.azureus2.ui.swt.views.table.utils.CoreTableColumn;
 
+import org.gudy.azureus2.plugins.ui.tables.TableCell;
+import org.gudy.azureus2.plugins.ui.tables.TableCellRefreshListener;
+import org.gudy.azureus2.plugins.ui.tables.TableColumnInfo;
+
 /**
  *
  * @author Olivier
@@ -36,9 +39,17 @@ public class InterestedItem
        extends CoreTableColumn 
        implements TableCellRefreshListener
 {
+	public static final String COLUMN_ID = "I1";
+
+	public void fillTableColumnInfo(TableColumnInfo info) {
+		info.addCategories(new String[] {
+			CAT_PROTOCOL,
+		});
+	}
+	
   /** Default Constructor */
   public InterestedItem(String table_id) {
-    super("I1", ALIGN_CENTER, POSITION_INVISIBLE, 20, table_id);
+    super(COLUMN_ID, ALIGN_CENTER, POSITION_INVISIBLE, 20, table_id);
     setRefreshInterval(INTERVAL_LIVE);
   }
 
