@@ -21,17 +21,17 @@
 
 package org.gudy.azureus2.ui.swt.wizard;
 
-import com.aelitis.azureus.core.AzureusCore;
+import java.util.ArrayList;
+import java.util.List;
+
 import org.eclipse.swt.SWT;
 import org.eclipse.swt.events.DisposeEvent;
 import org.eclipse.swt.events.DisposeListener;
-import org.eclipse.swt.graphics.Color;
-import org.eclipse.swt.graphics.Font;
-import org.eclipse.swt.graphics.FontData;
-import org.eclipse.swt.graphics.Point;
+import org.eclipse.swt.graphics.*;
 import org.eclipse.swt.layout.GridData;
 import org.eclipse.swt.layout.GridLayout;
 import org.eclipse.swt.widgets.*;
+
 import org.gudy.azureus2.core3.internat.MessageText;
 import org.gudy.azureus2.core3.util.AERunnable;
 import org.gudy.azureus2.core3.util.Debug;
@@ -39,9 +39,6 @@ import org.gudy.azureus2.ui.swt.Messages;
 import org.gudy.azureus2.ui.swt.Utils;
 import org.gudy.azureus2.ui.swt.components.shell.ShellFactory;
 
-import java.util.ArrayList;
-import java.util.List;
-
 /**
  * @author Olivier
  * 
@@ -51,7 +48,6 @@ public class Wizard {
 	private final static int DEFAULT_WIDTH = 500;
   List		listeners = new ArrayList();
   
-  AzureusCore	azureus_core;
   Display display;
   Shell wizardWindow;
   Label title;
@@ -68,29 +64,21 @@ public class Wizard {
   
   public 
   Wizard(
-  	AzureusCore		azureus_core,
   	String 			keyTitle,
   	boolean modal) 
   {
-    this(azureus_core, modal);
+    this(modal);
     setTitleKey(keyTitle);
   }
 
-	public Wizard(AzureusCore azureus_core, String keyTitle) {
-		this(azureus_core, keyTitle, false);
-	}
-
-	public Wizard(AzureusCore azureus_core) {
-		this(azureus_core, false);
+	public Wizard(String keyTitle) {
+		this(keyTitle, false);
 	}
 
   public 
   Wizard(
-  	AzureusCore		_azureus_core,
   	boolean modal) 
   {
-  	azureus_core	= _azureus_core;
-    
   	int style = SWT.DIALOG_TRIM | SWT.RESIZE;
   	if (modal) {
   		style |= SWT.APPLICATION_MODAL;
@@ -450,12 +438,6 @@ public class Wizard {
   		wizardWindow.setSize(p.x,height);
   }
   
-  public AzureusCore
-  getAzureusCore()
-  {
-  	return( azureus_core );
-  }
-
   public void
   addListener(
   	WizardListener	l )
