@@ -28,21 +28,26 @@ package org.gudy.azureus2.update;
  */
 
 import java.util.*;
+import java.util.zip.ZipEntry;
+import java.util.zip.ZipInputStream;
 import java.net.*;
 import java.io.*;
 
 import org.gudy.azureus2.core3.util.*;
 import org.gudy.azureus2.core3.logging.*;
-import org.gudy.azureus2.core3.logging.Logger;
 import org.gudy.azureus2.core3.config.*;
 import org.gudy.azureus2.core3.html.*;
 
+import org.gudy.azureus2.platform.win32.access.AEWin32Access;
+import org.gudy.azureus2.platform.win32.access.AEWin32Manager;
 import org.gudy.azureus2.plugins.*;
-import org.gudy.azureus2.plugins.logging.*;
+import org.gudy.azureus2.plugins.logging.LoggerChannel;
 import org.gudy.azureus2.plugins.update.*;
 import org.gudy.azureus2.plugins.utils.resourcedownloader.*;
 
 import com.aelitis.azureus.core.versioncheck.*;
+import com.aelitis.azureus.ui.UIFunctions;
+import com.aelitis.azureus.ui.UIFunctionsManager;
 
 
 public class 
@@ -153,6 +158,11 @@ CoreUpdateChecker
       
 			displayUserMessage( decoded );
 			
+			// No point complaining later if we don't have any data in the map (which is
+			// more likely due to network problems rather than the version check server
+			// *actually* returning a map with nothing in it. 
+			if (decoded.isEmpty()) {return;}
+			
 			String latest_version;
 			String latest_file_name;
 			
@@ -229,7 +239,7 @@ CoreUpdateChecker
 			if ( full_download_url == null ){
 				
 				ResourceDownloader[]	primary_mirrors;
-					
+									
 				primary_mirrors = getPrimaryDownloaders( latest_file_name );
 	
 					// the download hierarchy is primary mirrors first (randomised alternate)
@@ -317,7 +327,7 @@ CoreUpdateChecker
 			if ( info_b != null ){
 			
 				try{
-					info = new String( info_b );
+					info = new String( info_b, "UTF-8" );
 				
 				}catch( Throwable e ){
 					
@@ -325,6 +335,66 @@ CoreUpdateChecker
 				}
 			}
 			
+			byte[] info_url_bytes = (byte[])decoded.get("info_url");
+			
+			String info_url = null;
+			
+			if ( info_url_bytes != null ){
+				
+				try{
+					info_url = new String( info_url_bytes );
+					
+				}catch( Exception e ){
+					
+					Debug.out(e);
+				}
+			}
+			
+			if ( info != null || info_url != null ){
+				
+				String	check;
+				
+				if ( info == null ){
+					
+					check = info_url;
+					
+				}else if ( info_url == null ){
+					
+					check = info;
+					
+				}else{
+					
+					check = info + "|" + info_url;
+				}
+				
+				byte[]	sig = (byte[])decoded.get( "info_sig" );
+
+				boolean	ok = false;
+				
+				if ( sig == null ){
+				
+					Logger.log( new LogEvent( LogIDs.LOGGER, "info signature check failed - missing signature" ));
+
+				}else{
+					
+					try{
+						AEVerifier.verifyData( check, sig );
+
+						ok = true;
+						
+					}catch( Throwable e ){
+
+						Logger.log( new LogEvent( LogIDs.LOGGER, "info signature check failed", e  ));
+					}			
+				}
+				
+				if ( !ok ){
+					
+					info		= null;
+					info_url	= null;
+				}
+			}
+			
 			String[]	desc;
 			
 			if ( info == null ){
@@ -336,6 +406,8 @@ CoreUpdateChecker
 				desc = new String[]{"Core Azureus Version", info };
 			}
 			
+
+
 			final Update update = 
 				checker.addUpdate(
 						"Core Azureus Version",
@@ -344,6 +416,11 @@ CoreUpdateChecker
 						top_downloader,
 						Update.RESTART_REQUIRED_YES );
 			
+			if ( info_url != null ){
+				
+				update.setDescriptionURL(info_url);
+			}
+			
 			top_downloader.addListener( 
 					new ResourceDownloaderAdapter()
 					{
@@ -352,7 +429,7 @@ CoreUpdateChecker
 							final ResourceDownloader	downloader,
 							InputStream					data )
 						{	
-							installUpdate( checker, update, downloader, f_latest_version, data );
+							installUpdate( checker, update, downloader, f_latest_file_name, f_latest_version, data );
 									
 							return( true );
 						}
@@ -363,6 +440,8 @@ CoreUpdateChecker
 			
 			Debug.printStackTrace( e );
 			
+			checker.reportProgress( "Failed to check for core update: " + Debug.getNestedExceptionMessage(e));
+			
 			checker.failed();
 			
 		}finally{
@@ -380,63 +459,185 @@ CoreUpdateChecker
    * Log and display a user message if contained within reply.
    * @param reply from server
    */
-  private void displayUserMessage( Map reply ) {
-    //  pick up any user message in the reply
-    
-    try{
-      byte[]  message = (byte[])reply.get( "message" );
-          
-      if ( message != null && message.length > 0 ){
-        
-        String  s_message = new String(message);
-        
-        String  last = COConfigurationManager.getStringParameter( "CoreUpdateChecker.lastmessage", "" );
-        
-        if ( !s_message.equals( last )){
-          
-          byte[]	signature = (byte[])reply.get( "message_sig" );
-          
-          if ( signature == null ){
-        	  
-        	  Logger.log( new LogEvent( LogIDs.LOGGER, "Signature missing from message" ));
-        	  
-        	  return;
-          }
-          
-          try{
-	          AEVerifier.verifyData( s_message, signature );
-	          
-          }catch( Throwable e ){
-        	  
-           	  Logger.log( new LogEvent( LogIDs.LOGGER, "Message signature check failed", e  ));
-           	      
-           	  return;
-          }
-          
-          int   alert_type    = LogAlert.AT_WARNING;
-          String  alert_text    = s_message;
-          
-          if ( alert_text.startsWith("i:" )){
-          
-            alert_type = LogAlert.AT_INFORMATION;
-            
-            alert_text = alert_text.substring(2);
-          }
-          
-          plugin_interface.getPluginProperties().setProperty( MESSAGE_PROPERTY, alert_text );
+  private void 
+  displayUserMessage( 
+	Map reply ) 
+  {
+	  //  pick up any user message in the reply
 
+	  try{
+		  Iterator it = reply.keySet().iterator();
+		  
+		  while( it.hasNext()){
+			  
+			  String	key = (String)it.next();
+		  
+			  	// support message + message_sig
+			  	//		message_1 + message_sig_1   etc
+			  
+			  if ( key.startsWith( "message_sig" ) || !key.startsWith( "message" )){
+				  
+				  continue;
+			  }
+			  
+			  byte[]  message_bytes = (byte[])reply.get( key );
+	
+			  if ( message_bytes != null && message_bytes.length > 0 ){
+	
+				  String  message;
+				  
+				  try{
+					  message = new String(message_bytes, "UTF-8" );
+					  
+				  }catch( Throwable e ){
+					  
+					  message = new String( message_bytes );
+				  }
+	
+				  String sig_key;
+				  
+				  int	pos = key.indexOf('_');
+				  
+				  if ( pos == -1 ){
+					  
+					  sig_key = "message_sig";
+					  
+				  }else{
+					  
+					  sig_key = "message_sig" + key.substring( pos );
+				  }
+				  
+				  String	last_message_key = "CoreUpdateChecker.last" + key;
+				  
+				  String  last = COConfigurationManager.getStringParameter( last_message_key, "" );
+	
+				  if ( !message.equals( last )){
+	
+					  byte[]	signature = (byte[])reply.get( sig_key );
+	
+					  if ( signature == null ){
+	
+						  Logger.log( new LogEvent( LogIDs.LOGGER, "Signature missing from message" ));
+	
+						  return;
+					  }
+	
+					  try{
+						  AEVerifier.verifyData( message, signature );
+	
+					  }catch( Throwable e ){
+	
+						  Logger.log( new LogEvent( LogIDs.LOGGER, "Message signature check failed", e  ));
+	
+						  return;
+					  }
+	
+					  boolean	completed = false;
+					  
+					  if ( message.startsWith( "x:" )){
+						  
+						  	// emergency patch application
+						  
+						  try{
+							  URL jar_url = new URL( message.substring(2));
+							  
+							  Logger.log( new LogEvent( LogIDs.LOGGER, "Patch application requsted: url=" + jar_url ));
 
-          Logger.log(new LogAlert(LogAlert.UNREPEATABLE, alert_type, alert_text));
-          
-          COConfigurationManager.setParameter( "CoreUpdateChecker.lastmessage", s_message );
-          
-          COConfigurationManager.save();
-        }
-      }
-    }catch( Throwable e ){
-      
-      Debug.printStackTrace( e );
-    }
+							  File	temp_dir = AETemporaryFileHandler.createTempDir();
+							  
+							  File	jar_file = new File( temp_dir, "patch.jar" );
+							  
+							  InputStream is = rdf.create( jar_url ).download();
+							  
+							  try{
+								  FileUtil.copyFile( is, jar_file );
+								  
+								  is = null;
+								  
+								  AEVerifier.verifyData( jar_file );
+								  
+								  ClassLoader cl = CoreUpdateChecker.class.getClassLoader();
+								      		
+								  if ( cl instanceof URLClassLoader ){
+
+									  URL[]	old = ((URLClassLoader)cl).getURLs();
+
+									  URL[]	new_urls = new URL[old.length+1];
+
+									  System.arraycopy( old, 0, new_urls, 1, old.length );
+
+									  new_urls[0]= jar_file.toURL();
+
+									  cl = new URLClassLoader( new_urls, cl );
+
+								  }else{
+
+									  cl = new URLClassLoader( new URL[]{jar_file.toURL()}, cl );
+								  }
+	
+								  Class cla = cl.loadClass( "org.gudy.azureus2.update.version.Patch" );
+								  
+								  cla.newInstance();
+								  
+								  completed = true;
+								  
+							  }finally{
+								  
+								  if ( is != null ){
+								  
+									  is.close();
+								  }
+							  }
+						  }catch( Throwable e ){
+							  
+							  Logger.log( new LogEvent( LogIDs.LOGGER, "Patch application failed", e  ));
+						  }
+					  } else if ( message.startsWith("u:") && message.length() > 4 ) {
+					  	try {
+  					  	String type = message.substring(2, 3);
+  					  	String url = message.substring(4);
+  					  	UIFunctions uif = UIFunctionsManager.getUIFunctions();
+  					  	if (uif != null) {
+  					  		uif.viewURL(url, null, 0.9, 0.9, true, type.equals("1"));
+  					  	}
+					  	} catch (Throwable t) {
+							  Logger.log( new LogEvent( LogIDs.LOGGER, "URL message failed", t  ));
+					  	}
+					  	// mark as complete even if errored
+						  completed = true;
+					  }else{
+						  
+						  int   alert_type    = LogAlert.AT_WARNING;
+						  
+						  String  alert_text    = message;
+		
+						  if ( alert_text.startsWith("i:" )){
+		
+							  alert_type = LogAlert.AT_INFORMATION;
+		
+							  alert_text = alert_text.substring(2);
+						  }
+		
+						  plugin_interface.getPluginProperties().setProperty( MESSAGE_PROPERTY, alert_text );
+		
+						  Logger.log(new LogAlert(LogAlert.UNREPEATABLE, alert_type, alert_text, 0 ));
+						  
+						  completed = true;
+					  }
+					  
+					  if ( completed ){
+						  
+						  COConfigurationManager.setParameter( last_message_key, message );
+	
+						  COConfigurationManager.save();
+					  }
+				  }
+			  }
+		  }
+	  }catch( Throwable e ){
+
+		  Debug.printStackTrace( e );
+	  }
   } 
 	
 	protected ResourceDownloader[]
@@ -598,6 +799,7 @@ CoreUpdateChecker
 		UpdateChecker		checker,
 		Update				update,
 		ResourceDownloader	rd,
+		String				filename,
 		String				version,
 		InputStream			data )
 	{
@@ -606,23 +808,30 @@ CoreUpdateChecker
 
 			rd.reportActivity( "Data verified successfully" );
 			
-			String	temp_jar_name 	= "Azureus2_" + version + ".jar";
-			String	target_jar_name	= "Azureus2.jar";
-			
-			UpdateInstaller	installer = checker.createInstaller();
-			
-			installer.addResource( temp_jar_name, data );
-			
-			if ( Constants.isOSX ){
+			if ( filename.toLowerCase().endsWith( ".zip.torrent" )){
+				
+				handleZIPUpdate( checker, data );
 				
-				installer.addMoveAction( 
-					temp_jar_name,
-					installer.getInstallDir() + "/" + SystemProperties.getApplicationName() + ".app/Contents/Resources/Java/" + target_jar_name );        
 			}else{
 				
-				installer.addMoveAction( 
-					temp_jar_name,
-					installer.getInstallDir() + File.separator + target_jar_name );
+				String	temp_jar_name 	= "Azureus2_" + version + ".jar";
+				String	target_jar_name	= "Azureus2.jar";
+				
+				UpdateInstaller	installer = checker.createInstaller();
+				
+				installer.addResource( temp_jar_name, data );
+				
+				if ( Constants.isOSX ){
+					
+					installer.addMoveAction( 
+						temp_jar_name,
+						installer.getInstallDir() + "/" + SystemProperties.getApplicationName() + ".app/Contents/Resources/Java/" + target_jar_name );        
+				}else{
+					
+					installer.addMoveAction( 
+						temp_jar_name,
+						installer.getInstallDir() + File.separator + target_jar_name );
+				}
 			}
 		}catch( Throwable e ){
 			
@@ -630,6 +839,281 @@ CoreUpdateChecker
 		}
 	}
 	
+	protected void
+	handleZIPUpdate(
+		UpdateChecker		checker,
+		InputStream			data )
+	
+		throws Exception
+	{
+		ZipInputStream zip = null;
+		
+		Properties	update_properties = new Properties();
+		
+		File		temp_dir = AETemporaryFileHandler.createTempDir();
+		
+		File		update_file = null;
+		
+		try{
+			zip = new ZipInputStream(data);
+
+			ZipEntry entry = null;
+
+			while((entry = zip.getNextEntry()) != null) {
+
+				String name = entry.getName().trim();
+
+				if ( name.equals( "azureus.sig" ) || name.endsWith( "/" ) || name.length() == 0 ){
+					
+					continue;
+				}
+
+				if ( name.equals( "update.properties" )){
+					
+					update_properties.load( zip );
+					
+				}else{
+					
+					if ( update_file != null ){
+						
+						throw( new Exception( "Multiple update files are not supported" ));
+					}
+					
+					update_file = new File( temp_dir, name );
+					
+					FileUtil.copyFile( zip, update_file, false );
+				}
+			}
+		}finally{
+			
+			if ( zip != null ){
+				
+				try{
+					zip.close();
+					
+				}catch( Throwable e ){
+					
+				}
+			}
+		}
+		
+		if ( update_properties == null ){
+			
+			throw( new Exception( "Update properties missing" ));
+		}
+		
+		if ( update_file == null ){
+			
+			throw( new Exception( "Update file missing" ));
+		}
+		
+		String	info_url = update_properties.getProperty( "info.url" );
+		
+		if ( info_url == null ){
+			
+			throw( new Exception( "Update property 'info.url' missing" ));
+		}
+				
+		UIFunctions uif = UIFunctionsManager.getUIFunctions();
+
+		if ( uif == null ){
+			
+			throw( new Exception( "Update can't proceed - UI functions unavailable" ));
+		}
+		
+		checker.getCheckInstance().setProperty( UpdateCheckInstance.PT_CLOSE_OR_RESTART_ALREADY_IN_PROGRESS, true );
+
+		final File f_update_file = update_file;
+				
+		uif.performAction( 
+			UIFunctions.ACTION_FULL_UPDATE,
+			info_url,
+			new UIFunctions.actionListener()
+			{
+				public void
+				actionComplete(
+					Object	result )
+				{
+					if ((Boolean)result){
+						
+						launchUpdate( f_update_file );
+					}
+				}
+			});
+	}
+	
+	protected void
+	launchUpdate(
+		File		file )
+	{
+		try{
+				// hack here to allow testing of osx on windows (parg) - should replace with
+				// Constants.isWindows etc
+			
+			if ( file.getName().endsWith( ".exe" )){
+				
+				try{
+					AEWin32Access accessor = AEWin32Manager.getAccessor(true);
+					
+					// accessor.createProcess( , false );
+					
+					accessor.shellExecute( 
+						null, 
+						file.getAbsolutePath(), 
+						null,
+						SystemProperties.getApplicationPath(),
+						AEWin32Access.SW_NORMAL );
+					
+				}catch( Throwable e ){
+					
+					Logger.log( new LogEvent( LogIDs.LOGGER, "AEWin32Access failed", e  ));
+
+					Runtime.getRuntime().exec( file.getAbsolutePath() );
+				}
+			}else{
+					// osx, need to unzip .app and launch
+				
+				File	dir = file.getParentFile();
+				
+			   	ZipInputStream	zis = new ZipInputStream( new BufferedInputStream( new FileInputStream( file )));
+		    		
+			   	Throwable unzip_error = null;
+			   	
+		    	try{
+					while( true ){
+												
+						ZipEntry	entry = zis.getNextEntry();
+							
+						if ( entry == null ){
+							
+							break;
+						}
+						
+						if ( entry.isDirectory()){
+							
+							continue;
+						}
+						
+						String	name = entry.getName();
+						
+						FileOutputStream	entry_os 	= null;
+						File				entry_file 	= null;
+						
+						if ( !name.endsWith("/")){
+														
+							entry_file = new File( dir, name.replace('/', File.separatorChar ));
+														
+							entry_file.getParentFile().mkdirs();
+							
+							entry_os	= new FileOutputStream( entry_file );
+						}
+						
+						try{
+							byte[]	buffer = new byte[65536];
+							
+							while( true ){
+							
+								int	len = zis.read( buffer );
+								
+								if ( len <= 0 ){
+									
+									break;
+								}
+																									
+								if ( entry_os != null ){
+									
+									entry_os.write( buffer, 0, len );
+								}
+							}
+						}finally{
+							
+							if ( entry_os != null ){
+								
+								entry_os.close();
+																	
+								if ( name.endsWith( ".jnilib" ) || name.endsWith( "JavaApplicationStub" )){
+										
+									try{
+										String[] to_run = { "/bin/sh", "-c", "chmod a+x \"" + entry_file.getAbsolutePath() + "\""};
+									  	
+										runCommand( to_run, true );
+												
+									}catch( Throwable e ){
+										
+										unzip_error = e;
+									}
+								}
+							}
+						}
+					}
+		    	}finally{
+		    		
+		    		zis.close();
+		    	}
+				
+		    	if ( unzip_error != null ){
+		    		
+		    		throw( unzip_error );
+		    	}
+		    	
+		    	File[] files = dir.listFiles();
+		    	
+		    	boolean launched = false;
+		    	
+		    	for ( File f: files ){
+		    		
+		    		if ( f.getName().endsWith( ".app" )){
+		    	
+		    			String[] to_run = { "/bin/sh", "-c", "open \"" + f.getAbsolutePath() + "\""};
+			  		
+		    			runCommand( to_run, false );
+		    			
+		    			launched = true;
+		    		}
+		    	}
+		    	
+		    	if ( !launched ){
+		    		
+		    		throw( new Exception( "No .app files found in '" + dir + "'" ));
+		    	}
+			}
+		}catch( Throwable e ){
+			
+			Logger.log( new LogEvent( LogIDs.LOGGER, "Failed to launch update '" + file + "'", e  ));
+		}
+	}
+	
+	private static void
+	runCommand(
+		String[]	command,
+		boolean		wait )
+	
+		throws Throwable
+	{
+		try{
+			String	str = "";
+			
+			for ( String s: command ){
+				
+				str += (str.length()==0?"":", ") + s;
+			}
+			
+			System.out.println( "running " + str );
+			
+			Process proc = Runtime.getRuntime().exec( command );
+			
+			if ( wait ){
+				
+				proc.waitFor();
+			}
+		}catch( Throwable e ){
+			
+			System.err.println( e );
+			
+			throw( e );
+		}
+	}
+	
 	protected static boolean
 	shouldUpdate(
 		String	current_version,
